// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT

#include "manual.h"
#include <stdio.h>

#define PRINT_CONSTANT(CONSTANT_NAME) \
    printf("%s;", #CONSTANT_NAME); \
    printf(_Generic((CONSTANT_NAME), \
                    char *: "%s", \
                    const char *: "%s", \
                    char: "%c", \
                    signed char: "%hhd", \
                    unsigned char: "%hhu", \
                    short int: "%hd", \
                    unsigned short int: "%hu", \
                    int: "%d", \
                    unsigned int: "%u", \
                    long: "%ld", \
                    unsigned long: "%lu", \
                    long long: "%lld", \
                    unsigned long long: "%llu", \
                    float: "%.6f", \
                    double: "%.6f", \
                    long double: "%ld"), \
           CONSTANT_NAME); \
    printf("\n");

int main() {
    PRINT_CONSTANT((gint) GST_ANALYTICS_KEYPOINT_DIMENSIONS_2D);
    PRINT_CONSTANT((gint) GST_ANALYTICS_KEYPOINT_DIMENSIONS_3D);
    PRINT_CONSTANT((guint) GST_ANALYTICS_KEYPOINT_VISIBILITY_OCCLUDED);
    PRINT_CONSTANT((guint) GST_ANALYTICS_KEYPOINT_VISIBILITY_PROJECTED);
    PRINT_CONSTANT((guint) GST_ANALYTICS_KEYPOINT_VISIBILITY_UNKNOWN);
    PRINT_CONSTANT((guint) GST_ANALYTICS_KEYPOINT_VISIBILITY_VISIBLE);
    PRINT_CONSTANT(GST_ANALYTICS_MTD_TYPE_ANY);
    PRINT_CONSTANT((guint) GST_ANALYTICS_REL_TYPE_ANY);
    PRINT_CONSTANT((guint) GST_ANALYTICS_REL_TYPE_CONTAIN);
    PRINT_CONSTANT((guint) GST_ANALYTICS_REL_TYPE_IS_PART_OF);
    PRINT_CONSTANT((guint) GST_ANALYTICS_REL_TYPE_NONE);
    PRINT_CONSTANT((guint) GST_ANALYTICS_REL_TYPE_N_TO_N);
    PRINT_CONSTANT((guint) GST_ANALYTICS_REL_TYPE_RELATE_TO);
    PRINT_CONSTANT(GST_CAPS_FEATURE_META_GST_ANALYTICS_BATCH_META);
    PRINT_CONSTANT(GST_INF_RELATION_SPAN);
    PRINT_CONSTANT(GST_MODELINFO_SECTION_NAME);
    PRINT_CONSTANT(GST_MODELINFO_VERSION_MAJOR);
    PRINT_CONSTANT(GST_MODELINFO_VERSION_MINOR);
    PRINT_CONSTANT(GST_MODELINFO_VERSION_STR);
    PRINT_CONSTANT((gint) GST_SEGMENTATION_TYPE_INSTANCE);
    PRINT_CONSTANT((gint) GST_SEGMENTATION_TYPE_SEMANTIC);
    PRINT_CONSTANT((gint) GST_TENSOR_DATA_TYPE_BFLOAT16);
    PRINT_CONSTANT((gint) GST_TENSOR_DATA_TYPE_BOOL);
    PRINT_CONSTANT((gint) GST_TENSOR_DATA_TYPE_COMPLEX128);
    PRINT_CONSTANT((gint) GST_TENSOR_DATA_TYPE_COMPLEX64);
    PRINT_CONSTANT((gint) GST_TENSOR_DATA_TYPE_FLOAT16);
    PRINT_CONSTANT((gint) GST_TENSOR_DATA_TYPE_FLOAT32);
    PRINT_CONSTANT((gint) GST_TENSOR_DATA_TYPE_FLOAT64);
    PRINT_CONSTANT((gint) GST_TENSOR_DATA_TYPE_FLOAT8E4M3FN);
    PRINT_CONSTANT((gint) GST_TENSOR_DATA_TYPE_FLOAT8E4M3FNUZ);
    PRINT_CONSTANT((gint) GST_TENSOR_DATA_TYPE_FLOAT8E5M2);
    PRINT_CONSTANT((gint) GST_TENSOR_DATA_TYPE_FLOAT8E5M2FNUZ);
    PRINT_CONSTANT((gint) GST_TENSOR_DATA_TYPE_INT16);
    PRINT_CONSTANT((gint) GST_TENSOR_DATA_TYPE_INT32);
    PRINT_CONSTANT((gint) GST_TENSOR_DATA_TYPE_INT4);
    PRINT_CONSTANT((gint) GST_TENSOR_DATA_TYPE_INT64);
    PRINT_CONSTANT((gint) GST_TENSOR_DATA_TYPE_INT8);
    PRINT_CONSTANT((gint) GST_TENSOR_DATA_TYPE_STRING);
    PRINT_CONSTANT((gint) GST_TENSOR_DATA_TYPE_UINT16);
    PRINT_CONSTANT((gint) GST_TENSOR_DATA_TYPE_UINT32);
    PRINT_CONSTANT((gint) GST_TENSOR_DATA_TYPE_UINT4);
    PRINT_CONSTANT((gint) GST_TENSOR_DATA_TYPE_UINT64);
    PRINT_CONSTANT((gint) GST_TENSOR_DATA_TYPE_UINT8);
    PRINT_CONSTANT((gint) GST_TENSOR_DIM_ORDER_COL_MAJOR);
    PRINT_CONSTANT((gint) GST_TENSOR_DIM_ORDER_ROW_MAJOR);
    PRINT_CONSTANT((gint) GST_TENSOR_LAYOUT_CONTIGUOUS);
    PRINT_CONSTANT((gint) MODELINFO_DIRECTION_INPUT);
    PRINT_CONSTANT((gint) MODELINFO_DIRECTION_OUTPUT);
    PRINT_CONSTANT((gint) MODELINFO_DIRECTION_UNKNOWN);
    return 0;
}
