// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
pub fn ser_close_stream_event(
    #[allow(unused_variables)] object: &mut ::aws_smithy_json::serialize::JsonObjectWriter,
    #[allow(unused_variables)] input: &crate::types::CloseStreamEvent,
) -> ::std::result::Result<(), ::aws_smithy_types::error::operation::SerializationError> {
    Ok(())
}
