// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
pub(crate) static API_METADATA: ::aws_runtime::user_agent::ApiMetadata =
    ::aws_runtime::user_agent::ApiMetadata::new("kinesisvideo", crate::meta::PKG_VERSION);

/// Crate version number.
pub static PKG_VERSION: &str = env!("CARGO_PKG_VERSION");
