// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT

#include "manual.h"
#include <stdalign.h>
#include <stdio.h>

int main() {
    printf("%s;%zu;%zu\n", "GstMIKEYCacheType", sizeof(GstMIKEYCacheType), alignof(GstMIKEYCacheType));
    printf("%s;%zu;%zu\n", "GstMIKEYEncAlg", sizeof(GstMIKEYEncAlg), alignof(GstMIKEYEncAlg));
    printf("%s;%zu;%zu\n", "GstMIKEYKVType", sizeof(GstMIKEYKVType), alignof(GstMIKEYKVType));
    printf("%s;%zu;%zu\n", "GstMIKEYKeyDataType", sizeof(GstMIKEYKeyDataType), alignof(GstMIKEYKeyDataType));
    printf("%s;%zu;%zu\n", "GstMIKEYMacAlg", sizeof(GstMIKEYMacAlg), alignof(GstMIKEYMacAlg));
    printf("%s;%zu;%zu\n", "GstMIKEYMapSRTP", sizeof(GstMIKEYMapSRTP), alignof(GstMIKEYMapSRTP));
    printf("%s;%zu;%zu\n", "GstMIKEYMapType", sizeof(GstMIKEYMapType), alignof(GstMIKEYMapType));
    printf("%s;%zu;%zu\n", "GstMIKEYMessage", sizeof(GstMIKEYMessage), alignof(GstMIKEYMessage));
    printf("%s;%zu;%zu\n", "GstMIKEYPRFFunc", sizeof(GstMIKEYPRFFunc), alignof(GstMIKEYPRFFunc));
    printf("%s;%zu;%zu\n", "GstMIKEYPayload", sizeof(GstMIKEYPayload), alignof(GstMIKEYPayload));
    printf("%s;%zu;%zu\n", "GstMIKEYPayloadKEMAC", sizeof(GstMIKEYPayloadKEMAC), alignof(GstMIKEYPayloadKEMAC));
    printf("%s;%zu;%zu\n", "GstMIKEYPayloadKeyData", sizeof(GstMIKEYPayloadKeyData), alignof(GstMIKEYPayloadKeyData));
    printf("%s;%zu;%zu\n", "GstMIKEYPayloadPKE", sizeof(GstMIKEYPayloadPKE), alignof(GstMIKEYPayloadPKE));
    printf("%s;%zu;%zu\n", "GstMIKEYPayloadRAND", sizeof(GstMIKEYPayloadRAND), alignof(GstMIKEYPayloadRAND));
    printf("%s;%zu;%zu\n", "GstMIKEYPayloadSP", sizeof(GstMIKEYPayloadSP), alignof(GstMIKEYPayloadSP));
    printf("%s;%zu;%zu\n", "GstMIKEYPayloadSPParam", sizeof(GstMIKEYPayloadSPParam), alignof(GstMIKEYPayloadSPParam));
    printf("%s;%zu;%zu\n", "GstMIKEYPayloadT", sizeof(GstMIKEYPayloadT), alignof(GstMIKEYPayloadT));
    printf("%s;%zu;%zu\n", "GstMIKEYPayloadType", sizeof(GstMIKEYPayloadType), alignof(GstMIKEYPayloadType));
    printf("%s;%zu;%zu\n", "GstMIKEYSecProto", sizeof(GstMIKEYSecProto), alignof(GstMIKEYSecProto));
    printf("%s;%zu;%zu\n", "GstMIKEYSecSRTP", sizeof(GstMIKEYSecSRTP), alignof(GstMIKEYSecSRTP));
    printf("%s;%zu;%zu\n", "GstMIKEYTSType", sizeof(GstMIKEYTSType), alignof(GstMIKEYTSType));
    printf("%s;%zu;%zu\n", "GstMIKEYType", sizeof(GstMIKEYType), alignof(GstMIKEYType));
    printf("%s;%zu;%zu\n", "GstSDPAttribute", sizeof(GstSDPAttribute), alignof(GstSDPAttribute));
    printf("%s;%zu;%zu\n", "GstSDPBandwidth", sizeof(GstSDPBandwidth), alignof(GstSDPBandwidth));
    printf("%s;%zu;%zu\n", "GstSDPConnection", sizeof(GstSDPConnection), alignof(GstSDPConnection));
    printf("%s;%zu;%zu\n", "GstSDPKey", sizeof(GstSDPKey), alignof(GstSDPKey));
    printf("%s;%zu;%zu\n", "GstSDPMedia", sizeof(GstSDPMedia), alignof(GstSDPMedia));
    printf("%s;%zu;%zu\n", "GstSDPMessage", sizeof(GstSDPMessage), alignof(GstSDPMessage));
    printf("%s;%zu;%zu\n", "GstSDPOrigin", sizeof(GstSDPOrigin), alignof(GstSDPOrigin));
    printf("%s;%zu;%zu\n", "GstSDPResult", sizeof(GstSDPResult), alignof(GstSDPResult));
    printf("%s;%zu;%zu\n", "GstSDPTime", sizeof(GstSDPTime), alignof(GstSDPTime));
    printf("%s;%zu;%zu\n", "GstSDPZone", sizeof(GstSDPZone), alignof(GstSDPZone));
    return 0;
}
