// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
pub fn ser_bucket_lifecycle_configuration(
    input: &crate::types::BucketLifecycleConfiguration,
    writer: ::aws_smithy_xml::encode::ElWriter,
) -> ::std::result::Result<(), ::aws_smithy_types::error::operation::SerializationError> {
    #[allow(unused_mut)]
    let mut scope = writer.finish();
    {
        for list_item_1 in &input.rules {
            {
                let inner_writer = scope.start_el("Rule");
                crate::protocol_serde::shape_lifecycle_rule::ser_lifecycle_rule(list_item_1, inner_writer)?
            }
        }
    }
    scope.finish();
    Ok(())
}
