// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.

/// <p>Indicates the end of the input stream. After sending this event, the input stream will be closed and all audio will be returned.</p>
#[non_exhaustive]
#[derive(::std::clone::Clone, ::std::cmp::PartialEq, ::std::fmt::Debug)]
pub struct CloseStreamEvent {}
impl CloseStreamEvent {
    /// Creates a new builder-style object to manufacture [`CloseStreamEvent`](crate::types::CloseStreamEvent).
    pub fn builder() -> crate::types::builders::CloseStreamEventBuilder {
        crate::types::builders::CloseStreamEventBuilder::default()
    }
}

/// A builder for [`CloseStreamEvent`](crate::types::CloseStreamEvent).
#[derive(::std::clone::Clone, ::std::cmp::PartialEq, ::std::default::Default, ::std::fmt::Debug)]
#[non_exhaustive]
pub struct CloseStreamEventBuilder {}
impl CloseStreamEventBuilder {
    /// Consumes the builder and constructs a [`CloseStreamEvent`](crate::types::CloseStreamEvent).
    pub fn build(self) -> crate::types::CloseStreamEvent {
        crate::types::CloseStreamEvent {}
    }
}
