// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
pub(crate) fn type_erase_result<O, E>(
    result: ::std::result::Result<O, E>,
) -> ::std::result::Result<
    ::aws_smithy_runtime_api::client::interceptors::context::Output,
    ::aws_smithy_runtime_api::client::orchestrator::OrchestratorError<::aws_smithy_runtime_api::client::interceptors::context::Error>,
>
where
    O: ::std::fmt::Debug + ::std::marker::Send + ::std::marker::Sync + 'static,
    E: ::std::error::Error + std::fmt::Debug + ::std::marker::Send + ::std::marker::Sync + 'static,
{
    result
        .map(|output| ::aws_smithy_runtime_api::client::interceptors::context::Output::erase(output))
        .map_err(|error| ::aws_smithy_runtime_api::client::interceptors::context::Error::erase(error))
        .map_err(::std::convert::Into::into)
}

pub fn parse_http_error_metadata(
    _response_status: u16,
    response_headers: &::aws_smithy_runtime_api::http::Headers,
    response_body: &[u8],
) -> ::std::result::Result<::aws_smithy_types::error::metadata::Builder, ::aws_smithy_json::deserialize::error::DeserializeError> {
    crate::json_errors::parse_error_metadata(response_body, response_headers)
}

pub(crate) mod shape_get_medical_scribe_stream;

pub(crate) mod shape_start_call_analytics_stream_transcription;

pub(crate) mod shape_start_medical_scribe_stream;

pub(crate) mod shape_start_medical_stream_transcription;

pub(crate) mod shape_start_stream_transcription;

pub(crate) mod shape_audio_stream;

pub(crate) fn or_empty_doc(data: &[u8]) -> &[u8] {
    if data.is_empty() {
        b"{}"
    } else {
        data
    }
}

pub(crate) mod shape_bad_request_exception;

pub(crate) mod shape_conflict_exception;

pub(crate) mod shape_internal_failure_exception;

pub(crate) mod shape_limit_exceeded_exception;

pub(crate) mod shape_medical_scribe_input_stream;

pub(crate) mod shape_resource_not_found_exception;

pub(crate) mod shape_service_unavailable_exception;

pub(crate) mod shape_start_call_analytics_stream_transcription_output;

pub(crate) mod shape_start_medical_scribe_stream_output;

pub(crate) mod shape_start_medical_stream_transcription_output;

pub(crate) mod shape_start_stream_transcription_output;

pub fn parse_event_stream_error_metadata(
    payload: &::bytes::Bytes,
) -> ::std::result::Result<::aws_smithy_types::error::metadata::Builder, ::aws_smithy_json::deserialize::error::DeserializeError> {
    crate::json_errors::parse_error_metadata(payload, &::aws_smithy_runtime_api::http::Headers::new())
}

pub(crate) mod shape_configuration_event;

pub(crate) mod shape_medical_scribe_configuration_event;

pub(crate) mod shape_medical_scribe_session_control_event;

pub(crate) mod shape_medical_scribe_stream_details;

pub(crate) mod shape_category_event;

pub(crate) mod shape_channel_definition;

pub(crate) mod shape_medical_scribe_channel_definition;

pub(crate) mod shape_medical_scribe_channel_definitions;

pub(crate) mod shape_medical_scribe_context;

pub(crate) mod shape_medical_scribe_encryption_settings;

pub(crate) mod shape_medical_scribe_post_stream_analytics_result;

pub(crate) mod shape_medical_scribe_post_stream_analytics_settings;

pub(crate) mod shape_medical_scribe_transcript_event;

pub(crate) mod shape_medical_transcript_event;

pub(crate) mod shape_post_call_analytics_settings;

pub(crate) mod shape_transcript_event;

pub(crate) mod shape_utterance_event;

pub(crate) mod shape_clinical_note_generation_result;

pub(crate) mod shape_clinical_note_generation_settings;

pub(crate) mod shape_kms_encryption_context_map;

pub(crate) mod shape_medical_scribe_patient_context;

pub(crate) mod shape_call_analytics_entity_list;

pub(crate) mod shape_call_analytics_item_list;

pub(crate) mod shape_call_analytics_language_identification;

pub(crate) mod shape_issues_detected;

pub(crate) mod shape_matched_category_details;

pub(crate) mod shape_medical_scribe_transcript_segment;

pub(crate) mod shape_medical_transcript;

pub(crate) mod shape_string_list;

pub(crate) mod shape_transcript;

pub(crate) mod shape_call_analytics_entity;

pub(crate) mod shape_call_analytics_item;

pub(crate) mod shape_call_analytics_language_with_score;

pub(crate) mod shape_issue_detected;

pub(crate) mod shape_medical_result_list;

pub(crate) mod shape_medical_scribe_transcript_item_list;

pub(crate) mod shape_points_of_interest;

pub(crate) mod shape_result_list;

pub(crate) mod shape_character_offsets;

pub(crate) mod shape_medical_result;

pub(crate) mod shape_medical_scribe_transcript_item;

pub(crate) mod shape_result;

pub(crate) mod shape_timestamp_ranges;

pub(crate) mod shape_alternative_list;

pub(crate) mod shape_language_identification;

pub(crate) mod shape_medical_alternative_list;

pub(crate) mod shape_timestamp_range;

pub(crate) mod shape_alternative;

pub(crate) mod shape_language_with_score;

pub(crate) mod shape_medical_alternative;

pub(crate) mod shape_entity_list;

pub(crate) mod shape_item_list;

pub(crate) mod shape_medical_entity_list;

pub(crate) mod shape_medical_item_list;

pub(crate) mod shape_entity;

pub(crate) mod shape_item;

pub(crate) mod shape_medical_entity;

pub(crate) mod shape_medical_item;
