// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
impl super::Client {
    /// Constructs a fluent builder for the [`CreateSignalingChannel`](crate::operation::create_signaling_channel::builders::CreateSignalingChannelFluentBuilder) operation.
    ///
    /// - The fluent builder is configurable:
    ///   - [`channel_name(impl Into<String>)`](crate::operation::create_signaling_channel::builders::CreateSignalingChannelFluentBuilder::channel_name) / [`set_channel_name(Option<String>)`](crate::operation::create_signaling_channel::builders::CreateSignalingChannelFluentBuilder::set_channel_name):<br>required: **true**<br><p>A name for the signaling channel that you are creating. It must be unique for each Amazon Web Services account and Amazon Web Services Region.</p><br>
    ///   - [`channel_type(ChannelType)`](crate::operation::create_signaling_channel::builders::CreateSignalingChannelFluentBuilder::channel_type) / [`set_channel_type(Option<ChannelType>)`](crate::operation::create_signaling_channel::builders::CreateSignalingChannelFluentBuilder::set_channel_type):<br>required: **false**<br><p>A type of the signaling channel that you are creating. Currently, <code>SINGLE_MASTER</code> is the only supported channel type.</p><br>
    ///   - [`single_master_configuration(SingleMasterConfiguration)`](crate::operation::create_signaling_channel::builders::CreateSignalingChannelFluentBuilder::single_master_configuration) / [`set_single_master_configuration(Option<SingleMasterConfiguration>)`](crate::operation::create_signaling_channel::builders::CreateSignalingChannelFluentBuilder::set_single_master_configuration):<br>required: **false**<br><p>A structure containing the configuration for the <code>SINGLE_MASTER</code> channel type. The default configuration for the channel message's time to live is 60 seconds (1 minute).</p><br>
    ///   - [`tags(Tag)`](crate::operation::create_signaling_channel::builders::CreateSignalingChannelFluentBuilder::tags) / [`set_tags(Option<Vec::<Tag>>)`](crate::operation::create_signaling_channel::builders::CreateSignalingChannelFluentBuilder::set_tags):<br>required: **false**<br><p>A set of tags (key-value pairs) that you want to associate with this channel.</p><br>
    /// - On success, responds with [`CreateSignalingChannelOutput`](crate::operation::create_signaling_channel::CreateSignalingChannelOutput) with field(s):
    ///   - [`channel_arn(Option<String>)`](crate::operation::create_signaling_channel::CreateSignalingChannelOutput::channel_arn): <p>The Amazon Resource Name (ARN) of the created channel.</p>
    /// - On failure, responds with [`SdkError<CreateSignalingChannelError>`](crate::operation::create_signaling_channel::CreateSignalingChannelError)
    pub fn create_signaling_channel(&self) -> crate::operation::create_signaling_channel::builders::CreateSignalingChannelFluentBuilder {
        crate::operation::create_signaling_channel::builders::CreateSignalingChannelFluentBuilder::new(self.handle.clone())
    }
}
