// Copyright 2025 Don MacAskill. Licensed under MIT or Apache-2.0.

//! This module provides AArch64-specific implementations of the ArchOps trait.

#![cfg(target_arch = "aarch64")]

pub mod aes;
pub mod aes_sha3;
