// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
pub fn ser_get_delegated_access_token_input_input_input(
    input: &crate::operation::get_delegated_access_token::GetDelegatedAccessTokenInput,
) -> ::std::result::Result<::aws_smithy_types::body::SdkBody, ::aws_smithy_types::error::operation::SerializationError> {
    let mut out = String::new();
    #[allow(unused_mut)]
    let mut writer = ::aws_smithy_query::QueryWriter::new(&mut out, "GetDelegatedAccessToken", "2011-06-15");
    #[allow(unused_mut)]
    let mut scope_1 = writer.prefix("TradeInToken");
    if let Some(var_2) = &input.trade_in_token {
        scope_1.string(var_2);
    }
    writer.finish();
    Ok(::aws_smithy_types::body::SdkBody::from(out))
}
