// Take a look at the license at the top of the repository in the LICENSE file.

// rustdoc-stripper-ignore-next
//! Traits intended for subclassing [`WindowGroup`].

use crate::{WindowGroup, prelude::*, subclass::prelude::*};

pub trait WindowGroupImpl: ObjectImpl + ObjectSubclass<Type: IsA<WindowGroup>> {}

unsafe impl<T: WindowGroupImpl> IsSubclassable<T> for WindowGroup {}
