// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
impl super::Client {
    /// Constructs a fluent builder for the [`UpdateParallelData`](crate::operation::update_parallel_data::builders::UpdateParallelDataFluentBuilder) operation.
    ///
    /// - The fluent builder is configurable:
    ///   - [`name(impl Into<String>)`](crate::operation::update_parallel_data::builders::UpdateParallelDataFluentBuilder::name) / [`set_name(Option<String>)`](crate::operation::update_parallel_data::builders::UpdateParallelDataFluentBuilder::set_name):<br>required: **true**<br><p>The name of the parallel data resource being updated.</p><br>
    ///   - [`description(impl Into<String>)`](crate::operation::update_parallel_data::builders::UpdateParallelDataFluentBuilder::description) / [`set_description(Option<String>)`](crate::operation::update_parallel_data::builders::UpdateParallelDataFluentBuilder::set_description):<br>required: **false**<br><p>A custom description for the parallel data resource in Amazon Translate.</p><br>
    ///   - [`parallel_data_config(ParallelDataConfig)`](crate::operation::update_parallel_data::builders::UpdateParallelDataFluentBuilder::parallel_data_config) / [`set_parallel_data_config(Option<ParallelDataConfig>)`](crate::operation::update_parallel_data::builders::UpdateParallelDataFluentBuilder::set_parallel_data_config):<br>required: **true**<br><p>Specifies the format and S3 location of the parallel data input file.</p><br>
    ///   - [`client_token(impl Into<String>)`](crate::operation::update_parallel_data::builders::UpdateParallelDataFluentBuilder::client_token) / [`set_client_token(Option<String>)`](crate::operation::update_parallel_data::builders::UpdateParallelDataFluentBuilder::set_client_token):<br>required: **true**<br><p>A unique identifier for the request. This token is automatically generated when you use Amazon Translate through an AWS SDK.</p><br>
    /// - On success, responds with [`UpdateParallelDataOutput`](crate::operation::update_parallel_data::UpdateParallelDataOutput) with field(s):
    ///   - [`name(Option<String>)`](crate::operation::update_parallel_data::UpdateParallelDataOutput::name): <p>The name of the parallel data resource being updated.</p>
    ///   - [`status(Option<ParallelDataStatus>)`](crate::operation::update_parallel_data::UpdateParallelDataOutput::status): <p>The status of the parallel data resource that you are attempting to update. Your update request is accepted only if this status is either <code>ACTIVE</code> or <code>FAILED</code>.</p>
    ///   - [`latest_update_attempt_status(Option<ParallelDataStatus>)`](crate::operation::update_parallel_data::UpdateParallelDataOutput::latest_update_attempt_status): <p>The status of the parallel data update attempt. When the updated parallel data resource is ready for you to use, the status is <code>ACTIVE</code>.</p>
    ///   - [`latest_update_attempt_at(Option<DateTime>)`](crate::operation::update_parallel_data::UpdateParallelDataOutput::latest_update_attempt_at): <p>The time that the most recent update was attempted.</p>
    /// - On failure, responds with [`SdkError<UpdateParallelDataError>`](crate::operation::update_parallel_data::UpdateParallelDataError)
    pub fn update_parallel_data(&self) -> crate::operation::update_parallel_data::builders::UpdateParallelDataFluentBuilder {
        crate::operation::update_parallel_data::builders::UpdateParallelDataFluentBuilder::new(self.handle.clone())
    }
}
