// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT

use crate::ffi;
use glib::GStr;

#[doc(alias = "GST_BUFFER_POOL_OPTION_GL_SYNC_META")]
pub static BUFFER_POOL_OPTION_GL_SYNC_META: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GST_BUFFER_POOL_OPTION_GL_SYNC_META) };
#[doc(alias = "GST_BUFFER_POOL_OPTION_GL_TEXTURE_TARGET_2D")]
pub static BUFFER_POOL_OPTION_GL_TEXTURE_TARGET_2D: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GST_BUFFER_POOL_OPTION_GL_TEXTURE_TARGET_2D) };
#[doc(alias = "GST_BUFFER_POOL_OPTION_GL_TEXTURE_TARGET_EXTERNAL_OES")]
pub static BUFFER_POOL_OPTION_GL_TEXTURE_TARGET_EXTERNAL_OES: &GStr = unsafe {
    GStr::from_utf8_with_nul_unchecked(ffi::GST_BUFFER_POOL_OPTION_GL_TEXTURE_TARGET_EXTERNAL_OES)
};
#[doc(alias = "GST_BUFFER_POOL_OPTION_GL_TEXTURE_TARGET_RECTANGLE")]
pub static BUFFER_POOL_OPTION_GL_TEXTURE_TARGET_RECTANGLE: &GStr = unsafe {
    GStr::from_utf8_with_nul_unchecked(ffi::GST_BUFFER_POOL_OPTION_GL_TEXTURE_TARGET_RECTANGLE)
};
#[doc(alias = "GST_CAPS_FEATURE_MEMORY_GL_BUFFER")]
pub static CAPS_FEATURE_MEMORY_GL_BUFFER: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GST_CAPS_FEATURE_MEMORY_GL_BUFFER) };
#[doc(alias = "GST_CAPS_FEATURE_MEMORY_GL_MEMORY")]
pub static CAPS_FEATURE_MEMORY_GL_MEMORY: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GST_CAPS_FEATURE_MEMORY_GL_MEMORY) };
#[doc(alias = "GST_GL_API_GLES1_NAME")]
pub static GL_API_GLES1_NAME: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GST_GL_API_GLES1_NAME) };
#[doc(alias = "GST_GL_API_GLES2_NAME")]
pub static GL_API_GLES2_NAME: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GST_GL_API_GLES2_NAME) };
#[doc(alias = "GST_GL_API_OPENGL3_NAME")]
pub static GL_API_OPENGL3_NAME: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GST_GL_API_OPENGL3_NAME) };
#[doc(alias = "GST_GL_API_OPENGL_NAME")]
pub static GL_API_OPENGL_NAME: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GST_GL_API_OPENGL_NAME) };
#[doc(alias = "GST_GL_BASE_MEMORY_ALLOCATOR_NAME")]
pub static GL_BASE_MEMORY_ALLOCATOR_NAME: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GST_GL_BASE_MEMORY_ALLOCATOR_NAME) };
#[doc(alias = "GST_GL_BUFFER_ALLOCATOR_NAME")]
pub static GL_BUFFER_ALLOCATOR_NAME: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GST_GL_BUFFER_ALLOCATOR_NAME) };
#[doc(alias = "GST_GL_COLOR_CONVERT_VIDEO_CAPS")]
pub static GL_COLOR_CONVERT_VIDEO_CAPS: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GST_GL_COLOR_CONVERT_VIDEO_CAPS) };
#[cfg(feature = "v1_20")]
#[cfg_attr(docsrs, doc(cfg(feature = "v1_20")))]
#[doc(alias = "GST_GL_CONFIG_STRUCTURE_NAME")]
pub static GL_CONFIG_STRUCTURE_NAME: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GST_GL_CONFIG_STRUCTURE_NAME) };
#[doc(alias = "GST_GL_CONTEXT_TYPE_CGL")]
pub static GL_CONTEXT_TYPE_CGL: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GST_GL_CONTEXT_TYPE_CGL) };
#[doc(alias = "GST_GL_CONTEXT_TYPE_EAGL")]
pub static GL_CONTEXT_TYPE_EAGL: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GST_GL_CONTEXT_TYPE_EAGL) };
#[doc(alias = "GST_GL_CONTEXT_TYPE_EGL")]
pub static GL_CONTEXT_TYPE_EGL: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GST_GL_CONTEXT_TYPE_EGL) };
#[doc(alias = "GST_GL_CONTEXT_TYPE_GLX")]
pub static GL_CONTEXT_TYPE_GLX: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GST_GL_CONTEXT_TYPE_GLX) };
#[doc(alias = "GST_GL_CONTEXT_TYPE_WGL")]
pub static GL_CONTEXT_TYPE_WGL: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GST_GL_CONTEXT_TYPE_WGL) };
#[doc(alias = "GST_GL_DISPLAY_CONTEXT_TYPE")]
pub static GL_DISPLAY_CONTEXT_TYPE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GST_GL_DISPLAY_CONTEXT_TYPE) };
#[doc(alias = "GST_GL_MEMORY_ALLOCATOR_NAME")]
pub static GL_MEMORY_ALLOCATOR_NAME: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GST_GL_MEMORY_ALLOCATOR_NAME) };
#[doc(alias = "GST_GL_MEMORY_PBO_ALLOCATOR_NAME")]
pub static GL_MEMORY_PBO_ALLOCATOR_NAME: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GST_GL_MEMORY_PBO_ALLOCATOR_NAME) };
#[doc(alias = "GST_GL_RENDERBUFFER_ALLOCATOR_NAME")]
pub static GL_RENDERBUFFER_ALLOCATOR_NAME: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GST_GL_RENDERBUFFER_ALLOCATOR_NAME) };
#[doc(alias = "GST_GL_TEXTURE_TARGET_2D_STR")]
pub static GL_TEXTURE_TARGET_2D_STR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GST_GL_TEXTURE_TARGET_2D_STR) };
#[doc(alias = "GST_GL_TEXTURE_TARGET_EXTERNAL_OES_STR")]
pub static GL_TEXTURE_TARGET_EXTERNAL_OES_STR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GST_GL_TEXTURE_TARGET_EXTERNAL_OES_STR) };
#[doc(alias = "GST_GL_TEXTURE_TARGET_RECTANGLE_STR")]
pub static GL_TEXTURE_TARGET_RECTANGLE_STR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GST_GL_TEXTURE_TARGET_RECTANGLE_STR) };
