// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
pub fn ser_object_lock_rule(
    input: &crate::types::ObjectLockRule,
    writer: ::aws_smithy_xml::encode::ElWriter,
) -> ::std::result::Result<(), ::aws_smithy_types::error::operation::SerializationError> {
    #[allow(unused_mut)]
    let mut scope = writer.finish();
    if let Some(var_1) = &input.default_retention {
        let inner_writer = scope.start_el("DefaultRetention");
        crate::protocol_serde::shape_default_retention::ser_default_retention(var_1, inner_writer)?
    }
    scope.finish();
    Ok(())
}

#[allow(clippy::needless_question_mark)]
pub fn de_object_lock_rule(
    decoder: &mut ::aws_smithy_xml::decode::ScopedDecoder,
) -> ::std::result::Result<crate::types::ObjectLockRule, ::aws_smithy_xml::decode::XmlDecodeError> {
    #[allow(unused_mut)]
    let mut builder = crate::types::ObjectLockRule::builder();
    while let Some(mut tag) = decoder.next_tag() {
        match tag.start_el() {
            s if s.matches("DefaultRetention") /* DefaultRetention com.amazonaws.s3#ObjectLockRule$DefaultRetention */ =>  {
                let var_2 =
                    Some(
                        crate::protocol_serde::shape_default_retention::de_default_retention(&mut tag)
                        ?
                    )
                ;
                builder = builder.set_default_retention(var_2);
            }
            ,
            _ => {}
        }
    }
    Ok(builder.build())
}
