// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT

mod net_client_clock;
pub use self::net_client_clock::NetClientClock;

mod net_time_provider;
pub use self::net_time_provider::NetTimeProvider;

mod ntp_clock;
pub use self::ntp_clock::NtpClock;

mod ptp_clock;
pub use self::ptp_clock::PtpClock;
