// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
impl super::Client {
    /// Constructs a fluent builder for the [`DescribeNotificationConfiguration`](crate::operation::describe_notification_configuration::builders::DescribeNotificationConfigurationFluentBuilder) operation.
    ///
    /// - The fluent builder is configurable:
    ///   - [`stream_name(impl Into<String>)`](crate::operation::describe_notification_configuration::builders::DescribeNotificationConfigurationFluentBuilder::stream_name) / [`set_stream_name(Option<String>)`](crate::operation::describe_notification_configuration::builders::DescribeNotificationConfigurationFluentBuilder::set_stream_name):<br>required: **false**<br><p>The name of the stream from which to retrieve the notification configuration. You must specify either the <code>StreamName</code> or the <code>StreamARN</code>.</p><br>
    ///   - [`stream_arn(impl Into<String>)`](crate::operation::describe_notification_configuration::builders::DescribeNotificationConfigurationFluentBuilder::stream_arn) / [`set_stream_arn(Option<String>)`](crate::operation::describe_notification_configuration::builders::DescribeNotificationConfigurationFluentBuilder::set_stream_arn):<br>required: **false**<br><p>The Amazon Resource Name (ARN) of the Kinesis video stream from where you want to retrieve the notification configuration. You must specify either the <code>StreamName</code> or the StreamARN.</p><br>
    /// - On success, responds with [`DescribeNotificationConfigurationOutput`](crate::operation::describe_notification_configuration::DescribeNotificationConfigurationOutput) with field(s):
    ///   - [`notification_configuration(Option<NotificationConfiguration>)`](crate::operation::describe_notification_configuration::DescribeNotificationConfigurationOutput::notification_configuration): <p>The structure that contains the information required for notifications. If the structure is null, the configuration will be deleted from the stream.</p>
    /// - On failure, responds with [`SdkError<DescribeNotificationConfigurationError>`](crate::operation::describe_notification_configuration::DescribeNotificationConfigurationError)
    pub fn describe_notification_configuration(
        &self,
    ) -> crate::operation::describe_notification_configuration::builders::DescribeNotificationConfigurationFluentBuilder {
        crate::operation::describe_notification_configuration::builders::DescribeNotificationConfigurationFluentBuilder::new(self.handle.clone())
    }
}
