// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
pub fn ser_update_image_generation_configuration_input_input(
    object: &mut ::aws_smithy_json::serialize::JsonObjectWriter,
    input: &crate::operation::update_image_generation_configuration::UpdateImageGenerationConfigurationInput,
) -> ::std::result::Result<(), ::aws_smithy_types::error::operation::SerializationError> {
    if let Some(var_1) = &input.image_generation_configuration {
        #[allow(unused_mut)]
        let mut object_2 = object.key("ImageGenerationConfiguration").start_object();
        crate::protocol_serde::shape_image_generation_configuration::ser_image_generation_configuration(&mut object_2, var_1)?;
        object_2.finish();
    }
    if let Some(var_3) = &input.stream_arn {
        object.key("StreamARN").string(var_3.as_str());
    }
    if let Some(var_4) = &input.stream_name {
        object.key("StreamName").string(var_4.as_str());
    }
    Ok(())
}
