// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT

#include "manual.h"
#include <stdalign.h>
#include <stdio.h>

int main() {
    printf("%s;%zu;%zu\n", "GstAppLeakyType", sizeof(GstAppLeakyType), alignof(GstAppLeakyType));
    printf("%s;%zu;%zu\n", "GstAppSink", sizeof(GstAppSink), alignof(GstAppSink));
    printf("%s;%zu;%zu\n", "GstAppSinkCallbacks", sizeof(GstAppSinkCallbacks), alignof(GstAppSinkCallbacks));
    printf("%s;%zu;%zu\n", "GstAppSinkClass", sizeof(GstAppSinkClass), alignof(GstAppSinkClass));
    printf("%s;%zu;%zu\n", "GstAppSrc", sizeof(GstAppSrc), alignof(GstAppSrc));
    printf("%s;%zu;%zu\n", "GstAppSrcCallbacks", sizeof(GstAppSrcCallbacks), alignof(GstAppSrcCallbacks));
    printf("%s;%zu;%zu\n", "GstAppSrcClass", sizeof(GstAppSrcClass), alignof(GstAppSrcClass));
    printf("%s;%zu;%zu\n", "GstAppStreamType", sizeof(GstAppStreamType), alignof(GstAppStreamType));
    return 0;
}
