// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
impl super::Client {
    /// Constructs a fluent builder for the [`GetBucketAbac`](crate::operation::get_bucket_abac::builders::GetBucketAbacFluentBuilder) operation.
    ///
    /// - The fluent builder is configurable:
    ///   - [`bucket(impl Into<String>)`](crate::operation::get_bucket_abac::builders::GetBucketAbacFluentBuilder::bucket) / [`set_bucket(Option<String>)`](crate::operation::get_bucket_abac::builders::GetBucketAbacFluentBuilder::set_bucket):<br>required: **true**<br><p>The name of the general purpose bucket.</p><br>
    ///   - [`expected_bucket_owner(impl Into<String>)`](crate::operation::get_bucket_abac::builders::GetBucketAbacFluentBuilder::expected_bucket_owner) / [`set_expected_bucket_owner(Option<String>)`](crate::operation::get_bucket_abac::builders::GetBucketAbacFluentBuilder::set_expected_bucket_owner):<br>required: **false**<br><p>The Amazon Web Services account ID of the general purpose bucket's owner.</p><br>
    /// - On success, responds with [`GetBucketAbacOutput`](crate::operation::get_bucket_abac::GetBucketAbacOutput) with field(s):
    ///   - [`abac_status(Option<AbacStatus>)`](crate::operation::get_bucket_abac::GetBucketAbacOutput::abac_status): <p>The ABAC status of the general purpose bucket.</p>
    /// - On failure, responds with [`SdkError<GetBucketAbacError>`](crate::operation::get_bucket_abac::GetBucketAbacError)
    pub fn get_bucket_abac(&self) -> crate::operation::get_bucket_abac::builders::GetBucketAbacFluentBuilder {
        crate::operation::get_bucket_abac::builders::GetBucketAbacFluentBuilder::new(self.handle.clone())
    }
}
