// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
pub fn ser_select_parameters(
    input: &crate::types::SelectParameters,
    writer: ::aws_smithy_xml::encode::ElWriter,
) -> ::std::result::Result<(), ::aws_smithy_types::error::operation::SerializationError> {
    #[allow(unused_mut)]
    let mut scope = writer.finish();
    if let Some(var_1) = &input.input_serialization {
        let inner_writer = scope.start_el("InputSerialization");
        crate::protocol_serde::shape_input_serialization::ser_input_serialization(var_1, inner_writer)?
    }
    {
        let mut inner_writer = scope.start_el("ExpressionType").finish();
        inner_writer.data(input.expression_type.as_str());
    }
    {
        let mut inner_writer = scope.start_el("Expression").finish();
        inner_writer.data(input.expression.as_str());
    }
    if let Some(var_2) = &input.output_serialization {
        let inner_writer = scope.start_el("OutputSerialization");
        crate::protocol_serde::shape_output_serialization::ser_output_serialization(var_2, inner_writer)?
    }
    scope.finish();
    Ok(())
}
