// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
impl super::Client {
    /// Constructs a fluent builder for the [`DeleteStream`](crate::operation::delete_stream::builders::DeleteStreamFluentBuilder) operation.
    ///
    /// - The fluent builder is configurable:
    ///   - [`stream_arn(impl Into<String>)`](crate::operation::delete_stream::builders::DeleteStreamFluentBuilder::stream_arn) / [`set_stream_arn(Option<String>)`](crate::operation::delete_stream::builders::DeleteStreamFluentBuilder::set_stream_arn):<br>required: **true**<br><p>The Amazon Resource Name (ARN) of the stream that you want to delete.</p><br>
    ///   - [`current_version(impl Into<String>)`](crate::operation::delete_stream::builders::DeleteStreamFluentBuilder::current_version) / [`set_current_version(Option<String>)`](crate::operation::delete_stream::builders::DeleteStreamFluentBuilder::set_current_version):<br>required: **false**<br><p>Optional: The version of the stream that you want to delete.</p> <p>Specify the version as a safeguard to ensure that your are deleting the correct stream. To get the stream version, use the <code>DescribeStream</code> API.</p> <p>If not specified, only the <code>CreationTime</code> is checked before deleting the stream.</p><br>
    /// - On success, responds with [`DeleteStreamOutput`](crate::operation::delete_stream::DeleteStreamOutput)
    /// - On failure, responds with [`SdkError<DeleteStreamError>`](crate::operation::delete_stream::DeleteStreamError)
    pub fn delete_stream(&self) -> crate::operation::delete_stream::builders::DeleteStreamFluentBuilder {
        crate::operation::delete_stream::builders::DeleteStreamFluentBuilder::new(self.handle.clone())
    }
}
