pub mod barrier;
pub mod branch;
pub mod cmma;
pub mod synchronization;

/// Module containing compile-time information about the current runtime.
pub mod comptime;

mod base;
pub mod comptime_error;
mod const_expand;
mod container;
mod debug;
mod element;
mod indexation;
mod list;
mod operation;
mod options;
mod plane;
mod polyfills;
mod scalar;
mod topology;
mod trigonometry;
mod validation;

pub use branch::{RangeExpand, SteppedRangeExpand, range, range_stepped};
pub use const_expand::*;
pub use container::*;
pub use debug::*;
pub use element::*;
pub use indexation::*;
pub use list::*;
pub use operation::*;
pub use options::*;
pub use plane::*;
pub use polyfills::*;
pub use scalar::*;
pub use topology::*;
pub use trigonometry::*;
pub use validation::*;

pub use crate::{debug_print, debug_print_expand};
