// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
impl super::Client {
    /// Constructs a fluent builder for the [`DescribeStreamStorageConfiguration`](crate::operation::describe_stream_storage_configuration::builders::DescribeStreamStorageConfigurationFluentBuilder) operation.
    ///
    /// - The fluent builder is configurable:
    ///   - [`stream_name(impl Into<String>)`](crate::operation::describe_stream_storage_configuration::builders::DescribeStreamStorageConfigurationFluentBuilder::stream_name) / [`set_stream_name(Option<String>)`](crate::operation::describe_stream_storage_configuration::builders::DescribeStreamStorageConfigurationFluentBuilder::set_stream_name):<br>required: **false**<br><p>The name of the stream for which you want to retrieve the storage configuration.</p><br>
    ///   - [`stream_arn(impl Into<String>)`](crate::operation::describe_stream_storage_configuration::builders::DescribeStreamStorageConfigurationFluentBuilder::stream_arn) / [`set_stream_arn(Option<String>)`](crate::operation::describe_stream_storage_configuration::builders::DescribeStreamStorageConfigurationFluentBuilder::set_stream_arn):<br>required: **false**<br><p>The Amazon Resource Name (ARN) of the stream for which you want to retrieve the storage configuration.</p><br>
    /// - On success, responds with [`DescribeStreamStorageConfigurationOutput`](crate::operation::describe_stream_storage_configuration::DescribeStreamStorageConfigurationOutput) with field(s):
    ///   - [`stream_name(Option<String>)`](crate::operation::describe_stream_storage_configuration::DescribeStreamStorageConfigurationOutput::stream_name): <p>The name of the stream.</p>
    ///   - [`stream_arn(Option<String>)`](crate::operation::describe_stream_storage_configuration::DescribeStreamStorageConfigurationOutput::stream_arn): <p>The Amazon Resource Name (ARN) of the stream.</p>
    ///   - [`stream_storage_configuration(Option<StreamStorageConfiguration>)`](crate::operation::describe_stream_storage_configuration::DescribeStreamStorageConfigurationOutput::stream_storage_configuration): <p>The current storage configuration for the stream, including the default storage tier and other storage-related settings.</p>
    /// - On failure, responds with [`SdkError<DescribeStreamStorageConfigurationError>`](crate::operation::describe_stream_storage_configuration::DescribeStreamStorageConfigurationError)
    pub fn describe_stream_storage_configuration(
        &self,
    ) -> crate::operation::describe_stream_storage_configuration::builders::DescribeStreamStorageConfigurationFluentBuilder {
        crate::operation::describe_stream_storage_configuration::builders::DescribeStreamStorageConfigurationFluentBuilder::new(self.handle.clone())
    }
}
