// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT

use crate::ffi;
use glib::translate::*;

glib::wrapper! {
    #[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
    pub struct WebRTCICECandidateStats(Boxed<ffi::GstWebRTCICECandidateStats>);

    match fn {
        copy => |ptr| ffi::gst_webrtc_ice_candidate_stats_copy(mut_override(ptr)),
        free => |ptr| ffi::gst_webrtc_ice_candidate_stats_free(ptr),
        type_ => || ffi::gst_webrtc_ice_candidate_stats_get_type(),
    }
}

unsafe impl Send for WebRTCICECandidateStats {}
unsafe impl Sync for WebRTCICECandidateStats {}
