// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
#[allow(missing_docs)] // documentation missing in model
#[non_exhaustive]
#[derive(::std::clone::Clone, ::std::cmp::PartialEq, ::std::fmt::Debug)]
pub struct GetLexiconInput {
    /// <p>Name of the lexicon.</p>
    pub name: ::std::option::Option<::std::string::String>,
}
impl GetLexiconInput {
    /// <p>Name of the lexicon.</p>
    pub fn name(&self) -> ::std::option::Option<&str> {
        self.name.as_deref()
    }
}
impl GetLexiconInput {
    /// Creates a new builder-style object to manufacture [`GetLexiconInput`](crate::operation::get_lexicon::GetLexiconInput).
    pub fn builder() -> crate::operation::get_lexicon::builders::GetLexiconInputBuilder {
        crate::operation::get_lexicon::builders::GetLexiconInputBuilder::default()
    }
}

/// A builder for [`GetLexiconInput`](crate::operation::get_lexicon::GetLexiconInput).
#[derive(::std::clone::Clone, ::std::cmp::PartialEq, ::std::default::Default, ::std::fmt::Debug)]
#[non_exhaustive]
pub struct GetLexiconInputBuilder {
    pub(crate) name: ::std::option::Option<::std::string::String>,
}
impl GetLexiconInputBuilder {
    /// <p>Name of the lexicon.</p>
    /// This field is required.
    pub fn name(mut self, input: impl ::std::convert::Into<::std::string::String>) -> Self {
        self.name = ::std::option::Option::Some(input.into());
        self
    }
    /// <p>Name of the lexicon.</p>
    pub fn set_name(mut self, input: ::std::option::Option<::std::string::String>) -> Self {
        self.name = input;
        self
    }
    /// <p>Name of the lexicon.</p>
    pub fn get_name(&self) -> &::std::option::Option<::std::string::String> {
        &self.name
    }
    /// Consumes the builder and constructs a [`GetLexiconInput`](crate::operation::get_lexicon::GetLexiconInput).
    pub fn build(self) -> ::std::result::Result<crate::operation::get_lexicon::GetLexiconInput, ::aws_smithy_types::error::operation::BuildError> {
        ::std::result::Result::Ok(crate::operation::get_lexicon::GetLexiconInput { name: self.name })
    }
}
