/*
 * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * SPDX-License-Identifier: Apache-2.0
 */

//! Meta-providers that augment existing providers with new behavior

pub mod credentials;
pub mod region;
pub mod token;
