// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
impl super::Client {
    /// Constructs a fluent builder for the [`GetMedicalScribeStream`](crate::operation::get_medical_scribe_stream::builders::GetMedicalScribeStreamFluentBuilder) operation.
    ///
    /// - The fluent builder is configurable:
    ///   - [`session_id(impl Into<String>)`](crate::operation::get_medical_scribe_stream::builders::GetMedicalScribeStreamFluentBuilder::session_id) / [`set_session_id(Option<String>)`](crate::operation::get_medical_scribe_stream::builders::GetMedicalScribeStreamFluentBuilder::set_session_id):<br>required: **true**<br><p>The identifier of the HealthScribe streaming session you want information about.</p><br>
    /// - On success, responds with [`GetMedicalScribeStreamOutput`](crate::operation::get_medical_scribe_stream::GetMedicalScribeStreamOutput) with field(s):
    ///   - [`medical_scribe_stream_details(Option<MedicalScribeStreamDetails>)`](crate::operation::get_medical_scribe_stream::GetMedicalScribeStreamOutput::medical_scribe_stream_details): <p>Provides details about a HealthScribe streaming session.</p>
    /// - On failure, responds with [`SdkError<GetMedicalScribeStreamError>`](crate::operation::get_medical_scribe_stream::GetMedicalScribeStreamError)
    pub fn get_medical_scribe_stream(&self) -> crate::operation::get_medical_scribe_stream::builders::GetMedicalScribeStreamFluentBuilder {
        crate::operation::get_medical_scribe_stream::builders::GetMedicalScribeStreamFluentBuilder::new(self.handle.clone())
    }
}
