// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
#[allow(missing_docs)] // documentation missing in model
#[non_exhaustive]
#[derive(::std::clone::Clone, ::std::cmp::PartialEq, ::std::fmt::Debug)]
pub struct CreateSignalingChannelOutput {
    /// <p>The Amazon Resource Name (ARN) of the created channel.</p>
    pub channel_arn: ::std::option::Option<::std::string::String>,
    _request_id: Option<String>,
}
impl CreateSignalingChannelOutput {
    /// <p>The Amazon Resource Name (ARN) of the created channel.</p>
    pub fn channel_arn(&self) -> ::std::option::Option<&str> {
        self.channel_arn.as_deref()
    }
}
impl ::aws_types::request_id::RequestId for CreateSignalingChannelOutput {
    fn request_id(&self) -> Option<&str> {
        self._request_id.as_deref()
    }
}
impl CreateSignalingChannelOutput {
    /// Creates a new builder-style object to manufacture [`CreateSignalingChannelOutput`](crate::operation::create_signaling_channel::CreateSignalingChannelOutput).
    pub fn builder() -> crate::operation::create_signaling_channel::builders::CreateSignalingChannelOutputBuilder {
        crate::operation::create_signaling_channel::builders::CreateSignalingChannelOutputBuilder::default()
    }
}

/// A builder for [`CreateSignalingChannelOutput`](crate::operation::create_signaling_channel::CreateSignalingChannelOutput).
#[derive(::std::clone::Clone, ::std::cmp::PartialEq, ::std::default::Default, ::std::fmt::Debug)]
#[non_exhaustive]
pub struct CreateSignalingChannelOutputBuilder {
    pub(crate) channel_arn: ::std::option::Option<::std::string::String>,
    _request_id: Option<String>,
}
impl CreateSignalingChannelOutputBuilder {
    /// <p>The Amazon Resource Name (ARN) of the created channel.</p>
    pub fn channel_arn(mut self, input: impl ::std::convert::Into<::std::string::String>) -> Self {
        self.channel_arn = ::std::option::Option::Some(input.into());
        self
    }
    /// <p>The Amazon Resource Name (ARN) of the created channel.</p>
    pub fn set_channel_arn(mut self, input: ::std::option::Option<::std::string::String>) -> Self {
        self.channel_arn = input;
        self
    }
    /// <p>The Amazon Resource Name (ARN) of the created channel.</p>
    pub fn get_channel_arn(&self) -> &::std::option::Option<::std::string::String> {
        &self.channel_arn
    }
    pub(crate) fn _request_id(mut self, request_id: impl Into<String>) -> Self {
        self._request_id = Some(request_id.into());
        self
    }

    pub(crate) fn _set_request_id(&mut self, request_id: Option<String>) -> &mut Self {
        self._request_id = request_id;
        self
    }
    /// Consumes the builder and constructs a [`CreateSignalingChannelOutput`](crate::operation::create_signaling_channel::CreateSignalingChannelOutput).
    pub fn build(self) -> crate::operation::create_signaling_channel::CreateSignalingChannelOutput {
        crate::operation::create_signaling_channel::CreateSignalingChannelOutput {
            channel_arn: self.channel_arn,
            _request_id: self._request_id,
        }
    }
}
