// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
pub fn ser_text_event(
    object: &mut ::aws_smithy_json::serialize::JsonObjectWriter,
    input: &crate::types::TextEvent,
) -> ::std::result::Result<(), ::aws_smithy_types::error::operation::SerializationError> {
    {
        object.key("Text").string(input.text.as_str());
    }
    if let Some(var_1) = &input.text_type {
        object.key("TextType").string(var_1.as_str());
    }
    if let Some(var_2) = &input.flush_stream_configuration {
        #[allow(unused_mut)]
        let mut object_3 = object.key("FlushStreamConfiguration").start_object();
        crate::protocol_serde::shape_flush_stream_configuration::ser_flush_stream_configuration(&mut object_3, var_2)?;
        object_3.finish();
    }
    Ok(())
}
