// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT

#include "manual.h"
#include <stdalign.h>
#include <stdio.h>

int main() {
    printf("%s;%zu;%zu\n", "GstTagDemux", sizeof(GstTagDemux), alignof(GstTagDemux));
    printf("%s;%zu;%zu\n", "GstTagDemuxClass", sizeof(GstTagDemuxClass), alignof(GstTagDemuxClass));
    printf("%s;%zu;%zu\n", "GstTagDemuxResult", sizeof(GstTagDemuxResult), alignof(GstTagDemuxResult));
    printf("%s;%zu;%zu\n", "GstTagImageType", sizeof(GstTagImageType), alignof(GstTagImageType));
    printf("%s;%zu;%zu\n", "GstTagLicenseFlags", sizeof(GstTagLicenseFlags), alignof(GstTagLicenseFlags));
    printf("%s;%zu;%zu\n", "GstTagMux", sizeof(GstTagMux), alignof(GstTagMux));
    printf("%s;%zu;%zu\n", "GstTagMuxClass", sizeof(GstTagMuxClass), alignof(GstTagMuxClass));
    printf("%s;%zu;%zu\n", "GstTagXmpWriterInterface", sizeof(GstTagXmpWriterInterface), alignof(GstTagXmpWriterInterface));
    return 0;
}
