// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT

#include "manual.h"
#include <stdalign.h>
#include <stdio.h>

int main() {
    printf("%s;%zu;%zu\n", "GstWebRTCBundlePolicy", sizeof(GstWebRTCBundlePolicy), alignof(GstWebRTCBundlePolicy));
    printf("%s;%zu;%zu\n", "GstWebRTCDTLSRole", sizeof(GstWebRTCDTLSRole), alignof(GstWebRTCDTLSRole));
    printf("%s;%zu;%zu\n", "GstWebRTCDTLSSetup", sizeof(GstWebRTCDTLSSetup), alignof(GstWebRTCDTLSSetup));
    printf("%s;%zu;%zu\n", "GstWebRTCDTLSTransportState", sizeof(GstWebRTCDTLSTransportState), alignof(GstWebRTCDTLSTransportState));
    printf("%s;%zu;%zu\n", "GstWebRTCDataChannelState", sizeof(GstWebRTCDataChannelState), alignof(GstWebRTCDataChannelState));
    printf("%s;%zu;%zu\n", "GstWebRTCError", sizeof(GstWebRTCError), alignof(GstWebRTCError));
    printf("%s;%zu;%zu\n", "GstWebRTCFECType", sizeof(GstWebRTCFECType), alignof(GstWebRTCFECType));
    printf("%s;%zu;%zu\n", "GstWebRTCICE", sizeof(GstWebRTCICE), alignof(GstWebRTCICE));
    printf("%s;%zu;%zu\n", "GstWebRTCICECandidate", sizeof(GstWebRTCICECandidate), alignof(GstWebRTCICECandidate));
    printf("%s;%zu;%zu\n", "GstWebRTCICECandidatePair", sizeof(GstWebRTCICECandidatePair), alignof(GstWebRTCICECandidatePair));
    printf("%s;%zu;%zu\n", "GstWebRTCICECandidateProtocolType", sizeof(GstWebRTCICECandidateProtocolType), alignof(GstWebRTCICECandidateProtocolType));
    printf("%s;%zu;%zu\n", "GstWebRTCICECandidateStats", sizeof(GstWebRTCICECandidateStats), alignof(GstWebRTCICECandidateStats));
    printf("%s;%zu;%zu\n", "GstWebRTCICECandidateType", sizeof(GstWebRTCICECandidateType), alignof(GstWebRTCICECandidateType));
    printf("%s;%zu;%zu\n", "GstWebRTCICEClass", sizeof(GstWebRTCICEClass), alignof(GstWebRTCICEClass));
    printf("%s;%zu;%zu\n", "GstWebRTCICEComponent", sizeof(GstWebRTCICEComponent), alignof(GstWebRTCICEComponent));
    printf("%s;%zu;%zu\n", "GstWebRTCICEConnectionState", sizeof(GstWebRTCICEConnectionState), alignof(GstWebRTCICEConnectionState));
    printf("%s;%zu;%zu\n", "GstWebRTCICEGatheringState", sizeof(GstWebRTCICEGatheringState), alignof(GstWebRTCICEGatheringState));
    printf("%s;%zu;%zu\n", "GstWebRTCICERole", sizeof(GstWebRTCICERole), alignof(GstWebRTCICERole));
    printf("%s;%zu;%zu\n", "GstWebRTCICEStream", sizeof(GstWebRTCICEStream), alignof(GstWebRTCICEStream));
    printf("%s;%zu;%zu\n", "GstWebRTCICEStreamClass", sizeof(GstWebRTCICEStreamClass), alignof(GstWebRTCICEStreamClass));
    printf("%s;%zu;%zu\n", "GstWebRTCICETcpCandidateType", sizeof(GstWebRTCICETcpCandidateType), alignof(GstWebRTCICETcpCandidateType));
    printf("%s;%zu;%zu\n", "GstWebRTCICETransport", sizeof(GstWebRTCICETransport), alignof(GstWebRTCICETransport));
    printf("%s;%zu;%zu\n", "GstWebRTCICETransportClass", sizeof(GstWebRTCICETransportClass), alignof(GstWebRTCICETransportClass));
    printf("%s;%zu;%zu\n", "GstWebRTCICETransportPolicy", sizeof(GstWebRTCICETransportPolicy), alignof(GstWebRTCICETransportPolicy));
    printf("%s;%zu;%zu\n", "GstWebRTCKind", sizeof(GstWebRTCKind), alignof(GstWebRTCKind));
    printf("%s;%zu;%zu\n", "GstWebRTCPeerConnectionState", sizeof(GstWebRTCPeerConnectionState), alignof(GstWebRTCPeerConnectionState));
    printf("%s;%zu;%zu\n", "GstWebRTCPriorityType", sizeof(GstWebRTCPriorityType), alignof(GstWebRTCPriorityType));
    printf("%s;%zu;%zu\n", "GstWebRTCRTPTransceiverDirection", sizeof(GstWebRTCRTPTransceiverDirection), alignof(GstWebRTCRTPTransceiverDirection));
    printf("%s;%zu;%zu\n", "GstWebRTCSCTPTransportState", sizeof(GstWebRTCSCTPTransportState), alignof(GstWebRTCSCTPTransportState));
    printf("%s;%zu;%zu\n", "GstWebRTCSDPType", sizeof(GstWebRTCSDPType), alignof(GstWebRTCSDPType));
    printf("%s;%zu;%zu\n", "GstWebRTCSessionDescription", sizeof(GstWebRTCSessionDescription), alignof(GstWebRTCSessionDescription));
    printf("%s;%zu;%zu\n", "GstWebRTCSignalingState", sizeof(GstWebRTCSignalingState), alignof(GstWebRTCSignalingState));
    printf("%s;%zu;%zu\n", "GstWebRTCStatsType", sizeof(GstWebRTCStatsType), alignof(GstWebRTCStatsType));
    return 0;
}
