// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
pub fn ser_create_stream_input_input(
    object: &mut ::aws_smithy_json::serialize::JsonObjectWriter,
    input: &crate::operation::create_stream::CreateStreamInput,
) -> ::std::result::Result<(), ::aws_smithy_types::error::operation::SerializationError> {
    if let Some(var_1) = &input.data_retention_in_hours {
        object.key("DataRetentionInHours").number(
            #[allow(clippy::useless_conversion)]
            ::aws_smithy_types::Number::NegInt((*var_1).into()),
        );
    }
    if let Some(var_2) = &input.device_name {
        object.key("DeviceName").string(var_2.as_str());
    }
    if let Some(var_3) = &input.kms_key_id {
        object.key("KmsKeyId").string(var_3.as_str());
    }
    if let Some(var_4) = &input.media_type {
        object.key("MediaType").string(var_4.as_str());
    }
    if let Some(var_5) = &input.stream_name {
        object.key("StreamName").string(var_5.as_str());
    }
    if let Some(var_6) = &input.stream_storage_configuration {
        #[allow(unused_mut)]
        let mut object_7 = object.key("StreamStorageConfiguration").start_object();
        crate::protocol_serde::shape_stream_storage_configuration::ser_stream_storage_configuration(&mut object_7, var_6)?;
        object_7.finish();
    }
    if let Some(var_8) = &input.tags {
        #[allow(unused_mut)]
        let mut object_9 = object.key("Tags").start_object();
        for (key_10, value_11) in var_8 {
            {
                object_9.key(key_10.as_str()).string(value_11.as_str());
            }
        }
        object_9.finish();
    }
    Ok(())
}
