// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
#[allow(missing_docs)] // documentation missing in model
#[non_exhaustive]
#[derive(::std::clone::Clone, ::std::cmp::PartialEq, ::std::fmt::Debug)]
pub struct DeleteBucketMetadataTableConfigurationInput {
    /// <p>The general purpose bucket that you want to remove the metadata table configuration from.</p>
    pub bucket: ::std::option::Option<::std::string::String>,
    /// <p>The expected bucket owner of the general purpose bucket that you want to remove the metadata table configuration from.</p>
    pub expected_bucket_owner: ::std::option::Option<::std::string::String>,
}
impl DeleteBucketMetadataTableConfigurationInput {
    /// <p>The general purpose bucket that you want to remove the metadata table configuration from.</p>
    pub fn bucket(&self) -> ::std::option::Option<&str> {
        self.bucket.as_deref()
    }
    /// <p>The expected bucket owner of the general purpose bucket that you want to remove the metadata table configuration from.</p>
    pub fn expected_bucket_owner(&self) -> ::std::option::Option<&str> {
        self.expected_bucket_owner.as_deref()
    }
}
impl DeleteBucketMetadataTableConfigurationInput {
    /// Creates a new builder-style object to manufacture [`DeleteBucketMetadataTableConfigurationInput`](crate::operation::delete_bucket_metadata_table_configuration::DeleteBucketMetadataTableConfigurationInput).
    pub fn builder() -> crate::operation::delete_bucket_metadata_table_configuration::builders::DeleteBucketMetadataTableConfigurationInputBuilder {
        crate::operation::delete_bucket_metadata_table_configuration::builders::DeleteBucketMetadataTableConfigurationInputBuilder::default()
    }
}

/// A builder for [`DeleteBucketMetadataTableConfigurationInput`](crate::operation::delete_bucket_metadata_table_configuration::DeleteBucketMetadataTableConfigurationInput).
#[derive(::std::clone::Clone, ::std::cmp::PartialEq, ::std::default::Default, ::std::fmt::Debug)]
#[non_exhaustive]
pub struct DeleteBucketMetadataTableConfigurationInputBuilder {
    pub(crate) bucket: ::std::option::Option<::std::string::String>,
    pub(crate) expected_bucket_owner: ::std::option::Option<::std::string::String>,
}
impl DeleteBucketMetadataTableConfigurationInputBuilder {
    /// <p>The general purpose bucket that you want to remove the metadata table configuration from.</p>
    /// This field is required.
    pub fn bucket(mut self, input: impl ::std::convert::Into<::std::string::String>) -> Self {
        self.bucket = ::std::option::Option::Some(input.into());
        self
    }
    /// <p>The general purpose bucket that you want to remove the metadata table configuration from.</p>
    pub fn set_bucket(mut self, input: ::std::option::Option<::std::string::String>) -> Self {
        self.bucket = input;
        self
    }
    /// <p>The general purpose bucket that you want to remove the metadata table configuration from.</p>
    pub fn get_bucket(&self) -> &::std::option::Option<::std::string::String> {
        &self.bucket
    }
    /// <p>The expected bucket owner of the general purpose bucket that you want to remove the metadata table configuration from.</p>
    pub fn expected_bucket_owner(mut self, input: impl ::std::convert::Into<::std::string::String>) -> Self {
        self.expected_bucket_owner = ::std::option::Option::Some(input.into());
        self
    }
    /// <p>The expected bucket owner of the general purpose bucket that you want to remove the metadata table configuration from.</p>
    pub fn set_expected_bucket_owner(mut self, input: ::std::option::Option<::std::string::String>) -> Self {
        self.expected_bucket_owner = input;
        self
    }
    /// <p>The expected bucket owner of the general purpose bucket that you want to remove the metadata table configuration from.</p>
    pub fn get_expected_bucket_owner(&self) -> &::std::option::Option<::std::string::String> {
        &self.expected_bucket_owner
    }
    /// Consumes the builder and constructs a [`DeleteBucketMetadataTableConfigurationInput`](crate::operation::delete_bucket_metadata_table_configuration::DeleteBucketMetadataTableConfigurationInput).
    pub fn build(
        self,
    ) -> ::std::result::Result<
        crate::operation::delete_bucket_metadata_table_configuration::DeleteBucketMetadataTableConfigurationInput,
        ::aws_smithy_types::error::operation::BuildError,
    > {
        ::std::result::Result::Ok(
            crate::operation::delete_bucket_metadata_table_configuration::DeleteBucketMetadataTableConfigurationInput {
                bucket: self.bucket,
                expected_bucket_owner: self.expected_bucket_owner,
            },
        )
    }
}
