// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
impl super::Client {
    /// Constructs a fluent builder for the [`GetIceServerConfig`](crate::operation::get_ice_server_config::builders::GetIceServerConfigFluentBuilder) operation.
    ///
    /// - The fluent builder is configurable:
    ///   - [`channel_arn(impl Into<String>)`](crate::operation::get_ice_server_config::builders::GetIceServerConfigFluentBuilder::channel_arn) / [`set_channel_arn(Option<String>)`](crate::operation::get_ice_server_config::builders::GetIceServerConfigFluentBuilder::set_channel_arn):<br>required: **true**<br><p>The ARN of the signaling channel to be used for the peer-to-peer connection between configured peers.</p><br>
    ///   - [`client_id(impl Into<String>)`](crate::operation::get_ice_server_config::builders::GetIceServerConfigFluentBuilder::client_id) / [`set_client_id(Option<String>)`](crate::operation::get_ice_server_config::builders::GetIceServerConfigFluentBuilder::set_client_id):<br>required: **false**<br><p>Unique identifier for the viewer. Must be unique within the signaling channel.</p><br>
    ///   - [`service(Service)`](crate::operation::get_ice_server_config::builders::GetIceServerConfigFluentBuilder::service) / [`set_service(Option<Service>)`](crate::operation::get_ice_server_config::builders::GetIceServerConfigFluentBuilder::set_service):<br>required: **false**<br><p>Specifies the desired service. Currently, <code>TURN</code> is the only valid value.</p><br>
    ///   - [`username(impl Into<String>)`](crate::operation::get_ice_server_config::builders::GetIceServerConfigFluentBuilder::username) / [`set_username(Option<String>)`](crate::operation::get_ice_server_config::builders::GetIceServerConfigFluentBuilder::set_username):<br>required: **false**<br><p>An optional user ID to be associated with the credentials.</p><br>
    /// - On success, responds with [`GetIceServerConfigOutput`](crate::operation::get_ice_server_config::GetIceServerConfigOutput) with field(s):
    ///   - [`ice_server_list(Option<Vec::<IceServer>>)`](crate::operation::get_ice_server_config::GetIceServerConfigOutput::ice_server_list): <p>The list of ICE server information objects.</p>
    /// - On failure, responds with [`SdkError<GetIceServerConfigError>`](crate::operation::get_ice_server_config::GetIceServerConfigError)
    pub fn get_ice_server_config(&self) -> crate::operation::get_ice_server_config::builders::GetIceServerConfigFluentBuilder {
        crate::operation::get_ice_server_config::builders::GetIceServerConfigFluentBuilder::new(self.handle.clone())
    }
}
