// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
impl super::Client {
    /// Constructs a fluent builder for the [`DecodeAuthorizationMessage`](crate::operation::decode_authorization_message::builders::DecodeAuthorizationMessageFluentBuilder) operation.
    ///
    /// - The fluent builder is configurable:
    ///   - [`encoded_message(impl Into<String>)`](crate::operation::decode_authorization_message::builders::DecodeAuthorizationMessageFluentBuilder::encoded_message) / [`set_encoded_message(Option<String>)`](crate::operation::decode_authorization_message::builders::DecodeAuthorizationMessageFluentBuilder::set_encoded_message):<br>required: **true**<br><p>The encoded message that was returned with the response.</p><br>
    /// - On success, responds with [`DecodeAuthorizationMessageOutput`](crate::operation::decode_authorization_message::DecodeAuthorizationMessageOutput) with field(s):
    ///   - [`decoded_message(Option<String>)`](crate::operation::decode_authorization_message::DecodeAuthorizationMessageOutput::decoded_message): <p>The API returns a response with the decoded message.</p>
    /// - On failure, responds with [`SdkError<DecodeAuthorizationMessageError>`](crate::operation::decode_authorization_message::DecodeAuthorizationMessageError)
    pub fn decode_authorization_message(&self) -> crate::operation::decode_authorization_message::builders::DecodeAuthorizationMessageFluentBuilder {
        crate::operation::decode_authorization_message::builders::DecodeAuthorizationMessageFluentBuilder::new(self.handle.clone())
    }
}
