// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
impl super::Client {
    /// Constructs a fluent builder for the [`ListEdgeAgentConfigurations`](crate::operation::list_edge_agent_configurations::builders::ListEdgeAgentConfigurationsFluentBuilder) operation.
    /// This operation supports pagination; See [`into_paginator()`](crate::operation::list_edge_agent_configurations::builders::ListEdgeAgentConfigurationsFluentBuilder::into_paginator).
    ///
    /// - The fluent builder is configurable:
    ///   - [`hub_device_arn(impl Into<String>)`](crate::operation::list_edge_agent_configurations::builders::ListEdgeAgentConfigurationsFluentBuilder::hub_device_arn) / [`set_hub_device_arn(Option<String>)`](crate::operation::list_edge_agent_configurations::builders::ListEdgeAgentConfigurationsFluentBuilder::set_hub_device_arn):<br>required: **true**<br><p>The "Internet of Things (IoT) Thing" Arn of the edge agent.</p><br>
    ///   - [`max_results(i32)`](crate::operation::list_edge_agent_configurations::builders::ListEdgeAgentConfigurationsFluentBuilder::max_results) / [`set_max_results(Option<i32>)`](crate::operation::list_edge_agent_configurations::builders::ListEdgeAgentConfigurationsFluentBuilder::set_max_results):<br>required: **false**<br><p>The maximum number of edge configurations to return in the response. The default is 5.</p><br>
    ///   - [`next_token(impl Into<String>)`](crate::operation::list_edge_agent_configurations::builders::ListEdgeAgentConfigurationsFluentBuilder::next_token) / [`set_next_token(Option<String>)`](crate::operation::list_edge_agent_configurations::builders::ListEdgeAgentConfigurationsFluentBuilder::set_next_token):<br>required: **false**<br><p>If you specify this parameter, when the result of a <code>ListEdgeAgentConfigurations</code> operation is truncated, the call returns the <code>NextToken</code> in the response. To get another batch of edge configurations, provide this token in your next request.</p><br>
    /// - On success, responds with [`ListEdgeAgentConfigurationsOutput`](crate::operation::list_edge_agent_configurations::ListEdgeAgentConfigurationsOutput) with field(s):
    ///   - [`edge_configs(Option<Vec::<ListEdgeAgentConfigurationsEdgeConfig>>)`](crate::operation::list_edge_agent_configurations::ListEdgeAgentConfigurationsOutput::edge_configs): <p>A description of a single stream's edge configuration.</p>
    ///   - [`next_token(Option<String>)`](crate::operation::list_edge_agent_configurations::ListEdgeAgentConfigurationsOutput::next_token): <p>If the response is truncated, the call returns this element with a given token. To get the next batch of edge configurations, use this token in your next request.</p>
    /// - On failure, responds with [`SdkError<ListEdgeAgentConfigurationsError>`](crate::operation::list_edge_agent_configurations::ListEdgeAgentConfigurationsError)
    pub fn list_edge_agent_configurations(
        &self,
    ) -> crate::operation::list_edge_agent_configurations::builders::ListEdgeAgentConfigurationsFluentBuilder {
        crate::operation::list_edge_agent_configurations::builders::ListEdgeAgentConfigurationsFluentBuilder::new(self.handle.clone())
    }
}
