// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
pub use crate::types::error::_lexicon_not_found_exception::LexiconNotFoundExceptionBuilder;

pub use crate::types::error::_service_failure_exception::ServiceFailureExceptionBuilder;

pub use crate::types::error::_invalid_next_token_exception::InvalidNextTokenExceptionBuilder;

pub use crate::types::error::_invalid_task_id_exception::InvalidTaskIdExceptionBuilder;

pub use crate::types::error::_synthesis_task_not_found_exception::SynthesisTaskNotFoundExceptionBuilder;

pub use crate::types::error::_invalid_lexicon_exception::InvalidLexiconExceptionBuilder;

pub use crate::types::error::_lexicon_size_exceeded_exception::LexiconSizeExceededExceptionBuilder;

pub use crate::types::error::_max_lexeme_length_exceeded_exception::MaxLexemeLengthExceededExceptionBuilder;

pub use crate::types::error::_max_lexicons_number_exceeded_exception::MaxLexiconsNumberExceededExceptionBuilder;

pub use crate::types::error::_unsupported_pls_alphabet_exception::UnsupportedPlsAlphabetExceptionBuilder;

pub use crate::types::error::_unsupported_pls_language_exception::UnsupportedPlsLanguageExceptionBuilder;

pub use crate::types::error::_service_quota_exceeded_exception::ServiceQuotaExceededExceptionBuilder;

pub use crate::types::error::_throttling_exception::ThrottlingExceptionBuilder;

pub use crate::types::error::_validation_exception::ValidationExceptionBuilder;

pub use crate::types::error::_engine_not_supported_exception::EngineNotSupportedExceptionBuilder;

pub use crate::types::error::_invalid_s3_bucket_exception::InvalidS3BucketExceptionBuilder;

pub use crate::types::error::_invalid_s3_key_exception::InvalidS3KeyExceptionBuilder;

pub use crate::types::error::_invalid_sample_rate_exception::InvalidSampleRateExceptionBuilder;

pub use crate::types::error::_invalid_sns_topic_arn_exception::InvalidSnsTopicArnExceptionBuilder;

pub use crate::types::error::_invalid_ssml_exception::InvalidSsmlExceptionBuilder;

pub use crate::types::error::_language_not_supported_exception::LanguageNotSupportedExceptionBuilder;

pub use crate::types::error::_marks_not_supported_for_format_exception::MarksNotSupportedForFormatExceptionBuilder;

pub use crate::types::error::_ssml_marks_not_supported_for_text_type_exception::SsmlMarksNotSupportedForTextTypeExceptionBuilder;

pub use crate::types::error::_text_length_exceeded_exception::TextLengthExceededExceptionBuilder;
