// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
pub fn ser_policy_http_payload(
    payload: ::std::option::Option<::std::string::String>,
) -> ::std::result::Result<::std::vec::Vec<u8>, ::aws_smithy_types::error::operation::BuildError> {
    let payload = match payload {
        Some(t) => t,
        None => return Ok(Vec::new()),
    };
    Ok(payload.into_bytes())
}
