// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
#[allow(missing_docs)] // documentation missing in model
#[non_exhaustive]
#[derive(::std::clone::Clone, ::std::cmp::PartialEq, ::std::fmt::Debug)]
pub struct DescribeVoicesOutput {
    /// <p>A list of voices with their properties.</p>
    pub voices: ::std::option::Option<::std::vec::Vec<crate::types::Voice>>,
    /// <p>The pagination token to use in the next request to continue the listing of voices. <code>NextToken</code> is returned only if the response is truncated.</p>
    pub next_token: ::std::option::Option<::std::string::String>,
    _request_id: Option<String>,
}
impl DescribeVoicesOutput {
    /// <p>A list of voices with their properties.</p>
    ///
    /// If no value was sent for this field, a default will be set. If you want to determine if no value was sent, use `.voices.is_none()`.
    pub fn voices(&self) -> &[crate::types::Voice] {
        self.voices.as_deref().unwrap_or_default()
    }
    /// <p>The pagination token to use in the next request to continue the listing of voices. <code>NextToken</code> is returned only if the response is truncated.</p>
    pub fn next_token(&self) -> ::std::option::Option<&str> {
        self.next_token.as_deref()
    }
}
impl ::aws_types::request_id::RequestId for DescribeVoicesOutput {
    fn request_id(&self) -> Option<&str> {
        self._request_id.as_deref()
    }
}
impl DescribeVoicesOutput {
    /// Creates a new builder-style object to manufacture [`DescribeVoicesOutput`](crate::operation::describe_voices::DescribeVoicesOutput).
    pub fn builder() -> crate::operation::describe_voices::builders::DescribeVoicesOutputBuilder {
        crate::operation::describe_voices::builders::DescribeVoicesOutputBuilder::default()
    }
}

/// A builder for [`DescribeVoicesOutput`](crate::operation::describe_voices::DescribeVoicesOutput).
#[derive(::std::clone::Clone, ::std::cmp::PartialEq, ::std::default::Default, ::std::fmt::Debug)]
#[non_exhaustive]
pub struct DescribeVoicesOutputBuilder {
    pub(crate) voices: ::std::option::Option<::std::vec::Vec<crate::types::Voice>>,
    pub(crate) next_token: ::std::option::Option<::std::string::String>,
    _request_id: Option<String>,
}
impl DescribeVoicesOutputBuilder {
    /// Appends an item to `voices`.
    ///
    /// To override the contents of this collection use [`set_voices`](Self::set_voices).
    ///
    /// <p>A list of voices with their properties.</p>
    pub fn voices(mut self, input: crate::types::Voice) -> Self {
        let mut v = self.voices.unwrap_or_default();
        v.push(input);
        self.voices = ::std::option::Option::Some(v);
        self
    }
    /// <p>A list of voices with their properties.</p>
    pub fn set_voices(mut self, input: ::std::option::Option<::std::vec::Vec<crate::types::Voice>>) -> Self {
        self.voices = input;
        self
    }
    /// <p>A list of voices with their properties.</p>
    pub fn get_voices(&self) -> &::std::option::Option<::std::vec::Vec<crate::types::Voice>> {
        &self.voices
    }
    /// <p>The pagination token to use in the next request to continue the listing of voices. <code>NextToken</code> is returned only if the response is truncated.</p>
    pub fn next_token(mut self, input: impl ::std::convert::Into<::std::string::String>) -> Self {
        self.next_token = ::std::option::Option::Some(input.into());
        self
    }
    /// <p>The pagination token to use in the next request to continue the listing of voices. <code>NextToken</code> is returned only if the response is truncated.</p>
    pub fn set_next_token(mut self, input: ::std::option::Option<::std::string::String>) -> Self {
        self.next_token = input;
        self
    }
    /// <p>The pagination token to use in the next request to continue the listing of voices. <code>NextToken</code> is returned only if the response is truncated.</p>
    pub fn get_next_token(&self) -> &::std::option::Option<::std::string::String> {
        &self.next_token
    }
    pub(crate) fn _request_id(mut self, request_id: impl Into<String>) -> Self {
        self._request_id = Some(request_id.into());
        self
    }

    pub(crate) fn _set_request_id(&mut self, request_id: Option<String>) -> &mut Self {
        self._request_id = request_id;
        self
    }
    /// Consumes the builder and constructs a [`DescribeVoicesOutput`](crate::operation::describe_voices::DescribeVoicesOutput).
    pub fn build(self) -> crate::operation::describe_voices::DescribeVoicesOutput {
        crate::operation::describe_voices::DescribeVoicesOutput {
            voices: self.voices,
            next_token: self.next_token,
            _request_id: self._request_id,
        }
    }
}
