// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
impl super::Client {
    /// Constructs a fluent builder for the [`UpdateSignalingChannel`](crate::operation::update_signaling_channel::builders::UpdateSignalingChannelFluentBuilder) operation.
    ///
    /// - The fluent builder is configurable:
    ///   - [`channel_arn(impl Into<String>)`](crate::operation::update_signaling_channel::builders::UpdateSignalingChannelFluentBuilder::channel_arn) / [`set_channel_arn(Option<String>)`](crate::operation::update_signaling_channel::builders::UpdateSignalingChannelFluentBuilder::set_channel_arn):<br>required: **true**<br><p>The Amazon Resource Name (ARN) of the signaling channel that you want to update.</p><br>
    ///   - [`current_version(impl Into<String>)`](crate::operation::update_signaling_channel::builders::UpdateSignalingChannelFluentBuilder::current_version) / [`set_current_version(Option<String>)`](crate::operation::update_signaling_channel::builders::UpdateSignalingChannelFluentBuilder::set_current_version):<br>required: **true**<br><p>The current version of the signaling channel that you want to update.</p><br>
    ///   - [`single_master_configuration(SingleMasterConfiguration)`](crate::operation::update_signaling_channel::builders::UpdateSignalingChannelFluentBuilder::single_master_configuration) / [`set_single_master_configuration(Option<SingleMasterConfiguration>)`](crate::operation::update_signaling_channel::builders::UpdateSignalingChannelFluentBuilder::set_single_master_configuration):<br>required: **false**<br><p>The structure containing the configuration for the <code>SINGLE_MASTER</code> type of the signaling channel that you want to update. This parameter and the channel message's time-to-live are required for channels with the <code>SINGLE_MASTER</code> channel type.</p><br>
    /// - On success, responds with [`UpdateSignalingChannelOutput`](crate::operation::update_signaling_channel::UpdateSignalingChannelOutput)
    /// - On failure, responds with [`SdkError<UpdateSignalingChannelError>`](crate::operation::update_signaling_channel::UpdateSignalingChannelError)
    pub fn update_signaling_channel(&self) -> crate::operation::update_signaling_channel::builders::UpdateSignalingChannelFluentBuilder {
        crate::operation::update_signaling_channel::builders::UpdateSignalingChannelFluentBuilder::new(self.handle.clone())
    }
}
