// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT

#include "manual.h"
#include <stdalign.h>
#include <stdio.h>

int main() {
    printf("%s;%zu;%zu\n", "GstAnalyticsBatchMeta", sizeof(GstAnalyticsBatchMeta), alignof(GstAnalyticsBatchMeta));
    printf("%s;%zu;%zu\n", "GstAnalyticsBatchStream", sizeof(GstAnalyticsBatchStream), alignof(GstAnalyticsBatchStream));
    printf("%s;%zu;%zu\n", "GstAnalyticsClsMtd", sizeof(GstAnalyticsClsMtd), alignof(GstAnalyticsClsMtd));
    printf("%s;%zu;%zu\n", "GstAnalyticsGroupMtd", sizeof(GstAnalyticsGroupMtd), alignof(GstAnalyticsGroupMtd));
    printf("%s;%zu;%zu\n", "GstAnalyticsKeypointDimensions", sizeof(GstAnalyticsKeypointDimensions), alignof(GstAnalyticsKeypointDimensions));
    printf("%s;%zu;%zu\n", "GstAnalyticsKeypointMtd", sizeof(GstAnalyticsKeypointMtd), alignof(GstAnalyticsKeypointMtd));
    printf("%s;%zu;%zu\n", "GstAnalyticsKeypointVisibility", sizeof(GstAnalyticsKeypointVisibility), alignof(GstAnalyticsKeypointVisibility));
    printf("%s;%zu;%zu\n", "GstAnalyticsModelInfoTensorDirection", sizeof(GstAnalyticsModelInfoTensorDirection), alignof(GstAnalyticsModelInfoTensorDirection));
    printf("%s;%zu;%zu\n", "GstAnalyticsMtd", sizeof(GstAnalyticsMtd), alignof(GstAnalyticsMtd));
    printf("%s;%zu;%zu\n", "GstAnalyticsMtdImpl", sizeof(GstAnalyticsMtdImpl), alignof(GstAnalyticsMtdImpl));
    printf("%s;%zu;%zu\n", "GstAnalyticsMtdType", sizeof(GstAnalyticsMtdType), alignof(GstAnalyticsMtdType));
    printf("%s;%zu;%zu\n", "GstAnalyticsODMtd", sizeof(GstAnalyticsODMtd), alignof(GstAnalyticsODMtd));
    printf("%s;%zu;%zu\n", "GstAnalyticsRelTypes", sizeof(GstAnalyticsRelTypes), alignof(GstAnalyticsRelTypes));
    printf("%s;%zu;%zu\n", "GstAnalyticsRelationMetaInitParams", sizeof(GstAnalyticsRelationMetaInitParams), alignof(GstAnalyticsRelationMetaInitParams));
    printf("%s;%zu;%zu\n", "GstAnalyticsSegmentationMtd", sizeof(GstAnalyticsSegmentationMtd), alignof(GstAnalyticsSegmentationMtd));
    printf("%s;%zu;%zu\n", "GstAnalyticsTensorMtd", sizeof(GstAnalyticsTensorMtd), alignof(GstAnalyticsTensorMtd));
    printf("%s;%zu;%zu\n", "GstAnalyticsTrackingMtd", sizeof(GstAnalyticsTrackingMtd), alignof(GstAnalyticsTrackingMtd));
    printf("%s;%zu;%zu\n", "GstSegmentationType", sizeof(GstSegmentationType), alignof(GstSegmentationType));
    printf("%s;%zu;%zu\n", "GstTensor", sizeof(GstTensor), alignof(GstTensor));
    printf("%s;%zu;%zu\n", "GstTensorDataType", sizeof(GstTensorDataType), alignof(GstTensorDataType));
    printf("%s;%zu;%zu\n", "GstTensorDimOrder", sizeof(GstTensorDimOrder), alignof(GstTensorDimOrder));
    printf("%s;%zu;%zu\n", "GstTensorLayout", sizeof(GstTensorLayout), alignof(GstTensorLayout));
    printf("%s;%zu;%zu\n", "GstTensorMeta", sizeof(GstTensorMeta), alignof(GstTensorMeta));
    return 0;
}
