// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
pub fn ser_configuration_event(
    object: &mut ::aws_smithy_json::serialize::JsonObjectWriter,
    input: &crate::types::ConfigurationEvent,
) -> ::std::result::Result<(), ::aws_smithy_types::error::operation::SerializationError> {
    if let Some(var_1) = &input.channel_definitions {
        let mut array_2 = object.key("ChannelDefinitions").start_array();
        for item_3 in var_1 {
            {
                #[allow(unused_mut)]
                let mut object_4 = array_2.value().start_object();
                crate::protocol_serde::shape_channel_definition::ser_channel_definition(&mut object_4, item_3)?;
                object_4.finish();
            }
        }
        array_2.finish();
    }
    if let Some(var_5) = &input.post_call_analytics_settings {
        #[allow(unused_mut)]
        let mut object_6 = object.key("PostCallAnalyticsSettings").start_object();
        crate::protocol_serde::shape_post_call_analytics_settings::ser_post_call_analytics_settings(&mut object_6, var_5)?;
        object_6.finish();
    }
    Ok(())
}
