// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
#[allow(missing_docs)] // documentation missing in model
#[non_exhaustive]
#[derive(::std::clone::Clone, ::std::cmp::PartialEq, ::std::fmt::Debug)]
pub struct PutLexiconOutput {
    _request_id: Option<String>,
}
impl ::aws_types::request_id::RequestId for PutLexiconOutput {
    fn request_id(&self) -> Option<&str> {
        self._request_id.as_deref()
    }
}
impl PutLexiconOutput {
    /// Creates a new builder-style object to manufacture [`PutLexiconOutput`](crate::operation::put_lexicon::PutLexiconOutput).
    pub fn builder() -> crate::operation::put_lexicon::builders::PutLexiconOutputBuilder {
        crate::operation::put_lexicon::builders::PutLexiconOutputBuilder::default()
    }
}

/// A builder for [`PutLexiconOutput`](crate::operation::put_lexicon::PutLexiconOutput).
#[derive(::std::clone::Clone, ::std::cmp::PartialEq, ::std::default::Default, ::std::fmt::Debug)]
#[non_exhaustive]
pub struct PutLexiconOutputBuilder {
    _request_id: Option<String>,
}
impl PutLexiconOutputBuilder {
    pub(crate) fn _request_id(mut self, request_id: impl Into<String>) -> Self {
        self._request_id = Some(request_id.into());
        self
    }

    pub(crate) fn _set_request_id(&mut self, request_id: Option<String>) -> &mut Self {
        self._request_id = request_id;
        self
    }
    /// Consumes the builder and constructs a [`PutLexiconOutput`](crate::operation::put_lexicon::PutLexiconOutput).
    pub fn build(self) -> crate::operation::put_lexicon::PutLexiconOutput {
        crate::operation::put_lexicon::PutLexiconOutput {
            _request_id: self._request_id,
        }
    }
}
