#![cfg_attr(feature = "no-std", no_std)]
#![allow(non_camel_case_types)]
#![allow(non_snake_case)]
#![allow(dead_code)]
#[cfg(feature = "no-std")]
extern crate alloc;
#[cfg(feature = "no-std")]
extern crate no_std_compat as std;
pub use self::CUpti_driver_api_trace_cbid_enum as CUpti_driver_api_trace_cbid;
pub use self::CUpti_runtime_api_trace_cbid_enum as CUpti_runtime_api_trace_cbid;
use crate::driver::sys::*;
use crate::runtime::sys::*;
pub const CUPTILP64: u32 = 1;
#[cfg(any(feature = "cuda-11040"))]
pub const CUPTI_API_VERSION: u32 = 14;
#[cfg(any(feature = "cuda-11050", feature = "cuda-11060"))]
pub const CUPTI_API_VERSION: u32 = 16;
#[cfg(any(feature = "cuda-11070"))]
pub const CUPTI_API_VERSION: u32 = 17;
#[cfg(any(feature = "cuda-11080", feature = "cuda-12000", feature = "cuda-12010"))]
pub const CUPTI_API_VERSION: u32 = 18;
#[cfg(any(feature = "cuda-12020"))]
pub const CUPTI_API_VERSION: u32 = 20;
#[cfg(any(feature = "cuda-12030"))]
pub const CUPTI_API_VERSION: u32 = 21;
#[cfg(any(feature = "cuda-12040"))]
pub const CUPTI_API_VERSION: u32 = 22;
#[cfg(any(feature = "cuda-12050"))]
pub const CUPTI_API_VERSION: u32 = 23;
#[cfg(any(feature = "cuda-12060"))]
pub const CUPTI_API_VERSION: u32 = 24;
#[cfg(any(feature = "cuda-12080"))]
pub const CUPTI_API_VERSION: u32 = 26;
#[cfg(any(feature = "cuda-12090"))]
pub const CUPTI_API_VERSION: u32 = 28;
#[cfg(any(feature = "cuda-13000"))]
pub const CUPTI_API_VERSION: u32 = 130001;
#[cfg(any(feature = "cuda-13010"))]
pub const CUPTI_API_VERSION: u32 = 130100;
pub const CUPTI_AUTO_BOOST_INVALID_CLIENT_PID: u32 = 0;
pub const CUPTI_CORRELATION_ID_UNKNOWN: u32 = 0;
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub const CUPTI_DECOMPRESSED_BYTES_UNKNOWN: u32 = 0;
pub const CUPTI_FUNCTION_INDEX_ID_INVALID: u32 = 0;
pub const CUPTI_GRID_ID_UNKNOWN: u32 = 0;
pub const CUPTI_MAX_GPUS: u32 = 32;
#[cfg(any(feature = "cuda-11040"))]
pub const CUPTI_MAX_NVLINK_PORTS: u32 = 16;
#[cfg(any(
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub const CUPTI_MAX_NVLINK_PORTS: u32 = 32;
pub const CUPTI_NVLINK_INVALID_PORT: i32 = -1;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub const CUPTI_OLD_SUBSCRIBER_NAME_MIN_LEN: u32 = 64;
pub const CUPTI_SOURCE_LOCATOR_ID_UNKNOWN: u32 = 0;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub const CUPTI_SUBSCRIBER_NAME_MAX_LEN: u32 = 53;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060"
))]
pub const CUPTI_SYNCHRONIZATION_INVALID_VALUE: i32 = -1;
pub const CUPTI_TIMESTAMP_UNKNOWN: u32 = 0;
pub type CUDA_ARRAY3D_DESCRIPTOR_v1 = CUDA_ARRAY3D_DESCRIPTOR_v1_st;
pub type CUDA_ARRAY_DESCRIPTOR_v1 = CUDA_ARRAY_DESCRIPTOR_v1_st;
pub type CUDA_MEMCPY2D_v1 = CUDA_MEMCPY2D_v1_st;
pub type CUDA_MEMCPY3D_v1 = CUDA_MEMCPY3D_v1_st;
pub type CUdeviceptr_v1 = ::core::ffi::c_uint;
pub type CUpti_ActivityMemcpy2 = CUpti_ActivityMemcpyPtoP;
pub type CUpti_BuffersCallbackCompleteFunc = ::core::option::Option<
    unsafe extern "C" fn(
        context: CUcontext,
        streamId: u32,
        buffer: *mut u8,
        size: usize,
        validSize: usize,
    ),
>;
pub type CUpti_BuffersCallbackRequestFunc = ::core::option::Option<
    unsafe extern "C" fn(buffer: *mut *mut u8, size: *mut usize, maxNumRecords: *mut usize),
>;
pub type CUpti_CallbackFunc = ::core::option::Option<
    unsafe extern "C" fn(
        userdata: *mut ::core::ffi::c_void,
        domain: CUpti_CallbackDomain,
        cbid: CUpti_CallbackId,
        cbdata: *const ::core::ffi::c_void,
    ),
>;
pub type CUpti_CallbackId = u32;
pub type CUpti_DomainTable = *mut CUpti_CallbackDomain;
pub type CUpti_EventDomainID = u32;
pub type CUpti_EventGroup = *mut ::core::ffi::c_void;
pub type CUpti_EventID = u32;
pub type CUpti_KernelReplayUpdateFunc = ::core::option::Option<
    unsafe extern "C" fn(
        kernelName: *const ::core::ffi::c_char,
        numReplaysDone: ::core::ffi::c_int,
        customData: *mut ::core::ffi::c_void,
    ),
>;
pub type CUpti_MetricID = u32;
pub type CUpti_SubscriberHandle = *mut CUpti_Subscriber_st;
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type CUpti_TimestampCallbackFunc = ::core::option::Option<unsafe extern "C" fn() -> u64>;
pub type cuArray3DCreate_params = cuArray3DCreate_params_st;
pub type cuArray3DCreate_v2_params = cuArray3DCreate_v2_params_st;
pub type cuArray3DGetDescriptor_params = cuArray3DGetDescriptor_params_st;
pub type cuArray3DGetDescriptor_v2_params = cuArray3DGetDescriptor_v2_params_st;
pub type cuArrayCreate_params = cuArrayCreate_params_st;
pub type cuArrayCreate_v2_params = cuArrayCreate_v2_params_st;
pub type cuArrayDestroy_params = cuArrayDestroy_params_st;
pub type cuArrayGetDescriptor_params = cuArrayGetDescriptor_params_st;
pub type cuArrayGetDescriptor_v2_params = cuArrayGetDescriptor_v2_params_st;
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuArrayGetMemoryRequirements_params = cuArrayGetMemoryRequirements_params_st;
pub type cuArrayGetPlane_params = cuArrayGetPlane_params_st;
pub type cuArrayGetSparseProperties_params = cuArrayGetSparseProperties_params_st;
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuCheckpointProcessCheckpoint_params = cuCheckpointProcessCheckpoint_params_st;
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuCheckpointProcessGetRestoreThreadId_params =
    cuCheckpointProcessGetRestoreThreadId_params_st;
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuCheckpointProcessGetState_params = cuCheckpointProcessGetState_params_st;
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuCheckpointProcessLock_params = cuCheckpointProcessLock_params_st;
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuCheckpointProcessRestore_params = cuCheckpointProcessRestore_params_st;
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuCheckpointProcessUnlock_params = cuCheckpointProcessUnlock_params_st;
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuCoredumpGetAttributeGlobal_params = cuCoredumpGetAttributeGlobal_params_st;
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuCoredumpGetAttribute_params = cuCoredumpGetAttribute_params_st;
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuCoredumpSetAttributeGlobal_params = cuCoredumpSetAttributeGlobal_params_st;
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuCoredumpSetAttribute_params = cuCoredumpSetAttribute_params_st;
pub type cuCtxAttach_params = cuCtxAttach_params_st;
pub type cuCtxCreate_params = cuCtxCreate_params_st;
pub type cuCtxCreate_v2_params = cuCtxCreate_v2_params_st;
pub type cuCtxCreate_v3_params = cuCtxCreate_v3_params_st;
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuCtxCreate_v4_params = cuCtxCreate_v4_params_st;
pub type cuCtxDestroy_params = cuCtxDestroy_params_st;
pub type cuCtxDestroy_v2_params = cuCtxDestroy_v2_params_st;
pub type cuCtxDetach_params = cuCtxDetach_params_st;
pub type cuCtxDisablePeerAccess_params = cuCtxDisablePeerAccess_params_st;
pub type cuCtxEnablePeerAccess_params = cuCtxEnablePeerAccess_params_st;
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuCtxFromGreenCtx_params = cuCtxFromGreenCtx_params_st;
pub type cuCtxGetApiVersion_params = cuCtxGetApiVersion_params_st;
pub type cuCtxGetCacheConfig_params = cuCtxGetCacheConfig_params_st;
pub type cuCtxGetCurrent_params = cuCtxGetCurrent_params_st;
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuCtxGetDevResource_params = cuCtxGetDevResource_params_st;
pub type cuCtxGetDevice_params = cuCtxGetDevice_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cuCtxGetDevice_v2_params = cuCtxGetDevice_v2_params_st;
pub type cuCtxGetExecAffinity_params = cuCtxGetExecAffinity_params_st;
pub type cuCtxGetFlags_params = cuCtxGetFlags_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuCtxGetId_params = cuCtxGetId_params_st;
pub type cuCtxGetLimit_params = cuCtxGetLimit_params_st;
pub type cuCtxGetSharedMemConfig_params = cuCtxGetSharedMemConfig_params_st;
pub type cuCtxGetStreamPriorityRange_params = cuCtxGetStreamPriorityRange_params_st;
pub type cuCtxPopCurrent_params = cuCtxPopCurrent_params_st;
pub type cuCtxPopCurrent_v2_params = cuCtxPopCurrent_v2_params_st;
pub type cuCtxPushCurrent_params = cuCtxPushCurrent_params_st;
pub type cuCtxPushCurrent_v2_params = cuCtxPushCurrent_v2_params_st;
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuCtxRecordEvent_params = cuCtxRecordEvent_params_st;
pub type cuCtxSetCacheConfig_params = cuCtxSetCacheConfig_params_st;
pub type cuCtxSetCurrent_params = cuCtxSetCurrent_params_st;
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuCtxSetFlags_params = cuCtxSetFlags_params_st;
pub type cuCtxSetLimit_params = cuCtxSetLimit_params_st;
pub type cuCtxSetSharedMemConfig_params = cuCtxSetSharedMemConfig_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cuCtxSynchronize_v2_params = cuCtxSynchronize_v2_params_st;
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuCtxWaitEvent_params = cuCtxWaitEvent_params_st;
pub type cuDestroyExternalMemory_params = cuDestroyExternalMemory_params_st;
pub type cuDestroyExternalSemaphore_params = cuDestroyExternalSemaphore_params_st;
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuDevResourceGenerateDesc_params = cuDevResourceGenerateDesc_params_st;
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuDevSmResourceSplitByCount_params = cuDevSmResourceSplitByCount_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cuDevSmResourceSplit_params = cuDevSmResourceSplit_params_st;
pub type cuDeviceCanAccessPeer_params = cuDeviceCanAccessPeer_params_st;
pub type cuDeviceComputeCapability_params = cuDeviceComputeCapability_params_st;
pub type cuDeviceGetAttribute_params = cuDeviceGetAttribute_params_st;
pub type cuDeviceGetByPCIBusId_params = cuDeviceGetByPCIBusId_params_st;
pub type cuDeviceGetCount_params = cuDeviceGetCount_params_st;
pub type cuDeviceGetDefaultMemPool_params = cuDeviceGetDefaultMemPool_params_st;
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuDeviceGetDevResource_params = cuDeviceGetDevResource_params_st;
pub type cuDeviceGetExecAffinitySupport_params = cuDeviceGetExecAffinitySupport_params_st;
pub type cuDeviceGetGraphMemAttribute_params = cuDeviceGetGraphMemAttribute_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cuDeviceGetHostAtomicCapabilities_params = cuDeviceGetHostAtomicCapabilities_params_st;
pub type cuDeviceGetLuid_params = cuDeviceGetLuid_params_st;
pub type cuDeviceGetMemPool_params = cuDeviceGetMemPool_params_st;
pub type cuDeviceGetName_params = cuDeviceGetName_params_st;
pub type cuDeviceGetNvSciSyncAttributes_params = cuDeviceGetNvSciSyncAttributes_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cuDeviceGetP2PAtomicCapabilities_params = cuDeviceGetP2PAtomicCapabilities_params_st;
pub type cuDeviceGetP2PAttribute_params = cuDeviceGetP2PAttribute_params_st;
pub type cuDeviceGetPCIBusId_params = cuDeviceGetPCIBusId_params_st;
pub type cuDeviceGetProperties_params = cuDeviceGetProperties_params_st;
pub type cuDeviceGetTexture1DLinearMaxWidth_params = cuDeviceGetTexture1DLinearMaxWidth_params_st;
pub type cuDeviceGetUuid_params = cuDeviceGetUuid_params_st;
pub type cuDeviceGetUuid_v2_params = cuDeviceGetUuid_v2_params_st;
pub type cuDeviceGet_params = cuDeviceGet_params_st;
pub type cuDeviceGraphMemTrim_params = cuDeviceGraphMemTrim_params_st;
pub type cuDevicePrimaryCtxGetState_params = cuDevicePrimaryCtxGetState_params_st;
pub type cuDevicePrimaryCtxRelease_params = cuDevicePrimaryCtxRelease_params_st;
pub type cuDevicePrimaryCtxRelease_v2_params = cuDevicePrimaryCtxRelease_v2_params_st;
pub type cuDevicePrimaryCtxReset_params = cuDevicePrimaryCtxReset_params_st;
pub type cuDevicePrimaryCtxReset_v2_params = cuDevicePrimaryCtxReset_v2_params_st;
pub type cuDevicePrimaryCtxRetain_params = cuDevicePrimaryCtxRetain_params_st;
pub type cuDevicePrimaryCtxSetFlags_params = cuDevicePrimaryCtxSetFlags_params_st;
pub type cuDevicePrimaryCtxSetFlags_v2_params = cuDevicePrimaryCtxSetFlags_v2_params_st;
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuDeviceRegisterAsyncNotification_params = cuDeviceRegisterAsyncNotification_params_st;
pub type cuDeviceSetGraphMemAttribute_params = cuDeviceSetGraphMemAttribute_params_st;
pub type cuDeviceSetMemPool_params = cuDeviceSetMemPool_params_st;
pub type cuDeviceTotalMem_params = cuDeviceTotalMem_params_st;
pub type cuDeviceTotalMem_v2_params = cuDeviceTotalMem_v2_params_st;
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuDeviceUnregisterAsyncNotification_params = cuDeviceUnregisterAsyncNotification_params_st;
pub type cuDriverGetVersion_params = cuDriverGetVersion_params_st;
pub type cuEventCreate_params = cuEventCreate_params_st;
pub type cuEventDestroy_params = cuEventDestroy_params_st;
pub type cuEventDestroy_v2_params = cuEventDestroy_v2_params_st;
pub type cuEventElapsedTime_params = cuEventElapsedTime_params_st;
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuEventElapsedTime_v2_params = cuEventElapsedTime_v2_params_st;
pub type cuEventQuery_params = cuEventQuery_params_st;
pub type cuEventRecordWithFlags_params = cuEventRecordWithFlags_params_st;
pub type cuEventRecordWithFlags_ptsz_params = cuEventRecordWithFlags_ptsz_params_st;
pub type cuEventRecord_params = cuEventRecord_params_st;
pub type cuEventRecord_ptsz_params = cuEventRecord_ptsz_params_st;
pub type cuEventSynchronize_params = cuEventSynchronize_params_st;
pub type cuExternalMemoryGetMappedBuffer_params = cuExternalMemoryGetMappedBuffer_params_st;
pub type cuExternalMemoryGetMappedMipmappedArray_params =
    cuExternalMemoryGetMappedMipmappedArray_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuFlushGPUDirectRDMAWrites_params = cuFlushGPUDirectRDMAWrites_params_st;
pub type cuFuncGetAttribute_params = cuFuncGetAttribute_params_st;
pub type cuFuncGetModule_params = cuFuncGetModule_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuFuncGetName_params = cuFuncGetName_params_st;
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuFuncGetParamInfo_params = cuFuncGetParamInfo_params_st;
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuFuncIsLoaded_params = cuFuncIsLoaded_params_st;
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuFuncLoad_params = cuFuncLoad_params_st;
pub type cuFuncSetAttribute_params = cuFuncSetAttribute_params_st;
pub type cuFuncSetBlockShape_params = cuFuncSetBlockShape_params_st;
pub type cuFuncSetCacheConfig_params = cuFuncSetCacheConfig_params_st;
pub type cuFuncSetSharedMemConfig_params = cuFuncSetSharedMemConfig_params_st;
pub type cuFuncSetSharedSize_params = cuFuncSetSharedSize_params_st;
pub type cuGetErrorName_params = cuGetErrorName_params_st;
pub type cuGetErrorString_params = cuGetErrorString_params_st;
pub type cuGetExportTable_params = cuGetExportTable_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGetProcAddress_params = cuGetProcAddress_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGetProcAddress_v2_params = cuGetProcAddress_v2_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGraphAddBatchMemOpNode_params = cuGraphAddBatchMemOpNode_params_st;
pub type cuGraphAddChildGraphNode_params = cuGraphAddChildGraphNode_params_st;
pub type cuGraphAddDependencies_params = cuGraphAddDependencies_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGraphAddDependencies_v2_params = cuGraphAddDependencies_v2_params_st;
pub type cuGraphAddEmptyNode_params = cuGraphAddEmptyNode_params_st;
pub type cuGraphAddEventRecordNode_params = cuGraphAddEventRecordNode_params_st;
pub type cuGraphAddEventWaitNode_params = cuGraphAddEventWaitNode_params_st;
pub type cuGraphAddExternalSemaphoresSignalNode_params =
    cuGraphAddExternalSemaphoresSignalNode_params_st;
pub type cuGraphAddExternalSemaphoresWaitNode_params =
    cuGraphAddExternalSemaphoresWaitNode_params_st;
pub type cuGraphAddHostNode_params = cuGraphAddHostNode_params_st;
pub type cuGraphAddKernelNode_params = cuGraphAddKernelNode_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGraphAddKernelNode_v2_params = cuGraphAddKernelNode_v2_params_st;
pub type cuGraphAddMemAllocNode_params = cuGraphAddMemAllocNode_params_st;
pub type cuGraphAddMemFreeNode_params = cuGraphAddMemFreeNode_params_st;
pub type cuGraphAddMemcpyNode_params = cuGraphAddMemcpyNode_params_st;
pub type cuGraphAddMemsetNode_params = cuGraphAddMemsetNode_params_st;
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGraphAddNode_params = cuGraphAddNode_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGraphAddNode_v2_params = cuGraphAddNode_v2_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGraphBatchMemOpNodeGetParams_params = cuGraphBatchMemOpNodeGetParams_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGraphBatchMemOpNodeSetParams_params = cuGraphBatchMemOpNodeSetParams_params_st;
pub type cuGraphChildGraphNodeGetGraph_params = cuGraphChildGraphNodeGetGraph_params_st;
pub type cuGraphClone_params = cuGraphClone_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGraphConditionalHandleCreate_params = cuGraphConditionalHandleCreate_params_st;
pub type cuGraphCreate_params = cuGraphCreate_params_st;
pub type cuGraphDebugDotPrint_params = cuGraphDebugDotPrint_params_st;
pub type cuGraphDestroyNode_params = cuGraphDestroyNode_params_st;
pub type cuGraphDestroy_params = cuGraphDestroy_params_st;
pub type cuGraphEventRecordNodeGetEvent_params = cuGraphEventRecordNodeGetEvent_params_st;
pub type cuGraphEventRecordNodeSetEvent_params = cuGraphEventRecordNodeSetEvent_params_st;
pub type cuGraphEventWaitNodeGetEvent_params = cuGraphEventWaitNodeGetEvent_params_st;
pub type cuGraphEventWaitNodeSetEvent_params = cuGraphEventWaitNodeSetEvent_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGraphExecBatchMemOpNodeSetParams_params = cuGraphExecBatchMemOpNodeSetParams_params_st;
pub type cuGraphExecChildGraphNodeSetParams_params = cuGraphExecChildGraphNodeSetParams_params_st;
pub type cuGraphExecDestroy_params = cuGraphExecDestroy_params_st;
pub type cuGraphExecEventRecordNodeSetEvent_params = cuGraphExecEventRecordNodeSetEvent_params_st;
pub type cuGraphExecEventWaitNodeSetEvent_params = cuGraphExecEventWaitNodeSetEvent_params_st;
pub type cuGraphExecExternalSemaphoresSignalNodeSetParams_params =
    cuGraphExecExternalSemaphoresSignalNodeSetParams_params_st;
pub type cuGraphExecExternalSemaphoresWaitNodeSetParams_params =
    cuGraphExecExternalSemaphoresWaitNodeSetParams_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGraphExecGetFlags_params = cuGraphExecGetFlags_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cuGraphExecGetId_params = cuGraphExecGetId_params_st;
pub type cuGraphExecHostNodeSetParams_params = cuGraphExecHostNodeSetParams_params_st;
pub type cuGraphExecKernelNodeSetParams_params = cuGraphExecKernelNodeSetParams_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGraphExecKernelNodeSetParams_v2_params = cuGraphExecKernelNodeSetParams_v2_params_st;
pub type cuGraphExecMemcpyNodeSetParams_params = cuGraphExecMemcpyNodeSetParams_params_st;
pub type cuGraphExecMemsetNodeSetParams_params = cuGraphExecMemsetNodeSetParams_params_st;
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGraphExecNodeSetParams_params = cuGraphExecNodeSetParams_params_st;
pub type cuGraphExecUpdate_params = cuGraphExecUpdate_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGraphExecUpdate_v2_params = cuGraphExecUpdate_v2_params_st;
pub type cuGraphExternalSemaphoresSignalNodeGetParams_params =
    cuGraphExternalSemaphoresSignalNodeGetParams_params_st;
pub type cuGraphExternalSemaphoresSignalNodeSetParams_params =
    cuGraphExternalSemaphoresSignalNodeSetParams_params_st;
pub type cuGraphExternalSemaphoresWaitNodeGetParams_params =
    cuGraphExternalSemaphoresWaitNodeGetParams_params_st;
pub type cuGraphExternalSemaphoresWaitNodeSetParams_params =
    cuGraphExternalSemaphoresWaitNodeSetParams_params_st;
pub type cuGraphGetEdges_params = cuGraphGetEdges_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGraphGetEdges_v2_params = cuGraphGetEdges_v2_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cuGraphGetId_params = cuGraphGetId_params_st;
pub type cuGraphGetNodes_params = cuGraphGetNodes_params_st;
pub type cuGraphGetRootNodes_params = cuGraphGetRootNodes_params_st;
pub type cuGraphHostNodeGetParams_params = cuGraphHostNodeGetParams_params_st;
pub type cuGraphHostNodeSetParams_params = cuGraphHostNodeSetParams_params_st;
pub type cuGraphInstantiateWithFlags_params = cuGraphInstantiateWithFlags_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGraphInstantiateWithParams_params = cuGraphInstantiateWithParams_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGraphInstantiateWithParams_ptsz_params = cuGraphInstantiateWithParams_ptsz_params_st;
pub type cuGraphInstantiate_params = cuGraphInstantiate_params_st;
pub type cuGraphInstantiate_v2_params = cuGraphInstantiate_v2_params_st;
pub type cuGraphKernelNodeCopyAttributes_params = cuGraphKernelNodeCopyAttributes_params_st;
pub type cuGraphKernelNodeGetAttribute_params = cuGraphKernelNodeGetAttribute_params_st;
pub type cuGraphKernelNodeGetParams_params = cuGraphKernelNodeGetParams_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGraphKernelNodeGetParams_v2_params = cuGraphKernelNodeGetParams_v2_params_st;
pub type cuGraphKernelNodeSetAttribute_params = cuGraphKernelNodeSetAttribute_params_st;
pub type cuGraphKernelNodeSetParams_params = cuGraphKernelNodeSetParams_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGraphKernelNodeSetParams_v2_params = cuGraphKernelNodeSetParams_v2_params_st;
pub type cuGraphLaunch_params = cuGraphLaunch_params_st;
pub type cuGraphLaunch_ptsz_params = cuGraphLaunch_ptsz_params_st;
pub type cuGraphMemAllocNodeGetParams_params = cuGraphMemAllocNodeGetParams_params_st;
pub type cuGraphMemFreeNodeGetParams_params = cuGraphMemFreeNodeGetParams_params_st;
pub type cuGraphMemcpyNodeGetParams_params = cuGraphMemcpyNodeGetParams_params_st;
pub type cuGraphMemcpyNodeSetParams_params = cuGraphMemcpyNodeSetParams_params_st;
pub type cuGraphMemsetNodeGetParams_params = cuGraphMemsetNodeGetParams_params_st;
pub type cuGraphMemsetNodeSetParams_params = cuGraphMemsetNodeSetParams_params_st;
pub type cuGraphNodeFindInClone_params = cuGraphNodeFindInClone_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cuGraphNodeGetContainingGraph_params = cuGraphNodeGetContainingGraph_params_st;
pub type cuGraphNodeGetDependencies_params = cuGraphNodeGetDependencies_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGraphNodeGetDependencies_v2_params = cuGraphNodeGetDependencies_v2_params_st;
pub type cuGraphNodeGetDependentNodes_params = cuGraphNodeGetDependentNodes_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGraphNodeGetDependentNodes_v2_params = cuGraphNodeGetDependentNodes_v2_params_st;
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGraphNodeGetEnabled_params = cuGraphNodeGetEnabled_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cuGraphNodeGetLocalId_params = cuGraphNodeGetLocalId_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cuGraphNodeGetToolsId_params = cuGraphNodeGetToolsId_params_st;
pub type cuGraphNodeGetType_params = cuGraphNodeGetType_params_st;
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGraphNodeSetEnabled_params = cuGraphNodeSetEnabled_params_st;
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGraphNodeSetParams_params = cuGraphNodeSetParams_params_st;
pub type cuGraphReleaseUserObject_params = cuGraphReleaseUserObject_params_st;
pub type cuGraphRemoveDependencies_params = cuGraphRemoveDependencies_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGraphRemoveDependencies_v2_params = cuGraphRemoveDependencies_v2_params_st;
pub type cuGraphRetainUserObject_params = cuGraphRetainUserObject_params_st;
pub type cuGraphUpload_params = cuGraphUpload_params_st;
pub type cuGraphUpload_ptsz_params = cuGraphUpload_ptsz_params_st;
pub type cuGraphicsMapResources_params = cuGraphicsMapResources_params_st;
pub type cuGraphicsMapResources_ptsz_params = cuGraphicsMapResources_ptsz_params_st;
pub type cuGraphicsResourceGetMappedMipmappedArray_params =
    cuGraphicsResourceGetMappedMipmappedArray_params_st;
pub type cuGraphicsResourceGetMappedPointer_params = cuGraphicsResourceGetMappedPointer_params_st;
pub type cuGraphicsResourceGetMappedPointer_v2_params =
    cuGraphicsResourceGetMappedPointer_v2_params_st;
pub type cuGraphicsResourceSetMapFlags_params = cuGraphicsResourceSetMapFlags_params_st;
pub type cuGraphicsResourceSetMapFlags_v2_params = cuGraphicsResourceSetMapFlags_v2_params_st;
pub type cuGraphicsSubResourceGetMappedArray_params = cuGraphicsSubResourceGetMappedArray_params_st;
pub type cuGraphicsUnmapResources_params = cuGraphicsUnmapResources_params_st;
pub type cuGraphicsUnmapResources_ptsz_params = cuGraphicsUnmapResources_ptsz_params_st;
pub type cuGraphicsUnregisterResource_params = cuGraphicsUnregisterResource_params_st;
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGreenCtxCreate_params = cuGreenCtxCreate_params_st;
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGreenCtxDestroy_params = cuGreenCtxDestroy_params_st;
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGreenCtxGetDevResource_params = cuGreenCtxGetDevResource_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cuGreenCtxGetId_params = cuGreenCtxGetId_params_st;
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGreenCtxRecordEvent_params = cuGreenCtxRecordEvent_params_st;
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGreenCtxStreamCreate_params = cuGreenCtxStreamCreate_params_st;
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuGreenCtxWaitEvent_params = cuGreenCtxWaitEvent_params_st;
pub type cuImportExternalMemory_params = cuImportExternalMemory_params_st;
pub type cuImportExternalSemaphore_params = cuImportExternalSemaphore_params_st;
pub type cuInit_params = cuInit_params_st;
pub type cuIpcCloseMemHandle_params = cuIpcCloseMemHandle_params_st;
pub type cuIpcGetEventHandle_params = cuIpcGetEventHandle_params_st;
pub type cuIpcGetMemHandle_params = cuIpcGetMemHandle_params_st;
pub type cuIpcOpenEventHandle_params = cuIpcOpenEventHandle_params_st;
pub type cuIpcOpenMemHandle_params = cuIpcOpenMemHandle_params_st;
pub type cuIpcOpenMemHandle_v2_params = cuIpcOpenMemHandle_v2_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuKernelGetAttribute_params = cuKernelGetAttribute_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuKernelGetFunction_params = cuKernelGetFunction_params_st;
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuKernelGetLibrary_params = cuKernelGetLibrary_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuKernelGetName_params = cuKernelGetName_params_st;
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuKernelGetParamInfo_params = cuKernelGetParamInfo_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuKernelSetAttribute_params = cuKernelSetAttribute_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuKernelSetCacheConfig_params = cuKernelSetCacheConfig_params_st;
pub type cuLaunchCooperativeKernelMultiDevice_params =
    cuLaunchCooperativeKernelMultiDevice_params_st;
pub type cuLaunchCooperativeKernel_params = cuLaunchCooperativeKernel_params_st;
pub type cuLaunchCooperativeKernel_ptsz_params = cuLaunchCooperativeKernel_ptsz_params_st;
pub type cuLaunchGridAsync_params = cuLaunchGridAsync_params_st;
pub type cuLaunchGrid_params = cuLaunchGrid_params_st;
pub type cuLaunchHostFunc_params = cuLaunchHostFunc_params_st;
pub type cuLaunchHostFunc_ptsz_params = cuLaunchHostFunc_ptsz_params_st;
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuLaunchKernelEx_params = cuLaunchKernelEx_params_st;
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuLaunchKernelEx_ptsz_params = cuLaunchKernelEx_ptsz_params_st;
pub type cuLaunchKernel_params = cuLaunchKernel_params_st;
pub type cuLaunchKernel_ptsz_params = cuLaunchKernel_ptsz_params_st;
pub type cuLaunch_params = cuLaunch_params_st;
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuLibraryEnumerateKernels_params = cuLibraryEnumerateKernels_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuLibraryGetGlobal_params = cuLibraryGetGlobal_params_st;
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuLibraryGetKernelCount_params = cuLibraryGetKernelCount_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuLibraryGetKernel_params = cuLibraryGetKernel_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuLibraryGetManaged_params = cuLibraryGetManaged_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuLibraryGetModule_params = cuLibraryGetModule_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuLibraryGetUnifiedFunction_params = cuLibraryGetUnifiedFunction_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuLibraryLoadData_params = cuLibraryLoadData_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuLibraryLoadFromFile_params = cuLibraryLoadFromFile_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuLibraryUnload_params = cuLibraryUnload_params_st;
pub type cuLinkAddData_params = cuLinkAddData_params_st;
pub type cuLinkAddData_v2_params = cuLinkAddData_v2_params_st;
pub type cuLinkAddFile_params = cuLinkAddFile_params_st;
pub type cuLinkAddFile_v2_params = cuLinkAddFile_v2_params_st;
pub type cuLinkComplete_params = cuLinkComplete_params_st;
pub type cuLinkCreate_params = cuLinkCreate_params_st;
pub type cuLinkCreate_v2_params = cuLinkCreate_v2_params_st;
pub type cuLinkDestroy_params = cuLinkDestroy_params_st;
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
pub type cuLogsCurrent_params = cuLogsCurrent_params_st;
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
pub type cuLogsDumpToFile_params = cuLogsDumpToFile_params_st;
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
pub type cuLogsDumpToMemory_params = cuLogsDumpToMemory_params_st;
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
pub type cuLogsRegisterCallback_params = cuLogsRegisterCallback_params_st;
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
pub type cuLogsUnregisterCallback_params = cuLogsUnregisterCallback_params_st;
pub type cuMemAddressFree_params = cuMemAddressFree_params_st;
pub type cuMemAddressReserve_params = cuMemAddressReserve_params_st;
pub type cuMemAdvise_params = cuMemAdvise_params_st;
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuMemAdvise_v2_params = cuMemAdvise_v2_params_st;
pub type cuMemAllocAsync_params = cuMemAllocAsync_params_st;
pub type cuMemAllocAsync_ptsz_params = cuMemAllocAsync_ptsz_params_st;
pub type cuMemAllocFromPoolAsync_params = cuMemAllocFromPoolAsync_params_st;
pub type cuMemAllocFromPoolAsync_ptsz_params = cuMemAllocFromPoolAsync_ptsz_params_st;
pub type cuMemAllocHost_params = cuMemAllocHost_params_st;
pub type cuMemAllocHost_v2_params = cuMemAllocHost_v2_params_st;
pub type cuMemAllocManaged_params = cuMemAllocManaged_params_st;
pub type cuMemAllocPitch_params = cuMemAllocPitch_params_st;
pub type cuMemAllocPitch_v2_params = cuMemAllocPitch_v2_params_st;
pub type cuMemAlloc_params = cuMemAlloc_params_st;
pub type cuMemAlloc_v2_params = cuMemAlloc_v2_params_st;
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuMemBatchDecompressAsync_params = cuMemBatchDecompressAsync_params_st;
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuMemBatchDecompressAsync_ptsz_params = cuMemBatchDecompressAsync_ptsz_params_st;
pub type cuMemCreate_params = cuMemCreate_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cuMemDiscardAndPrefetchBatchAsync_params = cuMemDiscardAndPrefetchBatchAsync_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cuMemDiscardAndPrefetchBatchAsync_ptsz_params =
    cuMemDiscardAndPrefetchBatchAsync_ptsz_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cuMemDiscardBatchAsync_params = cuMemDiscardBatchAsync_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cuMemDiscardBatchAsync_ptsz_params = cuMemDiscardBatchAsync_ptsz_params_st;
pub type cuMemExportToShareableHandle_params = cuMemExportToShareableHandle_params_st;
pub type cuMemFreeAsync_params = cuMemFreeAsync_params_st;
pub type cuMemFreeAsync_ptsz_params = cuMemFreeAsync_ptsz_params_st;
pub type cuMemFreeHost_params = cuMemFreeHost_params_st;
pub type cuMemFree_params = cuMemFree_params_st;
pub type cuMemFree_v2_params = cuMemFree_v2_params_st;
pub type cuMemGetAccess_params = cuMemGetAccess_params_st;
pub type cuMemGetAddressRange_params = cuMemGetAddressRange_params_st;
pub type cuMemGetAddressRange_v2_params = cuMemGetAddressRange_v2_params_st;
pub type cuMemGetAllocationGranularity_params = cuMemGetAllocationGranularity_params_st;
pub type cuMemGetAllocationPropertiesFromHandle_params =
    cuMemGetAllocationPropertiesFromHandle_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cuMemGetDefaultMemPool_params = cuMemGetDefaultMemPool_params_st;
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuMemGetHandleForAddressRange_params = cuMemGetHandleForAddressRange_params_st;
pub type cuMemGetInfo_params = cuMemGetInfo_params_st;
pub type cuMemGetInfo_v2_params = cuMemGetInfo_v2_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cuMemGetMemPool_params = cuMemGetMemPool_params_st;
pub type cuMemHostAlloc_params = cuMemHostAlloc_params_st;
pub type cuMemHostGetDevicePointer_params = cuMemHostGetDevicePointer_params_st;
pub type cuMemHostGetDevicePointer_v2_params = cuMemHostGetDevicePointer_v2_params_st;
pub type cuMemHostGetFlags_params = cuMemHostGetFlags_params_st;
pub type cuMemHostRegister_params = cuMemHostRegister_params_st;
pub type cuMemHostRegister_v2_params = cuMemHostRegister_v2_params_st;
pub type cuMemHostUnregister_params = cuMemHostUnregister_params_st;
pub type cuMemImportFromShareableHandle_params = cuMemImportFromShareableHandle_params_st;
pub type cuMemMapArrayAsync_params = cuMemMapArrayAsync_params_st;
pub type cuMemMapArrayAsync_ptsz_params = cuMemMapArrayAsync_ptsz_params_st;
pub type cuMemMap_params = cuMemMap_params_st;
pub type cuMemPoolCreate_params = cuMemPoolCreate_params_st;
pub type cuMemPoolDestroy_params = cuMemPoolDestroy_params_st;
pub type cuMemPoolExportPointer_params = cuMemPoolExportPointer_params_st;
pub type cuMemPoolExportToShareableHandle_params = cuMemPoolExportToShareableHandle_params_st;
pub type cuMemPoolGetAccess_params = cuMemPoolGetAccess_params_st;
pub type cuMemPoolGetAttribute_params = cuMemPoolGetAttribute_params_st;
pub type cuMemPoolImportFromShareableHandle_params = cuMemPoolImportFromShareableHandle_params_st;
pub type cuMemPoolImportPointer_params = cuMemPoolImportPointer_params_st;
pub type cuMemPoolSetAccess_params = cuMemPoolSetAccess_params_st;
pub type cuMemPoolSetAttribute_params = cuMemPoolSetAttribute_params_st;
pub type cuMemPoolTrimTo_params = cuMemPoolTrimTo_params_st;
pub type cuMemPrefetchAsync_params = cuMemPrefetchAsync_params_st;
pub type cuMemPrefetchAsync_ptsz_params = cuMemPrefetchAsync_ptsz_params_st;
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuMemPrefetchAsync_v2_params = cuMemPrefetchAsync_v2_params_st;
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuMemPrefetchAsync_v2_ptsz_params = cuMemPrefetchAsync_v2_ptsz_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cuMemPrefetchBatchAsync_params = cuMemPrefetchBatchAsync_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cuMemPrefetchBatchAsync_ptsz_params = cuMemPrefetchBatchAsync_ptsz_params_st;
pub type cuMemRangeGetAttribute_params = cuMemRangeGetAttribute_params_st;
pub type cuMemRangeGetAttributes_params = cuMemRangeGetAttributes_params_st;
pub type cuMemRelease_params = cuMemRelease_params_st;
pub type cuMemRetainAllocationHandle_params = cuMemRetainAllocationHandle_params_st;
pub type cuMemSetAccess_params = cuMemSetAccess_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cuMemSetMemPool_params = cuMemSetMemPool_params_st;
pub type cuMemUnmap_params = cuMemUnmap_params_st;
pub type cuMemcpy2DAsync_params = cuMemcpy2DAsync_params_st;
pub type cuMemcpy2DAsync_v2_params = cuMemcpy2DAsync_v2_params_st;
pub type cuMemcpy2DAsync_v2_ptsz_params = cuMemcpy2DAsync_v2_ptsz_params_st;
pub type cuMemcpy2DUnaligned_params = cuMemcpy2DUnaligned_params_st;
pub type cuMemcpy2DUnaligned_v2_params = cuMemcpy2DUnaligned_v2_params_st;
pub type cuMemcpy2DUnaligned_v2_ptds_params = cuMemcpy2DUnaligned_v2_ptds_params_st;
pub type cuMemcpy2D_params = cuMemcpy2D_params_st;
pub type cuMemcpy2D_v2_params = cuMemcpy2D_v2_params_st;
pub type cuMemcpy2D_v2_ptds_params = cuMemcpy2D_v2_ptds_params_st;
pub type cuMemcpy3DAsync_params = cuMemcpy3DAsync_params_st;
pub type cuMemcpy3DAsync_v2_params = cuMemcpy3DAsync_v2_params_st;
pub type cuMemcpy3DAsync_v2_ptsz_params = cuMemcpy3DAsync_v2_ptsz_params_st;
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuMemcpy3DBatchAsync_params = cuMemcpy3DBatchAsync_params_st;
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuMemcpy3DBatchAsync_ptsz_params = cuMemcpy3DBatchAsync_ptsz_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cuMemcpy3DBatchAsync_v2_params = cuMemcpy3DBatchAsync_v2_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cuMemcpy3DBatchAsync_v2_ptsz_params = cuMemcpy3DBatchAsync_v2_ptsz_params_st;
pub type cuMemcpy3DPeerAsync_params = cuMemcpy3DPeerAsync_params_st;
pub type cuMemcpy3DPeerAsync_ptsz_params = cuMemcpy3DPeerAsync_ptsz_params_st;
pub type cuMemcpy3DPeer_params = cuMemcpy3DPeer_params_st;
pub type cuMemcpy3DPeer_ptds_params = cuMemcpy3DPeer_ptds_params_st;
pub type cuMemcpy3D_params = cuMemcpy3D_params_st;
pub type cuMemcpy3D_v2_params = cuMemcpy3D_v2_params_st;
pub type cuMemcpy3D_v2_ptds_params = cuMemcpy3D_v2_ptds_params_st;
pub type cuMemcpyAsync_params = cuMemcpyAsync_params_st;
pub type cuMemcpyAsync_ptsz_params = cuMemcpyAsync_ptsz_params_st;
pub type cuMemcpyAtoA_params = cuMemcpyAtoA_params_st;
pub type cuMemcpyAtoA_v2_params = cuMemcpyAtoA_v2_params_st;
pub type cuMemcpyAtoA_v2_ptds_params = cuMemcpyAtoA_v2_ptds_params_st;
pub type cuMemcpyAtoD_params = cuMemcpyAtoD_params_st;
pub type cuMemcpyAtoD_v2_params = cuMemcpyAtoD_v2_params_st;
pub type cuMemcpyAtoD_v2_ptds_params = cuMemcpyAtoD_v2_ptds_params_st;
pub type cuMemcpyAtoHAsync_params = cuMemcpyAtoHAsync_params_st;
pub type cuMemcpyAtoHAsync_v2_params = cuMemcpyAtoHAsync_v2_params_st;
pub type cuMemcpyAtoHAsync_v2_ptsz_params = cuMemcpyAtoHAsync_v2_ptsz_params_st;
pub type cuMemcpyAtoH_params = cuMemcpyAtoH_params_st;
pub type cuMemcpyAtoH_v2_params = cuMemcpyAtoH_v2_params_st;
pub type cuMemcpyAtoH_v2_ptds_params = cuMemcpyAtoH_v2_ptds_params_st;
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuMemcpyBatchAsync_params = cuMemcpyBatchAsync_params_st;
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuMemcpyBatchAsync_ptsz_params = cuMemcpyBatchAsync_ptsz_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cuMemcpyBatchAsync_v2_params = cuMemcpyBatchAsync_v2_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cuMemcpyBatchAsync_v2_ptsz_params = cuMemcpyBatchAsync_v2_ptsz_params_st;
pub type cuMemcpyDtoA_params = cuMemcpyDtoA_params_st;
pub type cuMemcpyDtoA_v2_params = cuMemcpyDtoA_v2_params_st;
pub type cuMemcpyDtoA_v2_ptds_params = cuMemcpyDtoA_v2_ptds_params_st;
pub type cuMemcpyDtoDAsync_params = cuMemcpyDtoDAsync_params_st;
pub type cuMemcpyDtoDAsync_v2_params = cuMemcpyDtoDAsync_v2_params_st;
pub type cuMemcpyDtoDAsync_v2_ptsz_params = cuMemcpyDtoDAsync_v2_ptsz_params_st;
pub type cuMemcpyDtoD_params = cuMemcpyDtoD_params_st;
pub type cuMemcpyDtoD_v2_params = cuMemcpyDtoD_v2_params_st;
pub type cuMemcpyDtoD_v2_ptds_params = cuMemcpyDtoD_v2_ptds_params_st;
pub type cuMemcpyDtoHAsync_params = cuMemcpyDtoHAsync_params_st;
pub type cuMemcpyDtoHAsync_v2_params = cuMemcpyDtoHAsync_v2_params_st;
pub type cuMemcpyDtoHAsync_v2_ptsz_params = cuMemcpyDtoHAsync_v2_ptsz_params_st;
pub type cuMemcpyDtoH_params = cuMemcpyDtoH_params_st;
pub type cuMemcpyDtoH_v2_params = cuMemcpyDtoH_v2_params_st;
pub type cuMemcpyDtoH_v2_ptds_params = cuMemcpyDtoH_v2_ptds_params_st;
pub type cuMemcpyHtoAAsync_params = cuMemcpyHtoAAsync_params_st;
pub type cuMemcpyHtoAAsync_v2_params = cuMemcpyHtoAAsync_v2_params_st;
pub type cuMemcpyHtoAAsync_v2_ptsz_params = cuMemcpyHtoAAsync_v2_ptsz_params_st;
pub type cuMemcpyHtoA_params = cuMemcpyHtoA_params_st;
pub type cuMemcpyHtoA_v2_params = cuMemcpyHtoA_v2_params_st;
pub type cuMemcpyHtoA_v2_ptds_params = cuMemcpyHtoA_v2_ptds_params_st;
pub type cuMemcpyHtoDAsync_params = cuMemcpyHtoDAsync_params_st;
pub type cuMemcpyHtoDAsync_v2_params = cuMemcpyHtoDAsync_v2_params_st;
pub type cuMemcpyHtoDAsync_v2_ptsz_params = cuMemcpyHtoDAsync_v2_ptsz_params_st;
pub type cuMemcpyHtoD_params = cuMemcpyHtoD_params_st;
pub type cuMemcpyHtoD_v2_params = cuMemcpyHtoD_v2_params_st;
pub type cuMemcpyHtoD_v2_ptds_params = cuMemcpyHtoD_v2_ptds_params_st;
pub type cuMemcpyPeerAsync_params = cuMemcpyPeerAsync_params_st;
pub type cuMemcpyPeerAsync_ptsz_params = cuMemcpyPeerAsync_ptsz_params_st;
pub type cuMemcpyPeer_params = cuMemcpyPeer_params_st;
pub type cuMemcpyPeer_ptds_params = cuMemcpyPeer_ptds_params_st;
pub type cuMemcpy_params = cuMemcpy_params_st;
pub type cuMemcpy_ptds_params = cuMemcpy_ptds_params_st;
pub type cuMemsetD16Async_params = cuMemsetD16Async_params_st;
pub type cuMemsetD16Async_ptsz_params = cuMemsetD16Async_ptsz_params_st;
pub type cuMemsetD16_params = cuMemsetD16_params_st;
pub type cuMemsetD16_v2_params = cuMemsetD16_v2_params_st;
pub type cuMemsetD16_v2_ptds_params = cuMemsetD16_v2_ptds_params_st;
pub type cuMemsetD2D16Async_params = cuMemsetD2D16Async_params_st;
pub type cuMemsetD2D16Async_ptsz_params = cuMemsetD2D16Async_ptsz_params_st;
pub type cuMemsetD2D16_params = cuMemsetD2D16_params_st;
pub type cuMemsetD2D16_v2_params = cuMemsetD2D16_v2_params_st;
pub type cuMemsetD2D16_v2_ptds_params = cuMemsetD2D16_v2_ptds_params_st;
pub type cuMemsetD2D32Async_params = cuMemsetD2D32Async_params_st;
pub type cuMemsetD2D32Async_ptsz_params = cuMemsetD2D32Async_ptsz_params_st;
pub type cuMemsetD2D32_params = cuMemsetD2D32_params_st;
pub type cuMemsetD2D32_v2_params = cuMemsetD2D32_v2_params_st;
pub type cuMemsetD2D32_v2_ptds_params = cuMemsetD2D32_v2_ptds_params_st;
pub type cuMemsetD2D8Async_params = cuMemsetD2D8Async_params_st;
pub type cuMemsetD2D8Async_ptsz_params = cuMemsetD2D8Async_ptsz_params_st;
pub type cuMemsetD2D8_params = cuMemsetD2D8_params_st;
pub type cuMemsetD2D8_v2_params = cuMemsetD2D8_v2_params_st;
pub type cuMemsetD2D8_v2_ptds_params = cuMemsetD2D8_v2_ptds_params_st;
pub type cuMemsetD32Async_params = cuMemsetD32Async_params_st;
pub type cuMemsetD32Async_ptsz_params = cuMemsetD32Async_ptsz_params_st;
pub type cuMemsetD32_params = cuMemsetD32_params_st;
pub type cuMemsetD32_v2_params = cuMemsetD32_v2_params_st;
pub type cuMemsetD32_v2_ptds_params = cuMemsetD32_v2_ptds_params_st;
pub type cuMemsetD8Async_params = cuMemsetD8Async_params_st;
pub type cuMemsetD8Async_ptsz_params = cuMemsetD8Async_ptsz_params_st;
pub type cuMemsetD8_params = cuMemsetD8_params_st;
pub type cuMemsetD8_v2_params = cuMemsetD8_v2_params_st;
pub type cuMemsetD8_v2_ptds_params = cuMemsetD8_v2_ptds_params_st;
pub type cuMipmappedArrayCreate_params = cuMipmappedArrayCreate_params_st;
pub type cuMipmappedArrayDestroy_params = cuMipmappedArrayDestroy_params_st;
pub type cuMipmappedArrayGetLevel_params = cuMipmappedArrayGetLevel_params_st;
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuMipmappedArrayGetMemoryRequirements_params =
    cuMipmappedArrayGetMemoryRequirements_params_st;
pub type cuMipmappedArrayGetSparseProperties_params = cuMipmappedArrayGetSparseProperties_params_st;
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuModuleEnumerateFunctions_params = cuModuleEnumerateFunctions_params_st;
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuModuleGetFunctionCount_params = cuModuleGetFunctionCount_params_st;
pub type cuModuleGetFunction_params = cuModuleGetFunction_params_st;
pub type cuModuleGetGlobal_params = cuModuleGetGlobal_params_st;
pub type cuModuleGetGlobal_v2_params = cuModuleGetGlobal_v2_params_st;
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuModuleGetLoadingMode_params = cuModuleGetLoadingMode_params_st;
pub type cuModuleGetSurfRef_params = cuModuleGetSurfRef_params_st;
pub type cuModuleGetTexRef_params = cuModuleGetTexRef_params_st;
pub type cuModuleLoadDataEx_params = cuModuleLoadDataEx_params_st;
pub type cuModuleLoadData_params = cuModuleLoadData_params_st;
pub type cuModuleLoadFatBinary_params = cuModuleLoadFatBinary_params_st;
pub type cuModuleLoad_params = cuModuleLoad_params_st;
pub type cuModuleUnload_params = cuModuleUnload_params_st;
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuMulticastAddDevice_params = cuMulticastAddDevice_params_st;
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuMulticastBindAddr_params = cuMulticastBindAddr_params_st;
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuMulticastBindMem_params = cuMulticastBindMem_params_st;
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuMulticastCreate_params = cuMulticastCreate_params_st;
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuMulticastGetGranularity_params = cuMulticastGetGranularity_params_st;
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuMulticastUnbind_params = cuMulticastUnbind_params_st;
pub type cuOccupancyAvailableDynamicSMemPerBlock_params =
    cuOccupancyAvailableDynamicSMemPerBlock_params_st;
pub type cuOccupancyMaxActiveBlocksPerMultiprocessorWithFlags_params =
    cuOccupancyMaxActiveBlocksPerMultiprocessorWithFlags_params_st;
pub type cuOccupancyMaxActiveBlocksPerMultiprocessor_params =
    cuOccupancyMaxActiveBlocksPerMultiprocessor_params_st;
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuOccupancyMaxActiveClusters_params = cuOccupancyMaxActiveClusters_params_st;
pub type cuOccupancyMaxPotentialBlockSizeWithFlags_params =
    cuOccupancyMaxPotentialBlockSizeWithFlags_params_st;
pub type cuOccupancyMaxPotentialBlockSize_params = cuOccupancyMaxPotentialBlockSize_params_st;
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuOccupancyMaxPotentialClusterSize_params = cuOccupancyMaxPotentialClusterSize_params_st;
pub type cuParamSetSize_params = cuParamSetSize_params_st;
pub type cuParamSetTexRef_params = cuParamSetTexRef_params_st;
pub type cuParamSetf_params = cuParamSetf_params_st;
pub type cuParamSeti_params = cuParamSeti_params_st;
pub type cuParamSetv_params = cuParamSetv_params_st;
pub type cuPointerGetAttribute_params = cuPointerGetAttribute_params_st;
pub type cuPointerGetAttributes_params = cuPointerGetAttributes_params_st;
pub type cuPointerSetAttribute_params = cuPointerSetAttribute_params_st;
pub type cuSignalExternalSemaphoresAsync_params = cuSignalExternalSemaphoresAsync_params_st;
pub type cuSignalExternalSemaphoresAsync_ptsz_params =
    cuSignalExternalSemaphoresAsync_ptsz_params_st;
pub type cuStreamAddCallback_params = cuStreamAddCallback_params_st;
pub type cuStreamAddCallback_ptsz_params = cuStreamAddCallback_ptsz_params_st;
pub type cuStreamAttachMemAsync_params = cuStreamAttachMemAsync_params_st;
pub type cuStreamAttachMemAsync_ptsz_params = cuStreamAttachMemAsync_ptsz_params_st;
pub type cuStreamBatchMemOp_params = cuStreamBatchMemOp_params_st;
pub type cuStreamBatchMemOp_ptsz_params = cuStreamBatchMemOp_ptsz_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuStreamBatchMemOp_v2_params = cuStreamBatchMemOp_v2_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuStreamBatchMemOp_v2_ptsz_params = cuStreamBatchMemOp_v2_ptsz_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuStreamBeginCaptureToGraph_params = cuStreamBeginCaptureToGraph_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuStreamBeginCaptureToGraph_ptsz_params = cuStreamBeginCaptureToGraph_ptsz_params_st;
pub type cuStreamBeginCapture_params = cuStreamBeginCapture_params_st;
pub type cuStreamBeginCapture_ptsz_params = cuStreamBeginCapture_ptsz_params_st;
pub type cuStreamBeginCapture_v2_params = cuStreamBeginCapture_v2_params_st;
pub type cuStreamBeginCapture_v2_ptsz_params = cuStreamBeginCapture_v2_ptsz_params_st;
pub type cuStreamCopyAttributes_params = cuStreamCopyAttributes_params_st;
pub type cuStreamCopyAttributes_ptsz_params = cuStreamCopyAttributes_ptsz_params_st;
pub type cuStreamCreateWithPriority_params = cuStreamCreateWithPriority_params_st;
pub type cuStreamCreate_params = cuStreamCreate_params_st;
pub type cuStreamDestroy_params = cuStreamDestroy_params_st;
pub type cuStreamDestroy_v2_params = cuStreamDestroy_v2_params_st;
pub type cuStreamEndCapture_params = cuStreamEndCapture_params_st;
pub type cuStreamEndCapture_ptsz_params = cuStreamEndCapture_ptsz_params_st;
pub type cuStreamGetAttribute_params = cuStreamGetAttribute_params_st;
pub type cuStreamGetAttribute_ptsz_params = cuStreamGetAttribute_ptsz_params_st;
pub type cuStreamGetCaptureInfo_params = cuStreamGetCaptureInfo_params_st;
pub type cuStreamGetCaptureInfo_ptsz_params = cuStreamGetCaptureInfo_ptsz_params_st;
pub type cuStreamGetCaptureInfo_v2_params = cuStreamGetCaptureInfo_v2_params_st;
pub type cuStreamGetCaptureInfo_v2_ptsz_params = cuStreamGetCaptureInfo_v2_ptsz_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuStreamGetCaptureInfo_v3_params = cuStreamGetCaptureInfo_v3_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuStreamGetCaptureInfo_v3_ptsz_params = cuStreamGetCaptureInfo_v3_ptsz_params_st;
pub type cuStreamGetCtx_params = cuStreamGetCtx_params_st;
pub type cuStreamGetCtx_ptsz_params = cuStreamGetCtx_ptsz_params_st;
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuStreamGetCtx_v2_params = cuStreamGetCtx_v2_params_st;
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuStreamGetCtx_v2_ptsz_params = cuStreamGetCtx_v2_ptsz_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cuStreamGetDevResource_params = cuStreamGetDevResource_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cuStreamGetDevResource_ptsz_params = cuStreamGetDevResource_ptsz_params_st;
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuStreamGetDevice_params = cuStreamGetDevice_params_st;
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuStreamGetDevice_ptsz_params = cuStreamGetDevice_ptsz_params_st;
pub type cuStreamGetFlags_params = cuStreamGetFlags_params_st;
pub type cuStreamGetFlags_ptsz_params = cuStreamGetFlags_ptsz_params_st;
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuStreamGetGreenCtx_params = cuStreamGetGreenCtx_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuStreamGetId_params = cuStreamGetId_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuStreamGetId_ptsz_params = cuStreamGetId_ptsz_params_st;
pub type cuStreamGetPriority_params = cuStreamGetPriority_params_st;
pub type cuStreamGetPriority_ptsz_params = cuStreamGetPriority_ptsz_params_st;
pub type cuStreamIsCapturing_params = cuStreamIsCapturing_params_st;
pub type cuStreamIsCapturing_ptsz_params = cuStreamIsCapturing_ptsz_params_st;
pub type cuStreamQuery_params = cuStreamQuery_params_st;
pub type cuStreamQuery_ptsz_params = cuStreamQuery_ptsz_params_st;
pub type cuStreamSetAttribute_params = cuStreamSetAttribute_params_st;
pub type cuStreamSetAttribute_ptsz_params = cuStreamSetAttribute_ptsz_params_st;
pub type cuStreamSynchronize_params = cuStreamSynchronize_params_st;
pub type cuStreamSynchronize_ptsz_params = cuStreamSynchronize_ptsz_params_st;
pub type cuStreamUpdateCaptureDependencies_params = cuStreamUpdateCaptureDependencies_params_st;
pub type cuStreamUpdateCaptureDependencies_ptsz_params =
    cuStreamUpdateCaptureDependencies_ptsz_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuStreamUpdateCaptureDependencies_v2_params =
    cuStreamUpdateCaptureDependencies_v2_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuStreamUpdateCaptureDependencies_v2_ptsz_params =
    cuStreamUpdateCaptureDependencies_v2_ptsz_params_st;
pub type cuStreamWaitEvent_params = cuStreamWaitEvent_params_st;
pub type cuStreamWaitEvent_ptsz_params = cuStreamWaitEvent_ptsz_params_st;
pub type cuStreamWaitValue32_params = cuStreamWaitValue32_params_st;
pub type cuStreamWaitValue32_ptsz_params = cuStreamWaitValue32_ptsz_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuStreamWaitValue32_v2_params = cuStreamWaitValue32_v2_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuStreamWaitValue32_v2_ptsz_params = cuStreamWaitValue32_v2_ptsz_params_st;
pub type cuStreamWaitValue64_params = cuStreamWaitValue64_params_st;
pub type cuStreamWaitValue64_ptsz_params = cuStreamWaitValue64_ptsz_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuStreamWaitValue64_v2_params = cuStreamWaitValue64_v2_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuStreamWaitValue64_v2_ptsz_params = cuStreamWaitValue64_v2_ptsz_params_st;
pub type cuStreamWriteValue32_params = cuStreamWriteValue32_params_st;
pub type cuStreamWriteValue32_ptsz_params = cuStreamWriteValue32_ptsz_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuStreamWriteValue32_v2_params = cuStreamWriteValue32_v2_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuStreamWriteValue32_v2_ptsz_params = cuStreamWriteValue32_v2_ptsz_params_st;
pub type cuStreamWriteValue64_params = cuStreamWriteValue64_params_st;
pub type cuStreamWriteValue64_ptsz_params = cuStreamWriteValue64_ptsz_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuStreamWriteValue64_v2_params = cuStreamWriteValue64_v2_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuStreamWriteValue64_v2_ptsz_params = cuStreamWriteValue64_v2_ptsz_params_st;
pub type cuSurfObjectCreate_params = cuSurfObjectCreate_params_st;
pub type cuSurfObjectDestroy_params = cuSurfObjectDestroy_params_st;
pub type cuSurfObjectGetResourceDesc_params = cuSurfObjectGetResourceDesc_params_st;
pub type cuSurfRefGetArray_params = cuSurfRefGetArray_params_st;
pub type cuSurfRefSetArray_params = cuSurfRefSetArray_params_st;
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
pub type cuTensorMapEncodeIm2colWide_params = cuTensorMapEncodeIm2colWide_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuTensorMapEncodeIm2col_params = cuTensorMapEncodeIm2col_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuTensorMapEncodeTiled_params = cuTensorMapEncodeTiled_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cuTensorMapReplaceAddress_params = cuTensorMapReplaceAddress_params_st;
pub type cuTexObjectCreate_params = cuTexObjectCreate_params_st;
pub type cuTexObjectDestroy_params = cuTexObjectDestroy_params_st;
pub type cuTexObjectGetResourceDesc_params = cuTexObjectGetResourceDesc_params_st;
pub type cuTexObjectGetResourceViewDesc_params = cuTexObjectGetResourceViewDesc_params_st;
pub type cuTexObjectGetTextureDesc_params = cuTexObjectGetTextureDesc_params_st;
pub type cuTexRefCreate_params = cuTexRefCreate_params_st;
pub type cuTexRefDestroy_params = cuTexRefDestroy_params_st;
pub type cuTexRefGetAddressMode_params = cuTexRefGetAddressMode_params_st;
pub type cuTexRefGetAddress_params = cuTexRefGetAddress_params_st;
pub type cuTexRefGetAddress_v2_params = cuTexRefGetAddress_v2_params_st;
pub type cuTexRefGetArray_params = cuTexRefGetArray_params_st;
pub type cuTexRefGetBorderColor_params = cuTexRefGetBorderColor_params_st;
pub type cuTexRefGetFilterMode_params = cuTexRefGetFilterMode_params_st;
pub type cuTexRefGetFlags_params = cuTexRefGetFlags_params_st;
pub type cuTexRefGetFormat_params = cuTexRefGetFormat_params_st;
pub type cuTexRefGetMaxAnisotropy_params = cuTexRefGetMaxAnisotropy_params_st;
pub type cuTexRefGetMipmapFilterMode_params = cuTexRefGetMipmapFilterMode_params_st;
pub type cuTexRefGetMipmapLevelBias_params = cuTexRefGetMipmapLevelBias_params_st;
pub type cuTexRefGetMipmapLevelClamp_params = cuTexRefGetMipmapLevelClamp_params_st;
pub type cuTexRefGetMipmappedArray_params = cuTexRefGetMipmappedArray_params_st;
pub type cuTexRefSetAddress2D_params = cuTexRefSetAddress2D_params_st;
pub type cuTexRefSetAddress2D_v2_params = cuTexRefSetAddress2D_v2_params_st;
pub type cuTexRefSetAddress2D_v3_params = cuTexRefSetAddress2D_v3_params_st;
pub type cuTexRefSetAddressMode_params = cuTexRefSetAddressMode_params_st;
pub type cuTexRefSetAddress_params = cuTexRefSetAddress_params_st;
pub type cuTexRefSetAddress_v2_params = cuTexRefSetAddress_v2_params_st;
pub type cuTexRefSetArray_params = cuTexRefSetArray_params_st;
pub type cuTexRefSetBorderColor_params = cuTexRefSetBorderColor_params_st;
pub type cuTexRefSetFilterMode_params = cuTexRefSetFilterMode_params_st;
pub type cuTexRefSetFlags_params = cuTexRefSetFlags_params_st;
pub type cuTexRefSetFormat_params = cuTexRefSetFormat_params_st;
pub type cuTexRefSetMaxAnisotropy_params = cuTexRefSetMaxAnisotropy_params_st;
pub type cuTexRefSetMipmapFilterMode_params = cuTexRefSetMipmapFilterMode_params_st;
pub type cuTexRefSetMipmapLevelBias_params = cuTexRefSetMipmapLevelBias_params_st;
pub type cuTexRefSetMipmapLevelClamp_params = cuTexRefSetMipmapLevelClamp_params_st;
pub type cuTexRefSetMipmappedArray_params = cuTexRefSetMipmappedArray_params_st;
pub type cuThreadExchangeStreamCaptureMode_params = cuThreadExchangeStreamCaptureMode_params_st;
pub type cuUserObjectCreate_params = cuUserObjectCreate_params_st;
pub type cuUserObjectRelease_params = cuUserObjectRelease_params_st;
pub type cuUserObjectRetain_params = cuUserObjectRetain_params_st;
pub type cuWaitExternalSemaphoresAsync_params = cuWaitExternalSemaphoresAsync_params_st;
pub type cuWaitExternalSemaphoresAsync_ptsz_params = cuWaitExternalSemaphoresAsync_ptsz_params_st;
pub type cudaArrayGetInfo_v4010_params = cudaArrayGetInfo_v4010_params_st;
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaArrayGetMemoryRequirements_v11060_params =
    cudaArrayGetMemoryRequirements_v11060_params_st;
pub type cudaArrayGetPlane_v11020_params = cudaArrayGetPlane_v11020_params_st;
pub type cudaArrayGetSparseProperties_v11010_params = cudaArrayGetSparseProperties_v11010_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
pub type cudaBindSurfaceToArray_v3020_params = cudaBindSurfaceToArray_v3020_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
pub type cudaBindTexture2D_v3020_params = cudaBindTexture2D_v3020_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
pub type cudaBindTextureToArray_v3020_params = cudaBindTextureToArray_v3020_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
pub type cudaBindTextureToMipmappedArray_v5000_params =
    cudaBindTextureToMipmappedArray_v5000_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
pub type cudaBindTexture_v3020_params = cudaBindTexture_v3020_params_st;
pub type cudaChooseDevice_v3020_params = cudaChooseDevice_v3020_params_st;
pub type cudaCreateChannelDesc_v3020_params = cudaCreateChannelDesc_v3020_params_st;
pub type cudaCreateSurfaceObject_v5000_params = cudaCreateSurfaceObject_v5000_params_st;
pub type cudaCreateTextureObject_v5000_params = cudaCreateTextureObject_v5000_params_st;
pub type cudaDestroyExternalMemory_v10000_params = cudaDestroyExternalMemory_v10000_params_st;
pub type cudaDestroyExternalSemaphore_v10000_params = cudaDestroyExternalSemaphore_v10000_params_st;
pub type cudaDestroySurfaceObject_v5000_params = cudaDestroySurfaceObject_v5000_params_st;
pub type cudaDestroyTextureObject_v5000_params = cudaDestroyTextureObject_v5000_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cudaDevResourceGenerateDesc_v13010_params = cudaDevResourceGenerateDesc_v13010_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cudaDevSmResourceSplitByCount_v13010_params =
    cudaDevSmResourceSplitByCount_v13010_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cudaDevSmResourceSplit_v13010_params = cudaDevSmResourceSplit_v13010_params_st;
pub type cudaDeviceCanAccessPeer_v4000_params = cudaDeviceCanAccessPeer_v4000_params_st;
pub type cudaDeviceDisablePeerAccess_v4000_params = cudaDeviceDisablePeerAccess_v4000_params_st;
pub type cudaDeviceEnablePeerAccess_v4000_params = cudaDeviceEnablePeerAccess_v4000_params_st;
pub type cudaDeviceFlushGPUDirectRDMAWrites_v11030_params =
    cudaDeviceFlushGPUDirectRDMAWrites_v11030_params_st;
pub type cudaDeviceGetAttribute_v5000_params = cudaDeviceGetAttribute_v5000_params_st;
pub type cudaDeviceGetByPCIBusId_v4010_params = cudaDeviceGetByPCIBusId_v4010_params_st;
pub type cudaDeviceGetCacheConfig_v3020_params = cudaDeviceGetCacheConfig_v3020_params_st;
pub type cudaDeviceGetDefaultMemPool_v11020_params = cudaDeviceGetDefaultMemPool_v11020_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cudaDeviceGetDevResource_v13010_params = cudaDeviceGetDevResource_v13010_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cudaDeviceGetExecutionCtx_v13010_params = cudaDeviceGetExecutionCtx_v13010_params_st;
pub type cudaDeviceGetGraphMemAttribute_v11040_params =
    cudaDeviceGetGraphMemAttribute_v11040_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaDeviceGetHostAtomicCapabilities_v13000_params =
    cudaDeviceGetHostAtomicCapabilities_v13000_params_st;
pub type cudaDeviceGetLimit_v3020_params = cudaDeviceGetLimit_v3020_params_st;
pub type cudaDeviceGetMemPool_v11020_params = cudaDeviceGetMemPool_v11020_params_st;
pub type cudaDeviceGetNvSciSyncAttributes_v10020_params =
    cudaDeviceGetNvSciSyncAttributes_v10020_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaDeviceGetP2PAtomicCapabilities_v13000_params =
    cudaDeviceGetP2PAtomicCapabilities_v13000_params_st;
pub type cudaDeviceGetP2PAttribute_v8000_params = cudaDeviceGetP2PAttribute_v8000_params_st;
pub type cudaDeviceGetPCIBusId_v4010_params = cudaDeviceGetPCIBusId_v4010_params_st;
pub type cudaDeviceGetSharedMemConfig_v4020_params = cudaDeviceGetSharedMemConfig_v4020_params_st;
pub type cudaDeviceGetStreamPriorityRange_v5050_params =
    cudaDeviceGetStreamPriorityRange_v5050_params_st;
pub type cudaDeviceGetTexture1DLinearMaxWidth_v11010_params =
    cudaDeviceGetTexture1DLinearMaxWidth_v11010_params_st;
pub type cudaDeviceGraphMemTrim_v11040_params = cudaDeviceGraphMemTrim_v11040_params_st;
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaDeviceRegisterAsyncNotification_v12040_params =
    cudaDeviceRegisterAsyncNotification_v12040_params_st;
pub type cudaDeviceSetCacheConfig_v3020_params = cudaDeviceSetCacheConfig_v3020_params_st;
pub type cudaDeviceSetGraphMemAttribute_v11040_params =
    cudaDeviceSetGraphMemAttribute_v11040_params_st;
pub type cudaDeviceSetLimit_v3020_params = cudaDeviceSetLimit_v3020_params_st;
pub type cudaDeviceSetMemPool_v11020_params = cudaDeviceSetMemPool_v11020_params_st;
pub type cudaDeviceSetSharedMemConfig_v4020_params = cudaDeviceSetSharedMemConfig_v4020_params_st;
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaDeviceUnregisterAsyncNotification_v12040_params =
    cudaDeviceUnregisterAsyncNotification_v12040_params_st;
pub type cudaDriverGetVersion_v3020_params = cudaDriverGetVersion_v3020_params_st;
pub type cudaEventCreateWithFlags_v3020_params = cudaEventCreateWithFlags_v3020_params_st;
pub type cudaEventCreate_v3020_params = cudaEventCreate_v3020_params_st;
pub type cudaEventDestroy_v3020_params = cudaEventDestroy_v3020_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaEventElapsedTime_v12080_params = cudaEventElapsedTime_v12080_params_st;
#[cfg(any(feature = "cuda-12080", feature = "cuda-12090"))]
pub type cudaEventElapsedTime_v2_v12080_params = cudaEventElapsedTime_v2_v12080_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaEventElapsedTime_v3020_params = cudaEventElapsedTime_v3020_params_st;
pub type cudaEventQuery_v3020_params = cudaEventQuery_v3020_params_st;
pub type cudaEventRecordWithFlags_ptsz_v11010_params =
    cudaEventRecordWithFlags_ptsz_v11010_params_st;
pub type cudaEventRecordWithFlags_v11010_params = cudaEventRecordWithFlags_v11010_params_st;
pub type cudaEventRecord_ptsz_v7000_params = cudaEventRecord_ptsz_v7000_params_st;
pub type cudaEventRecord_v3020_params = cudaEventRecord_v3020_params_st;
pub type cudaEventSynchronize_v3020_params = cudaEventSynchronize_v3020_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cudaExecutionCtxDestroy_v13010_params = cudaExecutionCtxDestroy_v13010_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cudaExecutionCtxGetDevResource_v13010_params =
    cudaExecutionCtxGetDevResource_v13010_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cudaExecutionCtxGetDevice_v13010_params = cudaExecutionCtxGetDevice_v13010_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cudaExecutionCtxGetId_v13010_params = cudaExecutionCtxGetId_v13010_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cudaExecutionCtxRecordEvent_v13010_params = cudaExecutionCtxRecordEvent_v13010_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cudaExecutionCtxStreamCreate_v13010_params = cudaExecutionCtxStreamCreate_v13010_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cudaExecutionCtxSynchronize_v13010_params = cudaExecutionCtxSynchronize_v13010_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cudaExecutionCtxWaitEvent_v13010_params = cudaExecutionCtxWaitEvent_v13010_params_st;
pub type cudaExternalMemoryGetMappedBuffer_v10000_params =
    cudaExternalMemoryGetMappedBuffer_v10000_params_st;
pub type cudaExternalMemoryGetMappedMipmappedArray_v10000_params =
    cudaExternalMemoryGetMappedMipmappedArray_v10000_params_st;
pub type cudaFreeArray_v3020_params = cudaFreeArray_v3020_params_st;
pub type cudaFreeAsync_ptsz_v11020_params = cudaFreeAsync_ptsz_v11020_params_st;
pub type cudaFreeAsync_v11020_params = cudaFreeAsync_v11020_params_st;
pub type cudaFreeHost_v3020_params = cudaFreeHost_v3020_params_st;
pub type cudaFreeMipmappedArray_v5000_params = cudaFreeMipmappedArray_v5000_params_st;
pub type cudaFree_v3020_params = cudaFree_v3020_params_st;
pub type cudaFuncGetAttributes_v3020_params = cudaFuncGetAttributes_v3020_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaFuncGetName_v12030_params = cudaFuncGetName_v12030_params_st;
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaFuncGetParamInfo_v12040_params = cudaFuncGetParamInfo_v12040_params_st;
pub type cudaFuncSetAttribute_v9000_params = cudaFuncSetAttribute_v9000_params_st;
pub type cudaFuncSetCacheConfig_v3020_params = cudaFuncSetCacheConfig_v3020_params_st;
pub type cudaFuncSetSharedMemConfig_v4020_params = cudaFuncSetSharedMemConfig_v4020_params_st;
pub type cudaGetChannelDesc_v3020_params = cudaGetChannelDesc_v3020_params_st;
pub type cudaGetDeviceCount_v3020_params = cudaGetDeviceCount_v3020_params_st;
pub type cudaGetDeviceFlags_v7000_params = cudaGetDeviceFlags_v7000_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaGetDeviceProperties_v12000_params = cudaGetDeviceProperties_v12000_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaGetDeviceProperties_v2_v12000_params = cudaGetDeviceProperties_v2_v12000_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaGetDeviceProperties_v3020_params = cudaGetDeviceProperties_v3020_params_st;
pub type cudaGetDevice_v3020_params = cudaGetDevice_v3020_params_st;
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaGetDriverEntryPointByVersion_ptsz_v12050_params =
    cudaGetDriverEntryPointByVersion_ptsz_v12050_params_st;
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaGetDriverEntryPointByVersion_v12050_params =
    cudaGetDriverEntryPointByVersion_v12050_params_st;
pub type cudaGetDriverEntryPoint_ptsz_v11030_params = cudaGetDriverEntryPoint_ptsz_v11030_params_st;
pub type cudaGetDriverEntryPoint_v11030_params = cudaGetDriverEntryPoint_v11030_params_st;
pub type cudaGetErrorName_v6050_params = cudaGetErrorName_v6050_params_st;
pub type cudaGetErrorString_v3020_params = cudaGetErrorString_v3020_params_st;
pub type cudaGetFuncBySymbol_v11000_params = cudaGetFuncBySymbol_v11000_params_st;
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaGetKernel_v12000_params = cudaGetKernel_v12000_params_st;
pub type cudaGetMipmappedArrayLevel_v5000_params = cudaGetMipmappedArrayLevel_v5000_params_st;
pub type cudaGetSurfaceObjectResourceDesc_v5000_params =
    cudaGetSurfaceObjectResourceDesc_v5000_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
pub type cudaGetSurfaceReference_v3020_params = cudaGetSurfaceReference_v3020_params_st;
pub type cudaGetSymbolAddress_v3020_params = cudaGetSymbolAddress_v3020_params_st;
pub type cudaGetSymbolSize_v3020_params = cudaGetSymbolSize_v3020_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
pub type cudaGetTextureAlignmentOffset_v3020_params = cudaGetTextureAlignmentOffset_v3020_params_st;
pub type cudaGetTextureObjectResourceDesc_v5000_params =
    cudaGetTextureObjectResourceDesc_v5000_params_st;
pub type cudaGetTextureObjectResourceViewDesc_v5000_params =
    cudaGetTextureObjectResourceViewDesc_v5000_params_st;
pub type cudaGetTextureObjectTextureDesc_v5000_params =
    cudaGetTextureObjectTextureDesc_v5000_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
pub type cudaGetTextureReference_v3020_params = cudaGetTextureReference_v3020_params_st;
pub type cudaGraphAddChildGraphNode_v10000_params = cudaGraphAddChildGraphNode_v10000_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaGraphAddDependencies_v10000_params = cudaGraphAddDependencies_v10000_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaGraphAddDependencies_v12030_params = cudaGraphAddDependencies_v12030_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaGraphAddDependencies_v2_v12030_params = cudaGraphAddDependencies_v2_v12030_params_st;
pub type cudaGraphAddEmptyNode_v10000_params = cudaGraphAddEmptyNode_v10000_params_st;
pub type cudaGraphAddEventRecordNode_v11010_params = cudaGraphAddEventRecordNode_v11010_params_st;
pub type cudaGraphAddEventWaitNode_v11010_params = cudaGraphAddEventWaitNode_v11010_params_st;
pub type cudaGraphAddExternalSemaphoresSignalNode_v11020_params =
    cudaGraphAddExternalSemaphoresSignalNode_v11020_params_st;
pub type cudaGraphAddExternalSemaphoresWaitNode_v11020_params =
    cudaGraphAddExternalSemaphoresWaitNode_v11020_params_st;
pub type cudaGraphAddHostNode_v10000_params = cudaGraphAddHostNode_v10000_params_st;
pub type cudaGraphAddKernelNode_v10000_params = cudaGraphAddKernelNode_v10000_params_st;
pub type cudaGraphAddMemAllocNode_v11040_params = cudaGraphAddMemAllocNode_v11040_params_st;
pub type cudaGraphAddMemFreeNode_v11040_params = cudaGraphAddMemFreeNode_v11040_params_st;
pub type cudaGraphAddMemcpyNode1D_v11010_params = cudaGraphAddMemcpyNode1D_v11010_params_st;
pub type cudaGraphAddMemcpyNodeFromSymbol_v11010_params =
    cudaGraphAddMemcpyNodeFromSymbol_v11010_params_st;
pub type cudaGraphAddMemcpyNodeToSymbol_v11010_params =
    cudaGraphAddMemcpyNodeToSymbol_v11010_params_st;
pub type cudaGraphAddMemcpyNode_v10000_params = cudaGraphAddMemcpyNode_v10000_params_st;
pub type cudaGraphAddMemsetNode_v10000_params = cudaGraphAddMemsetNode_v10000_params_st;
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaGraphAddNode_v12020_params = cudaGraphAddNode_v12020_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaGraphAddNode_v12030_params = cudaGraphAddNode_v12030_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaGraphAddNode_v2_v12030_params = cudaGraphAddNode_v2_v12030_params_st;
pub type cudaGraphChildGraphNodeGetGraph_v10000_params =
    cudaGraphChildGraphNodeGetGraph_v10000_params_st;
pub type cudaGraphClone_v10000_params = cudaGraphClone_v10000_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaGraphConditionalHandleCreate_v12030_params =
    cudaGraphConditionalHandleCreate_v12030_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cudaGraphConditionalHandleCreate_v2_v13010_params =
    cudaGraphConditionalHandleCreate_v2_v13010_params_st;
pub type cudaGraphCreate_v10000_params = cudaGraphCreate_v10000_params_st;
pub type cudaGraphDebugDotPrint_v11030_params = cudaGraphDebugDotPrint_v11030_params_st;
pub type cudaGraphDestroyNode_v10000_params = cudaGraphDestroyNode_v10000_params_st;
pub type cudaGraphDestroy_v10000_params = cudaGraphDestroy_v10000_params_st;
pub type cudaGraphEventRecordNodeGetEvent_v11010_params =
    cudaGraphEventRecordNodeGetEvent_v11010_params_st;
pub type cudaGraphEventRecordNodeSetEvent_v11010_params =
    cudaGraphEventRecordNodeSetEvent_v11010_params_st;
pub type cudaGraphEventWaitNodeGetEvent_v11010_params =
    cudaGraphEventWaitNodeGetEvent_v11010_params_st;
pub type cudaGraphEventWaitNodeSetEvent_v11010_params =
    cudaGraphEventWaitNodeSetEvent_v11010_params_st;
pub type cudaGraphExecChildGraphNodeSetParams_v11010_params =
    cudaGraphExecChildGraphNodeSetParams_v11010_params_st;
pub type cudaGraphExecDestroy_v10000_params = cudaGraphExecDestroy_v10000_params_st;
pub type cudaGraphExecEventRecordNodeSetEvent_v11010_params =
    cudaGraphExecEventRecordNodeSetEvent_v11010_params_st;
pub type cudaGraphExecEventWaitNodeSetEvent_v11010_params =
    cudaGraphExecEventWaitNodeSetEvent_v11010_params_st;
pub type cudaGraphExecExternalSemaphoresSignalNodeSetParams_v11020_params =
    cudaGraphExecExternalSemaphoresSignalNodeSetParams_v11020_params_st;
pub type cudaGraphExecExternalSemaphoresWaitNodeSetParams_v11020_params =
    cudaGraphExecExternalSemaphoresWaitNodeSetParams_v11020_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaGraphExecGetFlags_v12000_params = cudaGraphExecGetFlags_v12000_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cudaGraphExecGetId_v13010_params = cudaGraphExecGetId_v13010_params_st;
pub type cudaGraphExecHostNodeSetParams_v10020_params =
    cudaGraphExecHostNodeSetParams_v10020_params_st;
pub type cudaGraphExecKernelNodeSetParams_v10010_params =
    cudaGraphExecKernelNodeSetParams_v10010_params_st;
pub type cudaGraphExecMemcpyNodeSetParams1D_v11010_params =
    cudaGraphExecMemcpyNodeSetParams1D_v11010_params_st;
pub type cudaGraphExecMemcpyNodeSetParamsFromSymbol_v11010_params =
    cudaGraphExecMemcpyNodeSetParamsFromSymbol_v11010_params_st;
pub type cudaGraphExecMemcpyNodeSetParamsToSymbol_v11010_params =
    cudaGraphExecMemcpyNodeSetParamsToSymbol_v11010_params_st;
pub type cudaGraphExecMemcpyNodeSetParams_v10020_params =
    cudaGraphExecMemcpyNodeSetParams_v10020_params_st;
pub type cudaGraphExecMemsetNodeSetParams_v10020_params =
    cudaGraphExecMemsetNodeSetParams_v10020_params_st;
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaGraphExecNodeSetParams_v12020_params = cudaGraphExecNodeSetParams_v12020_params_st;
pub type cudaGraphExecUpdate_v10020_params = cudaGraphExecUpdate_v10020_params_st;
pub type cudaGraphExternalSemaphoresSignalNodeGetParams_v11020_params =
    cudaGraphExternalSemaphoresSignalNodeGetParams_v11020_params_st;
pub type cudaGraphExternalSemaphoresSignalNodeSetParams_v11020_params =
    cudaGraphExternalSemaphoresSignalNodeSetParams_v11020_params_st;
pub type cudaGraphExternalSemaphoresWaitNodeGetParams_v11020_params =
    cudaGraphExternalSemaphoresWaitNodeGetParams_v11020_params_st;
pub type cudaGraphExternalSemaphoresWaitNodeSetParams_v11020_params =
    cudaGraphExternalSemaphoresWaitNodeSetParams_v11020_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaGraphGetEdges_v10000_params = cudaGraphGetEdges_v10000_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaGraphGetEdges_v12030_params = cudaGraphGetEdges_v12030_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaGraphGetEdges_v2_v12030_params = cudaGraphGetEdges_v2_v12030_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cudaGraphGetId_v13010_params = cudaGraphGetId_v13010_params_st;
pub type cudaGraphGetNodes_v10000_params = cudaGraphGetNodes_v10000_params_st;
pub type cudaGraphGetRootNodes_v10000_params = cudaGraphGetRootNodes_v10000_params_st;
pub type cudaGraphHostNodeGetParams_v10000_params = cudaGraphHostNodeGetParams_v10000_params_st;
pub type cudaGraphHostNodeSetParams_v10000_params = cudaGraphHostNodeSetParams_v10000_params_st;
pub type cudaGraphInstantiateWithFlags_v11040_params =
    cudaGraphInstantiateWithFlags_v11040_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaGraphInstantiateWithParams_ptsz_v12000_params =
    cudaGraphInstantiateWithParams_ptsz_v12000_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaGraphInstantiateWithParams_v12000_params =
    cudaGraphInstantiateWithParams_v12000_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
pub type cudaGraphInstantiate_v10000_params = cudaGraphInstantiate_v10000_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaGraphInstantiate_v12000_params = cudaGraphInstantiate_v12000_params_st;
pub type cudaGraphKernelNodeCopyAttributes_v11000_params =
    cudaGraphKernelNodeCopyAttributes_v11000_params_st;
pub type cudaGraphKernelNodeGetAttribute_v11000_params =
    cudaGraphKernelNodeGetAttribute_v11000_params_st;
pub type cudaGraphKernelNodeGetParams_v10000_params = cudaGraphKernelNodeGetParams_v10000_params_st;
pub type cudaGraphKernelNodeSetAttribute_v11000_params =
    cudaGraphKernelNodeSetAttribute_v11000_params_st;
pub type cudaGraphKernelNodeSetParams_v10000_params = cudaGraphKernelNodeSetParams_v10000_params_st;
pub type cudaGraphLaunch_ptsz_v10000_params = cudaGraphLaunch_ptsz_v10000_params_st;
pub type cudaGraphLaunch_v10000_params = cudaGraphLaunch_v10000_params_st;
pub type cudaGraphMemAllocNodeGetParams_v11040_params =
    cudaGraphMemAllocNodeGetParams_v11040_params_st;
pub type cudaGraphMemFreeNodeGetParams_v11040_params =
    cudaGraphMemFreeNodeGetParams_v11040_params_st;
pub type cudaGraphMemcpyNodeGetParams_v10000_params = cudaGraphMemcpyNodeGetParams_v10000_params_st;
pub type cudaGraphMemcpyNodeSetParams1D_v11010_params =
    cudaGraphMemcpyNodeSetParams1D_v11010_params_st;
pub type cudaGraphMemcpyNodeSetParamsFromSymbol_v11010_params =
    cudaGraphMemcpyNodeSetParamsFromSymbol_v11010_params_st;
pub type cudaGraphMemcpyNodeSetParamsToSymbol_v11010_params =
    cudaGraphMemcpyNodeSetParamsToSymbol_v11010_params_st;
pub type cudaGraphMemcpyNodeSetParams_v10000_params = cudaGraphMemcpyNodeSetParams_v10000_params_st;
pub type cudaGraphMemsetNodeGetParams_v10000_params = cudaGraphMemsetNodeGetParams_v10000_params_st;
pub type cudaGraphMemsetNodeSetParams_v10000_params = cudaGraphMemsetNodeSetParams_v10000_params_st;
pub type cudaGraphNodeFindInClone_v10000_params = cudaGraphNodeFindInClone_v10000_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cudaGraphNodeGetContainingGraph_v13010_params =
    cudaGraphNodeGetContainingGraph_v13010_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaGraphNodeGetDependencies_v10000_params = cudaGraphNodeGetDependencies_v10000_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaGraphNodeGetDependencies_v12030_params = cudaGraphNodeGetDependencies_v12030_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaGraphNodeGetDependencies_v2_v12030_params =
    cudaGraphNodeGetDependencies_v2_v12030_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaGraphNodeGetDependentNodes_v10000_params =
    cudaGraphNodeGetDependentNodes_v10000_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaGraphNodeGetDependentNodes_v12030_params =
    cudaGraphNodeGetDependentNodes_v12030_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaGraphNodeGetDependentNodes_v2_v12030_params =
    cudaGraphNodeGetDependentNodes_v2_v12030_params_st;
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaGraphNodeGetEnabled_v11060_params = cudaGraphNodeGetEnabled_v11060_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cudaGraphNodeGetLocalId_v13010_params = cudaGraphNodeGetLocalId_v13010_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cudaGraphNodeGetToolsId_v13010_params = cudaGraphNodeGetToolsId_v13010_params_st;
pub type cudaGraphNodeGetType_v10000_params = cudaGraphNodeGetType_v10000_params_st;
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaGraphNodeSetEnabled_v11060_params = cudaGraphNodeSetEnabled_v11060_params_st;
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaGraphNodeSetParams_v12020_params = cudaGraphNodeSetParams_v12020_params_st;
pub type cudaGraphReleaseUserObject_v11030_params = cudaGraphReleaseUserObject_v11030_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaGraphRemoveDependencies_v10000_params = cudaGraphRemoveDependencies_v10000_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaGraphRemoveDependencies_v12030_params = cudaGraphRemoveDependencies_v12030_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaGraphRemoveDependencies_v2_v12030_params =
    cudaGraphRemoveDependencies_v2_v12030_params_st;
pub type cudaGraphRetainUserObject_v11030_params = cudaGraphRetainUserObject_v11030_params_st;
pub type cudaGraphUpload_ptsz_v10000_params = cudaGraphUpload_ptsz_v10000_params_st;
pub type cudaGraphUpload_v10000_params = cudaGraphUpload_v10000_params_st;
pub type cudaGraphicsMapResources_v3020_params = cudaGraphicsMapResources_v3020_params_st;
pub type cudaGraphicsResourceGetMappedMipmappedArray_v5000_params =
    cudaGraphicsResourceGetMappedMipmappedArray_v5000_params_st;
pub type cudaGraphicsResourceGetMappedPointer_v3020_params =
    cudaGraphicsResourceGetMappedPointer_v3020_params_st;
pub type cudaGraphicsResourceSetMapFlags_v3020_params =
    cudaGraphicsResourceSetMapFlags_v3020_params_st;
pub type cudaGraphicsSubResourceGetMappedArray_v3020_params =
    cudaGraphicsSubResourceGetMappedArray_v3020_params_st;
pub type cudaGraphicsUnmapResources_v3020_params = cudaGraphicsUnmapResources_v3020_params_st;
pub type cudaGraphicsUnregisterResource_v3020_params =
    cudaGraphicsUnregisterResource_v3020_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cudaGreenCtxCreate_v13010_params = cudaGreenCtxCreate_v13010_params_st;
pub type cudaHostAlloc_v3020_params = cudaHostAlloc_v3020_params_st;
pub type cudaHostGetDevicePointer_v3020_params = cudaHostGetDevicePointer_v3020_params_st;
pub type cudaHostGetFlags_v3020_params = cudaHostGetFlags_v3020_params_st;
pub type cudaHostRegister_v4000_params = cudaHostRegister_v4000_params_st;
pub type cudaHostUnregister_v4000_params = cudaHostUnregister_v4000_params_st;
pub type cudaImportExternalMemory_v10000_params = cudaImportExternalMemory_v10000_params_st;
pub type cudaImportExternalSemaphore_v10000_params = cudaImportExternalSemaphore_v10000_params_st;
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaInitDevice_v12000_params = cudaInitDevice_v12000_params_st;
pub type cudaIpcCloseMemHandle_v4010_params = cudaIpcCloseMemHandle_v4010_params_st;
pub type cudaIpcGetEventHandle_v4010_params = cudaIpcGetEventHandle_v4010_params_st;
pub type cudaIpcGetMemHandle_v4010_params = cudaIpcGetMemHandle_v4010_params_st;
pub type cudaIpcOpenEventHandle_v4010_params = cudaIpcOpenEventHandle_v4010_params_st;
pub type cudaIpcOpenMemHandle_v4010_params = cudaIpcOpenMemHandle_v4010_params_st;
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaKernelSetAttributeForDevice_v12060_params =
    cudaKernelSetAttributeForDevice_v12060_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaLaunchCooperativeKernelMultiDevice_v9000_params =
    cudaLaunchCooperativeKernelMultiDevice_v9000_params_st;
pub type cudaLaunchCooperativeKernel_ptsz_v9000_params =
    cudaLaunchCooperativeKernel_ptsz_v9000_params_st;
pub type cudaLaunchCooperativeKernel_v9000_params = cudaLaunchCooperativeKernel_v9000_params_st;
pub type cudaLaunchHostFunc_ptsz_v10000_params = cudaLaunchHostFunc_ptsz_v10000_params_st;
pub type cudaLaunchHostFunc_v10000_params = cudaLaunchHostFunc_v10000_params_st;
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaLaunchKernelExC_ptsz_v11060_params = cudaLaunchKernelExC_ptsz_v11060_params_st;
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaLaunchKernelExC_v11060_params = cudaLaunchKernelExC_v11060_params_st;
pub type cudaLaunchKernel_ptsz_v7000_params = cudaLaunchKernel_ptsz_v7000_params_st;
pub type cudaLaunchKernel_v7000_params = cudaLaunchKernel_v7000_params_st;
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaLibraryEnumerateKernels_v12060_params = cudaLibraryEnumerateKernels_v12060_params_st;
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaLibraryGetGlobal_v12060_params = cudaLibraryGetGlobal_v12060_params_st;
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaLibraryGetKernelCount_v12060_params = cudaLibraryGetKernelCount_v12060_params_st;
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaLibraryGetKernel_v12060_params = cudaLibraryGetKernel_v12060_params_st;
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaLibraryGetManaged_v12060_params = cudaLibraryGetManaged_v12060_params_st;
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaLibraryGetUnifiedFunction_v12060_params =
    cudaLibraryGetUnifiedFunction_v12060_params_st;
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaLibraryLoadData_v12060_params = cudaLibraryLoadData_v12060_params_st;
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaLibraryLoadFromFile_v12060_params = cudaLibraryLoadFromFile_v12060_params_st;
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaLibraryUnload_v12060_params = cudaLibraryUnload_v12060_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaLogsCurrent_v13000_params = cudaLogsCurrent_v13000_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaLogsDumpToFile_v13000_params = cudaLogsDumpToFile_v13000_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaLogsDumpToMemory_v13000_params = cudaLogsDumpToMemory_v13000_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaLogsRegisterCallback_v13000_params = cudaLogsRegisterCallback_v13000_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaLogsUnregisterCallback_v13000_params = cudaLogsUnregisterCallback_v13000_params_st;
pub type cudaMalloc3DArray_v3020_params = cudaMalloc3DArray_v3020_params_st;
pub type cudaMalloc3D_v3020_params = cudaMalloc3D_v3020_params_st;
pub type cudaMallocArray_v3020_params = cudaMallocArray_v3020_params_st;
pub type cudaMallocAsync_ptsz_v11020_params = cudaMallocAsync_ptsz_v11020_params_st;
pub type cudaMallocAsync_v11020_params = cudaMallocAsync_v11020_params_st;
pub type cudaMallocFromPoolAsync_ptsz_v11020_params = cudaMallocFromPoolAsync_ptsz_v11020_params_st;
pub type cudaMallocFromPoolAsync_v11020_params = cudaMallocFromPoolAsync_v11020_params_st;
pub type cudaMallocHost_v3020_params = cudaMallocHost_v3020_params_st;
pub type cudaMallocManaged_v6000_params = cudaMallocManaged_v6000_params_st;
pub type cudaMallocMipmappedArray_v5000_params = cudaMallocMipmappedArray_v5000_params_st;
pub type cudaMallocPitch_v3020_params = cudaMallocPitch_v3020_params_st;
pub type cudaMalloc_v3020_params = cudaMalloc_v3020_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaMemAdvise_v12020_params = cudaMemAdvise_v12020_params_st;
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaMemAdvise_v2_v12020_params = cudaMemAdvise_v2_v12020_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaMemAdvise_v8000_params = cudaMemAdvise_v8000_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaMemDiscardAndPrefetchBatchAsync_ptsz_v13000_params =
    cudaMemDiscardAndPrefetchBatchAsync_ptsz_v13000_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaMemDiscardAndPrefetchBatchAsync_v13000_params =
    cudaMemDiscardAndPrefetchBatchAsync_v13000_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaMemDiscardBatchAsync_ptsz_v13000_params =
    cudaMemDiscardBatchAsync_ptsz_v13000_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaMemDiscardBatchAsync_v13000_params = cudaMemDiscardBatchAsync_v13000_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaMemGetDefaultMemPool_v13000_params = cudaMemGetDefaultMemPool_v13000_params_st;
pub type cudaMemGetInfo_v3020_params = cudaMemGetInfo_v3020_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaMemGetMemPool_v13000_params = cudaMemGetMemPool_v13000_params_st;
pub type cudaMemPoolCreate_v11020_params = cudaMemPoolCreate_v11020_params_st;
pub type cudaMemPoolDestroy_v11020_params = cudaMemPoolDestroy_v11020_params_st;
pub type cudaMemPoolExportPointer_v11020_params = cudaMemPoolExportPointer_v11020_params_st;
pub type cudaMemPoolExportToShareableHandle_v11020_params =
    cudaMemPoolExportToShareableHandle_v11020_params_st;
pub type cudaMemPoolGetAccess_v11020_params = cudaMemPoolGetAccess_v11020_params_st;
pub type cudaMemPoolGetAttribute_v11020_params = cudaMemPoolGetAttribute_v11020_params_st;
pub type cudaMemPoolImportFromShareableHandle_v11020_params =
    cudaMemPoolImportFromShareableHandle_v11020_params_st;
pub type cudaMemPoolImportPointer_v11020_params = cudaMemPoolImportPointer_v11020_params_st;
pub type cudaMemPoolSetAccess_v11020_params = cudaMemPoolSetAccess_v11020_params_st;
pub type cudaMemPoolSetAttribute_v11020_params = cudaMemPoolSetAttribute_v11020_params_st;
pub type cudaMemPoolTrimTo_v11020_params = cudaMemPoolTrimTo_v11020_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaMemPrefetchAsync_ptsz_v12020_params = cudaMemPrefetchAsync_ptsz_v12020_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaMemPrefetchAsync_ptsz_v8000_params = cudaMemPrefetchAsync_ptsz_v8000_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaMemPrefetchAsync_v12020_params = cudaMemPrefetchAsync_v12020_params_st;
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaMemPrefetchAsync_v2_ptsz_v12020_params = cudaMemPrefetchAsync_v2_ptsz_v12020_params_st;
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaMemPrefetchAsync_v2_v12020_params = cudaMemPrefetchAsync_v2_v12020_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaMemPrefetchAsync_v8000_params = cudaMemPrefetchAsync_v8000_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaMemPrefetchBatchAsync_ptsz_v13000_params =
    cudaMemPrefetchBatchAsync_ptsz_v13000_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaMemPrefetchBatchAsync_v13000_params = cudaMemPrefetchBatchAsync_v13000_params_st;
pub type cudaMemRangeGetAttribute_v8000_params = cudaMemRangeGetAttribute_v8000_params_st;
pub type cudaMemRangeGetAttributes_v8000_params = cudaMemRangeGetAttributes_v8000_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaMemSetMemPool_v13000_params = cudaMemSetMemPool_v13000_params_st;
pub type cudaMemcpy2DArrayToArray_ptds_v7000_params = cudaMemcpy2DArrayToArray_ptds_v7000_params_st;
pub type cudaMemcpy2DArrayToArray_v3020_params = cudaMemcpy2DArrayToArray_v3020_params_st;
pub type cudaMemcpy2DAsync_ptsz_v7000_params = cudaMemcpy2DAsync_ptsz_v7000_params_st;
pub type cudaMemcpy2DAsync_v3020_params = cudaMemcpy2DAsync_v3020_params_st;
pub type cudaMemcpy2DFromArrayAsync_ptsz_v7000_params =
    cudaMemcpy2DFromArrayAsync_ptsz_v7000_params_st;
pub type cudaMemcpy2DFromArrayAsync_v3020_params = cudaMemcpy2DFromArrayAsync_v3020_params_st;
pub type cudaMemcpy2DFromArray_ptds_v7000_params = cudaMemcpy2DFromArray_ptds_v7000_params_st;
pub type cudaMemcpy2DFromArray_v3020_params = cudaMemcpy2DFromArray_v3020_params_st;
pub type cudaMemcpy2DToArrayAsync_ptsz_v7000_params = cudaMemcpy2DToArrayAsync_ptsz_v7000_params_st;
pub type cudaMemcpy2DToArrayAsync_v3020_params = cudaMemcpy2DToArrayAsync_v3020_params_st;
pub type cudaMemcpy2DToArray_ptds_v7000_params = cudaMemcpy2DToArray_ptds_v7000_params_st;
pub type cudaMemcpy2DToArray_v3020_params = cudaMemcpy2DToArray_v3020_params_st;
pub type cudaMemcpy2D_ptds_v7000_params = cudaMemcpy2D_ptds_v7000_params_st;
pub type cudaMemcpy2D_v3020_params = cudaMemcpy2D_v3020_params_st;
pub type cudaMemcpy3DAsync_ptsz_v7000_params = cudaMemcpy3DAsync_ptsz_v7000_params_st;
pub type cudaMemcpy3DAsync_v3020_params = cudaMemcpy3DAsync_v3020_params_st;
#[cfg(any(feature = "cuda-12080", feature = "cuda-12090"))]
pub type cudaMemcpy3DBatchAsync_ptsz_v12080_params = cudaMemcpy3DBatchAsync_ptsz_v12080_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaMemcpy3DBatchAsync_ptsz_v13000_params = cudaMemcpy3DBatchAsync_ptsz_v13000_params_st;
#[cfg(any(feature = "cuda-12080", feature = "cuda-12090"))]
pub type cudaMemcpy3DBatchAsync_v12080_params = cudaMemcpy3DBatchAsync_v12080_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaMemcpy3DBatchAsync_v13000_params = cudaMemcpy3DBatchAsync_v13000_params_st;
pub type cudaMemcpy3DPeerAsync_ptsz_v7000_params = cudaMemcpy3DPeerAsync_ptsz_v7000_params_st;
pub type cudaMemcpy3DPeerAsync_v4000_params = cudaMemcpy3DPeerAsync_v4000_params_st;
pub type cudaMemcpy3DPeer_ptds_v7000_params = cudaMemcpy3DPeer_ptds_v7000_params_st;
pub type cudaMemcpy3DPeer_v4000_params = cudaMemcpy3DPeer_v4000_params_st;
pub type cudaMemcpy3D_ptds_v7000_params = cudaMemcpy3D_ptds_v7000_params_st;
pub type cudaMemcpy3D_v3020_params = cudaMemcpy3D_v3020_params_st;
pub type cudaMemcpyArrayToArray_ptds_v7000_params = cudaMemcpyArrayToArray_ptds_v7000_params_st;
pub type cudaMemcpyArrayToArray_v3020_params = cudaMemcpyArrayToArray_v3020_params_st;
pub type cudaMemcpyAsync_ptsz_v7000_params = cudaMemcpyAsync_ptsz_v7000_params_st;
pub type cudaMemcpyAsync_v3020_params = cudaMemcpyAsync_v3020_params_st;
#[cfg(any(feature = "cuda-12080", feature = "cuda-12090"))]
pub type cudaMemcpyBatchAsync_ptsz_v12080_params = cudaMemcpyBatchAsync_ptsz_v12080_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaMemcpyBatchAsync_ptsz_v13000_params = cudaMemcpyBatchAsync_ptsz_v13000_params_st;
#[cfg(any(feature = "cuda-12080", feature = "cuda-12090"))]
pub type cudaMemcpyBatchAsync_v12080_params = cudaMemcpyBatchAsync_v12080_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaMemcpyBatchAsync_v13000_params = cudaMemcpyBatchAsync_v13000_params_st;
pub type cudaMemcpyFromArrayAsync_ptsz_v7000_params = cudaMemcpyFromArrayAsync_ptsz_v7000_params_st;
pub type cudaMemcpyFromArrayAsync_v3020_params = cudaMemcpyFromArrayAsync_v3020_params_st;
pub type cudaMemcpyFromArray_ptds_v7000_params = cudaMemcpyFromArray_ptds_v7000_params_st;
pub type cudaMemcpyFromArray_v3020_params = cudaMemcpyFromArray_v3020_params_st;
pub type cudaMemcpyFromSymbolAsync_ptsz_v7000_params =
    cudaMemcpyFromSymbolAsync_ptsz_v7000_params_st;
pub type cudaMemcpyFromSymbolAsync_v3020_params = cudaMemcpyFromSymbolAsync_v3020_params_st;
pub type cudaMemcpyFromSymbol_ptds_v7000_params = cudaMemcpyFromSymbol_ptds_v7000_params_st;
pub type cudaMemcpyFromSymbol_v3020_params = cudaMemcpyFromSymbol_v3020_params_st;
pub type cudaMemcpyPeerAsync_v4000_params = cudaMemcpyPeerAsync_v4000_params_st;
pub type cudaMemcpyPeer_v4000_params = cudaMemcpyPeer_v4000_params_st;
pub type cudaMemcpyToArrayAsync_ptsz_v7000_params = cudaMemcpyToArrayAsync_ptsz_v7000_params_st;
pub type cudaMemcpyToArrayAsync_v3020_params = cudaMemcpyToArrayAsync_v3020_params_st;
pub type cudaMemcpyToArray_ptds_v7000_params = cudaMemcpyToArray_ptds_v7000_params_st;
pub type cudaMemcpyToArray_v3020_params = cudaMemcpyToArray_v3020_params_st;
pub type cudaMemcpyToSymbolAsync_ptsz_v7000_params = cudaMemcpyToSymbolAsync_ptsz_v7000_params_st;
pub type cudaMemcpyToSymbolAsync_v3020_params = cudaMemcpyToSymbolAsync_v3020_params_st;
pub type cudaMemcpyToSymbol_ptds_v7000_params = cudaMemcpyToSymbol_ptds_v7000_params_st;
pub type cudaMemcpyToSymbol_v3020_params = cudaMemcpyToSymbol_v3020_params_st;
pub type cudaMemcpy_ptds_v7000_params = cudaMemcpy_ptds_v7000_params_st;
pub type cudaMemcpy_v3020_params = cudaMemcpy_v3020_params_st;
pub type cudaMemset2DAsync_ptsz_v7000_params = cudaMemset2DAsync_ptsz_v7000_params_st;
pub type cudaMemset2DAsync_v3020_params = cudaMemset2DAsync_v3020_params_st;
pub type cudaMemset2D_ptds_v7000_params = cudaMemset2D_ptds_v7000_params_st;
pub type cudaMemset2D_v3020_params = cudaMemset2D_v3020_params_st;
pub type cudaMemset3DAsync_ptsz_v7000_params = cudaMemset3DAsync_ptsz_v7000_params_st;
pub type cudaMemset3DAsync_v3020_params = cudaMemset3DAsync_v3020_params_st;
pub type cudaMemset3D_ptds_v7000_params = cudaMemset3D_ptds_v7000_params_st;
pub type cudaMemset3D_v3020_params = cudaMemset3D_v3020_params_st;
pub type cudaMemsetAsync_ptsz_v7000_params = cudaMemsetAsync_ptsz_v7000_params_st;
pub type cudaMemsetAsync_v3020_params = cudaMemsetAsync_v3020_params_st;
pub type cudaMemset_ptds_v7000_params = cudaMemset_ptds_v7000_params_st;
pub type cudaMemset_v3020_params = cudaMemset_v3020_params_st;
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaMipmappedArrayGetMemoryRequirements_v11060_params =
    cudaMipmappedArrayGetMemoryRequirements_v11060_params_st;
pub type cudaMipmappedArrayGetSparseProperties_v11010_params =
    cudaMipmappedArrayGetSparseProperties_v11010_params_st;
pub type cudaOccupancyAvailableDynamicSMemPerBlock_v10200_params =
    cudaOccupancyAvailableDynamicSMemPerBlock_v10200_params_st;
pub type cudaOccupancyMaxActiveBlocksPerMultiprocessorWithFlags_v7000_params =
    cudaOccupancyMaxActiveBlocksPerMultiprocessorWithFlags_v7000_params_st;
pub type cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6050_params =
    cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6050_params_st;
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaOccupancyMaxActiveClusters_v11070_params =
    cudaOccupancyMaxActiveClusters_v11070_params_st;
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaOccupancyMaxPotentialClusterSize_v11070_params =
    cudaOccupancyMaxPotentialClusterSize_v11070_params_st;
pub type cudaPointerGetAttributes_v4000_params = cudaPointerGetAttributes_v4000_params_st;
pub type cudaRuntimeGetVersion_v3020_params = cudaRuntimeGetVersion_v3020_params_st;
pub type cudaSetDeviceFlags_v3020_params = cudaSetDeviceFlags_v3020_params_st;
pub type cudaSetDevice_v3020_params = cudaSetDevice_v3020_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11070",
    feature = "cuda-12000",
    feature = "cuda-12090"
))]
pub type cudaSetDoubleForDevice_v3020_params = cudaSetDoubleForDevice_v3020_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11070",
    feature = "cuda-12000",
    feature = "cuda-12090"
))]
pub type cudaSetDoubleForHost_v3020_params = cudaSetDoubleForHost_v3020_params_st;
pub type cudaSetValidDevices_v3020_params = cudaSetValidDevices_v3020_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaSignalExternalSemaphoresAsync_ptsz_v11020_params =
    cudaSignalExternalSemaphoresAsync_ptsz_v11020_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaSignalExternalSemaphoresAsync_v11020_params =
    cudaSignalExternalSemaphoresAsync_v11020_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaSignalExternalSemaphoresAsync_v2_ptsz_v11020_params =
    cudaSignalExternalSemaphoresAsync_v2_ptsz_v11020_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaSignalExternalSemaphoresAsync_v2_v11020_params =
    cudaSignalExternalSemaphoresAsync_v2_v11020_params_st;
pub type cudaStreamAddCallback_ptsz_v7000_params = cudaStreamAddCallback_ptsz_v7000_params_st;
pub type cudaStreamAddCallback_v5000_params = cudaStreamAddCallback_v5000_params_st;
pub type cudaStreamAttachMemAsync_ptsz_v7000_params = cudaStreamAttachMemAsync_ptsz_v7000_params_st;
pub type cudaStreamAttachMemAsync_v6000_params = cudaStreamAttachMemAsync_v6000_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaStreamBeginCaptureToGraph_ptsz_v12030_params =
    cudaStreamBeginCaptureToGraph_ptsz_v12030_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaStreamBeginCaptureToGraph_v12030_params =
    cudaStreamBeginCaptureToGraph_v12030_params_st;
pub type cudaStreamBeginCapture_ptsz_v10000_params = cudaStreamBeginCapture_ptsz_v10000_params_st;
pub type cudaStreamBeginCapture_v10000_params = cudaStreamBeginCapture_v10000_params_st;
pub type cudaStreamCopyAttributes_ptsz_v11000_params =
    cudaStreamCopyAttributes_ptsz_v11000_params_st;
pub type cudaStreamCopyAttributes_v11000_params = cudaStreamCopyAttributes_v11000_params_st;
pub type cudaStreamCreateWithFlags_v5000_params = cudaStreamCreateWithFlags_v5000_params_st;
pub type cudaStreamCreateWithPriority_v5050_params = cudaStreamCreateWithPriority_v5050_params_st;
pub type cudaStreamCreate_v3020_params = cudaStreamCreate_v3020_params_st;
pub type cudaStreamDestroy_v5050_params = cudaStreamDestroy_v5050_params_st;
pub type cudaStreamEndCapture_ptsz_v10000_params = cudaStreamEndCapture_ptsz_v10000_params_st;
pub type cudaStreamEndCapture_v10000_params = cudaStreamEndCapture_v10000_params_st;
pub type cudaStreamGetAttribute_ptsz_v11000_params = cudaStreamGetAttribute_ptsz_v11000_params_st;
pub type cudaStreamGetAttribute_v11000_params = cudaStreamGetAttribute_v11000_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaStreamGetCaptureInfo_ptsz_v10010_params =
    cudaStreamGetCaptureInfo_ptsz_v10010_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaStreamGetCaptureInfo_ptsz_v12030_params =
    cudaStreamGetCaptureInfo_ptsz_v12030_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaStreamGetCaptureInfo_v10010_params = cudaStreamGetCaptureInfo_v10010_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaStreamGetCaptureInfo_v12030_params = cudaStreamGetCaptureInfo_v12030_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaStreamGetCaptureInfo_v2_ptsz_v11030_params =
    cudaStreamGetCaptureInfo_v2_ptsz_v11030_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaStreamGetCaptureInfo_v2_v11030_params = cudaStreamGetCaptureInfo_v2_v11030_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaStreamGetCaptureInfo_v3_ptsz_v12030_params =
    cudaStreamGetCaptureInfo_v3_ptsz_v12030_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaStreamGetCaptureInfo_v3_v12030_params = cudaStreamGetCaptureInfo_v3_v12030_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cudaStreamGetDevResource_ptsz_v13010_params =
    cudaStreamGetDevResource_ptsz_v13010_params_st;
#[cfg(any(feature = "cuda-13010"))]
pub type cudaStreamGetDevResource_v13010_params = cudaStreamGetDevResource_v13010_params_st;
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaStreamGetDevice_ptsz_v12080_params = cudaStreamGetDevice_ptsz_v12080_params_st;
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaStreamGetDevice_v12080_params = cudaStreamGetDevice_v12080_params_st;
pub type cudaStreamGetFlags_ptsz_v7000_params = cudaStreamGetFlags_ptsz_v7000_params_st;
pub type cudaStreamGetFlags_v5050_params = cudaStreamGetFlags_v5050_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaStreamGetId_ptsz_v12000_params = cudaStreamGetId_ptsz_v12000_params_st;
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
pub type cudaStreamGetId_v12000_params = cudaStreamGetId_v12000_params_st;
pub type cudaStreamGetPriority_ptsz_v7000_params = cudaStreamGetPriority_ptsz_v7000_params_st;
pub type cudaStreamGetPriority_v5050_params = cudaStreamGetPriority_v5050_params_st;
pub type cudaStreamIsCapturing_ptsz_v10000_params = cudaStreamIsCapturing_ptsz_v10000_params_st;
pub type cudaStreamIsCapturing_v10000_params = cudaStreamIsCapturing_v10000_params_st;
pub type cudaStreamQuery_ptsz_v7000_params = cudaStreamQuery_ptsz_v7000_params_st;
pub type cudaStreamQuery_v3020_params = cudaStreamQuery_v3020_params_st;
pub type cudaStreamSetAttribute_ptsz_v11000_params = cudaStreamSetAttribute_ptsz_v11000_params_st;
pub type cudaStreamSetAttribute_v11000_params = cudaStreamSetAttribute_v11000_params_st;
pub type cudaStreamSynchronize_ptsz_v7000_params = cudaStreamSynchronize_ptsz_v7000_params_st;
pub type cudaStreamSynchronize_v3020_params = cudaStreamSynchronize_v3020_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaStreamUpdateCaptureDependencies_ptsz_v11030_params =
    cudaStreamUpdateCaptureDependencies_ptsz_v11030_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaStreamUpdateCaptureDependencies_ptsz_v12030_params =
    cudaStreamUpdateCaptureDependencies_ptsz_v12030_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaStreamUpdateCaptureDependencies_v11030_params =
    cudaStreamUpdateCaptureDependencies_v11030_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaStreamUpdateCaptureDependencies_v12030_params =
    cudaStreamUpdateCaptureDependencies_v12030_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaStreamUpdateCaptureDependencies_v2_ptsz_v12030_params =
    cudaStreamUpdateCaptureDependencies_v2_ptsz_v12030_params_st;
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaStreamUpdateCaptureDependencies_v2_v12030_params =
    cudaStreamUpdateCaptureDependencies_v2_v12030_params_st;
pub type cudaStreamWaitEvent_ptsz_v7000_params = cudaStreamWaitEvent_ptsz_v7000_params_st;
pub type cudaStreamWaitEvent_v3020_params = cudaStreamWaitEvent_v3020_params_st;
pub type cudaThreadExchangeStreamCaptureMode_v10010_params =
    cudaThreadExchangeStreamCaptureMode_v10010_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11070",
    feature = "cuda-12000",
    feature = "cuda-12090"
))]
pub type cudaThreadGetCacheConfig_v3020_params = cudaThreadGetCacheConfig_v3020_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11070",
    feature = "cuda-12000",
    feature = "cuda-12090"
))]
pub type cudaThreadGetLimit_v3020_params = cudaThreadGetLimit_v3020_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11070",
    feature = "cuda-12000",
    feature = "cuda-12090"
))]
pub type cudaThreadSetCacheConfig_v3020_params = cudaThreadSetCacheConfig_v3020_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11070",
    feature = "cuda-12000",
    feature = "cuda-12090"
))]
pub type cudaThreadSetLimit_v3020_params = cudaThreadSetLimit_v3020_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
pub type cudaUnbindTexture_v3020_params = cudaUnbindTexture_v3020_params_st;
pub type cudaUserObjectCreate_v11030_params = cudaUserObjectCreate_v11030_params_st;
pub type cudaUserObjectRelease_v11030_params = cudaUserObjectRelease_v11030_params_st;
pub type cudaUserObjectRetain_v11030_params = cudaUserObjectRetain_v11030_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaWaitExternalSemaphoresAsync_ptsz_v11020_params =
    cudaWaitExternalSemaphoresAsync_ptsz_v11020_params_st;
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
pub type cudaWaitExternalSemaphoresAsync_v11020_params =
    cudaWaitExternalSemaphoresAsync_v11020_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaWaitExternalSemaphoresAsync_v2_ptsz_v11020_params =
    cudaWaitExternalSemaphoresAsync_v2_ptsz_v11020_params_st;
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
pub type cudaWaitExternalSemaphoresAsync_v2_v11020_params =
    cudaWaitExternalSemaphoresAsync_v2_v11020_params_st;
#[cfg(any(feature = "cuda-11040"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUptiResult {
    CUPTI_SUCCESS = 0,
    CUPTI_ERROR_INVALID_PARAMETER = 1,
    CUPTI_ERROR_INVALID_DEVICE = 2,
    CUPTI_ERROR_INVALID_CONTEXT = 3,
    CUPTI_ERROR_INVALID_EVENT_DOMAIN_ID = 4,
    CUPTI_ERROR_INVALID_EVENT_ID = 5,
    CUPTI_ERROR_INVALID_EVENT_NAME = 6,
    CUPTI_ERROR_INVALID_OPERATION = 7,
    CUPTI_ERROR_OUT_OF_MEMORY = 8,
    CUPTI_ERROR_HARDWARE = 9,
    CUPTI_ERROR_PARAMETER_SIZE_NOT_SUFFICIENT = 10,
    CUPTI_ERROR_API_NOT_IMPLEMENTED = 11,
    CUPTI_ERROR_MAX_LIMIT_REACHED = 12,
    CUPTI_ERROR_NOT_READY = 13,
    CUPTI_ERROR_NOT_COMPATIBLE = 14,
    CUPTI_ERROR_NOT_INITIALIZED = 15,
    CUPTI_ERROR_INVALID_METRIC_ID = 16,
    CUPTI_ERROR_INVALID_METRIC_NAME = 17,
    CUPTI_ERROR_QUEUE_EMPTY = 18,
    CUPTI_ERROR_INVALID_HANDLE = 19,
    CUPTI_ERROR_INVALID_STREAM = 20,
    CUPTI_ERROR_INVALID_KIND = 21,
    CUPTI_ERROR_INVALID_EVENT_VALUE = 22,
    CUPTI_ERROR_DISABLED = 23,
    CUPTI_ERROR_INVALID_MODULE = 24,
    CUPTI_ERROR_INVALID_METRIC_VALUE = 25,
    CUPTI_ERROR_HARDWARE_BUSY = 26,
    CUPTI_ERROR_NOT_SUPPORTED = 27,
    CUPTI_ERROR_UM_PROFILING_NOT_SUPPORTED = 28,
    CUPTI_ERROR_UM_PROFILING_NOT_SUPPORTED_ON_DEVICE = 29,
    CUPTI_ERROR_UM_PROFILING_NOT_SUPPORTED_ON_NON_P2P_DEVICES = 30,
    CUPTI_ERROR_UM_PROFILING_NOT_SUPPORTED_WITH_MPS = 31,
    CUPTI_ERROR_CDP_TRACING_NOT_SUPPORTED = 32,
    CUPTI_ERROR_VIRTUALIZED_DEVICE_NOT_SUPPORTED = 33,
    CUPTI_ERROR_CUDA_COMPILER_NOT_COMPATIBLE = 34,
    CUPTI_ERROR_INSUFFICIENT_PRIVILEGES = 35,
    CUPTI_ERROR_OLD_PROFILER_API_INITIALIZED = 36,
    CUPTI_ERROR_OPENACC_UNDEFINED_ROUTINE = 37,
    CUPTI_ERROR_LEGACY_PROFILER_NOT_SUPPORTED = 38,
    CUPTI_ERROR_MULTIPLE_SUBSCRIBERS_NOT_SUPPORTED = 39,
    CUPTI_ERROR_VIRTUALIZED_DEVICE_INSUFFICIENT_PRIVILEGES = 40,
    CUPTI_ERROR_CONFIDENTIAL_COMPUTING_NOT_SUPPORTED = 41,
    CUPTI_ERROR_UNKNOWN = 999,
    CUPTI_ERROR_FORCE_INT = 2147483647,
}
#[cfg(any(
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060"
))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUptiResult {
    CUPTI_SUCCESS = 0,
    CUPTI_ERROR_INVALID_PARAMETER = 1,
    CUPTI_ERROR_INVALID_DEVICE = 2,
    CUPTI_ERROR_INVALID_CONTEXT = 3,
    CUPTI_ERROR_INVALID_EVENT_DOMAIN_ID = 4,
    CUPTI_ERROR_INVALID_EVENT_ID = 5,
    CUPTI_ERROR_INVALID_EVENT_NAME = 6,
    CUPTI_ERROR_INVALID_OPERATION = 7,
    CUPTI_ERROR_OUT_OF_MEMORY = 8,
    CUPTI_ERROR_HARDWARE = 9,
    CUPTI_ERROR_PARAMETER_SIZE_NOT_SUFFICIENT = 10,
    CUPTI_ERROR_API_NOT_IMPLEMENTED = 11,
    CUPTI_ERROR_MAX_LIMIT_REACHED = 12,
    CUPTI_ERROR_NOT_READY = 13,
    CUPTI_ERROR_NOT_COMPATIBLE = 14,
    CUPTI_ERROR_NOT_INITIALIZED = 15,
    CUPTI_ERROR_INVALID_METRIC_ID = 16,
    CUPTI_ERROR_INVALID_METRIC_NAME = 17,
    CUPTI_ERROR_QUEUE_EMPTY = 18,
    CUPTI_ERROR_INVALID_HANDLE = 19,
    CUPTI_ERROR_INVALID_STREAM = 20,
    CUPTI_ERROR_INVALID_KIND = 21,
    CUPTI_ERROR_INVALID_EVENT_VALUE = 22,
    CUPTI_ERROR_DISABLED = 23,
    CUPTI_ERROR_INVALID_MODULE = 24,
    CUPTI_ERROR_INVALID_METRIC_VALUE = 25,
    CUPTI_ERROR_HARDWARE_BUSY = 26,
    CUPTI_ERROR_NOT_SUPPORTED = 27,
    CUPTI_ERROR_UM_PROFILING_NOT_SUPPORTED = 28,
    CUPTI_ERROR_UM_PROFILING_NOT_SUPPORTED_ON_DEVICE = 29,
    CUPTI_ERROR_UM_PROFILING_NOT_SUPPORTED_ON_NON_P2P_DEVICES = 30,
    CUPTI_ERROR_UM_PROFILING_NOT_SUPPORTED_WITH_MPS = 31,
    CUPTI_ERROR_CDP_TRACING_NOT_SUPPORTED = 32,
    CUPTI_ERROR_VIRTUALIZED_DEVICE_NOT_SUPPORTED = 33,
    CUPTI_ERROR_CUDA_COMPILER_NOT_COMPATIBLE = 34,
    CUPTI_ERROR_INSUFFICIENT_PRIVILEGES = 35,
    CUPTI_ERROR_OLD_PROFILER_API_INITIALIZED = 36,
    CUPTI_ERROR_OPENACC_UNDEFINED_ROUTINE = 37,
    CUPTI_ERROR_LEGACY_PROFILER_NOT_SUPPORTED = 38,
    CUPTI_ERROR_MULTIPLE_SUBSCRIBERS_NOT_SUPPORTED = 39,
    CUPTI_ERROR_VIRTUALIZED_DEVICE_INSUFFICIENT_PRIVILEGES = 40,
    CUPTI_ERROR_CONFIDENTIAL_COMPUTING_NOT_SUPPORTED = 41,
    CUPTI_ERROR_CMP_DEVICE_NOT_SUPPORTED = 42,
    CUPTI_ERROR_UNKNOWN = 999,
    CUPTI_ERROR_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-12080", feature = "cuda-12090"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUptiResult {
    CUPTI_SUCCESS = 0,
    CUPTI_ERROR_INVALID_PARAMETER = 1,
    CUPTI_ERROR_INVALID_DEVICE = 2,
    CUPTI_ERROR_INVALID_CONTEXT = 3,
    CUPTI_ERROR_INVALID_EVENT_DOMAIN_ID = 4,
    CUPTI_ERROR_INVALID_EVENT_ID = 5,
    CUPTI_ERROR_INVALID_EVENT_NAME = 6,
    CUPTI_ERROR_INVALID_OPERATION = 7,
    CUPTI_ERROR_OUT_OF_MEMORY = 8,
    CUPTI_ERROR_HARDWARE = 9,
    CUPTI_ERROR_PARAMETER_SIZE_NOT_SUFFICIENT = 10,
    CUPTI_ERROR_API_NOT_IMPLEMENTED = 11,
    CUPTI_ERROR_MAX_LIMIT_REACHED = 12,
    CUPTI_ERROR_NOT_READY = 13,
    CUPTI_ERROR_NOT_COMPATIBLE = 14,
    CUPTI_ERROR_NOT_INITIALIZED = 15,
    CUPTI_ERROR_INVALID_METRIC_ID = 16,
    CUPTI_ERROR_INVALID_METRIC_NAME = 17,
    CUPTI_ERROR_QUEUE_EMPTY = 18,
    CUPTI_ERROR_INVALID_HANDLE = 19,
    CUPTI_ERROR_INVALID_STREAM = 20,
    CUPTI_ERROR_INVALID_KIND = 21,
    CUPTI_ERROR_INVALID_EVENT_VALUE = 22,
    CUPTI_ERROR_DISABLED = 23,
    CUPTI_ERROR_INVALID_MODULE = 24,
    CUPTI_ERROR_INVALID_METRIC_VALUE = 25,
    CUPTI_ERROR_HARDWARE_BUSY = 26,
    CUPTI_ERROR_NOT_SUPPORTED = 27,
    CUPTI_ERROR_UM_PROFILING_NOT_SUPPORTED = 28,
    CUPTI_ERROR_UM_PROFILING_NOT_SUPPORTED_ON_DEVICE = 29,
    CUPTI_ERROR_UM_PROFILING_NOT_SUPPORTED_ON_NON_P2P_DEVICES = 30,
    CUPTI_ERROR_UM_PROFILING_NOT_SUPPORTED_WITH_MPS = 31,
    CUPTI_ERROR_CDP_TRACING_NOT_SUPPORTED = 32,
    CUPTI_ERROR_VIRTUALIZED_DEVICE_NOT_SUPPORTED = 33,
    CUPTI_ERROR_CUDA_COMPILER_NOT_COMPATIBLE = 34,
    CUPTI_ERROR_INSUFFICIENT_PRIVILEGES = 35,
    CUPTI_ERROR_OLD_PROFILER_API_INITIALIZED = 36,
    CUPTI_ERROR_OPENACC_UNDEFINED_ROUTINE = 37,
    CUPTI_ERROR_LEGACY_PROFILER_NOT_SUPPORTED = 38,
    CUPTI_ERROR_MULTIPLE_SUBSCRIBERS_NOT_SUPPORTED = 39,
    CUPTI_ERROR_VIRTUALIZED_DEVICE_INSUFFICIENT_PRIVILEGES = 40,
    CUPTI_ERROR_CONFIDENTIAL_COMPUTING_NOT_SUPPORTED = 41,
    CUPTI_ERROR_CMP_DEVICE_NOT_SUPPORTED = 42,
    CUPTI_ERROR_MIG_DEVICE_NOT_SUPPORTED = 43,
    CUPTI_ERROR_SLI_DEVICE_NOT_SUPPORTED = 44,
    CUPTI_ERROR_WSL_DEVICE_NOT_SUPPORTED = 45,
    CUPTI_ERROR_UNKNOWN = 999,
    CUPTI_ERROR_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-13000"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUptiResult {
    CUPTI_SUCCESS = 0,
    CUPTI_ERROR_INVALID_PARAMETER = 1,
    CUPTI_ERROR_INVALID_DEVICE = 2,
    CUPTI_ERROR_INVALID_CONTEXT = 3,
    CUPTI_ERROR_INVALID_EVENT_DOMAIN_ID = 4,
    CUPTI_ERROR_INVALID_EVENT_ID = 5,
    CUPTI_ERROR_INVALID_EVENT_NAME = 6,
    CUPTI_ERROR_INVALID_OPERATION = 7,
    CUPTI_ERROR_OUT_OF_MEMORY = 8,
    CUPTI_ERROR_HARDWARE = 9,
    CUPTI_ERROR_PARAMETER_SIZE_NOT_SUFFICIENT = 10,
    CUPTI_ERROR_API_NOT_IMPLEMENTED = 11,
    CUPTI_ERROR_MAX_LIMIT_REACHED = 12,
    CUPTI_ERROR_NOT_READY = 13,
    CUPTI_ERROR_NOT_COMPATIBLE = 14,
    CUPTI_ERROR_NOT_INITIALIZED = 15,
    CUPTI_ERROR_INVALID_METRIC_ID = 16,
    CUPTI_ERROR_INVALID_METRIC_NAME = 17,
    CUPTI_ERROR_QUEUE_EMPTY = 18,
    CUPTI_ERROR_INVALID_HANDLE = 19,
    CUPTI_ERROR_INVALID_STREAM = 20,
    CUPTI_ERROR_INVALID_KIND = 21,
    CUPTI_ERROR_INVALID_EVENT_VALUE = 22,
    CUPTI_ERROR_DISABLED = 23,
    CUPTI_ERROR_INVALID_MODULE = 24,
    CUPTI_ERROR_INVALID_METRIC_VALUE = 25,
    CUPTI_ERROR_HARDWARE_BUSY = 26,
    CUPTI_ERROR_NOT_SUPPORTED = 27,
    CUPTI_ERROR_UM_PROFILING_NOT_SUPPORTED = 28,
    CUPTI_ERROR_UM_PROFILING_NOT_SUPPORTED_ON_DEVICE = 29,
    CUPTI_ERROR_UM_PROFILING_NOT_SUPPORTED_ON_NON_P2P_DEVICES = 30,
    CUPTI_ERROR_UM_PROFILING_NOT_SUPPORTED_WITH_MPS = 31,
    CUPTI_ERROR_CDP_TRACING_NOT_SUPPORTED = 32,
    CUPTI_ERROR_VIRTUALIZED_DEVICE_NOT_SUPPORTED = 33,
    CUPTI_ERROR_CUDA_COMPILER_NOT_COMPATIBLE = 34,
    CUPTI_ERROR_INSUFFICIENT_PRIVILEGES = 35,
    CUPTI_ERROR_OLD_PROFILER_API_INITIALIZED = 36,
    CUPTI_ERROR_OPENACC_UNDEFINED_ROUTINE = 37,
    CUPTI_ERROR_LEGACY_PROFILER_NOT_SUPPORTED = 38,
    CUPTI_ERROR_MULTIPLE_SUBSCRIBERS_NOT_SUPPORTED = 39,
    CUPTI_ERROR_VIRTUALIZED_DEVICE_INSUFFICIENT_PRIVILEGES = 40,
    CUPTI_ERROR_CONFIDENTIAL_COMPUTING_NOT_SUPPORTED = 41,
    CUPTI_ERROR_CMP_DEVICE_NOT_SUPPORTED = 42,
    CUPTI_ERROR_MIG_DEVICE_NOT_SUPPORTED = 43,
    CUPTI_ERROR_SLI_DEVICE_NOT_SUPPORTED = 44,
    CUPTI_ERROR_WSL_DEVICE_NOT_SUPPORTED = 45,
    CUPTI_ERROR_INVALID_CHIP_NAME = 46,
    CUPTI_ERROR_UNKNOWN = 999,
    CUPTI_ERROR_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUptiResult {
    CUPTI_SUCCESS = 0,
    CUPTI_ERROR_INVALID_PARAMETER = 1,
    CUPTI_ERROR_INVALID_DEVICE = 2,
    CUPTI_ERROR_INVALID_CONTEXT = 3,
    CUPTI_ERROR_INVALID_EVENT_DOMAIN_ID = 4,
    CUPTI_ERROR_INVALID_EVENT_ID = 5,
    CUPTI_ERROR_INVALID_EVENT_NAME = 6,
    CUPTI_ERROR_INVALID_OPERATION = 7,
    CUPTI_ERROR_OUT_OF_MEMORY = 8,
    CUPTI_ERROR_HARDWARE = 9,
    CUPTI_ERROR_PARAMETER_SIZE_NOT_SUFFICIENT = 10,
    CUPTI_ERROR_API_NOT_IMPLEMENTED = 11,
    CUPTI_ERROR_MAX_LIMIT_REACHED = 12,
    CUPTI_ERROR_NOT_READY = 13,
    CUPTI_ERROR_NOT_COMPATIBLE = 14,
    CUPTI_ERROR_NOT_INITIALIZED = 15,
    CUPTI_ERROR_INVALID_METRIC_ID = 16,
    CUPTI_ERROR_INVALID_METRIC_NAME = 17,
    CUPTI_ERROR_QUEUE_EMPTY = 18,
    CUPTI_ERROR_INVALID_HANDLE = 19,
    CUPTI_ERROR_INVALID_STREAM = 20,
    CUPTI_ERROR_INVALID_KIND = 21,
    CUPTI_ERROR_INVALID_EVENT_VALUE = 22,
    CUPTI_ERROR_DISABLED = 23,
    CUPTI_ERROR_INVALID_MODULE = 24,
    CUPTI_ERROR_INVALID_METRIC_VALUE = 25,
    CUPTI_ERROR_HARDWARE_BUSY = 26,
    CUPTI_ERROR_NOT_SUPPORTED = 27,
    CUPTI_ERROR_UM_PROFILING_NOT_SUPPORTED = 28,
    CUPTI_ERROR_UM_PROFILING_NOT_SUPPORTED_ON_DEVICE = 29,
    CUPTI_ERROR_UM_PROFILING_NOT_SUPPORTED_ON_NON_P2P_DEVICES = 30,
    CUPTI_ERROR_UM_PROFILING_NOT_SUPPORTED_WITH_MPS = 31,
    CUPTI_ERROR_CDP_TRACING_NOT_SUPPORTED = 32,
    CUPTI_ERROR_VIRTUALIZED_DEVICE_NOT_SUPPORTED = 33,
    CUPTI_ERROR_CUDA_COMPILER_NOT_COMPATIBLE = 34,
    CUPTI_ERROR_INSUFFICIENT_PRIVILEGES = 35,
    CUPTI_ERROR_OLD_PROFILER_API_INITIALIZED = 36,
    CUPTI_ERROR_OPENACC_UNDEFINED_ROUTINE = 37,
    CUPTI_ERROR_LEGACY_PROFILER_NOT_SUPPORTED = 38,
    CUPTI_ERROR_MULTIPLE_SUBSCRIBERS_NOT_SUPPORTED = 39,
    CUPTI_ERROR_VIRTUALIZED_DEVICE_INSUFFICIENT_PRIVILEGES = 40,
    CUPTI_ERROR_CONFIDENTIAL_COMPUTING_NOT_SUPPORTED = 41,
    CUPTI_ERROR_CMP_DEVICE_NOT_SUPPORTED = 42,
    CUPTI_ERROR_MIG_DEVICE_NOT_SUPPORTED = 43,
    CUPTI_ERROR_SLI_DEVICE_NOT_SUPPORTED = 44,
    CUPTI_ERROR_WSL_DEVICE_NOT_SUPPORTED = 45,
    CUPTI_ERROR_INVALID_CHIP_NAME = 46,
    CUPTI_ERROR_HES_TRACE_NOT_SUPPORTED_ON_MPS = 47,
    CUPTI_ERROR_UNKNOWN = 999,
    CUPTI_ERROR_FORCE_INT = 2147483647,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020"
))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityAttribute {
    CUPTI_ACTIVITY_ATTR_DEVICE_BUFFER_SIZE = 0,
    CUPTI_ACTIVITY_ATTR_DEVICE_BUFFER_SIZE_CDP = 1,
    CUPTI_ACTIVITY_ATTR_DEVICE_BUFFER_POOL_LIMIT = 2,
    CUPTI_ACTIVITY_ATTR_PROFILING_SEMAPHORE_POOL_SIZE = 3,
    CUPTI_ACTIVITY_ATTR_PROFILING_SEMAPHORE_POOL_LIMIT = 4,
    CUPTI_ACTIVITY_ATTR_ZEROED_OUT_ACTIVITY_BUFFER = 5,
    CUPTI_ACTIVITY_ATTR_DEVICE_BUFFER_PRE_ALLOCATE_VALUE = 6,
    CUPTI_ACTIVITY_ATTR_PROFILING_SEMAPHORE_PRE_ALLOCATE_VALUE = 7,
    CUPTI_ACTIVITY_ATTR_MEM_ALLOCATION_TYPE_HOST_PINNED = 8,
    CUPTI_ACTIVITY_ATTR_DEVICE_BUFFER_FORCE_INT = 2147483647,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080"
))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityAttribute {
    CUPTI_ACTIVITY_ATTR_DEVICE_BUFFER_SIZE = 0,
    CUPTI_ACTIVITY_ATTR_DEVICE_BUFFER_SIZE_CDP = 1,
    CUPTI_ACTIVITY_ATTR_DEVICE_BUFFER_POOL_LIMIT = 2,
    CUPTI_ACTIVITY_ATTR_PROFILING_SEMAPHORE_POOL_SIZE = 3,
    CUPTI_ACTIVITY_ATTR_PROFILING_SEMAPHORE_POOL_LIMIT = 4,
    CUPTI_ACTIVITY_ATTR_ZEROED_OUT_ACTIVITY_BUFFER = 5,
    CUPTI_ACTIVITY_ATTR_DEVICE_BUFFER_PRE_ALLOCATE_VALUE = 6,
    CUPTI_ACTIVITY_ATTR_PROFILING_SEMAPHORE_PRE_ALLOCATE_VALUE = 7,
    CUPTI_ACTIVITY_ATTR_MEM_ALLOCATION_TYPE_HOST_PINNED = 8,
    CUPTI_ACTIVITY_ATTR_PER_THREAD_ACTIVITY_BUFFER = 9,
    CUPTI_ACTIVITY_ATTR_DEVICE_BUFFER_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityAttribute {
    CUPTI_ACTIVITY_ATTR_DEVICE_BUFFER_SIZE = 0,
    CUPTI_ACTIVITY_ATTR_DEVICE_BUFFER_SIZE_CDP = 1,
    CUPTI_ACTIVITY_ATTR_DEVICE_BUFFER_POOL_LIMIT = 2,
    CUPTI_ACTIVITY_ATTR_PROFILING_SEMAPHORE_POOL_SIZE = 3,
    CUPTI_ACTIVITY_ATTR_PROFILING_SEMAPHORE_POOL_LIMIT = 4,
    CUPTI_ACTIVITY_ATTR_ZEROED_OUT_ACTIVITY_BUFFER = 5,
    CUPTI_ACTIVITY_ATTR_DEVICE_BUFFER_PRE_ALLOCATE_VALUE = 6,
    CUPTI_ACTIVITY_ATTR_PROFILING_SEMAPHORE_PRE_ALLOCATE_VALUE = 7,
    CUPTI_ACTIVITY_ATTR_MEM_ALLOCATION_TYPE_HOST_PINNED = 8,
    CUPTI_ACTIVITY_ATTR_PER_THREAD_ACTIVITY_BUFFER = 9,
    CUPTI_ACTIVITY_ATTR_DEVICE_BUFFER_SIZE_DEVICE_GRAPHS = 10,
    CUPTI_ACTIVITY_ATTR_DEVICE_BUFFER_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityComputeApiKind {
    CUPTI_ACTIVITY_COMPUTE_API_UNKNOWN = 0,
    CUPTI_ACTIVITY_COMPUTE_API_CUDA = 1,
    CUPTI_ACTIVITY_COMPUTE_API_CUDA_MPS = 2,
    CUPTI_ACTIVITY_COMPUTE_API_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityEnvironmentKind {
    CUPTI_ACTIVITY_ENVIRONMENT_UNKNOWN = 0,
    CUPTI_ACTIVITY_ENVIRONMENT_SPEED = 1,
    CUPTI_ACTIVITY_ENVIRONMENT_TEMPERATURE = 2,
    CUPTI_ACTIVITY_ENVIRONMENT_POWER = 3,
    CUPTI_ACTIVITY_ENVIRONMENT_COOLING = 4,
    CUPTI_ACTIVITY_ENVIRONMENT_COUNT = 5,
    CUPTI_ACTIVITY_ENVIRONMENT_KIND_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityFlag {
    CUPTI_ACTIVITY_FLAG_NONE = 0,
    CUPTI_ACTIVITY_FLAG_DEVICE_CONCURRENT_KERNELS = 1,
    CUPTI_ACTIVITY_FLAG_MARKER_START = 2,
    CUPTI_ACTIVITY_FLAG_MARKER_END = 4,
    CUPTI_ACTIVITY_FLAG_MARKER_SYNC_ACQUIRE = 8,
    CUPTI_ACTIVITY_FLAG_MARKER_SYNC_ACQUIRE_SUCCESS = 16,
    CUPTI_ACTIVITY_FLAG_MARKER_SYNC_ACQUIRE_FAILED = 32,
    CUPTI_ACTIVITY_FLAG_MARKER_SYNC_RELEASE = 64,
    CUPTI_ACTIVITY_FLAG_GLOBAL_ACCESS_KIND_SIZE_MASK = 255,
    CUPTI_ACTIVITY_FLAG_GLOBAL_ACCESS_KIND_LOAD = 256,
    CUPTI_ACTIVITY_FLAG_GLOBAL_ACCESS_KIND_CACHED = 512,
    CUPTI_ACTIVITY_FLAG_INSTRUCTION_CLASS_MASK = 510,
    CUPTI_ACTIVITY_FLAG_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityInstructionClass {
    CUPTI_ACTIVITY_INSTRUCTION_CLASS_UNKNOWN = 0,
    CUPTI_ACTIVITY_INSTRUCTION_CLASS_FP_32 = 1,
    CUPTI_ACTIVITY_INSTRUCTION_CLASS_FP_64 = 2,
    CUPTI_ACTIVITY_INSTRUCTION_CLASS_INTEGER = 3,
    CUPTI_ACTIVITY_INSTRUCTION_CLASS_BIT_CONVERSION = 4,
    CUPTI_ACTIVITY_INSTRUCTION_CLASS_CONTROL_FLOW = 5,
    CUPTI_ACTIVITY_INSTRUCTION_CLASS_GLOBAL = 6,
    CUPTI_ACTIVITY_INSTRUCTION_CLASS_SHARED = 7,
    CUPTI_ACTIVITY_INSTRUCTION_CLASS_LOCAL = 8,
    CUPTI_ACTIVITY_INSTRUCTION_CLASS_GENERIC = 9,
    CUPTI_ACTIVITY_INSTRUCTION_CLASS_SURFACE = 10,
    CUPTI_ACTIVITY_INSTRUCTION_CLASS_CONSTANT = 11,
    CUPTI_ACTIVITY_INSTRUCTION_CLASS_TEXTURE = 12,
    CUPTI_ACTIVITY_INSTRUCTION_CLASS_GLOBAL_ATOMIC = 13,
    CUPTI_ACTIVITY_INSTRUCTION_CLASS_SHARED_ATOMIC = 14,
    CUPTI_ACTIVITY_INSTRUCTION_CLASS_SURFACE_ATOMIC = 15,
    CUPTI_ACTIVITY_INSTRUCTION_CLASS_INTER_THREAD_COMMUNICATION = 16,
    CUPTI_ACTIVITY_INSTRUCTION_CLASS_BARRIER = 17,
    CUPTI_ACTIVITY_INSTRUCTION_CLASS_MISCELLANEOUS = 18,
    CUPTI_ACTIVITY_INSTRUCTION_CLASS_FP_16 = 19,
    CUPTI_ACTIVITY_INSTRUCTION_CLASS_UNIFORM = 20,
    CUPTI_ACTIVITY_INSTRUCTION_CLASS_KIND_FORCE_INT = 2147483647,
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityJitEntryType {
    CUPTI_ACTIVITY_JIT_ENTRY_INVALID = 0,
    CUPTI_ACTIVITY_JIT_ENTRY_PTX_TO_CUBIN = 1,
    CUPTI_ACTIVITY_JIT_ENTRY_NVVM_IR_TO_PTX = 2,
    CUPTI_ACTIVITY_JIT_ENTRY_TYPE_FORCE_INT = 2147483647,
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityJitOperationType {
    CUPTI_ACTIVITY_JIT_OPERATION_INVALID = 0,
    CUPTI_ACTIVITY_JIT_OPERATION_CACHE_LOAD = 1,
    CUPTI_ACTIVITY_JIT_OPERATION_CACHE_STORE = 2,
    CUPTI_ACTIVITY_JIT_OPERATION_COMPILE = 3,
    CUPTI_ACTIVITY_JIT_OPERATION_TYPE_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-11040", feature = "cuda-11050", feature = "cuda-11060"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityKind {
    CUPTI_ACTIVITY_KIND_INVALID = 0,
    CUPTI_ACTIVITY_KIND_MEMCPY = 1,
    CUPTI_ACTIVITY_KIND_MEMSET = 2,
    CUPTI_ACTIVITY_KIND_KERNEL = 3,
    CUPTI_ACTIVITY_KIND_DRIVER = 4,
    CUPTI_ACTIVITY_KIND_RUNTIME = 5,
    CUPTI_ACTIVITY_KIND_EVENT = 6,
    CUPTI_ACTIVITY_KIND_METRIC = 7,
    CUPTI_ACTIVITY_KIND_DEVICE = 8,
    CUPTI_ACTIVITY_KIND_CONTEXT = 9,
    CUPTI_ACTIVITY_KIND_CONCURRENT_KERNEL = 10,
    CUPTI_ACTIVITY_KIND_NAME = 11,
    CUPTI_ACTIVITY_KIND_MARKER = 12,
    CUPTI_ACTIVITY_KIND_MARKER_DATA = 13,
    CUPTI_ACTIVITY_KIND_SOURCE_LOCATOR = 14,
    CUPTI_ACTIVITY_KIND_GLOBAL_ACCESS = 15,
    CUPTI_ACTIVITY_KIND_BRANCH = 16,
    CUPTI_ACTIVITY_KIND_OVERHEAD = 17,
    CUPTI_ACTIVITY_KIND_CDP_KERNEL = 18,
    CUPTI_ACTIVITY_KIND_PREEMPTION = 19,
    CUPTI_ACTIVITY_KIND_ENVIRONMENT = 20,
    CUPTI_ACTIVITY_KIND_EVENT_INSTANCE = 21,
    CUPTI_ACTIVITY_KIND_MEMCPY2 = 22,
    CUPTI_ACTIVITY_KIND_METRIC_INSTANCE = 23,
    CUPTI_ACTIVITY_KIND_INSTRUCTION_EXECUTION = 24,
    CUPTI_ACTIVITY_KIND_UNIFIED_MEMORY_COUNTER = 25,
    CUPTI_ACTIVITY_KIND_FUNCTION = 26,
    CUPTI_ACTIVITY_KIND_MODULE = 27,
    CUPTI_ACTIVITY_KIND_DEVICE_ATTRIBUTE = 28,
    CUPTI_ACTIVITY_KIND_SHARED_ACCESS = 29,
    CUPTI_ACTIVITY_KIND_PC_SAMPLING = 30,
    CUPTI_ACTIVITY_KIND_PC_SAMPLING_RECORD_INFO = 31,
    CUPTI_ACTIVITY_KIND_INSTRUCTION_CORRELATION = 32,
    CUPTI_ACTIVITY_KIND_OPENACC_DATA = 33,
    CUPTI_ACTIVITY_KIND_OPENACC_LAUNCH = 34,
    CUPTI_ACTIVITY_KIND_OPENACC_OTHER = 35,
    CUPTI_ACTIVITY_KIND_CUDA_EVENT = 36,
    CUPTI_ACTIVITY_KIND_STREAM = 37,
    CUPTI_ACTIVITY_KIND_SYNCHRONIZATION = 38,
    CUPTI_ACTIVITY_KIND_EXTERNAL_CORRELATION = 39,
    CUPTI_ACTIVITY_KIND_NVLINK = 40,
    CUPTI_ACTIVITY_KIND_INSTANTANEOUS_EVENT = 41,
    CUPTI_ACTIVITY_KIND_INSTANTANEOUS_EVENT_INSTANCE = 42,
    CUPTI_ACTIVITY_KIND_INSTANTANEOUS_METRIC = 43,
    CUPTI_ACTIVITY_KIND_INSTANTANEOUS_METRIC_INSTANCE = 44,
    CUPTI_ACTIVITY_KIND_MEMORY = 45,
    CUPTI_ACTIVITY_KIND_PCIE = 46,
    CUPTI_ACTIVITY_KIND_OPENMP = 47,
    CUPTI_ACTIVITY_KIND_INTERNAL_LAUNCH_API = 48,
    CUPTI_ACTIVITY_KIND_MEMORY2 = 49,
    CUPTI_ACTIVITY_KIND_MEMORY_POOL = 50,
    CUPTI_ACTIVITY_KIND_COUNT = 51,
    CUPTI_ACTIVITY_KIND_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-11070"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityKind {
    CUPTI_ACTIVITY_KIND_INVALID = 0,
    CUPTI_ACTIVITY_KIND_MEMCPY = 1,
    CUPTI_ACTIVITY_KIND_MEMSET = 2,
    CUPTI_ACTIVITY_KIND_KERNEL = 3,
    CUPTI_ACTIVITY_KIND_DRIVER = 4,
    CUPTI_ACTIVITY_KIND_RUNTIME = 5,
    CUPTI_ACTIVITY_KIND_EVENT = 6,
    CUPTI_ACTIVITY_KIND_METRIC = 7,
    CUPTI_ACTIVITY_KIND_DEVICE = 8,
    CUPTI_ACTIVITY_KIND_CONTEXT = 9,
    CUPTI_ACTIVITY_KIND_CONCURRENT_KERNEL = 10,
    CUPTI_ACTIVITY_KIND_NAME = 11,
    CUPTI_ACTIVITY_KIND_MARKER = 12,
    CUPTI_ACTIVITY_KIND_MARKER_DATA = 13,
    CUPTI_ACTIVITY_KIND_SOURCE_LOCATOR = 14,
    CUPTI_ACTIVITY_KIND_GLOBAL_ACCESS = 15,
    CUPTI_ACTIVITY_KIND_BRANCH = 16,
    CUPTI_ACTIVITY_KIND_OVERHEAD = 17,
    CUPTI_ACTIVITY_KIND_CDP_KERNEL = 18,
    CUPTI_ACTIVITY_KIND_PREEMPTION = 19,
    CUPTI_ACTIVITY_KIND_ENVIRONMENT = 20,
    CUPTI_ACTIVITY_KIND_EVENT_INSTANCE = 21,
    CUPTI_ACTIVITY_KIND_MEMCPY2 = 22,
    CUPTI_ACTIVITY_KIND_METRIC_INSTANCE = 23,
    CUPTI_ACTIVITY_KIND_INSTRUCTION_EXECUTION = 24,
    CUPTI_ACTIVITY_KIND_UNIFIED_MEMORY_COUNTER = 25,
    CUPTI_ACTIVITY_KIND_FUNCTION = 26,
    CUPTI_ACTIVITY_KIND_MODULE = 27,
    CUPTI_ACTIVITY_KIND_DEVICE_ATTRIBUTE = 28,
    CUPTI_ACTIVITY_KIND_SHARED_ACCESS = 29,
    CUPTI_ACTIVITY_KIND_PC_SAMPLING = 30,
    CUPTI_ACTIVITY_KIND_PC_SAMPLING_RECORD_INFO = 31,
    CUPTI_ACTIVITY_KIND_INSTRUCTION_CORRELATION = 32,
    CUPTI_ACTIVITY_KIND_OPENACC_DATA = 33,
    CUPTI_ACTIVITY_KIND_OPENACC_LAUNCH = 34,
    CUPTI_ACTIVITY_KIND_OPENACC_OTHER = 35,
    CUPTI_ACTIVITY_KIND_CUDA_EVENT = 36,
    CUPTI_ACTIVITY_KIND_STREAM = 37,
    CUPTI_ACTIVITY_KIND_SYNCHRONIZATION = 38,
    CUPTI_ACTIVITY_KIND_EXTERNAL_CORRELATION = 39,
    CUPTI_ACTIVITY_KIND_NVLINK = 40,
    CUPTI_ACTIVITY_KIND_INSTANTANEOUS_EVENT = 41,
    CUPTI_ACTIVITY_KIND_INSTANTANEOUS_EVENT_INSTANCE = 42,
    CUPTI_ACTIVITY_KIND_INSTANTANEOUS_METRIC = 43,
    CUPTI_ACTIVITY_KIND_INSTANTANEOUS_METRIC_INSTANCE = 44,
    CUPTI_ACTIVITY_KIND_MEMORY = 45,
    CUPTI_ACTIVITY_KIND_PCIE = 46,
    CUPTI_ACTIVITY_KIND_OPENMP = 47,
    CUPTI_ACTIVITY_KIND_INTERNAL_LAUNCH_API = 48,
    CUPTI_ACTIVITY_KIND_MEMORY2 = 49,
    CUPTI_ACTIVITY_KIND_MEMORY_POOL = 50,
    CUPTI_ACTIVITY_KIND_GRAPH_TRACE = 51,
    CUPTI_ACTIVITY_KIND_COUNT = 52,
    CUPTI_ACTIVITY_KIND_FORCE_INT = 2147483647,
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060"
))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityKind {
    CUPTI_ACTIVITY_KIND_INVALID = 0,
    CUPTI_ACTIVITY_KIND_MEMCPY = 1,
    CUPTI_ACTIVITY_KIND_MEMSET = 2,
    CUPTI_ACTIVITY_KIND_KERNEL = 3,
    CUPTI_ACTIVITY_KIND_DRIVER = 4,
    CUPTI_ACTIVITY_KIND_RUNTIME = 5,
    CUPTI_ACTIVITY_KIND_EVENT = 6,
    CUPTI_ACTIVITY_KIND_METRIC = 7,
    CUPTI_ACTIVITY_KIND_DEVICE = 8,
    CUPTI_ACTIVITY_KIND_CONTEXT = 9,
    CUPTI_ACTIVITY_KIND_CONCURRENT_KERNEL = 10,
    CUPTI_ACTIVITY_KIND_NAME = 11,
    CUPTI_ACTIVITY_KIND_MARKER = 12,
    CUPTI_ACTIVITY_KIND_MARKER_DATA = 13,
    CUPTI_ACTIVITY_KIND_SOURCE_LOCATOR = 14,
    CUPTI_ACTIVITY_KIND_GLOBAL_ACCESS = 15,
    CUPTI_ACTIVITY_KIND_BRANCH = 16,
    CUPTI_ACTIVITY_KIND_OVERHEAD = 17,
    CUPTI_ACTIVITY_KIND_CDP_KERNEL = 18,
    CUPTI_ACTIVITY_KIND_PREEMPTION = 19,
    CUPTI_ACTIVITY_KIND_ENVIRONMENT = 20,
    CUPTI_ACTIVITY_KIND_EVENT_INSTANCE = 21,
    CUPTI_ACTIVITY_KIND_MEMCPY2 = 22,
    CUPTI_ACTIVITY_KIND_METRIC_INSTANCE = 23,
    CUPTI_ACTIVITY_KIND_INSTRUCTION_EXECUTION = 24,
    CUPTI_ACTIVITY_KIND_UNIFIED_MEMORY_COUNTER = 25,
    CUPTI_ACTIVITY_KIND_FUNCTION = 26,
    CUPTI_ACTIVITY_KIND_MODULE = 27,
    CUPTI_ACTIVITY_KIND_DEVICE_ATTRIBUTE = 28,
    CUPTI_ACTIVITY_KIND_SHARED_ACCESS = 29,
    CUPTI_ACTIVITY_KIND_PC_SAMPLING = 30,
    CUPTI_ACTIVITY_KIND_PC_SAMPLING_RECORD_INFO = 31,
    CUPTI_ACTIVITY_KIND_INSTRUCTION_CORRELATION = 32,
    CUPTI_ACTIVITY_KIND_OPENACC_DATA = 33,
    CUPTI_ACTIVITY_KIND_OPENACC_LAUNCH = 34,
    CUPTI_ACTIVITY_KIND_OPENACC_OTHER = 35,
    CUPTI_ACTIVITY_KIND_CUDA_EVENT = 36,
    CUPTI_ACTIVITY_KIND_STREAM = 37,
    CUPTI_ACTIVITY_KIND_SYNCHRONIZATION = 38,
    CUPTI_ACTIVITY_KIND_EXTERNAL_CORRELATION = 39,
    CUPTI_ACTIVITY_KIND_NVLINK = 40,
    CUPTI_ACTIVITY_KIND_INSTANTANEOUS_EVENT = 41,
    CUPTI_ACTIVITY_KIND_INSTANTANEOUS_EVENT_INSTANCE = 42,
    CUPTI_ACTIVITY_KIND_INSTANTANEOUS_METRIC = 43,
    CUPTI_ACTIVITY_KIND_INSTANTANEOUS_METRIC_INSTANCE = 44,
    CUPTI_ACTIVITY_KIND_MEMORY = 45,
    CUPTI_ACTIVITY_KIND_PCIE = 46,
    CUPTI_ACTIVITY_KIND_OPENMP = 47,
    CUPTI_ACTIVITY_KIND_INTERNAL_LAUNCH_API = 48,
    CUPTI_ACTIVITY_KIND_MEMORY2 = 49,
    CUPTI_ACTIVITY_KIND_MEMORY_POOL = 50,
    CUPTI_ACTIVITY_KIND_GRAPH_TRACE = 51,
    CUPTI_ACTIVITY_KIND_JIT = 52,
    CUPTI_ACTIVITY_KIND_COUNT = 53,
    CUPTI_ACTIVITY_KIND_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-12080"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityKind {
    CUPTI_ACTIVITY_KIND_INVALID = 0,
    CUPTI_ACTIVITY_KIND_MEMCPY = 1,
    CUPTI_ACTIVITY_KIND_MEMSET = 2,
    CUPTI_ACTIVITY_KIND_KERNEL = 3,
    CUPTI_ACTIVITY_KIND_DRIVER = 4,
    CUPTI_ACTIVITY_KIND_RUNTIME = 5,
    CUPTI_ACTIVITY_KIND_EVENT = 6,
    CUPTI_ACTIVITY_KIND_METRIC = 7,
    CUPTI_ACTIVITY_KIND_DEVICE = 8,
    CUPTI_ACTIVITY_KIND_CONTEXT = 9,
    CUPTI_ACTIVITY_KIND_CONCURRENT_KERNEL = 10,
    CUPTI_ACTIVITY_KIND_NAME = 11,
    CUPTI_ACTIVITY_KIND_MARKER = 12,
    CUPTI_ACTIVITY_KIND_MARKER_DATA = 13,
    CUPTI_ACTIVITY_KIND_SOURCE_LOCATOR = 14,
    CUPTI_ACTIVITY_KIND_GLOBAL_ACCESS = 15,
    CUPTI_ACTIVITY_KIND_BRANCH = 16,
    CUPTI_ACTIVITY_KIND_OVERHEAD = 17,
    CUPTI_ACTIVITY_KIND_CDP_KERNEL = 18,
    CUPTI_ACTIVITY_KIND_PREEMPTION = 19,
    CUPTI_ACTIVITY_KIND_ENVIRONMENT = 20,
    CUPTI_ACTIVITY_KIND_EVENT_INSTANCE = 21,
    CUPTI_ACTIVITY_KIND_MEMCPY2 = 22,
    CUPTI_ACTIVITY_KIND_METRIC_INSTANCE = 23,
    CUPTI_ACTIVITY_KIND_INSTRUCTION_EXECUTION = 24,
    CUPTI_ACTIVITY_KIND_UNIFIED_MEMORY_COUNTER = 25,
    CUPTI_ACTIVITY_KIND_FUNCTION = 26,
    CUPTI_ACTIVITY_KIND_MODULE = 27,
    CUPTI_ACTIVITY_KIND_DEVICE_ATTRIBUTE = 28,
    CUPTI_ACTIVITY_KIND_SHARED_ACCESS = 29,
    CUPTI_ACTIVITY_KIND_PC_SAMPLING = 30,
    CUPTI_ACTIVITY_KIND_PC_SAMPLING_RECORD_INFO = 31,
    CUPTI_ACTIVITY_KIND_INSTRUCTION_CORRELATION = 32,
    CUPTI_ACTIVITY_KIND_OPENACC_DATA = 33,
    CUPTI_ACTIVITY_KIND_OPENACC_LAUNCH = 34,
    CUPTI_ACTIVITY_KIND_OPENACC_OTHER = 35,
    CUPTI_ACTIVITY_KIND_CUDA_EVENT = 36,
    CUPTI_ACTIVITY_KIND_STREAM = 37,
    CUPTI_ACTIVITY_KIND_SYNCHRONIZATION = 38,
    CUPTI_ACTIVITY_KIND_EXTERNAL_CORRELATION = 39,
    CUPTI_ACTIVITY_KIND_NVLINK = 40,
    CUPTI_ACTIVITY_KIND_INSTANTANEOUS_EVENT = 41,
    CUPTI_ACTIVITY_KIND_INSTANTANEOUS_EVENT_INSTANCE = 42,
    CUPTI_ACTIVITY_KIND_INSTANTANEOUS_METRIC = 43,
    CUPTI_ACTIVITY_KIND_INSTANTANEOUS_METRIC_INSTANCE = 44,
    CUPTI_ACTIVITY_KIND_MEMORY = 45,
    CUPTI_ACTIVITY_KIND_PCIE = 46,
    CUPTI_ACTIVITY_KIND_OPENMP = 47,
    CUPTI_ACTIVITY_KIND_INTERNAL_LAUNCH_API = 48,
    CUPTI_ACTIVITY_KIND_MEMORY2 = 49,
    CUPTI_ACTIVITY_KIND_MEMORY_POOL = 50,
    CUPTI_ACTIVITY_KIND_GRAPH_TRACE = 51,
    CUPTI_ACTIVITY_KIND_JIT = 52,
    CUPTI_ACTIVITY_KIND_DEVICE_GRAPH_TRACE = 53,
    CUPTI_ACTIVITY_KIND_MEM_DECOMPRESS = 54,
    CUPTI_ACTIVITY_KIND_COUNT = 55,
    CUPTI_ACTIVITY_KIND_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityKind {
    CUPTI_ACTIVITY_KIND_INVALID = 0,
    CUPTI_ACTIVITY_KIND_MEMCPY = 1,
    CUPTI_ACTIVITY_KIND_MEMSET = 2,
    CUPTI_ACTIVITY_KIND_KERNEL = 3,
    CUPTI_ACTIVITY_KIND_DRIVER = 4,
    CUPTI_ACTIVITY_KIND_RUNTIME = 5,
    CUPTI_ACTIVITY_KIND_EVENT = 6,
    CUPTI_ACTIVITY_KIND_METRIC = 7,
    CUPTI_ACTIVITY_KIND_DEVICE = 8,
    CUPTI_ACTIVITY_KIND_CONTEXT = 9,
    CUPTI_ACTIVITY_KIND_CONCURRENT_KERNEL = 10,
    CUPTI_ACTIVITY_KIND_NAME = 11,
    CUPTI_ACTIVITY_KIND_MARKER = 12,
    CUPTI_ACTIVITY_KIND_MARKER_DATA = 13,
    CUPTI_ACTIVITY_KIND_SOURCE_LOCATOR = 14,
    CUPTI_ACTIVITY_KIND_GLOBAL_ACCESS = 15,
    CUPTI_ACTIVITY_KIND_BRANCH = 16,
    CUPTI_ACTIVITY_KIND_OVERHEAD = 17,
    CUPTI_ACTIVITY_KIND_CDP_KERNEL = 18,
    CUPTI_ACTIVITY_KIND_PREEMPTION = 19,
    CUPTI_ACTIVITY_KIND_ENVIRONMENT = 20,
    CUPTI_ACTIVITY_KIND_EVENT_INSTANCE = 21,
    CUPTI_ACTIVITY_KIND_MEMCPY2 = 22,
    CUPTI_ACTIVITY_KIND_METRIC_INSTANCE = 23,
    CUPTI_ACTIVITY_KIND_INSTRUCTION_EXECUTION = 24,
    CUPTI_ACTIVITY_KIND_UNIFIED_MEMORY_COUNTER = 25,
    CUPTI_ACTIVITY_KIND_FUNCTION = 26,
    CUPTI_ACTIVITY_KIND_MODULE = 27,
    CUPTI_ACTIVITY_KIND_DEVICE_ATTRIBUTE = 28,
    CUPTI_ACTIVITY_KIND_SHARED_ACCESS = 29,
    CUPTI_ACTIVITY_KIND_PC_SAMPLING = 30,
    CUPTI_ACTIVITY_KIND_PC_SAMPLING_RECORD_INFO = 31,
    CUPTI_ACTIVITY_KIND_INSTRUCTION_CORRELATION = 32,
    CUPTI_ACTIVITY_KIND_OPENACC_DATA = 33,
    CUPTI_ACTIVITY_KIND_OPENACC_LAUNCH = 34,
    CUPTI_ACTIVITY_KIND_OPENACC_OTHER = 35,
    CUPTI_ACTIVITY_KIND_CUDA_EVENT = 36,
    CUPTI_ACTIVITY_KIND_STREAM = 37,
    CUPTI_ACTIVITY_KIND_SYNCHRONIZATION = 38,
    CUPTI_ACTIVITY_KIND_EXTERNAL_CORRELATION = 39,
    CUPTI_ACTIVITY_KIND_NVLINK = 40,
    CUPTI_ACTIVITY_KIND_INSTANTANEOUS_EVENT = 41,
    CUPTI_ACTIVITY_KIND_INSTANTANEOUS_EVENT_INSTANCE = 42,
    CUPTI_ACTIVITY_KIND_INSTANTANEOUS_METRIC = 43,
    CUPTI_ACTIVITY_KIND_INSTANTANEOUS_METRIC_INSTANCE = 44,
    CUPTI_ACTIVITY_KIND_MEMORY = 45,
    CUPTI_ACTIVITY_KIND_PCIE = 46,
    CUPTI_ACTIVITY_KIND_OPENMP = 47,
    CUPTI_ACTIVITY_KIND_INTERNAL_LAUNCH_API = 48,
    CUPTI_ACTIVITY_KIND_MEMORY2 = 49,
    CUPTI_ACTIVITY_KIND_MEMORY_POOL = 50,
    CUPTI_ACTIVITY_KIND_GRAPH_TRACE = 51,
    CUPTI_ACTIVITY_KIND_JIT = 52,
    CUPTI_ACTIVITY_KIND_DEVICE_GRAPH_TRACE = 53,
    CUPTI_ACTIVITY_KIND_MEM_DECOMPRESS = 54,
    CUPTI_ACTIVITY_KIND_CONFIDENTIAL_COMPUTE_ROTATION = 55,
    CUPTI_ACTIVITY_KIND_COUNT = 56,
    CUPTI_ACTIVITY_KIND_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityKind {
    CUPTI_ACTIVITY_KIND_INVALID = 0,
    CUPTI_ACTIVITY_KIND_MEMCPY = 1,
    CUPTI_ACTIVITY_KIND_MEMSET = 2,
    CUPTI_ACTIVITY_KIND_KERNEL = 3,
    CUPTI_ACTIVITY_KIND_DRIVER = 4,
    CUPTI_ACTIVITY_KIND_RUNTIME = 5,
    CUPTI_ACTIVITY_KIND_EVENT = 6,
    CUPTI_ACTIVITY_KIND_METRIC = 7,
    CUPTI_ACTIVITY_KIND_DEVICE = 8,
    CUPTI_ACTIVITY_KIND_CONTEXT = 9,
    CUPTI_ACTIVITY_KIND_CONCURRENT_KERNEL = 10,
    CUPTI_ACTIVITY_KIND_NAME = 11,
    CUPTI_ACTIVITY_KIND_MARKER = 12,
    CUPTI_ACTIVITY_KIND_MARKER_DATA = 13,
    CUPTI_ACTIVITY_KIND_SOURCE_LOCATOR = 14,
    CUPTI_ACTIVITY_KIND_GLOBAL_ACCESS = 15,
    CUPTI_ACTIVITY_KIND_BRANCH = 16,
    CUPTI_ACTIVITY_KIND_OVERHEAD = 17,
    CUPTI_ACTIVITY_KIND_CDP_KERNEL = 18,
    CUPTI_ACTIVITY_KIND_PREEMPTION = 19,
    CUPTI_ACTIVITY_KIND_ENVIRONMENT = 20,
    CUPTI_ACTIVITY_KIND_EVENT_INSTANCE = 21,
    CUPTI_ACTIVITY_KIND_MEMCPY2 = 22,
    CUPTI_ACTIVITY_KIND_METRIC_INSTANCE = 23,
    CUPTI_ACTIVITY_KIND_INSTRUCTION_EXECUTION = 24,
    CUPTI_ACTIVITY_KIND_UNIFIED_MEMORY_COUNTER = 25,
    CUPTI_ACTIVITY_KIND_FUNCTION = 26,
    CUPTI_ACTIVITY_KIND_MODULE = 27,
    CUPTI_ACTIVITY_KIND_DEVICE_ATTRIBUTE = 28,
    CUPTI_ACTIVITY_KIND_SHARED_ACCESS = 29,
    CUPTI_ACTIVITY_KIND_PC_SAMPLING = 30,
    CUPTI_ACTIVITY_KIND_PC_SAMPLING_RECORD_INFO = 31,
    CUPTI_ACTIVITY_KIND_INSTRUCTION_CORRELATION = 32,
    CUPTI_ACTIVITY_KIND_OPENACC_DATA = 33,
    CUPTI_ACTIVITY_KIND_OPENACC_LAUNCH = 34,
    CUPTI_ACTIVITY_KIND_OPENACC_OTHER = 35,
    CUPTI_ACTIVITY_KIND_CUDA_EVENT = 36,
    CUPTI_ACTIVITY_KIND_STREAM = 37,
    CUPTI_ACTIVITY_KIND_SYNCHRONIZATION = 38,
    CUPTI_ACTIVITY_KIND_EXTERNAL_CORRELATION = 39,
    CUPTI_ACTIVITY_KIND_NVLINK = 40,
    CUPTI_ACTIVITY_KIND_INSTANTANEOUS_EVENT = 41,
    CUPTI_ACTIVITY_KIND_INSTANTANEOUS_EVENT_INSTANCE = 42,
    CUPTI_ACTIVITY_KIND_INSTANTANEOUS_METRIC = 43,
    CUPTI_ACTIVITY_KIND_INSTANTANEOUS_METRIC_INSTANCE = 44,
    CUPTI_ACTIVITY_KIND_MEMORY = 45,
    CUPTI_ACTIVITY_KIND_PCIE = 46,
    CUPTI_ACTIVITY_KIND_OPENMP = 47,
    CUPTI_ACTIVITY_KIND_INTERNAL_LAUNCH_API = 48,
    CUPTI_ACTIVITY_KIND_MEMORY2 = 49,
    CUPTI_ACTIVITY_KIND_MEMORY_POOL = 50,
    CUPTI_ACTIVITY_KIND_GRAPH_TRACE = 51,
    CUPTI_ACTIVITY_KIND_JIT = 52,
    CUPTI_ACTIVITY_KIND_DEVICE_GRAPH_TRACE = 53,
    CUPTI_ACTIVITY_KIND_MEM_DECOMPRESS = 54,
    CUPTI_ACTIVITY_KIND_CONFIDENTIAL_COMPUTE_ROTATION = 55,
    CUPTI_ACTIVITY_KIND_GRAPH_HOST_NODE = 56,
    CUPTI_ACTIVITY_KIND_COMPUTE_ENGINE_CTX_SWITCH = 57,
    CUPTI_ACTIVITY_KIND_HOST_LAUNCH = 58,
    CUPTI_ACTIVITY_KIND_COUNT = 59,
    CUPTI_ACTIVITY_KIND_FORCE_INT = 2147483647,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityLaunchType {
    CUPTI_ACTIVITY_LAUNCH_TYPE_REGULAR = 0,
    CUPTI_ACTIVITY_LAUNCH_TYPE_COOPERATIVE_SINGLE_DEVICE = 1,
    CUPTI_ACTIVITY_LAUNCH_TYPE_COOPERATIVE_MULTI_DEVICE = 2,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityLaunchType {
    CUPTI_ACTIVITY_LAUNCH_TYPE_REGULAR = 0,
    CUPTI_ACTIVITY_LAUNCH_TYPE_COOPERATIVE_SINGLE_DEVICE = 1,
    CUPTI_ACTIVITY_LAUNCH_TYPE_COOPERATIVE_MULTI_DEVICE = 2,
    CUPTI_ACTIVITY_LAUNCH_TYPE_CBL_COMMANDLIST = 3,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityMemcpyKind {
    CUPTI_ACTIVITY_MEMCPY_KIND_UNKNOWN = 0,
    CUPTI_ACTIVITY_MEMCPY_KIND_HTOD = 1,
    CUPTI_ACTIVITY_MEMCPY_KIND_DTOH = 2,
    CUPTI_ACTIVITY_MEMCPY_KIND_HTOA = 3,
    CUPTI_ACTIVITY_MEMCPY_KIND_ATOH = 4,
    CUPTI_ACTIVITY_MEMCPY_KIND_ATOA = 5,
    CUPTI_ACTIVITY_MEMCPY_KIND_ATOD = 6,
    CUPTI_ACTIVITY_MEMCPY_KIND_DTOA = 7,
    CUPTI_ACTIVITY_MEMCPY_KIND_DTOD = 8,
    CUPTI_ACTIVITY_MEMCPY_KIND_HTOH = 9,
    CUPTI_ACTIVITY_MEMCPY_KIND_PTOP = 10,
    CUPTI_ACTIVITY_MEMCPY_KIND_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityMemoryKind {
    CUPTI_ACTIVITY_MEMORY_KIND_UNKNOWN = 0,
    CUPTI_ACTIVITY_MEMORY_KIND_PAGEABLE = 1,
    CUPTI_ACTIVITY_MEMORY_KIND_PINNED = 2,
    CUPTI_ACTIVITY_MEMORY_KIND_DEVICE = 3,
    CUPTI_ACTIVITY_MEMORY_KIND_ARRAY = 4,
    CUPTI_ACTIVITY_MEMORY_KIND_MANAGED = 5,
    CUPTI_ACTIVITY_MEMORY_KIND_DEVICE_STATIC = 6,
    CUPTI_ACTIVITY_MEMORY_KIND_MANAGED_STATIC = 7,
    CUPTI_ACTIVITY_MEMORY_KIND_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityMemoryOperationType {
    CUPTI_ACTIVITY_MEMORY_OPERATION_TYPE_INVALID = 0,
    CUPTI_ACTIVITY_MEMORY_OPERATION_TYPE_ALLOCATION = 1,
    CUPTI_ACTIVITY_MEMORY_OPERATION_TYPE_RELEASE = 2,
    CUPTI_ACTIVITY_MEMORY_OPERATION_TYPE_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityMemoryPoolOperationType {
    CUPTI_ACTIVITY_MEMORY_POOL_OPERATION_TYPE_INVALID = 0,
    CUPTI_ACTIVITY_MEMORY_POOL_OPERATION_TYPE_CREATED = 1,
    CUPTI_ACTIVITY_MEMORY_POOL_OPERATION_TYPE_DESTROYED = 2,
    CUPTI_ACTIVITY_MEMORY_POOL_OPERATION_TYPE_TRIMMED = 3,
    CUPTI_ACTIVITY_MEMORY_POOL_OPERATION_TYPE_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityMemoryPoolType {
    CUPTI_ACTIVITY_MEMORY_POOL_TYPE_INVALID = 0,
    CUPTI_ACTIVITY_MEMORY_POOL_TYPE_LOCAL = 1,
    CUPTI_ACTIVITY_MEMORY_POOL_TYPE_IMPORTED = 2,
    CUPTI_ACTIVITY_MEMORY_POOL_TYPE_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityObjectKind {
    CUPTI_ACTIVITY_OBJECT_UNKNOWN = 0,
    CUPTI_ACTIVITY_OBJECT_PROCESS = 1,
    CUPTI_ACTIVITY_OBJECT_THREAD = 2,
    CUPTI_ACTIVITY_OBJECT_DEVICE = 3,
    CUPTI_ACTIVITY_OBJECT_CONTEXT = 4,
    CUPTI_ACTIVITY_OBJECT_STREAM = 5,
    CUPTI_ACTIVITY_OBJECT_FORCE_INT = 2147483647,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020"
))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityOverheadKind {
    CUPTI_ACTIVITY_OVERHEAD_UNKNOWN = 0,
    CUPTI_ACTIVITY_OVERHEAD_DRIVER_COMPILER = 1,
    CUPTI_ACTIVITY_OVERHEAD_CUPTI_BUFFER_FLUSH = 65536,
    CUPTI_ACTIVITY_OVERHEAD_CUPTI_INSTRUMENTATION = 131072,
    CUPTI_ACTIVITY_OVERHEAD_CUPTI_RESOURCE = 196608,
    CUPTI_ACTIVITY_OVERHEAD_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-12030"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityOverheadKind {
    CUPTI_ACTIVITY_OVERHEAD_UNKNOWN = 0,
    CUPTI_ACTIVITY_OVERHEAD_DRIVER_COMPILER = 1,
    CUPTI_ACTIVITY_OVERHEAD_CUPTI_BUFFER_FLUSH = 65536,
    CUPTI_ACTIVITY_OVERHEAD_CUPTI_INSTRUMENTATION = 131072,
    CUPTI_ACTIVITY_OVERHEAD_CUPTI_RESOURCE = 196608,
    CUPTI_ACTIVITY_OVERHEAD_RUNTIME_TRIGGERED_MODULE_LOADING = 262144,
    CUPTI_ACTIVITY_OVERHEAD_LAZY_FUNCTION_LOADING = 327680,
    CUPTI_ACTIVITY_OVERHEAD_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-12040", feature = "cuda-12050"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityOverheadKind {
    CUPTI_ACTIVITY_OVERHEAD_UNKNOWN = 0,
    CUPTI_ACTIVITY_OVERHEAD_DRIVER_COMPILER = 1,
    CUPTI_ACTIVITY_OVERHEAD_CUPTI_BUFFER_FLUSH = 65536,
    CUPTI_ACTIVITY_OVERHEAD_CUPTI_INSTRUMENTATION = 131072,
    CUPTI_ACTIVITY_OVERHEAD_CUPTI_RESOURCE = 196608,
    CUPTI_ACTIVITY_OVERHEAD_RUNTIME_TRIGGERED_MODULE_LOADING = 262144,
    CUPTI_ACTIVITY_OVERHEAD_LAZY_FUNCTION_LOADING = 327680,
    CUPTI_ACTIVITY_OVERHEAD_COMMAND_BUFFER_FULL = 393216,
    CUPTI_ACTIVITY_OVERHEAD_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-12060"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityOverheadKind {
    CUPTI_ACTIVITY_OVERHEAD_UNKNOWN = 0,
    CUPTI_ACTIVITY_OVERHEAD_DRIVER_COMPILER = 1,
    CUPTI_ACTIVITY_OVERHEAD_CUPTI_BUFFER_FLUSH = 65536,
    CUPTI_ACTIVITY_OVERHEAD_CUPTI_INSTRUMENTATION = 131072,
    CUPTI_ACTIVITY_OVERHEAD_CUPTI_RESOURCE = 196608,
    CUPTI_ACTIVITY_OVERHEAD_RUNTIME_TRIGGERED_MODULE_LOADING = 262144,
    CUPTI_ACTIVITY_OVERHEAD_LAZY_FUNCTION_LOADING = 327680,
    CUPTI_ACTIVITY_OVERHEAD_COMMAND_BUFFER_FULL = 393216,
    CUPTI_ACTIVITY_OVERHEAD_ACTIVITY_BUFFER_REQUEST = 458752,
    CUPTI_ACTIVITY_OVERHEAD_FORCE_INT = 2147483647,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityOverheadKind {
    CUPTI_ACTIVITY_OVERHEAD_UNKNOWN = 0,
    CUPTI_ACTIVITY_OVERHEAD_DRIVER_COMPILER = 1,
    CUPTI_ACTIVITY_OVERHEAD_CUPTI_BUFFER_FLUSH = 65536,
    CUPTI_ACTIVITY_OVERHEAD_CUPTI_INSTRUMENTATION = 131072,
    CUPTI_ACTIVITY_OVERHEAD_CUPTI_RESOURCE = 196608,
    CUPTI_ACTIVITY_OVERHEAD_RUNTIME_TRIGGERED_MODULE_LOADING = 262144,
    CUPTI_ACTIVITY_OVERHEAD_LAZY_FUNCTION_LOADING = 327680,
    CUPTI_ACTIVITY_OVERHEAD_COMMAND_BUFFER_FULL = 393216,
    CUPTI_ACTIVITY_OVERHEAD_ACTIVITY_BUFFER_REQUEST = 458752,
    CUPTI_ACTIVITY_OVERHEAD_UVM_ACTIVITY_INIT = 524288,
    CUPTI_ACTIVITY_OVERHEAD_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityPCSamplingPeriod {
    CUPTI_ACTIVITY_PC_SAMPLING_PERIOD_INVALID = 0,
    CUPTI_ACTIVITY_PC_SAMPLING_PERIOD_MIN = 1,
    CUPTI_ACTIVITY_PC_SAMPLING_PERIOD_LOW = 2,
    CUPTI_ACTIVITY_PC_SAMPLING_PERIOD_MID = 3,
    CUPTI_ACTIVITY_PC_SAMPLING_PERIOD_HIGH = 4,
    CUPTI_ACTIVITY_PC_SAMPLING_PERIOD_MAX = 5,
    CUPTI_ACTIVITY_PC_SAMPLING_PERIOD_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityPCSamplingStallReason {
    CUPTI_ACTIVITY_PC_SAMPLING_STALL_INVALID = 0,
    CUPTI_ACTIVITY_PC_SAMPLING_STALL_NONE = 1,
    CUPTI_ACTIVITY_PC_SAMPLING_STALL_INST_FETCH = 2,
    CUPTI_ACTIVITY_PC_SAMPLING_STALL_EXEC_DEPENDENCY = 3,
    CUPTI_ACTIVITY_PC_SAMPLING_STALL_MEMORY_DEPENDENCY = 4,
    CUPTI_ACTIVITY_PC_SAMPLING_STALL_TEXTURE = 5,
    CUPTI_ACTIVITY_PC_SAMPLING_STALL_SYNC = 6,
    CUPTI_ACTIVITY_PC_SAMPLING_STALL_CONSTANT_MEMORY_DEPENDENCY = 7,
    CUPTI_ACTIVITY_PC_SAMPLING_STALL_PIPE_BUSY = 8,
    CUPTI_ACTIVITY_PC_SAMPLING_STALL_MEMORY_THROTTLE = 9,
    CUPTI_ACTIVITY_PC_SAMPLING_STALL_NOT_SELECTED = 10,
    CUPTI_ACTIVITY_PC_SAMPLING_STALL_OTHER = 11,
    CUPTI_ACTIVITY_PC_SAMPLING_STALL_SLEEPING = 12,
    CUPTI_ACTIVITY_PC_SAMPLING_STALL_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityPartitionedGlobalCacheConfig {
    CUPTI_ACTIVITY_PARTITIONED_GLOBAL_CACHE_CONFIG_UNKNOWN = 0,
    CUPTI_ACTIVITY_PARTITIONED_GLOBAL_CACHE_CONFIG_NOT_SUPPORTED = 1,
    CUPTI_ACTIVITY_PARTITIONED_GLOBAL_CACHE_CONFIG_OFF = 2,
    CUPTI_ACTIVITY_PARTITIONED_GLOBAL_CACHE_CONFIG_ON = 3,
    CUPTI_ACTIVITY_PARTITIONED_GLOBAL_CACHE_CONFIG_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityPreemptionKind {
    CUPTI_ACTIVITY_PREEMPTION_KIND_UNKNOWN = 0,
    CUPTI_ACTIVITY_PREEMPTION_KIND_SAVE = 1,
    CUPTI_ACTIVITY_PREEMPTION_KIND_RESTORE = 2,
    CUPTI_ACTIVITY_PREEMPTION_KIND_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityStreamFlag {
    CUPTI_ACTIVITY_STREAM_CREATE_FLAG_UNKNOWN = 0,
    CUPTI_ACTIVITY_STREAM_CREATE_FLAG_DEFAULT = 1,
    CUPTI_ACTIVITY_STREAM_CREATE_FLAG_NON_BLOCKING = 2,
    CUPTI_ACTIVITY_STREAM_CREATE_FLAG_NULL = 3,
    CUPTI_ACTIVITY_STREAM_CREATE_MASK = 65535,
    CUPTI_ACTIVITY_STREAM_CREATE_FLAG_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivitySynchronizationType {
    CUPTI_ACTIVITY_SYNCHRONIZATION_TYPE_UNKNOWN = 0,
    CUPTI_ACTIVITY_SYNCHRONIZATION_TYPE_EVENT_SYNCHRONIZE = 1,
    CUPTI_ACTIVITY_SYNCHRONIZATION_TYPE_STREAM_WAIT_EVENT = 2,
    CUPTI_ACTIVITY_SYNCHRONIZATION_TYPE_STREAM_SYNCHRONIZE = 3,
    CUPTI_ACTIVITY_SYNCHRONIZATION_TYPE_CONTEXT_SYNCHRONIZE = 4,
    CUPTI_ACTIVITY_SYNCHRONIZATION_TYPE_FORCE_INT = 2147483647,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030"
))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityThreadIdType {
    CUPTI_ACTIVITY_THREAD_ID_TYPE_DEFAULT = 0,
    CUPTI_ACTIVITY_THREAD_ID_TYPE_SYSTEM = 1,
    CUPTI_ACTIVITY_THREAD_ID_TYPE_FORCE_INT = 2147483647,
}
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityThreadIdType {
    CUPTI_ACTIVITY_THREAD_ID_TYPE_DEFAULT = 0,
    CUPTI_ACTIVITY_THREAD_ID_TYPE_SYSTEM = 1,
    CUPTI_ACTIVITY_THREAD_ID_TYPE_SIZE = 2,
    CUPTI_ACTIVITY_THREAD_ID_TYPE_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityUnifiedMemoryAccessType {
    CUPTI_ACTIVITY_UNIFIED_MEMORY_ACCESS_TYPE_UNKNOWN = 0,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_ACCESS_TYPE_READ = 1,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_ACCESS_TYPE_WRITE = 2,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_ACCESS_TYPE_ATOMIC = 3,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_ACCESS_TYPE_PREFETCH = 4,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityUnifiedMemoryCounterKind {
    CUPTI_ACTIVITY_UNIFIED_MEMORY_COUNTER_KIND_UNKNOWN = 0,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_COUNTER_KIND_BYTES_TRANSFER_HTOD = 1,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_COUNTER_KIND_BYTES_TRANSFER_DTOH = 2,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_COUNTER_KIND_CPU_PAGE_FAULT_COUNT = 3,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_COUNTER_KIND_GPU_PAGE_FAULT = 4,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_COUNTER_KIND_THRASHING = 5,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_COUNTER_KIND_THROTTLING = 6,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_COUNTER_KIND_REMOTE_MAP = 7,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_COUNTER_KIND_BYTES_TRANSFER_DTOD = 8,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_COUNTER_KIND_COUNT = 9,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_COUNTER_KIND_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityUnifiedMemoryCounterScope {
    CUPTI_ACTIVITY_UNIFIED_MEMORY_COUNTER_SCOPE_UNKNOWN = 0,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_COUNTER_SCOPE_PROCESS_SINGLE_DEVICE = 1,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_COUNTER_SCOPE_PROCESS_ALL_DEVICES = 2,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_COUNTER_SCOPE_COUNT = 3,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_COUNTER_SCOPE_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityUnifiedMemoryMigrationCause {
    CUPTI_ACTIVITY_UNIFIED_MEMORY_MIGRATION_CAUSE_UNKNOWN = 0,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_MIGRATION_CAUSE_USER = 1,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_MIGRATION_CAUSE_COHERENCE = 2,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_MIGRATION_CAUSE_PREFETCH = 3,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_MIGRATION_CAUSE_EVICTION = 4,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_MIGRATION_CAUSE_ACCESS_COUNTERS = 5,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ActivityUnifiedMemoryRemoteMapCause {
    CUPTI_ACTIVITY_UNIFIED_MEMORY_REMOTE_MAP_CAUSE_UNKNOWN = 0,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_REMOTE_MAP_CAUSE_COHERENCE = 1,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_REMOTE_MAP_CAUSE_THRASHING = 2,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_REMOTE_MAP_CAUSE_POLICY = 3,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_REMOTE_MAP_CAUSE_OUT_OF_MEMORY = 4,
    CUPTI_ACTIVITY_UNIFIED_MEMORY_REMOTE_MAP_CAUSE_EVICTION = 5,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ApiCallbackSite {
    CUPTI_API_ENTER = 0,
    CUPTI_API_EXIT = 1,
    CUPTI_API_CBSITE_FORCE_INT = 2147483647,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010"
))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_CallbackDomain {
    CUPTI_CB_DOMAIN_INVALID = 0,
    CUPTI_CB_DOMAIN_DRIVER_API = 1,
    CUPTI_CB_DOMAIN_RUNTIME_API = 2,
    CUPTI_CB_DOMAIN_RESOURCE = 3,
    CUPTI_CB_DOMAIN_SYNCHRONIZE = 4,
    CUPTI_CB_DOMAIN_NVTX = 5,
    CUPTI_CB_DOMAIN_SIZE = 6,
    CUPTI_CB_DOMAIN_FORCE_INT = 2147483647,
}
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_CallbackDomain {
    CUPTI_CB_DOMAIN_INVALID = 0,
    CUPTI_CB_DOMAIN_DRIVER_API = 1,
    CUPTI_CB_DOMAIN_RUNTIME_API = 2,
    CUPTI_CB_DOMAIN_RESOURCE = 3,
    CUPTI_CB_DOMAIN_SYNCHRONIZE = 4,
    CUPTI_CB_DOMAIN_NVTX = 5,
    CUPTI_CB_DOMAIN_STATE = 6,
    CUPTI_CB_DOMAIN_SIZE = 7,
    CUPTI_CB_DOMAIN_FORCE_INT = 2147483647,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010"
))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_CallbackIdResource {
    CUPTI_CBID_RESOURCE_INVALID = 0,
    CUPTI_CBID_RESOURCE_CONTEXT_CREATED = 1,
    CUPTI_CBID_RESOURCE_CONTEXT_DESTROY_STARTING = 2,
    CUPTI_CBID_RESOURCE_STREAM_CREATED = 3,
    CUPTI_CBID_RESOURCE_STREAM_DESTROY_STARTING = 4,
    CUPTI_CBID_RESOURCE_CU_INIT_FINISHED = 5,
    CUPTI_CBID_RESOURCE_MODULE_LOADED = 6,
    CUPTI_CBID_RESOURCE_MODULE_UNLOAD_STARTING = 7,
    CUPTI_CBID_RESOURCE_MODULE_PROFILED = 8,
    CUPTI_CBID_RESOURCE_GRAPH_CREATED = 9,
    CUPTI_CBID_RESOURCE_GRAPH_DESTROY_STARTING = 10,
    CUPTI_CBID_RESOURCE_GRAPH_CLONED = 11,
    CUPTI_CBID_RESOURCE_GRAPHNODE_CREATE_STARTING = 12,
    CUPTI_CBID_RESOURCE_GRAPHNODE_CREATED = 13,
    CUPTI_CBID_RESOURCE_GRAPHNODE_DESTROY_STARTING = 14,
    CUPTI_CBID_RESOURCE_GRAPHNODE_DEPENDENCY_CREATED = 15,
    CUPTI_CBID_RESOURCE_GRAPHNODE_DEPENDENCY_DESTROY_STARTING = 16,
    CUPTI_CBID_RESOURCE_GRAPHEXEC_CREATE_STARTING = 17,
    CUPTI_CBID_RESOURCE_GRAPHEXEC_CREATED = 18,
    CUPTI_CBID_RESOURCE_GRAPHEXEC_DESTROY_STARTING = 19,
    CUPTI_CBID_RESOURCE_GRAPHNODE_CLONED = 20,
    CUPTI_CBID_RESOURCE_SIZE = 21,
    CUPTI_CBID_RESOURCE_FORCE_INT = 2147483647,
}
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_CallbackIdResource {
    CUPTI_CBID_RESOURCE_INVALID = 0,
    CUPTI_CBID_RESOURCE_CONTEXT_CREATED = 1,
    CUPTI_CBID_RESOURCE_CONTEXT_DESTROY_STARTING = 2,
    CUPTI_CBID_RESOURCE_STREAM_CREATED = 3,
    CUPTI_CBID_RESOURCE_STREAM_DESTROY_STARTING = 4,
    CUPTI_CBID_RESOURCE_CU_INIT_FINISHED = 5,
    CUPTI_CBID_RESOURCE_MODULE_LOADED = 6,
    CUPTI_CBID_RESOURCE_MODULE_UNLOAD_STARTING = 7,
    CUPTI_CBID_RESOURCE_MODULE_PROFILED = 8,
    CUPTI_CBID_RESOURCE_GRAPH_CREATED = 9,
    CUPTI_CBID_RESOURCE_GRAPH_DESTROY_STARTING = 10,
    CUPTI_CBID_RESOURCE_GRAPH_CLONED = 11,
    CUPTI_CBID_RESOURCE_GRAPHNODE_CREATE_STARTING = 12,
    CUPTI_CBID_RESOURCE_GRAPHNODE_CREATED = 13,
    CUPTI_CBID_RESOURCE_GRAPHNODE_DESTROY_STARTING = 14,
    CUPTI_CBID_RESOURCE_GRAPHNODE_DEPENDENCY_CREATED = 15,
    CUPTI_CBID_RESOURCE_GRAPHNODE_DEPENDENCY_DESTROY_STARTING = 16,
    CUPTI_CBID_RESOURCE_GRAPHEXEC_CREATE_STARTING = 17,
    CUPTI_CBID_RESOURCE_GRAPHEXEC_CREATED = 18,
    CUPTI_CBID_RESOURCE_GRAPHEXEC_DESTROY_STARTING = 19,
    CUPTI_CBID_RESOURCE_GRAPHNODE_CLONED = 20,
    CUPTI_CBID_RESOURCE_STREAM_ATTRIBUTE_CHANGED = 21,
    CUPTI_CBID_RESOURCE_SIZE = 22,
    CUPTI_CBID_RESOURCE_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_CallbackIdResource {
    CUPTI_CBID_RESOURCE_INVALID = 0,
    CUPTI_CBID_RESOURCE_CONTEXT_CREATED = 1,
    CUPTI_CBID_RESOURCE_CONTEXT_DESTROY_STARTING = 2,
    CUPTI_CBID_RESOURCE_STREAM_CREATED = 3,
    CUPTI_CBID_RESOURCE_STREAM_DESTROY_STARTING = 4,
    CUPTI_CBID_RESOURCE_CU_INIT_FINISHED = 5,
    CUPTI_CBID_RESOURCE_MODULE_LOADED = 6,
    CUPTI_CBID_RESOURCE_MODULE_UNLOAD_STARTING = 7,
    CUPTI_CBID_RESOURCE_MODULE_PROFILED = 8,
    CUPTI_CBID_RESOURCE_GRAPH_CREATED = 9,
    CUPTI_CBID_RESOURCE_GRAPH_DESTROY_STARTING = 10,
    CUPTI_CBID_RESOURCE_GRAPH_CLONED = 11,
    CUPTI_CBID_RESOURCE_GRAPHNODE_CREATE_STARTING = 12,
    CUPTI_CBID_RESOURCE_GRAPHNODE_CREATED = 13,
    CUPTI_CBID_RESOURCE_GRAPHNODE_DESTROY_STARTING = 14,
    CUPTI_CBID_RESOURCE_GRAPHNODE_DEPENDENCY_CREATED = 15,
    CUPTI_CBID_RESOURCE_GRAPHNODE_DEPENDENCY_DESTROY_STARTING = 16,
    CUPTI_CBID_RESOURCE_GRAPHEXEC_CREATE_STARTING = 17,
    CUPTI_CBID_RESOURCE_GRAPHEXEC_CREATED = 18,
    CUPTI_CBID_RESOURCE_GRAPHEXEC_DESTROY_STARTING = 19,
    CUPTI_CBID_RESOURCE_GRAPHNODE_CLONED = 20,
    CUPTI_CBID_RESOURCE_STREAM_ATTRIBUTE_CHANGED = 21,
    CUPTI_CBID_RESOURCE_GRAPH_NODE_UPDATED = 22,
    CUPTI_CBID_RESOURCE_GRAPH_NODE_SET_PARAMS = 23,
    CUPTI_CBID_RESOURCE_SIZE = 24,
    CUPTI_CBID_RESOURCE_FORCE_INT = 2147483647,
}
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_CallbackIdState {
    CUPTI_CBID_STATE_INVALID = 0,
    CUPTI_CBID_STATE_FATAL_ERROR = 1,
    CUPTI_CBID_STATE_ERROR = 2,
    CUPTI_CBID_STATE_WARNING = 3,
    CUPTI_CBID_STATE_SIZE = 4,
    CUPTI_CBID_STATE_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_CallbackIdSync {
    CUPTI_CBID_SYNCHRONIZE_INVALID = 0,
    CUPTI_CBID_SYNCHRONIZE_STREAM_SYNCHRONIZED = 1,
    CUPTI_CBID_SYNCHRONIZE_CONTEXT_SYNCHRONIZED = 2,
    CUPTI_CBID_SYNCHRONIZE_SIZE = 3,
    CUPTI_CBID_SYNCHRONIZE_FORCE_INT = 2147483647,
}
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050"
))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ChannelType {
    CUPTI_CHANNEL_TYPE_INVALID = 0,
    CUPTI_CHANNEL_TYPE_COMPUTE = 1,
    CUPTI_CHANNEL_TYPE_ASYNC_MEMCPY = 2,
}
#[cfg(any(feature = "cuda-12060"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ChannelType {
    CUPTI_CHANNEL_TYPE_INVALID = 0,
    CUPTI_CHANNEL_TYPE_COMPUTE = 1,
    CUPTI_CHANNEL_TYPE_ASYNC_MEMCPY = 2,
    CUPTI_CHANNEL_TYPE_FORCE_INT = 2147483647,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ChannelType {
    CUPTI_CHANNEL_TYPE_INVALID = 0,
    CUPTI_CHANNEL_TYPE_COMPUTE = 1,
    CUPTI_CHANNEL_TYPE_ASYNC_MEMCPY = 2,
    CUPTI_CHANNEL_TYPE_DECOMP = 3,
    CUPTI_CHANNEL_TYPE_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ComputeEngineCtxSwitchOperationType {
    CUPTI_COMPUTE_ENGINE_CTX_SWITCH_OPERATION_INVALID = 0,
    CUPTI_COMPUTE_ENGINE_CTX_SWITCH_OPERATION_START = 1,
    CUPTI_COMPUTE_ENGINE_CTX_SWITCH_OPERATION_END = 2,
    CUPTI_COMPUTE_ENGINE_CTX_SWITCH_OPERATION_COUNT = 2147483647,
}
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ConfidentialComputeRotationEventType {
    CUPTI_CONFIDENTIAL_COMPUTE_INVALID_ROTATION_EVENT = 0,
    CUPTI_CONFIDENTIAL_COMPUTE_KEY_ROTATION_CHANNEL_BLOCKED = 1,
    CUPTI_CONFIDENTIAL_COMPUTE_KEY_ROTATION_CHANNEL_DRAINED = 2,
    CUPTI_CONFIDENTIAL_COMPUTE_KEY_ROTATION_CHANNEL_UNBLOCKED = 3,
    CUPTI_CONFIDENTIAL_COMPUTE_EVENT_TYPE_FORCE_INT = 2147483647,
}
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ContextCigMode {
    CUPTI_CONTEXT_CIG_MODE_NONE = 0,
    CUPTI_CONTEXT_CIG_MODE_CIG = 1,
    CUPTI_CONTEXT_CIG_MODE_CIG_FALLBACK = 2,
    CUPTI_CONTEXT_CIG_MODE_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_DevType {
    CUPTI_DEV_TYPE_INVALID = 0,
    CUPTI_DEV_TYPE_GPU = 1,
    CUPTI_DEV_TYPE_NPU = 2,
    CUPTI_DEV_TYPE_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_DeviceAttribute {
    CUPTI_DEVICE_ATTR_MAX_EVENT_ID = 1,
    CUPTI_DEVICE_ATTR_MAX_EVENT_DOMAIN_ID = 2,
    CUPTI_DEVICE_ATTR_GLOBAL_MEMORY_BANDWIDTH = 3,
    CUPTI_DEVICE_ATTR_INSTRUCTION_PER_CYCLE = 4,
    CUPTI_DEVICE_ATTR_INSTRUCTION_THROUGHPUT_SINGLE_PRECISION = 5,
    CUPTI_DEVICE_ATTR_MAX_FRAME_BUFFERS = 6,
    CUPTI_DEVICE_ATTR_PCIE_LINK_RATE = 7,
    CUPTI_DEVICE_ATTR_PCIE_LINK_WIDTH = 8,
    CUPTI_DEVICE_ATTR_PCIE_GEN = 9,
    CUPTI_DEVICE_ATTR_DEVICE_CLASS = 10,
    CUPTI_DEVICE_ATTR_FLOP_SP_PER_CYCLE = 11,
    CUPTI_DEVICE_ATTR_FLOP_DP_PER_CYCLE = 12,
    CUPTI_DEVICE_ATTR_MAX_L2_UNITS = 13,
    CUPTI_DEVICE_ATTR_MAX_SHARED_MEMORY_CACHE_CONFIG_PREFER_SHARED = 14,
    CUPTI_DEVICE_ATTR_MAX_SHARED_MEMORY_CACHE_CONFIG_PREFER_L1 = 15,
    CUPTI_DEVICE_ATTR_MAX_SHARED_MEMORY_CACHE_CONFIG_PREFER_EQUAL = 16,
    CUPTI_DEVICE_ATTR_FLOP_HP_PER_CYCLE = 17,
    CUPTI_DEVICE_ATTR_NVLINK_PRESENT = 18,
    CUPTI_DEVICE_ATTR_GPU_CPU_NVLINK_BW = 19,
    CUPTI_DEVICE_ATTR_NVSWITCH_PRESENT = 20,
    CUPTI_DEVICE_ATTR_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_DeviceAttributeDeviceClass {
    CUPTI_DEVICE_ATTR_DEVICE_CLASS_TESLA = 0,
    CUPTI_DEVICE_ATTR_DEVICE_CLASS_QUADRO = 1,
    CUPTI_DEVICE_ATTR_DEVICE_CLASS_GEFORCE = 2,
    CUPTI_DEVICE_ATTR_DEVICE_CLASS_TEGRA = 3,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_DeviceGraphLaunchMode {
    CUPTI_DEVICE_GRAPH_LAUNCH_MODE_INVALID = 0,
    CUPTI_DEVICE_GRAPH_LAUNCH_MODE_FIRE_AND_FORGET = 1,
    CUPTI_DEVICE_GRAPH_LAUNCH_MODE_TAIL = 2,
    CUPTI_DEVICE_GRAPH_LAUNCH_MODE_FIRE_AND_FORGET_AS_SIBLING = 3,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_DeviceVirtualizationMode {
    CUPTI_DEVICE_VIRTUALIZATION_MODE_NONE = 0,
    CUPTI_DEVICE_VIRTUALIZATION_MODE_PASS_THROUGH = 1,
    CUPTI_DEVICE_VIRTUALIZATION_MODE_VIRTUAL_GPU = 2,
    CUPTI_DEVICE_VIRTUALIZATION_MODE_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_EnvironmentClocksThrottleReason {
    CUPTI_CLOCKS_THROTTLE_REASON_GPU_IDLE = 1,
    CUPTI_CLOCKS_THROTTLE_REASON_USER_DEFINED_CLOCKS = 2,
    CUPTI_CLOCKS_THROTTLE_REASON_SW_POWER_CAP = 4,
    CUPTI_CLOCKS_THROTTLE_REASON_HW_SLOWDOWN = 8,
    CUPTI_CLOCKS_THROTTLE_REASON_UNKNOWN = 2147483648,
    CUPTI_CLOCKS_THROTTLE_REASON_UNSUPPORTED = 1073741824,
    CUPTI_CLOCKS_THROTTLE_REASON_NONE = 0,
    CUPTI_CLOCKS_THROTTLE_REASON_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_EventAttribute {
    CUPTI_EVENT_ATTR_NAME = 0,
    CUPTI_EVENT_ATTR_SHORT_DESCRIPTION = 1,
    CUPTI_EVENT_ATTR_LONG_DESCRIPTION = 2,
    CUPTI_EVENT_ATTR_CATEGORY = 3,
    CUPTI_EVENT_ATTR_PROFILING_SCOPE = 5,
    CUPTI_EVENT_ATTR_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_EventCategory {
    CUPTI_EVENT_CATEGORY_INSTRUCTION = 0,
    CUPTI_EVENT_CATEGORY_MEMORY = 1,
    CUPTI_EVENT_CATEGORY_CACHE = 2,
    CUPTI_EVENT_CATEGORY_PROFILE_TRIGGER = 3,
    CUPTI_EVENT_CATEGORY_SYSTEM = 4,
    CUPTI_EVENT_CATEGORY_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_EventCollectionMethod {
    CUPTI_EVENT_COLLECTION_METHOD_PM = 0,
    CUPTI_EVENT_COLLECTION_METHOD_SM = 1,
    CUPTI_EVENT_COLLECTION_METHOD_INSTRUMENTED = 2,
    CUPTI_EVENT_COLLECTION_METHOD_NVLINK_TC = 3,
    CUPTI_EVENT_COLLECTION_METHOD_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_EventCollectionMode {
    CUPTI_EVENT_COLLECTION_MODE_CONTINUOUS = 0,
    CUPTI_EVENT_COLLECTION_MODE_KERNEL = 1,
    CUPTI_EVENT_COLLECTION_MODE_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_EventDomainAttribute {
    CUPTI_EVENT_DOMAIN_ATTR_NAME = 0,
    CUPTI_EVENT_DOMAIN_ATTR_INSTANCE_COUNT = 1,
    CUPTI_EVENT_DOMAIN_ATTR_TOTAL_INSTANCE_COUNT = 3,
    CUPTI_EVENT_DOMAIN_ATTR_COLLECTION_METHOD = 4,
    CUPTI_EVENT_DOMAIN_ATTR_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_EventGroupAttribute {
    CUPTI_EVENT_GROUP_ATTR_EVENT_DOMAIN_ID = 0,
    CUPTI_EVENT_GROUP_ATTR_PROFILE_ALL_DOMAIN_INSTANCES = 1,
    CUPTI_EVENT_GROUP_ATTR_USER_DATA = 2,
    CUPTI_EVENT_GROUP_ATTR_NUM_EVENTS = 3,
    CUPTI_EVENT_GROUP_ATTR_EVENTS = 4,
    CUPTI_EVENT_GROUP_ATTR_INSTANCE_COUNT = 5,
    CUPTI_EVENT_GROUP_ATTR_PROFILING_SCOPE = 6,
    CUPTI_EVENT_GROUP_ATTR_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_EventProfilingScope {
    CUPTI_EVENT_PROFILING_SCOPE_CONTEXT = 0,
    CUPTI_EVENT_PROFILING_SCOPE_DEVICE = 1,
    CUPTI_EVENT_PROFILING_SCOPE_BOTH = 2,
    CUPTI_EVENT_PROFILING_SCOPE_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ExternalCorrelationKind {
    CUPTI_EXTERNAL_CORRELATION_KIND_INVALID = 0,
    CUPTI_EXTERNAL_CORRELATION_KIND_UNKNOWN = 1,
    CUPTI_EXTERNAL_CORRELATION_KIND_OPENACC = 2,
    CUPTI_EXTERNAL_CORRELATION_KIND_CUSTOM0 = 3,
    CUPTI_EXTERNAL_CORRELATION_KIND_CUSTOM1 = 4,
    CUPTI_EXTERNAL_CORRELATION_KIND_CUSTOM2 = 5,
    CUPTI_EXTERNAL_CORRELATION_KIND_SIZE = 6,
    CUPTI_EXTERNAL_CORRELATION_KIND_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_FuncShmemLimitConfig {
    CUPTI_FUNC_SHMEM_LIMIT_DEFAULT = 0,
    CUPTI_FUNC_SHMEM_LIMIT_OPTIN = 1,
    CUPTI_FUNC_SHMEM_LIMIT_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_LinkFlag {
    CUPTI_LINK_FLAG_INVALID = 0,
    CUPTI_LINK_FLAG_PEER_ACCESS = 2,
    CUPTI_LINK_FLAG_SYSMEM_ACCESS = 4,
    CUPTI_LINK_FLAG_PEER_ATOMICS = 8,
    CUPTI_LINK_FLAG_SYSMEM_ATOMICS = 16,
    CUPTI_LINK_FLAG_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_MetricAttribute {
    CUPTI_METRIC_ATTR_NAME = 0,
    CUPTI_METRIC_ATTR_SHORT_DESCRIPTION = 1,
    CUPTI_METRIC_ATTR_LONG_DESCRIPTION = 2,
    CUPTI_METRIC_ATTR_CATEGORY = 3,
    CUPTI_METRIC_ATTR_VALUE_KIND = 4,
    CUPTI_METRIC_ATTR_EVALUATION_MODE = 5,
    CUPTI_METRIC_ATTR_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_MetricCategory {
    CUPTI_METRIC_CATEGORY_MEMORY = 0,
    CUPTI_METRIC_CATEGORY_INSTRUCTION = 1,
    CUPTI_METRIC_CATEGORY_MULTIPROCESSOR = 2,
    CUPTI_METRIC_CATEGORY_CACHE = 3,
    CUPTI_METRIC_CATEGORY_TEXTURE = 4,
    CUPTI_METRIC_CATEGORY_NVLINK = 5,
    CUPTI_METRIC_CATEGORY_PCIE = 6,
    CUPTI_METRIC_CATEGORY_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_MetricEvaluationMode {
    CUPTI_METRIC_EVALUATION_MODE_PER_INSTANCE = 1,
    CUPTI_METRIC_EVALUATION_MODE_AGGREGATE = 2,
    CUPTI_METRIC_EVALUATION_MODE_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_MetricPropertyDeviceClass {
    CUPTI_METRIC_PROPERTY_DEVICE_CLASS_TESLA = 0,
    CUPTI_METRIC_PROPERTY_DEVICE_CLASS_QUADRO = 1,
    CUPTI_METRIC_PROPERTY_DEVICE_CLASS_GEFORCE = 2,
    CUPTI_METRIC_PROPERTY_DEVICE_CLASS_TEGRA = 3,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_MetricPropertyID {
    CUPTI_METRIC_PROPERTY_MULTIPROCESSOR_COUNT = 0,
    CUPTI_METRIC_PROPERTY_WARPS_PER_MULTIPROCESSOR = 1,
    CUPTI_METRIC_PROPERTY_KERNEL_GPU_TIME = 2,
    CUPTI_METRIC_PROPERTY_CLOCK_RATE = 3,
    CUPTI_METRIC_PROPERTY_FRAME_BUFFER_COUNT = 4,
    CUPTI_METRIC_PROPERTY_GLOBAL_MEMORY_BANDWIDTH = 5,
    CUPTI_METRIC_PROPERTY_PCIE_LINK_RATE = 6,
    CUPTI_METRIC_PROPERTY_PCIE_LINK_WIDTH = 7,
    CUPTI_METRIC_PROPERTY_PCIE_GEN = 8,
    CUPTI_METRIC_PROPERTY_DEVICE_CLASS = 9,
    CUPTI_METRIC_PROPERTY_FLOP_SP_PER_CYCLE = 10,
    CUPTI_METRIC_PROPERTY_FLOP_DP_PER_CYCLE = 11,
    CUPTI_METRIC_PROPERTY_L2_UNITS = 12,
    CUPTI_METRIC_PROPERTY_ECC_ENABLED = 13,
    CUPTI_METRIC_PROPERTY_FLOP_HP_PER_CYCLE = 14,
    CUPTI_METRIC_PROPERTY_GPU_CPU_NVLINK_BANDWIDTH = 15,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_MetricValueKind {
    CUPTI_METRIC_VALUE_KIND_DOUBLE = 0,
    CUPTI_METRIC_VALUE_KIND_UINT64 = 1,
    CUPTI_METRIC_VALUE_KIND_PERCENT = 2,
    CUPTI_METRIC_VALUE_KIND_THROUGHPUT = 3,
    CUPTI_METRIC_VALUE_KIND_INT64 = 4,
    CUPTI_METRIC_VALUE_KIND_UTILIZATION_LEVEL = 5,
    CUPTI_METRIC_VALUE_KIND_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_MetricValueKind {
    CUPTI_METRIC_VALUE_KIND_DOUBLE = 0,
    CUPTI_METRIC_VALUE_KIND_UINT64 = 1,
    CUPTI_METRIC_VALUE_KIND_PERCENT = 2,
    CUPTI_METRIC_VALUE_KIND_THROUGHPUT = 3,
    CUPTI_METRIC_VALUE_KIND_INT64 = 4,
    CUPTI_METRIC_VALUE_KIND_UTILIZATION_LEVEL = 5,
    CUPTI_METRIC_VALUE_KIND_NVTX_EXTENDED_PAYLOAD = 6,
    CUPTI_METRIC_VALUE_KIND_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_MetricValueUtilizationLevel {
    CUPTI_METRIC_VALUE_UTILIZATION_IDLE = 0,
    CUPTI_METRIC_VALUE_UTILIZATION_LOW = 2,
    CUPTI_METRIC_VALUE_UTILIZATION_MID = 5,
    CUPTI_METRIC_VALUE_UTILIZATION_HIGH = 8,
    CUPTI_METRIC_VALUE_UTILIZATION_MAX = 10,
    CUPTI_METRIC_VALUE_UTILIZATION_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_NvtxExtPayloadType {
    CUPTI_NVTX_EXT_PAYLOAD_TYPE_UNKNOWN = 0,
    CUPTI_NVTX_EXT_PAYLOAD_TYPE_SCHEMA = 1,
    CUPTI_NVTX_EXT_PAYLOAD_TYPE_ENUM = 2,
    CUPTI_NVTX_EXT_PAYLOAD_TYPE_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_OpenAccConstructKind {
    CUPTI_OPENACC_CONSTRUCT_KIND_UNKNOWN = 0,
    CUPTI_OPENACC_CONSTRUCT_KIND_PARALLEL = 1,
    CUPTI_OPENACC_CONSTRUCT_KIND_KERNELS = 2,
    CUPTI_OPENACC_CONSTRUCT_KIND_LOOP = 3,
    CUPTI_OPENACC_CONSTRUCT_KIND_DATA = 4,
    CUPTI_OPENACC_CONSTRUCT_KIND_ENTER_DATA = 5,
    CUPTI_OPENACC_CONSTRUCT_KIND_EXIT_DATA = 6,
    CUPTI_OPENACC_CONSTRUCT_KIND_HOST_DATA = 7,
    CUPTI_OPENACC_CONSTRUCT_KIND_ATOMIC = 8,
    CUPTI_OPENACC_CONSTRUCT_KIND_DECLARE = 9,
    CUPTI_OPENACC_CONSTRUCT_KIND_INIT = 10,
    CUPTI_OPENACC_CONSTRUCT_KIND_SHUTDOWN = 11,
    CUPTI_OPENACC_CONSTRUCT_KIND_SET = 12,
    CUPTI_OPENACC_CONSTRUCT_KIND_UPDATE = 13,
    CUPTI_OPENACC_CONSTRUCT_KIND_ROUTINE = 14,
    CUPTI_OPENACC_CONSTRUCT_KIND_WAIT = 15,
    CUPTI_OPENACC_CONSTRUCT_KIND_RUNTIME_API = 16,
    CUPTI_OPENACC_CONSTRUCT_KIND_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_OpenAccEventKind {
    CUPTI_OPENACC_EVENT_KIND_INVALID = 0,
    CUPTI_OPENACC_EVENT_KIND_DEVICE_INIT = 1,
    CUPTI_OPENACC_EVENT_KIND_DEVICE_SHUTDOWN = 2,
    CUPTI_OPENACC_EVENT_KIND_RUNTIME_SHUTDOWN = 3,
    CUPTI_OPENACC_EVENT_KIND_ENQUEUE_LAUNCH = 4,
    CUPTI_OPENACC_EVENT_KIND_ENQUEUE_UPLOAD = 5,
    CUPTI_OPENACC_EVENT_KIND_ENQUEUE_DOWNLOAD = 6,
    CUPTI_OPENACC_EVENT_KIND_WAIT = 7,
    CUPTI_OPENACC_EVENT_KIND_IMPLICIT_WAIT = 8,
    CUPTI_OPENACC_EVENT_KIND_COMPUTE_CONSTRUCT = 9,
    CUPTI_OPENACC_EVENT_KIND_UPDATE = 10,
    CUPTI_OPENACC_EVENT_KIND_ENTER_DATA = 11,
    CUPTI_OPENACC_EVENT_KIND_EXIT_DATA = 12,
    CUPTI_OPENACC_EVENT_KIND_CREATE = 13,
    CUPTI_OPENACC_EVENT_KIND_DELETE = 14,
    CUPTI_OPENACC_EVENT_KIND_ALLOC = 15,
    CUPTI_OPENACC_EVENT_KIND_FREE = 16,
    CUPTI_OPENACC_EVENT_KIND_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_OpenMpEventKind {
    CUPTI_OPENMP_EVENT_KIND_INVALID = 0,
    CUPTI_OPENMP_EVENT_KIND_PARALLEL = 1,
    CUPTI_OPENMP_EVENT_KIND_TASK = 2,
    CUPTI_OPENMP_EVENT_KIND_THREAD = 3,
    CUPTI_OPENMP_EVENT_KIND_IDLE = 4,
    CUPTI_OPENMP_EVENT_KIND_WAIT_BARRIER = 5,
    CUPTI_OPENMP_EVENT_KIND_WAIT_TASKWAIT = 6,
    CUPTI_OPENMP_EVENT_KIND_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_PcieDeviceType {
    CUPTI_PCIE_DEVICE_TYPE_GPU = 0,
    CUPTI_PCIE_DEVICE_TYPE_BRIDGE = 1,
    CUPTI_PCIE_DEVICE_TYPE_FORCE_INT = 2147483647,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060"
))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_PcieGen {
    CUPTI_PCIE_GEN_GEN1 = 1,
    CUPTI_PCIE_GEN_GEN2 = 2,
    CUPTI_PCIE_GEN_GEN3 = 3,
    CUPTI_PCIE_GEN_GEN4 = 4,
    CUPTI_PCIE_GEN_GEN5 = 5,
    CUPTI_PCIE_GEN_FORCE_INT = 2147483647,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_PcieGen {
    CUPTI_PCIE_GEN_GEN1 = 1,
    CUPTI_PCIE_GEN_GEN2 = 2,
    CUPTI_PCIE_GEN_GEN3 = 3,
    CUPTI_PCIE_GEN_GEN4 = 4,
    CUPTI_PCIE_GEN_GEN5 = 5,
    CUPTI_PCIE_GEN_GEN6 = 6,
    CUPTI_PCIE_GEN_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_ReadEventFlags {
    CUPTI_EVENT_READ_FLAG_NONE = 0,
    CUPTI_EVENT_READ_FLAG_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-11040", feature = "cuda-11050"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_driver_api_trace_cbid_enum {
    CUPTI_DRIVER_TRACE_CBID_INVALID = 0,
    CUPTI_DRIVER_TRACE_CBID_cuInit = 1,
    CUPTI_DRIVER_TRACE_CBID_cuDriverGetVersion = 2,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGet = 3,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetCount = 4,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetName = 5,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceComputeCapability = 6,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem = 7,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetProperties = 8,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetAttribute = 9,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate = 10,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy = 11,
    CUPTI_DRIVER_TRACE_CBID_cuCtxAttach = 12,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDetach = 13,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent = 14,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent = 15,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetDevice = 16,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSynchronize = 17,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoad = 18,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadData = 19,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadDataEx = 20,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadFatBinary = 21,
    CUPTI_DRIVER_TRACE_CBID_cuModuleUnload = 22,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetFunction = 23,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal = 24,
    CUPTI_DRIVER_TRACE_CBID_cu64ModuleGetGlobal = 25,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetTexRef = 26,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo = 27,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetInfo = 28,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc = 29,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAlloc = 30,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch = 31,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAllocPitch = 32,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree = 33,
    CUPTI_DRIVER_TRACE_CBID_cu64MemFree = 34,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange = 35,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetAddressRange = 36,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost = 37,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeHost = 38,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc = 39,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer = 40,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostGetDevicePointer = 41,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetFlags = 42,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD = 43,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoD = 44,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH = 45,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoH = 46,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD = 47,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoD = 48,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA = 49,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoA = 50,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD = 51,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyAtoD = 52,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA = 53,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH = 54,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA = 55,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D = 56,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned = 57,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D = 58,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3D = 59,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync = 60,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoDAsync = 61,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync = 62,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoHAsync = 63,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync = 64,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoDAsync = 65,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync = 66,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync = 67,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync = 68,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync = 69,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3DAsync = 70,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8 = 71,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8 = 72,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16 = 73,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16 = 74,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32 = 75,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32 = 76,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8 = 77,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8 = 78,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16 = 79,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16 = 80,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32 = 81,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32 = 82,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetBlockShape = 83,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedSize = 84,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetAttribute = 85,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetCacheConfig = 86,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate = 87,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor = 88,
    CUPTI_DRIVER_TRACE_CBID_cuArrayDestroy = 89,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate = 90,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor = 91,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefCreate = 92,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefDestroy = 93,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetArray = 94,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress = 95,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress = 96,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D = 97,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress2D = 98,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFormat = 99,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddressMode = 100,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFilterMode = 101,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFlags = 102,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress = 103,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefGetAddress = 104,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetArray = 105,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddressMode = 106,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFilterMode = 107,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFormat = 108,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFlags = 109,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetSize = 110,
    CUPTI_DRIVER_TRACE_CBID_cuParamSeti = 111,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetf = 112,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetv = 113,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetTexRef = 114,
    CUPTI_DRIVER_TRACE_CBID_cuLaunch = 115,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGrid = 116,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGridAsync = 117,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreate = 118,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord = 119,
    CUPTI_DRIVER_TRACE_CBID_cuEventQuery = 120,
    CUPTI_DRIVER_TRACE_CBID_cuEventSynchronize = 121,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy = 122,
    CUPTI_DRIVER_TRACE_CBID_cuEventElapsedTime = 123,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreate = 124,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery = 125,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize = 126,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy = 127,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnregisterResource = 128,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsSubResourceGetMappedArray = 129,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer = 130,
    CUPTI_DRIVER_TRACE_CBID_cu64GraphicsResourceGetMappedPointer = 131,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags = 132,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources = 133,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources = 134,
    CUPTI_DRIVER_TRACE_CBID_cuGetExportTable = 135,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetLimit = 136,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetLimit = 137,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevice = 138,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate = 139,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D10RegisterResource = 140,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10RegisterResource = 141,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnregisterResource = 142,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10MapResources = 143,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnmapResources = 144,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceSetMapFlags = 145,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedArray = 146,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer = 147,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize = 148,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch = 149,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions = 150,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevice = 151,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate = 152,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D11RegisterResource = 153,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevice = 154,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate = 155,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D9RegisterResource = 156,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDirect3DDevice = 157,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterResource = 158,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterResource = 159,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapResources = 160,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapResources = 161,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceSetMapFlags = 162,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions = 163,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedArray = 164,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer = 165,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize = 166,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch = 167,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9Begin = 168,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9End = 169,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterVertexBuffer = 170,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer = 171,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapVertexBuffer = 172,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterVertexBuffer = 173,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate = 174,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterBuffer = 175,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterImage = 176,
    CUPTI_DRIVER_TRACE_CBID_cuWGLGetDevice = 177,
    CUPTI_DRIVER_TRACE_CBID_cuGLInit = 178,
    CUPTI_DRIVER_TRACE_CBID_cuGLRegisterBufferObject = 179,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject = 180,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObject = 181,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnregisterBufferObject = 182,
    CUPTI_DRIVER_TRACE_CBID_cuGLSetBufferObjectMapFlags = 183,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync = 184,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObjectAsync = 185,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUGetDevice = 186,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate = 187,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterVideoSurface = 188,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterOutputSurface = 189,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetSurfRef = 190,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefCreate = 191,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefDestroy = 192,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetFormat = 193,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetArray = 194,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetFormat = 195,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetArray = 196,
    CUPTI_DRIVER_TRACE_CBID_cu64DeviceTotalMem = 197,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPointer = 198,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedSize = 199,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPitch = 200,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetSurfaceDimensions = 201,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetSurfaceDimensions = 202,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPointer = 203,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedSize = 204,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPitch = 205,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9MapVertexBuffer = 206,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObject = 207,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObjectAsync = 208,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevices = 209,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreateOnDevice = 210,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevices = 211,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreateOnDevice = 212,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevices = 213,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreateOnDevice = 214,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostAlloc = 215,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async = 216,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8Async = 217,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async = 218,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16Async = 219,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async = 220,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32Async = 221,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async = 222,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8Async = 223,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async = 224,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16Async = 225,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async = 226,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32Async = 227,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayCreate = 228,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayGetDescriptor = 229,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DCreate = 230,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DGetDescriptor = 231,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2D = 232,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DUnaligned = 233,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DAsync = 234,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v2 = 235,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate_v2 = 236,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate_v2 = 237,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate_v2 = 238,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate_v2 = 239,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate_v2 = 240,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal_v2 = 241,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo_v2 = 242,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc_v2 = 243,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch_v2 = 244,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree_v2 = 245,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange_v2 = 246,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer_v2 = 247,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_v2 = 248,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2 = 249,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2 = 250,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2 = 251,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2 = 252,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2 = 253,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2 = 254,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress_v2 = 255,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v2 = 256,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress_v2 = 257,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer_v2 = 258,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem_v2 = 259,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer_v2 = 260,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize_v2 = 261,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch_v2 = 262,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions_v2 = 263,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions_v2 = 264,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer_v2 = 265,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize_v2 = 266,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch_v2 = 267,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer_v2 = 268,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2 = 269,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2 = 270,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc_v2 = 271,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate_v2 = 272,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor_v2 = 273,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate_v2 = 274,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor_v2 = 275,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2 = 276,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2 = 277,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2 = 278,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2 = 279,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2 = 280,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2 = 281,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2 = 282,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2 = 283,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2 = 284,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2 = 285,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2 = 286,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2 = 287,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2 = 288,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2 = 289,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2 = 290,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2 = 291,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2 = 292,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2 = 293,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost_v2 = 294,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent = 295,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetApiVersion = 296,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDirect3DDevice = 297,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDirect3DDevice = 298,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCacheConfig = 299,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCacheConfig = 300,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister = 301,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostUnregister = 302,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCurrent = 303,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCurrent = 304,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy = 305,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync = 306,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel = 307,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStart = 308,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStop = 309,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttribute = 310,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerInitialize = 311,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceCanAccessPeer = 312,
    CUPTI_DRIVER_TRACE_CBID_cuCtxEnablePeerAccess = 313,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDisablePeerAccess = 314,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerRegister = 315,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerUnregister = 316,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerGetDevicePointer = 317,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer = 318,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync = 319,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer = 320,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync = 321,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy_v2 = 322,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent_v2 = 323,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent_v2 = 324,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy_v2 = 325,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy_v2 = 326,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v3 = 327,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetMemHandle = 328,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle = 329,
    CUPTI_DRIVER_TRACE_CBID_cuIpcCloseMemHandle = 330,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetByPCIBusId = 331,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetPCIBusId = 332,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices = 333,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetEventHandle = 334,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenEventHandle = 335,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetSharedMemConfig = 336,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetSharedMemConfig = 337,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedMemConfig = 338,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectCreate = 339,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectDestroy = 340,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceDesc = 341,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetTextureDesc = 342,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectCreate = 343,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectDestroy = 344,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectGetResourceDesc = 345,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback = 346,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayCreate = 347,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetLevel = 348,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayDestroy = 349,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmappedArray = 350,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapFilterMode = 351,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelBias = 352,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelClamp = 353,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMaxAnisotropy = 354,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmappedArray = 355,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapFilterMode = 356,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelBias = 357,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelClamp = 358,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMaxAnisotropy = 359,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedMipmappedArray = 360,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceViewDesc = 361,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate = 362,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData = 363,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile = 364,
    CUPTI_DRIVER_TRACE_CBID_cuLinkComplete = 365,
    CUPTI_DRIVER_TRACE_CBID_cuLinkDestroy = 366,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreateWithPriority = 367,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority = 368,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags = 369,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetStreamPriorityRange = 370,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocManaged = 371,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorString = 372,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorName = 373,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessor = 374,
    CUPTI_DRIVER_TRACE_CBID_cuCompilePtx = 375,
    CUPTI_DRIVER_TRACE_CBID_cuBinaryFree = 376,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync = 377,
    CUPTI_DRIVER_TRACE_CBID_cuPointerSetAttribute = 378,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister_v2 = 379,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags_v2 = 380,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate_v2 = 381,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData_v2 = 382,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile_v2 = 383,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSize = 384,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices_v2 = 385,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRetain = 386,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease = 387,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags = 388,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset = 389,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsEGLRegisterImage = 390,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetFlags = 391,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxGetState = 392,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnect = 393,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerDisconnect = 394,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerAcquireFrame = 395,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerReleaseFrame = 396,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2_ptds = 397,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2_ptds = 398,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2_ptds = 399,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2_ptds = 400,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2_ptds = 401,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2_ptds = 402,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2_ptds = 403,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2_ptds = 404,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2_ptds = 405,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2_ptds = 406,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2_ptds = 407,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_ptds = 408,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer_ptds = 409,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer_ptds = 410,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2_ptds = 411,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2_ptds = 412,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2_ptds = 413,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2_ptds = 414,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2_ptds = 415,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2_ptds = 416,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2_ptds = 417,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync_ptsz = 418,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2_ptsz = 419,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2_ptsz = 420,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2_ptsz = 421,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2_ptsz = 422,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2_ptsz = 423,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2_ptsz = 424,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2_ptsz = 425,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync_ptsz = 426,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync_ptsz = 427,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async_ptsz = 428,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async_ptsz = 429,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async_ptsz = 430,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async_ptsz = 431,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async_ptsz = 432,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async_ptsz = 433,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority_ptsz = 434,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags_ptsz = 435,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent_ptsz = 436,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback_ptsz = 437,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync_ptsz = 438,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery_ptsz = 439,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize_ptsz = 440,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord_ptsz = 441,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel_ptsz = 442,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources_ptsz = 443,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources_ptsz = 444,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2_ptsz = 445,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerConnect = 446,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerDisconnect = 447,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerPresentFrame = 448,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedEglFrame = 449,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttributes = 450,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessorWithFlags = 451,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSizeWithFlags = 452,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerReturnFrame = 453,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetP2PAttribute = 454,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetBorderColor = 455,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetBorderColor = 456,
    CUPTI_DRIVER_TRACE_CBID_cuMemAdvise = 457,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32 = 458,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_ptsz = 459,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32 = 460,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_ptsz = 461,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp = 462,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_ptsz = 463,
    CUPTI_DRIVER_TRACE_CBID_cuNVNbufferGetPointer = 464,
    CUPTI_DRIVER_TRACE_CBID_cuNVNtextureGetArray = 465,
    CUPTI_DRIVER_TRACE_CBID_cuNNSetAllocator = 466,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync = 467,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_ptsz = 468,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromNVNSync = 469,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnectWithFlags = 470,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttribute = 471,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttributes = 472,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64 = 473,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_ptsz = 474,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64 = 475,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_ptsz = 476,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel = 477,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel_ptsz = 478,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromEGLSync = 479,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernelMultiDevice = 480,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetAttribute = 481,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid = 482,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx = 483,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_ptsz = 484,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalMemory = 485,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedBuffer = 486,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedMipmappedArray = 487,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalMemory = 488,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalSemaphore = 489,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync = 490,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync_ptsz = 491,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync = 492,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync_ptsz = 493,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalSemaphore = 494,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture = 495,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_ptsz = 496,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture = 497,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture_ptsz = 498,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing = 499,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing_ptsz = 500,
    CUPTI_DRIVER_TRACE_CBID_cuGraphCreate = 501,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddKernelNode = 502,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetParams = 503,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemcpyNode = 504,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeGetParams = 505,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemsetNode = 506,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeGetParams = 507,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeSetParams = 508,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetType = 509,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetRootNodes = 510,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependencies = 511,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependentNodes = 512,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate = 513,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch = 514,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch_ptsz = 515,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecDestroy = 516,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroy = 517,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddDependencies = 518,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRemoveDependencies = 519,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeSetParams = 520,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetParams = 521,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroyNode = 522,
    CUPTI_DRIVER_TRACE_CBID_cuGraphClone = 523,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeFindInClone = 524,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddChildGraphNode = 525,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEmptyNode = 526,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc = 527,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc_ptsz = 528,
    CUPTI_DRIVER_TRACE_CBID_cuGraphChildGraphNodeGetGraph = 529,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddHostNode = 530,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeGetParams = 531,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetLuid = 532,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeSetParams = 533,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetNodes = 534,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetEdges = 535,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo = 536,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_ptsz = 537,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecKernelNodeSetParams = 538,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2 = 539,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2_ptsz = 540,
    CUPTI_DRIVER_TRACE_CBID_cuThreadExchangeStreamCaptureMode = 541,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetNvSciSyncAttributes = 542,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyAvailableDynamicSMemPerBlock = 543,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease_v2 = 544,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset_v2 = 545,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags_v2 = 546,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressReserve = 547,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressFree = 548,
    CUPTI_DRIVER_TRACE_CBID_cuMemCreate = 549,
    CUPTI_DRIVER_TRACE_CBID_cuMemRelease = 550,
    CUPTI_DRIVER_TRACE_CBID_cuMemMap = 551,
    CUPTI_DRIVER_TRACE_CBID_cuMemUnmap = 552,
    CUPTI_DRIVER_TRACE_CBID_cuMemSetAccess = 553,
    CUPTI_DRIVER_TRACE_CBID_cuMemExportToShareableHandle = 554,
    CUPTI_DRIVER_TRACE_CBID_cuMemImportFromShareableHandle = 555,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationGranularity = 556,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationPropertiesFromHandle = 557,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAccess = 558,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags = 559,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags_ptsz = 560,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecUpdate = 561,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemcpyNodeSetParams = 562,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemsetNodeSetParams = 563,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecHostNodeSetParams = 564,
    CUPTI_DRIVER_TRACE_CBID_cuMemRetainAllocationHandle = 565,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetModule = 566,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle_v2 = 567,
    CUPTI_DRIVER_TRACE_CBID_cuCtxResetPersistingL2Cache = 568,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeCopyAttributes = 569,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetAttribute = 570,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetAttribute = 571,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes = 572,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes_ptsz = 573,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute = 574,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute_ptsz = 575,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute = 576,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute_ptsz = 577,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate_v2 = 578,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetTexture1DLinearMaxWidth = 579,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload = 580,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload_ptsz = 581,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetSparseProperties = 582,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetSparseProperties = 583,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync = 584,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync_ptsz = 585,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecChildGraphNodeSetParams = 586,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags = 587,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags_ptsz = 588,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventRecordNode = 589,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventWaitNode = 590,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeGetEvent = 591,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeGetEvent = 592,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeSetEvent = 593,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeSetEvent = 594,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventRecordNodeSetEvent = 595,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventWaitNodeSetEvent = 596,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetPlane = 597,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync = 598,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync_ptsz = 599,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync = 600,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync_ptsz = 601,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolTrimTo = 602,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAttribute = 603,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAttribute = 604,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAccess = 605,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetDefaultMemPool = 606,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolCreate = 607,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolDestroy = 608,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetMemPool = 609,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetMemPool = 610,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync = 611,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync_ptsz = 612,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportToShareableHandle = 613,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportFromShareableHandle = 614,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportPointer = 615,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportPointer = 616,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAccess = 617,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresSignalNode = 618,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeGetParams = 619,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeSetParams = 620,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresWaitNode = 621,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeGetParams = 622,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeSetParams = 623,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresSignalNodeSetParams = 624,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresWaitNodeSetParams = 625,
    CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress = 626,
    CUPTI_DRIVER_TRACE_CBID_cuFlushGPUDirectRDMAWrites = 627,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDebugDotPrint = 628,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2 = 629,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2_ptsz = 630,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies = 631,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_ptsz = 632,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectCreate = 633,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRetain = 634,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRelease = 635,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRetainUserObject = 636,
    CUPTI_DRIVER_TRACE_CBID_cuGraphReleaseUserObject = 637,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemAllocNode = 638,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemFreeNode = 639,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGraphMemTrim = 640,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetGraphMemAttribute = 641,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetGraphMemAttribute = 642,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithFlags = 643,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetExecAffinitySupport = 644,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v3 = 645,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetExecAffinity = 646,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid_v2 = 647,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemAllocNodeGetParams = 648,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemFreeNodeGetParams = 649,
    CUPTI_DRIVER_TRACE_CBID_SIZE = 650,
    CUPTI_DRIVER_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-11060"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_driver_api_trace_cbid_enum {
    CUPTI_DRIVER_TRACE_CBID_INVALID = 0,
    CUPTI_DRIVER_TRACE_CBID_cuInit = 1,
    CUPTI_DRIVER_TRACE_CBID_cuDriverGetVersion = 2,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGet = 3,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetCount = 4,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetName = 5,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceComputeCapability = 6,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem = 7,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetProperties = 8,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetAttribute = 9,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate = 10,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy = 11,
    CUPTI_DRIVER_TRACE_CBID_cuCtxAttach = 12,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDetach = 13,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent = 14,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent = 15,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetDevice = 16,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSynchronize = 17,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoad = 18,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadData = 19,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadDataEx = 20,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadFatBinary = 21,
    CUPTI_DRIVER_TRACE_CBID_cuModuleUnload = 22,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetFunction = 23,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal = 24,
    CUPTI_DRIVER_TRACE_CBID_cu64ModuleGetGlobal = 25,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetTexRef = 26,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo = 27,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetInfo = 28,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc = 29,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAlloc = 30,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch = 31,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAllocPitch = 32,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree = 33,
    CUPTI_DRIVER_TRACE_CBID_cu64MemFree = 34,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange = 35,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetAddressRange = 36,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost = 37,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeHost = 38,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc = 39,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer = 40,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostGetDevicePointer = 41,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetFlags = 42,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD = 43,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoD = 44,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH = 45,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoH = 46,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD = 47,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoD = 48,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA = 49,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoA = 50,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD = 51,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyAtoD = 52,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA = 53,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH = 54,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA = 55,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D = 56,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned = 57,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D = 58,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3D = 59,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync = 60,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoDAsync = 61,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync = 62,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoHAsync = 63,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync = 64,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoDAsync = 65,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync = 66,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync = 67,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync = 68,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync = 69,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3DAsync = 70,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8 = 71,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8 = 72,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16 = 73,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16 = 74,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32 = 75,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32 = 76,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8 = 77,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8 = 78,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16 = 79,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16 = 80,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32 = 81,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32 = 82,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetBlockShape = 83,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedSize = 84,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetAttribute = 85,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetCacheConfig = 86,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate = 87,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor = 88,
    CUPTI_DRIVER_TRACE_CBID_cuArrayDestroy = 89,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate = 90,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor = 91,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefCreate = 92,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefDestroy = 93,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetArray = 94,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress = 95,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress = 96,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D = 97,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress2D = 98,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFormat = 99,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddressMode = 100,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFilterMode = 101,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFlags = 102,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress = 103,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefGetAddress = 104,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetArray = 105,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddressMode = 106,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFilterMode = 107,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFormat = 108,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFlags = 109,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetSize = 110,
    CUPTI_DRIVER_TRACE_CBID_cuParamSeti = 111,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetf = 112,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetv = 113,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetTexRef = 114,
    CUPTI_DRIVER_TRACE_CBID_cuLaunch = 115,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGrid = 116,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGridAsync = 117,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreate = 118,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord = 119,
    CUPTI_DRIVER_TRACE_CBID_cuEventQuery = 120,
    CUPTI_DRIVER_TRACE_CBID_cuEventSynchronize = 121,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy = 122,
    CUPTI_DRIVER_TRACE_CBID_cuEventElapsedTime = 123,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreate = 124,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery = 125,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize = 126,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy = 127,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnregisterResource = 128,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsSubResourceGetMappedArray = 129,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer = 130,
    CUPTI_DRIVER_TRACE_CBID_cu64GraphicsResourceGetMappedPointer = 131,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags = 132,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources = 133,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources = 134,
    CUPTI_DRIVER_TRACE_CBID_cuGetExportTable = 135,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetLimit = 136,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetLimit = 137,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevice = 138,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate = 139,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D10RegisterResource = 140,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10RegisterResource = 141,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnregisterResource = 142,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10MapResources = 143,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnmapResources = 144,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceSetMapFlags = 145,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedArray = 146,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer = 147,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize = 148,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch = 149,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions = 150,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevice = 151,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate = 152,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D11RegisterResource = 153,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevice = 154,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate = 155,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D9RegisterResource = 156,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDirect3DDevice = 157,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterResource = 158,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterResource = 159,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapResources = 160,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapResources = 161,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceSetMapFlags = 162,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions = 163,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedArray = 164,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer = 165,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize = 166,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch = 167,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9Begin = 168,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9End = 169,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterVertexBuffer = 170,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer = 171,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapVertexBuffer = 172,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterVertexBuffer = 173,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate = 174,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterBuffer = 175,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterImage = 176,
    CUPTI_DRIVER_TRACE_CBID_cuWGLGetDevice = 177,
    CUPTI_DRIVER_TRACE_CBID_cuGLInit = 178,
    CUPTI_DRIVER_TRACE_CBID_cuGLRegisterBufferObject = 179,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject = 180,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObject = 181,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnregisterBufferObject = 182,
    CUPTI_DRIVER_TRACE_CBID_cuGLSetBufferObjectMapFlags = 183,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync = 184,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObjectAsync = 185,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUGetDevice = 186,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate = 187,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterVideoSurface = 188,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterOutputSurface = 189,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetSurfRef = 190,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefCreate = 191,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefDestroy = 192,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetFormat = 193,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetArray = 194,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetFormat = 195,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetArray = 196,
    CUPTI_DRIVER_TRACE_CBID_cu64DeviceTotalMem = 197,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPointer = 198,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedSize = 199,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPitch = 200,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetSurfaceDimensions = 201,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetSurfaceDimensions = 202,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPointer = 203,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedSize = 204,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPitch = 205,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9MapVertexBuffer = 206,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObject = 207,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObjectAsync = 208,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevices = 209,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreateOnDevice = 210,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevices = 211,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreateOnDevice = 212,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevices = 213,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreateOnDevice = 214,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostAlloc = 215,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async = 216,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8Async = 217,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async = 218,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16Async = 219,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async = 220,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32Async = 221,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async = 222,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8Async = 223,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async = 224,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16Async = 225,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async = 226,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32Async = 227,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayCreate = 228,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayGetDescriptor = 229,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DCreate = 230,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DGetDescriptor = 231,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2D = 232,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DUnaligned = 233,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DAsync = 234,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v2 = 235,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate_v2 = 236,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate_v2 = 237,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate_v2 = 238,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate_v2 = 239,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate_v2 = 240,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal_v2 = 241,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo_v2 = 242,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc_v2 = 243,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch_v2 = 244,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree_v2 = 245,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange_v2 = 246,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer_v2 = 247,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_v2 = 248,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2 = 249,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2 = 250,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2 = 251,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2 = 252,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2 = 253,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2 = 254,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress_v2 = 255,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v2 = 256,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress_v2 = 257,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer_v2 = 258,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem_v2 = 259,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer_v2 = 260,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize_v2 = 261,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch_v2 = 262,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions_v2 = 263,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions_v2 = 264,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer_v2 = 265,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize_v2 = 266,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch_v2 = 267,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer_v2 = 268,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2 = 269,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2 = 270,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc_v2 = 271,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate_v2 = 272,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor_v2 = 273,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate_v2 = 274,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor_v2 = 275,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2 = 276,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2 = 277,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2 = 278,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2 = 279,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2 = 280,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2 = 281,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2 = 282,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2 = 283,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2 = 284,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2 = 285,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2 = 286,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2 = 287,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2 = 288,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2 = 289,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2 = 290,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2 = 291,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2 = 292,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2 = 293,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost_v2 = 294,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent = 295,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetApiVersion = 296,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDirect3DDevice = 297,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDirect3DDevice = 298,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCacheConfig = 299,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCacheConfig = 300,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister = 301,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostUnregister = 302,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCurrent = 303,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCurrent = 304,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy = 305,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync = 306,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel = 307,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStart = 308,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStop = 309,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttribute = 310,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerInitialize = 311,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceCanAccessPeer = 312,
    CUPTI_DRIVER_TRACE_CBID_cuCtxEnablePeerAccess = 313,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDisablePeerAccess = 314,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerRegister = 315,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerUnregister = 316,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerGetDevicePointer = 317,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer = 318,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync = 319,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer = 320,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync = 321,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy_v2 = 322,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent_v2 = 323,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent_v2 = 324,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy_v2 = 325,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy_v2 = 326,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v3 = 327,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetMemHandle = 328,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle = 329,
    CUPTI_DRIVER_TRACE_CBID_cuIpcCloseMemHandle = 330,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetByPCIBusId = 331,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetPCIBusId = 332,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices = 333,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetEventHandle = 334,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenEventHandle = 335,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetSharedMemConfig = 336,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetSharedMemConfig = 337,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedMemConfig = 338,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectCreate = 339,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectDestroy = 340,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceDesc = 341,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetTextureDesc = 342,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectCreate = 343,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectDestroy = 344,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectGetResourceDesc = 345,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback = 346,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayCreate = 347,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetLevel = 348,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayDestroy = 349,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmappedArray = 350,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapFilterMode = 351,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelBias = 352,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelClamp = 353,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMaxAnisotropy = 354,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmappedArray = 355,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapFilterMode = 356,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelBias = 357,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelClamp = 358,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMaxAnisotropy = 359,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedMipmappedArray = 360,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceViewDesc = 361,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate = 362,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData = 363,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile = 364,
    CUPTI_DRIVER_TRACE_CBID_cuLinkComplete = 365,
    CUPTI_DRIVER_TRACE_CBID_cuLinkDestroy = 366,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreateWithPriority = 367,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority = 368,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags = 369,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetStreamPriorityRange = 370,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocManaged = 371,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorString = 372,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorName = 373,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessor = 374,
    CUPTI_DRIVER_TRACE_CBID_cuCompilePtx = 375,
    CUPTI_DRIVER_TRACE_CBID_cuBinaryFree = 376,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync = 377,
    CUPTI_DRIVER_TRACE_CBID_cuPointerSetAttribute = 378,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister_v2 = 379,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags_v2 = 380,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate_v2 = 381,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData_v2 = 382,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile_v2 = 383,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSize = 384,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices_v2 = 385,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRetain = 386,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease = 387,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags = 388,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset = 389,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsEGLRegisterImage = 390,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetFlags = 391,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxGetState = 392,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnect = 393,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerDisconnect = 394,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerAcquireFrame = 395,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerReleaseFrame = 396,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2_ptds = 397,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2_ptds = 398,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2_ptds = 399,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2_ptds = 400,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2_ptds = 401,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2_ptds = 402,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2_ptds = 403,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2_ptds = 404,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2_ptds = 405,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2_ptds = 406,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2_ptds = 407,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_ptds = 408,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer_ptds = 409,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer_ptds = 410,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2_ptds = 411,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2_ptds = 412,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2_ptds = 413,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2_ptds = 414,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2_ptds = 415,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2_ptds = 416,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2_ptds = 417,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync_ptsz = 418,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2_ptsz = 419,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2_ptsz = 420,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2_ptsz = 421,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2_ptsz = 422,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2_ptsz = 423,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2_ptsz = 424,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2_ptsz = 425,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync_ptsz = 426,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync_ptsz = 427,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async_ptsz = 428,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async_ptsz = 429,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async_ptsz = 430,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async_ptsz = 431,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async_ptsz = 432,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async_ptsz = 433,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority_ptsz = 434,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags_ptsz = 435,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent_ptsz = 436,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback_ptsz = 437,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync_ptsz = 438,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery_ptsz = 439,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize_ptsz = 440,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord_ptsz = 441,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel_ptsz = 442,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources_ptsz = 443,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources_ptsz = 444,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2_ptsz = 445,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerConnect = 446,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerDisconnect = 447,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerPresentFrame = 448,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedEglFrame = 449,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttributes = 450,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessorWithFlags = 451,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSizeWithFlags = 452,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerReturnFrame = 453,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetP2PAttribute = 454,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetBorderColor = 455,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetBorderColor = 456,
    CUPTI_DRIVER_TRACE_CBID_cuMemAdvise = 457,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32 = 458,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_ptsz = 459,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32 = 460,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_ptsz = 461,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp = 462,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_ptsz = 463,
    CUPTI_DRIVER_TRACE_CBID_cuNVNbufferGetPointer = 464,
    CUPTI_DRIVER_TRACE_CBID_cuNVNtextureGetArray = 465,
    CUPTI_DRIVER_TRACE_CBID_cuNNSetAllocator = 466,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync = 467,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_ptsz = 468,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromNVNSync = 469,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnectWithFlags = 470,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttribute = 471,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttributes = 472,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64 = 473,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_ptsz = 474,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64 = 475,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_ptsz = 476,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel = 477,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel_ptsz = 478,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromEGLSync = 479,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernelMultiDevice = 480,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetAttribute = 481,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid = 482,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx = 483,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_ptsz = 484,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalMemory = 485,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedBuffer = 486,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedMipmappedArray = 487,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalMemory = 488,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalSemaphore = 489,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync = 490,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync_ptsz = 491,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync = 492,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync_ptsz = 493,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalSemaphore = 494,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture = 495,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_ptsz = 496,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture = 497,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture_ptsz = 498,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing = 499,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing_ptsz = 500,
    CUPTI_DRIVER_TRACE_CBID_cuGraphCreate = 501,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddKernelNode = 502,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetParams = 503,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemcpyNode = 504,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeGetParams = 505,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemsetNode = 506,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeGetParams = 507,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeSetParams = 508,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetType = 509,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetRootNodes = 510,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependencies = 511,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependentNodes = 512,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate = 513,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch = 514,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch_ptsz = 515,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecDestroy = 516,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroy = 517,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddDependencies = 518,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRemoveDependencies = 519,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeSetParams = 520,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetParams = 521,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroyNode = 522,
    CUPTI_DRIVER_TRACE_CBID_cuGraphClone = 523,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeFindInClone = 524,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddChildGraphNode = 525,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEmptyNode = 526,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc = 527,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc_ptsz = 528,
    CUPTI_DRIVER_TRACE_CBID_cuGraphChildGraphNodeGetGraph = 529,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddHostNode = 530,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeGetParams = 531,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetLuid = 532,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeSetParams = 533,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetNodes = 534,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetEdges = 535,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo = 536,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_ptsz = 537,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecKernelNodeSetParams = 538,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2 = 539,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2_ptsz = 540,
    CUPTI_DRIVER_TRACE_CBID_cuThreadExchangeStreamCaptureMode = 541,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetNvSciSyncAttributes = 542,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyAvailableDynamicSMemPerBlock = 543,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease_v2 = 544,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset_v2 = 545,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags_v2 = 546,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressReserve = 547,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressFree = 548,
    CUPTI_DRIVER_TRACE_CBID_cuMemCreate = 549,
    CUPTI_DRIVER_TRACE_CBID_cuMemRelease = 550,
    CUPTI_DRIVER_TRACE_CBID_cuMemMap = 551,
    CUPTI_DRIVER_TRACE_CBID_cuMemUnmap = 552,
    CUPTI_DRIVER_TRACE_CBID_cuMemSetAccess = 553,
    CUPTI_DRIVER_TRACE_CBID_cuMemExportToShareableHandle = 554,
    CUPTI_DRIVER_TRACE_CBID_cuMemImportFromShareableHandle = 555,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationGranularity = 556,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationPropertiesFromHandle = 557,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAccess = 558,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags = 559,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags_ptsz = 560,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecUpdate = 561,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemcpyNodeSetParams = 562,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemsetNodeSetParams = 563,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecHostNodeSetParams = 564,
    CUPTI_DRIVER_TRACE_CBID_cuMemRetainAllocationHandle = 565,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetModule = 566,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle_v2 = 567,
    CUPTI_DRIVER_TRACE_CBID_cuCtxResetPersistingL2Cache = 568,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeCopyAttributes = 569,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetAttribute = 570,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetAttribute = 571,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes = 572,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes_ptsz = 573,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute = 574,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute_ptsz = 575,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute = 576,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute_ptsz = 577,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate_v2 = 578,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetTexture1DLinearMaxWidth = 579,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload = 580,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload_ptsz = 581,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetSparseProperties = 582,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetSparseProperties = 583,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync = 584,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync_ptsz = 585,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecChildGraphNodeSetParams = 586,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags = 587,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags_ptsz = 588,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventRecordNode = 589,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventWaitNode = 590,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeGetEvent = 591,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeGetEvent = 592,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeSetEvent = 593,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeSetEvent = 594,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventRecordNodeSetEvent = 595,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventWaitNodeSetEvent = 596,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetPlane = 597,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync = 598,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync_ptsz = 599,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync = 600,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync_ptsz = 601,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolTrimTo = 602,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAttribute = 603,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAttribute = 604,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAccess = 605,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetDefaultMemPool = 606,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolCreate = 607,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolDestroy = 608,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetMemPool = 609,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetMemPool = 610,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync = 611,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync_ptsz = 612,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportToShareableHandle = 613,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportFromShareableHandle = 614,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportPointer = 615,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportPointer = 616,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAccess = 617,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresSignalNode = 618,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeGetParams = 619,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeSetParams = 620,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresWaitNode = 621,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeGetParams = 622,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeSetParams = 623,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresSignalNodeSetParams = 624,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresWaitNodeSetParams = 625,
    CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress = 626,
    CUPTI_DRIVER_TRACE_CBID_cuFlushGPUDirectRDMAWrites = 627,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDebugDotPrint = 628,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2 = 629,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2_ptsz = 630,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies = 631,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_ptsz = 632,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectCreate = 633,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRetain = 634,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRelease = 635,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRetainUserObject = 636,
    CUPTI_DRIVER_TRACE_CBID_cuGraphReleaseUserObject = 637,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemAllocNode = 638,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemFreeNode = 639,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGraphMemTrim = 640,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetGraphMemAttribute = 641,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetGraphMemAttribute = 642,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithFlags = 643,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetExecAffinitySupport = 644,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v3 = 645,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetExecAffinity = 646,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid_v2 = 647,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemAllocNodeGetParams = 648,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemFreeNodeGetParams = 649,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeSetEnabled = 650,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetEnabled = 651,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx = 652,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx_ptsz = 653,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetMemoryRequirements = 654,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetMemoryRequirements = 655,
    CUPTI_DRIVER_TRACE_CBID_SIZE = 656,
    CUPTI_DRIVER_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-11070", feature = "cuda-11080"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_driver_api_trace_cbid_enum {
    CUPTI_DRIVER_TRACE_CBID_INVALID = 0,
    CUPTI_DRIVER_TRACE_CBID_cuInit = 1,
    CUPTI_DRIVER_TRACE_CBID_cuDriverGetVersion = 2,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGet = 3,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetCount = 4,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetName = 5,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceComputeCapability = 6,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem = 7,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetProperties = 8,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetAttribute = 9,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate = 10,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy = 11,
    CUPTI_DRIVER_TRACE_CBID_cuCtxAttach = 12,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDetach = 13,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent = 14,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent = 15,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetDevice = 16,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSynchronize = 17,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoad = 18,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadData = 19,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadDataEx = 20,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadFatBinary = 21,
    CUPTI_DRIVER_TRACE_CBID_cuModuleUnload = 22,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetFunction = 23,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal = 24,
    CUPTI_DRIVER_TRACE_CBID_cu64ModuleGetGlobal = 25,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetTexRef = 26,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo = 27,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetInfo = 28,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc = 29,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAlloc = 30,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch = 31,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAllocPitch = 32,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree = 33,
    CUPTI_DRIVER_TRACE_CBID_cu64MemFree = 34,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange = 35,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetAddressRange = 36,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost = 37,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeHost = 38,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc = 39,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer = 40,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostGetDevicePointer = 41,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetFlags = 42,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD = 43,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoD = 44,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH = 45,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoH = 46,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD = 47,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoD = 48,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA = 49,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoA = 50,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD = 51,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyAtoD = 52,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA = 53,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH = 54,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA = 55,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D = 56,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned = 57,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D = 58,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3D = 59,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync = 60,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoDAsync = 61,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync = 62,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoHAsync = 63,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync = 64,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoDAsync = 65,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync = 66,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync = 67,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync = 68,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync = 69,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3DAsync = 70,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8 = 71,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8 = 72,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16 = 73,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16 = 74,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32 = 75,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32 = 76,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8 = 77,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8 = 78,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16 = 79,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16 = 80,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32 = 81,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32 = 82,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetBlockShape = 83,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedSize = 84,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetAttribute = 85,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetCacheConfig = 86,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate = 87,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor = 88,
    CUPTI_DRIVER_TRACE_CBID_cuArrayDestroy = 89,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate = 90,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor = 91,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefCreate = 92,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefDestroy = 93,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetArray = 94,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress = 95,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress = 96,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D = 97,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress2D = 98,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFormat = 99,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddressMode = 100,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFilterMode = 101,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFlags = 102,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress = 103,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefGetAddress = 104,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetArray = 105,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddressMode = 106,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFilterMode = 107,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFormat = 108,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFlags = 109,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetSize = 110,
    CUPTI_DRIVER_TRACE_CBID_cuParamSeti = 111,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetf = 112,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetv = 113,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetTexRef = 114,
    CUPTI_DRIVER_TRACE_CBID_cuLaunch = 115,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGrid = 116,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGridAsync = 117,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreate = 118,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord = 119,
    CUPTI_DRIVER_TRACE_CBID_cuEventQuery = 120,
    CUPTI_DRIVER_TRACE_CBID_cuEventSynchronize = 121,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy = 122,
    CUPTI_DRIVER_TRACE_CBID_cuEventElapsedTime = 123,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreate = 124,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery = 125,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize = 126,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy = 127,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnregisterResource = 128,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsSubResourceGetMappedArray = 129,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer = 130,
    CUPTI_DRIVER_TRACE_CBID_cu64GraphicsResourceGetMappedPointer = 131,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags = 132,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources = 133,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources = 134,
    CUPTI_DRIVER_TRACE_CBID_cuGetExportTable = 135,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetLimit = 136,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetLimit = 137,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevice = 138,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate = 139,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D10RegisterResource = 140,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10RegisterResource = 141,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnregisterResource = 142,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10MapResources = 143,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnmapResources = 144,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceSetMapFlags = 145,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedArray = 146,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer = 147,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize = 148,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch = 149,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions = 150,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevice = 151,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate = 152,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D11RegisterResource = 153,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevice = 154,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate = 155,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D9RegisterResource = 156,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDirect3DDevice = 157,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterResource = 158,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterResource = 159,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapResources = 160,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapResources = 161,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceSetMapFlags = 162,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions = 163,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedArray = 164,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer = 165,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize = 166,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch = 167,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9Begin = 168,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9End = 169,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterVertexBuffer = 170,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer = 171,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapVertexBuffer = 172,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterVertexBuffer = 173,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate = 174,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterBuffer = 175,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterImage = 176,
    CUPTI_DRIVER_TRACE_CBID_cuWGLGetDevice = 177,
    CUPTI_DRIVER_TRACE_CBID_cuGLInit = 178,
    CUPTI_DRIVER_TRACE_CBID_cuGLRegisterBufferObject = 179,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject = 180,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObject = 181,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnregisterBufferObject = 182,
    CUPTI_DRIVER_TRACE_CBID_cuGLSetBufferObjectMapFlags = 183,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync = 184,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObjectAsync = 185,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUGetDevice = 186,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate = 187,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterVideoSurface = 188,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterOutputSurface = 189,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetSurfRef = 190,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefCreate = 191,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefDestroy = 192,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetFormat = 193,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetArray = 194,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetFormat = 195,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetArray = 196,
    CUPTI_DRIVER_TRACE_CBID_cu64DeviceTotalMem = 197,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPointer = 198,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedSize = 199,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPitch = 200,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetSurfaceDimensions = 201,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetSurfaceDimensions = 202,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPointer = 203,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedSize = 204,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPitch = 205,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9MapVertexBuffer = 206,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObject = 207,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObjectAsync = 208,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevices = 209,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreateOnDevice = 210,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevices = 211,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreateOnDevice = 212,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevices = 213,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreateOnDevice = 214,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostAlloc = 215,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async = 216,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8Async = 217,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async = 218,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16Async = 219,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async = 220,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32Async = 221,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async = 222,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8Async = 223,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async = 224,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16Async = 225,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async = 226,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32Async = 227,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayCreate = 228,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayGetDescriptor = 229,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DCreate = 230,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DGetDescriptor = 231,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2D = 232,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DUnaligned = 233,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DAsync = 234,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v2 = 235,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate_v2 = 236,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate_v2 = 237,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate_v2 = 238,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate_v2 = 239,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate_v2 = 240,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal_v2 = 241,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo_v2 = 242,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc_v2 = 243,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch_v2 = 244,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree_v2 = 245,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange_v2 = 246,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer_v2 = 247,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_v2 = 248,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2 = 249,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2 = 250,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2 = 251,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2 = 252,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2 = 253,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2 = 254,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress_v2 = 255,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v2 = 256,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress_v2 = 257,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer_v2 = 258,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem_v2 = 259,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer_v2 = 260,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize_v2 = 261,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch_v2 = 262,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions_v2 = 263,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions_v2 = 264,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer_v2 = 265,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize_v2 = 266,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch_v2 = 267,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer_v2 = 268,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2 = 269,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2 = 270,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc_v2 = 271,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate_v2 = 272,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor_v2 = 273,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate_v2 = 274,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor_v2 = 275,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2 = 276,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2 = 277,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2 = 278,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2 = 279,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2 = 280,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2 = 281,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2 = 282,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2 = 283,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2 = 284,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2 = 285,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2 = 286,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2 = 287,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2 = 288,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2 = 289,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2 = 290,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2 = 291,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2 = 292,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2 = 293,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost_v2 = 294,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent = 295,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetApiVersion = 296,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDirect3DDevice = 297,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDirect3DDevice = 298,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCacheConfig = 299,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCacheConfig = 300,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister = 301,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostUnregister = 302,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCurrent = 303,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCurrent = 304,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy = 305,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync = 306,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel = 307,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStart = 308,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStop = 309,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttribute = 310,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerInitialize = 311,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceCanAccessPeer = 312,
    CUPTI_DRIVER_TRACE_CBID_cuCtxEnablePeerAccess = 313,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDisablePeerAccess = 314,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerRegister = 315,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerUnregister = 316,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerGetDevicePointer = 317,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer = 318,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync = 319,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer = 320,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync = 321,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy_v2 = 322,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent_v2 = 323,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent_v2 = 324,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy_v2 = 325,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy_v2 = 326,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v3 = 327,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetMemHandle = 328,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle = 329,
    CUPTI_DRIVER_TRACE_CBID_cuIpcCloseMemHandle = 330,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetByPCIBusId = 331,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetPCIBusId = 332,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices = 333,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetEventHandle = 334,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenEventHandle = 335,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetSharedMemConfig = 336,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetSharedMemConfig = 337,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedMemConfig = 338,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectCreate = 339,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectDestroy = 340,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceDesc = 341,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetTextureDesc = 342,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectCreate = 343,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectDestroy = 344,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectGetResourceDesc = 345,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback = 346,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayCreate = 347,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetLevel = 348,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayDestroy = 349,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmappedArray = 350,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapFilterMode = 351,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelBias = 352,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelClamp = 353,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMaxAnisotropy = 354,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmappedArray = 355,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapFilterMode = 356,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelBias = 357,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelClamp = 358,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMaxAnisotropy = 359,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedMipmappedArray = 360,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceViewDesc = 361,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate = 362,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData = 363,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile = 364,
    CUPTI_DRIVER_TRACE_CBID_cuLinkComplete = 365,
    CUPTI_DRIVER_TRACE_CBID_cuLinkDestroy = 366,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreateWithPriority = 367,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority = 368,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags = 369,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetStreamPriorityRange = 370,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocManaged = 371,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorString = 372,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorName = 373,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessor = 374,
    CUPTI_DRIVER_TRACE_CBID_cuCompilePtx = 375,
    CUPTI_DRIVER_TRACE_CBID_cuBinaryFree = 376,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync = 377,
    CUPTI_DRIVER_TRACE_CBID_cuPointerSetAttribute = 378,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister_v2 = 379,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags_v2 = 380,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate_v2 = 381,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData_v2 = 382,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile_v2 = 383,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSize = 384,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices_v2 = 385,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRetain = 386,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease = 387,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags = 388,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset = 389,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsEGLRegisterImage = 390,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetFlags = 391,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxGetState = 392,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnect = 393,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerDisconnect = 394,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerAcquireFrame = 395,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerReleaseFrame = 396,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2_ptds = 397,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2_ptds = 398,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2_ptds = 399,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2_ptds = 400,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2_ptds = 401,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2_ptds = 402,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2_ptds = 403,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2_ptds = 404,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2_ptds = 405,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2_ptds = 406,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2_ptds = 407,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_ptds = 408,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer_ptds = 409,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer_ptds = 410,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2_ptds = 411,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2_ptds = 412,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2_ptds = 413,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2_ptds = 414,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2_ptds = 415,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2_ptds = 416,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2_ptds = 417,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync_ptsz = 418,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2_ptsz = 419,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2_ptsz = 420,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2_ptsz = 421,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2_ptsz = 422,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2_ptsz = 423,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2_ptsz = 424,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2_ptsz = 425,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync_ptsz = 426,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync_ptsz = 427,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async_ptsz = 428,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async_ptsz = 429,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async_ptsz = 430,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async_ptsz = 431,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async_ptsz = 432,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async_ptsz = 433,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority_ptsz = 434,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags_ptsz = 435,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent_ptsz = 436,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback_ptsz = 437,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync_ptsz = 438,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery_ptsz = 439,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize_ptsz = 440,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord_ptsz = 441,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel_ptsz = 442,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources_ptsz = 443,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources_ptsz = 444,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2_ptsz = 445,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerConnect = 446,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerDisconnect = 447,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerPresentFrame = 448,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedEglFrame = 449,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttributes = 450,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessorWithFlags = 451,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSizeWithFlags = 452,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerReturnFrame = 453,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetP2PAttribute = 454,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetBorderColor = 455,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetBorderColor = 456,
    CUPTI_DRIVER_TRACE_CBID_cuMemAdvise = 457,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32 = 458,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_ptsz = 459,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32 = 460,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_ptsz = 461,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp = 462,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_ptsz = 463,
    CUPTI_DRIVER_TRACE_CBID_cuNVNbufferGetPointer = 464,
    CUPTI_DRIVER_TRACE_CBID_cuNVNtextureGetArray = 465,
    CUPTI_DRIVER_TRACE_CBID_cuNNSetAllocator = 466,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync = 467,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_ptsz = 468,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromNVNSync = 469,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnectWithFlags = 470,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttribute = 471,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttributes = 472,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64 = 473,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_ptsz = 474,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64 = 475,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_ptsz = 476,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel = 477,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel_ptsz = 478,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromEGLSync = 479,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernelMultiDevice = 480,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetAttribute = 481,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid = 482,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx = 483,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_ptsz = 484,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalMemory = 485,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedBuffer = 486,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedMipmappedArray = 487,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalMemory = 488,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalSemaphore = 489,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync = 490,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync_ptsz = 491,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync = 492,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync_ptsz = 493,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalSemaphore = 494,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture = 495,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_ptsz = 496,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture = 497,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture_ptsz = 498,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing = 499,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing_ptsz = 500,
    CUPTI_DRIVER_TRACE_CBID_cuGraphCreate = 501,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddKernelNode = 502,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetParams = 503,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemcpyNode = 504,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeGetParams = 505,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemsetNode = 506,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeGetParams = 507,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeSetParams = 508,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetType = 509,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetRootNodes = 510,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependencies = 511,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependentNodes = 512,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate = 513,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch = 514,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch_ptsz = 515,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecDestroy = 516,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroy = 517,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddDependencies = 518,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRemoveDependencies = 519,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeSetParams = 520,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetParams = 521,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroyNode = 522,
    CUPTI_DRIVER_TRACE_CBID_cuGraphClone = 523,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeFindInClone = 524,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddChildGraphNode = 525,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEmptyNode = 526,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc = 527,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc_ptsz = 528,
    CUPTI_DRIVER_TRACE_CBID_cuGraphChildGraphNodeGetGraph = 529,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddHostNode = 530,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeGetParams = 531,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetLuid = 532,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeSetParams = 533,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetNodes = 534,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetEdges = 535,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo = 536,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_ptsz = 537,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecKernelNodeSetParams = 538,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2 = 539,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2_ptsz = 540,
    CUPTI_DRIVER_TRACE_CBID_cuThreadExchangeStreamCaptureMode = 541,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetNvSciSyncAttributes = 542,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyAvailableDynamicSMemPerBlock = 543,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease_v2 = 544,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset_v2 = 545,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags_v2 = 546,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressReserve = 547,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressFree = 548,
    CUPTI_DRIVER_TRACE_CBID_cuMemCreate = 549,
    CUPTI_DRIVER_TRACE_CBID_cuMemRelease = 550,
    CUPTI_DRIVER_TRACE_CBID_cuMemMap = 551,
    CUPTI_DRIVER_TRACE_CBID_cuMemUnmap = 552,
    CUPTI_DRIVER_TRACE_CBID_cuMemSetAccess = 553,
    CUPTI_DRIVER_TRACE_CBID_cuMemExportToShareableHandle = 554,
    CUPTI_DRIVER_TRACE_CBID_cuMemImportFromShareableHandle = 555,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationGranularity = 556,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationPropertiesFromHandle = 557,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAccess = 558,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags = 559,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags_ptsz = 560,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecUpdate = 561,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemcpyNodeSetParams = 562,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemsetNodeSetParams = 563,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecHostNodeSetParams = 564,
    CUPTI_DRIVER_TRACE_CBID_cuMemRetainAllocationHandle = 565,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetModule = 566,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle_v2 = 567,
    CUPTI_DRIVER_TRACE_CBID_cuCtxResetPersistingL2Cache = 568,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeCopyAttributes = 569,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetAttribute = 570,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetAttribute = 571,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes = 572,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes_ptsz = 573,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute = 574,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute_ptsz = 575,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute = 576,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute_ptsz = 577,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate_v2 = 578,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetTexture1DLinearMaxWidth = 579,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload = 580,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload_ptsz = 581,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetSparseProperties = 582,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetSparseProperties = 583,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync = 584,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync_ptsz = 585,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecChildGraphNodeSetParams = 586,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags = 587,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags_ptsz = 588,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventRecordNode = 589,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventWaitNode = 590,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeGetEvent = 591,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeGetEvent = 592,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeSetEvent = 593,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeSetEvent = 594,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventRecordNodeSetEvent = 595,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventWaitNodeSetEvent = 596,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetPlane = 597,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync = 598,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync_ptsz = 599,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync = 600,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync_ptsz = 601,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolTrimTo = 602,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAttribute = 603,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAttribute = 604,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAccess = 605,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetDefaultMemPool = 606,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolCreate = 607,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolDestroy = 608,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetMemPool = 609,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetMemPool = 610,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync = 611,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync_ptsz = 612,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportToShareableHandle = 613,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportFromShareableHandle = 614,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportPointer = 615,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportPointer = 616,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAccess = 617,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresSignalNode = 618,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeGetParams = 619,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeSetParams = 620,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresWaitNode = 621,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeGetParams = 622,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeSetParams = 623,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresSignalNodeSetParams = 624,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresWaitNodeSetParams = 625,
    CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress = 626,
    CUPTI_DRIVER_TRACE_CBID_cuFlushGPUDirectRDMAWrites = 627,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDebugDotPrint = 628,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2 = 629,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2_ptsz = 630,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies = 631,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_ptsz = 632,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectCreate = 633,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRetain = 634,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRelease = 635,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRetainUserObject = 636,
    CUPTI_DRIVER_TRACE_CBID_cuGraphReleaseUserObject = 637,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemAllocNode = 638,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemFreeNode = 639,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGraphMemTrim = 640,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetGraphMemAttribute = 641,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetGraphMemAttribute = 642,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithFlags = 643,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetExecAffinitySupport = 644,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v3 = 645,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetExecAffinity = 646,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid_v2 = 647,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemAllocNodeGetParams = 648,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemFreeNodeGetParams = 649,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeSetEnabled = 650,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetEnabled = 651,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx = 652,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx_ptsz = 653,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetMemoryRequirements = 654,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetMemoryRequirements = 655,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithParams = 656,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithParams_ptsz = 657,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecGetFlags = 658,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_v2 = 659,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_v2_ptsz = 660,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_v2 = 661,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_v2_ptsz = 662,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_v2 = 663,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_v2_ptsz = 664,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_v2 = 665,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_v2_ptsz = 666,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_v2 = 667,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_v2_ptsz = 668,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddBatchMemOpNode = 669,
    CUPTI_DRIVER_TRACE_CBID_cuGraphBatchMemOpNodeGetParams = 670,
    CUPTI_DRIVER_TRACE_CBID_cuGraphBatchMemOpNodeSetParams = 671,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecBatchMemOpNodeSetParams = 672,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetLoadingMode = 673,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetHandleForAddressRange = 674,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialClusterSize = 675,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveClusters = 676,
    CUPTI_DRIVER_TRACE_CBID_SIZE = 677,
    CUPTI_DRIVER_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-12000"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_driver_api_trace_cbid_enum {
    CUPTI_DRIVER_TRACE_CBID_INVALID = 0,
    CUPTI_DRIVER_TRACE_CBID_cuInit = 1,
    CUPTI_DRIVER_TRACE_CBID_cuDriverGetVersion = 2,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGet = 3,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetCount = 4,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetName = 5,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceComputeCapability = 6,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem = 7,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetProperties = 8,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetAttribute = 9,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate = 10,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy = 11,
    CUPTI_DRIVER_TRACE_CBID_cuCtxAttach = 12,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDetach = 13,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent = 14,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent = 15,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetDevice = 16,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSynchronize = 17,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoad = 18,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadData = 19,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadDataEx = 20,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadFatBinary = 21,
    CUPTI_DRIVER_TRACE_CBID_cuModuleUnload = 22,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetFunction = 23,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal = 24,
    CUPTI_DRIVER_TRACE_CBID_cu64ModuleGetGlobal = 25,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetTexRef = 26,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo = 27,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetInfo = 28,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc = 29,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAlloc = 30,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch = 31,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAllocPitch = 32,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree = 33,
    CUPTI_DRIVER_TRACE_CBID_cu64MemFree = 34,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange = 35,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetAddressRange = 36,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost = 37,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeHost = 38,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc = 39,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer = 40,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostGetDevicePointer = 41,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetFlags = 42,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD = 43,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoD = 44,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH = 45,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoH = 46,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD = 47,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoD = 48,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA = 49,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoA = 50,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD = 51,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyAtoD = 52,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA = 53,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH = 54,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA = 55,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D = 56,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned = 57,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D = 58,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3D = 59,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync = 60,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoDAsync = 61,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync = 62,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoHAsync = 63,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync = 64,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoDAsync = 65,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync = 66,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync = 67,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync = 68,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync = 69,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3DAsync = 70,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8 = 71,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8 = 72,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16 = 73,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16 = 74,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32 = 75,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32 = 76,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8 = 77,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8 = 78,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16 = 79,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16 = 80,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32 = 81,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32 = 82,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetBlockShape = 83,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedSize = 84,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetAttribute = 85,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetCacheConfig = 86,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate = 87,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor = 88,
    CUPTI_DRIVER_TRACE_CBID_cuArrayDestroy = 89,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate = 90,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor = 91,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefCreate = 92,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefDestroy = 93,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetArray = 94,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress = 95,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress = 96,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D = 97,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress2D = 98,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFormat = 99,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddressMode = 100,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFilterMode = 101,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFlags = 102,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress = 103,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefGetAddress = 104,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetArray = 105,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddressMode = 106,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFilterMode = 107,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFormat = 108,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFlags = 109,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetSize = 110,
    CUPTI_DRIVER_TRACE_CBID_cuParamSeti = 111,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetf = 112,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetv = 113,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetTexRef = 114,
    CUPTI_DRIVER_TRACE_CBID_cuLaunch = 115,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGrid = 116,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGridAsync = 117,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreate = 118,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord = 119,
    CUPTI_DRIVER_TRACE_CBID_cuEventQuery = 120,
    CUPTI_DRIVER_TRACE_CBID_cuEventSynchronize = 121,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy = 122,
    CUPTI_DRIVER_TRACE_CBID_cuEventElapsedTime = 123,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreate = 124,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery = 125,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize = 126,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy = 127,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnregisterResource = 128,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsSubResourceGetMappedArray = 129,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer = 130,
    CUPTI_DRIVER_TRACE_CBID_cu64GraphicsResourceGetMappedPointer = 131,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags = 132,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources = 133,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources = 134,
    CUPTI_DRIVER_TRACE_CBID_cuGetExportTable = 135,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetLimit = 136,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetLimit = 137,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevice = 138,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate = 139,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D10RegisterResource = 140,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10RegisterResource = 141,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnregisterResource = 142,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10MapResources = 143,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnmapResources = 144,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceSetMapFlags = 145,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedArray = 146,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer = 147,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize = 148,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch = 149,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions = 150,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevice = 151,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate = 152,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D11RegisterResource = 153,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevice = 154,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate = 155,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D9RegisterResource = 156,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDirect3DDevice = 157,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterResource = 158,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterResource = 159,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapResources = 160,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapResources = 161,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceSetMapFlags = 162,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions = 163,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedArray = 164,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer = 165,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize = 166,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch = 167,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9Begin = 168,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9End = 169,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterVertexBuffer = 170,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer = 171,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapVertexBuffer = 172,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterVertexBuffer = 173,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate = 174,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterBuffer = 175,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterImage = 176,
    CUPTI_DRIVER_TRACE_CBID_cuWGLGetDevice = 177,
    CUPTI_DRIVER_TRACE_CBID_cuGLInit = 178,
    CUPTI_DRIVER_TRACE_CBID_cuGLRegisterBufferObject = 179,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject = 180,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObject = 181,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnregisterBufferObject = 182,
    CUPTI_DRIVER_TRACE_CBID_cuGLSetBufferObjectMapFlags = 183,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync = 184,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObjectAsync = 185,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUGetDevice = 186,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate = 187,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterVideoSurface = 188,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterOutputSurface = 189,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetSurfRef = 190,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefCreate = 191,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefDestroy = 192,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetFormat = 193,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetArray = 194,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetFormat = 195,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetArray = 196,
    CUPTI_DRIVER_TRACE_CBID_cu64DeviceTotalMem = 197,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPointer = 198,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedSize = 199,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPitch = 200,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetSurfaceDimensions = 201,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetSurfaceDimensions = 202,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPointer = 203,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedSize = 204,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPitch = 205,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9MapVertexBuffer = 206,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObject = 207,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObjectAsync = 208,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevices = 209,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreateOnDevice = 210,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevices = 211,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreateOnDevice = 212,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevices = 213,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreateOnDevice = 214,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostAlloc = 215,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async = 216,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8Async = 217,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async = 218,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16Async = 219,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async = 220,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32Async = 221,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async = 222,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8Async = 223,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async = 224,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16Async = 225,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async = 226,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32Async = 227,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayCreate = 228,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayGetDescriptor = 229,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DCreate = 230,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DGetDescriptor = 231,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2D = 232,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DUnaligned = 233,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DAsync = 234,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v2 = 235,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate_v2 = 236,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate_v2 = 237,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate_v2 = 238,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate_v2 = 239,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate_v2 = 240,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal_v2 = 241,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo_v2 = 242,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc_v2 = 243,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch_v2 = 244,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree_v2 = 245,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange_v2 = 246,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer_v2 = 247,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_v2 = 248,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2 = 249,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2 = 250,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2 = 251,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2 = 252,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2 = 253,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2 = 254,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress_v2 = 255,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v2 = 256,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress_v2 = 257,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer_v2 = 258,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem_v2 = 259,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer_v2 = 260,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize_v2 = 261,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch_v2 = 262,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions_v2 = 263,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions_v2 = 264,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer_v2 = 265,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize_v2 = 266,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch_v2 = 267,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer_v2 = 268,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2 = 269,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2 = 270,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc_v2 = 271,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate_v2 = 272,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor_v2 = 273,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate_v2 = 274,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor_v2 = 275,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2 = 276,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2 = 277,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2 = 278,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2 = 279,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2 = 280,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2 = 281,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2 = 282,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2 = 283,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2 = 284,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2 = 285,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2 = 286,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2 = 287,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2 = 288,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2 = 289,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2 = 290,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2 = 291,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2 = 292,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2 = 293,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost_v2 = 294,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent = 295,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetApiVersion = 296,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDirect3DDevice = 297,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDirect3DDevice = 298,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCacheConfig = 299,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCacheConfig = 300,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister = 301,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostUnregister = 302,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCurrent = 303,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCurrent = 304,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy = 305,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync = 306,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel = 307,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStart = 308,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStop = 309,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttribute = 310,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerInitialize = 311,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceCanAccessPeer = 312,
    CUPTI_DRIVER_TRACE_CBID_cuCtxEnablePeerAccess = 313,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDisablePeerAccess = 314,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerRegister = 315,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerUnregister = 316,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerGetDevicePointer = 317,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer = 318,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync = 319,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer = 320,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync = 321,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy_v2 = 322,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent_v2 = 323,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent_v2 = 324,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy_v2 = 325,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy_v2 = 326,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v3 = 327,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetMemHandle = 328,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle = 329,
    CUPTI_DRIVER_TRACE_CBID_cuIpcCloseMemHandle = 330,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetByPCIBusId = 331,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetPCIBusId = 332,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices = 333,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetEventHandle = 334,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenEventHandle = 335,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetSharedMemConfig = 336,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetSharedMemConfig = 337,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedMemConfig = 338,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectCreate = 339,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectDestroy = 340,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceDesc = 341,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetTextureDesc = 342,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectCreate = 343,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectDestroy = 344,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectGetResourceDesc = 345,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback = 346,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayCreate = 347,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetLevel = 348,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayDestroy = 349,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmappedArray = 350,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapFilterMode = 351,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelBias = 352,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelClamp = 353,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMaxAnisotropy = 354,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmappedArray = 355,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapFilterMode = 356,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelBias = 357,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelClamp = 358,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMaxAnisotropy = 359,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedMipmappedArray = 360,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceViewDesc = 361,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate = 362,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData = 363,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile = 364,
    CUPTI_DRIVER_TRACE_CBID_cuLinkComplete = 365,
    CUPTI_DRIVER_TRACE_CBID_cuLinkDestroy = 366,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreateWithPriority = 367,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority = 368,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags = 369,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetStreamPriorityRange = 370,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocManaged = 371,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorString = 372,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorName = 373,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessor = 374,
    CUPTI_DRIVER_TRACE_CBID_cuCompilePtx = 375,
    CUPTI_DRIVER_TRACE_CBID_cuBinaryFree = 376,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync = 377,
    CUPTI_DRIVER_TRACE_CBID_cuPointerSetAttribute = 378,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister_v2 = 379,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags_v2 = 380,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate_v2 = 381,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData_v2 = 382,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile_v2 = 383,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSize = 384,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices_v2 = 385,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRetain = 386,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease = 387,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags = 388,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset = 389,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsEGLRegisterImage = 390,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetFlags = 391,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxGetState = 392,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnect = 393,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerDisconnect = 394,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerAcquireFrame = 395,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerReleaseFrame = 396,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2_ptds = 397,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2_ptds = 398,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2_ptds = 399,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2_ptds = 400,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2_ptds = 401,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2_ptds = 402,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2_ptds = 403,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2_ptds = 404,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2_ptds = 405,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2_ptds = 406,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2_ptds = 407,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_ptds = 408,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer_ptds = 409,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer_ptds = 410,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2_ptds = 411,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2_ptds = 412,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2_ptds = 413,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2_ptds = 414,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2_ptds = 415,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2_ptds = 416,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2_ptds = 417,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync_ptsz = 418,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2_ptsz = 419,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2_ptsz = 420,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2_ptsz = 421,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2_ptsz = 422,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2_ptsz = 423,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2_ptsz = 424,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2_ptsz = 425,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync_ptsz = 426,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync_ptsz = 427,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async_ptsz = 428,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async_ptsz = 429,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async_ptsz = 430,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async_ptsz = 431,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async_ptsz = 432,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async_ptsz = 433,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority_ptsz = 434,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags_ptsz = 435,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent_ptsz = 436,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback_ptsz = 437,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync_ptsz = 438,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery_ptsz = 439,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize_ptsz = 440,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord_ptsz = 441,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel_ptsz = 442,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources_ptsz = 443,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources_ptsz = 444,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2_ptsz = 445,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerConnect = 446,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerDisconnect = 447,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerPresentFrame = 448,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedEglFrame = 449,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttributes = 450,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessorWithFlags = 451,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSizeWithFlags = 452,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerReturnFrame = 453,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetP2PAttribute = 454,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetBorderColor = 455,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetBorderColor = 456,
    CUPTI_DRIVER_TRACE_CBID_cuMemAdvise = 457,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32 = 458,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_ptsz = 459,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32 = 460,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_ptsz = 461,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp = 462,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_ptsz = 463,
    CUPTI_DRIVER_TRACE_CBID_cuNVNbufferGetPointer = 464,
    CUPTI_DRIVER_TRACE_CBID_cuNVNtextureGetArray = 465,
    CUPTI_DRIVER_TRACE_CBID_cuNNSetAllocator = 466,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync = 467,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_ptsz = 468,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromNVNSync = 469,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnectWithFlags = 470,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttribute = 471,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttributes = 472,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64 = 473,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_ptsz = 474,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64 = 475,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_ptsz = 476,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel = 477,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel_ptsz = 478,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromEGLSync = 479,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernelMultiDevice = 480,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetAttribute = 481,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid = 482,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx = 483,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_ptsz = 484,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalMemory = 485,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedBuffer = 486,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedMipmappedArray = 487,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalMemory = 488,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalSemaphore = 489,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync = 490,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync_ptsz = 491,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync = 492,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync_ptsz = 493,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalSemaphore = 494,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture = 495,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_ptsz = 496,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture = 497,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture_ptsz = 498,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing = 499,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing_ptsz = 500,
    CUPTI_DRIVER_TRACE_CBID_cuGraphCreate = 501,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddKernelNode = 502,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetParams = 503,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemcpyNode = 504,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeGetParams = 505,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemsetNode = 506,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeGetParams = 507,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeSetParams = 508,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetType = 509,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetRootNodes = 510,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependencies = 511,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependentNodes = 512,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate = 513,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch = 514,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch_ptsz = 515,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecDestroy = 516,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroy = 517,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddDependencies = 518,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRemoveDependencies = 519,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeSetParams = 520,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetParams = 521,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroyNode = 522,
    CUPTI_DRIVER_TRACE_CBID_cuGraphClone = 523,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeFindInClone = 524,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddChildGraphNode = 525,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEmptyNode = 526,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc = 527,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc_ptsz = 528,
    CUPTI_DRIVER_TRACE_CBID_cuGraphChildGraphNodeGetGraph = 529,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddHostNode = 530,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeGetParams = 531,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetLuid = 532,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeSetParams = 533,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetNodes = 534,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetEdges = 535,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo = 536,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_ptsz = 537,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecKernelNodeSetParams = 538,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2 = 539,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2_ptsz = 540,
    CUPTI_DRIVER_TRACE_CBID_cuThreadExchangeStreamCaptureMode = 541,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetNvSciSyncAttributes = 542,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyAvailableDynamicSMemPerBlock = 543,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease_v2 = 544,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset_v2 = 545,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags_v2 = 546,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressReserve = 547,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressFree = 548,
    CUPTI_DRIVER_TRACE_CBID_cuMemCreate = 549,
    CUPTI_DRIVER_TRACE_CBID_cuMemRelease = 550,
    CUPTI_DRIVER_TRACE_CBID_cuMemMap = 551,
    CUPTI_DRIVER_TRACE_CBID_cuMemUnmap = 552,
    CUPTI_DRIVER_TRACE_CBID_cuMemSetAccess = 553,
    CUPTI_DRIVER_TRACE_CBID_cuMemExportToShareableHandle = 554,
    CUPTI_DRIVER_TRACE_CBID_cuMemImportFromShareableHandle = 555,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationGranularity = 556,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationPropertiesFromHandle = 557,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAccess = 558,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags = 559,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags_ptsz = 560,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecUpdate = 561,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemcpyNodeSetParams = 562,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemsetNodeSetParams = 563,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecHostNodeSetParams = 564,
    CUPTI_DRIVER_TRACE_CBID_cuMemRetainAllocationHandle = 565,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetModule = 566,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle_v2 = 567,
    CUPTI_DRIVER_TRACE_CBID_cuCtxResetPersistingL2Cache = 568,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeCopyAttributes = 569,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetAttribute = 570,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetAttribute = 571,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes = 572,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes_ptsz = 573,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute = 574,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute_ptsz = 575,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute = 576,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute_ptsz = 577,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate_v2 = 578,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetTexture1DLinearMaxWidth = 579,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload = 580,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload_ptsz = 581,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetSparseProperties = 582,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetSparseProperties = 583,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync = 584,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync_ptsz = 585,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecChildGraphNodeSetParams = 586,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags = 587,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags_ptsz = 588,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventRecordNode = 589,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventWaitNode = 590,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeGetEvent = 591,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeGetEvent = 592,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeSetEvent = 593,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeSetEvent = 594,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventRecordNodeSetEvent = 595,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventWaitNodeSetEvent = 596,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetPlane = 597,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync = 598,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync_ptsz = 599,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync = 600,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync_ptsz = 601,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolTrimTo = 602,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAttribute = 603,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAttribute = 604,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAccess = 605,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetDefaultMemPool = 606,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolCreate = 607,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolDestroy = 608,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetMemPool = 609,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetMemPool = 610,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync = 611,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync_ptsz = 612,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportToShareableHandle = 613,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportFromShareableHandle = 614,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportPointer = 615,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportPointer = 616,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAccess = 617,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresSignalNode = 618,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeGetParams = 619,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeSetParams = 620,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresWaitNode = 621,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeGetParams = 622,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeSetParams = 623,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresSignalNodeSetParams = 624,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresWaitNodeSetParams = 625,
    CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress = 626,
    CUPTI_DRIVER_TRACE_CBID_cuFlushGPUDirectRDMAWrites = 627,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDebugDotPrint = 628,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2 = 629,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2_ptsz = 630,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies = 631,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_ptsz = 632,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectCreate = 633,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRetain = 634,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRelease = 635,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRetainUserObject = 636,
    CUPTI_DRIVER_TRACE_CBID_cuGraphReleaseUserObject = 637,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemAllocNode = 638,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemFreeNode = 639,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGraphMemTrim = 640,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetGraphMemAttribute = 641,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetGraphMemAttribute = 642,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithFlags = 643,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetExecAffinitySupport = 644,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v3 = 645,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetExecAffinity = 646,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid_v2 = 647,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemAllocNodeGetParams = 648,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemFreeNodeGetParams = 649,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeSetEnabled = 650,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetEnabled = 651,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx = 652,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx_ptsz = 653,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetMemoryRequirements = 654,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetMemoryRequirements = 655,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithParams = 656,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithParams_ptsz = 657,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecGetFlags = 658,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_v2 = 659,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_v2_ptsz = 660,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_v2 = 661,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_v2_ptsz = 662,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_v2 = 663,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_v2_ptsz = 664,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_v2 = 665,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_v2_ptsz = 666,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_v2 = 667,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_v2_ptsz = 668,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddBatchMemOpNode = 669,
    CUPTI_DRIVER_TRACE_CBID_cuGraphBatchMemOpNodeGetParams = 670,
    CUPTI_DRIVER_TRACE_CBID_cuGraphBatchMemOpNodeSetParams = 671,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecBatchMemOpNodeSetParams = 672,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetLoadingMode = 673,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetHandleForAddressRange = 674,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialClusterSize = 675,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveClusters = 676,
    CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress_v2 = 677,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryLoadData = 678,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryLoadFromFile = 679,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryUnload = 680,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetKernel = 681,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetModule = 682,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetFunction = 683,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetGlobal = 684,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetManaged = 685,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetAttribute = 686,
    CUPTI_DRIVER_TRACE_CBID_cuKernelSetAttribute = 687,
    CUPTI_DRIVER_TRACE_CBID_cuKernelSetCacheConfig = 688,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddKernelNode_v2 = 689,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetParams_v2 = 690,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetParams_v2 = 691,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecKernelNodeSetParams_v2 = 692,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetId = 693,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetId_ptsz = 694,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetId = 695,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecUpdate_v2 = 696,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeTiled = 697,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeIm2col = 698,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapReplaceAddress = 699,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetUnifiedFunction = 700,
    CUPTI_DRIVER_TRACE_CBID_SIZE = 701,
    CUPTI_DRIVER_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-12010"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_driver_api_trace_cbid_enum {
    CUPTI_DRIVER_TRACE_CBID_INVALID = 0,
    CUPTI_DRIVER_TRACE_CBID_cuInit = 1,
    CUPTI_DRIVER_TRACE_CBID_cuDriverGetVersion = 2,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGet = 3,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetCount = 4,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetName = 5,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceComputeCapability = 6,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem = 7,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetProperties = 8,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetAttribute = 9,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate = 10,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy = 11,
    CUPTI_DRIVER_TRACE_CBID_cuCtxAttach = 12,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDetach = 13,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent = 14,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent = 15,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetDevice = 16,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSynchronize = 17,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoad = 18,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadData = 19,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadDataEx = 20,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadFatBinary = 21,
    CUPTI_DRIVER_TRACE_CBID_cuModuleUnload = 22,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetFunction = 23,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal = 24,
    CUPTI_DRIVER_TRACE_CBID_cu64ModuleGetGlobal = 25,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetTexRef = 26,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo = 27,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetInfo = 28,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc = 29,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAlloc = 30,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch = 31,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAllocPitch = 32,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree = 33,
    CUPTI_DRIVER_TRACE_CBID_cu64MemFree = 34,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange = 35,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetAddressRange = 36,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost = 37,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeHost = 38,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc = 39,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer = 40,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostGetDevicePointer = 41,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetFlags = 42,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD = 43,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoD = 44,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH = 45,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoH = 46,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD = 47,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoD = 48,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA = 49,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoA = 50,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD = 51,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyAtoD = 52,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA = 53,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH = 54,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA = 55,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D = 56,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned = 57,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D = 58,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3D = 59,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync = 60,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoDAsync = 61,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync = 62,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoHAsync = 63,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync = 64,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoDAsync = 65,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync = 66,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync = 67,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync = 68,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync = 69,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3DAsync = 70,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8 = 71,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8 = 72,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16 = 73,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16 = 74,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32 = 75,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32 = 76,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8 = 77,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8 = 78,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16 = 79,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16 = 80,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32 = 81,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32 = 82,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetBlockShape = 83,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedSize = 84,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetAttribute = 85,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetCacheConfig = 86,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate = 87,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor = 88,
    CUPTI_DRIVER_TRACE_CBID_cuArrayDestroy = 89,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate = 90,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor = 91,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefCreate = 92,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefDestroy = 93,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetArray = 94,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress = 95,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress = 96,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D = 97,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress2D = 98,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFormat = 99,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddressMode = 100,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFilterMode = 101,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFlags = 102,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress = 103,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefGetAddress = 104,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetArray = 105,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddressMode = 106,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFilterMode = 107,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFormat = 108,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFlags = 109,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetSize = 110,
    CUPTI_DRIVER_TRACE_CBID_cuParamSeti = 111,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetf = 112,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetv = 113,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetTexRef = 114,
    CUPTI_DRIVER_TRACE_CBID_cuLaunch = 115,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGrid = 116,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGridAsync = 117,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreate = 118,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord = 119,
    CUPTI_DRIVER_TRACE_CBID_cuEventQuery = 120,
    CUPTI_DRIVER_TRACE_CBID_cuEventSynchronize = 121,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy = 122,
    CUPTI_DRIVER_TRACE_CBID_cuEventElapsedTime = 123,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreate = 124,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery = 125,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize = 126,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy = 127,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnregisterResource = 128,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsSubResourceGetMappedArray = 129,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer = 130,
    CUPTI_DRIVER_TRACE_CBID_cu64GraphicsResourceGetMappedPointer = 131,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags = 132,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources = 133,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources = 134,
    CUPTI_DRIVER_TRACE_CBID_cuGetExportTable = 135,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetLimit = 136,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetLimit = 137,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevice = 138,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate = 139,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D10RegisterResource = 140,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10RegisterResource = 141,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnregisterResource = 142,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10MapResources = 143,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnmapResources = 144,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceSetMapFlags = 145,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedArray = 146,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer = 147,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize = 148,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch = 149,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions = 150,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevice = 151,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate = 152,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D11RegisterResource = 153,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevice = 154,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate = 155,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D9RegisterResource = 156,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDirect3DDevice = 157,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterResource = 158,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterResource = 159,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapResources = 160,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapResources = 161,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceSetMapFlags = 162,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions = 163,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedArray = 164,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer = 165,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize = 166,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch = 167,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9Begin = 168,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9End = 169,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterVertexBuffer = 170,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer = 171,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapVertexBuffer = 172,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterVertexBuffer = 173,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate = 174,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterBuffer = 175,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterImage = 176,
    CUPTI_DRIVER_TRACE_CBID_cuWGLGetDevice = 177,
    CUPTI_DRIVER_TRACE_CBID_cuGLInit = 178,
    CUPTI_DRIVER_TRACE_CBID_cuGLRegisterBufferObject = 179,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject = 180,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObject = 181,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnregisterBufferObject = 182,
    CUPTI_DRIVER_TRACE_CBID_cuGLSetBufferObjectMapFlags = 183,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync = 184,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObjectAsync = 185,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUGetDevice = 186,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate = 187,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterVideoSurface = 188,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterOutputSurface = 189,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetSurfRef = 190,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefCreate = 191,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefDestroy = 192,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetFormat = 193,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetArray = 194,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetFormat = 195,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetArray = 196,
    CUPTI_DRIVER_TRACE_CBID_cu64DeviceTotalMem = 197,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPointer = 198,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedSize = 199,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPitch = 200,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetSurfaceDimensions = 201,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetSurfaceDimensions = 202,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPointer = 203,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedSize = 204,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPitch = 205,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9MapVertexBuffer = 206,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObject = 207,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObjectAsync = 208,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevices = 209,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreateOnDevice = 210,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevices = 211,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreateOnDevice = 212,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevices = 213,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreateOnDevice = 214,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostAlloc = 215,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async = 216,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8Async = 217,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async = 218,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16Async = 219,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async = 220,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32Async = 221,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async = 222,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8Async = 223,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async = 224,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16Async = 225,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async = 226,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32Async = 227,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayCreate = 228,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayGetDescriptor = 229,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DCreate = 230,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DGetDescriptor = 231,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2D = 232,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DUnaligned = 233,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DAsync = 234,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v2 = 235,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate_v2 = 236,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate_v2 = 237,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate_v2 = 238,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate_v2 = 239,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate_v2 = 240,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal_v2 = 241,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo_v2 = 242,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc_v2 = 243,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch_v2 = 244,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree_v2 = 245,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange_v2 = 246,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer_v2 = 247,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_v2 = 248,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2 = 249,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2 = 250,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2 = 251,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2 = 252,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2 = 253,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2 = 254,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress_v2 = 255,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v2 = 256,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress_v2 = 257,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer_v2 = 258,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem_v2 = 259,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer_v2 = 260,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize_v2 = 261,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch_v2 = 262,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions_v2 = 263,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions_v2 = 264,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer_v2 = 265,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize_v2 = 266,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch_v2 = 267,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer_v2 = 268,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2 = 269,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2 = 270,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc_v2 = 271,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate_v2 = 272,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor_v2 = 273,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate_v2 = 274,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor_v2 = 275,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2 = 276,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2 = 277,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2 = 278,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2 = 279,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2 = 280,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2 = 281,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2 = 282,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2 = 283,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2 = 284,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2 = 285,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2 = 286,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2 = 287,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2 = 288,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2 = 289,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2 = 290,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2 = 291,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2 = 292,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2 = 293,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost_v2 = 294,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent = 295,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetApiVersion = 296,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDirect3DDevice = 297,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDirect3DDevice = 298,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCacheConfig = 299,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCacheConfig = 300,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister = 301,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostUnregister = 302,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCurrent = 303,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCurrent = 304,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy = 305,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync = 306,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel = 307,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStart = 308,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStop = 309,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttribute = 310,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerInitialize = 311,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceCanAccessPeer = 312,
    CUPTI_DRIVER_TRACE_CBID_cuCtxEnablePeerAccess = 313,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDisablePeerAccess = 314,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerRegister = 315,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerUnregister = 316,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerGetDevicePointer = 317,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer = 318,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync = 319,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer = 320,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync = 321,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy_v2 = 322,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent_v2 = 323,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent_v2 = 324,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy_v2 = 325,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy_v2 = 326,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v3 = 327,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetMemHandle = 328,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle = 329,
    CUPTI_DRIVER_TRACE_CBID_cuIpcCloseMemHandle = 330,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetByPCIBusId = 331,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetPCIBusId = 332,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices = 333,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetEventHandle = 334,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenEventHandle = 335,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetSharedMemConfig = 336,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetSharedMemConfig = 337,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedMemConfig = 338,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectCreate = 339,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectDestroy = 340,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceDesc = 341,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetTextureDesc = 342,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectCreate = 343,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectDestroy = 344,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectGetResourceDesc = 345,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback = 346,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayCreate = 347,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetLevel = 348,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayDestroy = 349,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmappedArray = 350,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapFilterMode = 351,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelBias = 352,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelClamp = 353,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMaxAnisotropy = 354,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmappedArray = 355,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapFilterMode = 356,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelBias = 357,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelClamp = 358,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMaxAnisotropy = 359,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedMipmappedArray = 360,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceViewDesc = 361,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate = 362,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData = 363,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile = 364,
    CUPTI_DRIVER_TRACE_CBID_cuLinkComplete = 365,
    CUPTI_DRIVER_TRACE_CBID_cuLinkDestroy = 366,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreateWithPriority = 367,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority = 368,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags = 369,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetStreamPriorityRange = 370,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocManaged = 371,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorString = 372,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorName = 373,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessor = 374,
    CUPTI_DRIVER_TRACE_CBID_cuCompilePtx = 375,
    CUPTI_DRIVER_TRACE_CBID_cuBinaryFree = 376,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync = 377,
    CUPTI_DRIVER_TRACE_CBID_cuPointerSetAttribute = 378,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister_v2 = 379,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags_v2 = 380,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate_v2 = 381,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData_v2 = 382,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile_v2 = 383,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSize = 384,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices_v2 = 385,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRetain = 386,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease = 387,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags = 388,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset = 389,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsEGLRegisterImage = 390,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetFlags = 391,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxGetState = 392,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnect = 393,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerDisconnect = 394,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerAcquireFrame = 395,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerReleaseFrame = 396,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2_ptds = 397,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2_ptds = 398,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2_ptds = 399,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2_ptds = 400,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2_ptds = 401,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2_ptds = 402,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2_ptds = 403,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2_ptds = 404,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2_ptds = 405,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2_ptds = 406,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2_ptds = 407,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_ptds = 408,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer_ptds = 409,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer_ptds = 410,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2_ptds = 411,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2_ptds = 412,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2_ptds = 413,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2_ptds = 414,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2_ptds = 415,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2_ptds = 416,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2_ptds = 417,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync_ptsz = 418,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2_ptsz = 419,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2_ptsz = 420,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2_ptsz = 421,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2_ptsz = 422,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2_ptsz = 423,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2_ptsz = 424,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2_ptsz = 425,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync_ptsz = 426,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync_ptsz = 427,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async_ptsz = 428,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async_ptsz = 429,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async_ptsz = 430,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async_ptsz = 431,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async_ptsz = 432,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async_ptsz = 433,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority_ptsz = 434,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags_ptsz = 435,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent_ptsz = 436,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback_ptsz = 437,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync_ptsz = 438,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery_ptsz = 439,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize_ptsz = 440,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord_ptsz = 441,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel_ptsz = 442,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources_ptsz = 443,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources_ptsz = 444,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2_ptsz = 445,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerConnect = 446,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerDisconnect = 447,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerPresentFrame = 448,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedEglFrame = 449,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttributes = 450,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessorWithFlags = 451,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSizeWithFlags = 452,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerReturnFrame = 453,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetP2PAttribute = 454,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetBorderColor = 455,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetBorderColor = 456,
    CUPTI_DRIVER_TRACE_CBID_cuMemAdvise = 457,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32 = 458,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_ptsz = 459,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32 = 460,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_ptsz = 461,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp = 462,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_ptsz = 463,
    CUPTI_DRIVER_TRACE_CBID_cuNVNbufferGetPointer = 464,
    CUPTI_DRIVER_TRACE_CBID_cuNVNtextureGetArray = 465,
    CUPTI_DRIVER_TRACE_CBID_cuNNSetAllocator = 466,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync = 467,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_ptsz = 468,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromNVNSync = 469,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnectWithFlags = 470,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttribute = 471,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttributes = 472,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64 = 473,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_ptsz = 474,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64 = 475,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_ptsz = 476,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel = 477,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel_ptsz = 478,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromEGLSync = 479,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernelMultiDevice = 480,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetAttribute = 481,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid = 482,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx = 483,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_ptsz = 484,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalMemory = 485,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedBuffer = 486,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedMipmappedArray = 487,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalMemory = 488,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalSemaphore = 489,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync = 490,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync_ptsz = 491,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync = 492,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync_ptsz = 493,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalSemaphore = 494,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture = 495,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_ptsz = 496,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture = 497,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture_ptsz = 498,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing = 499,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing_ptsz = 500,
    CUPTI_DRIVER_TRACE_CBID_cuGraphCreate = 501,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddKernelNode = 502,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetParams = 503,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemcpyNode = 504,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeGetParams = 505,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemsetNode = 506,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeGetParams = 507,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeSetParams = 508,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetType = 509,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetRootNodes = 510,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependencies = 511,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependentNodes = 512,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate = 513,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch = 514,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch_ptsz = 515,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecDestroy = 516,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroy = 517,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddDependencies = 518,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRemoveDependencies = 519,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeSetParams = 520,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetParams = 521,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroyNode = 522,
    CUPTI_DRIVER_TRACE_CBID_cuGraphClone = 523,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeFindInClone = 524,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddChildGraphNode = 525,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEmptyNode = 526,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc = 527,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc_ptsz = 528,
    CUPTI_DRIVER_TRACE_CBID_cuGraphChildGraphNodeGetGraph = 529,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddHostNode = 530,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeGetParams = 531,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetLuid = 532,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeSetParams = 533,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetNodes = 534,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetEdges = 535,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo = 536,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_ptsz = 537,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecKernelNodeSetParams = 538,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2 = 539,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2_ptsz = 540,
    CUPTI_DRIVER_TRACE_CBID_cuThreadExchangeStreamCaptureMode = 541,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetNvSciSyncAttributes = 542,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyAvailableDynamicSMemPerBlock = 543,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease_v2 = 544,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset_v2 = 545,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags_v2 = 546,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressReserve = 547,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressFree = 548,
    CUPTI_DRIVER_TRACE_CBID_cuMemCreate = 549,
    CUPTI_DRIVER_TRACE_CBID_cuMemRelease = 550,
    CUPTI_DRIVER_TRACE_CBID_cuMemMap = 551,
    CUPTI_DRIVER_TRACE_CBID_cuMemUnmap = 552,
    CUPTI_DRIVER_TRACE_CBID_cuMemSetAccess = 553,
    CUPTI_DRIVER_TRACE_CBID_cuMemExportToShareableHandle = 554,
    CUPTI_DRIVER_TRACE_CBID_cuMemImportFromShareableHandle = 555,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationGranularity = 556,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationPropertiesFromHandle = 557,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAccess = 558,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags = 559,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags_ptsz = 560,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecUpdate = 561,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemcpyNodeSetParams = 562,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemsetNodeSetParams = 563,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecHostNodeSetParams = 564,
    CUPTI_DRIVER_TRACE_CBID_cuMemRetainAllocationHandle = 565,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetModule = 566,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle_v2 = 567,
    CUPTI_DRIVER_TRACE_CBID_cuCtxResetPersistingL2Cache = 568,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeCopyAttributes = 569,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetAttribute = 570,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetAttribute = 571,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes = 572,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes_ptsz = 573,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute = 574,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute_ptsz = 575,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute = 576,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute_ptsz = 577,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate_v2 = 578,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetTexture1DLinearMaxWidth = 579,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload = 580,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload_ptsz = 581,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetSparseProperties = 582,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetSparseProperties = 583,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync = 584,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync_ptsz = 585,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecChildGraphNodeSetParams = 586,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags = 587,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags_ptsz = 588,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventRecordNode = 589,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventWaitNode = 590,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeGetEvent = 591,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeGetEvent = 592,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeSetEvent = 593,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeSetEvent = 594,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventRecordNodeSetEvent = 595,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventWaitNodeSetEvent = 596,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetPlane = 597,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync = 598,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync_ptsz = 599,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync = 600,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync_ptsz = 601,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolTrimTo = 602,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAttribute = 603,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAttribute = 604,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAccess = 605,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetDefaultMemPool = 606,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolCreate = 607,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolDestroy = 608,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetMemPool = 609,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetMemPool = 610,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync = 611,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync_ptsz = 612,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportToShareableHandle = 613,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportFromShareableHandle = 614,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportPointer = 615,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportPointer = 616,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAccess = 617,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresSignalNode = 618,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeGetParams = 619,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeSetParams = 620,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresWaitNode = 621,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeGetParams = 622,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeSetParams = 623,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresSignalNodeSetParams = 624,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresWaitNodeSetParams = 625,
    CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress = 626,
    CUPTI_DRIVER_TRACE_CBID_cuFlushGPUDirectRDMAWrites = 627,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDebugDotPrint = 628,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2 = 629,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2_ptsz = 630,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies = 631,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_ptsz = 632,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectCreate = 633,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRetain = 634,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRelease = 635,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRetainUserObject = 636,
    CUPTI_DRIVER_TRACE_CBID_cuGraphReleaseUserObject = 637,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemAllocNode = 638,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemFreeNode = 639,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGraphMemTrim = 640,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetGraphMemAttribute = 641,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetGraphMemAttribute = 642,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithFlags = 643,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetExecAffinitySupport = 644,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v3 = 645,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetExecAffinity = 646,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid_v2 = 647,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemAllocNodeGetParams = 648,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemFreeNodeGetParams = 649,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeSetEnabled = 650,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetEnabled = 651,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx = 652,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx_ptsz = 653,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetMemoryRequirements = 654,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetMemoryRequirements = 655,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithParams = 656,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithParams_ptsz = 657,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecGetFlags = 658,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_v2 = 659,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_v2_ptsz = 660,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_v2 = 661,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_v2_ptsz = 662,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_v2 = 663,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_v2_ptsz = 664,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_v2 = 665,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_v2_ptsz = 666,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_v2 = 667,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_v2_ptsz = 668,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddBatchMemOpNode = 669,
    CUPTI_DRIVER_TRACE_CBID_cuGraphBatchMemOpNodeGetParams = 670,
    CUPTI_DRIVER_TRACE_CBID_cuGraphBatchMemOpNodeSetParams = 671,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecBatchMemOpNodeSetParams = 672,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetLoadingMode = 673,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetHandleForAddressRange = 674,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialClusterSize = 675,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveClusters = 676,
    CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress_v2 = 677,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryLoadData = 678,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryLoadFromFile = 679,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryUnload = 680,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetKernel = 681,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetModule = 682,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetFunction = 683,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetGlobal = 684,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetManaged = 685,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetAttribute = 686,
    CUPTI_DRIVER_TRACE_CBID_cuKernelSetAttribute = 687,
    CUPTI_DRIVER_TRACE_CBID_cuKernelSetCacheConfig = 688,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddKernelNode_v2 = 689,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetParams_v2 = 690,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetParams_v2 = 691,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecKernelNodeSetParams_v2 = 692,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetId = 693,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetId_ptsz = 694,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetId = 695,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecUpdate_v2 = 696,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeTiled = 697,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeIm2col = 698,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapReplaceAddress = 699,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetUnifiedFunction = 700,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpGetAttribute = 701,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpGetAttributeGlobal = 702,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpSetAttribute = 703,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpSetAttributeGlobal = 704,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetFlags = 705,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastCreate = 706,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastAddDevice = 707,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastBindMem = 708,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastBindAddr = 709,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastUnbind = 710,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastGetGranularity = 711,
    CUPTI_DRIVER_TRACE_CBID_SIZE = 712,
    CUPTI_DRIVER_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-12020"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_driver_api_trace_cbid_enum {
    CUPTI_DRIVER_TRACE_CBID_INVALID = 0,
    CUPTI_DRIVER_TRACE_CBID_cuInit = 1,
    CUPTI_DRIVER_TRACE_CBID_cuDriverGetVersion = 2,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGet = 3,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetCount = 4,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetName = 5,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceComputeCapability = 6,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem = 7,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetProperties = 8,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetAttribute = 9,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate = 10,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy = 11,
    CUPTI_DRIVER_TRACE_CBID_cuCtxAttach = 12,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDetach = 13,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent = 14,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent = 15,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetDevice = 16,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSynchronize = 17,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoad = 18,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadData = 19,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadDataEx = 20,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadFatBinary = 21,
    CUPTI_DRIVER_TRACE_CBID_cuModuleUnload = 22,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetFunction = 23,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal = 24,
    CUPTI_DRIVER_TRACE_CBID_cu64ModuleGetGlobal = 25,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetTexRef = 26,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo = 27,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetInfo = 28,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc = 29,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAlloc = 30,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch = 31,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAllocPitch = 32,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree = 33,
    CUPTI_DRIVER_TRACE_CBID_cu64MemFree = 34,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange = 35,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetAddressRange = 36,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost = 37,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeHost = 38,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc = 39,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer = 40,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostGetDevicePointer = 41,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetFlags = 42,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD = 43,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoD = 44,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH = 45,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoH = 46,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD = 47,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoD = 48,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA = 49,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoA = 50,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD = 51,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyAtoD = 52,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA = 53,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH = 54,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA = 55,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D = 56,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned = 57,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D = 58,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3D = 59,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync = 60,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoDAsync = 61,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync = 62,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoHAsync = 63,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync = 64,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoDAsync = 65,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync = 66,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync = 67,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync = 68,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync = 69,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3DAsync = 70,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8 = 71,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8 = 72,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16 = 73,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16 = 74,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32 = 75,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32 = 76,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8 = 77,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8 = 78,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16 = 79,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16 = 80,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32 = 81,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32 = 82,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetBlockShape = 83,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedSize = 84,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetAttribute = 85,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetCacheConfig = 86,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate = 87,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor = 88,
    CUPTI_DRIVER_TRACE_CBID_cuArrayDestroy = 89,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate = 90,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor = 91,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefCreate = 92,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefDestroy = 93,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetArray = 94,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress = 95,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress = 96,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D = 97,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress2D = 98,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFormat = 99,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddressMode = 100,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFilterMode = 101,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFlags = 102,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress = 103,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefGetAddress = 104,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetArray = 105,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddressMode = 106,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFilterMode = 107,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFormat = 108,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFlags = 109,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetSize = 110,
    CUPTI_DRIVER_TRACE_CBID_cuParamSeti = 111,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetf = 112,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetv = 113,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetTexRef = 114,
    CUPTI_DRIVER_TRACE_CBID_cuLaunch = 115,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGrid = 116,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGridAsync = 117,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreate = 118,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord = 119,
    CUPTI_DRIVER_TRACE_CBID_cuEventQuery = 120,
    CUPTI_DRIVER_TRACE_CBID_cuEventSynchronize = 121,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy = 122,
    CUPTI_DRIVER_TRACE_CBID_cuEventElapsedTime = 123,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreate = 124,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery = 125,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize = 126,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy = 127,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnregisterResource = 128,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsSubResourceGetMappedArray = 129,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer = 130,
    CUPTI_DRIVER_TRACE_CBID_cu64GraphicsResourceGetMappedPointer = 131,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags = 132,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources = 133,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources = 134,
    CUPTI_DRIVER_TRACE_CBID_cuGetExportTable = 135,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetLimit = 136,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetLimit = 137,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevice = 138,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate = 139,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D10RegisterResource = 140,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10RegisterResource = 141,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnregisterResource = 142,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10MapResources = 143,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnmapResources = 144,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceSetMapFlags = 145,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedArray = 146,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer = 147,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize = 148,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch = 149,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions = 150,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevice = 151,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate = 152,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D11RegisterResource = 153,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevice = 154,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate = 155,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D9RegisterResource = 156,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDirect3DDevice = 157,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterResource = 158,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterResource = 159,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapResources = 160,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapResources = 161,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceSetMapFlags = 162,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions = 163,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedArray = 164,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer = 165,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize = 166,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch = 167,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9Begin = 168,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9End = 169,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterVertexBuffer = 170,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer = 171,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapVertexBuffer = 172,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterVertexBuffer = 173,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate = 174,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterBuffer = 175,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterImage = 176,
    CUPTI_DRIVER_TRACE_CBID_cuWGLGetDevice = 177,
    CUPTI_DRIVER_TRACE_CBID_cuGLInit = 178,
    CUPTI_DRIVER_TRACE_CBID_cuGLRegisterBufferObject = 179,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject = 180,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObject = 181,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnregisterBufferObject = 182,
    CUPTI_DRIVER_TRACE_CBID_cuGLSetBufferObjectMapFlags = 183,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync = 184,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObjectAsync = 185,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUGetDevice = 186,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate = 187,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterVideoSurface = 188,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterOutputSurface = 189,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetSurfRef = 190,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefCreate = 191,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefDestroy = 192,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetFormat = 193,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetArray = 194,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetFormat = 195,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetArray = 196,
    CUPTI_DRIVER_TRACE_CBID_cu64DeviceTotalMem = 197,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPointer = 198,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedSize = 199,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPitch = 200,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetSurfaceDimensions = 201,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetSurfaceDimensions = 202,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPointer = 203,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedSize = 204,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPitch = 205,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9MapVertexBuffer = 206,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObject = 207,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObjectAsync = 208,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevices = 209,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreateOnDevice = 210,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevices = 211,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreateOnDevice = 212,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevices = 213,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreateOnDevice = 214,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostAlloc = 215,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async = 216,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8Async = 217,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async = 218,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16Async = 219,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async = 220,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32Async = 221,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async = 222,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8Async = 223,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async = 224,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16Async = 225,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async = 226,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32Async = 227,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayCreate = 228,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayGetDescriptor = 229,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DCreate = 230,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DGetDescriptor = 231,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2D = 232,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DUnaligned = 233,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DAsync = 234,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v2 = 235,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate_v2 = 236,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate_v2 = 237,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate_v2 = 238,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate_v2 = 239,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate_v2 = 240,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal_v2 = 241,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo_v2 = 242,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc_v2 = 243,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch_v2 = 244,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree_v2 = 245,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange_v2 = 246,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer_v2 = 247,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_v2 = 248,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2 = 249,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2 = 250,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2 = 251,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2 = 252,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2 = 253,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2 = 254,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress_v2 = 255,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v2 = 256,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress_v2 = 257,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer_v2 = 258,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem_v2 = 259,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer_v2 = 260,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize_v2 = 261,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch_v2 = 262,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions_v2 = 263,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions_v2 = 264,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer_v2 = 265,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize_v2 = 266,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch_v2 = 267,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer_v2 = 268,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2 = 269,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2 = 270,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc_v2 = 271,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate_v2 = 272,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor_v2 = 273,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate_v2 = 274,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor_v2 = 275,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2 = 276,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2 = 277,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2 = 278,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2 = 279,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2 = 280,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2 = 281,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2 = 282,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2 = 283,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2 = 284,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2 = 285,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2 = 286,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2 = 287,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2 = 288,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2 = 289,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2 = 290,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2 = 291,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2 = 292,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2 = 293,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost_v2 = 294,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent = 295,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetApiVersion = 296,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDirect3DDevice = 297,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDirect3DDevice = 298,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCacheConfig = 299,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCacheConfig = 300,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister = 301,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostUnregister = 302,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCurrent = 303,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCurrent = 304,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy = 305,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync = 306,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel = 307,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStart = 308,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStop = 309,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttribute = 310,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerInitialize = 311,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceCanAccessPeer = 312,
    CUPTI_DRIVER_TRACE_CBID_cuCtxEnablePeerAccess = 313,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDisablePeerAccess = 314,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerRegister = 315,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerUnregister = 316,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerGetDevicePointer = 317,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer = 318,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync = 319,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer = 320,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync = 321,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy_v2 = 322,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent_v2 = 323,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent_v2 = 324,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy_v2 = 325,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy_v2 = 326,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v3 = 327,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetMemHandle = 328,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle = 329,
    CUPTI_DRIVER_TRACE_CBID_cuIpcCloseMemHandle = 330,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetByPCIBusId = 331,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetPCIBusId = 332,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices = 333,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetEventHandle = 334,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenEventHandle = 335,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetSharedMemConfig = 336,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetSharedMemConfig = 337,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedMemConfig = 338,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectCreate = 339,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectDestroy = 340,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceDesc = 341,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetTextureDesc = 342,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectCreate = 343,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectDestroy = 344,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectGetResourceDesc = 345,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback = 346,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayCreate = 347,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetLevel = 348,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayDestroy = 349,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmappedArray = 350,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapFilterMode = 351,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelBias = 352,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelClamp = 353,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMaxAnisotropy = 354,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmappedArray = 355,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapFilterMode = 356,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelBias = 357,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelClamp = 358,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMaxAnisotropy = 359,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedMipmappedArray = 360,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceViewDesc = 361,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate = 362,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData = 363,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile = 364,
    CUPTI_DRIVER_TRACE_CBID_cuLinkComplete = 365,
    CUPTI_DRIVER_TRACE_CBID_cuLinkDestroy = 366,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreateWithPriority = 367,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority = 368,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags = 369,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetStreamPriorityRange = 370,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocManaged = 371,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorString = 372,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorName = 373,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessor = 374,
    CUPTI_DRIVER_TRACE_CBID_cuCompilePtx = 375,
    CUPTI_DRIVER_TRACE_CBID_cuBinaryFree = 376,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync = 377,
    CUPTI_DRIVER_TRACE_CBID_cuPointerSetAttribute = 378,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister_v2 = 379,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags_v2 = 380,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate_v2 = 381,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData_v2 = 382,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile_v2 = 383,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSize = 384,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices_v2 = 385,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRetain = 386,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease = 387,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags = 388,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset = 389,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsEGLRegisterImage = 390,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetFlags = 391,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxGetState = 392,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnect = 393,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerDisconnect = 394,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerAcquireFrame = 395,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerReleaseFrame = 396,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2_ptds = 397,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2_ptds = 398,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2_ptds = 399,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2_ptds = 400,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2_ptds = 401,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2_ptds = 402,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2_ptds = 403,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2_ptds = 404,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2_ptds = 405,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2_ptds = 406,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2_ptds = 407,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_ptds = 408,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer_ptds = 409,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer_ptds = 410,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2_ptds = 411,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2_ptds = 412,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2_ptds = 413,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2_ptds = 414,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2_ptds = 415,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2_ptds = 416,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2_ptds = 417,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync_ptsz = 418,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2_ptsz = 419,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2_ptsz = 420,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2_ptsz = 421,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2_ptsz = 422,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2_ptsz = 423,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2_ptsz = 424,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2_ptsz = 425,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync_ptsz = 426,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync_ptsz = 427,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async_ptsz = 428,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async_ptsz = 429,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async_ptsz = 430,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async_ptsz = 431,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async_ptsz = 432,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async_ptsz = 433,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority_ptsz = 434,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags_ptsz = 435,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent_ptsz = 436,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback_ptsz = 437,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync_ptsz = 438,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery_ptsz = 439,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize_ptsz = 440,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord_ptsz = 441,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel_ptsz = 442,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources_ptsz = 443,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources_ptsz = 444,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2_ptsz = 445,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerConnect = 446,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerDisconnect = 447,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerPresentFrame = 448,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedEglFrame = 449,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttributes = 450,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessorWithFlags = 451,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSizeWithFlags = 452,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerReturnFrame = 453,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetP2PAttribute = 454,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetBorderColor = 455,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetBorderColor = 456,
    CUPTI_DRIVER_TRACE_CBID_cuMemAdvise = 457,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32 = 458,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_ptsz = 459,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32 = 460,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_ptsz = 461,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp = 462,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_ptsz = 463,
    CUPTI_DRIVER_TRACE_CBID_cuNVNbufferGetPointer = 464,
    CUPTI_DRIVER_TRACE_CBID_cuNVNtextureGetArray = 465,
    CUPTI_DRIVER_TRACE_CBID_cuNNSetAllocator = 466,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync = 467,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_ptsz = 468,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromNVNSync = 469,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnectWithFlags = 470,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttribute = 471,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttributes = 472,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64 = 473,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_ptsz = 474,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64 = 475,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_ptsz = 476,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel = 477,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel_ptsz = 478,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromEGLSync = 479,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernelMultiDevice = 480,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetAttribute = 481,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid = 482,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx = 483,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_ptsz = 484,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalMemory = 485,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedBuffer = 486,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedMipmappedArray = 487,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalMemory = 488,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalSemaphore = 489,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync = 490,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync_ptsz = 491,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync = 492,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync_ptsz = 493,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalSemaphore = 494,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture = 495,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_ptsz = 496,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture = 497,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture_ptsz = 498,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing = 499,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing_ptsz = 500,
    CUPTI_DRIVER_TRACE_CBID_cuGraphCreate = 501,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddKernelNode = 502,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetParams = 503,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemcpyNode = 504,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeGetParams = 505,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemsetNode = 506,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeGetParams = 507,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeSetParams = 508,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetType = 509,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetRootNodes = 510,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependencies = 511,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependentNodes = 512,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate = 513,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch = 514,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch_ptsz = 515,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecDestroy = 516,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroy = 517,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddDependencies = 518,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRemoveDependencies = 519,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeSetParams = 520,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetParams = 521,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroyNode = 522,
    CUPTI_DRIVER_TRACE_CBID_cuGraphClone = 523,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeFindInClone = 524,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddChildGraphNode = 525,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEmptyNode = 526,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc = 527,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc_ptsz = 528,
    CUPTI_DRIVER_TRACE_CBID_cuGraphChildGraphNodeGetGraph = 529,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddHostNode = 530,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeGetParams = 531,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetLuid = 532,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeSetParams = 533,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetNodes = 534,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetEdges = 535,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo = 536,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_ptsz = 537,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecKernelNodeSetParams = 538,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2 = 539,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2_ptsz = 540,
    CUPTI_DRIVER_TRACE_CBID_cuThreadExchangeStreamCaptureMode = 541,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetNvSciSyncAttributes = 542,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyAvailableDynamicSMemPerBlock = 543,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease_v2 = 544,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset_v2 = 545,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags_v2 = 546,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressReserve = 547,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressFree = 548,
    CUPTI_DRIVER_TRACE_CBID_cuMemCreate = 549,
    CUPTI_DRIVER_TRACE_CBID_cuMemRelease = 550,
    CUPTI_DRIVER_TRACE_CBID_cuMemMap = 551,
    CUPTI_DRIVER_TRACE_CBID_cuMemUnmap = 552,
    CUPTI_DRIVER_TRACE_CBID_cuMemSetAccess = 553,
    CUPTI_DRIVER_TRACE_CBID_cuMemExportToShareableHandle = 554,
    CUPTI_DRIVER_TRACE_CBID_cuMemImportFromShareableHandle = 555,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationGranularity = 556,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationPropertiesFromHandle = 557,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAccess = 558,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags = 559,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags_ptsz = 560,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecUpdate = 561,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemcpyNodeSetParams = 562,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemsetNodeSetParams = 563,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecHostNodeSetParams = 564,
    CUPTI_DRIVER_TRACE_CBID_cuMemRetainAllocationHandle = 565,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetModule = 566,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle_v2 = 567,
    CUPTI_DRIVER_TRACE_CBID_cuCtxResetPersistingL2Cache = 568,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeCopyAttributes = 569,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetAttribute = 570,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetAttribute = 571,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes = 572,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes_ptsz = 573,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute = 574,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute_ptsz = 575,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute = 576,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute_ptsz = 577,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate_v2 = 578,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetTexture1DLinearMaxWidth = 579,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload = 580,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload_ptsz = 581,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetSparseProperties = 582,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetSparseProperties = 583,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync = 584,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync_ptsz = 585,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecChildGraphNodeSetParams = 586,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags = 587,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags_ptsz = 588,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventRecordNode = 589,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventWaitNode = 590,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeGetEvent = 591,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeGetEvent = 592,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeSetEvent = 593,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeSetEvent = 594,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventRecordNodeSetEvent = 595,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventWaitNodeSetEvent = 596,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetPlane = 597,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync = 598,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync_ptsz = 599,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync = 600,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync_ptsz = 601,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolTrimTo = 602,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAttribute = 603,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAttribute = 604,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAccess = 605,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetDefaultMemPool = 606,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolCreate = 607,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolDestroy = 608,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetMemPool = 609,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetMemPool = 610,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync = 611,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync_ptsz = 612,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportToShareableHandle = 613,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportFromShareableHandle = 614,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportPointer = 615,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportPointer = 616,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAccess = 617,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresSignalNode = 618,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeGetParams = 619,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeSetParams = 620,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresWaitNode = 621,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeGetParams = 622,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeSetParams = 623,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresSignalNodeSetParams = 624,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresWaitNodeSetParams = 625,
    CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress = 626,
    CUPTI_DRIVER_TRACE_CBID_cuFlushGPUDirectRDMAWrites = 627,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDebugDotPrint = 628,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2 = 629,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2_ptsz = 630,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies = 631,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_ptsz = 632,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectCreate = 633,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRetain = 634,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRelease = 635,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRetainUserObject = 636,
    CUPTI_DRIVER_TRACE_CBID_cuGraphReleaseUserObject = 637,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemAllocNode = 638,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemFreeNode = 639,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGraphMemTrim = 640,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetGraphMemAttribute = 641,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetGraphMemAttribute = 642,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithFlags = 643,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetExecAffinitySupport = 644,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v3 = 645,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetExecAffinity = 646,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid_v2 = 647,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemAllocNodeGetParams = 648,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemFreeNodeGetParams = 649,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeSetEnabled = 650,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetEnabled = 651,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx = 652,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx_ptsz = 653,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetMemoryRequirements = 654,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetMemoryRequirements = 655,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithParams = 656,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithParams_ptsz = 657,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecGetFlags = 658,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_v2 = 659,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_v2_ptsz = 660,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_v2 = 661,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_v2_ptsz = 662,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_v2 = 663,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_v2_ptsz = 664,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_v2 = 665,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_v2_ptsz = 666,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_v2 = 667,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_v2_ptsz = 668,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddBatchMemOpNode = 669,
    CUPTI_DRIVER_TRACE_CBID_cuGraphBatchMemOpNodeGetParams = 670,
    CUPTI_DRIVER_TRACE_CBID_cuGraphBatchMemOpNodeSetParams = 671,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecBatchMemOpNodeSetParams = 672,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetLoadingMode = 673,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetHandleForAddressRange = 674,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialClusterSize = 675,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveClusters = 676,
    CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress_v2 = 677,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryLoadData = 678,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryLoadFromFile = 679,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryUnload = 680,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetKernel = 681,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetModule = 682,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetFunction = 683,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetGlobal = 684,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetManaged = 685,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetAttribute = 686,
    CUPTI_DRIVER_TRACE_CBID_cuKernelSetAttribute = 687,
    CUPTI_DRIVER_TRACE_CBID_cuKernelSetCacheConfig = 688,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddKernelNode_v2 = 689,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetParams_v2 = 690,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetParams_v2 = 691,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecKernelNodeSetParams_v2 = 692,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetId = 693,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetId_ptsz = 694,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetId = 695,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecUpdate_v2 = 696,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeTiled = 697,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeIm2col = 698,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapReplaceAddress = 699,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetUnifiedFunction = 700,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpGetAttribute = 701,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpGetAttributeGlobal = 702,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpSetAttribute = 703,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpSetAttributeGlobal = 704,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetFlags = 705,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastCreate = 706,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastAddDevice = 707,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastBindMem = 708,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastBindAddr = 709,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastUnbind = 710,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastGetGranularity = 711,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddNode = 712,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeSetParams = 713,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecNodeSetParams = 714,
    CUPTI_DRIVER_TRACE_CBID_cuMemAdvise_v2 = 715,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_v2 = 716,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_v2_ptsz = 717,
    CUPTI_DRIVER_TRACE_CBID_SIZE = 718,
    CUPTI_DRIVER_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-12030"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_driver_api_trace_cbid_enum {
    CUPTI_DRIVER_TRACE_CBID_INVALID = 0,
    CUPTI_DRIVER_TRACE_CBID_cuInit = 1,
    CUPTI_DRIVER_TRACE_CBID_cuDriverGetVersion = 2,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGet = 3,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetCount = 4,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetName = 5,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceComputeCapability = 6,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem = 7,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetProperties = 8,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetAttribute = 9,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate = 10,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy = 11,
    CUPTI_DRIVER_TRACE_CBID_cuCtxAttach = 12,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDetach = 13,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent = 14,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent = 15,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetDevice = 16,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSynchronize = 17,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoad = 18,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadData = 19,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadDataEx = 20,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadFatBinary = 21,
    CUPTI_DRIVER_TRACE_CBID_cuModuleUnload = 22,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetFunction = 23,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal = 24,
    CUPTI_DRIVER_TRACE_CBID_cu64ModuleGetGlobal = 25,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetTexRef = 26,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo = 27,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetInfo = 28,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc = 29,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAlloc = 30,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch = 31,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAllocPitch = 32,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree = 33,
    CUPTI_DRIVER_TRACE_CBID_cu64MemFree = 34,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange = 35,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetAddressRange = 36,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost = 37,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeHost = 38,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc = 39,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer = 40,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostGetDevicePointer = 41,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetFlags = 42,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD = 43,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoD = 44,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH = 45,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoH = 46,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD = 47,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoD = 48,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA = 49,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoA = 50,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD = 51,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyAtoD = 52,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA = 53,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH = 54,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA = 55,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D = 56,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned = 57,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D = 58,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3D = 59,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync = 60,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoDAsync = 61,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync = 62,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoHAsync = 63,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync = 64,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoDAsync = 65,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync = 66,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync = 67,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync = 68,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync = 69,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3DAsync = 70,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8 = 71,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8 = 72,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16 = 73,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16 = 74,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32 = 75,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32 = 76,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8 = 77,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8 = 78,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16 = 79,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16 = 80,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32 = 81,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32 = 82,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetBlockShape = 83,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedSize = 84,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetAttribute = 85,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetCacheConfig = 86,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate = 87,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor = 88,
    CUPTI_DRIVER_TRACE_CBID_cuArrayDestroy = 89,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate = 90,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor = 91,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefCreate = 92,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefDestroy = 93,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetArray = 94,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress = 95,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress = 96,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D = 97,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress2D = 98,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFormat = 99,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddressMode = 100,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFilterMode = 101,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFlags = 102,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress = 103,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefGetAddress = 104,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetArray = 105,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddressMode = 106,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFilterMode = 107,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFormat = 108,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFlags = 109,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetSize = 110,
    CUPTI_DRIVER_TRACE_CBID_cuParamSeti = 111,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetf = 112,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetv = 113,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetTexRef = 114,
    CUPTI_DRIVER_TRACE_CBID_cuLaunch = 115,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGrid = 116,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGridAsync = 117,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreate = 118,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord = 119,
    CUPTI_DRIVER_TRACE_CBID_cuEventQuery = 120,
    CUPTI_DRIVER_TRACE_CBID_cuEventSynchronize = 121,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy = 122,
    CUPTI_DRIVER_TRACE_CBID_cuEventElapsedTime = 123,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreate = 124,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery = 125,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize = 126,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy = 127,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnregisterResource = 128,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsSubResourceGetMappedArray = 129,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer = 130,
    CUPTI_DRIVER_TRACE_CBID_cu64GraphicsResourceGetMappedPointer = 131,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags = 132,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources = 133,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources = 134,
    CUPTI_DRIVER_TRACE_CBID_cuGetExportTable = 135,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetLimit = 136,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetLimit = 137,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevice = 138,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate = 139,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D10RegisterResource = 140,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10RegisterResource = 141,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnregisterResource = 142,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10MapResources = 143,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnmapResources = 144,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceSetMapFlags = 145,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedArray = 146,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer = 147,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize = 148,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch = 149,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions = 150,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevice = 151,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate = 152,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D11RegisterResource = 153,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevice = 154,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate = 155,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D9RegisterResource = 156,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDirect3DDevice = 157,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterResource = 158,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterResource = 159,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapResources = 160,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapResources = 161,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceSetMapFlags = 162,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions = 163,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedArray = 164,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer = 165,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize = 166,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch = 167,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9Begin = 168,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9End = 169,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterVertexBuffer = 170,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer = 171,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapVertexBuffer = 172,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterVertexBuffer = 173,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate = 174,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterBuffer = 175,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterImage = 176,
    CUPTI_DRIVER_TRACE_CBID_cuWGLGetDevice = 177,
    CUPTI_DRIVER_TRACE_CBID_cuGLInit = 178,
    CUPTI_DRIVER_TRACE_CBID_cuGLRegisterBufferObject = 179,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject = 180,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObject = 181,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnregisterBufferObject = 182,
    CUPTI_DRIVER_TRACE_CBID_cuGLSetBufferObjectMapFlags = 183,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync = 184,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObjectAsync = 185,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUGetDevice = 186,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate = 187,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterVideoSurface = 188,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterOutputSurface = 189,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetSurfRef = 190,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefCreate = 191,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefDestroy = 192,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetFormat = 193,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetArray = 194,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetFormat = 195,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetArray = 196,
    CUPTI_DRIVER_TRACE_CBID_cu64DeviceTotalMem = 197,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPointer = 198,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedSize = 199,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPitch = 200,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetSurfaceDimensions = 201,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetSurfaceDimensions = 202,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPointer = 203,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedSize = 204,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPitch = 205,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9MapVertexBuffer = 206,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObject = 207,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObjectAsync = 208,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevices = 209,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreateOnDevice = 210,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevices = 211,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreateOnDevice = 212,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevices = 213,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreateOnDevice = 214,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostAlloc = 215,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async = 216,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8Async = 217,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async = 218,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16Async = 219,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async = 220,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32Async = 221,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async = 222,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8Async = 223,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async = 224,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16Async = 225,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async = 226,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32Async = 227,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayCreate = 228,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayGetDescriptor = 229,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DCreate = 230,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DGetDescriptor = 231,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2D = 232,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DUnaligned = 233,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DAsync = 234,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v2 = 235,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate_v2 = 236,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate_v2 = 237,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate_v2 = 238,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate_v2 = 239,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate_v2 = 240,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal_v2 = 241,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo_v2 = 242,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc_v2 = 243,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch_v2 = 244,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree_v2 = 245,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange_v2 = 246,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer_v2 = 247,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_v2 = 248,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2 = 249,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2 = 250,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2 = 251,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2 = 252,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2 = 253,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2 = 254,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress_v2 = 255,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v2 = 256,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress_v2 = 257,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer_v2 = 258,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem_v2 = 259,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer_v2 = 260,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize_v2 = 261,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch_v2 = 262,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions_v2 = 263,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions_v2 = 264,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer_v2 = 265,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize_v2 = 266,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch_v2 = 267,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer_v2 = 268,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2 = 269,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2 = 270,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc_v2 = 271,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate_v2 = 272,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor_v2 = 273,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate_v2 = 274,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor_v2 = 275,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2 = 276,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2 = 277,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2 = 278,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2 = 279,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2 = 280,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2 = 281,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2 = 282,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2 = 283,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2 = 284,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2 = 285,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2 = 286,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2 = 287,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2 = 288,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2 = 289,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2 = 290,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2 = 291,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2 = 292,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2 = 293,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost_v2 = 294,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent = 295,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetApiVersion = 296,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDirect3DDevice = 297,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDirect3DDevice = 298,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCacheConfig = 299,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCacheConfig = 300,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister = 301,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostUnregister = 302,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCurrent = 303,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCurrent = 304,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy = 305,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync = 306,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel = 307,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStart = 308,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStop = 309,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttribute = 310,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerInitialize = 311,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceCanAccessPeer = 312,
    CUPTI_DRIVER_TRACE_CBID_cuCtxEnablePeerAccess = 313,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDisablePeerAccess = 314,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerRegister = 315,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerUnregister = 316,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerGetDevicePointer = 317,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer = 318,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync = 319,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer = 320,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync = 321,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy_v2 = 322,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent_v2 = 323,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent_v2 = 324,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy_v2 = 325,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy_v2 = 326,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v3 = 327,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetMemHandle = 328,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle = 329,
    CUPTI_DRIVER_TRACE_CBID_cuIpcCloseMemHandle = 330,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetByPCIBusId = 331,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetPCIBusId = 332,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices = 333,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetEventHandle = 334,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenEventHandle = 335,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetSharedMemConfig = 336,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetSharedMemConfig = 337,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedMemConfig = 338,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectCreate = 339,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectDestroy = 340,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceDesc = 341,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetTextureDesc = 342,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectCreate = 343,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectDestroy = 344,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectGetResourceDesc = 345,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback = 346,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayCreate = 347,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetLevel = 348,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayDestroy = 349,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmappedArray = 350,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapFilterMode = 351,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelBias = 352,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelClamp = 353,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMaxAnisotropy = 354,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmappedArray = 355,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapFilterMode = 356,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelBias = 357,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelClamp = 358,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMaxAnisotropy = 359,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedMipmappedArray = 360,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceViewDesc = 361,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate = 362,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData = 363,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile = 364,
    CUPTI_DRIVER_TRACE_CBID_cuLinkComplete = 365,
    CUPTI_DRIVER_TRACE_CBID_cuLinkDestroy = 366,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreateWithPriority = 367,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority = 368,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags = 369,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetStreamPriorityRange = 370,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocManaged = 371,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorString = 372,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorName = 373,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessor = 374,
    CUPTI_DRIVER_TRACE_CBID_cuCompilePtx = 375,
    CUPTI_DRIVER_TRACE_CBID_cuBinaryFree = 376,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync = 377,
    CUPTI_DRIVER_TRACE_CBID_cuPointerSetAttribute = 378,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister_v2 = 379,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags_v2 = 380,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate_v2 = 381,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData_v2 = 382,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile_v2 = 383,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSize = 384,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices_v2 = 385,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRetain = 386,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease = 387,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags = 388,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset = 389,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsEGLRegisterImage = 390,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetFlags = 391,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxGetState = 392,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnect = 393,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerDisconnect = 394,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerAcquireFrame = 395,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerReleaseFrame = 396,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2_ptds = 397,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2_ptds = 398,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2_ptds = 399,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2_ptds = 400,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2_ptds = 401,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2_ptds = 402,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2_ptds = 403,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2_ptds = 404,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2_ptds = 405,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2_ptds = 406,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2_ptds = 407,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_ptds = 408,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer_ptds = 409,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer_ptds = 410,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2_ptds = 411,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2_ptds = 412,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2_ptds = 413,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2_ptds = 414,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2_ptds = 415,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2_ptds = 416,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2_ptds = 417,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync_ptsz = 418,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2_ptsz = 419,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2_ptsz = 420,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2_ptsz = 421,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2_ptsz = 422,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2_ptsz = 423,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2_ptsz = 424,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2_ptsz = 425,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync_ptsz = 426,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync_ptsz = 427,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async_ptsz = 428,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async_ptsz = 429,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async_ptsz = 430,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async_ptsz = 431,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async_ptsz = 432,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async_ptsz = 433,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority_ptsz = 434,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags_ptsz = 435,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent_ptsz = 436,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback_ptsz = 437,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync_ptsz = 438,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery_ptsz = 439,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize_ptsz = 440,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord_ptsz = 441,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel_ptsz = 442,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources_ptsz = 443,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources_ptsz = 444,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2_ptsz = 445,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerConnect = 446,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerDisconnect = 447,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerPresentFrame = 448,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedEglFrame = 449,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttributes = 450,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessorWithFlags = 451,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSizeWithFlags = 452,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerReturnFrame = 453,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetP2PAttribute = 454,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetBorderColor = 455,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetBorderColor = 456,
    CUPTI_DRIVER_TRACE_CBID_cuMemAdvise = 457,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32 = 458,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_ptsz = 459,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32 = 460,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_ptsz = 461,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp = 462,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_ptsz = 463,
    CUPTI_DRIVER_TRACE_CBID_cuNVNbufferGetPointer = 464,
    CUPTI_DRIVER_TRACE_CBID_cuNVNtextureGetArray = 465,
    CUPTI_DRIVER_TRACE_CBID_cuNNSetAllocator = 466,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync = 467,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_ptsz = 468,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromNVNSync = 469,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnectWithFlags = 470,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttribute = 471,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttributes = 472,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64 = 473,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_ptsz = 474,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64 = 475,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_ptsz = 476,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel = 477,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel_ptsz = 478,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromEGLSync = 479,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernelMultiDevice = 480,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetAttribute = 481,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid = 482,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx = 483,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_ptsz = 484,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalMemory = 485,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedBuffer = 486,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedMipmappedArray = 487,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalMemory = 488,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalSemaphore = 489,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync = 490,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync_ptsz = 491,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync = 492,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync_ptsz = 493,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalSemaphore = 494,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture = 495,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_ptsz = 496,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture = 497,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture_ptsz = 498,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing = 499,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing_ptsz = 500,
    CUPTI_DRIVER_TRACE_CBID_cuGraphCreate = 501,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddKernelNode = 502,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetParams = 503,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemcpyNode = 504,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeGetParams = 505,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemsetNode = 506,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeGetParams = 507,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeSetParams = 508,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetType = 509,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetRootNodes = 510,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependencies = 511,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependentNodes = 512,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate = 513,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch = 514,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch_ptsz = 515,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecDestroy = 516,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroy = 517,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddDependencies = 518,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRemoveDependencies = 519,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeSetParams = 520,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetParams = 521,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroyNode = 522,
    CUPTI_DRIVER_TRACE_CBID_cuGraphClone = 523,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeFindInClone = 524,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddChildGraphNode = 525,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEmptyNode = 526,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc = 527,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc_ptsz = 528,
    CUPTI_DRIVER_TRACE_CBID_cuGraphChildGraphNodeGetGraph = 529,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddHostNode = 530,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeGetParams = 531,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetLuid = 532,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeSetParams = 533,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetNodes = 534,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetEdges = 535,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo = 536,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_ptsz = 537,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecKernelNodeSetParams = 538,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2 = 539,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2_ptsz = 540,
    CUPTI_DRIVER_TRACE_CBID_cuThreadExchangeStreamCaptureMode = 541,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetNvSciSyncAttributes = 542,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyAvailableDynamicSMemPerBlock = 543,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease_v2 = 544,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset_v2 = 545,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags_v2 = 546,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressReserve = 547,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressFree = 548,
    CUPTI_DRIVER_TRACE_CBID_cuMemCreate = 549,
    CUPTI_DRIVER_TRACE_CBID_cuMemRelease = 550,
    CUPTI_DRIVER_TRACE_CBID_cuMemMap = 551,
    CUPTI_DRIVER_TRACE_CBID_cuMemUnmap = 552,
    CUPTI_DRIVER_TRACE_CBID_cuMemSetAccess = 553,
    CUPTI_DRIVER_TRACE_CBID_cuMemExportToShareableHandle = 554,
    CUPTI_DRIVER_TRACE_CBID_cuMemImportFromShareableHandle = 555,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationGranularity = 556,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationPropertiesFromHandle = 557,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAccess = 558,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags = 559,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags_ptsz = 560,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecUpdate = 561,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemcpyNodeSetParams = 562,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemsetNodeSetParams = 563,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecHostNodeSetParams = 564,
    CUPTI_DRIVER_TRACE_CBID_cuMemRetainAllocationHandle = 565,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetModule = 566,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle_v2 = 567,
    CUPTI_DRIVER_TRACE_CBID_cuCtxResetPersistingL2Cache = 568,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeCopyAttributes = 569,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetAttribute = 570,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetAttribute = 571,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes = 572,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes_ptsz = 573,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute = 574,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute_ptsz = 575,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute = 576,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute_ptsz = 577,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate_v2 = 578,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetTexture1DLinearMaxWidth = 579,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload = 580,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload_ptsz = 581,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetSparseProperties = 582,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetSparseProperties = 583,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync = 584,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync_ptsz = 585,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecChildGraphNodeSetParams = 586,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags = 587,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags_ptsz = 588,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventRecordNode = 589,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventWaitNode = 590,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeGetEvent = 591,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeGetEvent = 592,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeSetEvent = 593,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeSetEvent = 594,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventRecordNodeSetEvent = 595,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventWaitNodeSetEvent = 596,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetPlane = 597,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync = 598,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync_ptsz = 599,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync = 600,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync_ptsz = 601,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolTrimTo = 602,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAttribute = 603,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAttribute = 604,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAccess = 605,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetDefaultMemPool = 606,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolCreate = 607,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolDestroy = 608,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetMemPool = 609,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetMemPool = 610,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync = 611,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync_ptsz = 612,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportToShareableHandle = 613,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportFromShareableHandle = 614,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportPointer = 615,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportPointer = 616,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAccess = 617,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresSignalNode = 618,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeGetParams = 619,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeSetParams = 620,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresWaitNode = 621,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeGetParams = 622,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeSetParams = 623,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresSignalNodeSetParams = 624,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresWaitNodeSetParams = 625,
    CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress = 626,
    CUPTI_DRIVER_TRACE_CBID_cuFlushGPUDirectRDMAWrites = 627,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDebugDotPrint = 628,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2 = 629,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2_ptsz = 630,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies = 631,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_ptsz = 632,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectCreate = 633,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRetain = 634,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRelease = 635,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRetainUserObject = 636,
    CUPTI_DRIVER_TRACE_CBID_cuGraphReleaseUserObject = 637,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemAllocNode = 638,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemFreeNode = 639,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGraphMemTrim = 640,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetGraphMemAttribute = 641,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetGraphMemAttribute = 642,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithFlags = 643,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetExecAffinitySupport = 644,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v3 = 645,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetExecAffinity = 646,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid_v2 = 647,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemAllocNodeGetParams = 648,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemFreeNodeGetParams = 649,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeSetEnabled = 650,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetEnabled = 651,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx = 652,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx_ptsz = 653,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetMemoryRequirements = 654,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetMemoryRequirements = 655,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithParams = 656,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithParams_ptsz = 657,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecGetFlags = 658,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_v2 = 659,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_v2_ptsz = 660,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_v2 = 661,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_v2_ptsz = 662,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_v2 = 663,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_v2_ptsz = 664,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_v2 = 665,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_v2_ptsz = 666,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_v2 = 667,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_v2_ptsz = 668,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddBatchMemOpNode = 669,
    CUPTI_DRIVER_TRACE_CBID_cuGraphBatchMemOpNodeGetParams = 670,
    CUPTI_DRIVER_TRACE_CBID_cuGraphBatchMemOpNodeSetParams = 671,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecBatchMemOpNodeSetParams = 672,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetLoadingMode = 673,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetHandleForAddressRange = 674,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialClusterSize = 675,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveClusters = 676,
    CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress_v2 = 677,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryLoadData = 678,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryLoadFromFile = 679,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryUnload = 680,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetKernel = 681,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetModule = 682,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetFunction = 683,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetGlobal = 684,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetManaged = 685,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetAttribute = 686,
    CUPTI_DRIVER_TRACE_CBID_cuKernelSetAttribute = 687,
    CUPTI_DRIVER_TRACE_CBID_cuKernelSetCacheConfig = 688,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddKernelNode_v2 = 689,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetParams_v2 = 690,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetParams_v2 = 691,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecKernelNodeSetParams_v2 = 692,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetId = 693,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetId_ptsz = 694,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetId = 695,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecUpdate_v2 = 696,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeTiled = 697,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeIm2col = 698,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapReplaceAddress = 699,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetUnifiedFunction = 700,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpGetAttribute = 701,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpGetAttributeGlobal = 702,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpSetAttribute = 703,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpSetAttributeGlobal = 704,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetFlags = 705,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastCreate = 706,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastAddDevice = 707,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastBindMem = 708,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastBindAddr = 709,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastUnbind = 710,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastGetGranularity = 711,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddNode = 712,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeSetParams = 713,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecNodeSetParams = 714,
    CUPTI_DRIVER_TRACE_CBID_cuMemAdvise_v2 = 715,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_v2 = 716,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_v2_ptsz = 717,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetName = 718,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetName = 719,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCaptureToGraph = 720,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCaptureToGraph_ptsz = 721,
    CUPTI_DRIVER_TRACE_CBID_cuGraphConditionalHandleCreate = 722,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddNode_v2 = 723,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetEdges_v2 = 724,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependencies_v2 = 725,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependentNodes_v2 = 726,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddDependencies_v2 = 727,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRemoveDependencies_v2 = 728,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v3 = 729,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v3_ptsz = 730,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_v2 = 731,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_v2_ptsz = 732,
    CUPTI_DRIVER_TRACE_CBID_SIZE = 733,
    CUPTI_DRIVER_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-12040"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_driver_api_trace_cbid_enum {
    CUPTI_DRIVER_TRACE_CBID_INVALID = 0,
    CUPTI_DRIVER_TRACE_CBID_cuInit = 1,
    CUPTI_DRIVER_TRACE_CBID_cuDriverGetVersion = 2,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGet = 3,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetCount = 4,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetName = 5,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceComputeCapability = 6,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem = 7,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetProperties = 8,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetAttribute = 9,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate = 10,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy = 11,
    CUPTI_DRIVER_TRACE_CBID_cuCtxAttach = 12,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDetach = 13,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent = 14,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent = 15,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetDevice = 16,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSynchronize = 17,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoad = 18,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadData = 19,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadDataEx = 20,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadFatBinary = 21,
    CUPTI_DRIVER_TRACE_CBID_cuModuleUnload = 22,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetFunction = 23,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal = 24,
    CUPTI_DRIVER_TRACE_CBID_cu64ModuleGetGlobal = 25,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetTexRef = 26,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo = 27,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetInfo = 28,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc = 29,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAlloc = 30,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch = 31,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAllocPitch = 32,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree = 33,
    CUPTI_DRIVER_TRACE_CBID_cu64MemFree = 34,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange = 35,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetAddressRange = 36,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost = 37,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeHost = 38,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc = 39,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer = 40,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostGetDevicePointer = 41,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetFlags = 42,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD = 43,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoD = 44,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH = 45,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoH = 46,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD = 47,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoD = 48,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA = 49,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoA = 50,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD = 51,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyAtoD = 52,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA = 53,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH = 54,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA = 55,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D = 56,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned = 57,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D = 58,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3D = 59,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync = 60,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoDAsync = 61,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync = 62,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoHAsync = 63,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync = 64,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoDAsync = 65,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync = 66,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync = 67,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync = 68,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync = 69,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3DAsync = 70,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8 = 71,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8 = 72,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16 = 73,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16 = 74,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32 = 75,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32 = 76,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8 = 77,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8 = 78,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16 = 79,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16 = 80,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32 = 81,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32 = 82,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetBlockShape = 83,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedSize = 84,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetAttribute = 85,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetCacheConfig = 86,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate = 87,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor = 88,
    CUPTI_DRIVER_TRACE_CBID_cuArrayDestroy = 89,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate = 90,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor = 91,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefCreate = 92,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefDestroy = 93,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetArray = 94,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress = 95,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress = 96,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D = 97,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress2D = 98,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFormat = 99,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddressMode = 100,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFilterMode = 101,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFlags = 102,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress = 103,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefGetAddress = 104,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetArray = 105,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddressMode = 106,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFilterMode = 107,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFormat = 108,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFlags = 109,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetSize = 110,
    CUPTI_DRIVER_TRACE_CBID_cuParamSeti = 111,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetf = 112,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetv = 113,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetTexRef = 114,
    CUPTI_DRIVER_TRACE_CBID_cuLaunch = 115,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGrid = 116,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGridAsync = 117,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreate = 118,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord = 119,
    CUPTI_DRIVER_TRACE_CBID_cuEventQuery = 120,
    CUPTI_DRIVER_TRACE_CBID_cuEventSynchronize = 121,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy = 122,
    CUPTI_DRIVER_TRACE_CBID_cuEventElapsedTime = 123,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreate = 124,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery = 125,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize = 126,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy = 127,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnregisterResource = 128,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsSubResourceGetMappedArray = 129,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer = 130,
    CUPTI_DRIVER_TRACE_CBID_cu64GraphicsResourceGetMappedPointer = 131,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags = 132,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources = 133,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources = 134,
    CUPTI_DRIVER_TRACE_CBID_cuGetExportTable = 135,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetLimit = 136,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetLimit = 137,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevice = 138,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate = 139,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D10RegisterResource = 140,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10RegisterResource = 141,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnregisterResource = 142,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10MapResources = 143,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnmapResources = 144,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceSetMapFlags = 145,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedArray = 146,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer = 147,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize = 148,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch = 149,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions = 150,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevice = 151,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate = 152,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D11RegisterResource = 153,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevice = 154,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate = 155,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D9RegisterResource = 156,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDirect3DDevice = 157,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterResource = 158,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterResource = 159,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapResources = 160,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapResources = 161,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceSetMapFlags = 162,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions = 163,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedArray = 164,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer = 165,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize = 166,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch = 167,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9Begin = 168,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9End = 169,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterVertexBuffer = 170,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer = 171,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapVertexBuffer = 172,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterVertexBuffer = 173,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate = 174,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterBuffer = 175,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterImage = 176,
    CUPTI_DRIVER_TRACE_CBID_cuWGLGetDevice = 177,
    CUPTI_DRIVER_TRACE_CBID_cuGLInit = 178,
    CUPTI_DRIVER_TRACE_CBID_cuGLRegisterBufferObject = 179,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject = 180,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObject = 181,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnregisterBufferObject = 182,
    CUPTI_DRIVER_TRACE_CBID_cuGLSetBufferObjectMapFlags = 183,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync = 184,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObjectAsync = 185,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUGetDevice = 186,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate = 187,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterVideoSurface = 188,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterOutputSurface = 189,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetSurfRef = 190,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefCreate = 191,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefDestroy = 192,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetFormat = 193,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetArray = 194,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetFormat = 195,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetArray = 196,
    CUPTI_DRIVER_TRACE_CBID_cu64DeviceTotalMem = 197,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPointer = 198,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedSize = 199,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPitch = 200,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetSurfaceDimensions = 201,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetSurfaceDimensions = 202,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPointer = 203,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedSize = 204,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPitch = 205,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9MapVertexBuffer = 206,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObject = 207,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObjectAsync = 208,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevices = 209,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreateOnDevice = 210,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevices = 211,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreateOnDevice = 212,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevices = 213,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreateOnDevice = 214,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostAlloc = 215,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async = 216,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8Async = 217,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async = 218,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16Async = 219,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async = 220,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32Async = 221,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async = 222,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8Async = 223,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async = 224,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16Async = 225,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async = 226,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32Async = 227,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayCreate = 228,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayGetDescriptor = 229,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DCreate = 230,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DGetDescriptor = 231,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2D = 232,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DUnaligned = 233,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DAsync = 234,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v2 = 235,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate_v2 = 236,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate_v2 = 237,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate_v2 = 238,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate_v2 = 239,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate_v2 = 240,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal_v2 = 241,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo_v2 = 242,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc_v2 = 243,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch_v2 = 244,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree_v2 = 245,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange_v2 = 246,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer_v2 = 247,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_v2 = 248,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2 = 249,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2 = 250,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2 = 251,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2 = 252,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2 = 253,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2 = 254,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress_v2 = 255,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v2 = 256,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress_v2 = 257,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer_v2 = 258,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem_v2 = 259,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer_v2 = 260,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize_v2 = 261,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch_v2 = 262,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions_v2 = 263,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions_v2 = 264,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer_v2 = 265,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize_v2 = 266,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch_v2 = 267,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer_v2 = 268,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2 = 269,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2 = 270,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc_v2 = 271,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate_v2 = 272,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor_v2 = 273,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate_v2 = 274,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor_v2 = 275,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2 = 276,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2 = 277,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2 = 278,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2 = 279,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2 = 280,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2 = 281,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2 = 282,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2 = 283,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2 = 284,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2 = 285,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2 = 286,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2 = 287,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2 = 288,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2 = 289,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2 = 290,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2 = 291,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2 = 292,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2 = 293,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost_v2 = 294,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent = 295,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetApiVersion = 296,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDirect3DDevice = 297,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDirect3DDevice = 298,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCacheConfig = 299,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCacheConfig = 300,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister = 301,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostUnregister = 302,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCurrent = 303,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCurrent = 304,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy = 305,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync = 306,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel = 307,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStart = 308,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStop = 309,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttribute = 310,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerInitialize = 311,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceCanAccessPeer = 312,
    CUPTI_DRIVER_TRACE_CBID_cuCtxEnablePeerAccess = 313,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDisablePeerAccess = 314,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerRegister = 315,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerUnregister = 316,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerGetDevicePointer = 317,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer = 318,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync = 319,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer = 320,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync = 321,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy_v2 = 322,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent_v2 = 323,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent_v2 = 324,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy_v2 = 325,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy_v2 = 326,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v3 = 327,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetMemHandle = 328,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle = 329,
    CUPTI_DRIVER_TRACE_CBID_cuIpcCloseMemHandle = 330,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetByPCIBusId = 331,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetPCIBusId = 332,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices = 333,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetEventHandle = 334,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenEventHandle = 335,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetSharedMemConfig = 336,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetSharedMemConfig = 337,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedMemConfig = 338,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectCreate = 339,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectDestroy = 340,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceDesc = 341,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetTextureDesc = 342,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectCreate = 343,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectDestroy = 344,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectGetResourceDesc = 345,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback = 346,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayCreate = 347,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetLevel = 348,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayDestroy = 349,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmappedArray = 350,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapFilterMode = 351,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelBias = 352,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelClamp = 353,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMaxAnisotropy = 354,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmappedArray = 355,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapFilterMode = 356,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelBias = 357,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelClamp = 358,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMaxAnisotropy = 359,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedMipmappedArray = 360,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceViewDesc = 361,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate = 362,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData = 363,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile = 364,
    CUPTI_DRIVER_TRACE_CBID_cuLinkComplete = 365,
    CUPTI_DRIVER_TRACE_CBID_cuLinkDestroy = 366,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreateWithPriority = 367,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority = 368,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags = 369,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetStreamPriorityRange = 370,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocManaged = 371,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorString = 372,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorName = 373,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessor = 374,
    CUPTI_DRIVER_TRACE_CBID_cuCompilePtx = 375,
    CUPTI_DRIVER_TRACE_CBID_cuBinaryFree = 376,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync = 377,
    CUPTI_DRIVER_TRACE_CBID_cuPointerSetAttribute = 378,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister_v2 = 379,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags_v2 = 380,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate_v2 = 381,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData_v2 = 382,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile_v2 = 383,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSize = 384,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices_v2 = 385,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRetain = 386,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease = 387,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags = 388,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset = 389,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsEGLRegisterImage = 390,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetFlags = 391,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxGetState = 392,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnect = 393,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerDisconnect = 394,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerAcquireFrame = 395,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerReleaseFrame = 396,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2_ptds = 397,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2_ptds = 398,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2_ptds = 399,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2_ptds = 400,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2_ptds = 401,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2_ptds = 402,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2_ptds = 403,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2_ptds = 404,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2_ptds = 405,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2_ptds = 406,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2_ptds = 407,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_ptds = 408,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer_ptds = 409,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer_ptds = 410,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2_ptds = 411,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2_ptds = 412,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2_ptds = 413,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2_ptds = 414,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2_ptds = 415,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2_ptds = 416,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2_ptds = 417,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync_ptsz = 418,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2_ptsz = 419,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2_ptsz = 420,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2_ptsz = 421,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2_ptsz = 422,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2_ptsz = 423,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2_ptsz = 424,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2_ptsz = 425,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync_ptsz = 426,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync_ptsz = 427,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async_ptsz = 428,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async_ptsz = 429,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async_ptsz = 430,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async_ptsz = 431,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async_ptsz = 432,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async_ptsz = 433,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority_ptsz = 434,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags_ptsz = 435,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent_ptsz = 436,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback_ptsz = 437,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync_ptsz = 438,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery_ptsz = 439,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize_ptsz = 440,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord_ptsz = 441,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel_ptsz = 442,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources_ptsz = 443,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources_ptsz = 444,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2_ptsz = 445,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerConnect = 446,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerDisconnect = 447,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerPresentFrame = 448,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedEglFrame = 449,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttributes = 450,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessorWithFlags = 451,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSizeWithFlags = 452,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerReturnFrame = 453,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetP2PAttribute = 454,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetBorderColor = 455,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetBorderColor = 456,
    CUPTI_DRIVER_TRACE_CBID_cuMemAdvise = 457,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32 = 458,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_ptsz = 459,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32 = 460,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_ptsz = 461,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp = 462,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_ptsz = 463,
    CUPTI_DRIVER_TRACE_CBID_cuNVNbufferGetPointer = 464,
    CUPTI_DRIVER_TRACE_CBID_cuNVNtextureGetArray = 465,
    CUPTI_DRIVER_TRACE_CBID_cuNNSetAllocator = 466,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync = 467,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_ptsz = 468,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromNVNSync = 469,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnectWithFlags = 470,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttribute = 471,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttributes = 472,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64 = 473,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_ptsz = 474,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64 = 475,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_ptsz = 476,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel = 477,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel_ptsz = 478,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromEGLSync = 479,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernelMultiDevice = 480,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetAttribute = 481,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid = 482,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx = 483,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_ptsz = 484,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalMemory = 485,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedBuffer = 486,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedMipmappedArray = 487,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalMemory = 488,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalSemaphore = 489,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync = 490,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync_ptsz = 491,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync = 492,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync_ptsz = 493,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalSemaphore = 494,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture = 495,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_ptsz = 496,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture = 497,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture_ptsz = 498,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing = 499,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing_ptsz = 500,
    CUPTI_DRIVER_TRACE_CBID_cuGraphCreate = 501,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddKernelNode = 502,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetParams = 503,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemcpyNode = 504,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeGetParams = 505,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemsetNode = 506,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeGetParams = 507,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeSetParams = 508,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetType = 509,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetRootNodes = 510,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependencies = 511,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependentNodes = 512,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate = 513,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch = 514,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch_ptsz = 515,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecDestroy = 516,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroy = 517,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddDependencies = 518,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRemoveDependencies = 519,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeSetParams = 520,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetParams = 521,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroyNode = 522,
    CUPTI_DRIVER_TRACE_CBID_cuGraphClone = 523,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeFindInClone = 524,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddChildGraphNode = 525,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEmptyNode = 526,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc = 527,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc_ptsz = 528,
    CUPTI_DRIVER_TRACE_CBID_cuGraphChildGraphNodeGetGraph = 529,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddHostNode = 530,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeGetParams = 531,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetLuid = 532,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeSetParams = 533,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetNodes = 534,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetEdges = 535,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo = 536,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_ptsz = 537,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecKernelNodeSetParams = 538,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2 = 539,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2_ptsz = 540,
    CUPTI_DRIVER_TRACE_CBID_cuThreadExchangeStreamCaptureMode = 541,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetNvSciSyncAttributes = 542,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyAvailableDynamicSMemPerBlock = 543,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease_v2 = 544,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset_v2 = 545,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags_v2 = 546,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressReserve = 547,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressFree = 548,
    CUPTI_DRIVER_TRACE_CBID_cuMemCreate = 549,
    CUPTI_DRIVER_TRACE_CBID_cuMemRelease = 550,
    CUPTI_DRIVER_TRACE_CBID_cuMemMap = 551,
    CUPTI_DRIVER_TRACE_CBID_cuMemUnmap = 552,
    CUPTI_DRIVER_TRACE_CBID_cuMemSetAccess = 553,
    CUPTI_DRIVER_TRACE_CBID_cuMemExportToShareableHandle = 554,
    CUPTI_DRIVER_TRACE_CBID_cuMemImportFromShareableHandle = 555,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationGranularity = 556,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationPropertiesFromHandle = 557,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAccess = 558,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags = 559,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags_ptsz = 560,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecUpdate = 561,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemcpyNodeSetParams = 562,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemsetNodeSetParams = 563,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecHostNodeSetParams = 564,
    CUPTI_DRIVER_TRACE_CBID_cuMemRetainAllocationHandle = 565,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetModule = 566,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle_v2 = 567,
    CUPTI_DRIVER_TRACE_CBID_cuCtxResetPersistingL2Cache = 568,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeCopyAttributes = 569,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetAttribute = 570,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetAttribute = 571,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes = 572,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes_ptsz = 573,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute = 574,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute_ptsz = 575,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute = 576,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute_ptsz = 577,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate_v2 = 578,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetTexture1DLinearMaxWidth = 579,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload = 580,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload_ptsz = 581,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetSparseProperties = 582,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetSparseProperties = 583,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync = 584,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync_ptsz = 585,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecChildGraphNodeSetParams = 586,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags = 587,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags_ptsz = 588,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventRecordNode = 589,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventWaitNode = 590,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeGetEvent = 591,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeGetEvent = 592,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeSetEvent = 593,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeSetEvent = 594,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventRecordNodeSetEvent = 595,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventWaitNodeSetEvent = 596,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetPlane = 597,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync = 598,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync_ptsz = 599,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync = 600,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync_ptsz = 601,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolTrimTo = 602,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAttribute = 603,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAttribute = 604,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAccess = 605,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetDefaultMemPool = 606,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolCreate = 607,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolDestroy = 608,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetMemPool = 609,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetMemPool = 610,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync = 611,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync_ptsz = 612,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportToShareableHandle = 613,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportFromShareableHandle = 614,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportPointer = 615,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportPointer = 616,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAccess = 617,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresSignalNode = 618,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeGetParams = 619,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeSetParams = 620,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresWaitNode = 621,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeGetParams = 622,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeSetParams = 623,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresSignalNodeSetParams = 624,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresWaitNodeSetParams = 625,
    CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress = 626,
    CUPTI_DRIVER_TRACE_CBID_cuFlushGPUDirectRDMAWrites = 627,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDebugDotPrint = 628,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2 = 629,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2_ptsz = 630,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies = 631,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_ptsz = 632,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectCreate = 633,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRetain = 634,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRelease = 635,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRetainUserObject = 636,
    CUPTI_DRIVER_TRACE_CBID_cuGraphReleaseUserObject = 637,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemAllocNode = 638,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemFreeNode = 639,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGraphMemTrim = 640,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetGraphMemAttribute = 641,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetGraphMemAttribute = 642,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithFlags = 643,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetExecAffinitySupport = 644,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v3 = 645,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetExecAffinity = 646,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid_v2 = 647,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemAllocNodeGetParams = 648,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemFreeNodeGetParams = 649,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeSetEnabled = 650,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetEnabled = 651,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx = 652,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx_ptsz = 653,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetMemoryRequirements = 654,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetMemoryRequirements = 655,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithParams = 656,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithParams_ptsz = 657,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecGetFlags = 658,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_v2 = 659,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_v2_ptsz = 660,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_v2 = 661,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_v2_ptsz = 662,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_v2 = 663,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_v2_ptsz = 664,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_v2 = 665,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_v2_ptsz = 666,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_v2 = 667,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_v2_ptsz = 668,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddBatchMemOpNode = 669,
    CUPTI_DRIVER_TRACE_CBID_cuGraphBatchMemOpNodeGetParams = 670,
    CUPTI_DRIVER_TRACE_CBID_cuGraphBatchMemOpNodeSetParams = 671,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecBatchMemOpNodeSetParams = 672,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetLoadingMode = 673,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetHandleForAddressRange = 674,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialClusterSize = 675,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveClusters = 676,
    CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress_v2 = 677,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryLoadData = 678,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryLoadFromFile = 679,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryUnload = 680,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetKernel = 681,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetModule = 682,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetFunction = 683,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetGlobal = 684,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetManaged = 685,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetAttribute = 686,
    CUPTI_DRIVER_TRACE_CBID_cuKernelSetAttribute = 687,
    CUPTI_DRIVER_TRACE_CBID_cuKernelSetCacheConfig = 688,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddKernelNode_v2 = 689,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetParams_v2 = 690,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetParams_v2 = 691,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecKernelNodeSetParams_v2 = 692,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetId = 693,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetId_ptsz = 694,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetId = 695,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecUpdate_v2 = 696,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeTiled = 697,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeIm2col = 698,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapReplaceAddress = 699,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetUnifiedFunction = 700,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpGetAttribute = 701,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpGetAttributeGlobal = 702,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpSetAttribute = 703,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpSetAttributeGlobal = 704,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetFlags = 705,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastCreate = 706,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastAddDevice = 707,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastBindMem = 708,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastBindAddr = 709,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastUnbind = 710,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastGetGranularity = 711,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddNode = 712,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeSetParams = 713,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecNodeSetParams = 714,
    CUPTI_DRIVER_TRACE_CBID_cuMemAdvise_v2 = 715,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_v2 = 716,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_v2_ptsz = 717,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetName = 718,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetName = 719,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCaptureToGraph = 720,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCaptureToGraph_ptsz = 721,
    CUPTI_DRIVER_TRACE_CBID_cuGraphConditionalHandleCreate = 722,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddNode_v2 = 723,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetEdges_v2 = 724,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependencies_v2 = 725,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependentNodes_v2 = 726,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddDependencies_v2 = 727,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRemoveDependencies_v2 = 728,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v3 = 729,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v3_ptsz = 730,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_v2 = 731,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_v2_ptsz = 732,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetParamInfo = 733,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetParamInfo = 734,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceRegisterAsyncNotification = 735,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceUnregisterAsyncNotification = 736,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetFunctionCount = 737,
    CUPTI_DRIVER_TRACE_CBID_cuModuleEnumerateFunctions = 738,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetKernelCount = 739,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryEnumerateKernels = 740,
    CUPTI_DRIVER_TRACE_CBID_cuFuncIsLoaded = 741,
    CUPTI_DRIVER_TRACE_CBID_cuFuncLoad = 742,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxCreate = 743,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxDestroy = 744,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetDevResource = 745,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetDevResource = 746,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxGetDevResource = 747,
    CUPTI_DRIVER_TRACE_CBID_cuDevResourceGenerateDesc = 748,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxRecordEvent = 749,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxWaitEvent = 750,
    CUPTI_DRIVER_TRACE_CBID_cuDevSmResourceSplitByCount = 751,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetGreenCtx = 752,
    CUPTI_DRIVER_TRACE_CBID_cuCtxFromGreenCtx = 753,
    CUPTI_DRIVER_TRACE_CBID_SIZE = 754,
    CUPTI_DRIVER_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-12050"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_driver_api_trace_cbid_enum {
    CUPTI_DRIVER_TRACE_CBID_INVALID = 0,
    CUPTI_DRIVER_TRACE_CBID_cuInit = 1,
    CUPTI_DRIVER_TRACE_CBID_cuDriverGetVersion = 2,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGet = 3,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetCount = 4,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetName = 5,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceComputeCapability = 6,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem = 7,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetProperties = 8,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetAttribute = 9,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate = 10,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy = 11,
    CUPTI_DRIVER_TRACE_CBID_cuCtxAttach = 12,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDetach = 13,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent = 14,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent = 15,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetDevice = 16,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSynchronize = 17,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoad = 18,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadData = 19,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadDataEx = 20,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadFatBinary = 21,
    CUPTI_DRIVER_TRACE_CBID_cuModuleUnload = 22,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetFunction = 23,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal = 24,
    CUPTI_DRIVER_TRACE_CBID_cu64ModuleGetGlobal = 25,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetTexRef = 26,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo = 27,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetInfo = 28,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc = 29,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAlloc = 30,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch = 31,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAllocPitch = 32,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree = 33,
    CUPTI_DRIVER_TRACE_CBID_cu64MemFree = 34,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange = 35,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetAddressRange = 36,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost = 37,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeHost = 38,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc = 39,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer = 40,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostGetDevicePointer = 41,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetFlags = 42,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD = 43,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoD = 44,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH = 45,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoH = 46,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD = 47,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoD = 48,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA = 49,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoA = 50,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD = 51,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyAtoD = 52,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA = 53,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH = 54,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA = 55,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D = 56,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned = 57,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D = 58,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3D = 59,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync = 60,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoDAsync = 61,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync = 62,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoHAsync = 63,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync = 64,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoDAsync = 65,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync = 66,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync = 67,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync = 68,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync = 69,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3DAsync = 70,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8 = 71,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8 = 72,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16 = 73,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16 = 74,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32 = 75,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32 = 76,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8 = 77,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8 = 78,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16 = 79,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16 = 80,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32 = 81,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32 = 82,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetBlockShape = 83,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedSize = 84,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetAttribute = 85,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetCacheConfig = 86,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate = 87,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor = 88,
    CUPTI_DRIVER_TRACE_CBID_cuArrayDestroy = 89,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate = 90,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor = 91,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefCreate = 92,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefDestroy = 93,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetArray = 94,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress = 95,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress = 96,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D = 97,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress2D = 98,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFormat = 99,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddressMode = 100,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFilterMode = 101,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFlags = 102,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress = 103,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefGetAddress = 104,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetArray = 105,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddressMode = 106,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFilterMode = 107,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFormat = 108,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFlags = 109,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetSize = 110,
    CUPTI_DRIVER_TRACE_CBID_cuParamSeti = 111,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetf = 112,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetv = 113,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetTexRef = 114,
    CUPTI_DRIVER_TRACE_CBID_cuLaunch = 115,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGrid = 116,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGridAsync = 117,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreate = 118,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord = 119,
    CUPTI_DRIVER_TRACE_CBID_cuEventQuery = 120,
    CUPTI_DRIVER_TRACE_CBID_cuEventSynchronize = 121,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy = 122,
    CUPTI_DRIVER_TRACE_CBID_cuEventElapsedTime = 123,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreate = 124,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery = 125,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize = 126,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy = 127,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnregisterResource = 128,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsSubResourceGetMappedArray = 129,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer = 130,
    CUPTI_DRIVER_TRACE_CBID_cu64GraphicsResourceGetMappedPointer = 131,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags = 132,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources = 133,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources = 134,
    CUPTI_DRIVER_TRACE_CBID_cuGetExportTable = 135,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetLimit = 136,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetLimit = 137,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevice = 138,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate = 139,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D10RegisterResource = 140,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10RegisterResource = 141,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnregisterResource = 142,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10MapResources = 143,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnmapResources = 144,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceSetMapFlags = 145,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedArray = 146,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer = 147,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize = 148,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch = 149,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions = 150,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevice = 151,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate = 152,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D11RegisterResource = 153,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevice = 154,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate = 155,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D9RegisterResource = 156,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDirect3DDevice = 157,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterResource = 158,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterResource = 159,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapResources = 160,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapResources = 161,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceSetMapFlags = 162,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions = 163,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedArray = 164,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer = 165,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize = 166,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch = 167,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9Begin = 168,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9End = 169,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterVertexBuffer = 170,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer = 171,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapVertexBuffer = 172,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterVertexBuffer = 173,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate = 174,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterBuffer = 175,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterImage = 176,
    CUPTI_DRIVER_TRACE_CBID_cuWGLGetDevice = 177,
    CUPTI_DRIVER_TRACE_CBID_cuGLInit = 178,
    CUPTI_DRIVER_TRACE_CBID_cuGLRegisterBufferObject = 179,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject = 180,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObject = 181,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnregisterBufferObject = 182,
    CUPTI_DRIVER_TRACE_CBID_cuGLSetBufferObjectMapFlags = 183,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync = 184,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObjectAsync = 185,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUGetDevice = 186,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate = 187,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterVideoSurface = 188,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterOutputSurface = 189,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetSurfRef = 190,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefCreate = 191,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefDestroy = 192,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetFormat = 193,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetArray = 194,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetFormat = 195,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetArray = 196,
    CUPTI_DRIVER_TRACE_CBID_cu64DeviceTotalMem = 197,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPointer = 198,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedSize = 199,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPitch = 200,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetSurfaceDimensions = 201,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetSurfaceDimensions = 202,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPointer = 203,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedSize = 204,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPitch = 205,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9MapVertexBuffer = 206,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObject = 207,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObjectAsync = 208,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevices = 209,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreateOnDevice = 210,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevices = 211,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreateOnDevice = 212,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevices = 213,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreateOnDevice = 214,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostAlloc = 215,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async = 216,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8Async = 217,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async = 218,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16Async = 219,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async = 220,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32Async = 221,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async = 222,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8Async = 223,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async = 224,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16Async = 225,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async = 226,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32Async = 227,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayCreate = 228,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayGetDescriptor = 229,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DCreate = 230,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DGetDescriptor = 231,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2D = 232,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DUnaligned = 233,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DAsync = 234,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v2 = 235,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate_v2 = 236,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate_v2 = 237,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate_v2 = 238,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate_v2 = 239,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate_v2 = 240,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal_v2 = 241,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo_v2 = 242,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc_v2 = 243,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch_v2 = 244,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree_v2 = 245,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange_v2 = 246,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer_v2 = 247,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_v2 = 248,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2 = 249,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2 = 250,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2 = 251,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2 = 252,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2 = 253,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2 = 254,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress_v2 = 255,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v2 = 256,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress_v2 = 257,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer_v2 = 258,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem_v2 = 259,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer_v2 = 260,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize_v2 = 261,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch_v2 = 262,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions_v2 = 263,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions_v2 = 264,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer_v2 = 265,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize_v2 = 266,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch_v2 = 267,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer_v2 = 268,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2 = 269,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2 = 270,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc_v2 = 271,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate_v2 = 272,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor_v2 = 273,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate_v2 = 274,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor_v2 = 275,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2 = 276,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2 = 277,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2 = 278,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2 = 279,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2 = 280,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2 = 281,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2 = 282,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2 = 283,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2 = 284,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2 = 285,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2 = 286,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2 = 287,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2 = 288,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2 = 289,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2 = 290,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2 = 291,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2 = 292,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2 = 293,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost_v2 = 294,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent = 295,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetApiVersion = 296,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDirect3DDevice = 297,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDirect3DDevice = 298,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCacheConfig = 299,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCacheConfig = 300,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister = 301,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostUnregister = 302,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCurrent = 303,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCurrent = 304,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy = 305,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync = 306,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel = 307,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStart = 308,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStop = 309,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttribute = 310,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerInitialize = 311,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceCanAccessPeer = 312,
    CUPTI_DRIVER_TRACE_CBID_cuCtxEnablePeerAccess = 313,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDisablePeerAccess = 314,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerRegister = 315,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerUnregister = 316,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerGetDevicePointer = 317,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer = 318,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync = 319,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer = 320,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync = 321,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy_v2 = 322,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent_v2 = 323,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent_v2 = 324,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy_v2 = 325,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy_v2 = 326,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v3 = 327,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetMemHandle = 328,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle = 329,
    CUPTI_DRIVER_TRACE_CBID_cuIpcCloseMemHandle = 330,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetByPCIBusId = 331,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetPCIBusId = 332,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices = 333,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetEventHandle = 334,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenEventHandle = 335,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetSharedMemConfig = 336,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetSharedMemConfig = 337,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedMemConfig = 338,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectCreate = 339,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectDestroy = 340,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceDesc = 341,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetTextureDesc = 342,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectCreate = 343,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectDestroy = 344,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectGetResourceDesc = 345,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback = 346,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayCreate = 347,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetLevel = 348,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayDestroy = 349,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmappedArray = 350,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapFilterMode = 351,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelBias = 352,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelClamp = 353,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMaxAnisotropy = 354,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmappedArray = 355,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapFilterMode = 356,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelBias = 357,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelClamp = 358,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMaxAnisotropy = 359,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedMipmappedArray = 360,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceViewDesc = 361,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate = 362,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData = 363,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile = 364,
    CUPTI_DRIVER_TRACE_CBID_cuLinkComplete = 365,
    CUPTI_DRIVER_TRACE_CBID_cuLinkDestroy = 366,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreateWithPriority = 367,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority = 368,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags = 369,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetStreamPriorityRange = 370,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocManaged = 371,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorString = 372,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorName = 373,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessor = 374,
    CUPTI_DRIVER_TRACE_CBID_cuCompilePtx = 375,
    CUPTI_DRIVER_TRACE_CBID_cuBinaryFree = 376,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync = 377,
    CUPTI_DRIVER_TRACE_CBID_cuPointerSetAttribute = 378,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister_v2 = 379,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags_v2 = 380,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate_v2 = 381,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData_v2 = 382,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile_v2 = 383,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSize = 384,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices_v2 = 385,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRetain = 386,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease = 387,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags = 388,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset = 389,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsEGLRegisterImage = 390,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetFlags = 391,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxGetState = 392,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnect = 393,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerDisconnect = 394,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerAcquireFrame = 395,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerReleaseFrame = 396,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2_ptds = 397,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2_ptds = 398,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2_ptds = 399,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2_ptds = 400,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2_ptds = 401,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2_ptds = 402,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2_ptds = 403,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2_ptds = 404,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2_ptds = 405,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2_ptds = 406,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2_ptds = 407,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_ptds = 408,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer_ptds = 409,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer_ptds = 410,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2_ptds = 411,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2_ptds = 412,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2_ptds = 413,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2_ptds = 414,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2_ptds = 415,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2_ptds = 416,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2_ptds = 417,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync_ptsz = 418,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2_ptsz = 419,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2_ptsz = 420,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2_ptsz = 421,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2_ptsz = 422,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2_ptsz = 423,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2_ptsz = 424,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2_ptsz = 425,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync_ptsz = 426,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync_ptsz = 427,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async_ptsz = 428,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async_ptsz = 429,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async_ptsz = 430,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async_ptsz = 431,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async_ptsz = 432,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async_ptsz = 433,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority_ptsz = 434,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags_ptsz = 435,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent_ptsz = 436,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback_ptsz = 437,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync_ptsz = 438,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery_ptsz = 439,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize_ptsz = 440,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord_ptsz = 441,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel_ptsz = 442,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources_ptsz = 443,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources_ptsz = 444,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2_ptsz = 445,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerConnect = 446,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerDisconnect = 447,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerPresentFrame = 448,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedEglFrame = 449,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttributes = 450,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessorWithFlags = 451,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSizeWithFlags = 452,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerReturnFrame = 453,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetP2PAttribute = 454,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetBorderColor = 455,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetBorderColor = 456,
    CUPTI_DRIVER_TRACE_CBID_cuMemAdvise = 457,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32 = 458,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_ptsz = 459,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32 = 460,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_ptsz = 461,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp = 462,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_ptsz = 463,
    CUPTI_DRIVER_TRACE_CBID_cuNVNbufferGetPointer = 464,
    CUPTI_DRIVER_TRACE_CBID_cuNVNtextureGetArray = 465,
    CUPTI_DRIVER_TRACE_CBID_cuNNSetAllocator = 466,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync = 467,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_ptsz = 468,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromNVNSync = 469,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnectWithFlags = 470,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttribute = 471,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttributes = 472,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64 = 473,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_ptsz = 474,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64 = 475,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_ptsz = 476,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel = 477,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel_ptsz = 478,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromEGLSync = 479,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernelMultiDevice = 480,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetAttribute = 481,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid = 482,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx = 483,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_ptsz = 484,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalMemory = 485,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedBuffer = 486,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedMipmappedArray = 487,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalMemory = 488,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalSemaphore = 489,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync = 490,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync_ptsz = 491,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync = 492,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync_ptsz = 493,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalSemaphore = 494,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture = 495,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_ptsz = 496,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture = 497,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture_ptsz = 498,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing = 499,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing_ptsz = 500,
    CUPTI_DRIVER_TRACE_CBID_cuGraphCreate = 501,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddKernelNode = 502,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetParams = 503,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemcpyNode = 504,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeGetParams = 505,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemsetNode = 506,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeGetParams = 507,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeSetParams = 508,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetType = 509,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetRootNodes = 510,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependencies = 511,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependentNodes = 512,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate = 513,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch = 514,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch_ptsz = 515,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecDestroy = 516,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroy = 517,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddDependencies = 518,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRemoveDependencies = 519,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeSetParams = 520,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetParams = 521,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroyNode = 522,
    CUPTI_DRIVER_TRACE_CBID_cuGraphClone = 523,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeFindInClone = 524,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddChildGraphNode = 525,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEmptyNode = 526,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc = 527,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc_ptsz = 528,
    CUPTI_DRIVER_TRACE_CBID_cuGraphChildGraphNodeGetGraph = 529,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddHostNode = 530,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeGetParams = 531,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetLuid = 532,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeSetParams = 533,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetNodes = 534,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetEdges = 535,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo = 536,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_ptsz = 537,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecKernelNodeSetParams = 538,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2 = 539,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2_ptsz = 540,
    CUPTI_DRIVER_TRACE_CBID_cuThreadExchangeStreamCaptureMode = 541,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetNvSciSyncAttributes = 542,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyAvailableDynamicSMemPerBlock = 543,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease_v2 = 544,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset_v2 = 545,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags_v2 = 546,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressReserve = 547,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressFree = 548,
    CUPTI_DRIVER_TRACE_CBID_cuMemCreate = 549,
    CUPTI_DRIVER_TRACE_CBID_cuMemRelease = 550,
    CUPTI_DRIVER_TRACE_CBID_cuMemMap = 551,
    CUPTI_DRIVER_TRACE_CBID_cuMemUnmap = 552,
    CUPTI_DRIVER_TRACE_CBID_cuMemSetAccess = 553,
    CUPTI_DRIVER_TRACE_CBID_cuMemExportToShareableHandle = 554,
    CUPTI_DRIVER_TRACE_CBID_cuMemImportFromShareableHandle = 555,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationGranularity = 556,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationPropertiesFromHandle = 557,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAccess = 558,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags = 559,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags_ptsz = 560,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecUpdate = 561,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemcpyNodeSetParams = 562,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemsetNodeSetParams = 563,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecHostNodeSetParams = 564,
    CUPTI_DRIVER_TRACE_CBID_cuMemRetainAllocationHandle = 565,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetModule = 566,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle_v2 = 567,
    CUPTI_DRIVER_TRACE_CBID_cuCtxResetPersistingL2Cache = 568,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeCopyAttributes = 569,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetAttribute = 570,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetAttribute = 571,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes = 572,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes_ptsz = 573,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute = 574,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute_ptsz = 575,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute = 576,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute_ptsz = 577,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate_v2 = 578,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetTexture1DLinearMaxWidth = 579,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload = 580,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload_ptsz = 581,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetSparseProperties = 582,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetSparseProperties = 583,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync = 584,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync_ptsz = 585,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecChildGraphNodeSetParams = 586,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags = 587,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags_ptsz = 588,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventRecordNode = 589,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventWaitNode = 590,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeGetEvent = 591,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeGetEvent = 592,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeSetEvent = 593,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeSetEvent = 594,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventRecordNodeSetEvent = 595,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventWaitNodeSetEvent = 596,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetPlane = 597,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync = 598,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync_ptsz = 599,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync = 600,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync_ptsz = 601,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolTrimTo = 602,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAttribute = 603,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAttribute = 604,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAccess = 605,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetDefaultMemPool = 606,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolCreate = 607,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolDestroy = 608,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetMemPool = 609,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetMemPool = 610,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync = 611,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync_ptsz = 612,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportToShareableHandle = 613,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportFromShareableHandle = 614,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportPointer = 615,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportPointer = 616,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAccess = 617,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresSignalNode = 618,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeGetParams = 619,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeSetParams = 620,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresWaitNode = 621,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeGetParams = 622,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeSetParams = 623,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresSignalNodeSetParams = 624,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresWaitNodeSetParams = 625,
    CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress = 626,
    CUPTI_DRIVER_TRACE_CBID_cuFlushGPUDirectRDMAWrites = 627,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDebugDotPrint = 628,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2 = 629,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2_ptsz = 630,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies = 631,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_ptsz = 632,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectCreate = 633,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRetain = 634,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRelease = 635,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRetainUserObject = 636,
    CUPTI_DRIVER_TRACE_CBID_cuGraphReleaseUserObject = 637,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemAllocNode = 638,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemFreeNode = 639,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGraphMemTrim = 640,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetGraphMemAttribute = 641,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetGraphMemAttribute = 642,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithFlags = 643,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetExecAffinitySupport = 644,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v3 = 645,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetExecAffinity = 646,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid_v2 = 647,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemAllocNodeGetParams = 648,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemFreeNodeGetParams = 649,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeSetEnabled = 650,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetEnabled = 651,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx = 652,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx_ptsz = 653,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetMemoryRequirements = 654,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetMemoryRequirements = 655,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithParams = 656,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithParams_ptsz = 657,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecGetFlags = 658,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_v2 = 659,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_v2_ptsz = 660,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_v2 = 661,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_v2_ptsz = 662,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_v2 = 663,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_v2_ptsz = 664,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_v2 = 665,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_v2_ptsz = 666,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_v2 = 667,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_v2_ptsz = 668,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddBatchMemOpNode = 669,
    CUPTI_DRIVER_TRACE_CBID_cuGraphBatchMemOpNodeGetParams = 670,
    CUPTI_DRIVER_TRACE_CBID_cuGraphBatchMemOpNodeSetParams = 671,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecBatchMemOpNodeSetParams = 672,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetLoadingMode = 673,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetHandleForAddressRange = 674,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialClusterSize = 675,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveClusters = 676,
    CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress_v2 = 677,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryLoadData = 678,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryLoadFromFile = 679,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryUnload = 680,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetKernel = 681,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetModule = 682,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetFunction = 683,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetGlobal = 684,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetManaged = 685,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetAttribute = 686,
    CUPTI_DRIVER_TRACE_CBID_cuKernelSetAttribute = 687,
    CUPTI_DRIVER_TRACE_CBID_cuKernelSetCacheConfig = 688,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddKernelNode_v2 = 689,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetParams_v2 = 690,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetParams_v2 = 691,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecKernelNodeSetParams_v2 = 692,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetId = 693,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetId_ptsz = 694,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetId = 695,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecUpdate_v2 = 696,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeTiled = 697,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeIm2col = 698,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapReplaceAddress = 699,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetUnifiedFunction = 700,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpGetAttribute = 701,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpGetAttributeGlobal = 702,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpSetAttribute = 703,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpSetAttributeGlobal = 704,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetFlags = 705,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastCreate = 706,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastAddDevice = 707,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastBindMem = 708,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastBindAddr = 709,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastUnbind = 710,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastGetGranularity = 711,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddNode = 712,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeSetParams = 713,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecNodeSetParams = 714,
    CUPTI_DRIVER_TRACE_CBID_cuMemAdvise_v2 = 715,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_v2 = 716,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_v2_ptsz = 717,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetName = 718,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetName = 719,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCaptureToGraph = 720,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCaptureToGraph_ptsz = 721,
    CUPTI_DRIVER_TRACE_CBID_cuGraphConditionalHandleCreate = 722,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddNode_v2 = 723,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetEdges_v2 = 724,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependencies_v2 = 725,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependentNodes_v2 = 726,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddDependencies_v2 = 727,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRemoveDependencies_v2 = 728,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v3 = 729,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v3_ptsz = 730,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_v2 = 731,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_v2_ptsz = 732,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetParamInfo = 733,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetParamInfo = 734,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceRegisterAsyncNotification = 735,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceUnregisterAsyncNotification = 736,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetFunctionCount = 737,
    CUPTI_DRIVER_TRACE_CBID_cuModuleEnumerateFunctions = 738,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetKernelCount = 739,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryEnumerateKernels = 740,
    CUPTI_DRIVER_TRACE_CBID_cuFuncIsLoaded = 741,
    CUPTI_DRIVER_TRACE_CBID_cuFuncLoad = 742,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxCreate = 743,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxDestroy = 744,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetDevResource = 745,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetDevResource = 746,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxGetDevResource = 747,
    CUPTI_DRIVER_TRACE_CBID_cuDevResourceGenerateDesc = 748,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxRecordEvent = 749,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxWaitEvent = 750,
    CUPTI_DRIVER_TRACE_CBID_cuDevSmResourceSplitByCount = 751,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetGreenCtx = 752,
    CUPTI_DRIVER_TRACE_CBID_cuCtxFromGreenCtx = 753,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetLibrary = 754,
    CUPTI_DRIVER_TRACE_CBID_cuCtxRecordEvent = 755,
    CUPTI_DRIVER_TRACE_CBID_cuCtxWaitEvent = 756,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v4 = 757,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxStreamCreate = 758,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_v2 = 759,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_v2_ptsz = 760,
    CUPTI_DRIVER_TRACE_CBID_SIZE = 761,
    CUPTI_DRIVER_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-12060"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_driver_api_trace_cbid_enum {
    CUPTI_DRIVER_TRACE_CBID_INVALID = 0,
    CUPTI_DRIVER_TRACE_CBID_cuInit = 1,
    CUPTI_DRIVER_TRACE_CBID_cuDriverGetVersion = 2,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGet = 3,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetCount = 4,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetName = 5,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceComputeCapability = 6,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem = 7,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetProperties = 8,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetAttribute = 9,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate = 10,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy = 11,
    CUPTI_DRIVER_TRACE_CBID_cuCtxAttach = 12,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDetach = 13,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent = 14,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent = 15,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetDevice = 16,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSynchronize = 17,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoad = 18,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadData = 19,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadDataEx = 20,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadFatBinary = 21,
    CUPTI_DRIVER_TRACE_CBID_cuModuleUnload = 22,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetFunction = 23,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal = 24,
    CUPTI_DRIVER_TRACE_CBID_cu64ModuleGetGlobal = 25,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetTexRef = 26,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo = 27,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetInfo = 28,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc = 29,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAlloc = 30,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch = 31,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAllocPitch = 32,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree = 33,
    CUPTI_DRIVER_TRACE_CBID_cu64MemFree = 34,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange = 35,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetAddressRange = 36,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost = 37,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeHost = 38,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc = 39,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer = 40,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostGetDevicePointer = 41,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetFlags = 42,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD = 43,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoD = 44,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH = 45,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoH = 46,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD = 47,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoD = 48,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA = 49,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoA = 50,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD = 51,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyAtoD = 52,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA = 53,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH = 54,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA = 55,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D = 56,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned = 57,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D = 58,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3D = 59,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync = 60,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoDAsync = 61,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync = 62,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoHAsync = 63,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync = 64,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoDAsync = 65,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync = 66,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync = 67,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync = 68,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync = 69,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3DAsync = 70,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8 = 71,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8 = 72,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16 = 73,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16 = 74,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32 = 75,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32 = 76,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8 = 77,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8 = 78,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16 = 79,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16 = 80,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32 = 81,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32 = 82,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetBlockShape = 83,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedSize = 84,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetAttribute = 85,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetCacheConfig = 86,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate = 87,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor = 88,
    CUPTI_DRIVER_TRACE_CBID_cuArrayDestroy = 89,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate = 90,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor = 91,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefCreate = 92,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefDestroy = 93,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetArray = 94,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress = 95,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress = 96,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D = 97,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress2D = 98,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFormat = 99,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddressMode = 100,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFilterMode = 101,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFlags = 102,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress = 103,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefGetAddress = 104,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetArray = 105,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddressMode = 106,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFilterMode = 107,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFormat = 108,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFlags = 109,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetSize = 110,
    CUPTI_DRIVER_TRACE_CBID_cuParamSeti = 111,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetf = 112,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetv = 113,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetTexRef = 114,
    CUPTI_DRIVER_TRACE_CBID_cuLaunch = 115,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGrid = 116,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGridAsync = 117,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreate = 118,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord = 119,
    CUPTI_DRIVER_TRACE_CBID_cuEventQuery = 120,
    CUPTI_DRIVER_TRACE_CBID_cuEventSynchronize = 121,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy = 122,
    CUPTI_DRIVER_TRACE_CBID_cuEventElapsedTime = 123,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreate = 124,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery = 125,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize = 126,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy = 127,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnregisterResource = 128,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsSubResourceGetMappedArray = 129,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer = 130,
    CUPTI_DRIVER_TRACE_CBID_cu64GraphicsResourceGetMappedPointer = 131,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags = 132,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources = 133,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources = 134,
    CUPTI_DRIVER_TRACE_CBID_cuGetExportTable = 135,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetLimit = 136,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetLimit = 137,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevice = 138,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate = 139,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D10RegisterResource = 140,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10RegisterResource = 141,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnregisterResource = 142,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10MapResources = 143,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnmapResources = 144,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceSetMapFlags = 145,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedArray = 146,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer = 147,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize = 148,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch = 149,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions = 150,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevice = 151,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate = 152,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D11RegisterResource = 153,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevice = 154,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate = 155,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D9RegisterResource = 156,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDirect3DDevice = 157,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterResource = 158,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterResource = 159,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapResources = 160,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapResources = 161,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceSetMapFlags = 162,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions = 163,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedArray = 164,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer = 165,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize = 166,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch = 167,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9Begin = 168,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9End = 169,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterVertexBuffer = 170,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer = 171,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapVertexBuffer = 172,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterVertexBuffer = 173,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate = 174,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterBuffer = 175,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterImage = 176,
    CUPTI_DRIVER_TRACE_CBID_cuWGLGetDevice = 177,
    CUPTI_DRIVER_TRACE_CBID_cuGLInit = 178,
    CUPTI_DRIVER_TRACE_CBID_cuGLRegisterBufferObject = 179,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject = 180,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObject = 181,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnregisterBufferObject = 182,
    CUPTI_DRIVER_TRACE_CBID_cuGLSetBufferObjectMapFlags = 183,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync = 184,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObjectAsync = 185,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUGetDevice = 186,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate = 187,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterVideoSurface = 188,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterOutputSurface = 189,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetSurfRef = 190,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefCreate = 191,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefDestroy = 192,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetFormat = 193,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetArray = 194,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetFormat = 195,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetArray = 196,
    CUPTI_DRIVER_TRACE_CBID_cu64DeviceTotalMem = 197,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPointer = 198,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedSize = 199,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPitch = 200,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetSurfaceDimensions = 201,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetSurfaceDimensions = 202,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPointer = 203,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedSize = 204,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPitch = 205,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9MapVertexBuffer = 206,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObject = 207,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObjectAsync = 208,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevices = 209,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreateOnDevice = 210,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevices = 211,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreateOnDevice = 212,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevices = 213,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreateOnDevice = 214,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostAlloc = 215,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async = 216,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8Async = 217,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async = 218,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16Async = 219,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async = 220,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32Async = 221,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async = 222,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8Async = 223,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async = 224,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16Async = 225,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async = 226,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32Async = 227,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayCreate = 228,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayGetDescriptor = 229,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DCreate = 230,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DGetDescriptor = 231,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2D = 232,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DUnaligned = 233,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DAsync = 234,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v2 = 235,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate_v2 = 236,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate_v2 = 237,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate_v2 = 238,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate_v2 = 239,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate_v2 = 240,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal_v2 = 241,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo_v2 = 242,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc_v2 = 243,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch_v2 = 244,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree_v2 = 245,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange_v2 = 246,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer_v2 = 247,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_v2 = 248,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2 = 249,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2 = 250,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2 = 251,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2 = 252,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2 = 253,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2 = 254,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress_v2 = 255,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v2 = 256,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress_v2 = 257,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer_v2 = 258,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem_v2 = 259,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer_v2 = 260,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize_v2 = 261,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch_v2 = 262,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions_v2 = 263,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions_v2 = 264,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer_v2 = 265,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize_v2 = 266,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch_v2 = 267,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer_v2 = 268,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2 = 269,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2 = 270,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc_v2 = 271,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate_v2 = 272,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor_v2 = 273,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate_v2 = 274,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor_v2 = 275,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2 = 276,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2 = 277,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2 = 278,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2 = 279,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2 = 280,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2 = 281,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2 = 282,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2 = 283,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2 = 284,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2 = 285,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2 = 286,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2 = 287,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2 = 288,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2 = 289,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2 = 290,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2 = 291,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2 = 292,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2 = 293,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost_v2 = 294,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent = 295,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetApiVersion = 296,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDirect3DDevice = 297,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDirect3DDevice = 298,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCacheConfig = 299,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCacheConfig = 300,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister = 301,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostUnregister = 302,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCurrent = 303,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCurrent = 304,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy = 305,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync = 306,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel = 307,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStart = 308,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStop = 309,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttribute = 310,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerInitialize = 311,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceCanAccessPeer = 312,
    CUPTI_DRIVER_TRACE_CBID_cuCtxEnablePeerAccess = 313,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDisablePeerAccess = 314,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerRegister = 315,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerUnregister = 316,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerGetDevicePointer = 317,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer = 318,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync = 319,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer = 320,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync = 321,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy_v2 = 322,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent_v2 = 323,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent_v2 = 324,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy_v2 = 325,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy_v2 = 326,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v3 = 327,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetMemHandle = 328,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle = 329,
    CUPTI_DRIVER_TRACE_CBID_cuIpcCloseMemHandle = 330,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetByPCIBusId = 331,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetPCIBusId = 332,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices = 333,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetEventHandle = 334,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenEventHandle = 335,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetSharedMemConfig = 336,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetSharedMemConfig = 337,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedMemConfig = 338,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectCreate = 339,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectDestroy = 340,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceDesc = 341,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetTextureDesc = 342,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectCreate = 343,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectDestroy = 344,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectGetResourceDesc = 345,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback = 346,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayCreate = 347,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetLevel = 348,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayDestroy = 349,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmappedArray = 350,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapFilterMode = 351,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelBias = 352,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelClamp = 353,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMaxAnisotropy = 354,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmappedArray = 355,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapFilterMode = 356,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelBias = 357,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelClamp = 358,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMaxAnisotropy = 359,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedMipmappedArray = 360,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceViewDesc = 361,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate = 362,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData = 363,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile = 364,
    CUPTI_DRIVER_TRACE_CBID_cuLinkComplete = 365,
    CUPTI_DRIVER_TRACE_CBID_cuLinkDestroy = 366,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreateWithPriority = 367,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority = 368,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags = 369,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetStreamPriorityRange = 370,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocManaged = 371,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorString = 372,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorName = 373,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessor = 374,
    CUPTI_DRIVER_TRACE_CBID_cuCompilePtx = 375,
    CUPTI_DRIVER_TRACE_CBID_cuBinaryFree = 376,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync = 377,
    CUPTI_DRIVER_TRACE_CBID_cuPointerSetAttribute = 378,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister_v2 = 379,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags_v2 = 380,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate_v2 = 381,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData_v2 = 382,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile_v2 = 383,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSize = 384,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices_v2 = 385,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRetain = 386,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease = 387,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags = 388,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset = 389,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsEGLRegisterImage = 390,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetFlags = 391,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxGetState = 392,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnect = 393,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerDisconnect = 394,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerAcquireFrame = 395,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerReleaseFrame = 396,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2_ptds = 397,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2_ptds = 398,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2_ptds = 399,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2_ptds = 400,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2_ptds = 401,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2_ptds = 402,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2_ptds = 403,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2_ptds = 404,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2_ptds = 405,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2_ptds = 406,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2_ptds = 407,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_ptds = 408,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer_ptds = 409,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer_ptds = 410,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2_ptds = 411,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2_ptds = 412,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2_ptds = 413,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2_ptds = 414,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2_ptds = 415,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2_ptds = 416,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2_ptds = 417,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync_ptsz = 418,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2_ptsz = 419,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2_ptsz = 420,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2_ptsz = 421,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2_ptsz = 422,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2_ptsz = 423,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2_ptsz = 424,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2_ptsz = 425,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync_ptsz = 426,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync_ptsz = 427,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async_ptsz = 428,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async_ptsz = 429,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async_ptsz = 430,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async_ptsz = 431,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async_ptsz = 432,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async_ptsz = 433,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority_ptsz = 434,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags_ptsz = 435,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent_ptsz = 436,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback_ptsz = 437,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync_ptsz = 438,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery_ptsz = 439,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize_ptsz = 440,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord_ptsz = 441,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel_ptsz = 442,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources_ptsz = 443,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources_ptsz = 444,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2_ptsz = 445,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerConnect = 446,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerDisconnect = 447,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerPresentFrame = 448,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedEglFrame = 449,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttributes = 450,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessorWithFlags = 451,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSizeWithFlags = 452,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerReturnFrame = 453,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetP2PAttribute = 454,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetBorderColor = 455,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetBorderColor = 456,
    CUPTI_DRIVER_TRACE_CBID_cuMemAdvise = 457,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32 = 458,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_ptsz = 459,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32 = 460,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_ptsz = 461,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp = 462,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_ptsz = 463,
    CUPTI_DRIVER_TRACE_CBID_cuNVNbufferGetPointer = 464,
    CUPTI_DRIVER_TRACE_CBID_cuNVNtextureGetArray = 465,
    CUPTI_DRIVER_TRACE_CBID_cuNNSetAllocator = 466,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync = 467,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_ptsz = 468,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromNVNSync = 469,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnectWithFlags = 470,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttribute = 471,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttributes = 472,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64 = 473,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_ptsz = 474,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64 = 475,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_ptsz = 476,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel = 477,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel_ptsz = 478,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromEGLSync = 479,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernelMultiDevice = 480,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetAttribute = 481,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid = 482,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx = 483,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_ptsz = 484,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalMemory = 485,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedBuffer = 486,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedMipmappedArray = 487,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalMemory = 488,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalSemaphore = 489,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync = 490,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync_ptsz = 491,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync = 492,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync_ptsz = 493,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalSemaphore = 494,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture = 495,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_ptsz = 496,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture = 497,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture_ptsz = 498,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing = 499,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing_ptsz = 500,
    CUPTI_DRIVER_TRACE_CBID_cuGraphCreate = 501,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddKernelNode = 502,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetParams = 503,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemcpyNode = 504,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeGetParams = 505,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemsetNode = 506,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeGetParams = 507,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeSetParams = 508,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetType = 509,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetRootNodes = 510,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependencies = 511,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependentNodes = 512,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate = 513,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch = 514,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch_ptsz = 515,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecDestroy = 516,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroy = 517,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddDependencies = 518,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRemoveDependencies = 519,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeSetParams = 520,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetParams = 521,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroyNode = 522,
    CUPTI_DRIVER_TRACE_CBID_cuGraphClone = 523,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeFindInClone = 524,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddChildGraphNode = 525,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEmptyNode = 526,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc = 527,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc_ptsz = 528,
    CUPTI_DRIVER_TRACE_CBID_cuGraphChildGraphNodeGetGraph = 529,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddHostNode = 530,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeGetParams = 531,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetLuid = 532,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeSetParams = 533,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetNodes = 534,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetEdges = 535,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo = 536,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_ptsz = 537,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecKernelNodeSetParams = 538,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2 = 539,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2_ptsz = 540,
    CUPTI_DRIVER_TRACE_CBID_cuThreadExchangeStreamCaptureMode = 541,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetNvSciSyncAttributes = 542,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyAvailableDynamicSMemPerBlock = 543,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease_v2 = 544,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset_v2 = 545,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags_v2 = 546,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressReserve = 547,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressFree = 548,
    CUPTI_DRIVER_TRACE_CBID_cuMemCreate = 549,
    CUPTI_DRIVER_TRACE_CBID_cuMemRelease = 550,
    CUPTI_DRIVER_TRACE_CBID_cuMemMap = 551,
    CUPTI_DRIVER_TRACE_CBID_cuMemUnmap = 552,
    CUPTI_DRIVER_TRACE_CBID_cuMemSetAccess = 553,
    CUPTI_DRIVER_TRACE_CBID_cuMemExportToShareableHandle = 554,
    CUPTI_DRIVER_TRACE_CBID_cuMemImportFromShareableHandle = 555,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationGranularity = 556,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationPropertiesFromHandle = 557,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAccess = 558,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags = 559,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags_ptsz = 560,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecUpdate = 561,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemcpyNodeSetParams = 562,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemsetNodeSetParams = 563,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecHostNodeSetParams = 564,
    CUPTI_DRIVER_TRACE_CBID_cuMemRetainAllocationHandle = 565,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetModule = 566,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle_v2 = 567,
    CUPTI_DRIVER_TRACE_CBID_cuCtxResetPersistingL2Cache = 568,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeCopyAttributes = 569,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetAttribute = 570,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetAttribute = 571,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes = 572,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes_ptsz = 573,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute = 574,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute_ptsz = 575,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute = 576,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute_ptsz = 577,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate_v2 = 578,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetTexture1DLinearMaxWidth = 579,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload = 580,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload_ptsz = 581,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetSparseProperties = 582,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetSparseProperties = 583,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync = 584,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync_ptsz = 585,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecChildGraphNodeSetParams = 586,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags = 587,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags_ptsz = 588,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventRecordNode = 589,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventWaitNode = 590,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeGetEvent = 591,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeGetEvent = 592,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeSetEvent = 593,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeSetEvent = 594,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventRecordNodeSetEvent = 595,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventWaitNodeSetEvent = 596,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetPlane = 597,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync = 598,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync_ptsz = 599,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync = 600,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync_ptsz = 601,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolTrimTo = 602,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAttribute = 603,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAttribute = 604,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAccess = 605,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetDefaultMemPool = 606,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolCreate = 607,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolDestroy = 608,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetMemPool = 609,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetMemPool = 610,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync = 611,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync_ptsz = 612,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportToShareableHandle = 613,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportFromShareableHandle = 614,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportPointer = 615,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportPointer = 616,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAccess = 617,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresSignalNode = 618,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeGetParams = 619,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeSetParams = 620,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresWaitNode = 621,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeGetParams = 622,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeSetParams = 623,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresSignalNodeSetParams = 624,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresWaitNodeSetParams = 625,
    CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress = 626,
    CUPTI_DRIVER_TRACE_CBID_cuFlushGPUDirectRDMAWrites = 627,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDebugDotPrint = 628,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2 = 629,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2_ptsz = 630,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies = 631,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_ptsz = 632,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectCreate = 633,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRetain = 634,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRelease = 635,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRetainUserObject = 636,
    CUPTI_DRIVER_TRACE_CBID_cuGraphReleaseUserObject = 637,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemAllocNode = 638,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemFreeNode = 639,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGraphMemTrim = 640,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetGraphMemAttribute = 641,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetGraphMemAttribute = 642,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithFlags = 643,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetExecAffinitySupport = 644,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v3 = 645,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetExecAffinity = 646,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid_v2 = 647,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemAllocNodeGetParams = 648,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemFreeNodeGetParams = 649,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeSetEnabled = 650,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetEnabled = 651,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx = 652,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx_ptsz = 653,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetMemoryRequirements = 654,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetMemoryRequirements = 655,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithParams = 656,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithParams_ptsz = 657,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecGetFlags = 658,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_v2 = 659,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_v2_ptsz = 660,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_v2 = 661,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_v2_ptsz = 662,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_v2 = 663,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_v2_ptsz = 664,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_v2 = 665,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_v2_ptsz = 666,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_v2 = 667,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_v2_ptsz = 668,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddBatchMemOpNode = 669,
    CUPTI_DRIVER_TRACE_CBID_cuGraphBatchMemOpNodeGetParams = 670,
    CUPTI_DRIVER_TRACE_CBID_cuGraphBatchMemOpNodeSetParams = 671,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecBatchMemOpNodeSetParams = 672,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetLoadingMode = 673,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetHandleForAddressRange = 674,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialClusterSize = 675,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveClusters = 676,
    CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress_v2 = 677,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryLoadData = 678,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryLoadFromFile = 679,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryUnload = 680,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetKernel = 681,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetModule = 682,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetFunction = 683,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetGlobal = 684,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetManaged = 685,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetAttribute = 686,
    CUPTI_DRIVER_TRACE_CBID_cuKernelSetAttribute = 687,
    CUPTI_DRIVER_TRACE_CBID_cuKernelSetCacheConfig = 688,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddKernelNode_v2 = 689,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetParams_v2 = 690,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetParams_v2 = 691,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecKernelNodeSetParams_v2 = 692,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetId = 693,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetId_ptsz = 694,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetId = 695,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecUpdate_v2 = 696,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeTiled = 697,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeIm2col = 698,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapReplaceAddress = 699,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetUnifiedFunction = 700,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpGetAttribute = 701,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpGetAttributeGlobal = 702,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpSetAttribute = 703,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpSetAttributeGlobal = 704,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetFlags = 705,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastCreate = 706,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastAddDevice = 707,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastBindMem = 708,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastBindAddr = 709,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastUnbind = 710,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastGetGranularity = 711,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddNode = 712,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeSetParams = 713,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecNodeSetParams = 714,
    CUPTI_DRIVER_TRACE_CBID_cuMemAdvise_v2 = 715,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_v2 = 716,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_v2_ptsz = 717,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetName = 718,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetName = 719,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCaptureToGraph = 720,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCaptureToGraph_ptsz = 721,
    CUPTI_DRIVER_TRACE_CBID_cuGraphConditionalHandleCreate = 722,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddNode_v2 = 723,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetEdges_v2 = 724,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependencies_v2 = 725,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependentNodes_v2 = 726,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddDependencies_v2 = 727,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRemoveDependencies_v2 = 728,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v3 = 729,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v3_ptsz = 730,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_v2 = 731,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_v2_ptsz = 732,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetParamInfo = 733,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetParamInfo = 734,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceRegisterAsyncNotification = 735,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceUnregisterAsyncNotification = 736,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetFunctionCount = 737,
    CUPTI_DRIVER_TRACE_CBID_cuModuleEnumerateFunctions = 738,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetKernelCount = 739,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryEnumerateKernels = 740,
    CUPTI_DRIVER_TRACE_CBID_cuFuncIsLoaded = 741,
    CUPTI_DRIVER_TRACE_CBID_cuFuncLoad = 742,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxCreate = 743,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxDestroy = 744,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetDevResource = 745,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetDevResource = 746,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxGetDevResource = 747,
    CUPTI_DRIVER_TRACE_CBID_cuDevResourceGenerateDesc = 748,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxRecordEvent = 749,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxWaitEvent = 750,
    CUPTI_DRIVER_TRACE_CBID_cuDevSmResourceSplitByCount = 751,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetGreenCtx = 752,
    CUPTI_DRIVER_TRACE_CBID_cuCtxFromGreenCtx = 753,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetLibrary = 754,
    CUPTI_DRIVER_TRACE_CBID_cuCtxRecordEvent = 755,
    CUPTI_DRIVER_TRACE_CBID_cuCtxWaitEvent = 756,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v4 = 757,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxStreamCreate = 758,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_v2 = 759,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_v2_ptsz = 760,
    CUPTI_DRIVER_TRACE_CBID_cuMemBatchDecompressAsync = 761,
    CUPTI_DRIVER_TRACE_CBID_cuMemBatchDecompressAsync_ptsz = 762,
    CUPTI_DRIVER_TRACE_CBID_SIZE = 763,
    CUPTI_DRIVER_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-12080"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_driver_api_trace_cbid_enum {
    CUPTI_DRIVER_TRACE_CBID_INVALID = 0,
    CUPTI_DRIVER_TRACE_CBID_cuInit = 1,
    CUPTI_DRIVER_TRACE_CBID_cuDriverGetVersion = 2,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGet = 3,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetCount = 4,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetName = 5,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceComputeCapability = 6,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem = 7,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetProperties = 8,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetAttribute = 9,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate = 10,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy = 11,
    CUPTI_DRIVER_TRACE_CBID_cuCtxAttach = 12,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDetach = 13,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent = 14,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent = 15,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetDevice = 16,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSynchronize = 17,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoad = 18,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadData = 19,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadDataEx = 20,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadFatBinary = 21,
    CUPTI_DRIVER_TRACE_CBID_cuModuleUnload = 22,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetFunction = 23,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal = 24,
    CUPTI_DRIVER_TRACE_CBID_cu64ModuleGetGlobal = 25,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetTexRef = 26,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo = 27,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetInfo = 28,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc = 29,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAlloc = 30,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch = 31,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAllocPitch = 32,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree = 33,
    CUPTI_DRIVER_TRACE_CBID_cu64MemFree = 34,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange = 35,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetAddressRange = 36,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost = 37,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeHost = 38,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc = 39,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer = 40,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostGetDevicePointer = 41,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetFlags = 42,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD = 43,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoD = 44,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH = 45,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoH = 46,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD = 47,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoD = 48,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA = 49,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoA = 50,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD = 51,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyAtoD = 52,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA = 53,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH = 54,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA = 55,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D = 56,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned = 57,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D = 58,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3D = 59,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync = 60,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoDAsync = 61,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync = 62,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoHAsync = 63,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync = 64,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoDAsync = 65,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync = 66,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync = 67,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync = 68,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync = 69,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3DAsync = 70,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8 = 71,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8 = 72,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16 = 73,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16 = 74,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32 = 75,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32 = 76,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8 = 77,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8 = 78,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16 = 79,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16 = 80,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32 = 81,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32 = 82,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetBlockShape = 83,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedSize = 84,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetAttribute = 85,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetCacheConfig = 86,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate = 87,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor = 88,
    CUPTI_DRIVER_TRACE_CBID_cuArrayDestroy = 89,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate = 90,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor = 91,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefCreate = 92,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefDestroy = 93,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetArray = 94,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress = 95,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress = 96,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D = 97,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress2D = 98,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFormat = 99,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddressMode = 100,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFilterMode = 101,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFlags = 102,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress = 103,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefGetAddress = 104,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetArray = 105,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddressMode = 106,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFilterMode = 107,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFormat = 108,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFlags = 109,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetSize = 110,
    CUPTI_DRIVER_TRACE_CBID_cuParamSeti = 111,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetf = 112,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetv = 113,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetTexRef = 114,
    CUPTI_DRIVER_TRACE_CBID_cuLaunch = 115,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGrid = 116,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGridAsync = 117,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreate = 118,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord = 119,
    CUPTI_DRIVER_TRACE_CBID_cuEventQuery = 120,
    CUPTI_DRIVER_TRACE_CBID_cuEventSynchronize = 121,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy = 122,
    CUPTI_DRIVER_TRACE_CBID_cuEventElapsedTime = 123,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreate = 124,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery = 125,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize = 126,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy = 127,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnregisterResource = 128,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsSubResourceGetMappedArray = 129,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer = 130,
    CUPTI_DRIVER_TRACE_CBID_cu64GraphicsResourceGetMappedPointer = 131,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags = 132,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources = 133,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources = 134,
    CUPTI_DRIVER_TRACE_CBID_cuGetExportTable = 135,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetLimit = 136,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetLimit = 137,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevice = 138,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate = 139,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D10RegisterResource = 140,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10RegisterResource = 141,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnregisterResource = 142,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10MapResources = 143,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnmapResources = 144,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceSetMapFlags = 145,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedArray = 146,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer = 147,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize = 148,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch = 149,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions = 150,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevice = 151,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate = 152,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D11RegisterResource = 153,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevice = 154,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate = 155,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D9RegisterResource = 156,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDirect3DDevice = 157,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterResource = 158,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterResource = 159,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapResources = 160,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapResources = 161,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceSetMapFlags = 162,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions = 163,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedArray = 164,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer = 165,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize = 166,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch = 167,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9Begin = 168,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9End = 169,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterVertexBuffer = 170,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer = 171,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapVertexBuffer = 172,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterVertexBuffer = 173,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate = 174,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterBuffer = 175,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterImage = 176,
    CUPTI_DRIVER_TRACE_CBID_cuWGLGetDevice = 177,
    CUPTI_DRIVER_TRACE_CBID_cuGLInit = 178,
    CUPTI_DRIVER_TRACE_CBID_cuGLRegisterBufferObject = 179,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject = 180,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObject = 181,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnregisterBufferObject = 182,
    CUPTI_DRIVER_TRACE_CBID_cuGLSetBufferObjectMapFlags = 183,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync = 184,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObjectAsync = 185,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUGetDevice = 186,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate = 187,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterVideoSurface = 188,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterOutputSurface = 189,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetSurfRef = 190,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefCreate = 191,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefDestroy = 192,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetFormat = 193,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetArray = 194,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetFormat = 195,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetArray = 196,
    CUPTI_DRIVER_TRACE_CBID_cu64DeviceTotalMem = 197,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPointer = 198,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedSize = 199,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPitch = 200,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetSurfaceDimensions = 201,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetSurfaceDimensions = 202,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPointer = 203,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedSize = 204,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPitch = 205,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9MapVertexBuffer = 206,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObject = 207,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObjectAsync = 208,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevices = 209,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreateOnDevice = 210,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevices = 211,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreateOnDevice = 212,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevices = 213,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreateOnDevice = 214,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostAlloc = 215,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async = 216,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8Async = 217,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async = 218,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16Async = 219,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async = 220,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32Async = 221,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async = 222,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8Async = 223,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async = 224,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16Async = 225,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async = 226,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32Async = 227,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayCreate = 228,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayGetDescriptor = 229,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DCreate = 230,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DGetDescriptor = 231,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2D = 232,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DUnaligned = 233,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DAsync = 234,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v2 = 235,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate_v2 = 236,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate_v2 = 237,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate_v2 = 238,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate_v2 = 239,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate_v2 = 240,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal_v2 = 241,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo_v2 = 242,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc_v2 = 243,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch_v2 = 244,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree_v2 = 245,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange_v2 = 246,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer_v2 = 247,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_v2 = 248,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2 = 249,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2 = 250,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2 = 251,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2 = 252,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2 = 253,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2 = 254,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress_v2 = 255,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v2 = 256,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress_v2 = 257,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer_v2 = 258,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem_v2 = 259,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer_v2 = 260,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize_v2 = 261,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch_v2 = 262,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions_v2 = 263,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions_v2 = 264,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer_v2 = 265,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize_v2 = 266,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch_v2 = 267,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer_v2 = 268,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2 = 269,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2 = 270,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc_v2 = 271,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate_v2 = 272,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor_v2 = 273,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate_v2 = 274,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor_v2 = 275,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2 = 276,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2 = 277,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2 = 278,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2 = 279,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2 = 280,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2 = 281,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2 = 282,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2 = 283,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2 = 284,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2 = 285,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2 = 286,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2 = 287,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2 = 288,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2 = 289,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2 = 290,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2 = 291,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2 = 292,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2 = 293,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost_v2 = 294,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent = 295,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetApiVersion = 296,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDirect3DDevice = 297,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDirect3DDevice = 298,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCacheConfig = 299,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCacheConfig = 300,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister = 301,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostUnregister = 302,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCurrent = 303,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCurrent = 304,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy = 305,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync = 306,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel = 307,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStart = 308,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStop = 309,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttribute = 310,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerInitialize = 311,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceCanAccessPeer = 312,
    CUPTI_DRIVER_TRACE_CBID_cuCtxEnablePeerAccess = 313,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDisablePeerAccess = 314,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerRegister = 315,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerUnregister = 316,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerGetDevicePointer = 317,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer = 318,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync = 319,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer = 320,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync = 321,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy_v2 = 322,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent_v2 = 323,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent_v2 = 324,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy_v2 = 325,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy_v2 = 326,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v3 = 327,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetMemHandle = 328,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle = 329,
    CUPTI_DRIVER_TRACE_CBID_cuIpcCloseMemHandle = 330,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetByPCIBusId = 331,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetPCIBusId = 332,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices = 333,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetEventHandle = 334,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenEventHandle = 335,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetSharedMemConfig = 336,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetSharedMemConfig = 337,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedMemConfig = 338,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectCreate = 339,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectDestroy = 340,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceDesc = 341,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetTextureDesc = 342,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectCreate = 343,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectDestroy = 344,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectGetResourceDesc = 345,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback = 346,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayCreate = 347,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetLevel = 348,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayDestroy = 349,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmappedArray = 350,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapFilterMode = 351,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelBias = 352,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelClamp = 353,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMaxAnisotropy = 354,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmappedArray = 355,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapFilterMode = 356,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelBias = 357,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelClamp = 358,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMaxAnisotropy = 359,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedMipmappedArray = 360,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceViewDesc = 361,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate = 362,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData = 363,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile = 364,
    CUPTI_DRIVER_TRACE_CBID_cuLinkComplete = 365,
    CUPTI_DRIVER_TRACE_CBID_cuLinkDestroy = 366,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreateWithPriority = 367,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority = 368,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags = 369,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetStreamPriorityRange = 370,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocManaged = 371,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorString = 372,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorName = 373,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessor = 374,
    CUPTI_DRIVER_TRACE_CBID_cuCompilePtx = 375,
    CUPTI_DRIVER_TRACE_CBID_cuBinaryFree = 376,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync = 377,
    CUPTI_DRIVER_TRACE_CBID_cuPointerSetAttribute = 378,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister_v2 = 379,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags_v2 = 380,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate_v2 = 381,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData_v2 = 382,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile_v2 = 383,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSize = 384,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices_v2 = 385,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRetain = 386,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease = 387,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags = 388,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset = 389,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsEGLRegisterImage = 390,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetFlags = 391,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxGetState = 392,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnect = 393,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerDisconnect = 394,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerAcquireFrame = 395,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerReleaseFrame = 396,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2_ptds = 397,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2_ptds = 398,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2_ptds = 399,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2_ptds = 400,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2_ptds = 401,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2_ptds = 402,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2_ptds = 403,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2_ptds = 404,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2_ptds = 405,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2_ptds = 406,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2_ptds = 407,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_ptds = 408,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer_ptds = 409,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer_ptds = 410,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2_ptds = 411,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2_ptds = 412,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2_ptds = 413,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2_ptds = 414,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2_ptds = 415,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2_ptds = 416,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2_ptds = 417,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync_ptsz = 418,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2_ptsz = 419,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2_ptsz = 420,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2_ptsz = 421,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2_ptsz = 422,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2_ptsz = 423,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2_ptsz = 424,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2_ptsz = 425,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync_ptsz = 426,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync_ptsz = 427,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async_ptsz = 428,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async_ptsz = 429,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async_ptsz = 430,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async_ptsz = 431,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async_ptsz = 432,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async_ptsz = 433,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority_ptsz = 434,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags_ptsz = 435,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent_ptsz = 436,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback_ptsz = 437,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync_ptsz = 438,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery_ptsz = 439,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize_ptsz = 440,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord_ptsz = 441,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel_ptsz = 442,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources_ptsz = 443,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources_ptsz = 444,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2_ptsz = 445,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerConnect = 446,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerDisconnect = 447,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerPresentFrame = 448,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedEglFrame = 449,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttributes = 450,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessorWithFlags = 451,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSizeWithFlags = 452,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerReturnFrame = 453,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetP2PAttribute = 454,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetBorderColor = 455,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetBorderColor = 456,
    CUPTI_DRIVER_TRACE_CBID_cuMemAdvise = 457,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32 = 458,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_ptsz = 459,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32 = 460,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_ptsz = 461,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp = 462,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_ptsz = 463,
    CUPTI_DRIVER_TRACE_CBID_cuNVNbufferGetPointer = 464,
    CUPTI_DRIVER_TRACE_CBID_cuNVNtextureGetArray = 465,
    CUPTI_DRIVER_TRACE_CBID_cuNNSetAllocator = 466,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync = 467,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_ptsz = 468,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromNVNSync = 469,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnectWithFlags = 470,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttribute = 471,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttributes = 472,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64 = 473,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_ptsz = 474,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64 = 475,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_ptsz = 476,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel = 477,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel_ptsz = 478,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromEGLSync = 479,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernelMultiDevice = 480,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetAttribute = 481,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid = 482,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx = 483,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_ptsz = 484,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalMemory = 485,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedBuffer = 486,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedMipmappedArray = 487,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalMemory = 488,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalSemaphore = 489,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync = 490,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync_ptsz = 491,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync = 492,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync_ptsz = 493,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalSemaphore = 494,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture = 495,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_ptsz = 496,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture = 497,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture_ptsz = 498,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing = 499,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing_ptsz = 500,
    CUPTI_DRIVER_TRACE_CBID_cuGraphCreate = 501,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddKernelNode = 502,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetParams = 503,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemcpyNode = 504,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeGetParams = 505,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemsetNode = 506,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeGetParams = 507,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeSetParams = 508,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetType = 509,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetRootNodes = 510,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependencies = 511,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependentNodes = 512,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate = 513,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch = 514,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch_ptsz = 515,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecDestroy = 516,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroy = 517,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddDependencies = 518,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRemoveDependencies = 519,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeSetParams = 520,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetParams = 521,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroyNode = 522,
    CUPTI_DRIVER_TRACE_CBID_cuGraphClone = 523,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeFindInClone = 524,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddChildGraphNode = 525,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEmptyNode = 526,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc = 527,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc_ptsz = 528,
    CUPTI_DRIVER_TRACE_CBID_cuGraphChildGraphNodeGetGraph = 529,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddHostNode = 530,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeGetParams = 531,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetLuid = 532,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeSetParams = 533,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetNodes = 534,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetEdges = 535,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo = 536,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_ptsz = 537,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecKernelNodeSetParams = 538,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2 = 539,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2_ptsz = 540,
    CUPTI_DRIVER_TRACE_CBID_cuThreadExchangeStreamCaptureMode = 541,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetNvSciSyncAttributes = 542,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyAvailableDynamicSMemPerBlock = 543,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease_v2 = 544,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset_v2 = 545,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags_v2 = 546,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressReserve = 547,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressFree = 548,
    CUPTI_DRIVER_TRACE_CBID_cuMemCreate = 549,
    CUPTI_DRIVER_TRACE_CBID_cuMemRelease = 550,
    CUPTI_DRIVER_TRACE_CBID_cuMemMap = 551,
    CUPTI_DRIVER_TRACE_CBID_cuMemUnmap = 552,
    CUPTI_DRIVER_TRACE_CBID_cuMemSetAccess = 553,
    CUPTI_DRIVER_TRACE_CBID_cuMemExportToShareableHandle = 554,
    CUPTI_DRIVER_TRACE_CBID_cuMemImportFromShareableHandle = 555,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationGranularity = 556,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationPropertiesFromHandle = 557,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAccess = 558,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags = 559,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags_ptsz = 560,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecUpdate = 561,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemcpyNodeSetParams = 562,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemsetNodeSetParams = 563,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecHostNodeSetParams = 564,
    CUPTI_DRIVER_TRACE_CBID_cuMemRetainAllocationHandle = 565,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetModule = 566,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle_v2 = 567,
    CUPTI_DRIVER_TRACE_CBID_cuCtxResetPersistingL2Cache = 568,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeCopyAttributes = 569,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetAttribute = 570,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetAttribute = 571,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes = 572,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes_ptsz = 573,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute = 574,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute_ptsz = 575,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute = 576,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute_ptsz = 577,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate_v2 = 578,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetTexture1DLinearMaxWidth = 579,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload = 580,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload_ptsz = 581,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetSparseProperties = 582,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetSparseProperties = 583,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync = 584,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync_ptsz = 585,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecChildGraphNodeSetParams = 586,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags = 587,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags_ptsz = 588,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventRecordNode = 589,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventWaitNode = 590,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeGetEvent = 591,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeGetEvent = 592,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeSetEvent = 593,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeSetEvent = 594,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventRecordNodeSetEvent = 595,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventWaitNodeSetEvent = 596,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetPlane = 597,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync = 598,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync_ptsz = 599,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync = 600,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync_ptsz = 601,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolTrimTo = 602,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAttribute = 603,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAttribute = 604,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAccess = 605,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetDefaultMemPool = 606,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolCreate = 607,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolDestroy = 608,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetMemPool = 609,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetMemPool = 610,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync = 611,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync_ptsz = 612,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportToShareableHandle = 613,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportFromShareableHandle = 614,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportPointer = 615,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportPointer = 616,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAccess = 617,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresSignalNode = 618,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeGetParams = 619,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeSetParams = 620,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresWaitNode = 621,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeGetParams = 622,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeSetParams = 623,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresSignalNodeSetParams = 624,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresWaitNodeSetParams = 625,
    CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress = 626,
    CUPTI_DRIVER_TRACE_CBID_cuFlushGPUDirectRDMAWrites = 627,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDebugDotPrint = 628,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2 = 629,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2_ptsz = 630,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies = 631,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_ptsz = 632,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectCreate = 633,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRetain = 634,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRelease = 635,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRetainUserObject = 636,
    CUPTI_DRIVER_TRACE_CBID_cuGraphReleaseUserObject = 637,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemAllocNode = 638,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemFreeNode = 639,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGraphMemTrim = 640,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetGraphMemAttribute = 641,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetGraphMemAttribute = 642,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithFlags = 643,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetExecAffinitySupport = 644,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v3 = 645,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetExecAffinity = 646,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid_v2 = 647,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemAllocNodeGetParams = 648,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemFreeNodeGetParams = 649,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeSetEnabled = 650,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetEnabled = 651,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx = 652,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx_ptsz = 653,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetMemoryRequirements = 654,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetMemoryRequirements = 655,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithParams = 656,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithParams_ptsz = 657,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecGetFlags = 658,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_v2 = 659,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_v2_ptsz = 660,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_v2 = 661,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_v2_ptsz = 662,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_v2 = 663,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_v2_ptsz = 664,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_v2 = 665,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_v2_ptsz = 666,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_v2 = 667,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_v2_ptsz = 668,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddBatchMemOpNode = 669,
    CUPTI_DRIVER_TRACE_CBID_cuGraphBatchMemOpNodeGetParams = 670,
    CUPTI_DRIVER_TRACE_CBID_cuGraphBatchMemOpNodeSetParams = 671,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecBatchMemOpNodeSetParams = 672,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetLoadingMode = 673,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetHandleForAddressRange = 674,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialClusterSize = 675,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveClusters = 676,
    CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress_v2 = 677,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryLoadData = 678,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryLoadFromFile = 679,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryUnload = 680,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetKernel = 681,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetModule = 682,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetFunction = 683,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetGlobal = 684,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetManaged = 685,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetAttribute = 686,
    CUPTI_DRIVER_TRACE_CBID_cuKernelSetAttribute = 687,
    CUPTI_DRIVER_TRACE_CBID_cuKernelSetCacheConfig = 688,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddKernelNode_v2 = 689,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetParams_v2 = 690,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetParams_v2 = 691,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecKernelNodeSetParams_v2 = 692,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetId = 693,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetId_ptsz = 694,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetId = 695,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecUpdate_v2 = 696,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeTiled = 697,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeIm2col = 698,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapReplaceAddress = 699,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetUnifiedFunction = 700,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpGetAttribute = 701,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpGetAttributeGlobal = 702,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpSetAttribute = 703,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpSetAttributeGlobal = 704,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetFlags = 705,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastCreate = 706,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastAddDevice = 707,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastBindMem = 708,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastBindAddr = 709,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastUnbind = 710,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastGetGranularity = 711,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddNode = 712,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeSetParams = 713,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecNodeSetParams = 714,
    CUPTI_DRIVER_TRACE_CBID_cuMemAdvise_v2 = 715,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_v2 = 716,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_v2_ptsz = 717,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetName = 718,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetName = 719,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCaptureToGraph = 720,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCaptureToGraph_ptsz = 721,
    CUPTI_DRIVER_TRACE_CBID_cuGraphConditionalHandleCreate = 722,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddNode_v2 = 723,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetEdges_v2 = 724,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependencies_v2 = 725,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependentNodes_v2 = 726,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddDependencies_v2 = 727,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRemoveDependencies_v2 = 728,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v3 = 729,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v3_ptsz = 730,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_v2 = 731,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_v2_ptsz = 732,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetParamInfo = 733,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetParamInfo = 734,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceRegisterAsyncNotification = 735,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceUnregisterAsyncNotification = 736,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetFunctionCount = 737,
    CUPTI_DRIVER_TRACE_CBID_cuModuleEnumerateFunctions = 738,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetKernelCount = 739,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryEnumerateKernels = 740,
    CUPTI_DRIVER_TRACE_CBID_cuFuncIsLoaded = 741,
    CUPTI_DRIVER_TRACE_CBID_cuFuncLoad = 742,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxCreate = 743,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxDestroy = 744,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetDevResource = 745,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetDevResource = 746,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxGetDevResource = 747,
    CUPTI_DRIVER_TRACE_CBID_cuDevResourceGenerateDesc = 748,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxRecordEvent = 749,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxWaitEvent = 750,
    CUPTI_DRIVER_TRACE_CBID_cuDevSmResourceSplitByCount = 751,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetGreenCtx = 752,
    CUPTI_DRIVER_TRACE_CBID_cuCtxFromGreenCtx = 753,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetLibrary = 754,
    CUPTI_DRIVER_TRACE_CBID_cuCtxRecordEvent = 755,
    CUPTI_DRIVER_TRACE_CBID_cuCtxWaitEvent = 756,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v4 = 757,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxStreamCreate = 758,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_v2 = 759,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_v2_ptsz = 760,
    CUPTI_DRIVER_TRACE_CBID_cuMemBatchDecompressAsync = 761,
    CUPTI_DRIVER_TRACE_CBID_cuMemBatchDecompressAsync_ptsz = 762,
    CUPTI_DRIVER_TRACE_CBID_cuLogsRegisterCallback = 763,
    CUPTI_DRIVER_TRACE_CBID_cuLogsUnregisterCallback = 764,
    CUPTI_DRIVER_TRACE_CBID_cuLogsCurrent = 765,
    CUPTI_DRIVER_TRACE_CBID_cuLogsDumpToFile = 766,
    CUPTI_DRIVER_TRACE_CBID_cuLogsDumpToMemory = 767,
    CUPTI_DRIVER_TRACE_CBID_cuCheckpointProcessGetRestoreThreadId = 768,
    CUPTI_DRIVER_TRACE_CBID_cuCheckpointProcessGetState = 769,
    CUPTI_DRIVER_TRACE_CBID_cuCheckpointProcessLock = 770,
    CUPTI_DRIVER_TRACE_CBID_cuCheckpointProcessCheckpoint = 771,
    CUPTI_DRIVER_TRACE_CBID_cuCheckpointProcessRestore = 772,
    CUPTI_DRIVER_TRACE_CBID_cuCheckpointProcessUnlock = 773,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetDevice = 774,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetDevice_ptsz = 775,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyBatchAsync = 776,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyBatchAsync_ptsz = 777,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DBatchAsync = 778,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DBatchAsync_ptsz = 779,
    CUPTI_DRIVER_TRACE_CBID_cuEventElapsedTime_v2 = 780,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeIm2colWide = 781,
    CUPTI_DRIVER_TRACE_CBID_SIZE = 782,
    CUPTI_DRIVER_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-12090"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_driver_api_trace_cbid_enum {
    CUPTI_DRIVER_TRACE_CBID_INVALID = 0,
    CUPTI_DRIVER_TRACE_CBID_cuInit = 1,
    CUPTI_DRIVER_TRACE_CBID_cuDriverGetVersion = 2,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGet = 3,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetCount = 4,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetName = 5,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceComputeCapability = 6,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem = 7,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetProperties = 8,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetAttribute = 9,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate = 10,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy = 11,
    CUPTI_DRIVER_TRACE_CBID_cuCtxAttach = 12,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDetach = 13,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent = 14,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent = 15,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetDevice = 16,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSynchronize = 17,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoad = 18,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadData = 19,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadDataEx = 20,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadFatBinary = 21,
    CUPTI_DRIVER_TRACE_CBID_cuModuleUnload = 22,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetFunction = 23,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal = 24,
    CUPTI_DRIVER_TRACE_CBID_cu64ModuleGetGlobal = 25,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetTexRef = 26,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo = 27,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetInfo = 28,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc = 29,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAlloc = 30,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch = 31,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAllocPitch = 32,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree = 33,
    CUPTI_DRIVER_TRACE_CBID_cu64MemFree = 34,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange = 35,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetAddressRange = 36,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost = 37,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeHost = 38,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc = 39,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer = 40,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostGetDevicePointer = 41,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetFlags = 42,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD = 43,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoD = 44,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH = 45,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoH = 46,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD = 47,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoD = 48,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA = 49,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoA = 50,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD = 51,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyAtoD = 52,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA = 53,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH = 54,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA = 55,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D = 56,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned = 57,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D = 58,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3D = 59,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync = 60,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoDAsync = 61,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync = 62,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoHAsync = 63,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync = 64,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoDAsync = 65,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync = 66,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync = 67,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync = 68,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync = 69,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3DAsync = 70,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8 = 71,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8 = 72,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16 = 73,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16 = 74,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32 = 75,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32 = 76,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8 = 77,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8 = 78,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16 = 79,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16 = 80,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32 = 81,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32 = 82,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetBlockShape = 83,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedSize = 84,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetAttribute = 85,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetCacheConfig = 86,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate = 87,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor = 88,
    CUPTI_DRIVER_TRACE_CBID_cuArrayDestroy = 89,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate = 90,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor = 91,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefCreate = 92,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefDestroy = 93,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetArray = 94,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress = 95,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress = 96,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D = 97,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress2D = 98,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFormat = 99,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddressMode = 100,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFilterMode = 101,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFlags = 102,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress = 103,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefGetAddress = 104,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetArray = 105,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddressMode = 106,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFilterMode = 107,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFormat = 108,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFlags = 109,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetSize = 110,
    CUPTI_DRIVER_TRACE_CBID_cuParamSeti = 111,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetf = 112,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetv = 113,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetTexRef = 114,
    CUPTI_DRIVER_TRACE_CBID_cuLaunch = 115,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGrid = 116,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGridAsync = 117,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreate = 118,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord = 119,
    CUPTI_DRIVER_TRACE_CBID_cuEventQuery = 120,
    CUPTI_DRIVER_TRACE_CBID_cuEventSynchronize = 121,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy = 122,
    CUPTI_DRIVER_TRACE_CBID_cuEventElapsedTime = 123,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreate = 124,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery = 125,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize = 126,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy = 127,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnregisterResource = 128,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsSubResourceGetMappedArray = 129,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer = 130,
    CUPTI_DRIVER_TRACE_CBID_cu64GraphicsResourceGetMappedPointer = 131,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags = 132,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources = 133,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources = 134,
    CUPTI_DRIVER_TRACE_CBID_cuGetExportTable = 135,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetLimit = 136,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetLimit = 137,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevice = 138,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate = 139,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D10RegisterResource = 140,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10RegisterResource = 141,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnregisterResource = 142,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10MapResources = 143,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnmapResources = 144,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceSetMapFlags = 145,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedArray = 146,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer = 147,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize = 148,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch = 149,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions = 150,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevice = 151,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate = 152,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D11RegisterResource = 153,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevice = 154,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate = 155,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D9RegisterResource = 156,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDirect3DDevice = 157,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterResource = 158,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterResource = 159,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapResources = 160,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapResources = 161,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceSetMapFlags = 162,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions = 163,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedArray = 164,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer = 165,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize = 166,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch = 167,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9Begin = 168,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9End = 169,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterVertexBuffer = 170,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer = 171,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapVertexBuffer = 172,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterVertexBuffer = 173,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate = 174,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterBuffer = 175,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterImage = 176,
    CUPTI_DRIVER_TRACE_CBID_cuWGLGetDevice = 177,
    CUPTI_DRIVER_TRACE_CBID_cuGLInit = 178,
    CUPTI_DRIVER_TRACE_CBID_cuGLRegisterBufferObject = 179,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject = 180,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObject = 181,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnregisterBufferObject = 182,
    CUPTI_DRIVER_TRACE_CBID_cuGLSetBufferObjectMapFlags = 183,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync = 184,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObjectAsync = 185,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUGetDevice = 186,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate = 187,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterVideoSurface = 188,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterOutputSurface = 189,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetSurfRef = 190,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefCreate = 191,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefDestroy = 192,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetFormat = 193,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetArray = 194,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetFormat = 195,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetArray = 196,
    CUPTI_DRIVER_TRACE_CBID_cu64DeviceTotalMem = 197,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPointer = 198,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedSize = 199,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPitch = 200,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetSurfaceDimensions = 201,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetSurfaceDimensions = 202,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPointer = 203,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedSize = 204,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPitch = 205,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9MapVertexBuffer = 206,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObject = 207,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObjectAsync = 208,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevices = 209,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreateOnDevice = 210,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevices = 211,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreateOnDevice = 212,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevices = 213,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreateOnDevice = 214,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostAlloc = 215,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async = 216,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8Async = 217,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async = 218,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16Async = 219,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async = 220,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32Async = 221,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async = 222,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8Async = 223,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async = 224,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16Async = 225,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async = 226,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32Async = 227,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayCreate = 228,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayGetDescriptor = 229,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DCreate = 230,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DGetDescriptor = 231,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2D = 232,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DUnaligned = 233,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DAsync = 234,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v2 = 235,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate_v2 = 236,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate_v2 = 237,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate_v2 = 238,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate_v2 = 239,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate_v2 = 240,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal_v2 = 241,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo_v2 = 242,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc_v2 = 243,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch_v2 = 244,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree_v2 = 245,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange_v2 = 246,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer_v2 = 247,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_v2 = 248,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2 = 249,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2 = 250,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2 = 251,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2 = 252,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2 = 253,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2 = 254,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress_v2 = 255,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v2 = 256,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress_v2 = 257,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer_v2 = 258,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem_v2 = 259,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer_v2 = 260,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize_v2 = 261,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch_v2 = 262,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions_v2 = 263,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions_v2 = 264,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer_v2 = 265,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize_v2 = 266,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch_v2 = 267,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer_v2 = 268,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2 = 269,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2 = 270,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc_v2 = 271,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate_v2 = 272,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor_v2 = 273,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate_v2 = 274,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor_v2 = 275,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2 = 276,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2 = 277,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2 = 278,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2 = 279,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2 = 280,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2 = 281,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2 = 282,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2 = 283,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2 = 284,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2 = 285,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2 = 286,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2 = 287,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2 = 288,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2 = 289,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2 = 290,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2 = 291,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2 = 292,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2 = 293,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost_v2 = 294,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent = 295,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetApiVersion = 296,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDirect3DDevice = 297,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDirect3DDevice = 298,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCacheConfig = 299,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCacheConfig = 300,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister = 301,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostUnregister = 302,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCurrent = 303,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCurrent = 304,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy = 305,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync = 306,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel = 307,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStart = 308,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStop = 309,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttribute = 310,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerInitialize = 311,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceCanAccessPeer = 312,
    CUPTI_DRIVER_TRACE_CBID_cuCtxEnablePeerAccess = 313,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDisablePeerAccess = 314,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerRegister = 315,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerUnregister = 316,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerGetDevicePointer = 317,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer = 318,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync = 319,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer = 320,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync = 321,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy_v2 = 322,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent_v2 = 323,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent_v2 = 324,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy_v2 = 325,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy_v2 = 326,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v3 = 327,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetMemHandle = 328,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle = 329,
    CUPTI_DRIVER_TRACE_CBID_cuIpcCloseMemHandle = 330,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetByPCIBusId = 331,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetPCIBusId = 332,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices = 333,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetEventHandle = 334,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenEventHandle = 335,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetSharedMemConfig = 336,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetSharedMemConfig = 337,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedMemConfig = 338,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectCreate = 339,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectDestroy = 340,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceDesc = 341,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetTextureDesc = 342,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectCreate = 343,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectDestroy = 344,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectGetResourceDesc = 345,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback = 346,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayCreate = 347,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetLevel = 348,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayDestroy = 349,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmappedArray = 350,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapFilterMode = 351,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelBias = 352,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelClamp = 353,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMaxAnisotropy = 354,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmappedArray = 355,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapFilterMode = 356,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelBias = 357,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelClamp = 358,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMaxAnisotropy = 359,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedMipmappedArray = 360,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceViewDesc = 361,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate = 362,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData = 363,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile = 364,
    CUPTI_DRIVER_TRACE_CBID_cuLinkComplete = 365,
    CUPTI_DRIVER_TRACE_CBID_cuLinkDestroy = 366,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreateWithPriority = 367,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority = 368,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags = 369,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetStreamPriorityRange = 370,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocManaged = 371,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorString = 372,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorName = 373,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessor = 374,
    CUPTI_DRIVER_TRACE_CBID_cuCompilePtx = 375,
    CUPTI_DRIVER_TRACE_CBID_cuBinaryFree = 376,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync = 377,
    CUPTI_DRIVER_TRACE_CBID_cuPointerSetAttribute = 378,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister_v2 = 379,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags_v2 = 380,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate_v2 = 381,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData_v2 = 382,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile_v2 = 383,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSize = 384,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices_v2 = 385,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRetain = 386,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease = 387,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags = 388,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset = 389,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsEGLRegisterImage = 390,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetFlags = 391,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxGetState = 392,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnect = 393,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerDisconnect = 394,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerAcquireFrame = 395,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerReleaseFrame = 396,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2_ptds = 397,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2_ptds = 398,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2_ptds = 399,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2_ptds = 400,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2_ptds = 401,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2_ptds = 402,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2_ptds = 403,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2_ptds = 404,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2_ptds = 405,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2_ptds = 406,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2_ptds = 407,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_ptds = 408,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer_ptds = 409,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer_ptds = 410,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2_ptds = 411,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2_ptds = 412,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2_ptds = 413,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2_ptds = 414,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2_ptds = 415,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2_ptds = 416,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2_ptds = 417,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync_ptsz = 418,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2_ptsz = 419,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2_ptsz = 420,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2_ptsz = 421,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2_ptsz = 422,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2_ptsz = 423,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2_ptsz = 424,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2_ptsz = 425,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync_ptsz = 426,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync_ptsz = 427,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async_ptsz = 428,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async_ptsz = 429,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async_ptsz = 430,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async_ptsz = 431,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async_ptsz = 432,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async_ptsz = 433,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority_ptsz = 434,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags_ptsz = 435,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent_ptsz = 436,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback_ptsz = 437,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync_ptsz = 438,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery_ptsz = 439,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize_ptsz = 440,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord_ptsz = 441,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel_ptsz = 442,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources_ptsz = 443,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources_ptsz = 444,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2_ptsz = 445,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerConnect = 446,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerDisconnect = 447,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerPresentFrame = 448,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedEglFrame = 449,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttributes = 450,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessorWithFlags = 451,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSizeWithFlags = 452,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerReturnFrame = 453,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetP2PAttribute = 454,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetBorderColor = 455,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetBorderColor = 456,
    CUPTI_DRIVER_TRACE_CBID_cuMemAdvise = 457,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32 = 458,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_ptsz = 459,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32 = 460,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_ptsz = 461,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp = 462,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_ptsz = 463,
    CUPTI_DRIVER_TRACE_CBID_cuNVNbufferGetPointer = 464,
    CUPTI_DRIVER_TRACE_CBID_cuNVNtextureGetArray = 465,
    CUPTI_DRIVER_TRACE_CBID_cuNNSetAllocator = 466,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync = 467,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_ptsz = 468,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromNVNSync = 469,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnectWithFlags = 470,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttribute = 471,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttributes = 472,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64 = 473,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_ptsz = 474,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64 = 475,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_ptsz = 476,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel = 477,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel_ptsz = 478,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromEGLSync = 479,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernelMultiDevice = 480,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetAttribute = 481,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid = 482,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx = 483,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_ptsz = 484,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalMemory = 485,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedBuffer = 486,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedMipmappedArray = 487,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalMemory = 488,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalSemaphore = 489,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync = 490,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync_ptsz = 491,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync = 492,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync_ptsz = 493,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalSemaphore = 494,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture = 495,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_ptsz = 496,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture = 497,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture_ptsz = 498,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing = 499,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing_ptsz = 500,
    CUPTI_DRIVER_TRACE_CBID_cuGraphCreate = 501,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddKernelNode = 502,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetParams = 503,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemcpyNode = 504,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeGetParams = 505,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemsetNode = 506,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeGetParams = 507,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeSetParams = 508,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetType = 509,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetRootNodes = 510,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependencies = 511,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependentNodes = 512,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate = 513,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch = 514,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch_ptsz = 515,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecDestroy = 516,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroy = 517,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddDependencies = 518,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRemoveDependencies = 519,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeSetParams = 520,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetParams = 521,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroyNode = 522,
    CUPTI_DRIVER_TRACE_CBID_cuGraphClone = 523,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeFindInClone = 524,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddChildGraphNode = 525,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEmptyNode = 526,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc = 527,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc_ptsz = 528,
    CUPTI_DRIVER_TRACE_CBID_cuGraphChildGraphNodeGetGraph = 529,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddHostNode = 530,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeGetParams = 531,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetLuid = 532,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeSetParams = 533,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetNodes = 534,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetEdges = 535,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo = 536,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_ptsz = 537,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecKernelNodeSetParams = 538,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2 = 539,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2_ptsz = 540,
    CUPTI_DRIVER_TRACE_CBID_cuThreadExchangeStreamCaptureMode = 541,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetNvSciSyncAttributes = 542,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyAvailableDynamicSMemPerBlock = 543,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease_v2 = 544,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset_v2 = 545,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags_v2 = 546,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressReserve = 547,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressFree = 548,
    CUPTI_DRIVER_TRACE_CBID_cuMemCreate = 549,
    CUPTI_DRIVER_TRACE_CBID_cuMemRelease = 550,
    CUPTI_DRIVER_TRACE_CBID_cuMemMap = 551,
    CUPTI_DRIVER_TRACE_CBID_cuMemUnmap = 552,
    CUPTI_DRIVER_TRACE_CBID_cuMemSetAccess = 553,
    CUPTI_DRIVER_TRACE_CBID_cuMemExportToShareableHandle = 554,
    CUPTI_DRIVER_TRACE_CBID_cuMemImportFromShareableHandle = 555,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationGranularity = 556,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationPropertiesFromHandle = 557,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAccess = 558,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags = 559,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags_ptsz = 560,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecUpdate = 561,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemcpyNodeSetParams = 562,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemsetNodeSetParams = 563,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecHostNodeSetParams = 564,
    CUPTI_DRIVER_TRACE_CBID_cuMemRetainAllocationHandle = 565,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetModule = 566,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle_v2 = 567,
    CUPTI_DRIVER_TRACE_CBID_cuCtxResetPersistingL2Cache = 568,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeCopyAttributes = 569,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetAttribute = 570,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetAttribute = 571,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes = 572,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes_ptsz = 573,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute = 574,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute_ptsz = 575,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute = 576,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute_ptsz = 577,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate_v2 = 578,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetTexture1DLinearMaxWidth = 579,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload = 580,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload_ptsz = 581,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetSparseProperties = 582,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetSparseProperties = 583,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync = 584,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync_ptsz = 585,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecChildGraphNodeSetParams = 586,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags = 587,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags_ptsz = 588,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventRecordNode = 589,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventWaitNode = 590,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeGetEvent = 591,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeGetEvent = 592,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeSetEvent = 593,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeSetEvent = 594,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventRecordNodeSetEvent = 595,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventWaitNodeSetEvent = 596,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetPlane = 597,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync = 598,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync_ptsz = 599,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync = 600,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync_ptsz = 601,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolTrimTo = 602,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAttribute = 603,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAttribute = 604,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAccess = 605,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetDefaultMemPool = 606,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolCreate = 607,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolDestroy = 608,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetMemPool = 609,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetMemPool = 610,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync = 611,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync_ptsz = 612,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportToShareableHandle = 613,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportFromShareableHandle = 614,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportPointer = 615,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportPointer = 616,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAccess = 617,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresSignalNode = 618,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeGetParams = 619,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeSetParams = 620,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresWaitNode = 621,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeGetParams = 622,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeSetParams = 623,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresSignalNodeSetParams = 624,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresWaitNodeSetParams = 625,
    CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress = 626,
    CUPTI_DRIVER_TRACE_CBID_cuFlushGPUDirectRDMAWrites = 627,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDebugDotPrint = 628,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2 = 629,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2_ptsz = 630,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies = 631,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_ptsz = 632,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectCreate = 633,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRetain = 634,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRelease = 635,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRetainUserObject = 636,
    CUPTI_DRIVER_TRACE_CBID_cuGraphReleaseUserObject = 637,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemAllocNode = 638,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemFreeNode = 639,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGraphMemTrim = 640,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetGraphMemAttribute = 641,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetGraphMemAttribute = 642,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithFlags = 643,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetExecAffinitySupport = 644,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v3 = 645,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetExecAffinity = 646,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid_v2 = 647,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemAllocNodeGetParams = 648,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemFreeNodeGetParams = 649,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeSetEnabled = 650,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetEnabled = 651,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx = 652,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx_ptsz = 653,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetMemoryRequirements = 654,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetMemoryRequirements = 655,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithParams = 656,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithParams_ptsz = 657,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecGetFlags = 658,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_v2 = 659,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_v2_ptsz = 660,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_v2 = 661,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_v2_ptsz = 662,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_v2 = 663,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_v2_ptsz = 664,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_v2 = 665,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_v2_ptsz = 666,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_v2 = 667,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_v2_ptsz = 668,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddBatchMemOpNode = 669,
    CUPTI_DRIVER_TRACE_CBID_cuGraphBatchMemOpNodeGetParams = 670,
    CUPTI_DRIVER_TRACE_CBID_cuGraphBatchMemOpNodeSetParams = 671,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecBatchMemOpNodeSetParams = 672,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetLoadingMode = 673,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetHandleForAddressRange = 674,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialClusterSize = 675,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveClusters = 676,
    CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress_v2 = 677,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryLoadData = 678,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryLoadFromFile = 679,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryUnload = 680,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetKernel = 681,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetModule = 682,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetFunction = 683,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetGlobal = 684,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetManaged = 685,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetAttribute = 686,
    CUPTI_DRIVER_TRACE_CBID_cuKernelSetAttribute = 687,
    CUPTI_DRIVER_TRACE_CBID_cuKernelSetCacheConfig = 688,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddKernelNode_v2 = 689,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetParams_v2 = 690,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetParams_v2 = 691,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecKernelNodeSetParams_v2 = 692,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetId = 693,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetId_ptsz = 694,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetId = 695,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecUpdate_v2 = 696,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeTiled = 697,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeIm2col = 698,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapReplaceAddress = 699,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetUnifiedFunction = 700,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpGetAttribute = 701,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpGetAttributeGlobal = 702,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpSetAttribute = 703,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpSetAttributeGlobal = 704,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetFlags = 705,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastCreate = 706,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastAddDevice = 707,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastBindMem = 708,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastBindAddr = 709,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastUnbind = 710,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastGetGranularity = 711,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddNode = 712,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeSetParams = 713,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecNodeSetParams = 714,
    CUPTI_DRIVER_TRACE_CBID_cuMemAdvise_v2 = 715,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_v2 = 716,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_v2_ptsz = 717,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetName = 718,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetName = 719,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCaptureToGraph = 720,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCaptureToGraph_ptsz = 721,
    CUPTI_DRIVER_TRACE_CBID_cuGraphConditionalHandleCreate = 722,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddNode_v2 = 723,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetEdges_v2 = 724,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependencies_v2 = 725,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependentNodes_v2 = 726,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddDependencies_v2 = 727,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRemoveDependencies_v2 = 728,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v3 = 729,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v3_ptsz = 730,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_v2 = 731,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_v2_ptsz = 732,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetParamInfo = 733,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetParamInfo = 734,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceRegisterAsyncNotification = 735,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceUnregisterAsyncNotification = 736,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetFunctionCount = 737,
    CUPTI_DRIVER_TRACE_CBID_cuModuleEnumerateFunctions = 738,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetKernelCount = 739,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryEnumerateKernels = 740,
    CUPTI_DRIVER_TRACE_CBID_cuFuncIsLoaded = 741,
    CUPTI_DRIVER_TRACE_CBID_cuFuncLoad = 742,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxCreate = 743,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxDestroy = 744,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetDevResource = 745,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetDevResource = 746,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxGetDevResource = 747,
    CUPTI_DRIVER_TRACE_CBID_cuDevResourceGenerateDesc = 748,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxRecordEvent = 749,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxWaitEvent = 750,
    CUPTI_DRIVER_TRACE_CBID_cuDevSmResourceSplitByCount = 751,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetGreenCtx = 752,
    CUPTI_DRIVER_TRACE_CBID_cuCtxFromGreenCtx = 753,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetLibrary = 754,
    CUPTI_DRIVER_TRACE_CBID_cuCtxRecordEvent = 755,
    CUPTI_DRIVER_TRACE_CBID_cuCtxWaitEvent = 756,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v4 = 757,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxStreamCreate = 758,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_v2 = 759,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_v2_ptsz = 760,
    CUPTI_DRIVER_TRACE_CBID_cuMemBatchDecompressAsync = 761,
    CUPTI_DRIVER_TRACE_CBID_cuMemBatchDecompressAsync_ptsz = 762,
    CUPTI_DRIVER_TRACE_CBID_cuLogsRegisterCallback = 763,
    CUPTI_DRIVER_TRACE_CBID_cuLogsUnregisterCallback = 764,
    CUPTI_DRIVER_TRACE_CBID_cuLogsCurrent = 765,
    CUPTI_DRIVER_TRACE_CBID_cuLogsDumpToFile = 766,
    CUPTI_DRIVER_TRACE_CBID_cuLogsDumpToMemory = 767,
    CUPTI_DRIVER_TRACE_CBID_cuCheckpointProcessGetRestoreThreadId = 768,
    CUPTI_DRIVER_TRACE_CBID_cuCheckpointProcessGetState = 769,
    CUPTI_DRIVER_TRACE_CBID_cuCheckpointProcessLock = 770,
    CUPTI_DRIVER_TRACE_CBID_cuCheckpointProcessCheckpoint = 771,
    CUPTI_DRIVER_TRACE_CBID_cuCheckpointProcessRestore = 772,
    CUPTI_DRIVER_TRACE_CBID_cuCheckpointProcessUnlock = 773,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetDevice = 774,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetDevice_ptsz = 775,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyBatchAsync = 776,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyBatchAsync_ptsz = 777,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DBatchAsync = 778,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DBatchAsync_ptsz = 779,
    CUPTI_DRIVER_TRACE_CBID_cuEventElapsedTime_v2 = 780,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeIm2colWide = 781,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxGetId = 782,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreateForCaptureToCig = 783,
    CUPTI_DRIVER_TRACE_CBID_SIZE = 784,
    CUPTI_DRIVER_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-13000"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_driver_api_trace_cbid_enum {
    CUPTI_DRIVER_TRACE_CBID_INVALID = 0,
    CUPTI_DRIVER_TRACE_CBID_cuInit = 1,
    CUPTI_DRIVER_TRACE_CBID_cuDriverGetVersion = 2,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGet = 3,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetCount = 4,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetName = 5,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceComputeCapability = 6,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem = 7,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetProperties = 8,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetAttribute = 9,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate = 10,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy = 11,
    CUPTI_DRIVER_TRACE_CBID_cuCtxAttach = 12,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDetach = 13,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent = 14,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent = 15,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetDevice = 16,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSynchronize = 17,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoad = 18,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadData = 19,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadDataEx = 20,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadFatBinary = 21,
    CUPTI_DRIVER_TRACE_CBID_cuModuleUnload = 22,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetFunction = 23,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal = 24,
    CUPTI_DRIVER_TRACE_CBID_cu64ModuleGetGlobal = 25,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetTexRef = 26,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo = 27,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetInfo = 28,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc = 29,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAlloc = 30,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch = 31,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAllocPitch = 32,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree = 33,
    CUPTI_DRIVER_TRACE_CBID_cu64MemFree = 34,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange = 35,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetAddressRange = 36,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost = 37,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeHost = 38,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc = 39,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer = 40,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostGetDevicePointer = 41,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetFlags = 42,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD = 43,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoD = 44,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH = 45,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoH = 46,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD = 47,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoD = 48,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA = 49,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoA = 50,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD = 51,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyAtoD = 52,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA = 53,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH = 54,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA = 55,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D = 56,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned = 57,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D = 58,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3D = 59,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync = 60,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoDAsync = 61,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync = 62,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoHAsync = 63,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync = 64,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoDAsync = 65,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync = 66,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync = 67,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync = 68,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync = 69,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3DAsync = 70,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8 = 71,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8 = 72,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16 = 73,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16 = 74,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32 = 75,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32 = 76,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8 = 77,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8 = 78,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16 = 79,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16 = 80,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32 = 81,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32 = 82,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetBlockShape = 83,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedSize = 84,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetAttribute = 85,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetCacheConfig = 86,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate = 87,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor = 88,
    CUPTI_DRIVER_TRACE_CBID_cuArrayDestroy = 89,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate = 90,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor = 91,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefCreate = 92,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefDestroy = 93,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetArray = 94,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress = 95,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress = 96,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D = 97,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress2D = 98,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFormat = 99,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddressMode = 100,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFilterMode = 101,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFlags = 102,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress = 103,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefGetAddress = 104,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetArray = 105,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddressMode = 106,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFilterMode = 107,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFormat = 108,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFlags = 109,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetSize = 110,
    CUPTI_DRIVER_TRACE_CBID_cuParamSeti = 111,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetf = 112,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetv = 113,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetTexRef = 114,
    CUPTI_DRIVER_TRACE_CBID_cuLaunch = 115,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGrid = 116,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGridAsync = 117,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreate = 118,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord = 119,
    CUPTI_DRIVER_TRACE_CBID_cuEventQuery = 120,
    CUPTI_DRIVER_TRACE_CBID_cuEventSynchronize = 121,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy = 122,
    CUPTI_DRIVER_TRACE_CBID_cuEventElapsedTime = 123,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreate = 124,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery = 125,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize = 126,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy = 127,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnregisterResource = 128,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsSubResourceGetMappedArray = 129,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer = 130,
    CUPTI_DRIVER_TRACE_CBID_cu64GraphicsResourceGetMappedPointer = 131,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags = 132,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources = 133,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources = 134,
    CUPTI_DRIVER_TRACE_CBID_cuGetExportTable = 135,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetLimit = 136,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetLimit = 137,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevice = 138,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate = 139,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D10RegisterResource = 140,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10RegisterResource = 141,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnregisterResource = 142,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10MapResources = 143,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnmapResources = 144,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceSetMapFlags = 145,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedArray = 146,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer = 147,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize = 148,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch = 149,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions = 150,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevice = 151,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate = 152,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D11RegisterResource = 153,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevice = 154,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate = 155,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D9RegisterResource = 156,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDirect3DDevice = 157,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterResource = 158,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterResource = 159,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapResources = 160,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapResources = 161,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceSetMapFlags = 162,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions = 163,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedArray = 164,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer = 165,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize = 166,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch = 167,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9Begin = 168,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9End = 169,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterVertexBuffer = 170,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer = 171,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapVertexBuffer = 172,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterVertexBuffer = 173,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate = 174,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterBuffer = 175,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterImage = 176,
    CUPTI_DRIVER_TRACE_CBID_cuWGLGetDevice = 177,
    CUPTI_DRIVER_TRACE_CBID_cuGLInit = 178,
    CUPTI_DRIVER_TRACE_CBID_cuGLRegisterBufferObject = 179,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject = 180,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObject = 181,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnregisterBufferObject = 182,
    CUPTI_DRIVER_TRACE_CBID_cuGLSetBufferObjectMapFlags = 183,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync = 184,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObjectAsync = 185,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUGetDevice = 186,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate = 187,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterVideoSurface = 188,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterOutputSurface = 189,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetSurfRef = 190,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefCreate = 191,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefDestroy = 192,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetFormat = 193,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetArray = 194,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetFormat = 195,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetArray = 196,
    CUPTI_DRIVER_TRACE_CBID_cu64DeviceTotalMem = 197,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPointer = 198,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedSize = 199,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPitch = 200,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetSurfaceDimensions = 201,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetSurfaceDimensions = 202,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPointer = 203,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedSize = 204,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPitch = 205,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9MapVertexBuffer = 206,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObject = 207,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObjectAsync = 208,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevices = 209,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreateOnDevice = 210,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevices = 211,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreateOnDevice = 212,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevices = 213,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreateOnDevice = 214,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostAlloc = 215,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async = 216,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8Async = 217,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async = 218,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16Async = 219,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async = 220,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32Async = 221,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async = 222,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8Async = 223,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async = 224,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16Async = 225,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async = 226,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32Async = 227,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayCreate = 228,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayGetDescriptor = 229,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DCreate = 230,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DGetDescriptor = 231,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2D = 232,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DUnaligned = 233,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DAsync = 234,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v2 = 235,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate_v2 = 236,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate_v2 = 237,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate_v2 = 238,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate_v2 = 239,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate_v2 = 240,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal_v2 = 241,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo_v2 = 242,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc_v2 = 243,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch_v2 = 244,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree_v2 = 245,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange_v2 = 246,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer_v2 = 247,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_v2 = 248,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2 = 249,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2 = 250,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2 = 251,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2 = 252,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2 = 253,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2 = 254,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress_v2 = 255,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v2 = 256,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress_v2 = 257,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer_v2 = 258,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem_v2 = 259,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer_v2 = 260,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize_v2 = 261,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch_v2 = 262,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions_v2 = 263,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions_v2 = 264,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer_v2 = 265,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize_v2 = 266,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch_v2 = 267,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer_v2 = 268,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2 = 269,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2 = 270,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc_v2 = 271,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate_v2 = 272,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor_v2 = 273,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate_v2 = 274,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor_v2 = 275,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2 = 276,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2 = 277,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2 = 278,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2 = 279,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2 = 280,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2 = 281,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2 = 282,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2 = 283,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2 = 284,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2 = 285,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2 = 286,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2 = 287,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2 = 288,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2 = 289,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2 = 290,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2 = 291,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2 = 292,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2 = 293,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost_v2 = 294,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent = 295,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetApiVersion = 296,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDirect3DDevice = 297,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDirect3DDevice = 298,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCacheConfig = 299,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCacheConfig = 300,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister = 301,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostUnregister = 302,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCurrent = 303,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCurrent = 304,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy = 305,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync = 306,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel = 307,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStart = 308,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStop = 309,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttribute = 310,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerInitialize = 311,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceCanAccessPeer = 312,
    CUPTI_DRIVER_TRACE_CBID_cuCtxEnablePeerAccess = 313,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDisablePeerAccess = 314,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerRegister = 315,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerUnregister = 316,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerGetDevicePointer = 317,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer = 318,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync = 319,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer = 320,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync = 321,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy_v2 = 322,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent_v2 = 323,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent_v2 = 324,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy_v2 = 325,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy_v2 = 326,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v3 = 327,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetMemHandle = 328,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle = 329,
    CUPTI_DRIVER_TRACE_CBID_cuIpcCloseMemHandle = 330,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetByPCIBusId = 331,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetPCIBusId = 332,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices = 333,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetEventHandle = 334,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenEventHandle = 335,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetSharedMemConfig = 336,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetSharedMemConfig = 337,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedMemConfig = 338,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectCreate = 339,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectDestroy = 340,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceDesc = 341,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetTextureDesc = 342,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectCreate = 343,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectDestroy = 344,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectGetResourceDesc = 345,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback = 346,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayCreate = 347,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetLevel = 348,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayDestroy = 349,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmappedArray = 350,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapFilterMode = 351,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelBias = 352,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelClamp = 353,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMaxAnisotropy = 354,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmappedArray = 355,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapFilterMode = 356,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelBias = 357,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelClamp = 358,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMaxAnisotropy = 359,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedMipmappedArray = 360,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceViewDesc = 361,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate = 362,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData = 363,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile = 364,
    CUPTI_DRIVER_TRACE_CBID_cuLinkComplete = 365,
    CUPTI_DRIVER_TRACE_CBID_cuLinkDestroy = 366,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreateWithPriority = 367,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority = 368,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags = 369,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetStreamPriorityRange = 370,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocManaged = 371,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorString = 372,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorName = 373,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessor = 374,
    CUPTI_DRIVER_TRACE_CBID_cuCompilePtx = 375,
    CUPTI_DRIVER_TRACE_CBID_cuBinaryFree = 376,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync = 377,
    CUPTI_DRIVER_TRACE_CBID_cuPointerSetAttribute = 378,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister_v2 = 379,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags_v2 = 380,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate_v2 = 381,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData_v2 = 382,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile_v2 = 383,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSize = 384,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices_v2 = 385,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRetain = 386,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease = 387,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags = 388,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset = 389,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsEGLRegisterImage = 390,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetFlags = 391,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxGetState = 392,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnect = 393,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerDisconnect = 394,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerAcquireFrame = 395,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerReleaseFrame = 396,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2_ptds = 397,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2_ptds = 398,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2_ptds = 399,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2_ptds = 400,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2_ptds = 401,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2_ptds = 402,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2_ptds = 403,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2_ptds = 404,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2_ptds = 405,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2_ptds = 406,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2_ptds = 407,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_ptds = 408,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer_ptds = 409,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer_ptds = 410,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2_ptds = 411,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2_ptds = 412,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2_ptds = 413,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2_ptds = 414,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2_ptds = 415,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2_ptds = 416,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2_ptds = 417,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync_ptsz = 418,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2_ptsz = 419,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2_ptsz = 420,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2_ptsz = 421,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2_ptsz = 422,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2_ptsz = 423,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2_ptsz = 424,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2_ptsz = 425,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync_ptsz = 426,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync_ptsz = 427,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async_ptsz = 428,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async_ptsz = 429,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async_ptsz = 430,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async_ptsz = 431,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async_ptsz = 432,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async_ptsz = 433,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority_ptsz = 434,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags_ptsz = 435,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent_ptsz = 436,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback_ptsz = 437,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync_ptsz = 438,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery_ptsz = 439,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize_ptsz = 440,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord_ptsz = 441,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel_ptsz = 442,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources_ptsz = 443,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources_ptsz = 444,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2_ptsz = 445,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerConnect = 446,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerDisconnect = 447,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerPresentFrame = 448,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedEglFrame = 449,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttributes = 450,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessorWithFlags = 451,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSizeWithFlags = 452,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerReturnFrame = 453,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetP2PAttribute = 454,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetBorderColor = 455,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetBorderColor = 456,
    CUPTI_DRIVER_TRACE_CBID_cuMemAdvise = 457,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32 = 458,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_ptsz = 459,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32 = 460,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_ptsz = 461,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp = 462,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_ptsz = 463,
    CUPTI_DRIVER_TRACE_CBID_cuNVNbufferGetPointer = 464,
    CUPTI_DRIVER_TRACE_CBID_cuNVNtextureGetArray = 465,
    CUPTI_DRIVER_TRACE_CBID_cuNNSetAllocator = 466,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync = 467,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_ptsz = 468,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromNVNSync = 469,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnectWithFlags = 470,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttribute = 471,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttributes = 472,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64 = 473,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_ptsz = 474,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64 = 475,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_ptsz = 476,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel = 477,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel_ptsz = 478,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromEGLSync = 479,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernelMultiDevice = 480,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetAttribute = 481,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid = 482,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx = 483,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_ptsz = 484,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalMemory = 485,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedBuffer = 486,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedMipmappedArray = 487,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalMemory = 488,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalSemaphore = 489,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync = 490,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync_ptsz = 491,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync = 492,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync_ptsz = 493,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalSemaphore = 494,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture = 495,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_ptsz = 496,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture = 497,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture_ptsz = 498,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing = 499,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing_ptsz = 500,
    CUPTI_DRIVER_TRACE_CBID_cuGraphCreate = 501,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddKernelNode = 502,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetParams = 503,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemcpyNode = 504,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeGetParams = 505,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemsetNode = 506,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeGetParams = 507,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeSetParams = 508,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetType = 509,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetRootNodes = 510,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependencies = 511,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependentNodes = 512,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate = 513,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch = 514,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch_ptsz = 515,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecDestroy = 516,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroy = 517,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddDependencies = 518,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRemoveDependencies = 519,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeSetParams = 520,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetParams = 521,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroyNode = 522,
    CUPTI_DRIVER_TRACE_CBID_cuGraphClone = 523,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeFindInClone = 524,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddChildGraphNode = 525,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEmptyNode = 526,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc = 527,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc_ptsz = 528,
    CUPTI_DRIVER_TRACE_CBID_cuGraphChildGraphNodeGetGraph = 529,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddHostNode = 530,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeGetParams = 531,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetLuid = 532,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeSetParams = 533,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetNodes = 534,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetEdges = 535,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo = 536,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_ptsz = 537,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecKernelNodeSetParams = 538,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2 = 539,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2_ptsz = 540,
    CUPTI_DRIVER_TRACE_CBID_cuThreadExchangeStreamCaptureMode = 541,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetNvSciSyncAttributes = 542,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyAvailableDynamicSMemPerBlock = 543,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease_v2 = 544,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset_v2 = 545,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags_v2 = 546,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressReserve = 547,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressFree = 548,
    CUPTI_DRIVER_TRACE_CBID_cuMemCreate = 549,
    CUPTI_DRIVER_TRACE_CBID_cuMemRelease = 550,
    CUPTI_DRIVER_TRACE_CBID_cuMemMap = 551,
    CUPTI_DRIVER_TRACE_CBID_cuMemUnmap = 552,
    CUPTI_DRIVER_TRACE_CBID_cuMemSetAccess = 553,
    CUPTI_DRIVER_TRACE_CBID_cuMemExportToShareableHandle = 554,
    CUPTI_DRIVER_TRACE_CBID_cuMemImportFromShareableHandle = 555,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationGranularity = 556,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationPropertiesFromHandle = 557,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAccess = 558,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags = 559,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags_ptsz = 560,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecUpdate = 561,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemcpyNodeSetParams = 562,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemsetNodeSetParams = 563,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecHostNodeSetParams = 564,
    CUPTI_DRIVER_TRACE_CBID_cuMemRetainAllocationHandle = 565,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetModule = 566,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle_v2 = 567,
    CUPTI_DRIVER_TRACE_CBID_cuCtxResetPersistingL2Cache = 568,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeCopyAttributes = 569,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetAttribute = 570,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetAttribute = 571,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes = 572,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes_ptsz = 573,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute = 574,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute_ptsz = 575,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute = 576,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute_ptsz = 577,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate_v2 = 578,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetTexture1DLinearMaxWidth = 579,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload = 580,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload_ptsz = 581,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetSparseProperties = 582,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetSparseProperties = 583,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync = 584,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync_ptsz = 585,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecChildGraphNodeSetParams = 586,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags = 587,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags_ptsz = 588,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventRecordNode = 589,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventWaitNode = 590,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeGetEvent = 591,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeGetEvent = 592,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeSetEvent = 593,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeSetEvent = 594,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventRecordNodeSetEvent = 595,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventWaitNodeSetEvent = 596,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetPlane = 597,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync = 598,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync_ptsz = 599,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync = 600,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync_ptsz = 601,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolTrimTo = 602,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAttribute = 603,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAttribute = 604,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAccess = 605,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetDefaultMemPool = 606,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolCreate = 607,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolDestroy = 608,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetMemPool = 609,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetMemPool = 610,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync = 611,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync_ptsz = 612,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportToShareableHandle = 613,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportFromShareableHandle = 614,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportPointer = 615,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportPointer = 616,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAccess = 617,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresSignalNode = 618,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeGetParams = 619,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeSetParams = 620,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresWaitNode = 621,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeGetParams = 622,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeSetParams = 623,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresSignalNodeSetParams = 624,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresWaitNodeSetParams = 625,
    CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress = 626,
    CUPTI_DRIVER_TRACE_CBID_cuFlushGPUDirectRDMAWrites = 627,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDebugDotPrint = 628,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2 = 629,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2_ptsz = 630,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies = 631,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_ptsz = 632,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectCreate = 633,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRetain = 634,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRelease = 635,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRetainUserObject = 636,
    CUPTI_DRIVER_TRACE_CBID_cuGraphReleaseUserObject = 637,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemAllocNode = 638,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemFreeNode = 639,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGraphMemTrim = 640,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetGraphMemAttribute = 641,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetGraphMemAttribute = 642,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithFlags = 643,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetExecAffinitySupport = 644,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v3 = 645,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetExecAffinity = 646,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid_v2 = 647,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemAllocNodeGetParams = 648,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemFreeNodeGetParams = 649,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeSetEnabled = 650,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetEnabled = 651,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx = 652,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx_ptsz = 653,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetMemoryRequirements = 654,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetMemoryRequirements = 655,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithParams = 656,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithParams_ptsz = 657,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecGetFlags = 658,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_v2 = 659,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_v2_ptsz = 660,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_v2 = 661,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_v2_ptsz = 662,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_v2 = 663,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_v2_ptsz = 664,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_v2 = 665,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_v2_ptsz = 666,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_v2 = 667,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_v2_ptsz = 668,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddBatchMemOpNode = 669,
    CUPTI_DRIVER_TRACE_CBID_cuGraphBatchMemOpNodeGetParams = 670,
    CUPTI_DRIVER_TRACE_CBID_cuGraphBatchMemOpNodeSetParams = 671,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecBatchMemOpNodeSetParams = 672,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetLoadingMode = 673,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetHandleForAddressRange = 674,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialClusterSize = 675,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveClusters = 676,
    CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress_v2 = 677,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryLoadData = 678,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryLoadFromFile = 679,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryUnload = 680,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetKernel = 681,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetModule = 682,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetFunction = 683,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetGlobal = 684,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetManaged = 685,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetAttribute = 686,
    CUPTI_DRIVER_TRACE_CBID_cuKernelSetAttribute = 687,
    CUPTI_DRIVER_TRACE_CBID_cuKernelSetCacheConfig = 688,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddKernelNode_v2 = 689,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetParams_v2 = 690,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetParams_v2 = 691,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecKernelNodeSetParams_v2 = 692,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetId = 693,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetId_ptsz = 694,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetId = 695,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecUpdate_v2 = 696,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeTiled = 697,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeIm2col = 698,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapReplaceAddress = 699,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetUnifiedFunction = 700,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpGetAttribute = 701,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpGetAttributeGlobal = 702,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpSetAttribute = 703,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpSetAttributeGlobal = 704,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetFlags = 705,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastCreate = 706,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastAddDevice = 707,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastBindMem = 708,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastBindAddr = 709,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastUnbind = 710,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastGetGranularity = 711,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddNode = 712,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeSetParams = 713,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecNodeSetParams = 714,
    CUPTI_DRIVER_TRACE_CBID_cuMemAdvise_v2 = 715,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_v2 = 716,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_v2_ptsz = 717,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetName = 718,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetName = 719,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCaptureToGraph = 720,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCaptureToGraph_ptsz = 721,
    CUPTI_DRIVER_TRACE_CBID_cuGraphConditionalHandleCreate = 722,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddNode_v2 = 723,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetEdges_v2 = 724,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependencies_v2 = 725,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependentNodes_v2 = 726,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddDependencies_v2 = 727,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRemoveDependencies_v2 = 728,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v3 = 729,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v3_ptsz = 730,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_v2 = 731,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_v2_ptsz = 732,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetParamInfo = 733,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetParamInfo = 734,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceRegisterAsyncNotification = 735,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceUnregisterAsyncNotification = 736,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetFunctionCount = 737,
    CUPTI_DRIVER_TRACE_CBID_cuModuleEnumerateFunctions = 738,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetKernelCount = 739,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryEnumerateKernels = 740,
    CUPTI_DRIVER_TRACE_CBID_cuFuncIsLoaded = 741,
    CUPTI_DRIVER_TRACE_CBID_cuFuncLoad = 742,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxCreate = 743,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxDestroy = 744,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetDevResource = 745,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetDevResource = 746,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxGetDevResource = 747,
    CUPTI_DRIVER_TRACE_CBID_cuDevResourceGenerateDesc = 748,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxRecordEvent = 749,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxWaitEvent = 750,
    CUPTI_DRIVER_TRACE_CBID_cuDevSmResourceSplitByCount = 751,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetGreenCtx = 752,
    CUPTI_DRIVER_TRACE_CBID_cuCtxFromGreenCtx = 753,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetLibrary = 754,
    CUPTI_DRIVER_TRACE_CBID_cuCtxRecordEvent = 755,
    CUPTI_DRIVER_TRACE_CBID_cuCtxWaitEvent = 756,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v4 = 757,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxStreamCreate = 758,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_v2 = 759,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_v2_ptsz = 760,
    CUPTI_DRIVER_TRACE_CBID_cuMemBatchDecompressAsync = 761,
    CUPTI_DRIVER_TRACE_CBID_cuMemBatchDecompressAsync_ptsz = 762,
    CUPTI_DRIVER_TRACE_CBID_cuLogsRegisterCallback = 763,
    CUPTI_DRIVER_TRACE_CBID_cuLogsUnregisterCallback = 764,
    CUPTI_DRIVER_TRACE_CBID_cuLogsCurrent = 765,
    CUPTI_DRIVER_TRACE_CBID_cuLogsDumpToFile = 766,
    CUPTI_DRIVER_TRACE_CBID_cuLogsDumpToMemory = 767,
    CUPTI_DRIVER_TRACE_CBID_cuCheckpointProcessGetRestoreThreadId = 768,
    CUPTI_DRIVER_TRACE_CBID_cuCheckpointProcessGetState = 769,
    CUPTI_DRIVER_TRACE_CBID_cuCheckpointProcessLock = 770,
    CUPTI_DRIVER_TRACE_CBID_cuCheckpointProcessCheckpoint = 771,
    CUPTI_DRIVER_TRACE_CBID_cuCheckpointProcessRestore = 772,
    CUPTI_DRIVER_TRACE_CBID_cuCheckpointProcessUnlock = 773,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetDevice = 774,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetDevice_ptsz = 775,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyBatchAsync = 776,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyBatchAsync_ptsz = 777,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DBatchAsync = 778,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DBatchAsync_ptsz = 779,
    CUPTI_DRIVER_TRACE_CBID_cuEventElapsedTime_v2 = 780,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeIm2colWide = 781,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxGetId = 782,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreateForCaptureToCig = 783,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchBatchAsync = 784,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchBatchAsync_ptsz = 785,
    CUPTI_DRIVER_TRACE_CBID_cuSemaphoreCreate = 786,
    CUPTI_DRIVER_TRACE_CBID_cuSemaphoreExport = 787,
    CUPTI_DRIVER_TRACE_CBID_cuSemaphoreDestroy = 788,
    CUPTI_DRIVER_TRACE_CBID_cuMemDiscardBatchAsync = 789,
    CUPTI_DRIVER_TRACE_CBID_cuMemDiscardBatchAsync_ptsz = 790,
    CUPTI_DRIVER_TRACE_CBID_cuMemDiscardAndPrefetchBatchAsync = 791,
    CUPTI_DRIVER_TRACE_CBID_cuMemDiscardAndPrefetchBatchAsync_ptsz = 792,
    CUPTI_DRIVER_TRACE_CBID_cuMultiKernelCooperativeDomainCreate = 793,
    CUPTI_DRIVER_TRACE_CBID_cuMultiKernelCooperativeDomainDestroy = 794,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetDevice_v2 = 795,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyBatchAsync_v2 = 796,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyBatchAsync_v2_ptsz = 797,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DBatchAsync_v2 = 798,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DBatchAsync_v2_ptsz = 799,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSynchronize_v2 = 800,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetDefaultMemPool = 801,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetMemPool = 802,
    CUPTI_DRIVER_TRACE_CBID_cuMemSetMemPool = 803,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetP2PAtomicCapabilities = 804,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetHostAtomicCapabilities = 805,
    CUPTI_DRIVER_TRACE_CBID_cuDriverGetGpuCodeIsaVersion = 806,
    CUPTI_DRIVER_TRACE_CBID_SIZE = 807,
    CUPTI_DRIVER_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_driver_api_trace_cbid_enum {
    CUPTI_DRIVER_TRACE_CBID_INVALID = 0,
    CUPTI_DRIVER_TRACE_CBID_cuInit = 1,
    CUPTI_DRIVER_TRACE_CBID_cuDriverGetVersion = 2,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGet = 3,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetCount = 4,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetName = 5,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceComputeCapability = 6,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem = 7,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetProperties = 8,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetAttribute = 9,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate = 10,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy = 11,
    CUPTI_DRIVER_TRACE_CBID_cuCtxAttach = 12,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDetach = 13,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent = 14,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent = 15,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetDevice = 16,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSynchronize = 17,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoad = 18,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadData = 19,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadDataEx = 20,
    CUPTI_DRIVER_TRACE_CBID_cuModuleLoadFatBinary = 21,
    CUPTI_DRIVER_TRACE_CBID_cuModuleUnload = 22,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetFunction = 23,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal = 24,
    CUPTI_DRIVER_TRACE_CBID_cu64ModuleGetGlobal = 25,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetTexRef = 26,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo = 27,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetInfo = 28,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc = 29,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAlloc = 30,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch = 31,
    CUPTI_DRIVER_TRACE_CBID_cu64MemAllocPitch = 32,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree = 33,
    CUPTI_DRIVER_TRACE_CBID_cu64MemFree = 34,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange = 35,
    CUPTI_DRIVER_TRACE_CBID_cu64MemGetAddressRange = 36,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost = 37,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeHost = 38,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc = 39,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer = 40,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostGetDevicePointer = 41,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetFlags = 42,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD = 43,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoD = 44,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH = 45,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoH = 46,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD = 47,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoD = 48,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA = 49,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoA = 50,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD = 51,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyAtoD = 52,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA = 53,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH = 54,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA = 55,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D = 56,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned = 57,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D = 58,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3D = 59,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync = 60,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyHtoDAsync = 61,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync = 62,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoHAsync = 63,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync = 64,
    CUPTI_DRIVER_TRACE_CBID_cu64MemcpyDtoDAsync = 65,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync = 66,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync = 67,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync = 68,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync = 69,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy3DAsync = 70,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8 = 71,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8 = 72,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16 = 73,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16 = 74,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32 = 75,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32 = 76,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8 = 77,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8 = 78,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16 = 79,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16 = 80,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32 = 81,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32 = 82,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetBlockShape = 83,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedSize = 84,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetAttribute = 85,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetCacheConfig = 86,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate = 87,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor = 88,
    CUPTI_DRIVER_TRACE_CBID_cuArrayDestroy = 89,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate = 90,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor = 91,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefCreate = 92,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefDestroy = 93,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetArray = 94,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress = 95,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress = 96,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D = 97,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefSetAddress2D = 98,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFormat = 99,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddressMode = 100,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFilterMode = 101,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetFlags = 102,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress = 103,
    CUPTI_DRIVER_TRACE_CBID_cu64TexRefGetAddress = 104,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetArray = 105,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddressMode = 106,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFilterMode = 107,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFormat = 108,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetFlags = 109,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetSize = 110,
    CUPTI_DRIVER_TRACE_CBID_cuParamSeti = 111,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetf = 112,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetv = 113,
    CUPTI_DRIVER_TRACE_CBID_cuParamSetTexRef = 114,
    CUPTI_DRIVER_TRACE_CBID_cuLaunch = 115,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGrid = 116,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchGridAsync = 117,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreate = 118,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord = 119,
    CUPTI_DRIVER_TRACE_CBID_cuEventQuery = 120,
    CUPTI_DRIVER_TRACE_CBID_cuEventSynchronize = 121,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy = 122,
    CUPTI_DRIVER_TRACE_CBID_cuEventElapsedTime = 123,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreate = 124,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery = 125,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize = 126,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy = 127,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnregisterResource = 128,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsSubResourceGetMappedArray = 129,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer = 130,
    CUPTI_DRIVER_TRACE_CBID_cu64GraphicsResourceGetMappedPointer = 131,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags = 132,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources = 133,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources = 134,
    CUPTI_DRIVER_TRACE_CBID_cuGetExportTable = 135,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetLimit = 136,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetLimit = 137,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevice = 138,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate = 139,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D10RegisterResource = 140,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10RegisterResource = 141,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnregisterResource = 142,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10MapResources = 143,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10UnmapResources = 144,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceSetMapFlags = 145,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedArray = 146,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer = 147,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize = 148,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch = 149,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions = 150,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevice = 151,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate = 152,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D11RegisterResource = 153,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevice = 154,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate = 155,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsD3D9RegisterResource = 156,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDirect3DDevice = 157,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterResource = 158,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterResource = 159,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapResources = 160,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapResources = 161,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceSetMapFlags = 162,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions = 163,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedArray = 164,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer = 165,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize = 166,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch = 167,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9Begin = 168,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9End = 169,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9RegisterVertexBuffer = 170,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer = 171,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnmapVertexBuffer = 172,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9UnregisterVertexBuffer = 173,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate = 174,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterBuffer = 175,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsGLRegisterImage = 176,
    CUPTI_DRIVER_TRACE_CBID_cuWGLGetDevice = 177,
    CUPTI_DRIVER_TRACE_CBID_cuGLInit = 178,
    CUPTI_DRIVER_TRACE_CBID_cuGLRegisterBufferObject = 179,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject = 180,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObject = 181,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnregisterBufferObject = 182,
    CUPTI_DRIVER_TRACE_CBID_cuGLSetBufferObjectMapFlags = 183,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync = 184,
    CUPTI_DRIVER_TRACE_CBID_cuGLUnmapBufferObjectAsync = 185,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUGetDevice = 186,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate = 187,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterVideoSurface = 188,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsVDPAURegisterOutputSurface = 189,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetSurfRef = 190,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefCreate = 191,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefDestroy = 192,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetFormat = 193,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefSetArray = 194,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetFormat = 195,
    CUPTI_DRIVER_TRACE_CBID_cuSurfRefGetArray = 196,
    CUPTI_DRIVER_TRACE_CBID_cu64DeviceTotalMem = 197,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPointer = 198,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedSize = 199,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetMappedPitch = 200,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D10ResourceGetSurfaceDimensions = 201,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetSurfaceDimensions = 202,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPointer = 203,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedSize = 204,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9ResourceGetMappedPitch = 205,
    CUPTI_DRIVER_TRACE_CBID_cu64D3D9MapVertexBuffer = 206,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObject = 207,
    CUPTI_DRIVER_TRACE_CBID_cu64GLMapBufferObjectAsync = 208,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDevices = 209,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreateOnDevice = 210,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDevices = 211,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreateOnDevice = 212,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9GetDevices = 213,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreateOnDevice = 214,
    CUPTI_DRIVER_TRACE_CBID_cu64MemHostAlloc = 215,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async = 216,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD8Async = 217,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async = 218,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD16Async = 219,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async = 220,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD32Async = 221,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async = 222,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D8Async = 223,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async = 224,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D16Async = 225,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async = 226,
    CUPTI_DRIVER_TRACE_CBID_cu64MemsetD2D32Async = 227,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayCreate = 228,
    CUPTI_DRIVER_TRACE_CBID_cu64ArrayGetDescriptor = 229,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DCreate = 230,
    CUPTI_DRIVER_TRACE_CBID_cu64Array3DGetDescriptor = 231,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2D = 232,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DUnaligned = 233,
    CUPTI_DRIVER_TRACE_CBID_cu64Memcpy2DAsync = 234,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v2 = 235,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10CtxCreate_v2 = 236,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11CtxCreate_v2 = 237,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9CtxCreate_v2 = 238,
    CUPTI_DRIVER_TRACE_CBID_cuGLCtxCreate_v2 = 239,
    CUPTI_DRIVER_TRACE_CBID_cuVDPAUCtxCreate_v2 = 240,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetGlobal_v2 = 241,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetInfo_v2 = 242,
    CUPTI_DRIVER_TRACE_CBID_cuMemAlloc_v2 = 243,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocPitch_v2 = 244,
    CUPTI_DRIVER_TRACE_CBID_cuMemFree_v2 = 245,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAddressRange_v2 = 246,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostGetDevicePointer_v2 = 247,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_v2 = 248,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2 = 249,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2 = 250,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2 = 251,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2 = 252,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2 = 253,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2 = 254,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress_v2 = 255,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v2 = 256,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetAddress_v2 = 257,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedPointer_v2 = 258,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem_v2 = 259,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPointer_v2 = 260,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedSize_v2 = 261,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetMappedPitch_v2 = 262,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10ResourceGetSurfaceDimensions_v2 = 263,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetSurfaceDimensions_v2 = 264,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPointer_v2 = 265,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedSize_v2 = 266,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9ResourceGetMappedPitch_v2 = 267,
    CUPTI_DRIVER_TRACE_CBID_cuD3D9MapVertexBuffer_v2 = 268,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2 = 269,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2 = 270,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostAlloc_v2 = 271,
    CUPTI_DRIVER_TRACE_CBID_cuArrayCreate_v2 = 272,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetDescriptor_v2 = 273,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DCreate_v2 = 274,
    CUPTI_DRIVER_TRACE_CBID_cuArray3DGetDescriptor_v2 = 275,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2 = 276,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2 = 277,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2 = 278,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2 = 279,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2 = 280,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2 = 281,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2 = 282,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2 = 283,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2 = 284,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2 = 285,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2 = 286,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2 = 287,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2 = 288,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2 = 289,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2 = 290,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2 = 291,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2 = 292,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2 = 293,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocHost_v2 = 294,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent = 295,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetApiVersion = 296,
    CUPTI_DRIVER_TRACE_CBID_cuD3D10GetDirect3DDevice = 297,
    CUPTI_DRIVER_TRACE_CBID_cuD3D11GetDirect3DDevice = 298,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCacheConfig = 299,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCacheConfig = 300,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister = 301,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostUnregister = 302,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetCurrent = 303,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetCurrent = 304,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy = 305,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync = 306,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel = 307,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStart = 308,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerStop = 309,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttribute = 310,
    CUPTI_DRIVER_TRACE_CBID_cuProfilerInitialize = 311,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceCanAccessPeer = 312,
    CUPTI_DRIVER_TRACE_CBID_cuCtxEnablePeerAccess = 313,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDisablePeerAccess = 314,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerRegister = 315,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerUnregister = 316,
    CUPTI_DRIVER_TRACE_CBID_cuMemPeerGetDevicePointer = 317,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer = 318,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync = 319,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer = 320,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync = 321,
    CUPTI_DRIVER_TRACE_CBID_cuCtxDestroy_v2 = 322,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPushCurrent_v2 = 323,
    CUPTI_DRIVER_TRACE_CBID_cuCtxPopCurrent_v2 = 324,
    CUPTI_DRIVER_TRACE_CBID_cuEventDestroy_v2 = 325,
    CUPTI_DRIVER_TRACE_CBID_cuStreamDestroy_v2 = 326,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetAddress2D_v3 = 327,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetMemHandle = 328,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle = 329,
    CUPTI_DRIVER_TRACE_CBID_cuIpcCloseMemHandle = 330,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetByPCIBusId = 331,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetPCIBusId = 332,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices = 333,
    CUPTI_DRIVER_TRACE_CBID_cuIpcGetEventHandle = 334,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenEventHandle = 335,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetSharedMemConfig = 336,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetSharedMemConfig = 337,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetSharedMemConfig = 338,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectCreate = 339,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectDestroy = 340,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceDesc = 341,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetTextureDesc = 342,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectCreate = 343,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectDestroy = 344,
    CUPTI_DRIVER_TRACE_CBID_cuSurfObjectGetResourceDesc = 345,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback = 346,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayCreate = 347,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetLevel = 348,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayDestroy = 349,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmappedArray = 350,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapFilterMode = 351,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelBias = 352,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMipmapLevelClamp = 353,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetMaxAnisotropy = 354,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmappedArray = 355,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapFilterMode = 356,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelBias = 357,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMipmapLevelClamp = 358,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetMaxAnisotropy = 359,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedMipmappedArray = 360,
    CUPTI_DRIVER_TRACE_CBID_cuTexObjectGetResourceViewDesc = 361,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate = 362,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData = 363,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile = 364,
    CUPTI_DRIVER_TRACE_CBID_cuLinkComplete = 365,
    CUPTI_DRIVER_TRACE_CBID_cuLinkDestroy = 366,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCreateWithPriority = 367,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority = 368,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags = 369,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetStreamPriorityRange = 370,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocManaged = 371,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorString = 372,
    CUPTI_DRIVER_TRACE_CBID_cuGetErrorName = 373,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessor = 374,
    CUPTI_DRIVER_TRACE_CBID_cuCompilePtx = 375,
    CUPTI_DRIVER_TRACE_CBID_cuBinaryFree = 376,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync = 377,
    CUPTI_DRIVER_TRACE_CBID_cuPointerSetAttribute = 378,
    CUPTI_DRIVER_TRACE_CBID_cuMemHostRegister_v2 = 379,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceSetMapFlags_v2 = 380,
    CUPTI_DRIVER_TRACE_CBID_cuLinkCreate_v2 = 381,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddData_v2 = 382,
    CUPTI_DRIVER_TRACE_CBID_cuLinkAddFile_v2 = 383,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSize = 384,
    CUPTI_DRIVER_TRACE_CBID_cuGLGetDevices_v2 = 385,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRetain = 386,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease = 387,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags = 388,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset = 389,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsEGLRegisterImage = 390,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetFlags = 391,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxGetState = 392,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnect = 393,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerDisconnect = 394,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerAcquireFrame = 395,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerReleaseFrame = 396,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoD_v2_ptds = 397,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoH_v2_ptds = 398,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoD_v2_ptds = 399,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoA_v2_ptds = 400,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoD_v2_ptds = 401,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoA_v2_ptds = 402,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoH_v2_ptds = 403,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoA_v2_ptds = 404,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2D_v2_ptds = 405,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DUnaligned_v2_ptds = 406,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3D_v2_ptds = 407,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy_ptds = 408,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeer_ptds = 409,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeer_ptds = 410,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8_v2_ptds = 411,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16_v2_ptds = 412,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32_v2_ptds = 413,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8_v2_ptds = 414,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16_v2_ptds = 415,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32_v2_ptds = 416,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObject_v2_ptds = 417,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAsync_ptsz = 418,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoAAsync_v2_ptsz = 419,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyAtoHAsync_v2_ptsz = 420,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyHtoDAsync_v2_ptsz = 421,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoHAsync_v2_ptsz = 422,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyDtoDAsync_v2_ptsz = 423,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy2DAsync_v2_ptsz = 424,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DAsync_v2_ptsz = 425,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyPeerAsync_ptsz = 426,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DPeerAsync_ptsz = 427,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD8Async_ptsz = 428,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD16Async_ptsz = 429,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD32Async_ptsz = 430,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D8Async_ptsz = 431,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D16Async_ptsz = 432,
    CUPTI_DRIVER_TRACE_CBID_cuMemsetD2D32Async_ptsz = 433,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetPriority_ptsz = 434,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetFlags_ptsz = 435,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitEvent_ptsz = 436,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAddCallback_ptsz = 437,
    CUPTI_DRIVER_TRACE_CBID_cuStreamAttachMemAsync_ptsz = 438,
    CUPTI_DRIVER_TRACE_CBID_cuStreamQuery_ptsz = 439,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSynchronize_ptsz = 440,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecord_ptsz = 441,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernel_ptsz = 442,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsMapResources_ptsz = 443,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsUnmapResources_ptsz = 444,
    CUPTI_DRIVER_TRACE_CBID_cuGLMapBufferObjectAsync_v2_ptsz = 445,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerConnect = 446,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerDisconnect = 447,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerPresentFrame = 448,
    CUPTI_DRIVER_TRACE_CBID_cuGraphicsResourceGetMappedEglFrame = 449,
    CUPTI_DRIVER_TRACE_CBID_cuPointerGetAttributes = 450,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveBlocksPerMultiprocessorWithFlags = 451,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialBlockSizeWithFlags = 452,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamProducerReturnFrame = 453,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetP2PAttribute = 454,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefSetBorderColor = 455,
    CUPTI_DRIVER_TRACE_CBID_cuTexRefGetBorderColor = 456,
    CUPTI_DRIVER_TRACE_CBID_cuMemAdvise = 457,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32 = 458,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_ptsz = 459,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32 = 460,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_ptsz = 461,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp = 462,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_ptsz = 463,
    CUPTI_DRIVER_TRACE_CBID_cuNVNbufferGetPointer = 464,
    CUPTI_DRIVER_TRACE_CBID_cuNVNtextureGetArray = 465,
    CUPTI_DRIVER_TRACE_CBID_cuNNSetAllocator = 466,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync = 467,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_ptsz = 468,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromNVNSync = 469,
    CUPTI_DRIVER_TRACE_CBID_cuEGLStreamConsumerConnectWithFlags = 470,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttribute = 471,
    CUPTI_DRIVER_TRACE_CBID_cuMemRangeGetAttributes = 472,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64 = 473,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_ptsz = 474,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64 = 475,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_ptsz = 476,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel = 477,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernel_ptsz = 478,
    CUPTI_DRIVER_TRACE_CBID_cuEventCreateFromEGLSync = 479,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchCooperativeKernelMultiDevice = 480,
    CUPTI_DRIVER_TRACE_CBID_cuFuncSetAttribute = 481,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid = 482,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx = 483,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_ptsz = 484,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalMemory = 485,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedBuffer = 486,
    CUPTI_DRIVER_TRACE_CBID_cuExternalMemoryGetMappedMipmappedArray = 487,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalMemory = 488,
    CUPTI_DRIVER_TRACE_CBID_cuImportExternalSemaphore = 489,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync = 490,
    CUPTI_DRIVER_TRACE_CBID_cuSignalExternalSemaphoresAsync_ptsz = 491,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync = 492,
    CUPTI_DRIVER_TRACE_CBID_cuWaitExternalSemaphoresAsync_ptsz = 493,
    CUPTI_DRIVER_TRACE_CBID_cuDestroyExternalSemaphore = 494,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture = 495,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_ptsz = 496,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture = 497,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCapture_ptsz = 498,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing = 499,
    CUPTI_DRIVER_TRACE_CBID_cuStreamIsCapturing_ptsz = 500,
    CUPTI_DRIVER_TRACE_CBID_cuGraphCreate = 501,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddKernelNode = 502,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetParams = 503,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemcpyNode = 504,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeGetParams = 505,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemsetNode = 506,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeGetParams = 507,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemsetNodeSetParams = 508,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetType = 509,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetRootNodes = 510,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependencies = 511,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependentNodes = 512,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate = 513,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch = 514,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunch_ptsz = 515,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecDestroy = 516,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroy = 517,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddDependencies = 518,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRemoveDependencies = 519,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemcpyNodeSetParams = 520,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetParams = 521,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDestroyNode = 522,
    CUPTI_DRIVER_TRACE_CBID_cuGraphClone = 523,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeFindInClone = 524,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddChildGraphNode = 525,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEmptyNode = 526,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc = 527,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchHostFunc_ptsz = 528,
    CUPTI_DRIVER_TRACE_CBID_cuGraphChildGraphNodeGetGraph = 529,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddHostNode = 530,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeGetParams = 531,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetLuid = 532,
    CUPTI_DRIVER_TRACE_CBID_cuGraphHostNodeSetParams = 533,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetNodes = 534,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetEdges = 535,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo = 536,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_ptsz = 537,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecKernelNodeSetParams = 538,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2 = 539,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCapture_v2_ptsz = 540,
    CUPTI_DRIVER_TRACE_CBID_cuThreadExchangeStreamCaptureMode = 541,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetNvSciSyncAttributes = 542,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyAvailableDynamicSMemPerBlock = 543,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRelease_v2 = 544,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxReset_v2 = 545,
    CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxSetFlags_v2 = 546,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressReserve = 547,
    CUPTI_DRIVER_TRACE_CBID_cuMemAddressFree = 548,
    CUPTI_DRIVER_TRACE_CBID_cuMemCreate = 549,
    CUPTI_DRIVER_TRACE_CBID_cuMemRelease = 550,
    CUPTI_DRIVER_TRACE_CBID_cuMemMap = 551,
    CUPTI_DRIVER_TRACE_CBID_cuMemUnmap = 552,
    CUPTI_DRIVER_TRACE_CBID_cuMemSetAccess = 553,
    CUPTI_DRIVER_TRACE_CBID_cuMemExportToShareableHandle = 554,
    CUPTI_DRIVER_TRACE_CBID_cuMemImportFromShareableHandle = 555,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationGranularity = 556,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAllocationPropertiesFromHandle = 557,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetAccess = 558,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags = 559,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetFlags_ptsz = 560,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecUpdate = 561,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemcpyNodeSetParams = 562,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecMemsetNodeSetParams = 563,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecHostNodeSetParams = 564,
    CUPTI_DRIVER_TRACE_CBID_cuMemRetainAllocationHandle = 565,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetModule = 566,
    CUPTI_DRIVER_TRACE_CBID_cuIpcOpenMemHandle_v2 = 567,
    CUPTI_DRIVER_TRACE_CBID_cuCtxResetPersistingL2Cache = 568,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeCopyAttributes = 569,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetAttribute = 570,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetAttribute = 571,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes = 572,
    CUPTI_DRIVER_TRACE_CBID_cuStreamCopyAttributes_ptsz = 573,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute = 574,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetAttribute_ptsz = 575,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute = 576,
    CUPTI_DRIVER_TRACE_CBID_cuStreamSetAttribute_ptsz = 577,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiate_v2 = 578,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetTexture1DLinearMaxWidth = 579,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload = 580,
    CUPTI_DRIVER_TRACE_CBID_cuGraphUpload_ptsz = 581,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetSparseProperties = 582,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetSparseProperties = 583,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync = 584,
    CUPTI_DRIVER_TRACE_CBID_cuMemMapArrayAsync_ptsz = 585,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecChildGraphNodeSetParams = 586,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags = 587,
    CUPTI_DRIVER_TRACE_CBID_cuEventRecordWithFlags_ptsz = 588,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventRecordNode = 589,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddEventWaitNode = 590,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeGetEvent = 591,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeGetEvent = 592,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventRecordNodeSetEvent = 593,
    CUPTI_DRIVER_TRACE_CBID_cuGraphEventWaitNodeSetEvent = 594,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventRecordNodeSetEvent = 595,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecEventWaitNodeSetEvent = 596,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetPlane = 597,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync = 598,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocAsync_ptsz = 599,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync = 600,
    CUPTI_DRIVER_TRACE_CBID_cuMemFreeAsync_ptsz = 601,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolTrimTo = 602,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAttribute = 603,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAttribute = 604,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolSetAccess = 605,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetDefaultMemPool = 606,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolCreate = 607,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolDestroy = 608,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetMemPool = 609,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetMemPool = 610,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync = 611,
    CUPTI_DRIVER_TRACE_CBID_cuMemAllocFromPoolAsync_ptsz = 612,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportToShareableHandle = 613,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportFromShareableHandle = 614,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolExportPointer = 615,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolImportPointer = 616,
    CUPTI_DRIVER_TRACE_CBID_cuMemPoolGetAccess = 617,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresSignalNode = 618,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeGetParams = 619,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresSignalNodeSetParams = 620,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddExternalSemaphoresWaitNode = 621,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeGetParams = 622,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExternalSemaphoresWaitNodeSetParams = 623,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresSignalNodeSetParams = 624,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecExternalSemaphoresWaitNodeSetParams = 625,
    CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress = 626,
    CUPTI_DRIVER_TRACE_CBID_cuFlushGPUDirectRDMAWrites = 627,
    CUPTI_DRIVER_TRACE_CBID_cuGraphDebugDotPrint = 628,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2 = 629,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v2_ptsz = 630,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies = 631,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_ptsz = 632,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectCreate = 633,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRetain = 634,
    CUPTI_DRIVER_TRACE_CBID_cuUserObjectRelease = 635,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRetainUserObject = 636,
    CUPTI_DRIVER_TRACE_CBID_cuGraphReleaseUserObject = 637,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemAllocNode = 638,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddMemFreeNode = 639,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGraphMemTrim = 640,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetGraphMemAttribute = 641,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceSetGraphMemAttribute = 642,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithFlags = 643,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetExecAffinitySupport = 644,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v3 = 645,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetExecAffinity = 646,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid_v2 = 647,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemAllocNodeGetParams = 648,
    CUPTI_DRIVER_TRACE_CBID_cuGraphMemFreeNodeGetParams = 649,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeSetEnabled = 650,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetEnabled = 651,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx = 652,
    CUPTI_DRIVER_TRACE_CBID_cuLaunchKernelEx_ptsz = 653,
    CUPTI_DRIVER_TRACE_CBID_cuArrayGetMemoryRequirements = 654,
    CUPTI_DRIVER_TRACE_CBID_cuMipmappedArrayGetMemoryRequirements = 655,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithParams = 656,
    CUPTI_DRIVER_TRACE_CBID_cuGraphInstantiateWithParams_ptsz = 657,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecGetFlags = 658,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_v2 = 659,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue32_v2_ptsz = 660,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_v2 = 661,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWaitValue64_v2_ptsz = 662,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_v2 = 663,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue32_v2_ptsz = 664,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_v2 = 665,
    CUPTI_DRIVER_TRACE_CBID_cuStreamWriteValue64_v2_ptsz = 666,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_v2 = 667,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_v2_ptsz = 668,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddBatchMemOpNode = 669,
    CUPTI_DRIVER_TRACE_CBID_cuGraphBatchMemOpNodeGetParams = 670,
    CUPTI_DRIVER_TRACE_CBID_cuGraphBatchMemOpNodeSetParams = 671,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecBatchMemOpNodeSetParams = 672,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetLoadingMode = 673,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetHandleForAddressRange = 674,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxPotentialClusterSize = 675,
    CUPTI_DRIVER_TRACE_CBID_cuOccupancyMaxActiveClusters = 676,
    CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress_v2 = 677,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryLoadData = 678,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryLoadFromFile = 679,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryUnload = 680,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetKernel = 681,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetModule = 682,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetFunction = 683,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetGlobal = 684,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetManaged = 685,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetAttribute = 686,
    CUPTI_DRIVER_TRACE_CBID_cuKernelSetAttribute = 687,
    CUPTI_DRIVER_TRACE_CBID_cuKernelSetCacheConfig = 688,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddKernelNode_v2 = 689,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeGetParams_v2 = 690,
    CUPTI_DRIVER_TRACE_CBID_cuGraphKernelNodeSetParams_v2 = 691,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecKernelNodeSetParams_v2 = 692,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetId = 693,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetId_ptsz = 694,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetId = 695,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecUpdate_v2 = 696,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeTiled = 697,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeIm2col = 698,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapReplaceAddress = 699,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetUnifiedFunction = 700,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpGetAttribute = 701,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpGetAttributeGlobal = 702,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpSetAttribute = 703,
    CUPTI_DRIVER_TRACE_CBID_cuCoredumpSetAttributeGlobal = 704,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSetFlags = 705,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastCreate = 706,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastAddDevice = 707,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastBindMem = 708,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastBindAddr = 709,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastUnbind = 710,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastGetGranularity = 711,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddNode = 712,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeSetParams = 713,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecNodeSetParams = 714,
    CUPTI_DRIVER_TRACE_CBID_cuMemAdvise_v2 = 715,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_v2 = 716,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchAsync_v2_ptsz = 717,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetName = 718,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetName = 719,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCaptureToGraph = 720,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCaptureToGraph_ptsz = 721,
    CUPTI_DRIVER_TRACE_CBID_cuGraphConditionalHandleCreate = 722,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddNode_v2 = 723,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetEdges_v2 = 724,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependencies_v2 = 725,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetDependentNodes_v2 = 726,
    CUPTI_DRIVER_TRACE_CBID_cuGraphAddDependencies_v2 = 727,
    CUPTI_DRIVER_TRACE_CBID_cuGraphRemoveDependencies_v2 = 728,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v3 = 729,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCaptureInfo_v3_ptsz = 730,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_v2 = 731,
    CUPTI_DRIVER_TRACE_CBID_cuStreamUpdateCaptureDependencies_v2_ptsz = 732,
    CUPTI_DRIVER_TRACE_CBID_cuFuncGetParamInfo = 733,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetParamInfo = 734,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceRegisterAsyncNotification = 735,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceUnregisterAsyncNotification = 736,
    CUPTI_DRIVER_TRACE_CBID_cuModuleGetFunctionCount = 737,
    CUPTI_DRIVER_TRACE_CBID_cuModuleEnumerateFunctions = 738,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryGetKernelCount = 739,
    CUPTI_DRIVER_TRACE_CBID_cuLibraryEnumerateKernels = 740,
    CUPTI_DRIVER_TRACE_CBID_cuFuncIsLoaded = 741,
    CUPTI_DRIVER_TRACE_CBID_cuFuncLoad = 742,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxCreate = 743,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxDestroy = 744,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetDevResource = 745,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetDevResource = 746,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxGetDevResource = 747,
    CUPTI_DRIVER_TRACE_CBID_cuDevResourceGenerateDesc = 748,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxRecordEvent = 749,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxWaitEvent = 750,
    CUPTI_DRIVER_TRACE_CBID_cuDevSmResourceSplitByCount = 751,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetGreenCtx = 752,
    CUPTI_DRIVER_TRACE_CBID_cuCtxFromGreenCtx = 753,
    CUPTI_DRIVER_TRACE_CBID_cuKernelGetLibrary = 754,
    CUPTI_DRIVER_TRACE_CBID_cuCtxRecordEvent = 755,
    CUPTI_DRIVER_TRACE_CBID_cuCtxWaitEvent = 756,
    CUPTI_DRIVER_TRACE_CBID_cuCtxCreate_v4 = 757,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxStreamCreate = 758,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_v2 = 759,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetCtx_v2_ptsz = 760,
    CUPTI_DRIVER_TRACE_CBID_cuMemBatchDecompressAsync = 761,
    CUPTI_DRIVER_TRACE_CBID_cuMemBatchDecompressAsync_ptsz = 762,
    CUPTI_DRIVER_TRACE_CBID_cuLogsRegisterCallback = 763,
    CUPTI_DRIVER_TRACE_CBID_cuLogsUnregisterCallback = 764,
    CUPTI_DRIVER_TRACE_CBID_cuLogsCurrent = 765,
    CUPTI_DRIVER_TRACE_CBID_cuLogsDumpToFile = 766,
    CUPTI_DRIVER_TRACE_CBID_cuLogsDumpToMemory = 767,
    CUPTI_DRIVER_TRACE_CBID_cuCheckpointProcessGetRestoreThreadId = 768,
    CUPTI_DRIVER_TRACE_CBID_cuCheckpointProcessGetState = 769,
    CUPTI_DRIVER_TRACE_CBID_cuCheckpointProcessLock = 770,
    CUPTI_DRIVER_TRACE_CBID_cuCheckpointProcessCheckpoint = 771,
    CUPTI_DRIVER_TRACE_CBID_cuCheckpointProcessRestore = 772,
    CUPTI_DRIVER_TRACE_CBID_cuCheckpointProcessUnlock = 773,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetDevice = 774,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetDevice_ptsz = 775,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyBatchAsync = 776,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyBatchAsync_ptsz = 777,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DBatchAsync = 778,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DBatchAsync_ptsz = 779,
    CUPTI_DRIVER_TRACE_CBID_cuEventElapsedTime_v2 = 780,
    CUPTI_DRIVER_TRACE_CBID_cuTensorMapEncodeIm2colWide = 781,
    CUPTI_DRIVER_TRACE_CBID_cuGreenCtxGetId = 782,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCaptureToCig = 783,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchBatchAsync = 784,
    CUPTI_DRIVER_TRACE_CBID_cuMemPrefetchBatchAsync_ptsz = 785,
    CUPTI_DRIVER_TRACE_CBID_cuSemaphoreCreate = 786,
    CUPTI_DRIVER_TRACE_CBID_cuSemaphoreExport = 787,
    CUPTI_DRIVER_TRACE_CBID_cuSemaphoreDestroy = 788,
    CUPTI_DRIVER_TRACE_CBID_cuMemDiscardBatchAsync = 789,
    CUPTI_DRIVER_TRACE_CBID_cuMemDiscardBatchAsync_ptsz = 790,
    CUPTI_DRIVER_TRACE_CBID_cuMemDiscardAndPrefetchBatchAsync = 791,
    CUPTI_DRIVER_TRACE_CBID_cuMemDiscardAndPrefetchBatchAsync_ptsz = 792,
    CUPTI_DRIVER_TRACE_CBID_cuMultiKernelCooperativeDomainCreate = 793,
    CUPTI_DRIVER_TRACE_CBID_cuMultiKernelCooperativeDomainDestroy = 794,
    CUPTI_DRIVER_TRACE_CBID_cuCtxGetDevice_v2 = 795,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyBatchAsync_v2 = 796,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpyBatchAsync_v2_ptsz = 797,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DBatchAsync_v2 = 798,
    CUPTI_DRIVER_TRACE_CBID_cuMemcpy3DBatchAsync_v2_ptsz = 799,
    CUPTI_DRIVER_TRACE_CBID_cuCtxSynchronize_v2 = 800,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetDefaultMemPool = 801,
    CUPTI_DRIVER_TRACE_CBID_cuMemGetMemPool = 802,
    CUPTI_DRIVER_TRACE_CBID_cuMemSetMemPool = 803,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetP2PAtomicCapabilities = 804,
    CUPTI_DRIVER_TRACE_CBID_cuDeviceGetHostAtomicCapabilities = 805,
    CUPTI_DRIVER_TRACE_CBID_cuDriverGetGpuCodeIsaVersion = 806,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetDevResource = 807,
    CUPTI_DRIVER_TRACE_CBID_cuStreamGetDevResource_ptsz = 808,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetContainingGraph = 809,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetLocalId = 810,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeGetToolsId = 811,
    CUPTI_DRIVER_TRACE_CBID_cuGraphGetId = 812,
    CUPTI_DRIVER_TRACE_CBID_cuGraphExecGetId = 813,
    CUPTI_DRIVER_TRACE_CBID_cuStreamBeginCaptureToCig_ptsz = 814,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCaptureToCig = 815,
    CUPTI_DRIVER_TRACE_CBID_cuStreamEndCaptureToCig_ptsz = 816,
    CUPTI_DRIVER_TRACE_CBID_cuGraphCreateWithArguments = 817,
    CUPTI_DRIVER_TRACE_CBID_cuGraphNodeAddRelocation = 818,
    CUPTI_DRIVER_TRACE_CBID_cuGraphLaunchWithArguments = 819,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastBindMem_v2 = 820,
    CUPTI_DRIVER_TRACE_CBID_cuMulticastBindAddr_v2 = 821,
    CUPTI_DRIVER_TRACE_CBID_cuDevSmResourceSplit = 822,
    CUPTI_DRIVER_TRACE_CBID_SIZE = 823,
    CUPTI_DRIVER_TRACE_CBID_FORCE_INT = 2147483647,
}
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_nvtx_api_trace_cbid {
    CUPTI_CBID_NVTX_INVALID = 0,
    CUPTI_CBID_NVTX_nvtxMarkA = 1,
    CUPTI_CBID_NVTX_nvtxMarkW = 2,
    CUPTI_CBID_NVTX_nvtxMarkEx = 3,
    CUPTI_CBID_NVTX_nvtxRangeStartA = 4,
    CUPTI_CBID_NVTX_nvtxRangeStartW = 5,
    CUPTI_CBID_NVTX_nvtxRangeStartEx = 6,
    CUPTI_CBID_NVTX_nvtxRangeEnd = 7,
    CUPTI_CBID_NVTX_nvtxRangePushA = 8,
    CUPTI_CBID_NVTX_nvtxRangePushW = 9,
    CUPTI_CBID_NVTX_nvtxRangePushEx = 10,
    CUPTI_CBID_NVTX_nvtxRangePop = 11,
    CUPTI_CBID_NVTX_nvtxNameCategoryA = 12,
    CUPTI_CBID_NVTX_nvtxNameCategoryW = 13,
    CUPTI_CBID_NVTX_nvtxNameOsThreadA = 14,
    CUPTI_CBID_NVTX_nvtxNameOsThreadW = 15,
    CUPTI_CBID_NVTX_nvtxNameCuDeviceA = 16,
    CUPTI_CBID_NVTX_nvtxNameCuDeviceW = 17,
    CUPTI_CBID_NVTX_nvtxNameCuContextA = 18,
    CUPTI_CBID_NVTX_nvtxNameCuContextW = 19,
    CUPTI_CBID_NVTX_nvtxNameCuStreamA = 20,
    CUPTI_CBID_NVTX_nvtxNameCuStreamW = 21,
    CUPTI_CBID_NVTX_nvtxNameCuEventA = 22,
    CUPTI_CBID_NVTX_nvtxNameCuEventW = 23,
    CUPTI_CBID_NVTX_nvtxNameCudaDeviceA = 24,
    CUPTI_CBID_NVTX_nvtxNameCudaDeviceW = 25,
    CUPTI_CBID_NVTX_nvtxNameCudaStreamA = 26,
    CUPTI_CBID_NVTX_nvtxNameCudaStreamW = 27,
    CUPTI_CBID_NVTX_nvtxNameCudaEventA = 28,
    CUPTI_CBID_NVTX_nvtxNameCudaEventW = 29,
    CUPTI_CBID_NVTX_nvtxDomainMarkEx = 30,
    CUPTI_CBID_NVTX_nvtxDomainRangeStartEx = 31,
    CUPTI_CBID_NVTX_nvtxDomainRangeEnd = 32,
    CUPTI_CBID_NVTX_nvtxDomainRangePushEx = 33,
    CUPTI_CBID_NVTX_nvtxDomainRangePop = 34,
    CUPTI_CBID_NVTX_nvtxDomainResourceCreate = 35,
    CUPTI_CBID_NVTX_nvtxDomainResourceDestroy = 36,
    CUPTI_CBID_NVTX_nvtxDomainNameCategoryA = 37,
    CUPTI_CBID_NVTX_nvtxDomainNameCategoryW = 38,
    CUPTI_CBID_NVTX_nvtxDomainRegisterStringA = 39,
    CUPTI_CBID_NVTX_nvtxDomainRegisterStringW = 40,
    CUPTI_CBID_NVTX_nvtxDomainCreateA = 41,
    CUPTI_CBID_NVTX_nvtxDomainCreateW = 42,
    CUPTI_CBID_NVTX_nvtxDomainDestroy = 43,
    CUPTI_CBID_NVTX_nvtxDomainSyncUserCreate = 44,
    CUPTI_CBID_NVTX_nvtxDomainSyncUserDestroy = 45,
    CUPTI_CBID_NVTX_nvtxDomainSyncUserAcquireStart = 46,
    CUPTI_CBID_NVTX_nvtxDomainSyncUserAcquireFailed = 47,
    CUPTI_CBID_NVTX_nvtxDomainSyncUserAcquireSuccess = 48,
    CUPTI_CBID_NVTX_nvtxDomainSyncUserReleasing = 49,
    CUPTI_CBID_NVTX_SIZE = 50,
    CUPTI_CBID_NVTX_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-11040", feature = "cuda-11050"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_runtime_api_trace_cbid_enum {
    CUPTI_RUNTIME_TRACE_CBID_INVALID = 0,
    CUPTI_RUNTIME_TRACE_CBID_cudaDriverGetVersion_v3020 = 1,
    CUPTI_RUNTIME_TRACE_CBID_cudaRuntimeGetVersion_v3020 = 2,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceCount_v3020 = 3,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v3020 = 4,
    CUPTI_RUNTIME_TRACE_CBID_cudaChooseDevice_v3020 = 5,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetChannelDesc_v3020 = 6,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateChannelDesc_v3020 = 7,
    CUPTI_RUNTIME_TRACE_CBID_cudaConfigureCall_v3020 = 8,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetupArgument_v3020 = 9,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetLastError_v3020 = 10,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeekAtLastError_v3020 = 11,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorString_v3020 = 12,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_v3020 = 13,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetCacheConfig_v3020 = 14,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetAttributes_v3020 = 15,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDevice_v3020 = 16,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDevice_v3020 = 17,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetValidDevices_v3020 = 18,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDeviceFlags_v3020 = 19,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc_v3020 = 20,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocPitch_v3020 = 21,
    CUPTI_RUNTIME_TRACE_CBID_cudaFree_v3020 = 22,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocArray_v3020 = 23,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeArray_v3020 = 24,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocHost_v3020 = 25,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeHost_v3020 = 26,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostAlloc_v3020 = 27,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetDevicePointer_v3020 = 28,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetFlags_v3020 = 29,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemGetInfo_v3020 = 30,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_v3020 = 31,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_v3020 = 32,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_v3020 = 33,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_v3020 = 34,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_v3020 = 35,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_v3020 = 36,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_v3020 = 37,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_v3020 = 38,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_v3020 = 39,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_v3020 = 40,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_v3020 = 41,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_v3020 = 42,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_v3020 = 43,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_v3020 = 44,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_v3020 = 45,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_v3020 = 46,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_v3020 = 47,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_v3020 = 48,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_v3020 = 49,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_v3020 = 50,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_v3020 = 51,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_v3020 = 52,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolAddress_v3020 = 53,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolSize_v3020 = 54,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture_v3020 = 55,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture2D_v3020 = 56,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToArray_v3020 = 57,
    CUPTI_RUNTIME_TRACE_CBID_cudaUnbindTexture_v3020 = 58,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureAlignmentOffset_v3020 = 59,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureReference_v3020 = 60,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindSurfaceToArray_v3020 = 61,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceReference_v3020 = 62,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetGLDevice_v3020 = 63,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLRegisterBufferObject_v3020 = 64,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObject_v3020 = 65,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObject_v3020 = 66,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnregisterBufferObject_v3020 = 67,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetBufferObjectMapFlags_v3020 = 68,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObjectAsync_v3020 = 69,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObjectAsync_v3020 = 70,
    CUPTI_RUNTIME_TRACE_CBID_cudaWGLGetDevice_v3020 = 71,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterImage_v3020 = 72,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterBuffer_v3020 = 73,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnregisterResource_v3020 = 74,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceSetMapFlags_v3020 = 75,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsMapResources_v3020 = 76,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnmapResources_v3020 = 77,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedPointer_v3020 = 78,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsSubResourceGetMappedArray_v3020 = 79,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUGetDevice_v3020 = 80,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUSetVDPAUDevice_v3020 = 81,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterVideoSurface_v3020 = 82,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterOutputSurface_v3020 = 83,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevice_v3020 = 84,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevices_v3020 = 85,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11SetDirect3DDevice_v3020 = 86,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D11RegisterResource_v3020 = 87,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevice_v3020 = 88,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevices_v3020 = 89,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10SetDirect3DDevice_v3020 = 90,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D10RegisterResource_v3020 = 91,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10RegisterResource_v3020 = 92,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnregisterResource_v3020 = 93,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10MapResources_v3020 = 94,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnmapResources_v3020 = 95,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceSetMapFlags_v3020 = 96,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetSurfaceDimensions_v3020 = 97,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedArray_v3020 = 98,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPointer_v3020 = 99,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedSize_v3020 = 100,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPitch_v3020 = 101,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevice_v3020 = 102,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevices_v3020 = 103,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9SetDirect3DDevice_v3020 = 104,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDirect3DDevice_v3020 = 105,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D9RegisterResource_v3020 = 106,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterResource_v3020 = 107,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterResource_v3020 = 108,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapResources_v3020 = 109,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapResources_v3020 = 110,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceSetMapFlags_v3020 = 111,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetSurfaceDimensions_v3020 = 112,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedArray_v3020 = 113,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPointer_v3020 = 114,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedSize_v3020 = 115,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPitch_v3020 = 116,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9Begin_v3020 = 117,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9End_v3020 = 118,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterVertexBuffer_v3020 = 119,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterVertexBuffer_v3020 = 120,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapVertexBuffer_v3020 = 121,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapVertexBuffer_v3020 = 122,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExit_v3020 = 123,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForDevice_v3020 = 124,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForHost_v3020 = 125,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSynchronize_v3020 = 126,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetLimit_v3020 = 127,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetLimit_v3020 = 128,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreate_v3020 = 129,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v3020 = 130,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_v3020 = 131,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_v3020 = 132,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreate_v3020 = 133,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateWithFlags_v3020 = 134,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_v3020 = 135,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventDestroy_v3020 = 136,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventSynchronize_v3020 = 137,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventQuery_v3020 = 138,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventElapsedTime_v3020 = 139,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3D_v3020 = 140,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3DArray_v3020 = 141,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_v3020 = 142,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_v3020 = 143,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_v3020 = 144,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_v3020 = 145,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetCacheConfig_v3020 = 146,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_v3020 = 147,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDirect3DDevice_v3020 = 148,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDirect3DDevice_v3020 = 149,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetCacheConfig_v3020 = 150,
    CUPTI_RUNTIME_TRACE_CBID_cudaPointerGetAttributes_v4000 = 151,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostRegister_v4000 = 152,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostUnregister_v4000 = 153,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceCanAccessPeer_v4000 = 154,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceEnablePeerAccess_v4000 = 155,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceDisablePeerAccess_v4000 = 156,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerRegister_v4000 = 157,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerUnregister_v4000 = 158,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerGetDevicePointer_v4000 = 159,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeer_v4000 = 160,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeerAsync_v4000 = 161,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_v4000 = 162,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_v4000 = 163,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceReset_v3020 = 164,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSynchronize_v3020 = 165,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetLimit_v3020 = 166,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetLimit_v3020 = 167,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetCacheConfig_v3020 = 168,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetCacheConfig_v3020 = 169,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerInitialize_v4000 = 170,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStart_v4000 = 171,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStop_v4000 = 172,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetByPCIBusId_v4010 = 173,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetPCIBusId_v4010 = 174,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLGetDevices_v4010 = 175,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetEventHandle_v4010 = 176,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenEventHandle_v4010 = 177,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetMemHandle_v4010 = 178,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenMemHandle_v4010 = 179,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcCloseMemHandle_v4010 = 180,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetInfo_v4010 = 181,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetSharedMemConfig_v4020 = 182,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetSharedMemConfig_v4020 = 183,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetSharedMemConfig_v4020 = 184,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateTextureObject_v5000 = 185,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyTextureObject_v5000 = 186,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceDesc_v5000 = 187,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectTextureDesc_v5000 = 188,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateSurfaceObject_v5000 = 189,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroySurfaceObject_v5000 = 190,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceObjectResourceDesc_v5000 = 191,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocMipmappedArray_v5000 = 192,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetMipmappedArrayLevel_v5000 = 193,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeMipmappedArray_v5000 = 194,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToMipmappedArray_v5000 = 195,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedMipmappedArray_v5000 = 196,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_v5000 = 197,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithFlags_v5000 = 198,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceViewDesc_v5000 = 199,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetAttribute_v5000 = 200,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v5050 = 201,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithPriority_v5050 = 202,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_v5050 = 203,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_v5050 = 204,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetStreamPriorityRange_v5050 = 205,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocManaged_v6000 = 206,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6000 = 207,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_v6000 = 208,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorName_v6050 = 209,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6050 = 210,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_v7000 = 211,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceFlags_v7000 = 212,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_ptsz_v7000 = 213,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_ptsz_v7000 = 214,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_ptds_v7000 = 215,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_ptds_v7000 = 216,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_ptds_v7000 = 217,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_ptds_v7000 = 218,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_ptds_v7000 = 219,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_ptds_v7000 = 220,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_ptds_v7000 = 221,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_ptds_v7000 = 222,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_ptds_v7000 = 223,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_ptds_v7000 = 224,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_ptsz_v7000 = 225,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_ptsz_v7000 = 226,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_ptsz_v7000 = 227,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_ptsz_v7000 = 228,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_ptsz_v7000 = 229,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_ptsz_v7000 = 230,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_ptsz_v7000 = 231,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_ptsz_v7000 = 232,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_ptds_v7000 = 233,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_ptds_v7000 = 234,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_ptsz_v7000 = 235,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_ptsz_v7000 = 236,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_ptsz_v7000 = 237,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_ptsz_v7000 = 238,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_ptsz_v7000 = 239,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_ptsz_v7000 = 240,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_ptsz_v7000 = 241,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_ptsz_v7000 = 242,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_ptds_v7000 = 243,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_ptsz_v7000 = 244,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_ptds_v7000 = 245,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_ptsz_v7000 = 246,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_ptsz_v7000 = 247,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_ptsz_v7000 = 248,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_ptds_v7000 = 249,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_ptsz_v7000 = 250,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessorWithFlags_v7000 = 251,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v8000 = 252,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_ptsz_v8000 = 253,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemAdvise_v8000 = 254,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetP2PAttribute_v8000 = 255,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsEGLRegisterImage_v7000 = 256,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnect_v7000 = 257,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerDisconnect_v7000 = 258,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerAcquireFrame_v7000 = 259,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerReleaseFrame_v7000 = 260,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerConnect_v7000 = 261,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerDisconnect_v7000 = 262,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerPresentFrame_v7000 = 263,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerReturnFrame_v7000 = 264,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedEglFrame_v7000 = 265,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttribute_v8000 = 266,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttributes_v8000 = 267,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnectWithFlags_v7000 = 268,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_v9000 = 269,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_ptsz_v9000 = 270,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateFromEGLSync_v9000 = 271,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernelMultiDevice_v9000 = 272,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetAttribute_v9000 = 273,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalMemory_v10000 = 274,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedBuffer_v10000 = 275,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedMipmappedArray_v10000 = 276,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalMemory_v10000 = 277,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalSemaphore_v10000 = 278,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v10000 = 279,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_ptsz_v10000 = 280,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v10000 = 281,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_ptsz_v10000 = 282,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalSemaphore_v10000 = 283,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_v10000 = 284,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_ptsz_v10000 = 285,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphCreate_v10000 = 286,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetParams_v10000 = 287,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetParams_v10000 = 288,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddKernelNode_v10000 = 289,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode_v10000 = 290,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeGetParams_v10000 = 291,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams_v10000 = 292,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemsetNode_v10000 = 293,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeGetParams_v10000 = 294,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeSetParams_v10000 = 295,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddHostNode_v10000 = 296,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeGetParams_v10000 = 297,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddChildGraphNode_v10000 = 298,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphChildGraphNodeGetGraph_v10000 = 299,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEmptyNode_v10000 = 300,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphClone_v10000 = 301,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeFindInClone_v10000 = 302,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetType_v10000 = 303,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetRootNodes_v10000 = 304,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependencies_v10000 = 305,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependentNodes_v10000 = 306,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddDependencies_v10000 = 307,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRemoveDependencies_v10000 = 308,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroyNode_v10000 = 309,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiate_v10000 = 310,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_v10000 = 311,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_ptsz_v10000 = 312,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecDestroy_v10000 = 313,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroy_v10000 = 314,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_v10000 = 315,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_ptsz_v10000 = 316,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_v10000 = 317,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_ptsz_v10000 = 318,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_v10000 = 319,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_ptsz_v10000 = 320,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeSetParams_v10000 = 321,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetNodes_v10000 = 322,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetEdges_v10000 = 323,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v10010 = 324,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_ptsz_v10010 = 325,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecKernelNodeSetParams_v10010 = 326,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExchangeStreamCaptureMode_v10010 = 327,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetNvSciSyncAttributes_v10020 = 328,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyAvailableDynamicSMemPerBlock_v10200 = 329,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_v10200 = 330,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_ptsz_v10200 = 331,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams_v10020 = 332,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemsetNodeSetParams_v10020 = 333,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecHostNodeSetParams_v10020 = 334,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecUpdate_v10020 = 335,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetFuncBySymbol_v11000 = 336,
    CUPTI_RUNTIME_TRACE_CBID_cudaCtxResetPersistingL2Cache_v11000 = 337,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeCopyAttributes_v11000 = 338,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetAttribute_v11000 = 339,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetAttribute_v11000 = 340,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_v11000 = 341,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_ptsz_v11000 = 342,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_v11000 = 343,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_ptsz_v11000 = 344,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_v11000 = 345,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_ptsz_v11000 = 346,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetTexture1DLinearMaxWidth_v11010 = 347,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_v10000 = 348,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_ptsz_v10000 = 349,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeToSymbol_v11010 = 350,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeFromSymbol_v11010 = 351,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode1D_v11010 = 352,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsToSymbol_v11010 = 353,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsFromSymbol_v11010 = 354,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams1D_v11010 = 355,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsToSymbol_v11010 = 356,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsFromSymbol_v11010 = 357,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams1D_v11010 = 358,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetSparseProperties_v11010 = 359,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetSparseProperties_v11010 = 360,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecChildGraphNodeSetParams_v11010 = 361,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventRecordNode_v11010 = 362,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeGetEvent_v11010 = 363,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeSetEvent_v11010 = 364,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventWaitNode_v11010 = 365,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeGetEvent_v11010 = 366,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeSetEvent_v11010 = 367,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventRecordNodeSetEvent_v11010 = 368,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventWaitNodeSetEvent_v11010 = 369,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_v11010 = 370,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_ptsz_v11010 = 371,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetDefaultMemPool_v11020 = 372,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_v11020 = 373,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_ptsz_v11020 = 374,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_v11020 = 375,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_ptsz_v11020 = 376,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolTrimTo_v11020 = 377,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAttribute_v11020 = 378,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAttribute_v11020 = 379,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAccess_v11020 = 380,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetPlane_v11020 = 381,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAccess_v11020 = 382,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolCreate_v11020 = 383,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolDestroy_v11020 = 384,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetMemPool_v11020 = 385,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetMemPool_v11020 = 386,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportToShareableHandle_v11020 = 387,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportFromShareableHandle_v11020 = 388,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportPointer_v11020 = 389,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportPointer_v11020 = 390,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_v11020 = 391,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_ptsz_v11020 = 392,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v2_v11020 = 393,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v2_ptsz_v11020 = 394,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v2_v11020 = 395,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v2_ptsz_v11020 = 396,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresSignalNode_v11020 = 397,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeGetParams_v11020 = 398,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeSetParams_v11020 = 399,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresWaitNode_v11020 = 400,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeGetParams_v11020 = 401,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeSetParams_v11020 = 402,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresSignalNodeSetParams_v11020 = 403,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresWaitNodeSetParams_v11020 = 404,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceFlushGPUDirectRDMAWrites_v11030 = 405,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_v11030 = 406,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_ptsz_v11030 = 407,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDebugDotPrint_v11030 = 408,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_v11030 = 409,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_ptsz_v11030 = 410,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v11030 = 411,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_ptsz_v11030 = 412,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectCreate_v11030 = 413,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRetain_v11030 = 414,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRelease_v11030 = 415,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRetainUserObject_v11030 = 416,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphReleaseUserObject_v11030 = 417,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithFlags_v11040 = 418,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemAllocNode_v11040 = 419,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemAllocNodeGetParams_v11040 = 420,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemFreeNode_v11040 = 421,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemFreeNodeGetParams_v11040 = 422,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGraphMemTrim_v11040 = 423,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetGraphMemAttribute_v11040 = 424,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetGraphMemAttribute_v11040 = 425,
    CUPTI_RUNTIME_TRACE_CBID_SIZE = 426,
    CUPTI_RUNTIME_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-11060", feature = "cuda-11070"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_runtime_api_trace_cbid_enum {
    CUPTI_RUNTIME_TRACE_CBID_INVALID = 0,
    CUPTI_RUNTIME_TRACE_CBID_cudaDriverGetVersion_v3020 = 1,
    CUPTI_RUNTIME_TRACE_CBID_cudaRuntimeGetVersion_v3020 = 2,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceCount_v3020 = 3,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v3020 = 4,
    CUPTI_RUNTIME_TRACE_CBID_cudaChooseDevice_v3020 = 5,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetChannelDesc_v3020 = 6,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateChannelDesc_v3020 = 7,
    CUPTI_RUNTIME_TRACE_CBID_cudaConfigureCall_v3020 = 8,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetupArgument_v3020 = 9,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetLastError_v3020 = 10,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeekAtLastError_v3020 = 11,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorString_v3020 = 12,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_v3020 = 13,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetCacheConfig_v3020 = 14,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetAttributes_v3020 = 15,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDevice_v3020 = 16,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDevice_v3020 = 17,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetValidDevices_v3020 = 18,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDeviceFlags_v3020 = 19,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc_v3020 = 20,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocPitch_v3020 = 21,
    CUPTI_RUNTIME_TRACE_CBID_cudaFree_v3020 = 22,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocArray_v3020 = 23,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeArray_v3020 = 24,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocHost_v3020 = 25,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeHost_v3020 = 26,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostAlloc_v3020 = 27,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetDevicePointer_v3020 = 28,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetFlags_v3020 = 29,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemGetInfo_v3020 = 30,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_v3020 = 31,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_v3020 = 32,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_v3020 = 33,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_v3020 = 34,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_v3020 = 35,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_v3020 = 36,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_v3020 = 37,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_v3020 = 38,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_v3020 = 39,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_v3020 = 40,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_v3020 = 41,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_v3020 = 42,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_v3020 = 43,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_v3020 = 44,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_v3020 = 45,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_v3020 = 46,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_v3020 = 47,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_v3020 = 48,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_v3020 = 49,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_v3020 = 50,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_v3020 = 51,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_v3020 = 52,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolAddress_v3020 = 53,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolSize_v3020 = 54,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture_v3020 = 55,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture2D_v3020 = 56,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToArray_v3020 = 57,
    CUPTI_RUNTIME_TRACE_CBID_cudaUnbindTexture_v3020 = 58,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureAlignmentOffset_v3020 = 59,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureReference_v3020 = 60,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindSurfaceToArray_v3020 = 61,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceReference_v3020 = 62,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetGLDevice_v3020 = 63,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLRegisterBufferObject_v3020 = 64,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObject_v3020 = 65,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObject_v3020 = 66,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnregisterBufferObject_v3020 = 67,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetBufferObjectMapFlags_v3020 = 68,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObjectAsync_v3020 = 69,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObjectAsync_v3020 = 70,
    CUPTI_RUNTIME_TRACE_CBID_cudaWGLGetDevice_v3020 = 71,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterImage_v3020 = 72,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterBuffer_v3020 = 73,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnregisterResource_v3020 = 74,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceSetMapFlags_v3020 = 75,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsMapResources_v3020 = 76,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnmapResources_v3020 = 77,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedPointer_v3020 = 78,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsSubResourceGetMappedArray_v3020 = 79,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUGetDevice_v3020 = 80,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUSetVDPAUDevice_v3020 = 81,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterVideoSurface_v3020 = 82,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterOutputSurface_v3020 = 83,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevice_v3020 = 84,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevices_v3020 = 85,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11SetDirect3DDevice_v3020 = 86,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D11RegisterResource_v3020 = 87,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevice_v3020 = 88,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevices_v3020 = 89,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10SetDirect3DDevice_v3020 = 90,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D10RegisterResource_v3020 = 91,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10RegisterResource_v3020 = 92,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnregisterResource_v3020 = 93,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10MapResources_v3020 = 94,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnmapResources_v3020 = 95,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceSetMapFlags_v3020 = 96,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetSurfaceDimensions_v3020 = 97,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedArray_v3020 = 98,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPointer_v3020 = 99,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedSize_v3020 = 100,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPitch_v3020 = 101,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevice_v3020 = 102,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevices_v3020 = 103,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9SetDirect3DDevice_v3020 = 104,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDirect3DDevice_v3020 = 105,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D9RegisterResource_v3020 = 106,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterResource_v3020 = 107,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterResource_v3020 = 108,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapResources_v3020 = 109,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapResources_v3020 = 110,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceSetMapFlags_v3020 = 111,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetSurfaceDimensions_v3020 = 112,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedArray_v3020 = 113,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPointer_v3020 = 114,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedSize_v3020 = 115,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPitch_v3020 = 116,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9Begin_v3020 = 117,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9End_v3020 = 118,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterVertexBuffer_v3020 = 119,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterVertexBuffer_v3020 = 120,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapVertexBuffer_v3020 = 121,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapVertexBuffer_v3020 = 122,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExit_v3020 = 123,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForDevice_v3020 = 124,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForHost_v3020 = 125,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSynchronize_v3020 = 126,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetLimit_v3020 = 127,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetLimit_v3020 = 128,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreate_v3020 = 129,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v3020 = 130,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_v3020 = 131,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_v3020 = 132,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreate_v3020 = 133,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateWithFlags_v3020 = 134,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_v3020 = 135,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventDestroy_v3020 = 136,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventSynchronize_v3020 = 137,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventQuery_v3020 = 138,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventElapsedTime_v3020 = 139,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3D_v3020 = 140,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3DArray_v3020 = 141,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_v3020 = 142,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_v3020 = 143,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_v3020 = 144,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_v3020 = 145,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetCacheConfig_v3020 = 146,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_v3020 = 147,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDirect3DDevice_v3020 = 148,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDirect3DDevice_v3020 = 149,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetCacheConfig_v3020 = 150,
    CUPTI_RUNTIME_TRACE_CBID_cudaPointerGetAttributes_v4000 = 151,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostRegister_v4000 = 152,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostUnregister_v4000 = 153,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceCanAccessPeer_v4000 = 154,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceEnablePeerAccess_v4000 = 155,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceDisablePeerAccess_v4000 = 156,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerRegister_v4000 = 157,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerUnregister_v4000 = 158,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerGetDevicePointer_v4000 = 159,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeer_v4000 = 160,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeerAsync_v4000 = 161,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_v4000 = 162,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_v4000 = 163,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceReset_v3020 = 164,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSynchronize_v3020 = 165,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetLimit_v3020 = 166,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetLimit_v3020 = 167,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetCacheConfig_v3020 = 168,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetCacheConfig_v3020 = 169,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerInitialize_v4000 = 170,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStart_v4000 = 171,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStop_v4000 = 172,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetByPCIBusId_v4010 = 173,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetPCIBusId_v4010 = 174,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLGetDevices_v4010 = 175,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetEventHandle_v4010 = 176,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenEventHandle_v4010 = 177,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetMemHandle_v4010 = 178,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenMemHandle_v4010 = 179,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcCloseMemHandle_v4010 = 180,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetInfo_v4010 = 181,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetSharedMemConfig_v4020 = 182,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetSharedMemConfig_v4020 = 183,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetSharedMemConfig_v4020 = 184,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateTextureObject_v5000 = 185,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyTextureObject_v5000 = 186,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceDesc_v5000 = 187,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectTextureDesc_v5000 = 188,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateSurfaceObject_v5000 = 189,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroySurfaceObject_v5000 = 190,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceObjectResourceDesc_v5000 = 191,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocMipmappedArray_v5000 = 192,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetMipmappedArrayLevel_v5000 = 193,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeMipmappedArray_v5000 = 194,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToMipmappedArray_v5000 = 195,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedMipmappedArray_v5000 = 196,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_v5000 = 197,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithFlags_v5000 = 198,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceViewDesc_v5000 = 199,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetAttribute_v5000 = 200,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v5050 = 201,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithPriority_v5050 = 202,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_v5050 = 203,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_v5050 = 204,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetStreamPriorityRange_v5050 = 205,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocManaged_v6000 = 206,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6000 = 207,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_v6000 = 208,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorName_v6050 = 209,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6050 = 210,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_v7000 = 211,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceFlags_v7000 = 212,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_ptsz_v7000 = 213,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_ptsz_v7000 = 214,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_ptds_v7000 = 215,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_ptds_v7000 = 216,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_ptds_v7000 = 217,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_ptds_v7000 = 218,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_ptds_v7000 = 219,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_ptds_v7000 = 220,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_ptds_v7000 = 221,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_ptds_v7000 = 222,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_ptds_v7000 = 223,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_ptds_v7000 = 224,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_ptsz_v7000 = 225,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_ptsz_v7000 = 226,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_ptsz_v7000 = 227,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_ptsz_v7000 = 228,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_ptsz_v7000 = 229,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_ptsz_v7000 = 230,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_ptsz_v7000 = 231,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_ptsz_v7000 = 232,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_ptds_v7000 = 233,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_ptds_v7000 = 234,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_ptsz_v7000 = 235,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_ptsz_v7000 = 236,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_ptsz_v7000 = 237,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_ptsz_v7000 = 238,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_ptsz_v7000 = 239,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_ptsz_v7000 = 240,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_ptsz_v7000 = 241,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_ptsz_v7000 = 242,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_ptds_v7000 = 243,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_ptsz_v7000 = 244,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_ptds_v7000 = 245,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_ptsz_v7000 = 246,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_ptsz_v7000 = 247,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_ptsz_v7000 = 248,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_ptds_v7000 = 249,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_ptsz_v7000 = 250,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessorWithFlags_v7000 = 251,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v8000 = 252,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_ptsz_v8000 = 253,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemAdvise_v8000 = 254,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetP2PAttribute_v8000 = 255,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsEGLRegisterImage_v7000 = 256,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnect_v7000 = 257,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerDisconnect_v7000 = 258,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerAcquireFrame_v7000 = 259,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerReleaseFrame_v7000 = 260,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerConnect_v7000 = 261,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerDisconnect_v7000 = 262,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerPresentFrame_v7000 = 263,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerReturnFrame_v7000 = 264,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedEglFrame_v7000 = 265,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttribute_v8000 = 266,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttributes_v8000 = 267,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnectWithFlags_v7000 = 268,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_v9000 = 269,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_ptsz_v9000 = 270,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateFromEGLSync_v9000 = 271,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernelMultiDevice_v9000 = 272,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetAttribute_v9000 = 273,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalMemory_v10000 = 274,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedBuffer_v10000 = 275,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedMipmappedArray_v10000 = 276,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalMemory_v10000 = 277,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalSemaphore_v10000 = 278,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v10000 = 279,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_ptsz_v10000 = 280,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v10000 = 281,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_ptsz_v10000 = 282,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalSemaphore_v10000 = 283,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_v10000 = 284,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_ptsz_v10000 = 285,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphCreate_v10000 = 286,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetParams_v10000 = 287,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetParams_v10000 = 288,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddKernelNode_v10000 = 289,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode_v10000 = 290,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeGetParams_v10000 = 291,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams_v10000 = 292,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemsetNode_v10000 = 293,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeGetParams_v10000 = 294,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeSetParams_v10000 = 295,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddHostNode_v10000 = 296,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeGetParams_v10000 = 297,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddChildGraphNode_v10000 = 298,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphChildGraphNodeGetGraph_v10000 = 299,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEmptyNode_v10000 = 300,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphClone_v10000 = 301,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeFindInClone_v10000 = 302,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetType_v10000 = 303,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetRootNodes_v10000 = 304,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependencies_v10000 = 305,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependentNodes_v10000 = 306,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddDependencies_v10000 = 307,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRemoveDependencies_v10000 = 308,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroyNode_v10000 = 309,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiate_v10000 = 310,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_v10000 = 311,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_ptsz_v10000 = 312,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecDestroy_v10000 = 313,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroy_v10000 = 314,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_v10000 = 315,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_ptsz_v10000 = 316,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_v10000 = 317,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_ptsz_v10000 = 318,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_v10000 = 319,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_ptsz_v10000 = 320,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeSetParams_v10000 = 321,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetNodes_v10000 = 322,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetEdges_v10000 = 323,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v10010 = 324,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_ptsz_v10010 = 325,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecKernelNodeSetParams_v10010 = 326,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExchangeStreamCaptureMode_v10010 = 327,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetNvSciSyncAttributes_v10020 = 328,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyAvailableDynamicSMemPerBlock_v10200 = 329,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_v10200 = 330,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_ptsz_v10200 = 331,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams_v10020 = 332,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemsetNodeSetParams_v10020 = 333,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecHostNodeSetParams_v10020 = 334,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecUpdate_v10020 = 335,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetFuncBySymbol_v11000 = 336,
    CUPTI_RUNTIME_TRACE_CBID_cudaCtxResetPersistingL2Cache_v11000 = 337,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeCopyAttributes_v11000 = 338,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetAttribute_v11000 = 339,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetAttribute_v11000 = 340,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_v11000 = 341,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_ptsz_v11000 = 342,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_v11000 = 343,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_ptsz_v11000 = 344,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_v11000 = 345,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_ptsz_v11000 = 346,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetTexture1DLinearMaxWidth_v11010 = 347,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_v10000 = 348,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_ptsz_v10000 = 349,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeToSymbol_v11010 = 350,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeFromSymbol_v11010 = 351,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode1D_v11010 = 352,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsToSymbol_v11010 = 353,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsFromSymbol_v11010 = 354,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams1D_v11010 = 355,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsToSymbol_v11010 = 356,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsFromSymbol_v11010 = 357,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams1D_v11010 = 358,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetSparseProperties_v11010 = 359,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetSparseProperties_v11010 = 360,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecChildGraphNodeSetParams_v11010 = 361,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventRecordNode_v11010 = 362,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeGetEvent_v11010 = 363,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeSetEvent_v11010 = 364,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventWaitNode_v11010 = 365,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeGetEvent_v11010 = 366,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeSetEvent_v11010 = 367,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventRecordNodeSetEvent_v11010 = 368,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventWaitNodeSetEvent_v11010 = 369,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_v11010 = 370,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_ptsz_v11010 = 371,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetDefaultMemPool_v11020 = 372,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_v11020 = 373,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_ptsz_v11020 = 374,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_v11020 = 375,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_ptsz_v11020 = 376,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolTrimTo_v11020 = 377,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAttribute_v11020 = 378,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAttribute_v11020 = 379,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAccess_v11020 = 380,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetPlane_v11020 = 381,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAccess_v11020 = 382,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolCreate_v11020 = 383,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolDestroy_v11020 = 384,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetMemPool_v11020 = 385,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetMemPool_v11020 = 386,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportToShareableHandle_v11020 = 387,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportFromShareableHandle_v11020 = 388,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportPointer_v11020 = 389,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportPointer_v11020 = 390,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_v11020 = 391,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_ptsz_v11020 = 392,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v2_v11020 = 393,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v2_ptsz_v11020 = 394,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v2_v11020 = 395,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v2_ptsz_v11020 = 396,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresSignalNode_v11020 = 397,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeGetParams_v11020 = 398,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeSetParams_v11020 = 399,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresWaitNode_v11020 = 400,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeGetParams_v11020 = 401,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeSetParams_v11020 = 402,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresSignalNodeSetParams_v11020 = 403,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresWaitNodeSetParams_v11020 = 404,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceFlushGPUDirectRDMAWrites_v11030 = 405,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_v11030 = 406,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_ptsz_v11030 = 407,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDebugDotPrint_v11030 = 408,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_v11030 = 409,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_ptsz_v11030 = 410,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v11030 = 411,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_ptsz_v11030 = 412,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectCreate_v11030 = 413,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRetain_v11030 = 414,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRelease_v11030 = 415,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRetainUserObject_v11030 = 416,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphReleaseUserObject_v11030 = 417,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithFlags_v11040 = 418,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemAllocNode_v11040 = 419,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemAllocNodeGetParams_v11040 = 420,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemFreeNode_v11040 = 421,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemFreeNodeGetParams_v11040 = 422,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGraphMemTrim_v11040 = 423,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetGraphMemAttribute_v11040 = 424,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetGraphMemAttribute_v11040 = 425,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeSetEnabled_v11060 = 426,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetEnabled_v11060 = 427,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetMemoryRequirements_v11060 = 428,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetMemoryRequirements_v11060 = 429,
    CUPTI_RUNTIME_TRACE_CBID_SIZE = 430,
    CUPTI_RUNTIME_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-11080"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_runtime_api_trace_cbid_enum {
    CUPTI_RUNTIME_TRACE_CBID_INVALID = 0,
    CUPTI_RUNTIME_TRACE_CBID_cudaDriverGetVersion_v3020 = 1,
    CUPTI_RUNTIME_TRACE_CBID_cudaRuntimeGetVersion_v3020 = 2,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceCount_v3020 = 3,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v3020 = 4,
    CUPTI_RUNTIME_TRACE_CBID_cudaChooseDevice_v3020 = 5,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetChannelDesc_v3020 = 6,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateChannelDesc_v3020 = 7,
    CUPTI_RUNTIME_TRACE_CBID_cudaConfigureCall_v3020 = 8,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetupArgument_v3020 = 9,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetLastError_v3020 = 10,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeekAtLastError_v3020 = 11,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorString_v3020 = 12,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_v3020 = 13,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetCacheConfig_v3020 = 14,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetAttributes_v3020 = 15,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDevice_v3020 = 16,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDevice_v3020 = 17,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetValidDevices_v3020 = 18,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDeviceFlags_v3020 = 19,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc_v3020 = 20,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocPitch_v3020 = 21,
    CUPTI_RUNTIME_TRACE_CBID_cudaFree_v3020 = 22,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocArray_v3020 = 23,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeArray_v3020 = 24,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocHost_v3020 = 25,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeHost_v3020 = 26,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostAlloc_v3020 = 27,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetDevicePointer_v3020 = 28,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetFlags_v3020 = 29,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemGetInfo_v3020 = 30,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_v3020 = 31,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_v3020 = 32,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_v3020 = 33,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_v3020 = 34,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_v3020 = 35,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_v3020 = 36,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_v3020 = 37,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_v3020 = 38,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_v3020 = 39,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_v3020 = 40,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_v3020 = 41,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_v3020 = 42,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_v3020 = 43,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_v3020 = 44,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_v3020 = 45,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_v3020 = 46,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_v3020 = 47,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_v3020 = 48,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_v3020 = 49,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_v3020 = 50,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_v3020 = 51,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_v3020 = 52,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolAddress_v3020 = 53,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolSize_v3020 = 54,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture_v3020 = 55,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture2D_v3020 = 56,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToArray_v3020 = 57,
    CUPTI_RUNTIME_TRACE_CBID_cudaUnbindTexture_v3020 = 58,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureAlignmentOffset_v3020 = 59,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureReference_v3020 = 60,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindSurfaceToArray_v3020 = 61,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceReference_v3020 = 62,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetGLDevice_v3020 = 63,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLRegisterBufferObject_v3020 = 64,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObject_v3020 = 65,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObject_v3020 = 66,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnregisterBufferObject_v3020 = 67,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetBufferObjectMapFlags_v3020 = 68,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObjectAsync_v3020 = 69,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObjectAsync_v3020 = 70,
    CUPTI_RUNTIME_TRACE_CBID_cudaWGLGetDevice_v3020 = 71,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterImage_v3020 = 72,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterBuffer_v3020 = 73,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnregisterResource_v3020 = 74,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceSetMapFlags_v3020 = 75,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsMapResources_v3020 = 76,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnmapResources_v3020 = 77,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedPointer_v3020 = 78,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsSubResourceGetMappedArray_v3020 = 79,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUGetDevice_v3020 = 80,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUSetVDPAUDevice_v3020 = 81,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterVideoSurface_v3020 = 82,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterOutputSurface_v3020 = 83,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevice_v3020 = 84,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevices_v3020 = 85,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11SetDirect3DDevice_v3020 = 86,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D11RegisterResource_v3020 = 87,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevice_v3020 = 88,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevices_v3020 = 89,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10SetDirect3DDevice_v3020 = 90,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D10RegisterResource_v3020 = 91,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10RegisterResource_v3020 = 92,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnregisterResource_v3020 = 93,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10MapResources_v3020 = 94,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnmapResources_v3020 = 95,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceSetMapFlags_v3020 = 96,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetSurfaceDimensions_v3020 = 97,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedArray_v3020 = 98,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPointer_v3020 = 99,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedSize_v3020 = 100,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPitch_v3020 = 101,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevice_v3020 = 102,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevices_v3020 = 103,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9SetDirect3DDevice_v3020 = 104,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDirect3DDevice_v3020 = 105,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D9RegisterResource_v3020 = 106,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterResource_v3020 = 107,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterResource_v3020 = 108,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapResources_v3020 = 109,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapResources_v3020 = 110,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceSetMapFlags_v3020 = 111,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetSurfaceDimensions_v3020 = 112,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedArray_v3020 = 113,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPointer_v3020 = 114,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedSize_v3020 = 115,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPitch_v3020 = 116,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9Begin_v3020 = 117,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9End_v3020 = 118,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterVertexBuffer_v3020 = 119,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterVertexBuffer_v3020 = 120,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapVertexBuffer_v3020 = 121,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapVertexBuffer_v3020 = 122,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExit_v3020 = 123,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForDevice_v3020 = 124,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForHost_v3020 = 125,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSynchronize_v3020 = 126,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetLimit_v3020 = 127,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetLimit_v3020 = 128,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreate_v3020 = 129,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v3020 = 130,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_v3020 = 131,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_v3020 = 132,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreate_v3020 = 133,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateWithFlags_v3020 = 134,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_v3020 = 135,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventDestroy_v3020 = 136,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventSynchronize_v3020 = 137,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventQuery_v3020 = 138,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventElapsedTime_v3020 = 139,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3D_v3020 = 140,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3DArray_v3020 = 141,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_v3020 = 142,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_v3020 = 143,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_v3020 = 144,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_v3020 = 145,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetCacheConfig_v3020 = 146,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_v3020 = 147,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDirect3DDevice_v3020 = 148,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDirect3DDevice_v3020 = 149,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetCacheConfig_v3020 = 150,
    CUPTI_RUNTIME_TRACE_CBID_cudaPointerGetAttributes_v4000 = 151,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostRegister_v4000 = 152,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostUnregister_v4000 = 153,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceCanAccessPeer_v4000 = 154,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceEnablePeerAccess_v4000 = 155,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceDisablePeerAccess_v4000 = 156,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerRegister_v4000 = 157,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerUnregister_v4000 = 158,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerGetDevicePointer_v4000 = 159,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeer_v4000 = 160,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeerAsync_v4000 = 161,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_v4000 = 162,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_v4000 = 163,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceReset_v3020 = 164,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSynchronize_v3020 = 165,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetLimit_v3020 = 166,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetLimit_v3020 = 167,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetCacheConfig_v3020 = 168,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetCacheConfig_v3020 = 169,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerInitialize_v4000 = 170,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStart_v4000 = 171,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStop_v4000 = 172,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetByPCIBusId_v4010 = 173,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetPCIBusId_v4010 = 174,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLGetDevices_v4010 = 175,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetEventHandle_v4010 = 176,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenEventHandle_v4010 = 177,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetMemHandle_v4010 = 178,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenMemHandle_v4010 = 179,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcCloseMemHandle_v4010 = 180,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetInfo_v4010 = 181,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetSharedMemConfig_v4020 = 182,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetSharedMemConfig_v4020 = 183,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetSharedMemConfig_v4020 = 184,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateTextureObject_v5000 = 185,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyTextureObject_v5000 = 186,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceDesc_v5000 = 187,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectTextureDesc_v5000 = 188,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateSurfaceObject_v5000 = 189,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroySurfaceObject_v5000 = 190,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceObjectResourceDesc_v5000 = 191,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocMipmappedArray_v5000 = 192,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetMipmappedArrayLevel_v5000 = 193,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeMipmappedArray_v5000 = 194,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToMipmappedArray_v5000 = 195,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedMipmappedArray_v5000 = 196,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_v5000 = 197,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithFlags_v5000 = 198,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceViewDesc_v5000 = 199,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetAttribute_v5000 = 200,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v5050 = 201,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithPriority_v5050 = 202,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_v5050 = 203,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_v5050 = 204,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetStreamPriorityRange_v5050 = 205,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocManaged_v6000 = 206,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6000 = 207,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_v6000 = 208,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorName_v6050 = 209,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6050 = 210,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_v7000 = 211,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceFlags_v7000 = 212,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_ptsz_v7000 = 213,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_ptsz_v7000 = 214,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_ptds_v7000 = 215,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_ptds_v7000 = 216,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_ptds_v7000 = 217,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_ptds_v7000 = 218,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_ptds_v7000 = 219,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_ptds_v7000 = 220,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_ptds_v7000 = 221,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_ptds_v7000 = 222,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_ptds_v7000 = 223,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_ptds_v7000 = 224,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_ptsz_v7000 = 225,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_ptsz_v7000 = 226,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_ptsz_v7000 = 227,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_ptsz_v7000 = 228,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_ptsz_v7000 = 229,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_ptsz_v7000 = 230,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_ptsz_v7000 = 231,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_ptsz_v7000 = 232,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_ptds_v7000 = 233,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_ptds_v7000 = 234,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_ptsz_v7000 = 235,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_ptsz_v7000 = 236,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_ptsz_v7000 = 237,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_ptsz_v7000 = 238,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_ptsz_v7000 = 239,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_ptsz_v7000 = 240,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_ptsz_v7000 = 241,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_ptsz_v7000 = 242,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_ptds_v7000 = 243,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_ptsz_v7000 = 244,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_ptds_v7000 = 245,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_ptsz_v7000 = 246,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_ptsz_v7000 = 247,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_ptsz_v7000 = 248,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_ptds_v7000 = 249,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_ptsz_v7000 = 250,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessorWithFlags_v7000 = 251,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v8000 = 252,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_ptsz_v8000 = 253,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemAdvise_v8000 = 254,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetP2PAttribute_v8000 = 255,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsEGLRegisterImage_v7000 = 256,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnect_v7000 = 257,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerDisconnect_v7000 = 258,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerAcquireFrame_v7000 = 259,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerReleaseFrame_v7000 = 260,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerConnect_v7000 = 261,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerDisconnect_v7000 = 262,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerPresentFrame_v7000 = 263,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerReturnFrame_v7000 = 264,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedEglFrame_v7000 = 265,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttribute_v8000 = 266,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttributes_v8000 = 267,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnectWithFlags_v7000 = 268,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_v9000 = 269,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_ptsz_v9000 = 270,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateFromEGLSync_v9000 = 271,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernelMultiDevice_v9000 = 272,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetAttribute_v9000 = 273,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalMemory_v10000 = 274,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedBuffer_v10000 = 275,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedMipmappedArray_v10000 = 276,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalMemory_v10000 = 277,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalSemaphore_v10000 = 278,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v10000 = 279,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_ptsz_v10000 = 280,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v10000 = 281,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_ptsz_v10000 = 282,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalSemaphore_v10000 = 283,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_v10000 = 284,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_ptsz_v10000 = 285,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphCreate_v10000 = 286,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetParams_v10000 = 287,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetParams_v10000 = 288,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddKernelNode_v10000 = 289,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode_v10000 = 290,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeGetParams_v10000 = 291,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams_v10000 = 292,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemsetNode_v10000 = 293,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeGetParams_v10000 = 294,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeSetParams_v10000 = 295,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddHostNode_v10000 = 296,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeGetParams_v10000 = 297,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddChildGraphNode_v10000 = 298,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphChildGraphNodeGetGraph_v10000 = 299,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEmptyNode_v10000 = 300,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphClone_v10000 = 301,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeFindInClone_v10000 = 302,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetType_v10000 = 303,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetRootNodes_v10000 = 304,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependencies_v10000 = 305,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependentNodes_v10000 = 306,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddDependencies_v10000 = 307,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRemoveDependencies_v10000 = 308,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroyNode_v10000 = 309,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiate_v10000 = 310,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_v10000 = 311,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_ptsz_v10000 = 312,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecDestroy_v10000 = 313,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroy_v10000 = 314,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_v10000 = 315,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_ptsz_v10000 = 316,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_v10000 = 317,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_ptsz_v10000 = 318,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_v10000 = 319,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_ptsz_v10000 = 320,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeSetParams_v10000 = 321,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetNodes_v10000 = 322,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetEdges_v10000 = 323,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v10010 = 324,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_ptsz_v10010 = 325,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecKernelNodeSetParams_v10010 = 326,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExchangeStreamCaptureMode_v10010 = 327,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetNvSciSyncAttributes_v10020 = 328,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyAvailableDynamicSMemPerBlock_v10200 = 329,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_v10200 = 330,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_ptsz_v10200 = 331,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams_v10020 = 332,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemsetNodeSetParams_v10020 = 333,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecHostNodeSetParams_v10020 = 334,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecUpdate_v10020 = 335,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetFuncBySymbol_v11000 = 336,
    CUPTI_RUNTIME_TRACE_CBID_cudaCtxResetPersistingL2Cache_v11000 = 337,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeCopyAttributes_v11000 = 338,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetAttribute_v11000 = 339,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetAttribute_v11000 = 340,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_v11000 = 341,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_ptsz_v11000 = 342,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_v11000 = 343,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_ptsz_v11000 = 344,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_v11000 = 345,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_ptsz_v11000 = 346,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetTexture1DLinearMaxWidth_v11010 = 347,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_v10000 = 348,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_ptsz_v10000 = 349,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeToSymbol_v11010 = 350,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeFromSymbol_v11010 = 351,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode1D_v11010 = 352,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsToSymbol_v11010 = 353,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsFromSymbol_v11010 = 354,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams1D_v11010 = 355,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsToSymbol_v11010 = 356,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsFromSymbol_v11010 = 357,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams1D_v11010 = 358,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetSparseProperties_v11010 = 359,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetSparseProperties_v11010 = 360,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecChildGraphNodeSetParams_v11010 = 361,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventRecordNode_v11010 = 362,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeGetEvent_v11010 = 363,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeSetEvent_v11010 = 364,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventWaitNode_v11010 = 365,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeGetEvent_v11010 = 366,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeSetEvent_v11010 = 367,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventRecordNodeSetEvent_v11010 = 368,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventWaitNodeSetEvent_v11010 = 369,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_v11010 = 370,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_ptsz_v11010 = 371,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetDefaultMemPool_v11020 = 372,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_v11020 = 373,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_ptsz_v11020 = 374,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_v11020 = 375,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_ptsz_v11020 = 376,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolTrimTo_v11020 = 377,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAttribute_v11020 = 378,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAttribute_v11020 = 379,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAccess_v11020 = 380,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetPlane_v11020 = 381,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAccess_v11020 = 382,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolCreate_v11020 = 383,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolDestroy_v11020 = 384,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetMemPool_v11020 = 385,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetMemPool_v11020 = 386,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportToShareableHandle_v11020 = 387,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportFromShareableHandle_v11020 = 388,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportPointer_v11020 = 389,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportPointer_v11020 = 390,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_v11020 = 391,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_ptsz_v11020 = 392,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v2_v11020 = 393,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v2_ptsz_v11020 = 394,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v2_v11020 = 395,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v2_ptsz_v11020 = 396,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresSignalNode_v11020 = 397,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeGetParams_v11020 = 398,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeSetParams_v11020 = 399,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresWaitNode_v11020 = 400,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeGetParams_v11020 = 401,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeSetParams_v11020 = 402,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresSignalNodeSetParams_v11020 = 403,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresWaitNodeSetParams_v11020 = 404,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceFlushGPUDirectRDMAWrites_v11030 = 405,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_v11030 = 406,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_ptsz_v11030 = 407,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDebugDotPrint_v11030 = 408,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_v11030 = 409,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_ptsz_v11030 = 410,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v11030 = 411,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_ptsz_v11030 = 412,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectCreate_v11030 = 413,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRetain_v11030 = 414,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRelease_v11030 = 415,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRetainUserObject_v11030 = 416,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphReleaseUserObject_v11030 = 417,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithFlags_v11040 = 418,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemAllocNode_v11040 = 419,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemAllocNodeGetParams_v11040 = 420,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemFreeNode_v11040 = 421,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemFreeNodeGetParams_v11040 = 422,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGraphMemTrim_v11040 = 423,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetGraphMemAttribute_v11040 = 424,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetGraphMemAttribute_v11040 = 425,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeSetEnabled_v11060 = 426,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetEnabled_v11060 = 427,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetMemoryRequirements_v11060 = 428,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetMemoryRequirements_v11060 = 429,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernelExC_v11060 = 430,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernelExC_ptsz_v11060 = 431,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxPotentialClusterSize_v11070 = 432,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveClusters_v11070 = 433,
    CUPTI_RUNTIME_TRACE_CBID_SIZE = 434,
    CUPTI_RUNTIME_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-12000"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_runtime_api_trace_cbid_enum {
    CUPTI_RUNTIME_TRACE_CBID_INVALID = 0,
    CUPTI_RUNTIME_TRACE_CBID_cudaDriverGetVersion_v3020 = 1,
    CUPTI_RUNTIME_TRACE_CBID_cudaRuntimeGetVersion_v3020 = 2,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceCount_v3020 = 3,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v3020 = 4,
    CUPTI_RUNTIME_TRACE_CBID_cudaChooseDevice_v3020 = 5,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetChannelDesc_v3020 = 6,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateChannelDesc_v3020 = 7,
    CUPTI_RUNTIME_TRACE_CBID_cudaConfigureCall_v3020 = 8,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetupArgument_v3020 = 9,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetLastError_v3020 = 10,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeekAtLastError_v3020 = 11,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorString_v3020 = 12,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_v3020 = 13,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetCacheConfig_v3020 = 14,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetAttributes_v3020 = 15,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDevice_v3020 = 16,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDevice_v3020 = 17,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetValidDevices_v3020 = 18,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDeviceFlags_v3020 = 19,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc_v3020 = 20,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocPitch_v3020 = 21,
    CUPTI_RUNTIME_TRACE_CBID_cudaFree_v3020 = 22,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocArray_v3020 = 23,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeArray_v3020 = 24,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocHost_v3020 = 25,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeHost_v3020 = 26,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostAlloc_v3020 = 27,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetDevicePointer_v3020 = 28,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetFlags_v3020 = 29,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemGetInfo_v3020 = 30,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_v3020 = 31,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_v3020 = 32,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_v3020 = 33,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_v3020 = 34,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_v3020 = 35,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_v3020 = 36,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_v3020 = 37,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_v3020 = 38,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_v3020 = 39,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_v3020 = 40,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_v3020 = 41,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_v3020 = 42,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_v3020 = 43,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_v3020 = 44,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_v3020 = 45,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_v3020 = 46,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_v3020 = 47,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_v3020 = 48,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_v3020 = 49,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_v3020 = 50,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_v3020 = 51,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_v3020 = 52,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolAddress_v3020 = 53,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolSize_v3020 = 54,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture_v3020 = 55,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture2D_v3020 = 56,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToArray_v3020 = 57,
    CUPTI_RUNTIME_TRACE_CBID_cudaUnbindTexture_v3020 = 58,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureAlignmentOffset_v3020 = 59,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureReference_v3020 = 60,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindSurfaceToArray_v3020 = 61,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceReference_v3020 = 62,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetGLDevice_v3020 = 63,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLRegisterBufferObject_v3020 = 64,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObject_v3020 = 65,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObject_v3020 = 66,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnregisterBufferObject_v3020 = 67,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetBufferObjectMapFlags_v3020 = 68,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObjectAsync_v3020 = 69,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObjectAsync_v3020 = 70,
    CUPTI_RUNTIME_TRACE_CBID_cudaWGLGetDevice_v3020 = 71,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterImage_v3020 = 72,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterBuffer_v3020 = 73,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnregisterResource_v3020 = 74,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceSetMapFlags_v3020 = 75,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsMapResources_v3020 = 76,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnmapResources_v3020 = 77,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedPointer_v3020 = 78,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsSubResourceGetMappedArray_v3020 = 79,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUGetDevice_v3020 = 80,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUSetVDPAUDevice_v3020 = 81,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterVideoSurface_v3020 = 82,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterOutputSurface_v3020 = 83,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevice_v3020 = 84,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevices_v3020 = 85,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11SetDirect3DDevice_v3020 = 86,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D11RegisterResource_v3020 = 87,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevice_v3020 = 88,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevices_v3020 = 89,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10SetDirect3DDevice_v3020 = 90,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D10RegisterResource_v3020 = 91,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10RegisterResource_v3020 = 92,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnregisterResource_v3020 = 93,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10MapResources_v3020 = 94,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnmapResources_v3020 = 95,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceSetMapFlags_v3020 = 96,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetSurfaceDimensions_v3020 = 97,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedArray_v3020 = 98,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPointer_v3020 = 99,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedSize_v3020 = 100,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPitch_v3020 = 101,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevice_v3020 = 102,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevices_v3020 = 103,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9SetDirect3DDevice_v3020 = 104,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDirect3DDevice_v3020 = 105,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D9RegisterResource_v3020 = 106,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterResource_v3020 = 107,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterResource_v3020 = 108,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapResources_v3020 = 109,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapResources_v3020 = 110,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceSetMapFlags_v3020 = 111,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetSurfaceDimensions_v3020 = 112,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedArray_v3020 = 113,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPointer_v3020 = 114,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedSize_v3020 = 115,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPitch_v3020 = 116,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9Begin_v3020 = 117,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9End_v3020 = 118,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterVertexBuffer_v3020 = 119,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterVertexBuffer_v3020 = 120,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapVertexBuffer_v3020 = 121,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapVertexBuffer_v3020 = 122,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExit_v3020 = 123,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForDevice_v3020 = 124,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForHost_v3020 = 125,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSynchronize_v3020 = 126,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetLimit_v3020 = 127,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetLimit_v3020 = 128,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreate_v3020 = 129,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v3020 = 130,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_v3020 = 131,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_v3020 = 132,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreate_v3020 = 133,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateWithFlags_v3020 = 134,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_v3020 = 135,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventDestroy_v3020 = 136,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventSynchronize_v3020 = 137,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventQuery_v3020 = 138,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventElapsedTime_v3020 = 139,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3D_v3020 = 140,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3DArray_v3020 = 141,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_v3020 = 142,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_v3020 = 143,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_v3020 = 144,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_v3020 = 145,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetCacheConfig_v3020 = 146,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_v3020 = 147,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDirect3DDevice_v3020 = 148,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDirect3DDevice_v3020 = 149,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetCacheConfig_v3020 = 150,
    CUPTI_RUNTIME_TRACE_CBID_cudaPointerGetAttributes_v4000 = 151,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostRegister_v4000 = 152,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostUnregister_v4000 = 153,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceCanAccessPeer_v4000 = 154,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceEnablePeerAccess_v4000 = 155,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceDisablePeerAccess_v4000 = 156,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerRegister_v4000 = 157,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerUnregister_v4000 = 158,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerGetDevicePointer_v4000 = 159,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeer_v4000 = 160,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeerAsync_v4000 = 161,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_v4000 = 162,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_v4000 = 163,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceReset_v3020 = 164,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSynchronize_v3020 = 165,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetLimit_v3020 = 166,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetLimit_v3020 = 167,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetCacheConfig_v3020 = 168,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetCacheConfig_v3020 = 169,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerInitialize_v4000 = 170,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStart_v4000 = 171,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStop_v4000 = 172,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetByPCIBusId_v4010 = 173,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetPCIBusId_v4010 = 174,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLGetDevices_v4010 = 175,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetEventHandle_v4010 = 176,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenEventHandle_v4010 = 177,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetMemHandle_v4010 = 178,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenMemHandle_v4010 = 179,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcCloseMemHandle_v4010 = 180,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetInfo_v4010 = 181,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetSharedMemConfig_v4020 = 182,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetSharedMemConfig_v4020 = 183,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetSharedMemConfig_v4020 = 184,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateTextureObject_v5000 = 185,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyTextureObject_v5000 = 186,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceDesc_v5000 = 187,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectTextureDesc_v5000 = 188,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateSurfaceObject_v5000 = 189,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroySurfaceObject_v5000 = 190,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceObjectResourceDesc_v5000 = 191,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocMipmappedArray_v5000 = 192,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetMipmappedArrayLevel_v5000 = 193,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeMipmappedArray_v5000 = 194,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToMipmappedArray_v5000 = 195,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedMipmappedArray_v5000 = 196,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_v5000 = 197,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithFlags_v5000 = 198,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceViewDesc_v5000 = 199,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetAttribute_v5000 = 200,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v5050 = 201,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithPriority_v5050 = 202,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_v5050 = 203,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_v5050 = 204,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetStreamPriorityRange_v5050 = 205,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocManaged_v6000 = 206,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6000 = 207,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_v6000 = 208,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorName_v6050 = 209,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6050 = 210,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_v7000 = 211,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceFlags_v7000 = 212,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_ptsz_v7000 = 213,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_ptsz_v7000 = 214,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_ptds_v7000 = 215,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_ptds_v7000 = 216,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_ptds_v7000 = 217,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_ptds_v7000 = 218,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_ptds_v7000 = 219,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_ptds_v7000 = 220,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_ptds_v7000 = 221,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_ptds_v7000 = 222,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_ptds_v7000 = 223,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_ptds_v7000 = 224,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_ptsz_v7000 = 225,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_ptsz_v7000 = 226,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_ptsz_v7000 = 227,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_ptsz_v7000 = 228,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_ptsz_v7000 = 229,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_ptsz_v7000 = 230,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_ptsz_v7000 = 231,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_ptsz_v7000 = 232,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_ptds_v7000 = 233,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_ptds_v7000 = 234,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_ptsz_v7000 = 235,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_ptsz_v7000 = 236,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_ptsz_v7000 = 237,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_ptsz_v7000 = 238,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_ptsz_v7000 = 239,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_ptsz_v7000 = 240,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_ptsz_v7000 = 241,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_ptsz_v7000 = 242,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_ptds_v7000 = 243,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_ptsz_v7000 = 244,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_ptds_v7000 = 245,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_ptsz_v7000 = 246,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_ptsz_v7000 = 247,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_ptsz_v7000 = 248,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_ptds_v7000 = 249,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_ptsz_v7000 = 250,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessorWithFlags_v7000 = 251,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v8000 = 252,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_ptsz_v8000 = 253,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemAdvise_v8000 = 254,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetP2PAttribute_v8000 = 255,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsEGLRegisterImage_v7000 = 256,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnect_v7000 = 257,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerDisconnect_v7000 = 258,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerAcquireFrame_v7000 = 259,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerReleaseFrame_v7000 = 260,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerConnect_v7000 = 261,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerDisconnect_v7000 = 262,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerPresentFrame_v7000 = 263,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerReturnFrame_v7000 = 264,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedEglFrame_v7000 = 265,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttribute_v8000 = 266,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttributes_v8000 = 267,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnectWithFlags_v7000 = 268,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_v9000 = 269,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_ptsz_v9000 = 270,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateFromEGLSync_v9000 = 271,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernelMultiDevice_v9000 = 272,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetAttribute_v9000 = 273,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalMemory_v10000 = 274,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedBuffer_v10000 = 275,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedMipmappedArray_v10000 = 276,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalMemory_v10000 = 277,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalSemaphore_v10000 = 278,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v10000 = 279,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_ptsz_v10000 = 280,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v10000 = 281,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_ptsz_v10000 = 282,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalSemaphore_v10000 = 283,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_v10000 = 284,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_ptsz_v10000 = 285,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphCreate_v10000 = 286,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetParams_v10000 = 287,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetParams_v10000 = 288,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddKernelNode_v10000 = 289,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode_v10000 = 290,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeGetParams_v10000 = 291,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams_v10000 = 292,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemsetNode_v10000 = 293,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeGetParams_v10000 = 294,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeSetParams_v10000 = 295,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddHostNode_v10000 = 296,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeGetParams_v10000 = 297,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddChildGraphNode_v10000 = 298,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphChildGraphNodeGetGraph_v10000 = 299,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEmptyNode_v10000 = 300,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphClone_v10000 = 301,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeFindInClone_v10000 = 302,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetType_v10000 = 303,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetRootNodes_v10000 = 304,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependencies_v10000 = 305,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependentNodes_v10000 = 306,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddDependencies_v10000 = 307,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRemoveDependencies_v10000 = 308,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroyNode_v10000 = 309,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiate_v10000 = 310,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_v10000 = 311,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_ptsz_v10000 = 312,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecDestroy_v10000 = 313,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroy_v10000 = 314,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_v10000 = 315,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_ptsz_v10000 = 316,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_v10000 = 317,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_ptsz_v10000 = 318,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_v10000 = 319,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_ptsz_v10000 = 320,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeSetParams_v10000 = 321,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetNodes_v10000 = 322,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetEdges_v10000 = 323,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v10010 = 324,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_ptsz_v10010 = 325,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecKernelNodeSetParams_v10010 = 326,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExchangeStreamCaptureMode_v10010 = 327,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetNvSciSyncAttributes_v10020 = 328,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyAvailableDynamicSMemPerBlock_v10200 = 329,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_v10200 = 330,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_ptsz_v10200 = 331,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams_v10020 = 332,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemsetNodeSetParams_v10020 = 333,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecHostNodeSetParams_v10020 = 334,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecUpdate_v10020 = 335,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetFuncBySymbol_v11000 = 336,
    CUPTI_RUNTIME_TRACE_CBID_cudaCtxResetPersistingL2Cache_v11000 = 337,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeCopyAttributes_v11000 = 338,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetAttribute_v11000 = 339,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetAttribute_v11000 = 340,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_v11000 = 341,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_ptsz_v11000 = 342,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_v11000 = 343,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_ptsz_v11000 = 344,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_v11000 = 345,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_ptsz_v11000 = 346,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetTexture1DLinearMaxWidth_v11010 = 347,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_v10000 = 348,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_ptsz_v10000 = 349,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeToSymbol_v11010 = 350,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeFromSymbol_v11010 = 351,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode1D_v11010 = 352,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsToSymbol_v11010 = 353,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsFromSymbol_v11010 = 354,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams1D_v11010 = 355,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsToSymbol_v11010 = 356,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsFromSymbol_v11010 = 357,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams1D_v11010 = 358,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetSparseProperties_v11010 = 359,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetSparseProperties_v11010 = 360,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecChildGraphNodeSetParams_v11010 = 361,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventRecordNode_v11010 = 362,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeGetEvent_v11010 = 363,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeSetEvent_v11010 = 364,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventWaitNode_v11010 = 365,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeGetEvent_v11010 = 366,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeSetEvent_v11010 = 367,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventRecordNodeSetEvent_v11010 = 368,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventWaitNodeSetEvent_v11010 = 369,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_v11010 = 370,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_ptsz_v11010 = 371,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetDefaultMemPool_v11020 = 372,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_v11020 = 373,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_ptsz_v11020 = 374,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_v11020 = 375,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_ptsz_v11020 = 376,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolTrimTo_v11020 = 377,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAttribute_v11020 = 378,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAttribute_v11020 = 379,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAccess_v11020 = 380,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetPlane_v11020 = 381,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAccess_v11020 = 382,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolCreate_v11020 = 383,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolDestroy_v11020 = 384,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetMemPool_v11020 = 385,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetMemPool_v11020 = 386,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportToShareableHandle_v11020 = 387,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportFromShareableHandle_v11020 = 388,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportPointer_v11020 = 389,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportPointer_v11020 = 390,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_v11020 = 391,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_ptsz_v11020 = 392,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v2_v11020 = 393,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v2_ptsz_v11020 = 394,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v2_v11020 = 395,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v2_ptsz_v11020 = 396,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresSignalNode_v11020 = 397,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeGetParams_v11020 = 398,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeSetParams_v11020 = 399,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresWaitNode_v11020 = 400,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeGetParams_v11020 = 401,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeSetParams_v11020 = 402,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresSignalNodeSetParams_v11020 = 403,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresWaitNodeSetParams_v11020 = 404,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceFlushGPUDirectRDMAWrites_v11030 = 405,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_v11030 = 406,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_ptsz_v11030 = 407,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDebugDotPrint_v11030 = 408,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_v11030 = 409,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_ptsz_v11030 = 410,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v11030 = 411,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_ptsz_v11030 = 412,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectCreate_v11030 = 413,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRetain_v11030 = 414,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRelease_v11030 = 415,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRetainUserObject_v11030 = 416,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphReleaseUserObject_v11030 = 417,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithFlags_v11040 = 418,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemAllocNode_v11040 = 419,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemAllocNodeGetParams_v11040 = 420,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemFreeNode_v11040 = 421,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemFreeNodeGetParams_v11040 = 422,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGraphMemTrim_v11040 = 423,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetGraphMemAttribute_v11040 = 424,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetGraphMemAttribute_v11040 = 425,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeSetEnabled_v11060 = 426,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetEnabled_v11060 = 427,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetMemoryRequirements_v11060 = 428,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetMemoryRequirements_v11060 = 429,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernelExC_v11060 = 430,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernelExC_ptsz_v11060 = 431,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxPotentialClusterSize_v11070 = 432,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveClusters_v11070 = 433,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateTextureObject_v2_v11080 = 434,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectTextureDesc_v2_v11080 = 435,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithParams_v12000 = 436,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithParams_ptsz_v12000 = 437,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecGetFlags_v12000 = 438,
    CUPTI_RUNTIME_TRACE_CBID_cuda439_v12000 = 439,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v2_v12000 = 440,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetId_v12000 = 441,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetId_ptsz_v12000 = 442,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiate_v12000 = 443,
    CUPTI_RUNTIME_TRACE_CBID_cuda444_v12000 = 444,
    CUPTI_RUNTIME_TRACE_CBID_SIZE = 445,
    CUPTI_RUNTIME_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-12010"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_runtime_api_trace_cbid_enum {
    CUPTI_RUNTIME_TRACE_CBID_INVALID = 0,
    CUPTI_RUNTIME_TRACE_CBID_cudaDriverGetVersion_v3020 = 1,
    CUPTI_RUNTIME_TRACE_CBID_cudaRuntimeGetVersion_v3020 = 2,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceCount_v3020 = 3,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v3020 = 4,
    CUPTI_RUNTIME_TRACE_CBID_cudaChooseDevice_v3020 = 5,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetChannelDesc_v3020 = 6,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateChannelDesc_v3020 = 7,
    CUPTI_RUNTIME_TRACE_CBID_cudaConfigureCall_v3020 = 8,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetupArgument_v3020 = 9,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetLastError_v3020 = 10,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeekAtLastError_v3020 = 11,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorString_v3020 = 12,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_v3020 = 13,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetCacheConfig_v3020 = 14,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetAttributes_v3020 = 15,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDevice_v3020 = 16,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDevice_v3020 = 17,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetValidDevices_v3020 = 18,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDeviceFlags_v3020 = 19,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc_v3020 = 20,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocPitch_v3020 = 21,
    CUPTI_RUNTIME_TRACE_CBID_cudaFree_v3020 = 22,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocArray_v3020 = 23,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeArray_v3020 = 24,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocHost_v3020 = 25,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeHost_v3020 = 26,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostAlloc_v3020 = 27,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetDevicePointer_v3020 = 28,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetFlags_v3020 = 29,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemGetInfo_v3020 = 30,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_v3020 = 31,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_v3020 = 32,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_v3020 = 33,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_v3020 = 34,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_v3020 = 35,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_v3020 = 36,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_v3020 = 37,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_v3020 = 38,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_v3020 = 39,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_v3020 = 40,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_v3020 = 41,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_v3020 = 42,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_v3020 = 43,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_v3020 = 44,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_v3020 = 45,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_v3020 = 46,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_v3020 = 47,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_v3020 = 48,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_v3020 = 49,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_v3020 = 50,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_v3020 = 51,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_v3020 = 52,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolAddress_v3020 = 53,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolSize_v3020 = 54,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture_v3020 = 55,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture2D_v3020 = 56,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToArray_v3020 = 57,
    CUPTI_RUNTIME_TRACE_CBID_cudaUnbindTexture_v3020 = 58,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureAlignmentOffset_v3020 = 59,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureReference_v3020 = 60,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindSurfaceToArray_v3020 = 61,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceReference_v3020 = 62,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetGLDevice_v3020 = 63,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLRegisterBufferObject_v3020 = 64,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObject_v3020 = 65,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObject_v3020 = 66,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnregisterBufferObject_v3020 = 67,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetBufferObjectMapFlags_v3020 = 68,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObjectAsync_v3020 = 69,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObjectAsync_v3020 = 70,
    CUPTI_RUNTIME_TRACE_CBID_cudaWGLGetDevice_v3020 = 71,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterImage_v3020 = 72,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterBuffer_v3020 = 73,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnregisterResource_v3020 = 74,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceSetMapFlags_v3020 = 75,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsMapResources_v3020 = 76,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnmapResources_v3020 = 77,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedPointer_v3020 = 78,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsSubResourceGetMappedArray_v3020 = 79,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUGetDevice_v3020 = 80,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUSetVDPAUDevice_v3020 = 81,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterVideoSurface_v3020 = 82,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterOutputSurface_v3020 = 83,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevice_v3020 = 84,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevices_v3020 = 85,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11SetDirect3DDevice_v3020 = 86,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D11RegisterResource_v3020 = 87,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevice_v3020 = 88,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevices_v3020 = 89,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10SetDirect3DDevice_v3020 = 90,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D10RegisterResource_v3020 = 91,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10RegisterResource_v3020 = 92,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnregisterResource_v3020 = 93,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10MapResources_v3020 = 94,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnmapResources_v3020 = 95,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceSetMapFlags_v3020 = 96,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetSurfaceDimensions_v3020 = 97,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedArray_v3020 = 98,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPointer_v3020 = 99,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedSize_v3020 = 100,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPitch_v3020 = 101,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevice_v3020 = 102,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevices_v3020 = 103,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9SetDirect3DDevice_v3020 = 104,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDirect3DDevice_v3020 = 105,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D9RegisterResource_v3020 = 106,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterResource_v3020 = 107,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterResource_v3020 = 108,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapResources_v3020 = 109,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapResources_v3020 = 110,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceSetMapFlags_v3020 = 111,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetSurfaceDimensions_v3020 = 112,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedArray_v3020 = 113,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPointer_v3020 = 114,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedSize_v3020 = 115,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPitch_v3020 = 116,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9Begin_v3020 = 117,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9End_v3020 = 118,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterVertexBuffer_v3020 = 119,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterVertexBuffer_v3020 = 120,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapVertexBuffer_v3020 = 121,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapVertexBuffer_v3020 = 122,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExit_v3020 = 123,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForDevice_v3020 = 124,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForHost_v3020 = 125,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSynchronize_v3020 = 126,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetLimit_v3020 = 127,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetLimit_v3020 = 128,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreate_v3020 = 129,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v3020 = 130,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_v3020 = 131,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_v3020 = 132,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreate_v3020 = 133,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateWithFlags_v3020 = 134,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_v3020 = 135,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventDestroy_v3020 = 136,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventSynchronize_v3020 = 137,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventQuery_v3020 = 138,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventElapsedTime_v3020 = 139,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3D_v3020 = 140,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3DArray_v3020 = 141,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_v3020 = 142,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_v3020 = 143,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_v3020 = 144,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_v3020 = 145,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetCacheConfig_v3020 = 146,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_v3020 = 147,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDirect3DDevice_v3020 = 148,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDirect3DDevice_v3020 = 149,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetCacheConfig_v3020 = 150,
    CUPTI_RUNTIME_TRACE_CBID_cudaPointerGetAttributes_v4000 = 151,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostRegister_v4000 = 152,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostUnregister_v4000 = 153,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceCanAccessPeer_v4000 = 154,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceEnablePeerAccess_v4000 = 155,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceDisablePeerAccess_v4000 = 156,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerRegister_v4000 = 157,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerUnregister_v4000 = 158,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerGetDevicePointer_v4000 = 159,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeer_v4000 = 160,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeerAsync_v4000 = 161,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_v4000 = 162,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_v4000 = 163,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceReset_v3020 = 164,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSynchronize_v3020 = 165,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetLimit_v3020 = 166,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetLimit_v3020 = 167,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetCacheConfig_v3020 = 168,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetCacheConfig_v3020 = 169,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerInitialize_v4000 = 170,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStart_v4000 = 171,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStop_v4000 = 172,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetByPCIBusId_v4010 = 173,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetPCIBusId_v4010 = 174,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLGetDevices_v4010 = 175,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetEventHandle_v4010 = 176,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenEventHandle_v4010 = 177,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetMemHandle_v4010 = 178,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenMemHandle_v4010 = 179,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcCloseMemHandle_v4010 = 180,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetInfo_v4010 = 181,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetSharedMemConfig_v4020 = 182,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetSharedMemConfig_v4020 = 183,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetSharedMemConfig_v4020 = 184,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateTextureObject_v5000 = 185,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyTextureObject_v5000 = 186,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceDesc_v5000 = 187,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectTextureDesc_v5000 = 188,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateSurfaceObject_v5000 = 189,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroySurfaceObject_v5000 = 190,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceObjectResourceDesc_v5000 = 191,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocMipmappedArray_v5000 = 192,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetMipmappedArrayLevel_v5000 = 193,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeMipmappedArray_v5000 = 194,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToMipmappedArray_v5000 = 195,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedMipmappedArray_v5000 = 196,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_v5000 = 197,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithFlags_v5000 = 198,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceViewDesc_v5000 = 199,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetAttribute_v5000 = 200,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v5050 = 201,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithPriority_v5050 = 202,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_v5050 = 203,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_v5050 = 204,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetStreamPriorityRange_v5050 = 205,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocManaged_v6000 = 206,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6000 = 207,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_v6000 = 208,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorName_v6050 = 209,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6050 = 210,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_v7000 = 211,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceFlags_v7000 = 212,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_ptsz_v7000 = 213,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_ptsz_v7000 = 214,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_ptds_v7000 = 215,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_ptds_v7000 = 216,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_ptds_v7000 = 217,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_ptds_v7000 = 218,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_ptds_v7000 = 219,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_ptds_v7000 = 220,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_ptds_v7000 = 221,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_ptds_v7000 = 222,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_ptds_v7000 = 223,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_ptds_v7000 = 224,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_ptsz_v7000 = 225,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_ptsz_v7000 = 226,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_ptsz_v7000 = 227,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_ptsz_v7000 = 228,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_ptsz_v7000 = 229,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_ptsz_v7000 = 230,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_ptsz_v7000 = 231,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_ptsz_v7000 = 232,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_ptds_v7000 = 233,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_ptds_v7000 = 234,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_ptsz_v7000 = 235,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_ptsz_v7000 = 236,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_ptsz_v7000 = 237,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_ptsz_v7000 = 238,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_ptsz_v7000 = 239,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_ptsz_v7000 = 240,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_ptsz_v7000 = 241,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_ptsz_v7000 = 242,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_ptds_v7000 = 243,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_ptsz_v7000 = 244,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_ptds_v7000 = 245,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_ptsz_v7000 = 246,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_ptsz_v7000 = 247,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_ptsz_v7000 = 248,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_ptds_v7000 = 249,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_ptsz_v7000 = 250,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessorWithFlags_v7000 = 251,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v8000 = 252,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_ptsz_v8000 = 253,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemAdvise_v8000 = 254,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetP2PAttribute_v8000 = 255,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsEGLRegisterImage_v7000 = 256,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnect_v7000 = 257,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerDisconnect_v7000 = 258,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerAcquireFrame_v7000 = 259,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerReleaseFrame_v7000 = 260,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerConnect_v7000 = 261,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerDisconnect_v7000 = 262,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerPresentFrame_v7000 = 263,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerReturnFrame_v7000 = 264,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedEglFrame_v7000 = 265,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttribute_v8000 = 266,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttributes_v8000 = 267,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnectWithFlags_v7000 = 268,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_v9000 = 269,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_ptsz_v9000 = 270,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateFromEGLSync_v9000 = 271,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernelMultiDevice_v9000 = 272,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetAttribute_v9000 = 273,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalMemory_v10000 = 274,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedBuffer_v10000 = 275,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedMipmappedArray_v10000 = 276,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalMemory_v10000 = 277,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalSemaphore_v10000 = 278,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v10000 = 279,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_ptsz_v10000 = 280,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v10000 = 281,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_ptsz_v10000 = 282,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalSemaphore_v10000 = 283,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_v10000 = 284,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_ptsz_v10000 = 285,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphCreate_v10000 = 286,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetParams_v10000 = 287,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetParams_v10000 = 288,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddKernelNode_v10000 = 289,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode_v10000 = 290,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeGetParams_v10000 = 291,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams_v10000 = 292,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemsetNode_v10000 = 293,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeGetParams_v10000 = 294,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeSetParams_v10000 = 295,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddHostNode_v10000 = 296,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeGetParams_v10000 = 297,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddChildGraphNode_v10000 = 298,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphChildGraphNodeGetGraph_v10000 = 299,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEmptyNode_v10000 = 300,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphClone_v10000 = 301,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeFindInClone_v10000 = 302,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetType_v10000 = 303,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetRootNodes_v10000 = 304,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependencies_v10000 = 305,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependentNodes_v10000 = 306,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddDependencies_v10000 = 307,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRemoveDependencies_v10000 = 308,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroyNode_v10000 = 309,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiate_v10000 = 310,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_v10000 = 311,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_ptsz_v10000 = 312,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecDestroy_v10000 = 313,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroy_v10000 = 314,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_v10000 = 315,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_ptsz_v10000 = 316,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_v10000 = 317,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_ptsz_v10000 = 318,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_v10000 = 319,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_ptsz_v10000 = 320,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeSetParams_v10000 = 321,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetNodes_v10000 = 322,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetEdges_v10000 = 323,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v10010 = 324,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_ptsz_v10010 = 325,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecKernelNodeSetParams_v10010 = 326,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExchangeStreamCaptureMode_v10010 = 327,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetNvSciSyncAttributes_v10020 = 328,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyAvailableDynamicSMemPerBlock_v10200 = 329,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_v10200 = 330,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_ptsz_v10200 = 331,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams_v10020 = 332,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemsetNodeSetParams_v10020 = 333,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecHostNodeSetParams_v10020 = 334,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecUpdate_v10020 = 335,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetFuncBySymbol_v11000 = 336,
    CUPTI_RUNTIME_TRACE_CBID_cudaCtxResetPersistingL2Cache_v11000 = 337,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeCopyAttributes_v11000 = 338,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetAttribute_v11000 = 339,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetAttribute_v11000 = 340,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_v11000 = 341,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_ptsz_v11000 = 342,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_v11000 = 343,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_ptsz_v11000 = 344,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_v11000 = 345,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_ptsz_v11000 = 346,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetTexture1DLinearMaxWidth_v11010 = 347,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_v10000 = 348,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_ptsz_v10000 = 349,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeToSymbol_v11010 = 350,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeFromSymbol_v11010 = 351,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode1D_v11010 = 352,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsToSymbol_v11010 = 353,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsFromSymbol_v11010 = 354,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams1D_v11010 = 355,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsToSymbol_v11010 = 356,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsFromSymbol_v11010 = 357,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams1D_v11010 = 358,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetSparseProperties_v11010 = 359,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetSparseProperties_v11010 = 360,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecChildGraphNodeSetParams_v11010 = 361,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventRecordNode_v11010 = 362,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeGetEvent_v11010 = 363,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeSetEvent_v11010 = 364,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventWaitNode_v11010 = 365,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeGetEvent_v11010 = 366,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeSetEvent_v11010 = 367,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventRecordNodeSetEvent_v11010 = 368,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventWaitNodeSetEvent_v11010 = 369,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_v11010 = 370,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_ptsz_v11010 = 371,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetDefaultMemPool_v11020 = 372,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_v11020 = 373,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_ptsz_v11020 = 374,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_v11020 = 375,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_ptsz_v11020 = 376,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolTrimTo_v11020 = 377,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAttribute_v11020 = 378,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAttribute_v11020 = 379,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAccess_v11020 = 380,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetPlane_v11020 = 381,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAccess_v11020 = 382,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolCreate_v11020 = 383,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolDestroy_v11020 = 384,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetMemPool_v11020 = 385,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetMemPool_v11020 = 386,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportToShareableHandle_v11020 = 387,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportFromShareableHandle_v11020 = 388,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportPointer_v11020 = 389,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportPointer_v11020 = 390,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_v11020 = 391,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_ptsz_v11020 = 392,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v2_v11020 = 393,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v2_ptsz_v11020 = 394,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v2_v11020 = 395,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v2_ptsz_v11020 = 396,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresSignalNode_v11020 = 397,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeGetParams_v11020 = 398,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeSetParams_v11020 = 399,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresWaitNode_v11020 = 400,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeGetParams_v11020 = 401,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeSetParams_v11020 = 402,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresSignalNodeSetParams_v11020 = 403,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresWaitNodeSetParams_v11020 = 404,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceFlushGPUDirectRDMAWrites_v11030 = 405,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_v11030 = 406,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_ptsz_v11030 = 407,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDebugDotPrint_v11030 = 408,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_v11030 = 409,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_ptsz_v11030 = 410,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v11030 = 411,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_ptsz_v11030 = 412,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectCreate_v11030 = 413,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRetain_v11030 = 414,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRelease_v11030 = 415,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRetainUserObject_v11030 = 416,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphReleaseUserObject_v11030 = 417,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithFlags_v11040 = 418,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemAllocNode_v11040 = 419,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemAllocNodeGetParams_v11040 = 420,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemFreeNode_v11040 = 421,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemFreeNodeGetParams_v11040 = 422,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGraphMemTrim_v11040 = 423,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetGraphMemAttribute_v11040 = 424,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetGraphMemAttribute_v11040 = 425,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeSetEnabled_v11060 = 426,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetEnabled_v11060 = 427,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetMemoryRequirements_v11060 = 428,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetMemoryRequirements_v11060 = 429,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernelExC_v11060 = 430,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernelExC_ptsz_v11060 = 431,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxPotentialClusterSize_v11070 = 432,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveClusters_v11070 = 433,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateTextureObject_v2_v11080 = 434,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectTextureDesc_v2_v11080 = 435,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithParams_v12000 = 436,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithParams_ptsz_v12000 = 437,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecGetFlags_v12000 = 438,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetKernel_v12000 = 439,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v2_v12000 = 440,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetId_v12000 = 441,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetId_ptsz_v12000 = 442,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiate_v12000 = 443,
    CUPTI_RUNTIME_TRACE_CBID_cudaInitDevice_v12000 = 444,
    CUPTI_RUNTIME_TRACE_CBID_SIZE = 445,
    CUPTI_RUNTIME_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-12020"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_runtime_api_trace_cbid_enum {
    CUPTI_RUNTIME_TRACE_CBID_INVALID = 0,
    CUPTI_RUNTIME_TRACE_CBID_cudaDriverGetVersion_v3020 = 1,
    CUPTI_RUNTIME_TRACE_CBID_cudaRuntimeGetVersion_v3020 = 2,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceCount_v3020 = 3,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v3020 = 4,
    CUPTI_RUNTIME_TRACE_CBID_cudaChooseDevice_v3020 = 5,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetChannelDesc_v3020 = 6,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateChannelDesc_v3020 = 7,
    CUPTI_RUNTIME_TRACE_CBID_cudaConfigureCall_v3020 = 8,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetupArgument_v3020 = 9,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetLastError_v3020 = 10,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeekAtLastError_v3020 = 11,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorString_v3020 = 12,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_v3020 = 13,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetCacheConfig_v3020 = 14,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetAttributes_v3020 = 15,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDevice_v3020 = 16,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDevice_v3020 = 17,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetValidDevices_v3020 = 18,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDeviceFlags_v3020 = 19,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc_v3020 = 20,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocPitch_v3020 = 21,
    CUPTI_RUNTIME_TRACE_CBID_cudaFree_v3020 = 22,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocArray_v3020 = 23,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeArray_v3020 = 24,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocHost_v3020 = 25,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeHost_v3020 = 26,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostAlloc_v3020 = 27,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetDevicePointer_v3020 = 28,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetFlags_v3020 = 29,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemGetInfo_v3020 = 30,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_v3020 = 31,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_v3020 = 32,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_v3020 = 33,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_v3020 = 34,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_v3020 = 35,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_v3020 = 36,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_v3020 = 37,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_v3020 = 38,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_v3020 = 39,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_v3020 = 40,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_v3020 = 41,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_v3020 = 42,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_v3020 = 43,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_v3020 = 44,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_v3020 = 45,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_v3020 = 46,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_v3020 = 47,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_v3020 = 48,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_v3020 = 49,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_v3020 = 50,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_v3020 = 51,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_v3020 = 52,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolAddress_v3020 = 53,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolSize_v3020 = 54,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture_v3020 = 55,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture2D_v3020 = 56,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToArray_v3020 = 57,
    CUPTI_RUNTIME_TRACE_CBID_cudaUnbindTexture_v3020 = 58,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureAlignmentOffset_v3020 = 59,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureReference_v3020 = 60,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindSurfaceToArray_v3020 = 61,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceReference_v3020 = 62,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetGLDevice_v3020 = 63,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLRegisterBufferObject_v3020 = 64,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObject_v3020 = 65,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObject_v3020 = 66,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnregisterBufferObject_v3020 = 67,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetBufferObjectMapFlags_v3020 = 68,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObjectAsync_v3020 = 69,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObjectAsync_v3020 = 70,
    CUPTI_RUNTIME_TRACE_CBID_cudaWGLGetDevice_v3020 = 71,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterImage_v3020 = 72,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterBuffer_v3020 = 73,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnregisterResource_v3020 = 74,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceSetMapFlags_v3020 = 75,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsMapResources_v3020 = 76,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnmapResources_v3020 = 77,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedPointer_v3020 = 78,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsSubResourceGetMappedArray_v3020 = 79,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUGetDevice_v3020 = 80,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUSetVDPAUDevice_v3020 = 81,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterVideoSurface_v3020 = 82,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterOutputSurface_v3020 = 83,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevice_v3020 = 84,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevices_v3020 = 85,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11SetDirect3DDevice_v3020 = 86,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D11RegisterResource_v3020 = 87,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevice_v3020 = 88,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevices_v3020 = 89,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10SetDirect3DDevice_v3020 = 90,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D10RegisterResource_v3020 = 91,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10RegisterResource_v3020 = 92,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnregisterResource_v3020 = 93,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10MapResources_v3020 = 94,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnmapResources_v3020 = 95,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceSetMapFlags_v3020 = 96,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetSurfaceDimensions_v3020 = 97,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedArray_v3020 = 98,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPointer_v3020 = 99,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedSize_v3020 = 100,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPitch_v3020 = 101,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevice_v3020 = 102,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevices_v3020 = 103,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9SetDirect3DDevice_v3020 = 104,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDirect3DDevice_v3020 = 105,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D9RegisterResource_v3020 = 106,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterResource_v3020 = 107,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterResource_v3020 = 108,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapResources_v3020 = 109,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapResources_v3020 = 110,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceSetMapFlags_v3020 = 111,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetSurfaceDimensions_v3020 = 112,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedArray_v3020 = 113,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPointer_v3020 = 114,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedSize_v3020 = 115,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPitch_v3020 = 116,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9Begin_v3020 = 117,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9End_v3020 = 118,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterVertexBuffer_v3020 = 119,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterVertexBuffer_v3020 = 120,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapVertexBuffer_v3020 = 121,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapVertexBuffer_v3020 = 122,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExit_v3020 = 123,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForDevice_v3020 = 124,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForHost_v3020 = 125,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSynchronize_v3020 = 126,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetLimit_v3020 = 127,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetLimit_v3020 = 128,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreate_v3020 = 129,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v3020 = 130,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_v3020 = 131,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_v3020 = 132,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreate_v3020 = 133,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateWithFlags_v3020 = 134,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_v3020 = 135,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventDestroy_v3020 = 136,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventSynchronize_v3020 = 137,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventQuery_v3020 = 138,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventElapsedTime_v3020 = 139,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3D_v3020 = 140,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3DArray_v3020 = 141,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_v3020 = 142,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_v3020 = 143,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_v3020 = 144,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_v3020 = 145,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetCacheConfig_v3020 = 146,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_v3020 = 147,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDirect3DDevice_v3020 = 148,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDirect3DDevice_v3020 = 149,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetCacheConfig_v3020 = 150,
    CUPTI_RUNTIME_TRACE_CBID_cudaPointerGetAttributes_v4000 = 151,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostRegister_v4000 = 152,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostUnregister_v4000 = 153,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceCanAccessPeer_v4000 = 154,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceEnablePeerAccess_v4000 = 155,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceDisablePeerAccess_v4000 = 156,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerRegister_v4000 = 157,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerUnregister_v4000 = 158,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerGetDevicePointer_v4000 = 159,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeer_v4000 = 160,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeerAsync_v4000 = 161,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_v4000 = 162,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_v4000 = 163,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceReset_v3020 = 164,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSynchronize_v3020 = 165,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetLimit_v3020 = 166,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetLimit_v3020 = 167,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetCacheConfig_v3020 = 168,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetCacheConfig_v3020 = 169,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerInitialize_v4000 = 170,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStart_v4000 = 171,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStop_v4000 = 172,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetByPCIBusId_v4010 = 173,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetPCIBusId_v4010 = 174,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLGetDevices_v4010 = 175,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetEventHandle_v4010 = 176,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenEventHandle_v4010 = 177,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetMemHandle_v4010 = 178,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenMemHandle_v4010 = 179,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcCloseMemHandle_v4010 = 180,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetInfo_v4010 = 181,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetSharedMemConfig_v4020 = 182,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetSharedMemConfig_v4020 = 183,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetSharedMemConfig_v4020 = 184,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateTextureObject_v5000 = 185,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyTextureObject_v5000 = 186,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceDesc_v5000 = 187,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectTextureDesc_v5000 = 188,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateSurfaceObject_v5000 = 189,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroySurfaceObject_v5000 = 190,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceObjectResourceDesc_v5000 = 191,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocMipmappedArray_v5000 = 192,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetMipmappedArrayLevel_v5000 = 193,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeMipmappedArray_v5000 = 194,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToMipmappedArray_v5000 = 195,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedMipmappedArray_v5000 = 196,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_v5000 = 197,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithFlags_v5000 = 198,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceViewDesc_v5000 = 199,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetAttribute_v5000 = 200,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v5050 = 201,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithPriority_v5050 = 202,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_v5050 = 203,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_v5050 = 204,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetStreamPriorityRange_v5050 = 205,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocManaged_v6000 = 206,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6000 = 207,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_v6000 = 208,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorName_v6050 = 209,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6050 = 210,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_v7000 = 211,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceFlags_v7000 = 212,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_ptsz_v7000 = 213,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_ptsz_v7000 = 214,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_ptds_v7000 = 215,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_ptds_v7000 = 216,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_ptds_v7000 = 217,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_ptds_v7000 = 218,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_ptds_v7000 = 219,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_ptds_v7000 = 220,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_ptds_v7000 = 221,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_ptds_v7000 = 222,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_ptds_v7000 = 223,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_ptds_v7000 = 224,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_ptsz_v7000 = 225,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_ptsz_v7000 = 226,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_ptsz_v7000 = 227,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_ptsz_v7000 = 228,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_ptsz_v7000 = 229,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_ptsz_v7000 = 230,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_ptsz_v7000 = 231,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_ptsz_v7000 = 232,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_ptds_v7000 = 233,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_ptds_v7000 = 234,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_ptsz_v7000 = 235,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_ptsz_v7000 = 236,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_ptsz_v7000 = 237,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_ptsz_v7000 = 238,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_ptsz_v7000 = 239,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_ptsz_v7000 = 240,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_ptsz_v7000 = 241,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_ptsz_v7000 = 242,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_ptds_v7000 = 243,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_ptsz_v7000 = 244,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_ptds_v7000 = 245,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_ptsz_v7000 = 246,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_ptsz_v7000 = 247,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_ptsz_v7000 = 248,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_ptds_v7000 = 249,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_ptsz_v7000 = 250,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessorWithFlags_v7000 = 251,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v8000 = 252,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_ptsz_v8000 = 253,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemAdvise_v8000 = 254,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetP2PAttribute_v8000 = 255,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsEGLRegisterImage_v7000 = 256,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnect_v7000 = 257,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerDisconnect_v7000 = 258,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerAcquireFrame_v7000 = 259,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerReleaseFrame_v7000 = 260,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerConnect_v7000 = 261,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerDisconnect_v7000 = 262,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerPresentFrame_v7000 = 263,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerReturnFrame_v7000 = 264,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedEglFrame_v7000 = 265,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttribute_v8000 = 266,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttributes_v8000 = 267,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnectWithFlags_v7000 = 268,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_v9000 = 269,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_ptsz_v9000 = 270,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateFromEGLSync_v9000 = 271,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernelMultiDevice_v9000 = 272,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetAttribute_v9000 = 273,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalMemory_v10000 = 274,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedBuffer_v10000 = 275,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedMipmappedArray_v10000 = 276,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalMemory_v10000 = 277,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalSemaphore_v10000 = 278,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v10000 = 279,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_ptsz_v10000 = 280,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v10000 = 281,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_ptsz_v10000 = 282,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalSemaphore_v10000 = 283,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_v10000 = 284,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_ptsz_v10000 = 285,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphCreate_v10000 = 286,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetParams_v10000 = 287,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetParams_v10000 = 288,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddKernelNode_v10000 = 289,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode_v10000 = 290,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeGetParams_v10000 = 291,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams_v10000 = 292,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemsetNode_v10000 = 293,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeGetParams_v10000 = 294,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeSetParams_v10000 = 295,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddHostNode_v10000 = 296,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeGetParams_v10000 = 297,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddChildGraphNode_v10000 = 298,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphChildGraphNodeGetGraph_v10000 = 299,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEmptyNode_v10000 = 300,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphClone_v10000 = 301,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeFindInClone_v10000 = 302,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetType_v10000 = 303,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetRootNodes_v10000 = 304,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependencies_v10000 = 305,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependentNodes_v10000 = 306,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddDependencies_v10000 = 307,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRemoveDependencies_v10000 = 308,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroyNode_v10000 = 309,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiate_v10000 = 310,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_v10000 = 311,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_ptsz_v10000 = 312,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecDestroy_v10000 = 313,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroy_v10000 = 314,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_v10000 = 315,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_ptsz_v10000 = 316,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_v10000 = 317,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_ptsz_v10000 = 318,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_v10000 = 319,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_ptsz_v10000 = 320,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeSetParams_v10000 = 321,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetNodes_v10000 = 322,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetEdges_v10000 = 323,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v10010 = 324,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_ptsz_v10010 = 325,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecKernelNodeSetParams_v10010 = 326,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExchangeStreamCaptureMode_v10010 = 327,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetNvSciSyncAttributes_v10020 = 328,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyAvailableDynamicSMemPerBlock_v10200 = 329,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_v10200 = 330,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_ptsz_v10200 = 331,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams_v10020 = 332,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemsetNodeSetParams_v10020 = 333,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecHostNodeSetParams_v10020 = 334,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecUpdate_v10020 = 335,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetFuncBySymbol_v11000 = 336,
    CUPTI_RUNTIME_TRACE_CBID_cudaCtxResetPersistingL2Cache_v11000 = 337,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeCopyAttributes_v11000 = 338,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetAttribute_v11000 = 339,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetAttribute_v11000 = 340,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_v11000 = 341,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_ptsz_v11000 = 342,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_v11000 = 343,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_ptsz_v11000 = 344,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_v11000 = 345,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_ptsz_v11000 = 346,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetTexture1DLinearMaxWidth_v11010 = 347,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_v10000 = 348,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_ptsz_v10000 = 349,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeToSymbol_v11010 = 350,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeFromSymbol_v11010 = 351,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode1D_v11010 = 352,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsToSymbol_v11010 = 353,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsFromSymbol_v11010 = 354,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams1D_v11010 = 355,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsToSymbol_v11010 = 356,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsFromSymbol_v11010 = 357,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams1D_v11010 = 358,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetSparseProperties_v11010 = 359,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetSparseProperties_v11010 = 360,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecChildGraphNodeSetParams_v11010 = 361,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventRecordNode_v11010 = 362,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeGetEvent_v11010 = 363,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeSetEvent_v11010 = 364,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventWaitNode_v11010 = 365,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeGetEvent_v11010 = 366,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeSetEvent_v11010 = 367,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventRecordNodeSetEvent_v11010 = 368,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventWaitNodeSetEvent_v11010 = 369,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_v11010 = 370,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_ptsz_v11010 = 371,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetDefaultMemPool_v11020 = 372,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_v11020 = 373,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_ptsz_v11020 = 374,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_v11020 = 375,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_ptsz_v11020 = 376,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolTrimTo_v11020 = 377,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAttribute_v11020 = 378,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAttribute_v11020 = 379,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAccess_v11020 = 380,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetPlane_v11020 = 381,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAccess_v11020 = 382,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolCreate_v11020 = 383,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolDestroy_v11020 = 384,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetMemPool_v11020 = 385,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetMemPool_v11020 = 386,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportToShareableHandle_v11020 = 387,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportFromShareableHandle_v11020 = 388,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportPointer_v11020 = 389,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportPointer_v11020 = 390,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_v11020 = 391,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_ptsz_v11020 = 392,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v2_v11020 = 393,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v2_ptsz_v11020 = 394,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v2_v11020 = 395,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v2_ptsz_v11020 = 396,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresSignalNode_v11020 = 397,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeGetParams_v11020 = 398,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeSetParams_v11020 = 399,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresWaitNode_v11020 = 400,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeGetParams_v11020 = 401,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeSetParams_v11020 = 402,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresSignalNodeSetParams_v11020 = 403,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresWaitNodeSetParams_v11020 = 404,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceFlushGPUDirectRDMAWrites_v11030 = 405,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_v11030 = 406,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_ptsz_v11030 = 407,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDebugDotPrint_v11030 = 408,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_v11030 = 409,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_ptsz_v11030 = 410,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v11030 = 411,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_ptsz_v11030 = 412,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectCreate_v11030 = 413,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRetain_v11030 = 414,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRelease_v11030 = 415,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRetainUserObject_v11030 = 416,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphReleaseUserObject_v11030 = 417,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithFlags_v11040 = 418,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemAllocNode_v11040 = 419,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemAllocNodeGetParams_v11040 = 420,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemFreeNode_v11040 = 421,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemFreeNodeGetParams_v11040 = 422,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGraphMemTrim_v11040 = 423,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetGraphMemAttribute_v11040 = 424,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetGraphMemAttribute_v11040 = 425,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeSetEnabled_v11060 = 426,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetEnabled_v11060 = 427,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetMemoryRequirements_v11060 = 428,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetMemoryRequirements_v11060 = 429,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernelExC_v11060 = 430,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernelExC_ptsz_v11060 = 431,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxPotentialClusterSize_v11070 = 432,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveClusters_v11070 = 433,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateTextureObject_v2_v11080 = 434,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectTextureDesc_v2_v11080 = 435,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithParams_v12000 = 436,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithParams_ptsz_v12000 = 437,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecGetFlags_v12000 = 438,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetKernel_v12000 = 439,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v2_v12000 = 440,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetId_v12000 = 441,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetId_ptsz_v12000 = 442,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiate_v12000 = 443,
    CUPTI_RUNTIME_TRACE_CBID_cudaInitDevice_v12000 = 444,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddNode_v12020 = 445,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeSetParams_v12020 = 446,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecNodeSetParams_v12020 = 447,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemAdvise_v2_v12020 = 448,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v2_v12020 = 449,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v2_ptsz_v12020 = 450,
    CUPTI_RUNTIME_TRACE_CBID_SIZE = 451,
    CUPTI_RUNTIME_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-12030"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_runtime_api_trace_cbid_enum {
    CUPTI_RUNTIME_TRACE_CBID_INVALID = 0,
    CUPTI_RUNTIME_TRACE_CBID_cudaDriverGetVersion_v3020 = 1,
    CUPTI_RUNTIME_TRACE_CBID_cudaRuntimeGetVersion_v3020 = 2,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceCount_v3020 = 3,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v3020 = 4,
    CUPTI_RUNTIME_TRACE_CBID_cudaChooseDevice_v3020 = 5,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetChannelDesc_v3020 = 6,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateChannelDesc_v3020 = 7,
    CUPTI_RUNTIME_TRACE_CBID_cudaConfigureCall_v3020 = 8,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetupArgument_v3020 = 9,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetLastError_v3020 = 10,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeekAtLastError_v3020 = 11,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorString_v3020 = 12,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_v3020 = 13,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetCacheConfig_v3020 = 14,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetAttributes_v3020 = 15,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDevice_v3020 = 16,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDevice_v3020 = 17,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetValidDevices_v3020 = 18,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDeviceFlags_v3020 = 19,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc_v3020 = 20,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocPitch_v3020 = 21,
    CUPTI_RUNTIME_TRACE_CBID_cudaFree_v3020 = 22,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocArray_v3020 = 23,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeArray_v3020 = 24,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocHost_v3020 = 25,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeHost_v3020 = 26,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostAlloc_v3020 = 27,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetDevicePointer_v3020 = 28,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetFlags_v3020 = 29,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemGetInfo_v3020 = 30,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_v3020 = 31,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_v3020 = 32,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_v3020 = 33,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_v3020 = 34,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_v3020 = 35,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_v3020 = 36,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_v3020 = 37,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_v3020 = 38,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_v3020 = 39,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_v3020 = 40,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_v3020 = 41,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_v3020 = 42,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_v3020 = 43,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_v3020 = 44,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_v3020 = 45,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_v3020 = 46,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_v3020 = 47,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_v3020 = 48,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_v3020 = 49,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_v3020 = 50,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_v3020 = 51,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_v3020 = 52,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolAddress_v3020 = 53,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolSize_v3020 = 54,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture_v3020 = 55,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture2D_v3020 = 56,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToArray_v3020 = 57,
    CUPTI_RUNTIME_TRACE_CBID_cudaUnbindTexture_v3020 = 58,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureAlignmentOffset_v3020 = 59,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureReference_v3020 = 60,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindSurfaceToArray_v3020 = 61,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceReference_v3020 = 62,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetGLDevice_v3020 = 63,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLRegisterBufferObject_v3020 = 64,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObject_v3020 = 65,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObject_v3020 = 66,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnregisterBufferObject_v3020 = 67,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetBufferObjectMapFlags_v3020 = 68,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObjectAsync_v3020 = 69,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObjectAsync_v3020 = 70,
    CUPTI_RUNTIME_TRACE_CBID_cudaWGLGetDevice_v3020 = 71,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterImage_v3020 = 72,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterBuffer_v3020 = 73,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnregisterResource_v3020 = 74,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceSetMapFlags_v3020 = 75,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsMapResources_v3020 = 76,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnmapResources_v3020 = 77,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedPointer_v3020 = 78,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsSubResourceGetMappedArray_v3020 = 79,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUGetDevice_v3020 = 80,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUSetVDPAUDevice_v3020 = 81,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterVideoSurface_v3020 = 82,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterOutputSurface_v3020 = 83,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevice_v3020 = 84,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevices_v3020 = 85,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11SetDirect3DDevice_v3020 = 86,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D11RegisterResource_v3020 = 87,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevice_v3020 = 88,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevices_v3020 = 89,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10SetDirect3DDevice_v3020 = 90,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D10RegisterResource_v3020 = 91,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10RegisterResource_v3020 = 92,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnregisterResource_v3020 = 93,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10MapResources_v3020 = 94,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnmapResources_v3020 = 95,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceSetMapFlags_v3020 = 96,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetSurfaceDimensions_v3020 = 97,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedArray_v3020 = 98,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPointer_v3020 = 99,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedSize_v3020 = 100,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPitch_v3020 = 101,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevice_v3020 = 102,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevices_v3020 = 103,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9SetDirect3DDevice_v3020 = 104,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDirect3DDevice_v3020 = 105,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D9RegisterResource_v3020 = 106,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterResource_v3020 = 107,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterResource_v3020 = 108,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapResources_v3020 = 109,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapResources_v3020 = 110,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceSetMapFlags_v3020 = 111,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetSurfaceDimensions_v3020 = 112,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedArray_v3020 = 113,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPointer_v3020 = 114,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedSize_v3020 = 115,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPitch_v3020 = 116,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9Begin_v3020 = 117,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9End_v3020 = 118,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterVertexBuffer_v3020 = 119,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterVertexBuffer_v3020 = 120,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapVertexBuffer_v3020 = 121,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapVertexBuffer_v3020 = 122,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExit_v3020 = 123,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForDevice_v3020 = 124,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForHost_v3020 = 125,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSynchronize_v3020 = 126,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetLimit_v3020 = 127,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetLimit_v3020 = 128,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreate_v3020 = 129,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v3020 = 130,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_v3020 = 131,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_v3020 = 132,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreate_v3020 = 133,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateWithFlags_v3020 = 134,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_v3020 = 135,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventDestroy_v3020 = 136,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventSynchronize_v3020 = 137,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventQuery_v3020 = 138,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventElapsedTime_v3020 = 139,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3D_v3020 = 140,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3DArray_v3020 = 141,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_v3020 = 142,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_v3020 = 143,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_v3020 = 144,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_v3020 = 145,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetCacheConfig_v3020 = 146,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_v3020 = 147,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDirect3DDevice_v3020 = 148,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDirect3DDevice_v3020 = 149,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetCacheConfig_v3020 = 150,
    CUPTI_RUNTIME_TRACE_CBID_cudaPointerGetAttributes_v4000 = 151,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostRegister_v4000 = 152,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostUnregister_v4000 = 153,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceCanAccessPeer_v4000 = 154,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceEnablePeerAccess_v4000 = 155,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceDisablePeerAccess_v4000 = 156,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerRegister_v4000 = 157,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerUnregister_v4000 = 158,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerGetDevicePointer_v4000 = 159,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeer_v4000 = 160,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeerAsync_v4000 = 161,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_v4000 = 162,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_v4000 = 163,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceReset_v3020 = 164,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSynchronize_v3020 = 165,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetLimit_v3020 = 166,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetLimit_v3020 = 167,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetCacheConfig_v3020 = 168,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetCacheConfig_v3020 = 169,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerInitialize_v4000 = 170,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStart_v4000 = 171,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStop_v4000 = 172,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetByPCIBusId_v4010 = 173,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetPCIBusId_v4010 = 174,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLGetDevices_v4010 = 175,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetEventHandle_v4010 = 176,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenEventHandle_v4010 = 177,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetMemHandle_v4010 = 178,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenMemHandle_v4010 = 179,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcCloseMemHandle_v4010 = 180,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetInfo_v4010 = 181,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetSharedMemConfig_v4020 = 182,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetSharedMemConfig_v4020 = 183,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetSharedMemConfig_v4020 = 184,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateTextureObject_v5000 = 185,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyTextureObject_v5000 = 186,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceDesc_v5000 = 187,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectTextureDesc_v5000 = 188,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateSurfaceObject_v5000 = 189,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroySurfaceObject_v5000 = 190,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceObjectResourceDesc_v5000 = 191,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocMipmappedArray_v5000 = 192,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetMipmappedArrayLevel_v5000 = 193,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeMipmappedArray_v5000 = 194,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToMipmappedArray_v5000 = 195,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedMipmappedArray_v5000 = 196,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_v5000 = 197,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithFlags_v5000 = 198,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceViewDesc_v5000 = 199,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetAttribute_v5000 = 200,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v5050 = 201,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithPriority_v5050 = 202,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_v5050 = 203,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_v5050 = 204,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetStreamPriorityRange_v5050 = 205,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocManaged_v6000 = 206,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6000 = 207,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_v6000 = 208,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorName_v6050 = 209,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6050 = 210,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_v7000 = 211,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceFlags_v7000 = 212,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_ptsz_v7000 = 213,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_ptsz_v7000 = 214,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_ptds_v7000 = 215,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_ptds_v7000 = 216,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_ptds_v7000 = 217,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_ptds_v7000 = 218,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_ptds_v7000 = 219,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_ptds_v7000 = 220,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_ptds_v7000 = 221,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_ptds_v7000 = 222,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_ptds_v7000 = 223,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_ptds_v7000 = 224,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_ptsz_v7000 = 225,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_ptsz_v7000 = 226,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_ptsz_v7000 = 227,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_ptsz_v7000 = 228,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_ptsz_v7000 = 229,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_ptsz_v7000 = 230,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_ptsz_v7000 = 231,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_ptsz_v7000 = 232,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_ptds_v7000 = 233,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_ptds_v7000 = 234,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_ptsz_v7000 = 235,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_ptsz_v7000 = 236,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_ptsz_v7000 = 237,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_ptsz_v7000 = 238,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_ptsz_v7000 = 239,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_ptsz_v7000 = 240,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_ptsz_v7000 = 241,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_ptsz_v7000 = 242,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_ptds_v7000 = 243,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_ptsz_v7000 = 244,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_ptds_v7000 = 245,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_ptsz_v7000 = 246,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_ptsz_v7000 = 247,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_ptsz_v7000 = 248,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_ptds_v7000 = 249,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_ptsz_v7000 = 250,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessorWithFlags_v7000 = 251,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v8000 = 252,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_ptsz_v8000 = 253,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemAdvise_v8000 = 254,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetP2PAttribute_v8000 = 255,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsEGLRegisterImage_v7000 = 256,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnect_v7000 = 257,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerDisconnect_v7000 = 258,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerAcquireFrame_v7000 = 259,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerReleaseFrame_v7000 = 260,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerConnect_v7000 = 261,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerDisconnect_v7000 = 262,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerPresentFrame_v7000 = 263,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerReturnFrame_v7000 = 264,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedEglFrame_v7000 = 265,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttribute_v8000 = 266,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttributes_v8000 = 267,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnectWithFlags_v7000 = 268,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_v9000 = 269,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_ptsz_v9000 = 270,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateFromEGLSync_v9000 = 271,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernelMultiDevice_v9000 = 272,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetAttribute_v9000 = 273,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalMemory_v10000 = 274,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedBuffer_v10000 = 275,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedMipmappedArray_v10000 = 276,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalMemory_v10000 = 277,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalSemaphore_v10000 = 278,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v10000 = 279,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_ptsz_v10000 = 280,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v10000 = 281,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_ptsz_v10000 = 282,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalSemaphore_v10000 = 283,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_v10000 = 284,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_ptsz_v10000 = 285,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphCreate_v10000 = 286,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetParams_v10000 = 287,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetParams_v10000 = 288,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddKernelNode_v10000 = 289,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode_v10000 = 290,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeGetParams_v10000 = 291,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams_v10000 = 292,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemsetNode_v10000 = 293,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeGetParams_v10000 = 294,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeSetParams_v10000 = 295,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddHostNode_v10000 = 296,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeGetParams_v10000 = 297,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddChildGraphNode_v10000 = 298,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphChildGraphNodeGetGraph_v10000 = 299,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEmptyNode_v10000 = 300,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphClone_v10000 = 301,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeFindInClone_v10000 = 302,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetType_v10000 = 303,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetRootNodes_v10000 = 304,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependencies_v10000 = 305,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependentNodes_v10000 = 306,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddDependencies_v10000 = 307,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRemoveDependencies_v10000 = 308,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroyNode_v10000 = 309,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiate_v10000 = 310,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_v10000 = 311,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_ptsz_v10000 = 312,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecDestroy_v10000 = 313,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroy_v10000 = 314,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_v10000 = 315,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_ptsz_v10000 = 316,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_v10000 = 317,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_ptsz_v10000 = 318,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_v10000 = 319,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_ptsz_v10000 = 320,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeSetParams_v10000 = 321,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetNodes_v10000 = 322,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetEdges_v10000 = 323,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v10010 = 324,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_ptsz_v10010 = 325,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecKernelNodeSetParams_v10010 = 326,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExchangeStreamCaptureMode_v10010 = 327,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetNvSciSyncAttributes_v10020 = 328,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyAvailableDynamicSMemPerBlock_v10200 = 329,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_v10200 = 330,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_ptsz_v10200 = 331,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams_v10020 = 332,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemsetNodeSetParams_v10020 = 333,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecHostNodeSetParams_v10020 = 334,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecUpdate_v10020 = 335,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetFuncBySymbol_v11000 = 336,
    CUPTI_RUNTIME_TRACE_CBID_cudaCtxResetPersistingL2Cache_v11000 = 337,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeCopyAttributes_v11000 = 338,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetAttribute_v11000 = 339,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetAttribute_v11000 = 340,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_v11000 = 341,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_ptsz_v11000 = 342,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_v11000 = 343,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_ptsz_v11000 = 344,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_v11000 = 345,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_ptsz_v11000 = 346,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetTexture1DLinearMaxWidth_v11010 = 347,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_v10000 = 348,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_ptsz_v10000 = 349,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeToSymbol_v11010 = 350,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeFromSymbol_v11010 = 351,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode1D_v11010 = 352,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsToSymbol_v11010 = 353,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsFromSymbol_v11010 = 354,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams1D_v11010 = 355,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsToSymbol_v11010 = 356,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsFromSymbol_v11010 = 357,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams1D_v11010 = 358,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetSparseProperties_v11010 = 359,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetSparseProperties_v11010 = 360,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecChildGraphNodeSetParams_v11010 = 361,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventRecordNode_v11010 = 362,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeGetEvent_v11010 = 363,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeSetEvent_v11010 = 364,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventWaitNode_v11010 = 365,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeGetEvent_v11010 = 366,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeSetEvent_v11010 = 367,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventRecordNodeSetEvent_v11010 = 368,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventWaitNodeSetEvent_v11010 = 369,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_v11010 = 370,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_ptsz_v11010 = 371,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetDefaultMemPool_v11020 = 372,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_v11020 = 373,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_ptsz_v11020 = 374,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_v11020 = 375,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_ptsz_v11020 = 376,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolTrimTo_v11020 = 377,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAttribute_v11020 = 378,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAttribute_v11020 = 379,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAccess_v11020 = 380,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetPlane_v11020 = 381,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAccess_v11020 = 382,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolCreate_v11020 = 383,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolDestroy_v11020 = 384,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetMemPool_v11020 = 385,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetMemPool_v11020 = 386,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportToShareableHandle_v11020 = 387,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportFromShareableHandle_v11020 = 388,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportPointer_v11020 = 389,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportPointer_v11020 = 390,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_v11020 = 391,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_ptsz_v11020 = 392,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v2_v11020 = 393,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v2_ptsz_v11020 = 394,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v2_v11020 = 395,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v2_ptsz_v11020 = 396,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresSignalNode_v11020 = 397,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeGetParams_v11020 = 398,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeSetParams_v11020 = 399,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresWaitNode_v11020 = 400,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeGetParams_v11020 = 401,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeSetParams_v11020 = 402,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresSignalNodeSetParams_v11020 = 403,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresWaitNodeSetParams_v11020 = 404,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceFlushGPUDirectRDMAWrites_v11030 = 405,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_v11030 = 406,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_ptsz_v11030 = 407,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDebugDotPrint_v11030 = 408,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_v11030 = 409,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_ptsz_v11030 = 410,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v11030 = 411,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_ptsz_v11030 = 412,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectCreate_v11030 = 413,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRetain_v11030 = 414,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRelease_v11030 = 415,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRetainUserObject_v11030 = 416,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphReleaseUserObject_v11030 = 417,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithFlags_v11040 = 418,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemAllocNode_v11040 = 419,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemAllocNodeGetParams_v11040 = 420,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemFreeNode_v11040 = 421,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemFreeNodeGetParams_v11040 = 422,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGraphMemTrim_v11040 = 423,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetGraphMemAttribute_v11040 = 424,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetGraphMemAttribute_v11040 = 425,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeSetEnabled_v11060 = 426,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetEnabled_v11060 = 427,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetMemoryRequirements_v11060 = 428,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetMemoryRequirements_v11060 = 429,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernelExC_v11060 = 430,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernelExC_ptsz_v11060 = 431,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxPotentialClusterSize_v11070 = 432,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveClusters_v11070 = 433,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateTextureObject_v2_v11080 = 434,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectTextureDesc_v2_v11080 = 435,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithParams_v12000 = 436,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithParams_ptsz_v12000 = 437,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecGetFlags_v12000 = 438,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetKernel_v12000 = 439,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v2_v12000 = 440,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetId_v12000 = 441,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetId_ptsz_v12000 = 442,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiate_v12000 = 443,
    CUPTI_RUNTIME_TRACE_CBID_cudaInitDevice_v12000 = 444,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddNode_v12020 = 445,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeSetParams_v12020 = 446,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecNodeSetParams_v12020 = 447,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemAdvise_v2_v12020 = 448,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v2_v12020 = 449,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v2_ptsz_v12020 = 450,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetName_v12030 = 451,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCaptureToGraph_v12030 = 452,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCaptureToGraph_ptsz_v12030 = 453,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphConditionalHandleCreate_v12030 = 454,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetEdges_v2_v12030 = 455,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependencies_v2_v12030 = 456,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependentNodes_v2_v12030 = 457,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddDependencies_v2_v12030 = 458,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRemoveDependencies_v2_v12030 = 459,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddNode_v2_v12030 = 460,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v3_v12030 = 461,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v3_ptsz_v12030 = 462,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v2_v12030 = 463,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v2_ptsz_v12030 = 464,
    CUPTI_RUNTIME_TRACE_CBID_SIZE = 465,
    CUPTI_RUNTIME_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-12040"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_runtime_api_trace_cbid_enum {
    CUPTI_RUNTIME_TRACE_CBID_INVALID = 0,
    CUPTI_RUNTIME_TRACE_CBID_cudaDriverGetVersion_v3020 = 1,
    CUPTI_RUNTIME_TRACE_CBID_cudaRuntimeGetVersion_v3020 = 2,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceCount_v3020 = 3,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v3020 = 4,
    CUPTI_RUNTIME_TRACE_CBID_cudaChooseDevice_v3020 = 5,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetChannelDesc_v3020 = 6,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateChannelDesc_v3020 = 7,
    CUPTI_RUNTIME_TRACE_CBID_cudaConfigureCall_v3020 = 8,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetupArgument_v3020 = 9,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetLastError_v3020 = 10,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeekAtLastError_v3020 = 11,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorString_v3020 = 12,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_v3020 = 13,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetCacheConfig_v3020 = 14,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetAttributes_v3020 = 15,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDevice_v3020 = 16,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDevice_v3020 = 17,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetValidDevices_v3020 = 18,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDeviceFlags_v3020 = 19,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc_v3020 = 20,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocPitch_v3020 = 21,
    CUPTI_RUNTIME_TRACE_CBID_cudaFree_v3020 = 22,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocArray_v3020 = 23,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeArray_v3020 = 24,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocHost_v3020 = 25,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeHost_v3020 = 26,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostAlloc_v3020 = 27,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetDevicePointer_v3020 = 28,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetFlags_v3020 = 29,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemGetInfo_v3020 = 30,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_v3020 = 31,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_v3020 = 32,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_v3020 = 33,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_v3020 = 34,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_v3020 = 35,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_v3020 = 36,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_v3020 = 37,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_v3020 = 38,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_v3020 = 39,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_v3020 = 40,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_v3020 = 41,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_v3020 = 42,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_v3020 = 43,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_v3020 = 44,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_v3020 = 45,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_v3020 = 46,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_v3020 = 47,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_v3020 = 48,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_v3020 = 49,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_v3020 = 50,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_v3020 = 51,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_v3020 = 52,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolAddress_v3020 = 53,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolSize_v3020 = 54,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture_v3020 = 55,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture2D_v3020 = 56,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToArray_v3020 = 57,
    CUPTI_RUNTIME_TRACE_CBID_cudaUnbindTexture_v3020 = 58,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureAlignmentOffset_v3020 = 59,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureReference_v3020 = 60,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindSurfaceToArray_v3020 = 61,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceReference_v3020 = 62,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetGLDevice_v3020 = 63,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLRegisterBufferObject_v3020 = 64,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObject_v3020 = 65,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObject_v3020 = 66,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnregisterBufferObject_v3020 = 67,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetBufferObjectMapFlags_v3020 = 68,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObjectAsync_v3020 = 69,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObjectAsync_v3020 = 70,
    CUPTI_RUNTIME_TRACE_CBID_cudaWGLGetDevice_v3020 = 71,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterImage_v3020 = 72,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterBuffer_v3020 = 73,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnregisterResource_v3020 = 74,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceSetMapFlags_v3020 = 75,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsMapResources_v3020 = 76,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnmapResources_v3020 = 77,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedPointer_v3020 = 78,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsSubResourceGetMappedArray_v3020 = 79,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUGetDevice_v3020 = 80,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUSetVDPAUDevice_v3020 = 81,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterVideoSurface_v3020 = 82,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterOutputSurface_v3020 = 83,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevice_v3020 = 84,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevices_v3020 = 85,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11SetDirect3DDevice_v3020 = 86,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D11RegisterResource_v3020 = 87,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevice_v3020 = 88,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevices_v3020 = 89,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10SetDirect3DDevice_v3020 = 90,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D10RegisterResource_v3020 = 91,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10RegisterResource_v3020 = 92,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnregisterResource_v3020 = 93,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10MapResources_v3020 = 94,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnmapResources_v3020 = 95,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceSetMapFlags_v3020 = 96,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetSurfaceDimensions_v3020 = 97,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedArray_v3020 = 98,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPointer_v3020 = 99,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedSize_v3020 = 100,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPitch_v3020 = 101,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevice_v3020 = 102,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevices_v3020 = 103,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9SetDirect3DDevice_v3020 = 104,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDirect3DDevice_v3020 = 105,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D9RegisterResource_v3020 = 106,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterResource_v3020 = 107,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterResource_v3020 = 108,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapResources_v3020 = 109,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapResources_v3020 = 110,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceSetMapFlags_v3020 = 111,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetSurfaceDimensions_v3020 = 112,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedArray_v3020 = 113,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPointer_v3020 = 114,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedSize_v3020 = 115,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPitch_v3020 = 116,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9Begin_v3020 = 117,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9End_v3020 = 118,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterVertexBuffer_v3020 = 119,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterVertexBuffer_v3020 = 120,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapVertexBuffer_v3020 = 121,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapVertexBuffer_v3020 = 122,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExit_v3020 = 123,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForDevice_v3020 = 124,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForHost_v3020 = 125,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSynchronize_v3020 = 126,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetLimit_v3020 = 127,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetLimit_v3020 = 128,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreate_v3020 = 129,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v3020 = 130,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_v3020 = 131,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_v3020 = 132,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreate_v3020 = 133,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateWithFlags_v3020 = 134,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_v3020 = 135,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventDestroy_v3020 = 136,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventSynchronize_v3020 = 137,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventQuery_v3020 = 138,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventElapsedTime_v3020 = 139,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3D_v3020 = 140,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3DArray_v3020 = 141,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_v3020 = 142,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_v3020 = 143,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_v3020 = 144,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_v3020 = 145,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetCacheConfig_v3020 = 146,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_v3020 = 147,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDirect3DDevice_v3020 = 148,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDirect3DDevice_v3020 = 149,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetCacheConfig_v3020 = 150,
    CUPTI_RUNTIME_TRACE_CBID_cudaPointerGetAttributes_v4000 = 151,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostRegister_v4000 = 152,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostUnregister_v4000 = 153,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceCanAccessPeer_v4000 = 154,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceEnablePeerAccess_v4000 = 155,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceDisablePeerAccess_v4000 = 156,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerRegister_v4000 = 157,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerUnregister_v4000 = 158,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerGetDevicePointer_v4000 = 159,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeer_v4000 = 160,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeerAsync_v4000 = 161,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_v4000 = 162,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_v4000 = 163,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceReset_v3020 = 164,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSynchronize_v3020 = 165,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetLimit_v3020 = 166,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetLimit_v3020 = 167,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetCacheConfig_v3020 = 168,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetCacheConfig_v3020 = 169,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerInitialize_v4000 = 170,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStart_v4000 = 171,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStop_v4000 = 172,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetByPCIBusId_v4010 = 173,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetPCIBusId_v4010 = 174,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLGetDevices_v4010 = 175,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetEventHandle_v4010 = 176,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenEventHandle_v4010 = 177,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetMemHandle_v4010 = 178,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenMemHandle_v4010 = 179,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcCloseMemHandle_v4010 = 180,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetInfo_v4010 = 181,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetSharedMemConfig_v4020 = 182,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetSharedMemConfig_v4020 = 183,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetSharedMemConfig_v4020 = 184,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateTextureObject_v5000 = 185,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyTextureObject_v5000 = 186,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceDesc_v5000 = 187,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectTextureDesc_v5000 = 188,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateSurfaceObject_v5000 = 189,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroySurfaceObject_v5000 = 190,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceObjectResourceDesc_v5000 = 191,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocMipmappedArray_v5000 = 192,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetMipmappedArrayLevel_v5000 = 193,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeMipmappedArray_v5000 = 194,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToMipmappedArray_v5000 = 195,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedMipmappedArray_v5000 = 196,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_v5000 = 197,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithFlags_v5000 = 198,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceViewDesc_v5000 = 199,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetAttribute_v5000 = 200,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v5050 = 201,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithPriority_v5050 = 202,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_v5050 = 203,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_v5050 = 204,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetStreamPriorityRange_v5050 = 205,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocManaged_v6000 = 206,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6000 = 207,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_v6000 = 208,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorName_v6050 = 209,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6050 = 210,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_v7000 = 211,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceFlags_v7000 = 212,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_ptsz_v7000 = 213,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_ptsz_v7000 = 214,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_ptds_v7000 = 215,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_ptds_v7000 = 216,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_ptds_v7000 = 217,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_ptds_v7000 = 218,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_ptds_v7000 = 219,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_ptds_v7000 = 220,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_ptds_v7000 = 221,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_ptds_v7000 = 222,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_ptds_v7000 = 223,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_ptds_v7000 = 224,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_ptsz_v7000 = 225,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_ptsz_v7000 = 226,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_ptsz_v7000 = 227,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_ptsz_v7000 = 228,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_ptsz_v7000 = 229,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_ptsz_v7000 = 230,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_ptsz_v7000 = 231,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_ptsz_v7000 = 232,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_ptds_v7000 = 233,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_ptds_v7000 = 234,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_ptsz_v7000 = 235,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_ptsz_v7000 = 236,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_ptsz_v7000 = 237,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_ptsz_v7000 = 238,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_ptsz_v7000 = 239,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_ptsz_v7000 = 240,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_ptsz_v7000 = 241,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_ptsz_v7000 = 242,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_ptds_v7000 = 243,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_ptsz_v7000 = 244,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_ptds_v7000 = 245,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_ptsz_v7000 = 246,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_ptsz_v7000 = 247,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_ptsz_v7000 = 248,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_ptds_v7000 = 249,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_ptsz_v7000 = 250,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessorWithFlags_v7000 = 251,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v8000 = 252,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_ptsz_v8000 = 253,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemAdvise_v8000 = 254,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetP2PAttribute_v8000 = 255,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsEGLRegisterImage_v7000 = 256,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnect_v7000 = 257,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerDisconnect_v7000 = 258,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerAcquireFrame_v7000 = 259,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerReleaseFrame_v7000 = 260,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerConnect_v7000 = 261,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerDisconnect_v7000 = 262,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerPresentFrame_v7000 = 263,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerReturnFrame_v7000 = 264,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedEglFrame_v7000 = 265,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttribute_v8000 = 266,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttributes_v8000 = 267,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnectWithFlags_v7000 = 268,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_v9000 = 269,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_ptsz_v9000 = 270,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateFromEGLSync_v9000 = 271,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernelMultiDevice_v9000 = 272,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetAttribute_v9000 = 273,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalMemory_v10000 = 274,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedBuffer_v10000 = 275,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedMipmappedArray_v10000 = 276,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalMemory_v10000 = 277,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalSemaphore_v10000 = 278,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v10000 = 279,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_ptsz_v10000 = 280,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v10000 = 281,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_ptsz_v10000 = 282,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalSemaphore_v10000 = 283,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_v10000 = 284,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_ptsz_v10000 = 285,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphCreate_v10000 = 286,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetParams_v10000 = 287,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetParams_v10000 = 288,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddKernelNode_v10000 = 289,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode_v10000 = 290,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeGetParams_v10000 = 291,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams_v10000 = 292,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemsetNode_v10000 = 293,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeGetParams_v10000 = 294,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeSetParams_v10000 = 295,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddHostNode_v10000 = 296,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeGetParams_v10000 = 297,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddChildGraphNode_v10000 = 298,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphChildGraphNodeGetGraph_v10000 = 299,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEmptyNode_v10000 = 300,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphClone_v10000 = 301,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeFindInClone_v10000 = 302,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetType_v10000 = 303,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetRootNodes_v10000 = 304,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependencies_v10000 = 305,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependentNodes_v10000 = 306,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddDependencies_v10000 = 307,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRemoveDependencies_v10000 = 308,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroyNode_v10000 = 309,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiate_v10000 = 310,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_v10000 = 311,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_ptsz_v10000 = 312,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecDestroy_v10000 = 313,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroy_v10000 = 314,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_v10000 = 315,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_ptsz_v10000 = 316,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_v10000 = 317,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_ptsz_v10000 = 318,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_v10000 = 319,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_ptsz_v10000 = 320,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeSetParams_v10000 = 321,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetNodes_v10000 = 322,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetEdges_v10000 = 323,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v10010 = 324,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_ptsz_v10010 = 325,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecKernelNodeSetParams_v10010 = 326,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExchangeStreamCaptureMode_v10010 = 327,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetNvSciSyncAttributes_v10020 = 328,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyAvailableDynamicSMemPerBlock_v10200 = 329,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_v10200 = 330,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_ptsz_v10200 = 331,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams_v10020 = 332,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemsetNodeSetParams_v10020 = 333,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecHostNodeSetParams_v10020 = 334,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecUpdate_v10020 = 335,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetFuncBySymbol_v11000 = 336,
    CUPTI_RUNTIME_TRACE_CBID_cudaCtxResetPersistingL2Cache_v11000 = 337,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeCopyAttributes_v11000 = 338,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetAttribute_v11000 = 339,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetAttribute_v11000 = 340,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_v11000 = 341,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_ptsz_v11000 = 342,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_v11000 = 343,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_ptsz_v11000 = 344,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_v11000 = 345,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_ptsz_v11000 = 346,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetTexture1DLinearMaxWidth_v11010 = 347,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_v10000 = 348,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_ptsz_v10000 = 349,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeToSymbol_v11010 = 350,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeFromSymbol_v11010 = 351,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode1D_v11010 = 352,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsToSymbol_v11010 = 353,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsFromSymbol_v11010 = 354,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams1D_v11010 = 355,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsToSymbol_v11010 = 356,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsFromSymbol_v11010 = 357,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams1D_v11010 = 358,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetSparseProperties_v11010 = 359,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetSparseProperties_v11010 = 360,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecChildGraphNodeSetParams_v11010 = 361,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventRecordNode_v11010 = 362,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeGetEvent_v11010 = 363,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeSetEvent_v11010 = 364,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventWaitNode_v11010 = 365,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeGetEvent_v11010 = 366,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeSetEvent_v11010 = 367,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventRecordNodeSetEvent_v11010 = 368,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventWaitNodeSetEvent_v11010 = 369,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_v11010 = 370,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_ptsz_v11010 = 371,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetDefaultMemPool_v11020 = 372,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_v11020 = 373,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_ptsz_v11020 = 374,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_v11020 = 375,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_ptsz_v11020 = 376,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolTrimTo_v11020 = 377,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAttribute_v11020 = 378,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAttribute_v11020 = 379,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAccess_v11020 = 380,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetPlane_v11020 = 381,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAccess_v11020 = 382,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolCreate_v11020 = 383,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolDestroy_v11020 = 384,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetMemPool_v11020 = 385,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetMemPool_v11020 = 386,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportToShareableHandle_v11020 = 387,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportFromShareableHandle_v11020 = 388,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportPointer_v11020 = 389,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportPointer_v11020 = 390,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_v11020 = 391,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_ptsz_v11020 = 392,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v2_v11020 = 393,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v2_ptsz_v11020 = 394,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v2_v11020 = 395,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v2_ptsz_v11020 = 396,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresSignalNode_v11020 = 397,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeGetParams_v11020 = 398,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeSetParams_v11020 = 399,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresWaitNode_v11020 = 400,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeGetParams_v11020 = 401,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeSetParams_v11020 = 402,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresSignalNodeSetParams_v11020 = 403,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresWaitNodeSetParams_v11020 = 404,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceFlushGPUDirectRDMAWrites_v11030 = 405,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_v11030 = 406,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_ptsz_v11030 = 407,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDebugDotPrint_v11030 = 408,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_v11030 = 409,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_ptsz_v11030 = 410,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v11030 = 411,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_ptsz_v11030 = 412,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectCreate_v11030 = 413,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRetain_v11030 = 414,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRelease_v11030 = 415,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRetainUserObject_v11030 = 416,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphReleaseUserObject_v11030 = 417,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithFlags_v11040 = 418,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemAllocNode_v11040 = 419,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemAllocNodeGetParams_v11040 = 420,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemFreeNode_v11040 = 421,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemFreeNodeGetParams_v11040 = 422,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGraphMemTrim_v11040 = 423,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetGraphMemAttribute_v11040 = 424,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetGraphMemAttribute_v11040 = 425,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeSetEnabled_v11060 = 426,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetEnabled_v11060 = 427,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetMemoryRequirements_v11060 = 428,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetMemoryRequirements_v11060 = 429,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernelExC_v11060 = 430,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernelExC_ptsz_v11060 = 431,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxPotentialClusterSize_v11070 = 432,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveClusters_v11070 = 433,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateTextureObject_v2_v11080 = 434,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectTextureDesc_v2_v11080 = 435,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithParams_v12000 = 436,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithParams_ptsz_v12000 = 437,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecGetFlags_v12000 = 438,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetKernel_v12000 = 439,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v2_v12000 = 440,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetId_v12000 = 441,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetId_ptsz_v12000 = 442,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiate_v12000 = 443,
    CUPTI_RUNTIME_TRACE_CBID_cudaInitDevice_v12000 = 444,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddNode_v12020 = 445,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeSetParams_v12020 = 446,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecNodeSetParams_v12020 = 447,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemAdvise_v2_v12020 = 448,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v2_v12020 = 449,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v2_ptsz_v12020 = 450,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetName_v12030 = 451,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCaptureToGraph_v12030 = 452,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCaptureToGraph_ptsz_v12030 = 453,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphConditionalHandleCreate_v12030 = 454,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetEdges_v2_v12030 = 455,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependencies_v2_v12030 = 456,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependentNodes_v2_v12030 = 457,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddDependencies_v2_v12030 = 458,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRemoveDependencies_v2_v12030 = 459,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddNode_v2_v12030 = 460,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v3_v12030 = 461,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v3_ptsz_v12030 = 462,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v2_v12030 = 463,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v2_ptsz_v12030 = 464,
    CUPTI_RUNTIME_TRACE_CBID_cuda465_v12040 = 465,
    CUPTI_RUNTIME_TRACE_CBID_cuda466_v12040 = 466,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetParamInfo_v12040 = 467,
    CUPTI_RUNTIME_TRACE_CBID_SIZE = 468,
    CUPTI_RUNTIME_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-12050"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_runtime_api_trace_cbid_enum {
    CUPTI_RUNTIME_TRACE_CBID_INVALID = 0,
    CUPTI_RUNTIME_TRACE_CBID_cudaDriverGetVersion_v3020 = 1,
    CUPTI_RUNTIME_TRACE_CBID_cudaRuntimeGetVersion_v3020 = 2,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceCount_v3020 = 3,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v3020 = 4,
    CUPTI_RUNTIME_TRACE_CBID_cudaChooseDevice_v3020 = 5,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetChannelDesc_v3020 = 6,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateChannelDesc_v3020 = 7,
    CUPTI_RUNTIME_TRACE_CBID_cudaConfigureCall_v3020 = 8,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetupArgument_v3020 = 9,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetLastError_v3020 = 10,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeekAtLastError_v3020 = 11,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorString_v3020 = 12,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_v3020 = 13,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetCacheConfig_v3020 = 14,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetAttributes_v3020 = 15,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDevice_v3020 = 16,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDevice_v3020 = 17,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetValidDevices_v3020 = 18,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDeviceFlags_v3020 = 19,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc_v3020 = 20,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocPitch_v3020 = 21,
    CUPTI_RUNTIME_TRACE_CBID_cudaFree_v3020 = 22,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocArray_v3020 = 23,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeArray_v3020 = 24,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocHost_v3020 = 25,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeHost_v3020 = 26,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostAlloc_v3020 = 27,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetDevicePointer_v3020 = 28,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetFlags_v3020 = 29,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemGetInfo_v3020 = 30,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_v3020 = 31,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_v3020 = 32,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_v3020 = 33,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_v3020 = 34,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_v3020 = 35,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_v3020 = 36,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_v3020 = 37,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_v3020 = 38,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_v3020 = 39,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_v3020 = 40,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_v3020 = 41,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_v3020 = 42,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_v3020 = 43,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_v3020 = 44,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_v3020 = 45,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_v3020 = 46,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_v3020 = 47,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_v3020 = 48,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_v3020 = 49,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_v3020 = 50,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_v3020 = 51,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_v3020 = 52,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolAddress_v3020 = 53,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolSize_v3020 = 54,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture_v3020 = 55,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture2D_v3020 = 56,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToArray_v3020 = 57,
    CUPTI_RUNTIME_TRACE_CBID_cudaUnbindTexture_v3020 = 58,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureAlignmentOffset_v3020 = 59,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureReference_v3020 = 60,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindSurfaceToArray_v3020 = 61,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceReference_v3020 = 62,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetGLDevice_v3020 = 63,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLRegisterBufferObject_v3020 = 64,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObject_v3020 = 65,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObject_v3020 = 66,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnregisterBufferObject_v3020 = 67,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetBufferObjectMapFlags_v3020 = 68,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObjectAsync_v3020 = 69,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObjectAsync_v3020 = 70,
    CUPTI_RUNTIME_TRACE_CBID_cudaWGLGetDevice_v3020 = 71,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterImage_v3020 = 72,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterBuffer_v3020 = 73,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnregisterResource_v3020 = 74,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceSetMapFlags_v3020 = 75,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsMapResources_v3020 = 76,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnmapResources_v3020 = 77,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedPointer_v3020 = 78,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsSubResourceGetMappedArray_v3020 = 79,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUGetDevice_v3020 = 80,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUSetVDPAUDevice_v3020 = 81,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterVideoSurface_v3020 = 82,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterOutputSurface_v3020 = 83,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevice_v3020 = 84,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevices_v3020 = 85,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11SetDirect3DDevice_v3020 = 86,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D11RegisterResource_v3020 = 87,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevice_v3020 = 88,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevices_v3020 = 89,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10SetDirect3DDevice_v3020 = 90,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D10RegisterResource_v3020 = 91,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10RegisterResource_v3020 = 92,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnregisterResource_v3020 = 93,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10MapResources_v3020 = 94,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnmapResources_v3020 = 95,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceSetMapFlags_v3020 = 96,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetSurfaceDimensions_v3020 = 97,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedArray_v3020 = 98,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPointer_v3020 = 99,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedSize_v3020 = 100,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPitch_v3020 = 101,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevice_v3020 = 102,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevices_v3020 = 103,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9SetDirect3DDevice_v3020 = 104,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDirect3DDevice_v3020 = 105,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D9RegisterResource_v3020 = 106,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterResource_v3020 = 107,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterResource_v3020 = 108,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapResources_v3020 = 109,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapResources_v3020 = 110,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceSetMapFlags_v3020 = 111,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetSurfaceDimensions_v3020 = 112,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedArray_v3020 = 113,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPointer_v3020 = 114,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedSize_v3020 = 115,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPitch_v3020 = 116,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9Begin_v3020 = 117,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9End_v3020 = 118,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterVertexBuffer_v3020 = 119,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterVertexBuffer_v3020 = 120,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapVertexBuffer_v3020 = 121,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapVertexBuffer_v3020 = 122,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExit_v3020 = 123,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForDevice_v3020 = 124,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForHost_v3020 = 125,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSynchronize_v3020 = 126,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetLimit_v3020 = 127,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetLimit_v3020 = 128,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreate_v3020 = 129,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v3020 = 130,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_v3020 = 131,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_v3020 = 132,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreate_v3020 = 133,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateWithFlags_v3020 = 134,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_v3020 = 135,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventDestroy_v3020 = 136,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventSynchronize_v3020 = 137,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventQuery_v3020 = 138,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventElapsedTime_v3020 = 139,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3D_v3020 = 140,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3DArray_v3020 = 141,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_v3020 = 142,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_v3020 = 143,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_v3020 = 144,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_v3020 = 145,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetCacheConfig_v3020 = 146,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_v3020 = 147,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDirect3DDevice_v3020 = 148,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDirect3DDevice_v3020 = 149,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetCacheConfig_v3020 = 150,
    CUPTI_RUNTIME_TRACE_CBID_cudaPointerGetAttributes_v4000 = 151,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostRegister_v4000 = 152,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostUnregister_v4000 = 153,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceCanAccessPeer_v4000 = 154,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceEnablePeerAccess_v4000 = 155,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceDisablePeerAccess_v4000 = 156,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerRegister_v4000 = 157,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerUnregister_v4000 = 158,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerGetDevicePointer_v4000 = 159,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeer_v4000 = 160,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeerAsync_v4000 = 161,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_v4000 = 162,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_v4000 = 163,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceReset_v3020 = 164,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSynchronize_v3020 = 165,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetLimit_v3020 = 166,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetLimit_v3020 = 167,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetCacheConfig_v3020 = 168,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetCacheConfig_v3020 = 169,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerInitialize_v4000 = 170,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStart_v4000 = 171,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStop_v4000 = 172,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetByPCIBusId_v4010 = 173,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetPCIBusId_v4010 = 174,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLGetDevices_v4010 = 175,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetEventHandle_v4010 = 176,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenEventHandle_v4010 = 177,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetMemHandle_v4010 = 178,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenMemHandle_v4010 = 179,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcCloseMemHandle_v4010 = 180,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetInfo_v4010 = 181,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetSharedMemConfig_v4020 = 182,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetSharedMemConfig_v4020 = 183,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetSharedMemConfig_v4020 = 184,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateTextureObject_v5000 = 185,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyTextureObject_v5000 = 186,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceDesc_v5000 = 187,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectTextureDesc_v5000 = 188,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateSurfaceObject_v5000 = 189,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroySurfaceObject_v5000 = 190,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceObjectResourceDesc_v5000 = 191,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocMipmappedArray_v5000 = 192,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetMipmappedArrayLevel_v5000 = 193,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeMipmappedArray_v5000 = 194,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToMipmappedArray_v5000 = 195,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedMipmappedArray_v5000 = 196,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_v5000 = 197,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithFlags_v5000 = 198,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceViewDesc_v5000 = 199,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetAttribute_v5000 = 200,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v5050 = 201,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithPriority_v5050 = 202,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_v5050 = 203,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_v5050 = 204,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetStreamPriorityRange_v5050 = 205,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocManaged_v6000 = 206,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6000 = 207,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_v6000 = 208,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorName_v6050 = 209,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6050 = 210,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_v7000 = 211,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceFlags_v7000 = 212,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_ptsz_v7000 = 213,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_ptsz_v7000 = 214,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_ptds_v7000 = 215,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_ptds_v7000 = 216,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_ptds_v7000 = 217,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_ptds_v7000 = 218,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_ptds_v7000 = 219,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_ptds_v7000 = 220,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_ptds_v7000 = 221,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_ptds_v7000 = 222,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_ptds_v7000 = 223,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_ptds_v7000 = 224,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_ptsz_v7000 = 225,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_ptsz_v7000 = 226,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_ptsz_v7000 = 227,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_ptsz_v7000 = 228,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_ptsz_v7000 = 229,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_ptsz_v7000 = 230,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_ptsz_v7000 = 231,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_ptsz_v7000 = 232,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_ptds_v7000 = 233,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_ptds_v7000 = 234,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_ptsz_v7000 = 235,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_ptsz_v7000 = 236,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_ptsz_v7000 = 237,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_ptsz_v7000 = 238,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_ptsz_v7000 = 239,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_ptsz_v7000 = 240,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_ptsz_v7000 = 241,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_ptsz_v7000 = 242,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_ptds_v7000 = 243,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_ptsz_v7000 = 244,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_ptds_v7000 = 245,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_ptsz_v7000 = 246,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_ptsz_v7000 = 247,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_ptsz_v7000 = 248,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_ptds_v7000 = 249,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_ptsz_v7000 = 250,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessorWithFlags_v7000 = 251,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v8000 = 252,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_ptsz_v8000 = 253,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemAdvise_v8000 = 254,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetP2PAttribute_v8000 = 255,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsEGLRegisterImage_v7000 = 256,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnect_v7000 = 257,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerDisconnect_v7000 = 258,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerAcquireFrame_v7000 = 259,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerReleaseFrame_v7000 = 260,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerConnect_v7000 = 261,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerDisconnect_v7000 = 262,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerPresentFrame_v7000 = 263,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerReturnFrame_v7000 = 264,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedEglFrame_v7000 = 265,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttribute_v8000 = 266,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttributes_v8000 = 267,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnectWithFlags_v7000 = 268,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_v9000 = 269,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_ptsz_v9000 = 270,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateFromEGLSync_v9000 = 271,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernelMultiDevice_v9000 = 272,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetAttribute_v9000 = 273,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalMemory_v10000 = 274,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedBuffer_v10000 = 275,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedMipmappedArray_v10000 = 276,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalMemory_v10000 = 277,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalSemaphore_v10000 = 278,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v10000 = 279,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_ptsz_v10000 = 280,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v10000 = 281,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_ptsz_v10000 = 282,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalSemaphore_v10000 = 283,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_v10000 = 284,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_ptsz_v10000 = 285,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphCreate_v10000 = 286,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetParams_v10000 = 287,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetParams_v10000 = 288,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddKernelNode_v10000 = 289,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode_v10000 = 290,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeGetParams_v10000 = 291,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams_v10000 = 292,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemsetNode_v10000 = 293,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeGetParams_v10000 = 294,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeSetParams_v10000 = 295,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddHostNode_v10000 = 296,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeGetParams_v10000 = 297,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddChildGraphNode_v10000 = 298,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphChildGraphNodeGetGraph_v10000 = 299,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEmptyNode_v10000 = 300,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphClone_v10000 = 301,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeFindInClone_v10000 = 302,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetType_v10000 = 303,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetRootNodes_v10000 = 304,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependencies_v10000 = 305,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependentNodes_v10000 = 306,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddDependencies_v10000 = 307,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRemoveDependencies_v10000 = 308,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroyNode_v10000 = 309,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiate_v10000 = 310,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_v10000 = 311,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_ptsz_v10000 = 312,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecDestroy_v10000 = 313,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroy_v10000 = 314,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_v10000 = 315,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_ptsz_v10000 = 316,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_v10000 = 317,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_ptsz_v10000 = 318,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_v10000 = 319,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_ptsz_v10000 = 320,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeSetParams_v10000 = 321,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetNodes_v10000 = 322,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetEdges_v10000 = 323,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v10010 = 324,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_ptsz_v10010 = 325,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecKernelNodeSetParams_v10010 = 326,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExchangeStreamCaptureMode_v10010 = 327,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetNvSciSyncAttributes_v10020 = 328,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyAvailableDynamicSMemPerBlock_v10200 = 329,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_v10200 = 330,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_ptsz_v10200 = 331,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams_v10020 = 332,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemsetNodeSetParams_v10020 = 333,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecHostNodeSetParams_v10020 = 334,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecUpdate_v10020 = 335,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetFuncBySymbol_v11000 = 336,
    CUPTI_RUNTIME_TRACE_CBID_cudaCtxResetPersistingL2Cache_v11000 = 337,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeCopyAttributes_v11000 = 338,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetAttribute_v11000 = 339,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetAttribute_v11000 = 340,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_v11000 = 341,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_ptsz_v11000 = 342,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_v11000 = 343,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_ptsz_v11000 = 344,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_v11000 = 345,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_ptsz_v11000 = 346,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetTexture1DLinearMaxWidth_v11010 = 347,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_v10000 = 348,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_ptsz_v10000 = 349,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeToSymbol_v11010 = 350,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeFromSymbol_v11010 = 351,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode1D_v11010 = 352,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsToSymbol_v11010 = 353,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsFromSymbol_v11010 = 354,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams1D_v11010 = 355,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsToSymbol_v11010 = 356,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsFromSymbol_v11010 = 357,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams1D_v11010 = 358,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetSparseProperties_v11010 = 359,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetSparseProperties_v11010 = 360,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecChildGraphNodeSetParams_v11010 = 361,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventRecordNode_v11010 = 362,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeGetEvent_v11010 = 363,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeSetEvent_v11010 = 364,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventWaitNode_v11010 = 365,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeGetEvent_v11010 = 366,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeSetEvent_v11010 = 367,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventRecordNodeSetEvent_v11010 = 368,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventWaitNodeSetEvent_v11010 = 369,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_v11010 = 370,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_ptsz_v11010 = 371,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetDefaultMemPool_v11020 = 372,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_v11020 = 373,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_ptsz_v11020 = 374,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_v11020 = 375,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_ptsz_v11020 = 376,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolTrimTo_v11020 = 377,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAttribute_v11020 = 378,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAttribute_v11020 = 379,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAccess_v11020 = 380,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetPlane_v11020 = 381,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAccess_v11020 = 382,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolCreate_v11020 = 383,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolDestroy_v11020 = 384,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetMemPool_v11020 = 385,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetMemPool_v11020 = 386,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportToShareableHandle_v11020 = 387,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportFromShareableHandle_v11020 = 388,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportPointer_v11020 = 389,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportPointer_v11020 = 390,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_v11020 = 391,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_ptsz_v11020 = 392,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v2_v11020 = 393,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v2_ptsz_v11020 = 394,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v2_v11020 = 395,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v2_ptsz_v11020 = 396,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresSignalNode_v11020 = 397,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeGetParams_v11020 = 398,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeSetParams_v11020 = 399,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresWaitNode_v11020 = 400,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeGetParams_v11020 = 401,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeSetParams_v11020 = 402,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresSignalNodeSetParams_v11020 = 403,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresWaitNodeSetParams_v11020 = 404,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceFlushGPUDirectRDMAWrites_v11030 = 405,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_v11030 = 406,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_ptsz_v11030 = 407,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDebugDotPrint_v11030 = 408,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_v11030 = 409,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_ptsz_v11030 = 410,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v11030 = 411,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_ptsz_v11030 = 412,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectCreate_v11030 = 413,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRetain_v11030 = 414,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRelease_v11030 = 415,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRetainUserObject_v11030 = 416,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphReleaseUserObject_v11030 = 417,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithFlags_v11040 = 418,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemAllocNode_v11040 = 419,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemAllocNodeGetParams_v11040 = 420,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemFreeNode_v11040 = 421,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemFreeNodeGetParams_v11040 = 422,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGraphMemTrim_v11040 = 423,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetGraphMemAttribute_v11040 = 424,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetGraphMemAttribute_v11040 = 425,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeSetEnabled_v11060 = 426,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetEnabled_v11060 = 427,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetMemoryRequirements_v11060 = 428,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetMemoryRequirements_v11060 = 429,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernelExC_v11060 = 430,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernelExC_ptsz_v11060 = 431,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxPotentialClusterSize_v11070 = 432,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveClusters_v11070 = 433,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateTextureObject_v2_v11080 = 434,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectTextureDesc_v2_v11080 = 435,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithParams_v12000 = 436,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithParams_ptsz_v12000 = 437,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecGetFlags_v12000 = 438,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetKernel_v12000 = 439,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v2_v12000 = 440,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetId_v12000 = 441,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetId_ptsz_v12000 = 442,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiate_v12000 = 443,
    CUPTI_RUNTIME_TRACE_CBID_cudaInitDevice_v12000 = 444,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddNode_v12020 = 445,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeSetParams_v12020 = 446,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecNodeSetParams_v12020 = 447,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemAdvise_v2_v12020 = 448,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v2_v12020 = 449,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v2_ptsz_v12020 = 450,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetName_v12030 = 451,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCaptureToGraph_v12030 = 452,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCaptureToGraph_ptsz_v12030 = 453,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphConditionalHandleCreate_v12030 = 454,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetEdges_v2_v12030 = 455,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependencies_v2_v12030 = 456,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependentNodes_v2_v12030 = 457,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddDependencies_v2_v12030 = 458,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRemoveDependencies_v2_v12030 = 459,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddNode_v2_v12030 = 460,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v3_v12030 = 461,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v3_ptsz_v12030 = 462,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v2_v12030 = 463,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v2_ptsz_v12030 = 464,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceRegisterAsyncNotification_v12040 = 465,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceUnregisterAsyncNotification_v12040 = 466,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetParamInfo_v12040 = 467,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPointByVersion_v12050 = 468,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPointByVersion_ptsz_v12050 = 469,
    CUPTI_RUNTIME_TRACE_CBID_SIZE = 470,
    CUPTI_RUNTIME_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-12060"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_runtime_api_trace_cbid_enum {
    CUPTI_RUNTIME_TRACE_CBID_INVALID = 0,
    CUPTI_RUNTIME_TRACE_CBID_cudaDriverGetVersion_v3020 = 1,
    CUPTI_RUNTIME_TRACE_CBID_cudaRuntimeGetVersion_v3020 = 2,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceCount_v3020 = 3,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v3020 = 4,
    CUPTI_RUNTIME_TRACE_CBID_cudaChooseDevice_v3020 = 5,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetChannelDesc_v3020 = 6,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateChannelDesc_v3020 = 7,
    CUPTI_RUNTIME_TRACE_CBID_cudaConfigureCall_v3020 = 8,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetupArgument_v3020 = 9,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetLastError_v3020 = 10,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeekAtLastError_v3020 = 11,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorString_v3020 = 12,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_v3020 = 13,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetCacheConfig_v3020 = 14,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetAttributes_v3020 = 15,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDevice_v3020 = 16,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDevice_v3020 = 17,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetValidDevices_v3020 = 18,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDeviceFlags_v3020 = 19,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc_v3020 = 20,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocPitch_v3020 = 21,
    CUPTI_RUNTIME_TRACE_CBID_cudaFree_v3020 = 22,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocArray_v3020 = 23,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeArray_v3020 = 24,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocHost_v3020 = 25,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeHost_v3020 = 26,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostAlloc_v3020 = 27,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetDevicePointer_v3020 = 28,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetFlags_v3020 = 29,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemGetInfo_v3020 = 30,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_v3020 = 31,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_v3020 = 32,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_v3020 = 33,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_v3020 = 34,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_v3020 = 35,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_v3020 = 36,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_v3020 = 37,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_v3020 = 38,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_v3020 = 39,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_v3020 = 40,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_v3020 = 41,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_v3020 = 42,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_v3020 = 43,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_v3020 = 44,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_v3020 = 45,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_v3020 = 46,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_v3020 = 47,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_v3020 = 48,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_v3020 = 49,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_v3020 = 50,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_v3020 = 51,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_v3020 = 52,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolAddress_v3020 = 53,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolSize_v3020 = 54,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture_v3020 = 55,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture2D_v3020 = 56,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToArray_v3020 = 57,
    CUPTI_RUNTIME_TRACE_CBID_cudaUnbindTexture_v3020 = 58,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureAlignmentOffset_v3020 = 59,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureReference_v3020 = 60,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindSurfaceToArray_v3020 = 61,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceReference_v3020 = 62,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetGLDevice_v3020 = 63,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLRegisterBufferObject_v3020 = 64,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObject_v3020 = 65,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObject_v3020 = 66,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnregisterBufferObject_v3020 = 67,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetBufferObjectMapFlags_v3020 = 68,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObjectAsync_v3020 = 69,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObjectAsync_v3020 = 70,
    CUPTI_RUNTIME_TRACE_CBID_cudaWGLGetDevice_v3020 = 71,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterImage_v3020 = 72,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterBuffer_v3020 = 73,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnregisterResource_v3020 = 74,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceSetMapFlags_v3020 = 75,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsMapResources_v3020 = 76,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnmapResources_v3020 = 77,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedPointer_v3020 = 78,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsSubResourceGetMappedArray_v3020 = 79,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUGetDevice_v3020 = 80,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUSetVDPAUDevice_v3020 = 81,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterVideoSurface_v3020 = 82,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterOutputSurface_v3020 = 83,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevice_v3020 = 84,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevices_v3020 = 85,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11SetDirect3DDevice_v3020 = 86,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D11RegisterResource_v3020 = 87,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevice_v3020 = 88,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevices_v3020 = 89,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10SetDirect3DDevice_v3020 = 90,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D10RegisterResource_v3020 = 91,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10RegisterResource_v3020 = 92,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnregisterResource_v3020 = 93,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10MapResources_v3020 = 94,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnmapResources_v3020 = 95,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceSetMapFlags_v3020 = 96,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetSurfaceDimensions_v3020 = 97,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedArray_v3020 = 98,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPointer_v3020 = 99,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedSize_v3020 = 100,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPitch_v3020 = 101,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevice_v3020 = 102,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevices_v3020 = 103,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9SetDirect3DDevice_v3020 = 104,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDirect3DDevice_v3020 = 105,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D9RegisterResource_v3020 = 106,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterResource_v3020 = 107,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterResource_v3020 = 108,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapResources_v3020 = 109,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapResources_v3020 = 110,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceSetMapFlags_v3020 = 111,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetSurfaceDimensions_v3020 = 112,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedArray_v3020 = 113,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPointer_v3020 = 114,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedSize_v3020 = 115,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPitch_v3020 = 116,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9Begin_v3020 = 117,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9End_v3020 = 118,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterVertexBuffer_v3020 = 119,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterVertexBuffer_v3020 = 120,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapVertexBuffer_v3020 = 121,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapVertexBuffer_v3020 = 122,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExit_v3020 = 123,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForDevice_v3020 = 124,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForHost_v3020 = 125,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSynchronize_v3020 = 126,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetLimit_v3020 = 127,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetLimit_v3020 = 128,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreate_v3020 = 129,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v3020 = 130,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_v3020 = 131,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_v3020 = 132,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreate_v3020 = 133,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateWithFlags_v3020 = 134,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_v3020 = 135,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventDestroy_v3020 = 136,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventSynchronize_v3020 = 137,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventQuery_v3020 = 138,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventElapsedTime_v3020 = 139,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3D_v3020 = 140,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3DArray_v3020 = 141,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_v3020 = 142,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_v3020 = 143,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_v3020 = 144,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_v3020 = 145,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetCacheConfig_v3020 = 146,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_v3020 = 147,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDirect3DDevice_v3020 = 148,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDirect3DDevice_v3020 = 149,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetCacheConfig_v3020 = 150,
    CUPTI_RUNTIME_TRACE_CBID_cudaPointerGetAttributes_v4000 = 151,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostRegister_v4000 = 152,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostUnregister_v4000 = 153,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceCanAccessPeer_v4000 = 154,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceEnablePeerAccess_v4000 = 155,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceDisablePeerAccess_v4000 = 156,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerRegister_v4000 = 157,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerUnregister_v4000 = 158,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerGetDevicePointer_v4000 = 159,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeer_v4000 = 160,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeerAsync_v4000 = 161,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_v4000 = 162,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_v4000 = 163,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceReset_v3020 = 164,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSynchronize_v3020 = 165,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetLimit_v3020 = 166,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetLimit_v3020 = 167,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetCacheConfig_v3020 = 168,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetCacheConfig_v3020 = 169,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerInitialize_v4000 = 170,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStart_v4000 = 171,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStop_v4000 = 172,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetByPCIBusId_v4010 = 173,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetPCIBusId_v4010 = 174,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLGetDevices_v4010 = 175,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetEventHandle_v4010 = 176,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenEventHandle_v4010 = 177,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetMemHandle_v4010 = 178,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenMemHandle_v4010 = 179,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcCloseMemHandle_v4010 = 180,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetInfo_v4010 = 181,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetSharedMemConfig_v4020 = 182,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetSharedMemConfig_v4020 = 183,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetSharedMemConfig_v4020 = 184,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateTextureObject_v5000 = 185,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyTextureObject_v5000 = 186,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceDesc_v5000 = 187,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectTextureDesc_v5000 = 188,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateSurfaceObject_v5000 = 189,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroySurfaceObject_v5000 = 190,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceObjectResourceDesc_v5000 = 191,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocMipmappedArray_v5000 = 192,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetMipmappedArrayLevel_v5000 = 193,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeMipmappedArray_v5000 = 194,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToMipmappedArray_v5000 = 195,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedMipmappedArray_v5000 = 196,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_v5000 = 197,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithFlags_v5000 = 198,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceViewDesc_v5000 = 199,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetAttribute_v5000 = 200,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v5050 = 201,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithPriority_v5050 = 202,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_v5050 = 203,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_v5050 = 204,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetStreamPriorityRange_v5050 = 205,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocManaged_v6000 = 206,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6000 = 207,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_v6000 = 208,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorName_v6050 = 209,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6050 = 210,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_v7000 = 211,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceFlags_v7000 = 212,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_ptsz_v7000 = 213,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_ptsz_v7000 = 214,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_ptds_v7000 = 215,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_ptds_v7000 = 216,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_ptds_v7000 = 217,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_ptds_v7000 = 218,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_ptds_v7000 = 219,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_ptds_v7000 = 220,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_ptds_v7000 = 221,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_ptds_v7000 = 222,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_ptds_v7000 = 223,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_ptds_v7000 = 224,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_ptsz_v7000 = 225,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_ptsz_v7000 = 226,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_ptsz_v7000 = 227,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_ptsz_v7000 = 228,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_ptsz_v7000 = 229,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_ptsz_v7000 = 230,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_ptsz_v7000 = 231,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_ptsz_v7000 = 232,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_ptds_v7000 = 233,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_ptds_v7000 = 234,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_ptsz_v7000 = 235,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_ptsz_v7000 = 236,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_ptsz_v7000 = 237,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_ptsz_v7000 = 238,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_ptsz_v7000 = 239,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_ptsz_v7000 = 240,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_ptsz_v7000 = 241,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_ptsz_v7000 = 242,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_ptds_v7000 = 243,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_ptsz_v7000 = 244,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_ptds_v7000 = 245,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_ptsz_v7000 = 246,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_ptsz_v7000 = 247,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_ptsz_v7000 = 248,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_ptds_v7000 = 249,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_ptsz_v7000 = 250,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessorWithFlags_v7000 = 251,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v8000 = 252,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_ptsz_v8000 = 253,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemAdvise_v8000 = 254,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetP2PAttribute_v8000 = 255,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsEGLRegisterImage_v7000 = 256,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnect_v7000 = 257,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerDisconnect_v7000 = 258,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerAcquireFrame_v7000 = 259,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerReleaseFrame_v7000 = 260,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerConnect_v7000 = 261,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerDisconnect_v7000 = 262,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerPresentFrame_v7000 = 263,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerReturnFrame_v7000 = 264,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedEglFrame_v7000 = 265,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttribute_v8000 = 266,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttributes_v8000 = 267,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnectWithFlags_v7000 = 268,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_v9000 = 269,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_ptsz_v9000 = 270,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateFromEGLSync_v9000 = 271,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernelMultiDevice_v9000 = 272,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetAttribute_v9000 = 273,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalMemory_v10000 = 274,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedBuffer_v10000 = 275,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedMipmappedArray_v10000 = 276,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalMemory_v10000 = 277,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalSemaphore_v10000 = 278,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v10000 = 279,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_ptsz_v10000 = 280,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v10000 = 281,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_ptsz_v10000 = 282,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalSemaphore_v10000 = 283,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_v10000 = 284,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_ptsz_v10000 = 285,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphCreate_v10000 = 286,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetParams_v10000 = 287,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetParams_v10000 = 288,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddKernelNode_v10000 = 289,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode_v10000 = 290,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeGetParams_v10000 = 291,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams_v10000 = 292,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemsetNode_v10000 = 293,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeGetParams_v10000 = 294,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeSetParams_v10000 = 295,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddHostNode_v10000 = 296,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeGetParams_v10000 = 297,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddChildGraphNode_v10000 = 298,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphChildGraphNodeGetGraph_v10000 = 299,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEmptyNode_v10000 = 300,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphClone_v10000 = 301,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeFindInClone_v10000 = 302,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetType_v10000 = 303,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetRootNodes_v10000 = 304,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependencies_v10000 = 305,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependentNodes_v10000 = 306,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddDependencies_v10000 = 307,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRemoveDependencies_v10000 = 308,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroyNode_v10000 = 309,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiate_v10000 = 310,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_v10000 = 311,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_ptsz_v10000 = 312,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecDestroy_v10000 = 313,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroy_v10000 = 314,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_v10000 = 315,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_ptsz_v10000 = 316,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_v10000 = 317,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_ptsz_v10000 = 318,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_v10000 = 319,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_ptsz_v10000 = 320,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeSetParams_v10000 = 321,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetNodes_v10000 = 322,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetEdges_v10000 = 323,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v10010 = 324,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_ptsz_v10010 = 325,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecKernelNodeSetParams_v10010 = 326,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExchangeStreamCaptureMode_v10010 = 327,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetNvSciSyncAttributes_v10020 = 328,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyAvailableDynamicSMemPerBlock_v10200 = 329,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_v10200 = 330,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_ptsz_v10200 = 331,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams_v10020 = 332,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemsetNodeSetParams_v10020 = 333,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecHostNodeSetParams_v10020 = 334,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecUpdate_v10020 = 335,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetFuncBySymbol_v11000 = 336,
    CUPTI_RUNTIME_TRACE_CBID_cudaCtxResetPersistingL2Cache_v11000 = 337,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeCopyAttributes_v11000 = 338,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetAttribute_v11000 = 339,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetAttribute_v11000 = 340,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_v11000 = 341,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_ptsz_v11000 = 342,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_v11000 = 343,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_ptsz_v11000 = 344,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_v11000 = 345,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_ptsz_v11000 = 346,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetTexture1DLinearMaxWidth_v11010 = 347,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_v10000 = 348,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_ptsz_v10000 = 349,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeToSymbol_v11010 = 350,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeFromSymbol_v11010 = 351,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode1D_v11010 = 352,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsToSymbol_v11010 = 353,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsFromSymbol_v11010 = 354,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams1D_v11010 = 355,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsToSymbol_v11010 = 356,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsFromSymbol_v11010 = 357,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams1D_v11010 = 358,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetSparseProperties_v11010 = 359,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetSparseProperties_v11010 = 360,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecChildGraphNodeSetParams_v11010 = 361,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventRecordNode_v11010 = 362,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeGetEvent_v11010 = 363,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeSetEvent_v11010 = 364,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventWaitNode_v11010 = 365,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeGetEvent_v11010 = 366,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeSetEvent_v11010 = 367,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventRecordNodeSetEvent_v11010 = 368,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventWaitNodeSetEvent_v11010 = 369,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_v11010 = 370,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_ptsz_v11010 = 371,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetDefaultMemPool_v11020 = 372,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_v11020 = 373,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_ptsz_v11020 = 374,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_v11020 = 375,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_ptsz_v11020 = 376,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolTrimTo_v11020 = 377,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAttribute_v11020 = 378,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAttribute_v11020 = 379,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAccess_v11020 = 380,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetPlane_v11020 = 381,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAccess_v11020 = 382,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolCreate_v11020 = 383,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolDestroy_v11020 = 384,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetMemPool_v11020 = 385,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetMemPool_v11020 = 386,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportToShareableHandle_v11020 = 387,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportFromShareableHandle_v11020 = 388,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportPointer_v11020 = 389,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportPointer_v11020 = 390,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_v11020 = 391,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_ptsz_v11020 = 392,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v2_v11020 = 393,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v2_ptsz_v11020 = 394,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v2_v11020 = 395,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v2_ptsz_v11020 = 396,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresSignalNode_v11020 = 397,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeGetParams_v11020 = 398,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeSetParams_v11020 = 399,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresWaitNode_v11020 = 400,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeGetParams_v11020 = 401,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeSetParams_v11020 = 402,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresSignalNodeSetParams_v11020 = 403,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresWaitNodeSetParams_v11020 = 404,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceFlushGPUDirectRDMAWrites_v11030 = 405,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_v11030 = 406,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_ptsz_v11030 = 407,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDebugDotPrint_v11030 = 408,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_v11030 = 409,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_ptsz_v11030 = 410,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v11030 = 411,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_ptsz_v11030 = 412,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectCreate_v11030 = 413,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRetain_v11030 = 414,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRelease_v11030 = 415,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRetainUserObject_v11030 = 416,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphReleaseUserObject_v11030 = 417,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithFlags_v11040 = 418,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemAllocNode_v11040 = 419,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemAllocNodeGetParams_v11040 = 420,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemFreeNode_v11040 = 421,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemFreeNodeGetParams_v11040 = 422,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGraphMemTrim_v11040 = 423,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetGraphMemAttribute_v11040 = 424,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetGraphMemAttribute_v11040 = 425,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeSetEnabled_v11060 = 426,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetEnabled_v11060 = 427,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetMemoryRequirements_v11060 = 428,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetMemoryRequirements_v11060 = 429,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernelExC_v11060 = 430,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernelExC_ptsz_v11060 = 431,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxPotentialClusterSize_v11070 = 432,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveClusters_v11070 = 433,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateTextureObject_v2_v11080 = 434,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectTextureDesc_v2_v11080 = 435,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithParams_v12000 = 436,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithParams_ptsz_v12000 = 437,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecGetFlags_v12000 = 438,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetKernel_v12000 = 439,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v2_v12000 = 440,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetId_v12000 = 441,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetId_ptsz_v12000 = 442,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiate_v12000 = 443,
    CUPTI_RUNTIME_TRACE_CBID_cudaInitDevice_v12000 = 444,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddNode_v12020 = 445,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeSetParams_v12020 = 446,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecNodeSetParams_v12020 = 447,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemAdvise_v2_v12020 = 448,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v2_v12020 = 449,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v2_ptsz_v12020 = 450,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetName_v12030 = 451,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCaptureToGraph_v12030 = 452,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCaptureToGraph_ptsz_v12030 = 453,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphConditionalHandleCreate_v12030 = 454,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetEdges_v2_v12030 = 455,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependencies_v2_v12030 = 456,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependentNodes_v2_v12030 = 457,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddDependencies_v2_v12030 = 458,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRemoveDependencies_v2_v12030 = 459,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddNode_v2_v12030 = 460,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v3_v12030 = 461,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v3_ptsz_v12030 = 462,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v2_v12030 = 463,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v2_ptsz_v12030 = 464,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceRegisterAsyncNotification_v12040 = 465,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceUnregisterAsyncNotification_v12040 = 466,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetParamInfo_v12040 = 467,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPointByVersion_v12050 = 468,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPointByVersion_ptsz_v12050 = 469,
    CUPTI_RUNTIME_TRACE_CBID_cuda470_v12060 = 470,
    CUPTI_RUNTIME_TRACE_CBID_cuda471_v12060 = 471,
    CUPTI_RUNTIME_TRACE_CBID_cuda472_v12060 = 472,
    CUPTI_RUNTIME_TRACE_CBID_cuda473_v12060 = 473,
    CUPTI_RUNTIME_TRACE_CBID_cuda474_v12060 = 474,
    CUPTI_RUNTIME_TRACE_CBID_cuda475_v12060 = 475,
    CUPTI_RUNTIME_TRACE_CBID_cuda476_v12060 = 476,
    CUPTI_RUNTIME_TRACE_CBID_cuda477_v12060 = 477,
    CUPTI_RUNTIME_TRACE_CBID_cuda478_v12060 = 478,
    CUPTI_RUNTIME_TRACE_CBID_cuda479_v12060 = 479,
    CUPTI_RUNTIME_TRACE_CBID_SIZE = 480,
    CUPTI_RUNTIME_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-12080"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_runtime_api_trace_cbid_enum {
    CUPTI_RUNTIME_TRACE_CBID_INVALID = 0,
    CUPTI_RUNTIME_TRACE_CBID_cudaDriverGetVersion_v3020 = 1,
    CUPTI_RUNTIME_TRACE_CBID_cudaRuntimeGetVersion_v3020 = 2,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceCount_v3020 = 3,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v3020 = 4,
    CUPTI_RUNTIME_TRACE_CBID_cudaChooseDevice_v3020 = 5,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetChannelDesc_v3020 = 6,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateChannelDesc_v3020 = 7,
    CUPTI_RUNTIME_TRACE_CBID_cudaConfigureCall_v3020 = 8,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetupArgument_v3020 = 9,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetLastError_v3020 = 10,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeekAtLastError_v3020 = 11,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorString_v3020 = 12,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_v3020 = 13,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetCacheConfig_v3020 = 14,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetAttributes_v3020 = 15,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDevice_v3020 = 16,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDevice_v3020 = 17,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetValidDevices_v3020 = 18,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDeviceFlags_v3020 = 19,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc_v3020 = 20,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocPitch_v3020 = 21,
    CUPTI_RUNTIME_TRACE_CBID_cudaFree_v3020 = 22,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocArray_v3020 = 23,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeArray_v3020 = 24,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocHost_v3020 = 25,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeHost_v3020 = 26,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostAlloc_v3020 = 27,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetDevicePointer_v3020 = 28,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetFlags_v3020 = 29,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemGetInfo_v3020 = 30,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_v3020 = 31,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_v3020 = 32,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_v3020 = 33,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_v3020 = 34,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_v3020 = 35,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_v3020 = 36,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_v3020 = 37,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_v3020 = 38,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_v3020 = 39,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_v3020 = 40,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_v3020 = 41,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_v3020 = 42,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_v3020 = 43,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_v3020 = 44,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_v3020 = 45,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_v3020 = 46,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_v3020 = 47,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_v3020 = 48,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_v3020 = 49,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_v3020 = 50,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_v3020 = 51,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_v3020 = 52,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolAddress_v3020 = 53,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolSize_v3020 = 54,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture_v3020 = 55,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture2D_v3020 = 56,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToArray_v3020 = 57,
    CUPTI_RUNTIME_TRACE_CBID_cudaUnbindTexture_v3020 = 58,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureAlignmentOffset_v3020 = 59,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureReference_v3020 = 60,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindSurfaceToArray_v3020 = 61,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceReference_v3020 = 62,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetGLDevice_v3020 = 63,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLRegisterBufferObject_v3020 = 64,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObject_v3020 = 65,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObject_v3020 = 66,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnregisterBufferObject_v3020 = 67,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetBufferObjectMapFlags_v3020 = 68,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObjectAsync_v3020 = 69,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObjectAsync_v3020 = 70,
    CUPTI_RUNTIME_TRACE_CBID_cudaWGLGetDevice_v3020 = 71,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterImage_v3020 = 72,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterBuffer_v3020 = 73,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnregisterResource_v3020 = 74,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceSetMapFlags_v3020 = 75,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsMapResources_v3020 = 76,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnmapResources_v3020 = 77,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedPointer_v3020 = 78,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsSubResourceGetMappedArray_v3020 = 79,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUGetDevice_v3020 = 80,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUSetVDPAUDevice_v3020 = 81,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterVideoSurface_v3020 = 82,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterOutputSurface_v3020 = 83,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevice_v3020 = 84,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevices_v3020 = 85,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11SetDirect3DDevice_v3020 = 86,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D11RegisterResource_v3020 = 87,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevice_v3020 = 88,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevices_v3020 = 89,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10SetDirect3DDevice_v3020 = 90,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D10RegisterResource_v3020 = 91,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10RegisterResource_v3020 = 92,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnregisterResource_v3020 = 93,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10MapResources_v3020 = 94,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnmapResources_v3020 = 95,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceSetMapFlags_v3020 = 96,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetSurfaceDimensions_v3020 = 97,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedArray_v3020 = 98,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPointer_v3020 = 99,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedSize_v3020 = 100,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPitch_v3020 = 101,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevice_v3020 = 102,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevices_v3020 = 103,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9SetDirect3DDevice_v3020 = 104,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDirect3DDevice_v3020 = 105,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D9RegisterResource_v3020 = 106,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterResource_v3020 = 107,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterResource_v3020 = 108,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapResources_v3020 = 109,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapResources_v3020 = 110,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceSetMapFlags_v3020 = 111,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetSurfaceDimensions_v3020 = 112,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedArray_v3020 = 113,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPointer_v3020 = 114,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedSize_v3020 = 115,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPitch_v3020 = 116,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9Begin_v3020 = 117,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9End_v3020 = 118,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterVertexBuffer_v3020 = 119,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterVertexBuffer_v3020 = 120,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapVertexBuffer_v3020 = 121,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapVertexBuffer_v3020 = 122,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExit_v3020 = 123,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForDevice_v3020 = 124,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForHost_v3020 = 125,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSynchronize_v3020 = 126,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetLimit_v3020 = 127,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetLimit_v3020 = 128,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreate_v3020 = 129,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v3020 = 130,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_v3020 = 131,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_v3020 = 132,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreate_v3020 = 133,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateWithFlags_v3020 = 134,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_v3020 = 135,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventDestroy_v3020 = 136,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventSynchronize_v3020 = 137,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventQuery_v3020 = 138,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventElapsedTime_v3020 = 139,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3D_v3020 = 140,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3DArray_v3020 = 141,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_v3020 = 142,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_v3020 = 143,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_v3020 = 144,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_v3020 = 145,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetCacheConfig_v3020 = 146,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_v3020 = 147,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDirect3DDevice_v3020 = 148,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDirect3DDevice_v3020 = 149,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetCacheConfig_v3020 = 150,
    CUPTI_RUNTIME_TRACE_CBID_cudaPointerGetAttributes_v4000 = 151,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostRegister_v4000 = 152,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostUnregister_v4000 = 153,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceCanAccessPeer_v4000 = 154,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceEnablePeerAccess_v4000 = 155,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceDisablePeerAccess_v4000 = 156,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerRegister_v4000 = 157,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerUnregister_v4000 = 158,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerGetDevicePointer_v4000 = 159,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeer_v4000 = 160,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeerAsync_v4000 = 161,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_v4000 = 162,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_v4000 = 163,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceReset_v3020 = 164,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSynchronize_v3020 = 165,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetLimit_v3020 = 166,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetLimit_v3020 = 167,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetCacheConfig_v3020 = 168,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetCacheConfig_v3020 = 169,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerInitialize_v4000 = 170,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStart_v4000 = 171,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStop_v4000 = 172,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetByPCIBusId_v4010 = 173,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetPCIBusId_v4010 = 174,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLGetDevices_v4010 = 175,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetEventHandle_v4010 = 176,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenEventHandle_v4010 = 177,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetMemHandle_v4010 = 178,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenMemHandle_v4010 = 179,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcCloseMemHandle_v4010 = 180,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetInfo_v4010 = 181,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetSharedMemConfig_v4020 = 182,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetSharedMemConfig_v4020 = 183,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetSharedMemConfig_v4020 = 184,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateTextureObject_v5000 = 185,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyTextureObject_v5000 = 186,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceDesc_v5000 = 187,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectTextureDesc_v5000 = 188,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateSurfaceObject_v5000 = 189,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroySurfaceObject_v5000 = 190,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceObjectResourceDesc_v5000 = 191,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocMipmappedArray_v5000 = 192,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetMipmappedArrayLevel_v5000 = 193,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeMipmappedArray_v5000 = 194,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToMipmappedArray_v5000 = 195,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedMipmappedArray_v5000 = 196,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_v5000 = 197,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithFlags_v5000 = 198,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceViewDesc_v5000 = 199,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetAttribute_v5000 = 200,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v5050 = 201,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithPriority_v5050 = 202,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_v5050 = 203,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_v5050 = 204,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetStreamPriorityRange_v5050 = 205,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocManaged_v6000 = 206,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6000 = 207,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_v6000 = 208,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorName_v6050 = 209,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6050 = 210,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_v7000 = 211,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceFlags_v7000 = 212,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_ptsz_v7000 = 213,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_ptsz_v7000 = 214,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_ptds_v7000 = 215,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_ptds_v7000 = 216,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_ptds_v7000 = 217,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_ptds_v7000 = 218,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_ptds_v7000 = 219,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_ptds_v7000 = 220,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_ptds_v7000 = 221,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_ptds_v7000 = 222,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_ptds_v7000 = 223,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_ptds_v7000 = 224,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_ptsz_v7000 = 225,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_ptsz_v7000 = 226,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_ptsz_v7000 = 227,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_ptsz_v7000 = 228,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_ptsz_v7000 = 229,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_ptsz_v7000 = 230,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_ptsz_v7000 = 231,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_ptsz_v7000 = 232,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_ptds_v7000 = 233,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_ptds_v7000 = 234,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_ptsz_v7000 = 235,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_ptsz_v7000 = 236,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_ptsz_v7000 = 237,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_ptsz_v7000 = 238,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_ptsz_v7000 = 239,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_ptsz_v7000 = 240,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_ptsz_v7000 = 241,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_ptsz_v7000 = 242,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_ptds_v7000 = 243,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_ptsz_v7000 = 244,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_ptds_v7000 = 245,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_ptsz_v7000 = 246,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_ptsz_v7000 = 247,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_ptsz_v7000 = 248,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_ptds_v7000 = 249,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_ptsz_v7000 = 250,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessorWithFlags_v7000 = 251,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v8000 = 252,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_ptsz_v8000 = 253,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemAdvise_v8000 = 254,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetP2PAttribute_v8000 = 255,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsEGLRegisterImage_v7000 = 256,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnect_v7000 = 257,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerDisconnect_v7000 = 258,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerAcquireFrame_v7000 = 259,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerReleaseFrame_v7000 = 260,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerConnect_v7000 = 261,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerDisconnect_v7000 = 262,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerPresentFrame_v7000 = 263,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerReturnFrame_v7000 = 264,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedEglFrame_v7000 = 265,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttribute_v8000 = 266,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttributes_v8000 = 267,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnectWithFlags_v7000 = 268,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_v9000 = 269,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_ptsz_v9000 = 270,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateFromEGLSync_v9000 = 271,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernelMultiDevice_v9000 = 272,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetAttribute_v9000 = 273,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalMemory_v10000 = 274,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedBuffer_v10000 = 275,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedMipmappedArray_v10000 = 276,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalMemory_v10000 = 277,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalSemaphore_v10000 = 278,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v10000 = 279,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_ptsz_v10000 = 280,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v10000 = 281,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_ptsz_v10000 = 282,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalSemaphore_v10000 = 283,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_v10000 = 284,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_ptsz_v10000 = 285,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphCreate_v10000 = 286,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetParams_v10000 = 287,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetParams_v10000 = 288,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddKernelNode_v10000 = 289,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode_v10000 = 290,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeGetParams_v10000 = 291,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams_v10000 = 292,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemsetNode_v10000 = 293,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeGetParams_v10000 = 294,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeSetParams_v10000 = 295,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddHostNode_v10000 = 296,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeGetParams_v10000 = 297,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddChildGraphNode_v10000 = 298,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphChildGraphNodeGetGraph_v10000 = 299,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEmptyNode_v10000 = 300,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphClone_v10000 = 301,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeFindInClone_v10000 = 302,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetType_v10000 = 303,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetRootNodes_v10000 = 304,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependencies_v10000 = 305,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependentNodes_v10000 = 306,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddDependencies_v10000 = 307,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRemoveDependencies_v10000 = 308,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroyNode_v10000 = 309,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiate_v10000 = 310,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_v10000 = 311,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_ptsz_v10000 = 312,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecDestroy_v10000 = 313,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroy_v10000 = 314,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_v10000 = 315,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_ptsz_v10000 = 316,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_v10000 = 317,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_ptsz_v10000 = 318,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_v10000 = 319,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_ptsz_v10000 = 320,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeSetParams_v10000 = 321,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetNodes_v10000 = 322,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetEdges_v10000 = 323,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v10010 = 324,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_ptsz_v10010 = 325,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecKernelNodeSetParams_v10010 = 326,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExchangeStreamCaptureMode_v10010 = 327,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetNvSciSyncAttributes_v10020 = 328,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyAvailableDynamicSMemPerBlock_v10200 = 329,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_v10200 = 330,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_ptsz_v10200 = 331,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams_v10020 = 332,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemsetNodeSetParams_v10020 = 333,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecHostNodeSetParams_v10020 = 334,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecUpdate_v10020 = 335,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetFuncBySymbol_v11000 = 336,
    CUPTI_RUNTIME_TRACE_CBID_cudaCtxResetPersistingL2Cache_v11000 = 337,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeCopyAttributes_v11000 = 338,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetAttribute_v11000 = 339,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetAttribute_v11000 = 340,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_v11000 = 341,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_ptsz_v11000 = 342,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_v11000 = 343,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_ptsz_v11000 = 344,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_v11000 = 345,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_ptsz_v11000 = 346,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetTexture1DLinearMaxWidth_v11010 = 347,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_v10000 = 348,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_ptsz_v10000 = 349,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeToSymbol_v11010 = 350,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeFromSymbol_v11010 = 351,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode1D_v11010 = 352,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsToSymbol_v11010 = 353,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsFromSymbol_v11010 = 354,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams1D_v11010 = 355,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsToSymbol_v11010 = 356,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsFromSymbol_v11010 = 357,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams1D_v11010 = 358,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetSparseProperties_v11010 = 359,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetSparseProperties_v11010 = 360,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecChildGraphNodeSetParams_v11010 = 361,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventRecordNode_v11010 = 362,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeGetEvent_v11010 = 363,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeSetEvent_v11010 = 364,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventWaitNode_v11010 = 365,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeGetEvent_v11010 = 366,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeSetEvent_v11010 = 367,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventRecordNodeSetEvent_v11010 = 368,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventWaitNodeSetEvent_v11010 = 369,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_v11010 = 370,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_ptsz_v11010 = 371,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetDefaultMemPool_v11020 = 372,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_v11020 = 373,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_ptsz_v11020 = 374,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_v11020 = 375,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_ptsz_v11020 = 376,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolTrimTo_v11020 = 377,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAttribute_v11020 = 378,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAttribute_v11020 = 379,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAccess_v11020 = 380,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetPlane_v11020 = 381,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAccess_v11020 = 382,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolCreate_v11020 = 383,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolDestroy_v11020 = 384,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetMemPool_v11020 = 385,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetMemPool_v11020 = 386,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportToShareableHandle_v11020 = 387,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportFromShareableHandle_v11020 = 388,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportPointer_v11020 = 389,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportPointer_v11020 = 390,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_v11020 = 391,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_ptsz_v11020 = 392,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v2_v11020 = 393,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v2_ptsz_v11020 = 394,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v2_v11020 = 395,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v2_ptsz_v11020 = 396,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresSignalNode_v11020 = 397,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeGetParams_v11020 = 398,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeSetParams_v11020 = 399,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresWaitNode_v11020 = 400,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeGetParams_v11020 = 401,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeSetParams_v11020 = 402,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresSignalNodeSetParams_v11020 = 403,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresWaitNodeSetParams_v11020 = 404,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceFlushGPUDirectRDMAWrites_v11030 = 405,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_v11030 = 406,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_ptsz_v11030 = 407,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDebugDotPrint_v11030 = 408,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_v11030 = 409,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_ptsz_v11030 = 410,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v11030 = 411,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_ptsz_v11030 = 412,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectCreate_v11030 = 413,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRetain_v11030 = 414,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRelease_v11030 = 415,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRetainUserObject_v11030 = 416,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphReleaseUserObject_v11030 = 417,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithFlags_v11040 = 418,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemAllocNode_v11040 = 419,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemAllocNodeGetParams_v11040 = 420,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemFreeNode_v11040 = 421,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemFreeNodeGetParams_v11040 = 422,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGraphMemTrim_v11040 = 423,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetGraphMemAttribute_v11040 = 424,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetGraphMemAttribute_v11040 = 425,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeSetEnabled_v11060 = 426,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetEnabled_v11060 = 427,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetMemoryRequirements_v11060 = 428,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetMemoryRequirements_v11060 = 429,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernelExC_v11060 = 430,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernelExC_ptsz_v11060 = 431,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxPotentialClusterSize_v11070 = 432,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveClusters_v11070 = 433,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateTextureObject_v2_v11080 = 434,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectTextureDesc_v2_v11080 = 435,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithParams_v12000 = 436,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithParams_ptsz_v12000 = 437,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecGetFlags_v12000 = 438,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetKernel_v12000 = 439,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v2_v12000 = 440,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetId_v12000 = 441,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetId_ptsz_v12000 = 442,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiate_v12000 = 443,
    CUPTI_RUNTIME_TRACE_CBID_cudaInitDevice_v12000 = 444,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddNode_v12020 = 445,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeSetParams_v12020 = 446,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecNodeSetParams_v12020 = 447,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemAdvise_v2_v12020 = 448,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v2_v12020 = 449,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v2_ptsz_v12020 = 450,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetName_v12030 = 451,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCaptureToGraph_v12030 = 452,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCaptureToGraph_ptsz_v12030 = 453,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphConditionalHandleCreate_v12030 = 454,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetEdges_v2_v12030 = 455,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependencies_v2_v12030 = 456,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependentNodes_v2_v12030 = 457,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddDependencies_v2_v12030 = 458,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRemoveDependencies_v2_v12030 = 459,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddNode_v2_v12030 = 460,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v3_v12030 = 461,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v3_ptsz_v12030 = 462,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v2_v12030 = 463,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v2_ptsz_v12030 = 464,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceRegisterAsyncNotification_v12040 = 465,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceUnregisterAsyncNotification_v12040 = 466,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetParamInfo_v12040 = 467,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPointByVersion_v12050 = 468,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPointByVersion_ptsz_v12050 = 469,
    CUPTI_RUNTIME_TRACE_CBID_cuda470_v12060 = 470,
    CUPTI_RUNTIME_TRACE_CBID_cuda471_v12060 = 471,
    CUPTI_RUNTIME_TRACE_CBID_cuda472_v12060 = 472,
    CUPTI_RUNTIME_TRACE_CBID_cuda473_v12060 = 473,
    CUPTI_RUNTIME_TRACE_CBID_cuda474_v12060 = 474,
    CUPTI_RUNTIME_TRACE_CBID_cuda475_v12060 = 475,
    CUPTI_RUNTIME_TRACE_CBID_cuda476_v12060 = 476,
    CUPTI_RUNTIME_TRACE_CBID_cuda477_v12060 = 477,
    CUPTI_RUNTIME_TRACE_CBID_cuda478_v12060 = 478,
    CUPTI_RUNTIME_TRACE_CBID_cuda479_v12060 = 479,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetDevice_v12080 = 480,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetDevice_ptsz_v12080 = 481,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyBatchAsync_v12080 = 482,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyBatchAsync_ptsz_v12080 = 483,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DBatchAsync_v12080 = 484,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DBatchAsync_ptsz_v12080 = 485,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventElapsedTime_v2_v12080 = 486,
    CUPTI_RUNTIME_TRACE_CBID_SIZE = 487,
    CUPTI_RUNTIME_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-12090"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_runtime_api_trace_cbid_enum {
    CUPTI_RUNTIME_TRACE_CBID_INVALID = 0,
    CUPTI_RUNTIME_TRACE_CBID_cudaDriverGetVersion_v3020 = 1,
    CUPTI_RUNTIME_TRACE_CBID_cudaRuntimeGetVersion_v3020 = 2,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceCount_v3020 = 3,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v3020 = 4,
    CUPTI_RUNTIME_TRACE_CBID_cudaChooseDevice_v3020 = 5,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetChannelDesc_v3020 = 6,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateChannelDesc_v3020 = 7,
    CUPTI_RUNTIME_TRACE_CBID_cudaConfigureCall_v3020 = 8,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetupArgument_v3020 = 9,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetLastError_v3020 = 10,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeekAtLastError_v3020 = 11,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorString_v3020 = 12,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_v3020 = 13,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetCacheConfig_v3020 = 14,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetAttributes_v3020 = 15,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDevice_v3020 = 16,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDevice_v3020 = 17,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetValidDevices_v3020 = 18,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDeviceFlags_v3020 = 19,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc_v3020 = 20,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocPitch_v3020 = 21,
    CUPTI_RUNTIME_TRACE_CBID_cudaFree_v3020 = 22,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocArray_v3020 = 23,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeArray_v3020 = 24,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocHost_v3020 = 25,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeHost_v3020 = 26,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostAlloc_v3020 = 27,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetDevicePointer_v3020 = 28,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetFlags_v3020 = 29,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemGetInfo_v3020 = 30,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_v3020 = 31,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_v3020 = 32,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_v3020 = 33,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_v3020 = 34,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_v3020 = 35,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_v3020 = 36,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_v3020 = 37,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_v3020 = 38,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_v3020 = 39,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_v3020 = 40,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_v3020 = 41,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_v3020 = 42,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_v3020 = 43,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_v3020 = 44,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_v3020 = 45,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_v3020 = 46,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_v3020 = 47,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_v3020 = 48,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_v3020 = 49,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_v3020 = 50,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_v3020 = 51,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_v3020 = 52,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolAddress_v3020 = 53,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolSize_v3020 = 54,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture_v3020 = 55,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture2D_v3020 = 56,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToArray_v3020 = 57,
    CUPTI_RUNTIME_TRACE_CBID_cudaUnbindTexture_v3020 = 58,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureAlignmentOffset_v3020 = 59,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureReference_v3020 = 60,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindSurfaceToArray_v3020 = 61,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceReference_v3020 = 62,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetGLDevice_v3020 = 63,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLRegisterBufferObject_v3020 = 64,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObject_v3020 = 65,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObject_v3020 = 66,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnregisterBufferObject_v3020 = 67,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetBufferObjectMapFlags_v3020 = 68,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObjectAsync_v3020 = 69,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObjectAsync_v3020 = 70,
    CUPTI_RUNTIME_TRACE_CBID_cudaWGLGetDevice_v3020 = 71,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterImage_v3020 = 72,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterBuffer_v3020 = 73,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnregisterResource_v3020 = 74,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceSetMapFlags_v3020 = 75,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsMapResources_v3020 = 76,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnmapResources_v3020 = 77,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedPointer_v3020 = 78,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsSubResourceGetMappedArray_v3020 = 79,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUGetDevice_v3020 = 80,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUSetVDPAUDevice_v3020 = 81,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterVideoSurface_v3020 = 82,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterOutputSurface_v3020 = 83,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevice_v3020 = 84,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevices_v3020 = 85,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11SetDirect3DDevice_v3020 = 86,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D11RegisterResource_v3020 = 87,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevice_v3020 = 88,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevices_v3020 = 89,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10SetDirect3DDevice_v3020 = 90,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D10RegisterResource_v3020 = 91,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10RegisterResource_v3020 = 92,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnregisterResource_v3020 = 93,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10MapResources_v3020 = 94,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnmapResources_v3020 = 95,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceSetMapFlags_v3020 = 96,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetSurfaceDimensions_v3020 = 97,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedArray_v3020 = 98,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPointer_v3020 = 99,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedSize_v3020 = 100,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPitch_v3020 = 101,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevice_v3020 = 102,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevices_v3020 = 103,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9SetDirect3DDevice_v3020 = 104,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDirect3DDevice_v3020 = 105,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D9RegisterResource_v3020 = 106,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterResource_v3020 = 107,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterResource_v3020 = 108,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapResources_v3020 = 109,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapResources_v3020 = 110,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceSetMapFlags_v3020 = 111,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetSurfaceDimensions_v3020 = 112,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedArray_v3020 = 113,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPointer_v3020 = 114,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedSize_v3020 = 115,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPitch_v3020 = 116,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9Begin_v3020 = 117,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9End_v3020 = 118,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterVertexBuffer_v3020 = 119,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterVertexBuffer_v3020 = 120,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapVertexBuffer_v3020 = 121,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapVertexBuffer_v3020 = 122,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExit_v3020 = 123,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForDevice_v3020 = 124,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForHost_v3020 = 125,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSynchronize_v3020 = 126,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetLimit_v3020 = 127,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetLimit_v3020 = 128,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreate_v3020 = 129,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v3020 = 130,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_v3020 = 131,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_v3020 = 132,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreate_v3020 = 133,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateWithFlags_v3020 = 134,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_v3020 = 135,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventDestroy_v3020 = 136,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventSynchronize_v3020 = 137,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventQuery_v3020 = 138,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventElapsedTime_v3020 = 139,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3D_v3020 = 140,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3DArray_v3020 = 141,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_v3020 = 142,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_v3020 = 143,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_v3020 = 144,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_v3020 = 145,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetCacheConfig_v3020 = 146,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_v3020 = 147,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDirect3DDevice_v3020 = 148,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDirect3DDevice_v3020 = 149,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetCacheConfig_v3020 = 150,
    CUPTI_RUNTIME_TRACE_CBID_cudaPointerGetAttributes_v4000 = 151,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostRegister_v4000 = 152,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostUnregister_v4000 = 153,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceCanAccessPeer_v4000 = 154,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceEnablePeerAccess_v4000 = 155,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceDisablePeerAccess_v4000 = 156,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerRegister_v4000 = 157,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerUnregister_v4000 = 158,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerGetDevicePointer_v4000 = 159,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeer_v4000 = 160,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeerAsync_v4000 = 161,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_v4000 = 162,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_v4000 = 163,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceReset_v3020 = 164,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSynchronize_v3020 = 165,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetLimit_v3020 = 166,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetLimit_v3020 = 167,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetCacheConfig_v3020 = 168,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetCacheConfig_v3020 = 169,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerInitialize_v4000 = 170,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStart_v4000 = 171,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStop_v4000 = 172,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetByPCIBusId_v4010 = 173,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetPCIBusId_v4010 = 174,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLGetDevices_v4010 = 175,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetEventHandle_v4010 = 176,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenEventHandle_v4010 = 177,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetMemHandle_v4010 = 178,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenMemHandle_v4010 = 179,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcCloseMemHandle_v4010 = 180,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetInfo_v4010 = 181,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetSharedMemConfig_v4020 = 182,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetSharedMemConfig_v4020 = 183,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetSharedMemConfig_v4020 = 184,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateTextureObject_v5000 = 185,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyTextureObject_v5000 = 186,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceDesc_v5000 = 187,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectTextureDesc_v5000 = 188,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateSurfaceObject_v5000 = 189,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroySurfaceObject_v5000 = 190,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceObjectResourceDesc_v5000 = 191,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocMipmappedArray_v5000 = 192,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetMipmappedArrayLevel_v5000 = 193,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeMipmappedArray_v5000 = 194,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToMipmappedArray_v5000 = 195,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedMipmappedArray_v5000 = 196,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_v5000 = 197,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithFlags_v5000 = 198,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceViewDesc_v5000 = 199,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetAttribute_v5000 = 200,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v5050 = 201,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithPriority_v5050 = 202,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_v5050 = 203,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_v5050 = 204,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetStreamPriorityRange_v5050 = 205,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocManaged_v6000 = 206,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6000 = 207,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_v6000 = 208,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorName_v6050 = 209,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6050 = 210,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_v7000 = 211,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceFlags_v7000 = 212,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_ptsz_v7000 = 213,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_ptsz_v7000 = 214,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_ptds_v7000 = 215,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_ptds_v7000 = 216,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_ptds_v7000 = 217,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_ptds_v7000 = 218,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_ptds_v7000 = 219,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_ptds_v7000 = 220,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_ptds_v7000 = 221,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_ptds_v7000 = 222,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_ptds_v7000 = 223,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_ptds_v7000 = 224,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_ptsz_v7000 = 225,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_ptsz_v7000 = 226,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_ptsz_v7000 = 227,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_ptsz_v7000 = 228,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_ptsz_v7000 = 229,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_ptsz_v7000 = 230,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_ptsz_v7000 = 231,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_ptsz_v7000 = 232,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_ptds_v7000 = 233,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_ptds_v7000 = 234,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_ptsz_v7000 = 235,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_ptsz_v7000 = 236,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_ptsz_v7000 = 237,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_ptsz_v7000 = 238,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_ptsz_v7000 = 239,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_ptsz_v7000 = 240,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_ptsz_v7000 = 241,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_ptsz_v7000 = 242,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_ptds_v7000 = 243,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_ptsz_v7000 = 244,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_ptds_v7000 = 245,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_ptsz_v7000 = 246,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_ptsz_v7000 = 247,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_ptsz_v7000 = 248,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_ptds_v7000 = 249,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_ptsz_v7000 = 250,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessorWithFlags_v7000 = 251,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v8000 = 252,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_ptsz_v8000 = 253,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemAdvise_v8000 = 254,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetP2PAttribute_v8000 = 255,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsEGLRegisterImage_v7000 = 256,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnect_v7000 = 257,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerDisconnect_v7000 = 258,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerAcquireFrame_v7000 = 259,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerReleaseFrame_v7000 = 260,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerConnect_v7000 = 261,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerDisconnect_v7000 = 262,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerPresentFrame_v7000 = 263,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerReturnFrame_v7000 = 264,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedEglFrame_v7000 = 265,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttribute_v8000 = 266,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttributes_v8000 = 267,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnectWithFlags_v7000 = 268,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_v9000 = 269,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_ptsz_v9000 = 270,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateFromEGLSync_v9000 = 271,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernelMultiDevice_v9000 = 272,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetAttribute_v9000 = 273,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalMemory_v10000 = 274,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedBuffer_v10000 = 275,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedMipmappedArray_v10000 = 276,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalMemory_v10000 = 277,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalSemaphore_v10000 = 278,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v10000 = 279,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_ptsz_v10000 = 280,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v10000 = 281,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_ptsz_v10000 = 282,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalSemaphore_v10000 = 283,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_v10000 = 284,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_ptsz_v10000 = 285,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphCreate_v10000 = 286,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetParams_v10000 = 287,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetParams_v10000 = 288,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddKernelNode_v10000 = 289,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode_v10000 = 290,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeGetParams_v10000 = 291,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams_v10000 = 292,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemsetNode_v10000 = 293,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeGetParams_v10000 = 294,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeSetParams_v10000 = 295,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddHostNode_v10000 = 296,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeGetParams_v10000 = 297,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddChildGraphNode_v10000 = 298,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphChildGraphNodeGetGraph_v10000 = 299,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEmptyNode_v10000 = 300,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphClone_v10000 = 301,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeFindInClone_v10000 = 302,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetType_v10000 = 303,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetRootNodes_v10000 = 304,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependencies_v10000 = 305,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependentNodes_v10000 = 306,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddDependencies_v10000 = 307,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRemoveDependencies_v10000 = 308,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroyNode_v10000 = 309,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiate_v10000 = 310,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_v10000 = 311,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_ptsz_v10000 = 312,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecDestroy_v10000 = 313,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroy_v10000 = 314,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_v10000 = 315,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_ptsz_v10000 = 316,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_v10000 = 317,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_ptsz_v10000 = 318,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_v10000 = 319,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_ptsz_v10000 = 320,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeSetParams_v10000 = 321,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetNodes_v10000 = 322,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetEdges_v10000 = 323,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v10010 = 324,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_ptsz_v10010 = 325,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecKernelNodeSetParams_v10010 = 326,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExchangeStreamCaptureMode_v10010 = 327,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetNvSciSyncAttributes_v10020 = 328,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyAvailableDynamicSMemPerBlock_v10200 = 329,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_v10200 = 330,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_ptsz_v10200 = 331,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams_v10020 = 332,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemsetNodeSetParams_v10020 = 333,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecHostNodeSetParams_v10020 = 334,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecUpdate_v10020 = 335,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetFuncBySymbol_v11000 = 336,
    CUPTI_RUNTIME_TRACE_CBID_cudaCtxResetPersistingL2Cache_v11000 = 337,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeCopyAttributes_v11000 = 338,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetAttribute_v11000 = 339,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetAttribute_v11000 = 340,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_v11000 = 341,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_ptsz_v11000 = 342,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_v11000 = 343,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_ptsz_v11000 = 344,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_v11000 = 345,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_ptsz_v11000 = 346,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetTexture1DLinearMaxWidth_v11010 = 347,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_v10000 = 348,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_ptsz_v10000 = 349,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeToSymbol_v11010 = 350,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeFromSymbol_v11010 = 351,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode1D_v11010 = 352,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsToSymbol_v11010 = 353,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsFromSymbol_v11010 = 354,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams1D_v11010 = 355,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsToSymbol_v11010 = 356,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsFromSymbol_v11010 = 357,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams1D_v11010 = 358,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetSparseProperties_v11010 = 359,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetSparseProperties_v11010 = 360,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecChildGraphNodeSetParams_v11010 = 361,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventRecordNode_v11010 = 362,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeGetEvent_v11010 = 363,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeSetEvent_v11010 = 364,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventWaitNode_v11010 = 365,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeGetEvent_v11010 = 366,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeSetEvent_v11010 = 367,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventRecordNodeSetEvent_v11010 = 368,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventWaitNodeSetEvent_v11010 = 369,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_v11010 = 370,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_ptsz_v11010 = 371,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetDefaultMemPool_v11020 = 372,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_v11020 = 373,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_ptsz_v11020 = 374,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_v11020 = 375,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_ptsz_v11020 = 376,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolTrimTo_v11020 = 377,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAttribute_v11020 = 378,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAttribute_v11020 = 379,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAccess_v11020 = 380,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetPlane_v11020 = 381,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAccess_v11020 = 382,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolCreate_v11020 = 383,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolDestroy_v11020 = 384,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetMemPool_v11020 = 385,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetMemPool_v11020 = 386,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportToShareableHandle_v11020 = 387,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportFromShareableHandle_v11020 = 388,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportPointer_v11020 = 389,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportPointer_v11020 = 390,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_v11020 = 391,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_ptsz_v11020 = 392,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v2_v11020 = 393,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v2_ptsz_v11020 = 394,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v2_v11020 = 395,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v2_ptsz_v11020 = 396,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresSignalNode_v11020 = 397,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeGetParams_v11020 = 398,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeSetParams_v11020 = 399,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresWaitNode_v11020 = 400,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeGetParams_v11020 = 401,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeSetParams_v11020 = 402,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresSignalNodeSetParams_v11020 = 403,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresWaitNodeSetParams_v11020 = 404,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceFlushGPUDirectRDMAWrites_v11030 = 405,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_v11030 = 406,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_ptsz_v11030 = 407,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDebugDotPrint_v11030 = 408,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_v11030 = 409,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_ptsz_v11030 = 410,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v11030 = 411,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_ptsz_v11030 = 412,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectCreate_v11030 = 413,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRetain_v11030 = 414,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRelease_v11030 = 415,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRetainUserObject_v11030 = 416,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphReleaseUserObject_v11030 = 417,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithFlags_v11040 = 418,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemAllocNode_v11040 = 419,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemAllocNodeGetParams_v11040 = 420,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemFreeNode_v11040 = 421,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemFreeNodeGetParams_v11040 = 422,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGraphMemTrim_v11040 = 423,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetGraphMemAttribute_v11040 = 424,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetGraphMemAttribute_v11040 = 425,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeSetEnabled_v11060 = 426,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetEnabled_v11060 = 427,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetMemoryRequirements_v11060 = 428,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetMemoryRequirements_v11060 = 429,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernelExC_v11060 = 430,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernelExC_ptsz_v11060 = 431,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxPotentialClusterSize_v11070 = 432,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveClusters_v11070 = 433,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateTextureObject_v2_v11080 = 434,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectTextureDesc_v2_v11080 = 435,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithParams_v12000 = 436,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithParams_ptsz_v12000 = 437,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecGetFlags_v12000 = 438,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetKernel_v12000 = 439,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v2_v12000 = 440,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetId_v12000 = 441,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetId_ptsz_v12000 = 442,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiate_v12000 = 443,
    CUPTI_RUNTIME_TRACE_CBID_cudaInitDevice_v12000 = 444,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddNode_v12020 = 445,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeSetParams_v12020 = 446,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecNodeSetParams_v12020 = 447,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemAdvise_v2_v12020 = 448,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v2_v12020 = 449,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v2_ptsz_v12020 = 450,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetName_v12030 = 451,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCaptureToGraph_v12030 = 452,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCaptureToGraph_ptsz_v12030 = 453,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphConditionalHandleCreate_v12030 = 454,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetEdges_v2_v12030 = 455,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependencies_v2_v12030 = 456,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependentNodes_v2_v12030 = 457,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddDependencies_v2_v12030 = 458,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRemoveDependencies_v2_v12030 = 459,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddNode_v2_v12030 = 460,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v3_v12030 = 461,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v3_ptsz_v12030 = 462,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v2_v12030 = 463,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v2_ptsz_v12030 = 464,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceRegisterAsyncNotification_v12040 = 465,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceUnregisterAsyncNotification_v12040 = 466,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetParamInfo_v12040 = 467,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPointByVersion_v12050 = 468,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPointByVersion_ptsz_v12050 = 469,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryLoadData_v12060 = 470,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryLoadFromFile_v12060 = 471,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryUnload_v12060 = 472,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryGetKernel_v12060 = 473,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryGetGlobal_v12060 = 474,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryGetManaged_v12060 = 475,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryGetUnifiedFunction_v12060 = 476,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryGetKernelCount_v12060 = 477,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryEnumerateKernels_v12060 = 478,
    CUPTI_RUNTIME_TRACE_CBID_cudaKernelSetAttributeForDevice_v12060 = 479,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetDevice_v12080 = 480,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetDevice_ptsz_v12080 = 481,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyBatchAsync_v12080 = 482,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyBatchAsync_ptsz_v12080 = 483,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DBatchAsync_v12080 = 484,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DBatchAsync_ptsz_v12080 = 485,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventElapsedTime_v2_v12080 = 486,
    CUPTI_RUNTIME_TRACE_CBID_SIZE = 487,
    CUPTI_RUNTIME_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-13000"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_runtime_api_trace_cbid_enum {
    CUPTI_RUNTIME_TRACE_CBID_INVALID = 0,
    CUPTI_RUNTIME_TRACE_CBID_cudaDriverGetVersion_v3020 = 1,
    CUPTI_RUNTIME_TRACE_CBID_cudaRuntimeGetVersion_v3020 = 2,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceCount_v3020 = 3,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v3020 = 4,
    CUPTI_RUNTIME_TRACE_CBID_cudaChooseDevice_v3020 = 5,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetChannelDesc_v3020 = 6,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateChannelDesc_v3020 = 7,
    CUPTI_RUNTIME_TRACE_CBID_cudaConfigureCall_v3020 = 8,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetupArgument_v3020 = 9,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetLastError_v3020 = 10,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeekAtLastError_v3020 = 11,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorString_v3020 = 12,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_v3020 = 13,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetCacheConfig_v3020 = 14,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetAttributes_v3020 = 15,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDevice_v3020 = 16,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDevice_v3020 = 17,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetValidDevices_v3020 = 18,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDeviceFlags_v3020 = 19,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc_v3020 = 20,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocPitch_v3020 = 21,
    CUPTI_RUNTIME_TRACE_CBID_cudaFree_v3020 = 22,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocArray_v3020 = 23,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeArray_v3020 = 24,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocHost_v3020 = 25,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeHost_v3020 = 26,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostAlloc_v3020 = 27,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetDevicePointer_v3020 = 28,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetFlags_v3020 = 29,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemGetInfo_v3020 = 30,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_v3020 = 31,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_v3020 = 32,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_v3020 = 33,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_v3020 = 34,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_v3020 = 35,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_v3020 = 36,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_v3020 = 37,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_v3020 = 38,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_v3020 = 39,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_v3020 = 40,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_v3020 = 41,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_v3020 = 42,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_v3020 = 43,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_v3020 = 44,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_v3020 = 45,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_v3020 = 46,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_v3020 = 47,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_v3020 = 48,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_v3020 = 49,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_v3020 = 50,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_v3020 = 51,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_v3020 = 52,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolAddress_v3020 = 53,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolSize_v3020 = 54,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture_v3020 = 55,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture2D_v3020 = 56,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToArray_v3020 = 57,
    CUPTI_RUNTIME_TRACE_CBID_cudaUnbindTexture_v3020 = 58,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureAlignmentOffset_v3020 = 59,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureReference_v3020 = 60,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindSurfaceToArray_v3020 = 61,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceReference_v3020 = 62,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetGLDevice_v3020 = 63,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLRegisterBufferObject_v3020 = 64,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObject_v3020 = 65,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObject_v3020 = 66,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnregisterBufferObject_v3020 = 67,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetBufferObjectMapFlags_v3020 = 68,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObjectAsync_v3020 = 69,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObjectAsync_v3020 = 70,
    CUPTI_RUNTIME_TRACE_CBID_cudaWGLGetDevice_v3020 = 71,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterImage_v3020 = 72,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterBuffer_v3020 = 73,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnregisterResource_v3020 = 74,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceSetMapFlags_v3020 = 75,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsMapResources_v3020 = 76,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnmapResources_v3020 = 77,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedPointer_v3020 = 78,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsSubResourceGetMappedArray_v3020 = 79,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUGetDevice_v3020 = 80,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUSetVDPAUDevice_v3020 = 81,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterVideoSurface_v3020 = 82,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterOutputSurface_v3020 = 83,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevice_v3020 = 84,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevices_v3020 = 85,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11SetDirect3DDevice_v3020 = 86,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D11RegisterResource_v3020 = 87,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevice_v3020 = 88,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevices_v3020 = 89,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10SetDirect3DDevice_v3020 = 90,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D10RegisterResource_v3020 = 91,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10RegisterResource_v3020 = 92,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnregisterResource_v3020 = 93,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10MapResources_v3020 = 94,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnmapResources_v3020 = 95,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceSetMapFlags_v3020 = 96,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetSurfaceDimensions_v3020 = 97,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedArray_v3020 = 98,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPointer_v3020 = 99,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedSize_v3020 = 100,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPitch_v3020 = 101,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevice_v3020 = 102,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevices_v3020 = 103,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9SetDirect3DDevice_v3020 = 104,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDirect3DDevice_v3020 = 105,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D9RegisterResource_v3020 = 106,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterResource_v3020 = 107,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterResource_v3020 = 108,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapResources_v3020 = 109,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapResources_v3020 = 110,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceSetMapFlags_v3020 = 111,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetSurfaceDimensions_v3020 = 112,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedArray_v3020 = 113,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPointer_v3020 = 114,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedSize_v3020 = 115,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPitch_v3020 = 116,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9Begin_v3020 = 117,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9End_v3020 = 118,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterVertexBuffer_v3020 = 119,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterVertexBuffer_v3020 = 120,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapVertexBuffer_v3020 = 121,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapVertexBuffer_v3020 = 122,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExit_v3020 = 123,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForDevice_v3020 = 124,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForHost_v3020 = 125,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSynchronize_v3020 = 126,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetLimit_v3020 = 127,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetLimit_v3020 = 128,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreate_v3020 = 129,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v3020 = 130,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_v3020 = 131,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_v3020 = 132,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreate_v3020 = 133,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateWithFlags_v3020 = 134,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_v3020 = 135,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventDestroy_v3020 = 136,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventSynchronize_v3020 = 137,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventQuery_v3020 = 138,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventElapsedTime_v3020 = 139,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3D_v3020 = 140,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3DArray_v3020 = 141,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_v3020 = 142,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_v3020 = 143,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_v3020 = 144,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_v3020 = 145,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetCacheConfig_v3020 = 146,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_v3020 = 147,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDirect3DDevice_v3020 = 148,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDirect3DDevice_v3020 = 149,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetCacheConfig_v3020 = 150,
    CUPTI_RUNTIME_TRACE_CBID_cudaPointerGetAttributes_v4000 = 151,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostRegister_v4000 = 152,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostUnregister_v4000 = 153,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceCanAccessPeer_v4000 = 154,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceEnablePeerAccess_v4000 = 155,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceDisablePeerAccess_v4000 = 156,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerRegister_v4000 = 157,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerUnregister_v4000 = 158,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerGetDevicePointer_v4000 = 159,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeer_v4000 = 160,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeerAsync_v4000 = 161,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_v4000 = 162,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_v4000 = 163,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceReset_v3020 = 164,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSynchronize_v3020 = 165,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetLimit_v3020 = 166,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetLimit_v3020 = 167,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetCacheConfig_v3020 = 168,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetCacheConfig_v3020 = 169,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerInitialize_v4000 = 170,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStart_v4000 = 171,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStop_v4000 = 172,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetByPCIBusId_v4010 = 173,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetPCIBusId_v4010 = 174,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLGetDevices_v4010 = 175,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetEventHandle_v4010 = 176,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenEventHandle_v4010 = 177,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetMemHandle_v4010 = 178,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenMemHandle_v4010 = 179,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcCloseMemHandle_v4010 = 180,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetInfo_v4010 = 181,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetSharedMemConfig_v4020 = 182,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetSharedMemConfig_v4020 = 183,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetSharedMemConfig_v4020 = 184,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateTextureObject_v5000 = 185,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyTextureObject_v5000 = 186,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceDesc_v5000 = 187,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectTextureDesc_v5000 = 188,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateSurfaceObject_v5000 = 189,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroySurfaceObject_v5000 = 190,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceObjectResourceDesc_v5000 = 191,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocMipmappedArray_v5000 = 192,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetMipmappedArrayLevel_v5000 = 193,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeMipmappedArray_v5000 = 194,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToMipmappedArray_v5000 = 195,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedMipmappedArray_v5000 = 196,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_v5000 = 197,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithFlags_v5000 = 198,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceViewDesc_v5000 = 199,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetAttribute_v5000 = 200,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v5050 = 201,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithPriority_v5050 = 202,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_v5050 = 203,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_v5050 = 204,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetStreamPriorityRange_v5050 = 205,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocManaged_v6000 = 206,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6000 = 207,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_v6000 = 208,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorName_v6050 = 209,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6050 = 210,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_v7000 = 211,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceFlags_v7000 = 212,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_ptsz_v7000 = 213,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_ptsz_v7000 = 214,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_ptds_v7000 = 215,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_ptds_v7000 = 216,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_ptds_v7000 = 217,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_ptds_v7000 = 218,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_ptds_v7000 = 219,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_ptds_v7000 = 220,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_ptds_v7000 = 221,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_ptds_v7000 = 222,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_ptds_v7000 = 223,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_ptds_v7000 = 224,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_ptsz_v7000 = 225,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_ptsz_v7000 = 226,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_ptsz_v7000 = 227,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_ptsz_v7000 = 228,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_ptsz_v7000 = 229,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_ptsz_v7000 = 230,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_ptsz_v7000 = 231,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_ptsz_v7000 = 232,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_ptds_v7000 = 233,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_ptds_v7000 = 234,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_ptsz_v7000 = 235,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_ptsz_v7000 = 236,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_ptsz_v7000 = 237,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_ptsz_v7000 = 238,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_ptsz_v7000 = 239,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_ptsz_v7000 = 240,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_ptsz_v7000 = 241,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_ptsz_v7000 = 242,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_ptds_v7000 = 243,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_ptsz_v7000 = 244,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_ptds_v7000 = 245,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_ptsz_v7000 = 246,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_ptsz_v7000 = 247,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_ptsz_v7000 = 248,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_ptds_v7000 = 249,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_ptsz_v7000 = 250,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessorWithFlags_v7000 = 251,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v8000 = 252,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_ptsz_v8000 = 253,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemAdvise_v8000 = 254,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetP2PAttribute_v8000 = 255,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsEGLRegisterImage_v7000 = 256,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnect_v7000 = 257,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerDisconnect_v7000 = 258,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerAcquireFrame_v7000 = 259,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerReleaseFrame_v7000 = 260,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerConnect_v7000 = 261,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerDisconnect_v7000 = 262,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerPresentFrame_v7000 = 263,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerReturnFrame_v7000 = 264,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedEglFrame_v7000 = 265,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttribute_v8000 = 266,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttributes_v8000 = 267,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnectWithFlags_v7000 = 268,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_v9000 = 269,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_ptsz_v9000 = 270,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateFromEGLSync_v9000 = 271,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernelMultiDevice_v9000 = 272,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetAttribute_v9000 = 273,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalMemory_v10000 = 274,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedBuffer_v10000 = 275,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedMipmappedArray_v10000 = 276,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalMemory_v10000 = 277,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalSemaphore_v10000 = 278,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v10000 = 279,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_ptsz_v10000 = 280,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v10000 = 281,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_ptsz_v10000 = 282,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalSemaphore_v10000 = 283,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_v10000 = 284,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_ptsz_v10000 = 285,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphCreate_v10000 = 286,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetParams_v10000 = 287,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetParams_v10000 = 288,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddKernelNode_v10000 = 289,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode_v10000 = 290,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeGetParams_v10000 = 291,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams_v10000 = 292,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemsetNode_v10000 = 293,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeGetParams_v10000 = 294,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeSetParams_v10000 = 295,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddHostNode_v10000 = 296,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeGetParams_v10000 = 297,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddChildGraphNode_v10000 = 298,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphChildGraphNodeGetGraph_v10000 = 299,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEmptyNode_v10000 = 300,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphClone_v10000 = 301,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeFindInClone_v10000 = 302,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetType_v10000 = 303,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetRootNodes_v10000 = 304,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependencies_v10000 = 305,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependentNodes_v10000 = 306,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddDependencies_v10000 = 307,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRemoveDependencies_v10000 = 308,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroyNode_v10000 = 309,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiate_v10000 = 310,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_v10000 = 311,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_ptsz_v10000 = 312,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecDestroy_v10000 = 313,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroy_v10000 = 314,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_v10000 = 315,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_ptsz_v10000 = 316,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_v10000 = 317,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_ptsz_v10000 = 318,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_v10000 = 319,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_ptsz_v10000 = 320,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeSetParams_v10000 = 321,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetNodes_v10000 = 322,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetEdges_v10000 = 323,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v10010 = 324,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_ptsz_v10010 = 325,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecKernelNodeSetParams_v10010 = 326,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExchangeStreamCaptureMode_v10010 = 327,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetNvSciSyncAttributes_v10020 = 328,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyAvailableDynamicSMemPerBlock_v10200 = 329,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_v10200 = 330,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_ptsz_v10200 = 331,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams_v10020 = 332,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemsetNodeSetParams_v10020 = 333,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecHostNodeSetParams_v10020 = 334,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecUpdate_v10020 = 335,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetFuncBySymbol_v11000 = 336,
    CUPTI_RUNTIME_TRACE_CBID_cudaCtxResetPersistingL2Cache_v11000 = 337,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeCopyAttributes_v11000 = 338,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetAttribute_v11000 = 339,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetAttribute_v11000 = 340,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_v11000 = 341,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_ptsz_v11000 = 342,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_v11000 = 343,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_ptsz_v11000 = 344,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_v11000 = 345,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_ptsz_v11000 = 346,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetTexture1DLinearMaxWidth_v11010 = 347,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_v10000 = 348,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_ptsz_v10000 = 349,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeToSymbol_v11010 = 350,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeFromSymbol_v11010 = 351,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode1D_v11010 = 352,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsToSymbol_v11010 = 353,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsFromSymbol_v11010 = 354,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams1D_v11010 = 355,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsToSymbol_v11010 = 356,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsFromSymbol_v11010 = 357,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams1D_v11010 = 358,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetSparseProperties_v11010 = 359,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetSparseProperties_v11010 = 360,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecChildGraphNodeSetParams_v11010 = 361,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventRecordNode_v11010 = 362,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeGetEvent_v11010 = 363,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeSetEvent_v11010 = 364,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventWaitNode_v11010 = 365,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeGetEvent_v11010 = 366,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeSetEvent_v11010 = 367,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventRecordNodeSetEvent_v11010 = 368,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventWaitNodeSetEvent_v11010 = 369,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_v11010 = 370,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_ptsz_v11010 = 371,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetDefaultMemPool_v11020 = 372,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_v11020 = 373,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_ptsz_v11020 = 374,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_v11020 = 375,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_ptsz_v11020 = 376,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolTrimTo_v11020 = 377,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAttribute_v11020 = 378,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAttribute_v11020 = 379,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAccess_v11020 = 380,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetPlane_v11020 = 381,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAccess_v11020 = 382,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolCreate_v11020 = 383,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolDestroy_v11020 = 384,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetMemPool_v11020 = 385,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetMemPool_v11020 = 386,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportToShareableHandle_v11020 = 387,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportFromShareableHandle_v11020 = 388,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportPointer_v11020 = 389,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportPointer_v11020 = 390,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_v11020 = 391,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_ptsz_v11020 = 392,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v11020 = 393,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_ptsz_v11020 = 394,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v11020 = 395,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_ptsz_v11020 = 396,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresSignalNode_v11020 = 397,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeGetParams_v11020 = 398,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeSetParams_v11020 = 399,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresWaitNode_v11020 = 400,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeGetParams_v11020 = 401,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeSetParams_v11020 = 402,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresSignalNodeSetParams_v11020 = 403,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresWaitNodeSetParams_v11020 = 404,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceFlushGPUDirectRDMAWrites_v11030 = 405,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_v11030 = 406,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_ptsz_v11030 = 407,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDebugDotPrint_v11030 = 408,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_v11030 = 409,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_ptsz_v11030 = 410,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v11030 = 411,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_ptsz_v11030 = 412,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectCreate_v11030 = 413,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRetain_v11030 = 414,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRelease_v11030 = 415,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRetainUserObject_v11030 = 416,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphReleaseUserObject_v11030 = 417,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithFlags_v11040 = 418,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemAllocNode_v11040 = 419,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemAllocNodeGetParams_v11040 = 420,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemFreeNode_v11040 = 421,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemFreeNodeGetParams_v11040 = 422,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGraphMemTrim_v11040 = 423,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetGraphMemAttribute_v11040 = 424,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetGraphMemAttribute_v11040 = 425,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeSetEnabled_v11060 = 426,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetEnabled_v11060 = 427,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetMemoryRequirements_v11060 = 428,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetMemoryRequirements_v11060 = 429,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernelExC_v11060 = 430,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernelExC_ptsz_v11060 = 431,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxPotentialClusterSize_v11070 = 432,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveClusters_v11070 = 433,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateTextureObject_v2_v11080 = 434,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectTextureDesc_v2_v11080 = 435,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithParams_v12000 = 436,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithParams_ptsz_v12000 = 437,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecGetFlags_v12000 = 438,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetKernel_v12000 = 439,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v12000 = 440,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetId_v12000 = 441,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetId_ptsz_v12000 = 442,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiate_v12000 = 443,
    CUPTI_RUNTIME_TRACE_CBID_cudaInitDevice_v12000 = 444,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddNode_v12020 = 445,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeSetParams_v12020 = 446,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecNodeSetParams_v12020 = 447,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemAdvise_v12020 = 448,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v12020 = 449,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_ptsz_v12020 = 450,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetName_v12030 = 451,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCaptureToGraph_v12030 = 452,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCaptureToGraph_ptsz_v12030 = 453,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphConditionalHandleCreate_v12030 = 454,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetEdges_v12030 = 455,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependencies_v12030 = 456,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependentNodes_v12030 = 457,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddDependencies_v12030 = 458,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRemoveDependencies_v12030 = 459,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddNode_v12030 = 460,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v12030 = 461,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_ptsz_v12030 = 462,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v12030 = 463,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_ptsz_v12030 = 464,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceRegisterAsyncNotification_v12040 = 465,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceUnregisterAsyncNotification_v12040 = 466,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetParamInfo_v12040 = 467,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPointByVersion_v12050 = 468,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPointByVersion_ptsz_v12050 = 469,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryLoadData_v12060 = 470,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryLoadFromFile_v12060 = 471,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryUnload_v12060 = 472,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryGetKernel_v12060 = 473,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryGetGlobal_v12060 = 474,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryGetManaged_v12060 = 475,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryGetUnifiedFunction_v12060 = 476,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryGetKernelCount_v12060 = 477,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryEnumerateKernels_v12060 = 478,
    CUPTI_RUNTIME_TRACE_CBID_cudaKernelSetAttributeForDevice_v12060 = 479,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetDevice_v12080 = 480,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetDevice_ptsz_v12080 = 481,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyBatchAsync_v12080 = 482,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyBatchAsync_ptsz_v12080 = 483,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DBatchAsync_v12080 = 484,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DBatchAsync_ptsz_v12080 = 485,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventElapsedTime_v12080 = 486,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchBatchAsync_v13000 = 487,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchBatchAsync_ptsz_v13000 = 488,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemDiscardBatchAsync_v13000 = 489,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemDiscardBatchAsync_ptsz_v13000 = 490,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemDiscardAndPrefetchBatchAsync_v13000 = 491,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemDiscardAndPrefetchBatchAsync_ptsz_v13000 = 492,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetExportTable_v13000 = 493,
    CUPTI_RUNTIME_TRACE_CBID___cudaRegisterFatBinary_v13000 = 494,
    CUPTI_RUNTIME_TRACE_CBID___cudaRegisterFatBinaryEnd_v13000 = 495,
    CUPTI_RUNTIME_TRACE_CBID___cudaUnregisterFatBinary_v13000 = 496,
    CUPTI_RUNTIME_TRACE_CBID___cudaRegisterVar_v13000 = 497,
    CUPTI_RUNTIME_TRACE_CBID___cudaRegisterHostVar_v13000 = 498,
    CUPTI_RUNTIME_TRACE_CBID___cudaRegisterFunction_v13000 = 499,
    CUPTI_RUNTIME_TRACE_CBID___cudaRegisterManagedVar_v13000 = 500,
    CUPTI_RUNTIME_TRACE_CBID___cudaInitModule_v13000 = 501,
    CUPTI_RUNTIME_TRACE_CBID___cudaPushCallConfiguration_v13000 = 502,
    CUPTI_RUNTIME_TRACE_CBID___cudaPopCallConfiguration_v13000 = 503,
    CUPTI_RUNTIME_TRACE_CBID___cudaGetKernel_v13000 = 504,
    CUPTI_RUNTIME_TRACE_CBID___cudaLaunchKernel_v13000 = 505,
    CUPTI_RUNTIME_TRACE_CBID___cudaLaunchKernel_ptsz_v13000 = 506,
    CUPTI_RUNTIME_TRACE_CBID___cudaRegisterUnifiedTable_v13000 = 507,
    CUPTI_RUNTIME_TRACE_CBID___cudaGetProcAddress_v13000 = 508,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyBatchAsync_v13000 = 509,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyBatchAsync_ptsz_v13000 = 510,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DBatchAsync_v13000 = 511,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DBatchAsync_ptsz_v13000 = 512,
    CUPTI_RUNTIME_TRACE_CBID_cudaLogsRegisterCallback_v13000 = 513,
    CUPTI_RUNTIME_TRACE_CBID_cudaLogsUnregisterCallback_v13000 = 514,
    CUPTI_RUNTIME_TRACE_CBID_cudaLogsCurrent_v13000 = 515,
    CUPTI_RUNTIME_TRACE_CBID_cudaLogsDumpToFile_v13000 = 516,
    CUPTI_RUNTIME_TRACE_CBID_cudaLogsDumpToMemory_v13000 = 517,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemGetDefaultMemPool_v13000 = 518,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemGetMemPool_v13000 = 519,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemSetMemPool_v13000 = 520,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetHostAtomicCapabilities_v13000 = 521,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetP2PAtomicCapabilities_v13000 = 522,
    CUPTI_RUNTIME_TRACE_CBID_SIZE = 523,
    CUPTI_RUNTIME_TRACE_CBID_FORCE_INT = 2147483647,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(u32)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub enum CUpti_runtime_api_trace_cbid_enum {
    CUPTI_RUNTIME_TRACE_CBID_INVALID = 0,
    CUPTI_RUNTIME_TRACE_CBID_cudaDriverGetVersion_v3020 = 1,
    CUPTI_RUNTIME_TRACE_CBID_cudaRuntimeGetVersion_v3020 = 2,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceCount_v3020 = 3,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v3020 = 4,
    CUPTI_RUNTIME_TRACE_CBID_cudaChooseDevice_v3020 = 5,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetChannelDesc_v3020 = 6,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateChannelDesc_v3020 = 7,
    CUPTI_RUNTIME_TRACE_CBID_cudaConfigureCall_v3020 = 8,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetupArgument_v3020 = 9,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetLastError_v3020 = 10,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeekAtLastError_v3020 = 11,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorString_v3020 = 12,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_v3020 = 13,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetCacheConfig_v3020 = 14,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetAttributes_v3020 = 15,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDevice_v3020 = 16,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDevice_v3020 = 17,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetValidDevices_v3020 = 18,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDeviceFlags_v3020 = 19,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc_v3020 = 20,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocPitch_v3020 = 21,
    CUPTI_RUNTIME_TRACE_CBID_cudaFree_v3020 = 22,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocArray_v3020 = 23,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeArray_v3020 = 24,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocHost_v3020 = 25,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeHost_v3020 = 26,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostAlloc_v3020 = 27,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetDevicePointer_v3020 = 28,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostGetFlags_v3020 = 29,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemGetInfo_v3020 = 30,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_v3020 = 31,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_v3020 = 32,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_v3020 = 33,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_v3020 = 34,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_v3020 = 35,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_v3020 = 36,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_v3020 = 37,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_v3020 = 38,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_v3020 = 39,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_v3020 = 40,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_v3020 = 41,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_v3020 = 42,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_v3020 = 43,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_v3020 = 44,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_v3020 = 45,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_v3020 = 46,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_v3020 = 47,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_v3020 = 48,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_v3020 = 49,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_v3020 = 50,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_v3020 = 51,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_v3020 = 52,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolAddress_v3020 = 53,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSymbolSize_v3020 = 54,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture_v3020 = 55,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTexture2D_v3020 = 56,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToArray_v3020 = 57,
    CUPTI_RUNTIME_TRACE_CBID_cudaUnbindTexture_v3020 = 58,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureAlignmentOffset_v3020 = 59,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureReference_v3020 = 60,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindSurfaceToArray_v3020 = 61,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceReference_v3020 = 62,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetGLDevice_v3020 = 63,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLRegisterBufferObject_v3020 = 64,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObject_v3020 = 65,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObject_v3020 = 66,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnregisterBufferObject_v3020 = 67,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLSetBufferObjectMapFlags_v3020 = 68,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLMapBufferObjectAsync_v3020 = 69,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLUnmapBufferObjectAsync_v3020 = 70,
    CUPTI_RUNTIME_TRACE_CBID_cudaWGLGetDevice_v3020 = 71,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterImage_v3020 = 72,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsGLRegisterBuffer_v3020 = 73,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnregisterResource_v3020 = 74,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceSetMapFlags_v3020 = 75,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsMapResources_v3020 = 76,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsUnmapResources_v3020 = 77,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedPointer_v3020 = 78,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsSubResourceGetMappedArray_v3020 = 79,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUGetDevice_v3020 = 80,
    CUPTI_RUNTIME_TRACE_CBID_cudaVDPAUSetVDPAUDevice_v3020 = 81,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterVideoSurface_v3020 = 82,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsVDPAURegisterOutputSurface_v3020 = 83,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevice_v3020 = 84,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDevices_v3020 = 85,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11SetDirect3DDevice_v3020 = 86,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D11RegisterResource_v3020 = 87,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevice_v3020 = 88,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDevices_v3020 = 89,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10SetDirect3DDevice_v3020 = 90,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D10RegisterResource_v3020 = 91,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10RegisterResource_v3020 = 92,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnregisterResource_v3020 = 93,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10MapResources_v3020 = 94,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10UnmapResources_v3020 = 95,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceSetMapFlags_v3020 = 96,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetSurfaceDimensions_v3020 = 97,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedArray_v3020 = 98,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPointer_v3020 = 99,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedSize_v3020 = 100,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10ResourceGetMappedPitch_v3020 = 101,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevice_v3020 = 102,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDevices_v3020 = 103,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9SetDirect3DDevice_v3020 = 104,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9GetDirect3DDevice_v3020 = 105,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsD3D9RegisterResource_v3020 = 106,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterResource_v3020 = 107,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterResource_v3020 = 108,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapResources_v3020 = 109,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapResources_v3020 = 110,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceSetMapFlags_v3020 = 111,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetSurfaceDimensions_v3020 = 112,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedArray_v3020 = 113,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPointer_v3020 = 114,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedSize_v3020 = 115,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9ResourceGetMappedPitch_v3020 = 116,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9Begin_v3020 = 117,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9End_v3020 = 118,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9RegisterVertexBuffer_v3020 = 119,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnregisterVertexBuffer_v3020 = 120,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9MapVertexBuffer_v3020 = 121,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D9UnmapVertexBuffer_v3020 = 122,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExit_v3020 = 123,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForDevice_v3020 = 124,
    CUPTI_RUNTIME_TRACE_CBID_cudaSetDoubleForHost_v3020 = 125,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSynchronize_v3020 = 126,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetLimit_v3020 = 127,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetLimit_v3020 = 128,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreate_v3020 = 129,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v3020 = 130,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_v3020 = 131,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_v3020 = 132,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreate_v3020 = 133,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateWithFlags_v3020 = 134,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_v3020 = 135,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventDestroy_v3020 = 136,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventSynchronize_v3020 = 137,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventQuery_v3020 = 138,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventElapsedTime_v3020 = 139,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3D_v3020 = 140,
    CUPTI_RUNTIME_TRACE_CBID_cudaMalloc3DArray_v3020 = 141,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_v3020 = 142,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_v3020 = 143,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_v3020 = 144,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_v3020 = 145,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadSetCacheConfig_v3020 = 146,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_v3020 = 147,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D11GetDirect3DDevice_v3020 = 148,
    CUPTI_RUNTIME_TRACE_CBID_cudaD3D10GetDirect3DDevice_v3020 = 149,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadGetCacheConfig_v3020 = 150,
    CUPTI_RUNTIME_TRACE_CBID_cudaPointerGetAttributes_v4000 = 151,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostRegister_v4000 = 152,
    CUPTI_RUNTIME_TRACE_CBID_cudaHostUnregister_v4000 = 153,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceCanAccessPeer_v4000 = 154,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceEnablePeerAccess_v4000 = 155,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceDisablePeerAccess_v4000 = 156,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerRegister_v4000 = 157,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerUnregister_v4000 = 158,
    CUPTI_RUNTIME_TRACE_CBID_cudaPeerGetDevicePointer_v4000 = 159,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeer_v4000 = 160,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyPeerAsync_v4000 = 161,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_v4000 = 162,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_v4000 = 163,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceReset_v3020 = 164,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSynchronize_v3020 = 165,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetLimit_v3020 = 166,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetLimit_v3020 = 167,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetCacheConfig_v3020 = 168,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetCacheConfig_v3020 = 169,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerInitialize_v4000 = 170,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStart_v4000 = 171,
    CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStop_v4000 = 172,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetByPCIBusId_v4010 = 173,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetPCIBusId_v4010 = 174,
    CUPTI_RUNTIME_TRACE_CBID_cudaGLGetDevices_v4010 = 175,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetEventHandle_v4010 = 176,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenEventHandle_v4010 = 177,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcGetMemHandle_v4010 = 178,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcOpenMemHandle_v4010 = 179,
    CUPTI_RUNTIME_TRACE_CBID_cudaIpcCloseMemHandle_v4010 = 180,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetInfo_v4010 = 181,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetSharedMemConfig_v4020 = 182,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetSharedMemConfig_v4020 = 183,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetSharedMemConfig_v4020 = 184,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateTextureObject_v5000 = 185,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyTextureObject_v5000 = 186,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceDesc_v5000 = 187,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectTextureDesc_v5000 = 188,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateSurfaceObject_v5000 = 189,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroySurfaceObject_v5000 = 190,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetSurfaceObjectResourceDesc_v5000 = 191,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocMipmappedArray_v5000 = 192,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetMipmappedArrayLevel_v5000 = 193,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeMipmappedArray_v5000 = 194,
    CUPTI_RUNTIME_TRACE_CBID_cudaBindTextureToMipmappedArray_v5000 = 195,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedMipmappedArray_v5000 = 196,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_v5000 = 197,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithFlags_v5000 = 198,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectResourceViewDesc_v5000 = 199,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetAttribute_v5000 = 200,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamDestroy_v5050 = 201,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCreateWithPriority_v5050 = 202,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_v5050 = 203,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_v5050 = 204,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetStreamPriorityRange_v5050 = 205,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocManaged_v6000 = 206,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6000 = 207,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_v6000 = 208,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetErrorName_v6050 = 209,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6050 = 210,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_v7000 = 211,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceFlags_v7000 = 212,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_ptsz_v7000 = 213,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernel_ptsz_v7000 = 214,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy_ptds_v7000 = 215,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2D_ptds_v7000 = 216,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArray_ptds_v7000 = 217,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArray_ptds_v7000 = 218,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArray_ptds_v7000 = 219,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArray_ptds_v7000 = 220,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyArrayToArray_ptds_v7000 = 221,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DArrayToArray_ptds_v7000 = 222,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbol_ptds_v7000 = 223,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbol_ptds_v7000 = 224,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyAsync_ptsz_v7000 = 225,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToArrayAsync_ptsz_v7000 = 226,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromArrayAsync_ptsz_v7000 = 227,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DAsync_ptsz_v7000 = 228,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DToArrayAsync_ptsz_v7000 = 229,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy2DFromArrayAsync_ptsz_v7000 = 230,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyToSymbolAsync_ptsz_v7000 = 231,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyFromSymbolAsync_ptsz_v7000 = 232,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset_ptds_v7000 = 233,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2D_ptds_v7000 = 234,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemsetAsync_ptsz_v7000 = 235,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset2DAsync_ptsz_v7000 = 236,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetPriority_ptsz_v7000 = 237,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetFlags_ptsz_v7000 = 238,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSynchronize_ptsz_v7000 = 239,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamQuery_ptsz_v7000 = 240,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAttachMemAsync_ptsz_v7000 = 241,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecord_ptsz_v7000 = 242,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3D_ptds_v7000 = 243,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemset3DAsync_ptsz_v7000 = 244,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3D_ptds_v7000 = 245,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DAsync_ptsz_v7000 = 246,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamWaitEvent_ptsz_v7000 = 247,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamAddCallback_ptsz_v7000 = 248,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeer_ptds_v7000 = 249,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DPeerAsync_ptsz_v7000 = 250,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveBlocksPerMultiprocessorWithFlags_v7000 = 251,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v8000 = 252,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_ptsz_v8000 = 253,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemAdvise_v8000 = 254,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetP2PAttribute_v8000 = 255,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsEGLRegisterImage_v7000 = 256,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnect_v7000 = 257,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerDisconnect_v7000 = 258,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerAcquireFrame_v7000 = 259,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerReleaseFrame_v7000 = 260,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerConnect_v7000 = 261,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerDisconnect_v7000 = 262,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerPresentFrame_v7000 = 263,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamProducerReturnFrame_v7000 = 264,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphicsResourceGetMappedEglFrame_v7000 = 265,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttribute_v8000 = 266,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemRangeGetAttributes_v8000 = 267,
    CUPTI_RUNTIME_TRACE_CBID_cudaEGLStreamConsumerConnectWithFlags_v7000 = 268,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_v9000 = 269,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernel_ptsz_v9000 = 270,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateFromEGLSync_v9000 = 271,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchCooperativeKernelMultiDevice_v9000 = 272,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncSetAttribute_v9000 = 273,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalMemory_v10000 = 274,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedBuffer_v10000 = 275,
    CUPTI_RUNTIME_TRACE_CBID_cudaExternalMemoryGetMappedMipmappedArray_v10000 = 276,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalMemory_v10000 = 277,
    CUPTI_RUNTIME_TRACE_CBID_cudaImportExternalSemaphore_v10000 = 278,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v10000 = 279,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_ptsz_v10000 = 280,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v10000 = 281,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_ptsz_v10000 = 282,
    CUPTI_RUNTIME_TRACE_CBID_cudaDestroyExternalSemaphore_v10000 = 283,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_v10000 = 284,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchHostFunc_ptsz_v10000 = 285,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphCreate_v10000 = 286,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetParams_v10000 = 287,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetParams_v10000 = 288,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddKernelNode_v10000 = 289,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode_v10000 = 290,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeGetParams_v10000 = 291,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams_v10000 = 292,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemsetNode_v10000 = 293,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeGetParams_v10000 = 294,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemsetNodeSetParams_v10000 = 295,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddHostNode_v10000 = 296,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeGetParams_v10000 = 297,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddChildGraphNode_v10000 = 298,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphChildGraphNodeGetGraph_v10000 = 299,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEmptyNode_v10000 = 300,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphClone_v10000 = 301,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeFindInClone_v10000 = 302,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetType_v10000 = 303,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetRootNodes_v10000 = 304,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependencies_v10000 = 305,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependentNodes_v10000 = 306,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddDependencies_v10000 = 307,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRemoveDependencies_v10000 = 308,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroyNode_v10000 = 309,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiate_v10000 = 310,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_v10000 = 311,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphLaunch_ptsz_v10000 = 312,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecDestroy_v10000 = 313,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDestroy_v10000 = 314,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_v10000 = 315,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCapture_ptsz_v10000 = 316,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_v10000 = 317,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamIsCapturing_ptsz_v10000 = 318,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_v10000 = 319,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamEndCapture_ptsz_v10000 = 320,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphHostNodeSetParams_v10000 = 321,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetNodes_v10000 = 322,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetEdges_v10000 = 323,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v10010 = 324,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_ptsz_v10010 = 325,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecKernelNodeSetParams_v10010 = 326,
    CUPTI_RUNTIME_TRACE_CBID_cudaThreadExchangeStreamCaptureMode_v10010 = 327,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetNvSciSyncAttributes_v10020 = 328,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyAvailableDynamicSMemPerBlock_v10200 = 329,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_v10200 = 330,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetFlags_ptsz_v10200 = 331,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams_v10020 = 332,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemsetNodeSetParams_v10020 = 333,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecHostNodeSetParams_v10020 = 334,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecUpdate_v10020 = 335,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetFuncBySymbol_v11000 = 336,
    CUPTI_RUNTIME_TRACE_CBID_cudaCtxResetPersistingL2Cache_v11000 = 337,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeCopyAttributes_v11000 = 338,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeGetAttribute_v11000 = 339,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphKernelNodeSetAttribute_v11000 = 340,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_v11000 = 341,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamCopyAttributes_ptsz_v11000 = 342,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_v11000 = 343,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetAttribute_ptsz_v11000 = 344,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_v11000 = 345,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamSetAttribute_ptsz_v11000 = 346,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetTexture1DLinearMaxWidth_v11010 = 347,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_v10000 = 348,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphUpload_ptsz_v10000 = 349,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeToSymbol_v11010 = 350,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNodeFromSymbol_v11010 = 351,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemcpyNode1D_v11010 = 352,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsToSymbol_v11010 = 353,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParamsFromSymbol_v11010 = 354,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemcpyNodeSetParams1D_v11010 = 355,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsToSymbol_v11010 = 356,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParamsFromSymbol_v11010 = 357,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecMemcpyNodeSetParams1D_v11010 = 358,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetSparseProperties_v11010 = 359,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetSparseProperties_v11010 = 360,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecChildGraphNodeSetParams_v11010 = 361,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventRecordNode_v11010 = 362,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeGetEvent_v11010 = 363,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventRecordNodeSetEvent_v11010 = 364,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddEventWaitNode_v11010 = 365,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeGetEvent_v11010 = 366,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphEventWaitNodeSetEvent_v11010 = 367,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventRecordNodeSetEvent_v11010 = 368,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecEventWaitNodeSetEvent_v11010 = 369,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_v11010 = 370,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventRecordWithFlags_ptsz_v11010 = 371,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetDefaultMemPool_v11020 = 372,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_v11020 = 373,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocAsync_ptsz_v11020 = 374,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_v11020 = 375,
    CUPTI_RUNTIME_TRACE_CBID_cudaFreeAsync_ptsz_v11020 = 376,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolTrimTo_v11020 = 377,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAttribute_v11020 = 378,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAttribute_v11020 = 379,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolSetAccess_v11020 = 380,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetPlane_v11020 = 381,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolGetAccess_v11020 = 382,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolCreate_v11020 = 383,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolDestroy_v11020 = 384,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetMemPool_v11020 = 385,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetMemPool_v11020 = 386,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportToShareableHandle_v11020 = 387,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportFromShareableHandle_v11020 = 388,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolExportPointer_v11020 = 389,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPoolImportPointer_v11020 = 390,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_v11020 = 391,
    CUPTI_RUNTIME_TRACE_CBID_cudaMallocFromPoolAsync_ptsz_v11020 = 392,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_v11020 = 393,
    CUPTI_RUNTIME_TRACE_CBID_cudaSignalExternalSemaphoresAsync_ptsz_v11020 = 394,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_v11020 = 395,
    CUPTI_RUNTIME_TRACE_CBID_cudaWaitExternalSemaphoresAsync_ptsz_v11020 = 396,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresSignalNode_v11020 = 397,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeGetParams_v11020 = 398,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresSignalNodeSetParams_v11020 = 399,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddExternalSemaphoresWaitNode_v11020 = 400,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeGetParams_v11020 = 401,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExternalSemaphoresWaitNodeSetParams_v11020 = 402,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresSignalNodeSetParams_v11020 = 403,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecExternalSemaphoresWaitNodeSetParams_v11020 = 404,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceFlushGPUDirectRDMAWrites_v11030 = 405,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_v11030 = 406,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPoint_ptsz_v11030 = 407,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphDebugDotPrint_v11030 = 408,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_v11030 = 409,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v2_ptsz_v11030 = 410,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v11030 = 411,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_ptsz_v11030 = 412,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectCreate_v11030 = 413,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRetain_v11030 = 414,
    CUPTI_RUNTIME_TRACE_CBID_cudaUserObjectRelease_v11030 = 415,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRetainUserObject_v11030 = 416,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphReleaseUserObject_v11030 = 417,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithFlags_v11040 = 418,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemAllocNode_v11040 = 419,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemAllocNodeGetParams_v11040 = 420,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddMemFreeNode_v11040 = 421,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphMemFreeNodeGetParams_v11040 = 422,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGraphMemTrim_v11040 = 423,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetGraphMemAttribute_v11040 = 424,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceSetGraphMemAttribute_v11040 = 425,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeSetEnabled_v11060 = 426,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetEnabled_v11060 = 427,
    CUPTI_RUNTIME_TRACE_CBID_cudaArrayGetMemoryRequirements_v11060 = 428,
    CUPTI_RUNTIME_TRACE_CBID_cudaMipmappedArrayGetMemoryRequirements_v11060 = 429,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernelExC_v11060 = 430,
    CUPTI_RUNTIME_TRACE_CBID_cudaLaunchKernelExC_ptsz_v11060 = 431,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxPotentialClusterSize_v11070 = 432,
    CUPTI_RUNTIME_TRACE_CBID_cudaOccupancyMaxActiveClusters_v11070 = 433,
    CUPTI_RUNTIME_TRACE_CBID_cudaCreateTextureObject_v2_v11080 = 434,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetTextureObjectTextureDesc_v2_v11080 = 435,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithParams_v12000 = 436,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiateWithParams_ptsz_v12000 = 437,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecGetFlags_v12000 = 438,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetKernel_v12000 = 439,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceProperties_v12000 = 440,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetId_v12000 = 441,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetId_ptsz_v12000 = 442,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphInstantiate_v12000 = 443,
    CUPTI_RUNTIME_TRACE_CBID_cudaInitDevice_v12000 = 444,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddNode_v12020 = 445,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeSetParams_v12020 = 446,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecNodeSetParams_v12020 = 447,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemAdvise_v12020 = 448,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_v12020 = 449,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchAsync_ptsz_v12020 = 450,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetName_v12030 = 451,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCaptureToGraph_v12030 = 452,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamBeginCaptureToGraph_ptsz_v12030 = 453,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphConditionalHandleCreate_v12030 = 454,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetEdges_v12030 = 455,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependencies_v12030 = 456,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetDependentNodes_v12030 = 457,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddDependencies_v12030 = 458,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphRemoveDependencies_v12030 = 459,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphAddNode_v12030 = 460,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_v12030 = 461,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetCaptureInfo_ptsz_v12030 = 462,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_v12030 = 463,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamUpdateCaptureDependencies_ptsz_v12030 = 464,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceRegisterAsyncNotification_v12040 = 465,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceUnregisterAsyncNotification_v12040 = 466,
    CUPTI_RUNTIME_TRACE_CBID_cudaFuncGetParamInfo_v12040 = 467,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPointByVersion_v12050 = 468,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetDriverEntryPointByVersion_ptsz_v12050 = 469,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryLoadData_v12060 = 470,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryLoadFromFile_v12060 = 471,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryUnload_v12060 = 472,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryGetKernel_v12060 = 473,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryGetGlobal_v12060 = 474,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryGetManaged_v12060 = 475,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryGetUnifiedFunction_v12060 = 476,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryGetKernelCount_v12060 = 477,
    CUPTI_RUNTIME_TRACE_CBID_cudaLibraryEnumerateKernels_v12060 = 478,
    CUPTI_RUNTIME_TRACE_CBID_cudaKernelSetAttributeForDevice_v12060 = 479,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetDevice_v12080 = 480,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetDevice_ptsz_v12080 = 481,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyBatchAsync_v12080 = 482,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyBatchAsync_ptsz_v12080 = 483,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DBatchAsync_v12080 = 484,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DBatchAsync_ptsz_v12080 = 485,
    CUPTI_RUNTIME_TRACE_CBID_cudaEventElapsedTime_v12080 = 486,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchBatchAsync_v13000 = 487,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemPrefetchBatchAsync_ptsz_v13000 = 488,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemDiscardBatchAsync_v13000 = 489,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemDiscardBatchAsync_ptsz_v13000 = 490,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemDiscardAndPrefetchBatchAsync_v13000 = 491,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemDiscardAndPrefetchBatchAsync_ptsz_v13000 = 492,
    CUPTI_RUNTIME_TRACE_CBID_cudaGetExportTable_v13000 = 493,
    CUPTI_RUNTIME_TRACE_CBID___cudaRegisterFatBinary_v13000 = 494,
    CUPTI_RUNTIME_TRACE_CBID___cudaRegisterFatBinaryEnd_v13000 = 495,
    CUPTI_RUNTIME_TRACE_CBID___cudaUnregisterFatBinary_v13000 = 496,
    CUPTI_RUNTIME_TRACE_CBID___cudaRegisterVar_v13000 = 497,
    CUPTI_RUNTIME_TRACE_CBID___cudaRegisterHostVar_v13000 = 498,
    CUPTI_RUNTIME_TRACE_CBID___cudaRegisterFunction_v13000 = 499,
    CUPTI_RUNTIME_TRACE_CBID___cudaRegisterManagedVar_v13000 = 500,
    CUPTI_RUNTIME_TRACE_CBID___cudaInitModule_v13000 = 501,
    CUPTI_RUNTIME_TRACE_CBID___cudaPushCallConfiguration_v13000 = 502,
    CUPTI_RUNTIME_TRACE_CBID___cudaPopCallConfiguration_v13000 = 503,
    CUPTI_RUNTIME_TRACE_CBID___cudaGetKernel_v13000 = 504,
    CUPTI_RUNTIME_TRACE_CBID___cudaLaunchKernel_v13000 = 505,
    CUPTI_RUNTIME_TRACE_CBID___cudaLaunchKernel_ptsz_v13000 = 506,
    CUPTI_RUNTIME_TRACE_CBID___cudaRegisterUnifiedTable_v13000 = 507,
    CUPTI_RUNTIME_TRACE_CBID___cudaGetProcAddress_v13000 = 508,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyBatchAsync_v13000 = 509,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpyBatchAsync_ptsz_v13000 = 510,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DBatchAsync_v13000 = 511,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemcpy3DBatchAsync_ptsz_v13000 = 512,
    CUPTI_RUNTIME_TRACE_CBID_cudaLogsRegisterCallback_v13000 = 513,
    CUPTI_RUNTIME_TRACE_CBID_cudaLogsUnregisterCallback_v13000 = 514,
    CUPTI_RUNTIME_TRACE_CBID_cudaLogsCurrent_v13000 = 515,
    CUPTI_RUNTIME_TRACE_CBID_cudaLogsDumpToFile_v13000 = 516,
    CUPTI_RUNTIME_TRACE_CBID_cudaLogsDumpToMemory_v13000 = 517,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemGetDefaultMemPool_v13000 = 518,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemGetMemPool_v13000 = 519,
    CUPTI_RUNTIME_TRACE_CBID_cudaMemSetMemPool_v13000 = 520,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetHostAtomicCapabilities_v13000 = 521,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetP2PAtomicCapabilities_v13000 = 522,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetDevResource_v13010 = 523,
    CUPTI_RUNTIME_TRACE_CBID_cudaDevSmResourceSplitByCount_v13010 = 524,
    CUPTI_RUNTIME_TRACE_CBID_cudaDevResourceGenerateDesc_v13010 = 525,
    CUPTI_RUNTIME_TRACE_CBID_cudaGreenCtxCreate_v13010 = 526,
    CUPTI_RUNTIME_TRACE_CBID_cudaExecutionCtxDestroy_v13010 = 527,
    CUPTI_RUNTIME_TRACE_CBID_cudaExecutionCtxGetDevResource_v13010 = 528,
    CUPTI_RUNTIME_TRACE_CBID_cudaExecutionCtxGetDevice_v13010 = 529,
    CUPTI_RUNTIME_TRACE_CBID_cudaExecutionCtxGetId_v13010 = 530,
    CUPTI_RUNTIME_TRACE_CBID_cuda531_v13010 = 531,
    CUPTI_RUNTIME_TRACE_CBID_cuda532_v13010 = 532,
    CUPTI_RUNTIME_TRACE_CBID_cudaExecutionCtxStreamCreate_v13010 = 533,
    CUPTI_RUNTIME_TRACE_CBID_cudaExecutionCtxSynchronize_v13010 = 534,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphConditionalHandleCreate_v2_v13010 = 535,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetDevResource_v13010 = 536,
    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetDevResource_ptsz_v13010 = 537,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetContainingGraph_v13010 = 538,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetLocalId_v13010 = 539,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetToolsId_v13010 = 540,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetId_v13010 = 541,
    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecGetId_v13010 = 542,
    CUPTI_RUNTIME_TRACE_CBID_cudaExecutionCtxRecordEvent_v13010 = 543,
    CUPTI_RUNTIME_TRACE_CBID_cudaExecutionCtxWaitEvent_v13010 = 544,
    CUPTI_RUNTIME_TRACE_CBID_cuda545_v13010 = 545,
    CUPTI_RUNTIME_TRACE_CBID_cuda546_v13010 = 546,
    CUPTI_RUNTIME_TRACE_CBID_cuda547_v13010 = 547,
    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetExecutionCtx_v13010 = 548,
    CUPTI_RUNTIME_TRACE_CBID_cudaDevSmResourceSplit_v13010 = 549,
    CUPTI_RUNTIME_TRACE_CBID_SIZE = 550,
    CUPTI_RUNTIME_TRACE_CBID_FORCE_INT = 2147483647,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUDA_ARRAY3D_DESCRIPTOR_v1_st {
    pub dummy: ::core::ffi::c_int,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUDA_ARRAY_DESCRIPTOR_v1_st {
    pub dummy: ::core::ffi::c_int,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUDA_MEMCPY2D_v1_st {
    pub dummy: ::core::ffi::c_int,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUDA_MEMCPY3D_v1_st {
    pub dummy: ::core::ffi::c_int,
}
#[repr(C)]
#[repr(align(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_Activity {
    pub kind: CUpti_ActivityKind,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityAPI {
    pub kind: CUpti_ActivityKind,
    pub cbid: CUpti_CallbackId,
    pub start: u64,
    pub end: u64,
    pub processId: u32,
    pub threadId: u32,
    pub correlationId: u32,
    pub returnValue: u32,
}
#[repr(C)]
#[repr(align(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityAutoBoostState {
    pub enabled: u32,
    pub pid: u32,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityBranch {
    pub kind: CUpti_ActivityKind,
    pub sourceLocatorId: u32,
    pub correlationId: u32,
    pub pcOffset: u32,
    pub executed: u32,
    pub diverged: u32,
    pub threadsExecuted: u64,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityBranch2 {
    pub kind: CUpti_ActivityKind,
    pub sourceLocatorId: u32,
    pub correlationId: u32,
    pub functionId: u32,
    pub pcOffset: u32,
    pub diverged: u32,
    pub threadsExecuted: u64,
    pub executed: u32,
    pub pad: u32,
}
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityCdpKernel {
    pub kind: CUpti_ActivityKind,
    pub cacheConfig: CUpti_ActivityCdpKernel__bindgen_ty_1,
    pub sharedMemoryConfig: u8,
    pub registersPerThread: u16,
    pub start: u64,
    pub end: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub gridX: i32,
    pub gridY: i32,
    pub gridZ: i32,
    pub blockX: i32,
    pub blockY: i32,
    pub blockZ: i32,
    pub staticSharedMemory: i32,
    pub dynamicSharedMemory: i32,
    pub localMemoryPerThread: u32,
    pub localMemoryTotal: u32,
    pub correlationId: u32,
    pub gridId: i64,
    pub parentGridId: i64,
    pub queued: u64,
    pub submitted: u64,
    pub completed: u64,
    pub parentBlockX: u32,
    pub parentBlockY: u32,
    pub parentBlockZ: u32,
    pub pad: u32,
    pub name: *const ::core::ffi::c_char,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityCdpKernel__bindgen_ty_1__bindgen_ty_1 {
    pub _bitfield_align_1: [u8; 0],
    pub _bitfield_1: __BindgenBitfieldUnit<[u8; 1usize]>,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityComputeEngineCtxSwitch {
    pub kind: CUpti_ActivityKind,
    pub contextId: u32,
    pub timestamp: u64,
    pub operationType: CUpti_ComputeEngineCtxSwitchOperationType,
    pub padding: u32,
}
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityConfidentialComputeRotation {
    pub kind: CUpti_ActivityKind,
    pub eventType: CUpti_ConfidentialComputeRotationEventType,
    pub deviceId: u32,
    pub contextId: u32,
    pub channelId: u32,
    pub channelType: CUpti_ChannelType,
    pub timestamp: u64,
}
#[repr(C)]
#[repr(align(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityContext {
    pub kind: CUpti_ActivityKind,
    pub contextId: u32,
    pub deviceId: u32,
    pub computeApiKind: u16,
    pub nullStreamId: u16,
}
#[cfg(any(feature = "cuda-12040"))]
#[repr(C)]
#[repr(align(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityContext2 {
    pub kind: CUpti_ActivityKind,
    pub contextId: u32,
    pub deviceId: u32,
    pub computeApiKind: u16,
    pub nullStreamId: u16,
    pub parentContextId: u32,
    pub isGreenContext: u8,
    pub padding: [u8; 3usize],
}
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[repr(align(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityContext2 {
    pub kind: CUpti_ActivityKind,
    pub contextId: u32,
    pub deviceId: u32,
    pub computeApiKind: u16,
    pub nullStreamId: u16,
    pub parentContextId: u32,
    pub isGreenContext: u8,
    pub padding: u8,
    pub numMultiprocessors: u16,
}
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[repr(align(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityContext3 {
    pub kind: CUpti_ActivityKind,
    pub contextId: u32,
    pub deviceId: u32,
    pub computeApiKind: u16,
    pub nullStreamId: u16,
    pub parentContextId: u32,
    pub isGreenContext: u8,
    pub padding: u8,
    pub numMultiprocessors: u16,
    pub cigMode: CUpti_ContextCigMode,
    pub padding2: u32,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityContext4 {
    pub kind: CUpti_ActivityKind,
    pub contextId: u32,
    pub deviceId: u32,
    pub computeApiKind: u16,
    pub nullStreamId: u16,
    pub parentContextId: u32,
    pub isGreenContext: u8,
    pub padding: u8,
    pub numMultiprocessors: u16,
    pub cigMode: CUpti_ContextCigMode,
    pub padding2: u32,
    pub processId: u64,
}
#[repr(C)]
#[repr(align(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityCudaEvent {
    pub kind: CUpti_ActivityKind,
    pub correlationId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub eventId: u32,
    pub pad: u32,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityCudaEvent2 {
    pub kind: CUpti_ActivityKind,
    pub correlationId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub eventId: u32,
    pub pad: u32,
    pub deviceId: u32,
    pub pad2: u32,
    pub reserved0: *mut ::core::ffi::c_void,
    pub deviceTimestamp: u64,
    pub cudaEventSyncId: u64,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityDevice {
    pub kind: CUpti_ActivityKind,
    pub flags: CUpti_ActivityFlag,
    pub globalMemoryBandwidth: u64,
    pub globalMemorySize: u64,
    pub constantMemorySize: u32,
    pub l2CacheSize: u32,
    pub numThreadsPerWarp: u32,
    pub coreClockRate: u32,
    pub numMemcpyEngines: u32,
    pub numMultiprocessors: u32,
    pub maxIPC: u32,
    pub maxWarpsPerMultiprocessor: u32,
    pub maxBlocksPerMultiprocessor: u32,
    pub maxRegistersPerBlock: u32,
    pub maxSharedMemoryPerBlock: u32,
    pub maxThreadsPerBlock: u32,
    pub maxBlockDimX: u32,
    pub maxBlockDimY: u32,
    pub maxBlockDimZ: u32,
    pub maxGridDimX: u32,
    pub maxGridDimY: u32,
    pub maxGridDimZ: u32,
    pub computeCapabilityMajor: u32,
    pub computeCapabilityMinor: u32,
    pub id: u32,
    pub pad: u32,
    pub name: *const ::core::ffi::c_char,
}
#[repr(C, packed(8))]
pub struct CUpti_ActivityDevice2 {
    pub kind: CUpti_ActivityKind,
    pub flags: CUpti_ActivityFlag,
    pub globalMemoryBandwidth: u64,
    pub globalMemorySize: u64,
    pub constantMemorySize: u32,
    pub l2CacheSize: u32,
    pub numThreadsPerWarp: u32,
    pub coreClockRate: u32,
    pub numMemcpyEngines: u32,
    pub numMultiprocessors: u32,
    pub maxIPC: u32,
    pub maxWarpsPerMultiprocessor: u32,
    pub maxBlocksPerMultiprocessor: u32,
    pub maxSharedMemoryPerMultiprocessor: u32,
    pub maxRegistersPerMultiprocessor: u32,
    pub maxRegistersPerBlock: u32,
    pub maxSharedMemoryPerBlock: u32,
    pub maxThreadsPerBlock: u32,
    pub maxBlockDimX: u32,
    pub maxBlockDimY: u32,
    pub maxBlockDimZ: u32,
    pub maxGridDimX: u32,
    pub maxGridDimY: u32,
    pub maxGridDimZ: u32,
    pub computeCapabilityMajor: u32,
    pub computeCapabilityMinor: u32,
    pub id: u32,
    pub eccEnabled: u32,
    pub uuid: CUuuid,
    pub name: *const ::core::ffi::c_char,
}
#[cfg(any(
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C, packed(8))]
pub struct CUpti_ActivityDevice3 {
    pub kind: CUpti_ActivityKind,
    pub flags: CUpti_ActivityFlag,
    pub globalMemoryBandwidth: u64,
    pub globalMemorySize: u64,
    pub constantMemorySize: u32,
    pub l2CacheSize: u32,
    pub numThreadsPerWarp: u32,
    pub coreClockRate: u32,
    pub numMemcpyEngines: u32,
    pub numMultiprocessors: u32,
    pub maxIPC: u32,
    pub maxWarpsPerMultiprocessor: u32,
    pub maxBlocksPerMultiprocessor: u32,
    pub maxSharedMemoryPerMultiprocessor: u32,
    pub maxRegistersPerMultiprocessor: u32,
    pub maxRegistersPerBlock: u32,
    pub maxSharedMemoryPerBlock: u32,
    pub maxThreadsPerBlock: u32,
    pub maxBlockDimX: u32,
    pub maxBlockDimY: u32,
    pub maxBlockDimZ: u32,
    pub maxGridDimX: u32,
    pub maxGridDimY: u32,
    pub maxGridDimZ: u32,
    pub computeCapabilityMajor: u32,
    pub computeCapabilityMinor: u32,
    pub id: u32,
    pub eccEnabled: u32,
    pub uuid: CUuuid,
    pub name: *const ::core::ffi::c_char,
    pub isCudaVisible: u8,
    pub reserved: [u8; 7usize],
}
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C, packed(8))]
pub struct CUpti_ActivityDevice4 {
    pub kind: CUpti_ActivityKind,
    pub flags: CUpti_ActivityFlag,
    pub globalMemoryBandwidth: u64,
    pub globalMemorySize: u64,
    pub constantMemorySize: u32,
    pub l2CacheSize: u32,
    pub numThreadsPerWarp: u32,
    pub coreClockRate: u32,
    pub numMemcpyEngines: u32,
    pub numMultiprocessors: u32,
    pub maxIPC: u32,
    pub maxWarpsPerMultiprocessor: u32,
    pub maxBlocksPerMultiprocessor: u32,
    pub maxSharedMemoryPerMultiprocessor: u32,
    pub maxRegistersPerMultiprocessor: u32,
    pub maxRegistersPerBlock: u32,
    pub maxSharedMemoryPerBlock: u32,
    pub maxThreadsPerBlock: u32,
    pub maxBlockDimX: u32,
    pub maxBlockDimY: u32,
    pub maxBlockDimZ: u32,
    pub maxGridDimX: u32,
    pub maxGridDimY: u32,
    pub maxGridDimZ: u32,
    pub computeCapabilityMajor: u32,
    pub computeCapabilityMinor: u32,
    pub id: u32,
    pub eccEnabled: u32,
    pub uuid: CUuuid,
    pub name: *const ::core::ffi::c_char,
    pub isCudaVisible: u8,
    pub isMigEnabled: u8,
    pub reserved: [u8; 6usize],
    pub gpuInstanceId: u32,
    pub computeInstanceId: u32,
    pub migUuid: CUuuid,
}
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C, packed(8))]
pub struct CUpti_ActivityDevice5 {
    pub kind: CUpti_ActivityKind,
    pub flags: CUpti_ActivityFlag,
    pub globalMemoryBandwidth: u64,
    pub globalMemorySize: u64,
    pub constantMemorySize: u32,
    pub l2CacheSize: u32,
    pub numThreadsPerWarp: u32,
    pub coreClockRate: u32,
    pub numMemcpyEngines: u32,
    pub numMultiprocessors: u32,
    pub maxIPC: u32,
    pub maxWarpsPerMultiprocessor: u32,
    pub maxBlocksPerMultiprocessor: u32,
    pub maxSharedMemoryPerMultiprocessor: u32,
    pub maxRegistersPerMultiprocessor: u32,
    pub maxRegistersPerBlock: u32,
    pub maxSharedMemoryPerBlock: u32,
    pub maxThreadsPerBlock: u32,
    pub maxBlockDimX: u32,
    pub maxBlockDimY: u32,
    pub maxBlockDimZ: u32,
    pub maxGridDimX: u32,
    pub maxGridDimY: u32,
    pub maxGridDimZ: u32,
    pub computeCapabilityMajor: u32,
    pub computeCapabilityMinor: u32,
    pub id: u32,
    pub eccEnabled: u32,
    pub uuid: CUuuid,
    pub name: *const ::core::ffi::c_char,
    pub isCudaVisible: u8,
    pub isMigEnabled: u8,
    pub reserved: [u8; 6usize],
    pub gpuInstanceId: u32,
    pub computeInstanceId: u32,
    pub migUuid: CUuuid,
    pub isNumaNode: u32,
    pub numaId: u32,
}
#[repr(C, packed(8))]
pub struct CUpti_ActivityDeviceAttribute {
    pub kind: CUpti_ActivityKind,
    pub flags: CUpti_ActivityFlag,
    pub deviceId: u32,
    pub attribute: CUpti_ActivityDeviceAttribute__bindgen_ty_1,
    pub value: CUpti_ActivityDeviceAttribute__bindgen_ty_2,
}
#[repr(C)]
pub struct CUpti_ActivityDeviceAttribute__bindgen_ty_1 {
    pub cu: __BindgenUnionField<CUdevice_attribute>,
    pub cupti: __BindgenUnionField<CUpti_DeviceAttribute>,
    pub bindgen_union_field: u32,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityDeviceGraphTrace {
    pub kind: CUpti_ActivityKind,
    pub deviceId: u32,
    pub start: u64,
    pub end: u64,
    pub graphId: u32,
    pub launcherGraphId: u32,
    pub deviceLaunchMode: u32,
    pub contextId: u32,
    pub streamId: u64,
    pub reserved: *mut ::core::ffi::c_void,
}
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityEnvironment {
    pub kind: CUpti_ActivityKind,
    pub deviceId: u32,
    pub timestamp: u64,
    pub environmentKind: CUpti_ActivityEnvironmentKind,
    pub data: CUpti_ActivityEnvironment__bindgen_ty_1,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityEnvironment__bindgen_ty_1__bindgen_ty_1 {
    pub smClock: u32,
    pub memoryClock: u32,
    pub pcieLinkGen: u32,
    pub pcieLinkWidth: u32,
    pub clocksThrottleReasons: CUpti_EnvironmentClocksThrottleReason,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityEnvironment__bindgen_ty_1__bindgen_ty_2 {
    pub gpuTemperature: u32,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityEnvironment__bindgen_ty_1__bindgen_ty_3 {
    pub power: u32,
    pub powerLimit: u32,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityEnvironment__bindgen_ty_1__bindgen_ty_4 {
    pub fanSpeed: u32,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityEvent {
    pub kind: CUpti_ActivityKind,
    pub id: CUpti_EventID,
    pub value: u64,
    pub domain: CUpti_EventDomainID,
    pub correlationId: u32,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityEventInstance {
    pub kind: CUpti_ActivityKind,
    pub id: CUpti_EventID,
    pub domain: CUpti_EventDomainID,
    pub instance: u32,
    pub value: u64,
    pub correlationId: u32,
    pub pad: u32,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityExternalCorrelation {
    pub kind: CUpti_ActivityKind,
    pub externalKind: CUpti_ExternalCorrelationKind,
    pub externalId: u64,
    pub correlationId: u32,
    pub reserved: u32,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityFunction {
    pub kind: CUpti_ActivityKind,
    pub id: u32,
    pub contextId: u32,
    pub moduleId: u32,
    pub functionIndex: u32,
    pub pad: u32,
    pub name: *const ::core::ffi::c_char,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityGlobalAccess {
    pub kind: CUpti_ActivityKind,
    pub flags: CUpti_ActivityFlag,
    pub sourceLocatorId: u32,
    pub correlationId: u32,
    pub pcOffset: u32,
    pub executed: u32,
    pub threadsExecuted: u64,
    pub l2_transactions: u64,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityGlobalAccess2 {
    pub kind: CUpti_ActivityKind,
    pub flags: CUpti_ActivityFlag,
    pub sourceLocatorId: u32,
    pub correlationId: u32,
    pub functionId: u32,
    pub pcOffset: u32,
    pub threadsExecuted: u64,
    pub l2_transactions: u64,
    pub theoreticalL2Transactions: u64,
    pub executed: u32,
    pub pad: u32,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityGlobalAccess3 {
    pub kind: CUpti_ActivityKind,
    pub flags: CUpti_ActivityFlag,
    pub sourceLocatorId: u32,
    pub correlationId: u32,
    pub functionId: u32,
    pub executed: u32,
    pub pcOffset: u64,
    pub threadsExecuted: u64,
    pub l2_transactions: u64,
    pub theoreticalL2Transactions: u64,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityGraphHostNode {
    pub kind: CUpti_ActivityKind,
    pub streamId: u32,
    pub contextId: u32,
    pub deviceId: u32,
    pub correlationId: u32,
    pub graphId: u32,
    pub graphNodeId: u64,
    pub processId: u32,
    pub threadId: u32,
    pub start: u64,
    pub end: u64,
}
#[cfg(any(
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityGraphTrace {
    pub kind: CUpti_ActivityKind,
    pub correlationId: u32,
    pub start: u64,
    pub end: u64,
    pub deviceId: u32,
    pub graphId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub reserved: *mut ::core::ffi::c_void,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityGraphTrace2 {
    pub kind: CUpti_ActivityKind,
    pub correlationId: u32,
    pub start: u64,
    pub end: u64,
    pub deviceId: u32,
    pub graphId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub reserved: *mut ::core::ffi::c_void,
    pub endDeviceId: u32,
    pub endContextId: u32,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityHostLaunch {
    pub kind: CUpti_ActivityKind,
    pub streamId: u32,
    pub contextId: u32,
    pub deviceId: u32,
    pub correlationId: u32,
    pub processId: u32,
    pub threadId: u32,
    pub padding: u32,
    pub start: u64,
    pub end: u64,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityInstantaneousEvent {
    pub kind: CUpti_ActivityKind,
    pub id: CUpti_EventID,
    pub value: u64,
    pub timestamp: u64,
    pub deviceId: u32,
    pub reserved: u32,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityInstantaneousEventInstance {
    pub kind: CUpti_ActivityKind,
    pub id: CUpti_EventID,
    pub value: u64,
    pub timestamp: u64,
    pub deviceId: u32,
    pub instance: u8,
    pub pad: [u8; 3usize],
}
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityInstantaneousMetric {
    pub kind: CUpti_ActivityKind,
    pub id: CUpti_MetricID,
    pub value: CUpti_MetricValue,
    pub timestamp: u64,
    pub deviceId: u32,
    pub flags: u8,
    pub pad: [u8; 3usize],
}
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityInstantaneousMetricInstance {
    pub kind: CUpti_ActivityKind,
    pub id: CUpti_MetricID,
    pub value: CUpti_MetricValue,
    pub timestamp: u64,
    pub deviceId: u32,
    pub flags: u8,
    pub instance: u8,
    pub pad: [u8; 2usize],
}
#[repr(C)]
#[repr(align(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityInstructionCorrelation {
    pub kind: CUpti_ActivityKind,
    pub flags: CUpti_ActivityFlag,
    pub sourceLocatorId: u32,
    pub functionId: u32,
    pub pcOffset: u32,
    pub pad: u32,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityInstructionExecution {
    pub kind: CUpti_ActivityKind,
    pub flags: CUpti_ActivityFlag,
    pub sourceLocatorId: u32,
    pub correlationId: u32,
    pub functionId: u32,
    pub pcOffset: u32,
    pub threadsExecuted: u64,
    pub notPredOffThreadsExecuted: u64,
    pub executed: u32,
    pub pad: u32,
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityJit {
    pub kind: CUpti_ActivityKind,
    pub jitEntryType: CUpti_ActivityJitEntryType,
    pub jitOperationType: CUpti_ActivityJitOperationType,
    pub deviceId: u32,
    pub start: u64,
    pub end: u64,
    pub correlationId: u32,
    pub padding: u32,
    pub jitOperationCorrelationId: u64,
    pub cacheSize: u64,
    pub cachePath: *const ::core::ffi::c_char,
}
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityJit2 {
    pub kind: CUpti_ActivityKind,
    pub jitEntryType: CUpti_ActivityJitEntryType,
    pub jitOperationType: CUpti_ActivityJitOperationType,
    pub deviceId: u32,
    pub start: u64,
    pub end: u64,
    pub correlationId: u32,
    pub padding: u32,
    pub jitOperationCorrelationId: u64,
    pub cacheSize: u64,
    pub cachePath: *const ::core::ffi::c_char,
    pub processId: u32,
    pub threadId: u32,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityKernel {
    pub kind: CUpti_ActivityKind,
    pub cacheConfigRequested: u8,
    pub cacheConfigExecuted: u8,
    pub registersPerThread: u16,
    pub start: u64,
    pub end: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub gridX: i32,
    pub gridY: i32,
    pub gridZ: i32,
    pub blockX: i32,
    pub blockY: i32,
    pub blockZ: i32,
    pub staticSharedMemory: i32,
    pub dynamicSharedMemory: i32,
    pub localMemoryPerThread: u32,
    pub localMemoryTotal: u32,
    pub correlationId: u32,
    pub runtimeCorrelationId: u32,
    pub pad: u32,
    pub name: *const ::core::ffi::c_char,
    pub reserved0: *mut ::core::ffi::c_void,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityKernel10 {
    pub kind: CUpti_ActivityKind,
    pub cacheConfig: CUpti_ActivityKernel10__bindgen_ty_1,
    pub sharedMemoryConfig: u8,
    pub registersPerThread: u16,
    pub partitionedGlobalCacheRequested: CUpti_ActivityPartitionedGlobalCacheConfig,
    pub partitionedGlobalCacheExecuted: CUpti_ActivityPartitionedGlobalCacheConfig,
    pub start: u64,
    pub end: u64,
    pub completed: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub gridX: i32,
    pub gridY: i32,
    pub gridZ: i32,
    pub blockX: i32,
    pub blockY: i32,
    pub blockZ: i32,
    pub staticSharedMemory: i32,
    pub dynamicSharedMemory: i32,
    pub localMemoryPerThread: u32,
    pub localMemoryTotal: u32,
    pub correlationId: u32,
    pub gridId: i64,
    pub name: *const ::core::ffi::c_char,
    pub reserved0: *mut ::core::ffi::c_void,
    pub queued: u64,
    pub submitted: u64,
    pub launchType: u8,
    pub isSharedMemoryCarveoutRequested: u8,
    pub sharedMemoryCarveoutRequested: u8,
    pub padding: u8,
    pub sharedMemoryExecuted: u32,
    pub graphNodeId: u64,
    pub shmemLimitConfig: CUpti_FuncShmemLimitConfig,
    pub graphId: u32,
    pub pAccessPolicyWindow: *mut CUaccessPolicyWindow,
    pub channelID: u32,
    pub channelType: CUpti_ChannelType,
    pub clusterX: u32,
    pub clusterY: u32,
    pub clusterZ: u32,
    pub clusterSchedulingPolicy: u32,
    pub localMemoryTotal_v2: u64,
    pub maxPotentialClusterSize: u32,
    pub maxActiveClusters: u32,
    pub isDeviceLaunched: u8,
    pub padding3: [u8; 7usize],
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityKernel10__bindgen_ty_1__bindgen_ty_1 {
    pub _bitfield_align_1: [u8; 0],
    pub _bitfield_1: __BindgenBitfieldUnit<[u8; 1usize]>,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityKernel11 {
    pub kind: CUpti_ActivityKind,
    pub cacheConfig: CUpti_ActivityKernel11__bindgen_ty_1,
    pub sharedMemoryConfig: u8,
    pub registersPerThread: u16,
    pub partitionedGlobalCacheRequested: CUpti_ActivityPartitionedGlobalCacheConfig,
    pub partitionedGlobalCacheExecuted: CUpti_ActivityPartitionedGlobalCacheConfig,
    pub start: u64,
    pub end: u64,
    pub completed: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub gridX: i32,
    pub gridY: i32,
    pub gridZ: i32,
    pub blockX: i32,
    pub blockY: i32,
    pub blockZ: i32,
    pub staticSharedMemory: i32,
    pub dynamicSharedMemory: i32,
    pub localMemoryPerThread: u32,
    pub localMemoryTotal: u32,
    pub correlationId: u32,
    pub gridId: i64,
    pub name: *const ::core::ffi::c_char,
    pub reserved0: *mut ::core::ffi::c_void,
    pub queued: u64,
    pub submitted: u64,
    pub launchType: u8,
    pub isSharedMemoryCarveoutRequested: u8,
    pub sharedMemoryCarveoutRequested: u8,
    pub padding: u8,
    pub sharedMemoryExecuted: u32,
    pub graphNodeId: u64,
    pub shmemLimitConfig: CUpti_FuncShmemLimitConfig,
    pub graphId: u32,
    pub pAccessPolicyWindow: *mut CUaccessPolicyWindow,
    pub channelID: u32,
    pub channelType: CUpti_ChannelType,
    pub clusterX: u32,
    pub clusterY: u32,
    pub clusterZ: u32,
    pub clusterSchedulingPolicy: u32,
    pub localMemoryTotal_v2: u64,
    pub maxPotentialClusterSize: u32,
    pub maxActiveClusters: u32,
    pub isDeviceLaunched: u8,
    pub padding3: [u8; 7usize],
    pub priority: i32,
    pub padding4: u32,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityKernel11__bindgen_ty_1__bindgen_ty_1 {
    pub _bitfield_align_1: [u8; 0],
    pub _bitfield_1: __BindgenBitfieldUnit<[u8; 1usize]>,
}
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityKernel2 {
    pub kind: CUpti_ActivityKind,
    pub cacheConfig: CUpti_ActivityKernel2__bindgen_ty_1,
    pub sharedMemoryConfig: u8,
    pub registersPerThread: u16,
    pub start: u64,
    pub end: u64,
    pub completed: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub gridX: i32,
    pub gridY: i32,
    pub gridZ: i32,
    pub blockX: i32,
    pub blockY: i32,
    pub blockZ: i32,
    pub staticSharedMemory: i32,
    pub dynamicSharedMemory: i32,
    pub localMemoryPerThread: u32,
    pub localMemoryTotal: u32,
    pub correlationId: u32,
    pub gridId: i64,
    pub name: *const ::core::ffi::c_char,
    pub reserved0: *mut ::core::ffi::c_void,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityKernel2__bindgen_ty_1__bindgen_ty_1 {
    pub _bitfield_align_1: [u8; 0],
    pub _bitfield_1: __BindgenBitfieldUnit<[u8; 1usize]>,
}
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityKernel3 {
    pub kind: CUpti_ActivityKind,
    pub cacheConfig: CUpti_ActivityKernel3__bindgen_ty_1,
    pub sharedMemoryConfig: u8,
    pub registersPerThread: u16,
    pub partitionedGlobalCacheRequested: CUpti_ActivityPartitionedGlobalCacheConfig,
    pub partitionedGlobalCacheExecuted: CUpti_ActivityPartitionedGlobalCacheConfig,
    pub start: u64,
    pub end: u64,
    pub completed: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub gridX: i32,
    pub gridY: i32,
    pub gridZ: i32,
    pub blockX: i32,
    pub blockY: i32,
    pub blockZ: i32,
    pub staticSharedMemory: i32,
    pub dynamicSharedMemory: i32,
    pub localMemoryPerThread: u32,
    pub localMemoryTotal: u32,
    pub correlationId: u32,
    pub gridId: i64,
    pub name: *const ::core::ffi::c_char,
    pub reserved0: *mut ::core::ffi::c_void,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityKernel3__bindgen_ty_1__bindgen_ty_1 {
    pub _bitfield_align_1: [u8; 0],
    pub _bitfield_1: __BindgenBitfieldUnit<[u8; 1usize]>,
}
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityKernel4 {
    pub kind: CUpti_ActivityKind,
    pub cacheConfig: CUpti_ActivityKernel4__bindgen_ty_1,
    pub sharedMemoryConfig: u8,
    pub registersPerThread: u16,
    pub partitionedGlobalCacheRequested: CUpti_ActivityPartitionedGlobalCacheConfig,
    pub partitionedGlobalCacheExecuted: CUpti_ActivityPartitionedGlobalCacheConfig,
    pub start: u64,
    pub end: u64,
    pub completed: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub gridX: i32,
    pub gridY: i32,
    pub gridZ: i32,
    pub blockX: i32,
    pub blockY: i32,
    pub blockZ: i32,
    pub staticSharedMemory: i32,
    pub dynamicSharedMemory: i32,
    pub localMemoryPerThread: u32,
    pub localMemoryTotal: u32,
    pub correlationId: u32,
    pub gridId: i64,
    pub name: *const ::core::ffi::c_char,
    pub reserved0: *mut ::core::ffi::c_void,
    pub queued: u64,
    pub submitted: u64,
    pub launchType: u8,
    pub isSharedMemoryCarveoutRequested: u8,
    pub sharedMemoryCarveoutRequested: u8,
    pub padding: u8,
    pub sharedMemoryExecuted: u32,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityKernel4__bindgen_ty_1__bindgen_ty_1 {
    pub _bitfield_align_1: [u8; 0],
    pub _bitfield_1: __BindgenBitfieldUnit<[u8; 1usize]>,
}
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityKernel5 {
    pub kind: CUpti_ActivityKind,
    pub cacheConfig: CUpti_ActivityKernel5__bindgen_ty_1,
    pub sharedMemoryConfig: u8,
    pub registersPerThread: u16,
    pub partitionedGlobalCacheRequested: CUpti_ActivityPartitionedGlobalCacheConfig,
    pub partitionedGlobalCacheExecuted: CUpti_ActivityPartitionedGlobalCacheConfig,
    pub start: u64,
    pub end: u64,
    pub completed: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub gridX: i32,
    pub gridY: i32,
    pub gridZ: i32,
    pub blockX: i32,
    pub blockY: i32,
    pub blockZ: i32,
    pub staticSharedMemory: i32,
    pub dynamicSharedMemory: i32,
    pub localMemoryPerThread: u32,
    pub localMemoryTotal: u32,
    pub correlationId: u32,
    pub gridId: i64,
    pub name: *const ::core::ffi::c_char,
    pub reserved0: *mut ::core::ffi::c_void,
    pub queued: u64,
    pub submitted: u64,
    pub launchType: u8,
    pub isSharedMemoryCarveoutRequested: u8,
    pub sharedMemoryCarveoutRequested: u8,
    pub padding: u8,
    pub sharedMemoryExecuted: u32,
    pub graphNodeId: u64,
    pub shmemLimitConfig: CUpti_FuncShmemLimitConfig,
    pub graphId: u32,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityKernel5__bindgen_ty_1__bindgen_ty_1 {
    pub _bitfield_align_1: [u8; 0],
    pub _bitfield_1: __BindgenBitfieldUnit<[u8; 1usize]>,
}
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityKernel6 {
    pub kind: CUpti_ActivityKind,
    pub cacheConfig: CUpti_ActivityKernel6__bindgen_ty_1,
    pub sharedMemoryConfig: u8,
    pub registersPerThread: u16,
    pub partitionedGlobalCacheRequested: CUpti_ActivityPartitionedGlobalCacheConfig,
    pub partitionedGlobalCacheExecuted: CUpti_ActivityPartitionedGlobalCacheConfig,
    pub start: u64,
    pub end: u64,
    pub completed: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub gridX: i32,
    pub gridY: i32,
    pub gridZ: i32,
    pub blockX: i32,
    pub blockY: i32,
    pub blockZ: i32,
    pub staticSharedMemory: i32,
    pub dynamicSharedMemory: i32,
    pub localMemoryPerThread: u32,
    pub localMemoryTotal: u32,
    pub correlationId: u32,
    pub gridId: i64,
    pub name: *const ::core::ffi::c_char,
    pub reserved0: *mut ::core::ffi::c_void,
    pub queued: u64,
    pub submitted: u64,
    pub launchType: u8,
    pub isSharedMemoryCarveoutRequested: u8,
    pub sharedMemoryCarveoutRequested: u8,
    pub padding: u8,
    pub sharedMemoryExecuted: u32,
    pub graphNodeId: u64,
    pub shmemLimitConfig: CUpti_FuncShmemLimitConfig,
    pub graphId: u32,
    pub pAccessPolicyWindow: *mut CUaccessPolicyWindow,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityKernel6__bindgen_ty_1__bindgen_ty_1 {
    pub _bitfield_align_1: [u8; 0],
    pub _bitfield_1: __BindgenBitfieldUnit<[u8; 1usize]>,
}
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityKernel7 {
    pub kind: CUpti_ActivityKind,
    pub cacheConfig: CUpti_ActivityKernel7__bindgen_ty_1,
    pub sharedMemoryConfig: u8,
    pub registersPerThread: u16,
    pub partitionedGlobalCacheRequested: CUpti_ActivityPartitionedGlobalCacheConfig,
    pub partitionedGlobalCacheExecuted: CUpti_ActivityPartitionedGlobalCacheConfig,
    pub start: u64,
    pub end: u64,
    pub completed: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub gridX: i32,
    pub gridY: i32,
    pub gridZ: i32,
    pub blockX: i32,
    pub blockY: i32,
    pub blockZ: i32,
    pub staticSharedMemory: i32,
    pub dynamicSharedMemory: i32,
    pub localMemoryPerThread: u32,
    pub localMemoryTotal: u32,
    pub correlationId: u32,
    pub gridId: i64,
    pub name: *const ::core::ffi::c_char,
    pub reserved0: *mut ::core::ffi::c_void,
    pub queued: u64,
    pub submitted: u64,
    pub launchType: u8,
    pub isSharedMemoryCarveoutRequested: u8,
    pub sharedMemoryCarveoutRequested: u8,
    pub padding: u8,
    pub sharedMemoryExecuted: u32,
    pub graphNodeId: u64,
    pub shmemLimitConfig: CUpti_FuncShmemLimitConfig,
    pub graphId: u32,
    pub pAccessPolicyWindow: *mut CUaccessPolicyWindow,
    pub channelID: u32,
    pub channelType: CUpti_ChannelType,
}
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityKernel7__bindgen_ty_1__bindgen_ty_1 {
    pub _bitfield_align_1: [u8; 0],
    pub _bitfield_1: __BindgenBitfieldUnit<[u8; 1usize]>,
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityKernel8 {
    pub kind: CUpti_ActivityKind,
    pub cacheConfig: CUpti_ActivityKernel8__bindgen_ty_1,
    pub sharedMemoryConfig: u8,
    pub registersPerThread: u16,
    pub partitionedGlobalCacheRequested: CUpti_ActivityPartitionedGlobalCacheConfig,
    pub partitionedGlobalCacheExecuted: CUpti_ActivityPartitionedGlobalCacheConfig,
    pub start: u64,
    pub end: u64,
    pub completed: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub gridX: i32,
    pub gridY: i32,
    pub gridZ: i32,
    pub blockX: i32,
    pub blockY: i32,
    pub blockZ: i32,
    pub staticSharedMemory: i32,
    pub dynamicSharedMemory: i32,
    pub localMemoryPerThread: u32,
    pub localMemoryTotal: u32,
    pub correlationId: u32,
    pub gridId: i64,
    pub name: *const ::core::ffi::c_char,
    pub reserved0: *mut ::core::ffi::c_void,
    pub queued: u64,
    pub submitted: u64,
    pub launchType: u8,
    pub isSharedMemoryCarveoutRequested: u8,
    pub sharedMemoryCarveoutRequested: u8,
    pub padding: u8,
    pub sharedMemoryExecuted: u32,
    pub graphNodeId: u64,
    pub shmemLimitConfig: CUpti_FuncShmemLimitConfig,
    pub graphId: u32,
    pub pAccessPolicyWindow: *mut CUaccessPolicyWindow,
    pub channelID: u32,
    pub channelType: CUpti_ChannelType,
    pub clusterX: u32,
    pub clusterY: u32,
    pub clusterZ: u32,
    pub clusterSchedulingPolicy: u32,
    pub localMemoryTotal_v2: u64,
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityKernel8__bindgen_ty_1__bindgen_ty_1 {
    pub _bitfield_align_1: [u8; 0],
    pub _bitfield_1: __BindgenBitfieldUnit<[u8; 1usize]>,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityKernel9 {
    pub kind: CUpti_ActivityKind,
    pub cacheConfig: CUpti_ActivityKernel9__bindgen_ty_1,
    pub sharedMemoryConfig: u8,
    pub registersPerThread: u16,
    pub partitionedGlobalCacheRequested: CUpti_ActivityPartitionedGlobalCacheConfig,
    pub partitionedGlobalCacheExecuted: CUpti_ActivityPartitionedGlobalCacheConfig,
    pub start: u64,
    pub end: u64,
    pub completed: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub gridX: i32,
    pub gridY: i32,
    pub gridZ: i32,
    pub blockX: i32,
    pub blockY: i32,
    pub blockZ: i32,
    pub staticSharedMemory: i32,
    pub dynamicSharedMemory: i32,
    pub localMemoryPerThread: u32,
    pub localMemoryTotal: u32,
    pub correlationId: u32,
    pub gridId: i64,
    pub name: *const ::core::ffi::c_char,
    pub reserved0: *mut ::core::ffi::c_void,
    pub queued: u64,
    pub submitted: u64,
    pub launchType: u8,
    pub isSharedMemoryCarveoutRequested: u8,
    pub sharedMemoryCarveoutRequested: u8,
    pub padding: u8,
    pub sharedMemoryExecuted: u32,
    pub graphNodeId: u64,
    pub shmemLimitConfig: CUpti_FuncShmemLimitConfig,
    pub graphId: u32,
    pub pAccessPolicyWindow: *mut CUaccessPolicyWindow,
    pub channelID: u32,
    pub channelType: CUpti_ChannelType,
    pub clusterX: u32,
    pub clusterY: u32,
    pub clusterZ: u32,
    pub clusterSchedulingPolicy: u32,
    pub localMemoryTotal_v2: u64,
    pub maxPotentialClusterSize: u32,
    pub maxActiveClusters: u32,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityKernel9__bindgen_ty_1__bindgen_ty_1 {
    pub _bitfield_align_1: [u8; 0],
    pub _bitfield_1: __BindgenBitfieldUnit<[u8; 1usize]>,
}
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityMarker {
    pub kind: CUpti_ActivityKind,
    pub flags: CUpti_ActivityFlag,
    pub timestamp: u64,
    pub id: u32,
    pub objectKind: CUpti_ActivityObjectKind,
    pub objectId: CUpti_ActivityObjectKindId,
    pub pad: u32,
    pub name: *const ::core::ffi::c_char,
}
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityMarker2 {
    pub kind: CUpti_ActivityKind,
    pub flags: CUpti_ActivityFlag,
    pub timestamp: u64,
    pub id: u32,
    pub objectKind: CUpti_ActivityObjectKind,
    pub objectId: CUpti_ActivityObjectKindId,
    pub pad: u32,
    pub name: *const ::core::ffi::c_char,
    pub domain: *const ::core::ffi::c_char,
}
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityMarkerData {
    pub kind: CUpti_ActivityKind,
    pub flags: CUpti_ActivityFlag,
    pub id: u32,
    pub payloadKind: CUpti_MetricValueKind,
    pub payload: CUpti_MetricValue,
    pub color: u32,
    pub category: u32,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityMarkerData2 {
    pub kind: CUpti_ActivityKind,
    pub flags: CUpti_ActivityFlag,
    pub id: u32,
    pub payloadKind: CUpti_MetricValueKind,
    pub payload: CUpti_MetricValue,
    pub color: u32,
    pub category: u32,
    pub cuptiDomainId: u32,
    pub padding: u32,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityMemDecompress {
    pub kind: CUpti_ActivityKind,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub channelID: u32,
    pub channelType: CUpti_ChannelType,
    pub correlationId: u32,
    pub numberOfOperations: u32,
    pub sourceBytes: u64,
    pub reserved0: *mut ::core::ffi::c_void,
    pub start: u64,
    pub end: u64,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityMemcpy {
    pub kind: CUpti_ActivityKind,
    pub copyKind: u8,
    pub srcKind: u8,
    pub dstKind: u8,
    pub flags: u8,
    pub bytes: u64,
    pub start: u64,
    pub end: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub correlationId: u32,
    pub runtimeCorrelationId: u32,
    pub pad: u32,
    pub reserved0: *mut ::core::ffi::c_void,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityMemcpy3 {
    pub kind: CUpti_ActivityKind,
    pub copyKind: u8,
    pub srcKind: u8,
    pub dstKind: u8,
    pub flags: u8,
    pub bytes: u64,
    pub start: u64,
    pub end: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub correlationId: u32,
    pub runtimeCorrelationId: u32,
    pub pad: u32,
    pub reserved0: *mut ::core::ffi::c_void,
    pub graphNodeId: u64,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityMemcpy4 {
    pub kind: CUpti_ActivityKind,
    pub copyKind: u8,
    pub srcKind: u8,
    pub dstKind: u8,
    pub flags: u8,
    pub bytes: u64,
    pub start: u64,
    pub end: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub correlationId: u32,
    pub runtimeCorrelationId: u32,
    pub pad: u32,
    pub reserved0: *mut ::core::ffi::c_void,
    pub graphNodeId: u64,
    pub graphId: u32,
    pub padding: u32,
}
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityMemcpy5 {
    pub kind: CUpti_ActivityKind,
    pub copyKind: u8,
    pub srcKind: u8,
    pub dstKind: u8,
    pub flags: u8,
    pub bytes: u64,
    pub start: u64,
    pub end: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub correlationId: u32,
    pub runtimeCorrelationId: u32,
    pub pad: u32,
    pub reserved0: *mut ::core::ffi::c_void,
    pub graphNodeId: u64,
    pub graphId: u32,
    pub channelID: u32,
    pub channelType: CUpti_ChannelType,
    pub pad2: u32,
}
#[cfg(any(feature = "cuda-12080", feature = "cuda-12090"))]
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityMemcpy6 {
    pub kind: CUpti_ActivityKind,
    pub copyKind: u8,
    pub srcKind: u8,
    pub dstKind: u8,
    pub flags: u8,
    pub bytes: u64,
    pub start: u64,
    pub end: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub correlationId: u32,
    pub runtimeCorrelationId: u32,
    pub pad: u32,
    pub reserved0: *mut ::core::ffi::c_void,
    pub graphNodeId: u64,
    pub graphId: u32,
    pub channelID: u32,
    pub channelType: CUpti_ChannelType,
    pub pad2: u32,
    pub copyCount: u64,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityMemcpy6 {
    pub kind: CUpti_ActivityKind,
    pub copyKind: u8,
    pub srcKind: u8,
    pub dstKind: u8,
    pub flags: u8,
    pub bytes: u64,
    pub start: u64,
    pub end: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub correlationId: u32,
    pub runtimeCorrelationId: u32,
    pub pad: u32,
    pub reserved0: *mut ::core::ffi::c_void,
    pub graphNodeId: u64,
    pub graphId: u32,
    pub channelID: u32,
    pub channelType: CUpti_ChannelType,
    pub isDeviceLaunched: u8,
    pub pad2: [u8; 3usize],
    pub copyCount: u64,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityMemcpyPtoP {
    pub kind: CUpti_ActivityKind,
    pub copyKind: u8,
    pub srcKind: u8,
    pub dstKind: u8,
    pub flags: u8,
    pub bytes: u64,
    pub start: u64,
    pub end: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub srcDeviceId: u32,
    pub srcContextId: u32,
    pub dstDeviceId: u32,
    pub dstContextId: u32,
    pub correlationId: u32,
    pub reserved0: *mut ::core::ffi::c_void,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityMemcpyPtoP2 {
    pub kind: CUpti_ActivityKind,
    pub copyKind: u8,
    pub srcKind: u8,
    pub dstKind: u8,
    pub flags: u8,
    pub bytes: u64,
    pub start: u64,
    pub end: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub srcDeviceId: u32,
    pub srcContextId: u32,
    pub dstDeviceId: u32,
    pub dstContextId: u32,
    pub correlationId: u32,
    pub reserved0: *mut ::core::ffi::c_void,
    pub graphNodeId: u64,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityMemcpyPtoP3 {
    pub kind: CUpti_ActivityKind,
    pub copyKind: u8,
    pub srcKind: u8,
    pub dstKind: u8,
    pub flags: u8,
    pub bytes: u64,
    pub start: u64,
    pub end: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub srcDeviceId: u32,
    pub srcContextId: u32,
    pub dstDeviceId: u32,
    pub dstContextId: u32,
    pub correlationId: u32,
    pub reserved0: *mut ::core::ffi::c_void,
    pub graphNodeId: u64,
    pub graphId: u32,
    pub padding: u32,
}
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityMemcpyPtoP4 {
    pub kind: CUpti_ActivityKind,
    pub copyKind: u8,
    pub srcKind: u8,
    pub dstKind: u8,
    pub flags: u8,
    pub bytes: u64,
    pub start: u64,
    pub end: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub srcDeviceId: u32,
    pub srcContextId: u32,
    pub dstDeviceId: u32,
    pub dstContextId: u32,
    pub correlationId: u32,
    pub reserved0: *mut ::core::ffi::c_void,
    pub graphNodeId: u64,
    pub graphId: u32,
    pub channelID: u32,
    pub channelType: CUpti_ChannelType,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityMemory {
    pub kind: CUpti_ActivityKind,
    pub memoryKind: CUpti_ActivityMemoryKind,
    pub address: u64,
    pub bytes: u64,
    pub start: u64,
    pub end: u64,
    pub allocPC: u64,
    pub freePC: u64,
    pub processId: u32,
    pub deviceId: u32,
    pub contextId: u32,
    pub pad: u32,
    pub name: *const ::core::ffi::c_char,
}
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityMemory2 {
    pub kind: CUpti_ActivityKind,
    pub memoryOperationType: CUpti_ActivityMemoryOperationType,
    pub memoryKind: CUpti_ActivityMemoryKind,
    pub correlationId: u32,
    pub address: u64,
    pub bytes: u64,
    pub timestamp: u64,
    pub PC: u64,
    pub processId: u32,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub name: *const ::core::ffi::c_char,
    pub isAsync: u32,
    pub pad1: u32,
    pub memoryPoolConfig: CUpti_ActivityMemory2__bindgen_ty_1,
}
#[cfg(any(feature = "cuda-11040", feature = "cuda-11050"))]
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityMemory2__bindgen_ty_1 {
    pub memoryPoolType: CUpti_ActivityMemoryPoolType,
    pub pad2: u32,
    pub address: u64,
    pub releaseThreshold: u64,
    pub pool: CUpti_ActivityMemory2__bindgen_ty_1__bindgen_ty_1,
}
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityMemory2__bindgen_ty_1 {
    pub memoryPoolType: CUpti_ActivityMemoryPoolType,
    pub pad2: u32,
    pub address: u64,
    pub releaseThreshold: u64,
    pub pool: CUpti_ActivityMemory2__bindgen_ty_1__bindgen_ty_1,
}
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityMemory3 {
    pub kind: CUpti_ActivityKind,
    pub memoryOperationType: CUpti_ActivityMemoryOperationType,
    pub memoryKind: CUpti_ActivityMemoryKind,
    pub correlationId: u32,
    pub address: u64,
    pub bytes: u64,
    pub timestamp: u64,
    pub PC: u64,
    pub processId: u32,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub name: *const ::core::ffi::c_char,
    pub isAsync: u32,
    pub pad1: u32,
    pub memoryPoolConfig: CUpti_ActivityMemory3__bindgen_ty_1,
}
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityMemory3__bindgen_ty_1 {
    pub memoryPoolType: CUpti_ActivityMemoryPoolType,
    pub pad2: u32,
    pub address: u64,
    pub releaseThreshold: u64,
    pub pool: CUpti_ActivityMemory3__bindgen_ty_1__bindgen_ty_1,
    pub utilizedSize: u64,
}
#[cfg(any(
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityMemory4 {
    pub kind: CUpti_ActivityKind,
    pub memoryOperationType: CUpti_ActivityMemoryOperationType,
    pub memoryKind: CUpti_ActivityMemoryKind,
    pub correlationId: u32,
    pub address: u64,
    pub bytes: u64,
    pub timestamp: u64,
    pub PC: u64,
    pub processId: u32,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub name: *const ::core::ffi::c_char,
    pub isAsync: u32,
    pub pad1: u32,
    pub memoryPoolConfig: CUpti_ActivityMemory4__bindgen_ty_1,
    pub source: *const ::core::ffi::c_char,
}
#[cfg(any(
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityMemory4__bindgen_ty_1 {
    pub memoryPoolType: CUpti_ActivityMemoryPoolType,
    pub pad2: u32,
    pub address: u64,
    pub releaseThreshold: u64,
    pub pool: CUpti_ActivityMemory4__bindgen_ty_1__bindgen_ty_1,
    pub utilizedSize: u64,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityMemoryPool {
    pub kind: CUpti_ActivityKind,
    pub memoryPoolOperationType: CUpti_ActivityMemoryPoolOperationType,
    pub memoryPoolType: CUpti_ActivityMemoryPoolType,
    pub correlationId: u32,
    pub processId: u32,
    pub deviceId: u32,
    pub minBytesToKeep: usize,
    pub address: u64,
    pub size: u64,
    pub releaseThreshold: u64,
    pub timestamp: u64,
}
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityMemoryPool2 {
    pub kind: CUpti_ActivityKind,
    pub memoryPoolOperationType: CUpti_ActivityMemoryPoolOperationType,
    pub memoryPoolType: CUpti_ActivityMemoryPoolType,
    pub correlationId: u32,
    pub processId: u32,
    pub deviceId: u32,
    pub minBytesToKeep: usize,
    pub address: u64,
    pub size: u64,
    pub releaseThreshold: u64,
    pub timestamp: u64,
    pub utilizedSize: u64,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityMemoryPool3 {
    pub kind: CUpti_ActivityKind,
    pub memoryPoolOperationType: CUpti_ActivityMemoryPoolOperationType,
    pub memoryPoolType: CUpti_ActivityMemoryPoolType,
    pub correlationId: u32,
    pub processId: u32,
    pub deviceId: u32,
    pub minBytesToKeep: usize,
    pub address: u64,
    pub size: u64,
    pub releaseThreshold: u64,
    pub timestamp: u64,
    pub utilizedSize: u64,
    pub isManagedPool: u8,
    pub pad2: [u8; 7usize],
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityMemset {
    pub kind: CUpti_ActivityKind,
    pub value: u32,
    pub bytes: u64,
    pub start: u64,
    pub end: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub correlationId: u32,
    pub flags: u16,
    pub memoryKind: u16,
    pub pad: u32,
    pub reserved0: *mut ::core::ffi::c_void,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityMemset2 {
    pub kind: CUpti_ActivityKind,
    pub value: u32,
    pub bytes: u64,
    pub start: u64,
    pub end: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub correlationId: u32,
    pub flags: u16,
    pub memoryKind: u16,
    pub pad: u32,
    pub reserved0: *mut ::core::ffi::c_void,
    pub graphNodeId: u64,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityMemset3 {
    pub kind: CUpti_ActivityKind,
    pub value: u32,
    pub bytes: u64,
    pub start: u64,
    pub end: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub correlationId: u32,
    pub flags: u16,
    pub memoryKind: u16,
    pub pad: u32,
    pub reserved0: *mut ::core::ffi::c_void,
    pub graphNodeId: u64,
    pub graphId: u32,
    pub padding: u32,
}
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityMemset4 {
    pub kind: CUpti_ActivityKind,
    pub value: u32,
    pub bytes: u64,
    pub start: u64,
    pub end: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub correlationId: u32,
    pub flags: u16,
    pub memoryKind: u16,
    pub pad: u32,
    pub reserved0: *mut ::core::ffi::c_void,
    pub graphNodeId: u64,
    pub graphId: u32,
    pub channelID: u32,
    pub channelType: CUpti_ChannelType,
    pub pad2: u32,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityMemset4 {
    pub kind: CUpti_ActivityKind,
    pub value: u32,
    pub bytes: u64,
    pub start: u64,
    pub end: u64,
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub correlationId: u32,
    pub flags: u16,
    pub memoryKind: u16,
    pub pad: u32,
    pub reserved0: *mut ::core::ffi::c_void,
    pub graphNodeId: u64,
    pub graphId: u32,
    pub channelID: u32,
    pub channelType: CUpti_ChannelType,
    pub isDeviceLaunched: u8,
    pub pad2: [u8; 3usize],
}
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityMetric {
    pub kind: CUpti_ActivityKind,
    pub id: CUpti_MetricID,
    pub value: CUpti_MetricValue,
    pub correlationId: u32,
    pub flags: u8,
    pub pad: [u8; 3usize],
}
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityMetricInstance {
    pub kind: CUpti_ActivityKind,
    pub id: CUpti_MetricID,
    pub value: CUpti_MetricValue,
    pub instance: u32,
    pub correlationId: u32,
    pub flags: u8,
    pub pad: [u8; 7usize],
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityModule {
    pub kind: CUpti_ActivityKind,
    pub contextId: u32,
    pub id: u32,
    pub cubinSize: u32,
    pub cubin: *const ::core::ffi::c_void,
}
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityName {
    pub kind: CUpti_ActivityKind,
    pub objectKind: CUpti_ActivityObjectKind,
    pub objectId: CUpti_ActivityObjectKindId,
    pub pad: u32,
    pub name: *const ::core::ffi::c_char,
}
#[repr(C, packed(8))]
pub struct CUpti_ActivityNvLink {
    pub kind: CUpti_ActivityKind,
    pub nvlinkVersion: u32,
    pub typeDev0: CUpti_DevType,
    pub typeDev1: CUpti_DevType,
    pub idDev0: CUpti_ActivityNvLink__bindgen_ty_1,
    pub idDev1: CUpti_ActivityNvLink__bindgen_ty_2,
    pub flag: u32,
    pub physicalNvLinkCount: u32,
    pub portDev0: [i8; 4usize],
    pub portDev1: [i8; 4usize],
    pub bandwidth: u64,
}
#[cfg(any(feature = "cuda-11040"))]
#[repr(C, packed(8))]
pub struct CUpti_ActivityNvLink2 {
    pub kind: CUpti_ActivityKind,
    pub nvlinkVersion: u32,
    pub typeDev0: CUpti_DevType,
    pub typeDev1: CUpti_DevType,
    pub idDev0: CUpti_ActivityNvLink2__bindgen_ty_1,
    pub idDev1: CUpti_ActivityNvLink2__bindgen_ty_2,
    pub flag: u32,
    pub physicalNvLinkCount: u32,
    pub portDev0: [i8; 16usize],
    pub portDev1: [i8; 16usize],
    pub bandwidth: u64,
}
#[cfg(any(
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C, packed(8))]
pub struct CUpti_ActivityNvLink2 {
    pub kind: CUpti_ActivityKind,
    pub nvlinkVersion: u32,
    pub typeDev0: CUpti_DevType,
    pub typeDev1: CUpti_DevType,
    pub idDev0: CUpti_ActivityNvLink2__bindgen_ty_1,
    pub idDev1: CUpti_ActivityNvLink2__bindgen_ty_2,
    pub flag: u32,
    pub physicalNvLinkCount: u32,
    pub portDev0: [i8; 32usize],
    pub portDev1: [i8; 32usize],
    pub bandwidth: u64,
}
#[repr(C)]
pub struct CUpti_ActivityNvLink2__bindgen_ty_1 {
    pub uuidDev: __BindgenUnionField<CUuuid>,
    pub npu: __BindgenUnionField<CUpti_ActivityNvLink2__bindgen_ty_1__bindgen_ty_1>,
    pub bindgen_union_field: [u32; 4usize],
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityNvLink2__bindgen_ty_1__bindgen_ty_1 {
    pub index: u32,
    pub domainId: u32,
}
#[repr(C)]
pub struct CUpti_ActivityNvLink2__bindgen_ty_2 {
    pub uuidDev: __BindgenUnionField<CUuuid>,
    pub npu: __BindgenUnionField<CUpti_ActivityNvLink2__bindgen_ty_2__bindgen_ty_1>,
    pub bindgen_union_field: [u32; 4usize],
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityNvLink2__bindgen_ty_2__bindgen_ty_1 {
    pub index: u32,
    pub domainId: u32,
}
#[cfg(any(feature = "cuda-11040"))]
#[repr(C, packed(8))]
pub struct CUpti_ActivityNvLink3 {
    pub kind: CUpti_ActivityKind,
    pub nvlinkVersion: u32,
    pub typeDev0: CUpti_DevType,
    pub typeDev1: CUpti_DevType,
    pub idDev0: CUpti_ActivityNvLink3__bindgen_ty_1,
    pub idDev1: CUpti_ActivityNvLink3__bindgen_ty_2,
    pub flag: u32,
    pub physicalNvLinkCount: u32,
    pub portDev0: [i8; 16usize],
    pub portDev1: [i8; 16usize],
    pub bandwidth: u64,
    pub nvswitchConnected: u8,
    pub pad: [u8; 7usize],
}
#[cfg(any(
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C, packed(8))]
pub struct CUpti_ActivityNvLink3 {
    pub kind: CUpti_ActivityKind,
    pub nvlinkVersion: u32,
    pub typeDev0: CUpti_DevType,
    pub typeDev1: CUpti_DevType,
    pub idDev0: CUpti_ActivityNvLink3__bindgen_ty_1,
    pub idDev1: CUpti_ActivityNvLink3__bindgen_ty_2,
    pub flag: u32,
    pub physicalNvLinkCount: u32,
    pub portDev0: [i8; 32usize],
    pub portDev1: [i8; 32usize],
    pub bandwidth: u64,
    pub nvswitchConnected: u8,
    pub pad: [u8; 7usize],
}
#[repr(C)]
pub struct CUpti_ActivityNvLink3__bindgen_ty_1 {
    pub uuidDev: __BindgenUnionField<CUuuid>,
    pub npu: __BindgenUnionField<CUpti_ActivityNvLink3__bindgen_ty_1__bindgen_ty_1>,
    pub bindgen_union_field: [u32; 4usize],
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityNvLink3__bindgen_ty_1__bindgen_ty_1 {
    pub index: u32,
    pub domainId: u32,
}
#[repr(C)]
pub struct CUpti_ActivityNvLink3__bindgen_ty_2 {
    pub uuidDev: __BindgenUnionField<CUuuid>,
    pub npu: __BindgenUnionField<CUpti_ActivityNvLink3__bindgen_ty_2__bindgen_ty_1>,
    pub bindgen_union_field: [u32; 4usize],
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityNvLink3__bindgen_ty_2__bindgen_ty_1 {
    pub index: u32,
    pub domainId: u32,
}
#[cfg(any(
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C, packed(8))]
pub struct CUpti_ActivityNvLink4 {
    pub kind: CUpti_ActivityKind,
    pub nvlinkVersion: u32,
    pub typeDev0: CUpti_DevType,
    pub typeDev1: CUpti_DevType,
    pub idDev0: CUpti_ActivityNvLink4__bindgen_ty_1,
    pub idDev1: CUpti_ActivityNvLink4__bindgen_ty_2,
    pub flag: u32,
    pub physicalNvLinkCount: u32,
    pub portDev0: [i8; 32usize],
    pub portDev1: [i8; 32usize],
    pub bandwidth: u64,
    pub nvswitchConnected: u8,
    pub pad: [u8; 7usize],
}
#[cfg(any(
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct CUpti_ActivityNvLink4__bindgen_ty_1 {
    pub uuidDev: __BindgenUnionField<CUuuid>,
    pub npu: __BindgenUnionField<CUpti_ActivityNvLink4__bindgen_ty_1__bindgen_ty_1>,
    pub bindgen_union_field: [u32; 4usize],
}
#[cfg(any(
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityNvLink4__bindgen_ty_1__bindgen_ty_1 {
    pub index: u32,
    pub domainId: u32,
}
#[cfg(any(
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct CUpti_ActivityNvLink4__bindgen_ty_2 {
    pub uuidDev: __BindgenUnionField<CUuuid>,
    pub npu: __BindgenUnionField<CUpti_ActivityNvLink4__bindgen_ty_2__bindgen_ty_1>,
    pub bindgen_union_field: [u32; 4usize],
}
#[cfg(any(
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityNvLink4__bindgen_ty_2__bindgen_ty_1 {
    pub index: u32,
    pub domainId: u32,
}
#[repr(C)]
pub struct CUpti_ActivityNvLink__bindgen_ty_1 {
    pub uuidDev: __BindgenUnionField<CUuuid>,
    pub npu: __BindgenUnionField<CUpti_ActivityNvLink__bindgen_ty_1__bindgen_ty_1>,
    pub bindgen_union_field: [u32; 4usize],
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityNvLink__bindgen_ty_1__bindgen_ty_1 {
    pub index: u32,
    pub domainId: u32,
}
#[repr(C)]
pub struct CUpti_ActivityNvLink__bindgen_ty_2 {
    pub uuidDev: __BindgenUnionField<CUuuid>,
    pub npu: __BindgenUnionField<CUpti_ActivityNvLink__bindgen_ty_2__bindgen_ty_1>,
    pub bindgen_union_field: [u32; 4usize],
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityNvLink__bindgen_ty_2__bindgen_ty_1 {
    pub index: u32,
    pub domainId: u32,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityObjectKindId__bindgen_ty_1 {
    pub processId: u32,
    pub threadId: u32,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityObjectKindId__bindgen_ty_2 {
    pub deviceId: u32,
    pub contextId: u32,
    pub streamId: u32,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityOpenAcc {
    pub kind: CUpti_ActivityKind,
    pub eventKind: CUpti_OpenAccEventKind,
    pub parentConstruct: CUpti_OpenAccConstructKind,
    pub version: u32,
    pub implicit: u32,
    pub deviceType: u32,
    pub deviceNumber: u32,
    pub threadId: u32,
    pub async_: u64,
    pub asyncMap: u64,
    pub lineNo: u32,
    pub endLineNo: u32,
    pub funcLineNo: u32,
    pub funcEndLineNo: u32,
    pub start: u64,
    pub end: u64,
    pub cuDeviceId: u32,
    pub cuContextId: u32,
    pub cuStreamId: u32,
    pub cuProcessId: u32,
    pub cuThreadId: u32,
    pub externalId: u32,
    pub srcFile: *const ::core::ffi::c_char,
    pub funcName: *const ::core::ffi::c_char,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityOpenAccData {
    pub kind: CUpti_ActivityKind,
    pub eventKind: CUpti_OpenAccEventKind,
    pub parentConstruct: CUpti_OpenAccConstructKind,
    pub version: u32,
    pub implicit: u32,
    pub deviceType: u32,
    pub deviceNumber: u32,
    pub threadId: u32,
    pub async_: u64,
    pub asyncMap: u64,
    pub lineNo: u32,
    pub endLineNo: u32,
    pub funcLineNo: u32,
    pub funcEndLineNo: u32,
    pub start: u64,
    pub end: u64,
    pub cuDeviceId: u32,
    pub cuContextId: u32,
    pub cuStreamId: u32,
    pub cuProcessId: u32,
    pub cuThreadId: u32,
    pub externalId: u32,
    pub srcFile: *const ::core::ffi::c_char,
    pub funcName: *const ::core::ffi::c_char,
    pub bytes: u64,
    pub hostPtr: u64,
    pub devicePtr: u64,
    pub varName: *const ::core::ffi::c_char,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityOpenAccLaunch {
    pub kind: CUpti_ActivityKind,
    pub eventKind: CUpti_OpenAccEventKind,
    pub parentConstruct: CUpti_OpenAccConstructKind,
    pub version: u32,
    pub implicit: u32,
    pub deviceType: u32,
    pub deviceNumber: u32,
    pub threadId: u32,
    pub async_: u64,
    pub asyncMap: u64,
    pub lineNo: u32,
    pub endLineNo: u32,
    pub funcLineNo: u32,
    pub funcEndLineNo: u32,
    pub start: u64,
    pub end: u64,
    pub cuDeviceId: u32,
    pub cuContextId: u32,
    pub cuStreamId: u32,
    pub cuProcessId: u32,
    pub cuThreadId: u32,
    pub externalId: u32,
    pub srcFile: *const ::core::ffi::c_char,
    pub funcName: *const ::core::ffi::c_char,
    pub numGangs: u64,
    pub numWorkers: u64,
    pub vectorLength: u64,
    pub kernelName: *const ::core::ffi::c_char,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityOpenAccOther {
    pub kind: CUpti_ActivityKind,
    pub eventKind: CUpti_OpenAccEventKind,
    pub parentConstruct: CUpti_OpenAccConstructKind,
    pub version: u32,
    pub implicit: u32,
    pub deviceType: u32,
    pub deviceNumber: u32,
    pub threadId: u32,
    pub async_: u64,
    pub asyncMap: u64,
    pub lineNo: u32,
    pub endLineNo: u32,
    pub funcLineNo: u32,
    pub funcEndLineNo: u32,
    pub start: u64,
    pub end: u64,
    pub cuDeviceId: u32,
    pub cuContextId: u32,
    pub cuStreamId: u32,
    pub cuProcessId: u32,
    pub cuThreadId: u32,
    pub externalId: u32,
    pub srcFile: *const ::core::ffi::c_char,
    pub funcName: *const ::core::ffi::c_char,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityOpenMp {
    pub kind: CUpti_ActivityKind,
    pub eventKind: CUpti_OpenMpEventKind,
    pub version: u32,
    pub threadId: u32,
    pub start: u64,
    pub end: u64,
    pub cuProcessId: u32,
    pub cuThreadId: u32,
}
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityOverhead {
    pub kind: CUpti_ActivityKind,
    pub overheadKind: CUpti_ActivityOverheadKind,
    pub objectKind: CUpti_ActivityObjectKind,
    pub objectId: CUpti_ActivityObjectKindId,
    pub start: u64,
    pub end: u64,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityOverhead2 {
    pub kind: CUpti_ActivityKind,
    pub overheadKind: CUpti_ActivityOverheadKind,
    pub objectKind: CUpti_ActivityObjectKind,
    pub objectId: CUpti_ActivityObjectKindId,
    pub start: u64,
    pub end: u64,
    pub correlationId: u32,
    pub reserved0: u32,
}
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C, packed(8))]
#[derive(Copy, Clone)]
pub struct CUpti_ActivityOverhead3 {
    pub kind: CUpti_ActivityKind,
    pub overheadKind: CUpti_ActivityOverheadKind,
    pub objectKind: CUpti_ActivityObjectKind,
    pub objectId: CUpti_ActivityObjectKindId,
    pub start: u64,
    pub end: u64,
    pub correlationId: u32,
    pub reserved0: u32,
    pub overheadData: *mut ::core::ffi::c_void,
}
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityOverheadCommandBufferFullData {
    pub commandBufferLength: u32,
    pub channelID: u32,
    pub channelType: u32,
}
#[repr(C)]
#[repr(align(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityPCSampling {
    pub kind: CUpti_ActivityKind,
    pub flags: CUpti_ActivityFlag,
    pub sourceLocatorId: u32,
    pub correlationId: u32,
    pub functionId: u32,
    pub pcOffset: u32,
    pub samples: u32,
    pub stallReason: CUpti_ActivityPCSamplingStallReason,
}
#[repr(C)]
#[repr(align(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityPCSampling2 {
    pub kind: CUpti_ActivityKind,
    pub flags: CUpti_ActivityFlag,
    pub sourceLocatorId: u32,
    pub correlationId: u32,
    pub functionId: u32,
    pub pcOffset: u32,
    pub latencySamples: u32,
    pub samples: u32,
    pub stallReason: CUpti_ActivityPCSamplingStallReason,
    pub pad: u32,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityPCSampling3 {
    pub kind: CUpti_ActivityKind,
    pub flags: CUpti_ActivityFlag,
    pub sourceLocatorId: u32,
    pub correlationId: u32,
    pub functionId: u32,
    pub latencySamples: u32,
    pub samples: u32,
    pub stallReason: CUpti_ActivityPCSamplingStallReason,
    pub pcOffset: u64,
}
#[repr(C)]
#[repr(align(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityPCSamplingConfig {
    pub size: u32,
    pub samplingPeriod: CUpti_ActivityPCSamplingPeriod,
    pub samplingPeriod2: u32,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityPCSamplingRecordInfo {
    pub kind: CUpti_ActivityKind,
    pub correlationId: u32,
    pub totalSamples: u64,
    pub droppedSamples: u64,
    pub samplingPeriodInCycles: u64,
}
#[repr(C)]
#[repr(align(8))]
pub struct CUpti_ActivityPcie {
    pub kind: CUpti_ActivityKind,
    pub type_: CUpti_PcieDeviceType,
    pub id: CUpti_ActivityPcie__bindgen_ty_1,
    pub domain: u32,
    pub pcieGeneration: u16,
    pub linkRate: u16,
    pub linkWidth: u16,
    pub upstreamBus: u16,
    pub attr: CUpti_ActivityPcie__bindgen_ty_2,
}
#[repr(C)]
pub struct CUpti_ActivityPcie__bindgen_ty_1 {
    pub devId: __BindgenUnionField<CUdevice>,
    pub bridgeId: __BindgenUnionField<u32>,
    pub bindgen_union_field: u32,
}
#[repr(C)]
pub struct CUpti_ActivityPcie__bindgen_ty_2 {
    pub gpuAttr: __BindgenUnionField<CUpti_ActivityPcie__bindgen_ty_2__bindgen_ty_1>,
    pub bridgeAttr: __BindgenUnionField<CUpti_ActivityPcie__bindgen_ty_2__bindgen_ty_2>,
    pub bindgen_union_field: [u32; 36usize],
}
#[repr(C)]
pub struct CUpti_ActivityPcie__bindgen_ty_2__bindgen_ty_1 {
    pub uuidDev: CUuuid,
    pub peerDev: [CUdevice; 32usize],
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityPcie__bindgen_ty_2__bindgen_ty_2 {
    pub secondaryBus: u16,
    pub deviceId: u16,
    pub vendorId: u16,
    pub pad0: u16,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityPreemption {
    pub kind: CUpti_ActivityKind,
    pub preemptionKind: CUpti_ActivityPreemptionKind,
    pub timestamp: u64,
    pub gridId: i64,
    pub blockX: u32,
    pub blockY: u32,
    pub blockZ: u32,
    pub pad: u32,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivitySharedAccess {
    pub kind: CUpti_ActivityKind,
    pub flags: CUpti_ActivityFlag,
    pub sourceLocatorId: u32,
    pub correlationId: u32,
    pub functionId: u32,
    pub pcOffset: u32,
    pub threadsExecuted: u64,
    pub sharedTransactions: u64,
    pub theoreticalSharedTransactions: u64,
    pub executed: u32,
    pub pad: u32,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivitySourceLocator {
    pub kind: CUpti_ActivityKind,
    pub id: u32,
    pub lineNumber: u32,
    pub pad: u32,
    pub fileName: *const ::core::ffi::c_char,
}
#[repr(C)]
#[repr(align(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityStream {
    pub kind: CUpti_ActivityKind,
    pub contextId: u32,
    pub streamId: u32,
    pub priority: u32,
    pub flag: CUpti_ActivityStreamFlag,
    pub correlationId: u32,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivitySynchronization {
    pub kind: CUpti_ActivityKind,
    pub type_: CUpti_ActivitySynchronizationType,
    pub start: u64,
    pub end: u64,
    pub correlationId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub cudaEventId: u32,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivitySynchronization2 {
    pub kind: CUpti_ActivityKind,
    pub type_: CUpti_ActivitySynchronizationType,
    pub start: u64,
    pub end: u64,
    pub correlationId: u32,
    pub contextId: u32,
    pub streamId: u32,
    pub cudaEventId: u32,
    pub cudaEventSyncId: u64,
    pub returnValue: u32,
    pub pad: u32,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityUnifiedMemoryCounter {
    pub kind: CUpti_ActivityKind,
    pub counterKind: CUpti_ActivityUnifiedMemoryCounterKind,
    pub scope: CUpti_ActivityUnifiedMemoryCounterScope,
    pub deviceId: u32,
    pub value: u64,
    pub timestamp: u64,
    pub processId: u32,
    pub pad: u32,
}
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityUnifiedMemoryCounter2 {
    pub kind: CUpti_ActivityKind,
    pub counterKind: CUpti_ActivityUnifiedMemoryCounterKind,
    pub value: u64,
    pub start: u64,
    pub end: u64,
    pub address: u64,
    pub srcId: u32,
    pub dstId: u32,
    pub streamId: u32,
    pub processId: u32,
    pub flags: u32,
    pub pad: u32,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C, packed(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityUnifiedMemoryCounter3 {
    pub kind: CUpti_ActivityKind,
    pub counterKind: CUpti_ActivityUnifiedMemoryCounterKind,
    pub value: u64,
    pub start: u64,
    pub end: u64,
    pub address: u64,
    pub srcId: u32,
    pub dstId: u32,
    pub streamId: u32,
    pub processId: u32,
    pub flags: u32,
    pub pad: u32,
    pub processors: [u64; 5usize],
}
#[repr(C)]
#[repr(align(8))]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ActivityUnifiedMemoryCounterConfig {
    pub scope: CUpti_ActivityUnifiedMemoryCounterScope,
    pub kind: CUpti_ActivityUnifiedMemoryCounterKind,
    pub deviceId: u32,
    pub enable: u32,
}
#[repr(C)]
pub struct CUpti_CallbackData {
    pub callbackSite: CUpti_ApiCallbackSite,
    pub functionName: *const ::core::ffi::c_char,
    pub functionParams: *const ::core::ffi::c_void,
    pub functionReturnValue: *mut ::core::ffi::c_void,
    pub symbolName: *const ::core::ffi::c_char,
    pub context: CUcontext,
    pub contextUid: u32,
    pub correlationData: *mut u64,
    pub correlationId: u32,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_EventGroupSet {
    pub numEventGroups: u32,
    pub eventGroups: *mut CUpti_EventGroup,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_EventGroupSets {
    pub numSets: u32,
    pub sets: *mut CUpti_EventGroupSet,
}
#[repr(C)]
pub struct CUpti_GraphData {
    pub graph: CUgraph,
    pub originalGraph: CUgraph,
    pub node: CUgraphNode,
    pub originalNode: CUgraphNode,
    pub nodeType: CUgraphNodeType,
    pub dependency: CUgraphNode,
    pub graphExec: CUgraphExec,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_ModuleResourceData {
    pub moduleId: u32,
    pub cubinSize: usize,
    pub pCubin: *const ::core::ffi::c_char,
}
#[cfg(any(feature = "cuda-11040"))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_NvtxData {
    pub functionName: *const ::core::ffi::c_char,
    pub functionParams: *const ::core::ffi::c_void,
}
#[cfg(any(
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_NvtxData {
    pub functionName: *const ::core::ffi::c_char,
    pub functionParams: *const ::core::ffi::c_void,
    pub functionReturnValue: *const ::core::ffi::c_void,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_NvtxExtPayloadAttr {
    pub structSize: u32,
    pub type_: u32,
    pub attributes: *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct CUpti_ResourceData {
    pub context: CUcontext,
    pub resourceHandle: CUpti_ResourceData__bindgen_ty_1,
    pub resourceDescriptor: *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct CUpti_ResourceData__bindgen_ty_1 {
    pub stream: __BindgenUnionField<CUstream>,
    pub bindgen_union_field: u64,
}
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Copy, Clone)]
pub struct CUpti_StateData {
    pub __bindgen_anon_1: CUpti_StateData__bindgen_ty_1,
}
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_StateData__bindgen_ty_1__bindgen_ty_1 {
    pub result: CUptiResult,
    pub message: *const ::core::ffi::c_char,
}
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct CUpti_StreamAttrData {
    pub stream: CUstream,
    pub attr: CUstreamAttrID,
    pub value: *const CUstreamAttrValue,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct CUpti_SubscriberParams {
    pub structSize: usize,
    pub subscriberName: *const ::core::ffi::c_char,
    pub oldSubscriberName: *mut ::core::ffi::c_char,
    pub oldSubscriberSize: usize,
}
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct CUpti_Subscriber_st {
    _unused: [u8; 0],
}
#[repr(C)]
pub struct CUpti_SynchronizeData {
    pub context: CUcontext,
    pub stream: CUstream,
}
#[repr(C)]
#[derive(Copy, Clone, Debug, Default, Eq, Hash, Ord, PartialEq, PartialOrd)]
pub struct __BindgenBitfieldUnit<Storage> {
    storage: Storage,
}
#[repr(C)]
pub struct __BindgenUnionField<T>(::core::marker::PhantomData<T>);
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuArray3DCreate_params_st {
    pub pHandle: *mut CUarray,
    pub pAllocateArray: *const CUDA_ARRAY3D_DESCRIPTOR_v1,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuArray3DCreate_v2_params_st {
    pub pHandle: *mut CUarray,
    pub pAllocateArray: *const CUDA_ARRAY3D_DESCRIPTOR,
}
#[repr(C)]
pub struct cuArray3DGetDescriptor_params_st {
    pub pArrayDescriptor: *mut CUDA_ARRAY3D_DESCRIPTOR_v1,
    pub hArray: CUarray,
}
#[repr(C)]
pub struct cuArray3DGetDescriptor_v2_params_st {
    pub pArrayDescriptor: *mut CUDA_ARRAY3D_DESCRIPTOR,
    pub hArray: CUarray,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuArrayCreate_params_st {
    pub pHandle: *mut CUarray,
    pub pAllocateArray: *const CUDA_ARRAY_DESCRIPTOR_v1,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuArrayCreate_v2_params_st {
    pub pHandle: *mut CUarray,
    pub pAllocateArray: *const CUDA_ARRAY_DESCRIPTOR,
}
#[repr(C)]
pub struct cuArrayDestroy_params_st {
    pub hArray: CUarray,
}
#[repr(C)]
pub struct cuArrayGetDescriptor_params_st {
    pub pArrayDescriptor: *mut CUDA_ARRAY_DESCRIPTOR_v1,
    pub hArray: CUarray,
}
#[repr(C)]
pub struct cuArrayGetDescriptor_v2_params_st {
    pub pArrayDescriptor: *mut CUDA_ARRAY_DESCRIPTOR,
    pub hArray: CUarray,
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuArrayGetMemoryRequirements_params_st {
    pub memoryRequirements: *mut CUDA_ARRAY_MEMORY_REQUIREMENTS,
    pub array: CUarray,
    pub device: CUdevice,
}
#[repr(C)]
pub struct cuArrayGetPlane_params_st {
    pub pPlaneArray: *mut CUarray,
    pub hArray: CUarray,
    pub planeIdx: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuArrayGetSparseProperties_params_st {
    pub sparseProperties: *mut CUDA_ARRAY_SPARSE_PROPERTIES,
    pub array: CUarray,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuCheckpointProcessCheckpoint_params_st {
    pub pid: ::core::ffi::c_int,
    pub args: *mut CUcheckpointCheckpointArgs,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuCheckpointProcessGetRestoreThreadId_params_st {
    pub pid: ::core::ffi::c_int,
    pub tid: *mut ::core::ffi::c_int,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuCheckpointProcessGetState_params_st {
    pub pid: ::core::ffi::c_int,
    pub state: *mut CUprocessState,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuCheckpointProcessLock_params_st {
    pub pid: ::core::ffi::c_int,
    pub args: *mut CUcheckpointLockArgs,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuCheckpointProcessRestore_params_st {
    pub pid: ::core::ffi::c_int,
    pub args: *mut CUcheckpointRestoreArgs,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuCheckpointProcessUnlock_params_st {
    pub pid: ::core::ffi::c_int,
    pub args: *mut CUcheckpointUnlockArgs,
}
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuCoredumpGetAttributeGlobal_params_st {
    pub attrib: CUcoredumpSettings,
    pub value: *mut ::core::ffi::c_void,
    pub size: *mut usize,
}
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuCoredumpGetAttribute_params_st {
    pub attrib: CUcoredumpSettings,
    pub value: *mut ::core::ffi::c_void,
    pub size: *mut usize,
}
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuCoredumpSetAttributeGlobal_params_st {
    pub attrib: CUcoredumpSettings,
    pub value: *mut ::core::ffi::c_void,
    pub size: *mut usize,
}
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuCoredumpSetAttribute_params_st {
    pub attrib: CUcoredumpSettings,
    pub value: *mut ::core::ffi::c_void,
    pub size: *mut usize,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuCtxAttach_params_st {
    pub pctx: *mut CUcontext,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuCtxCreate_params_st {
    pub pctx: *mut CUcontext,
    pub flags: ::core::ffi::c_uint,
    pub dev: CUdevice,
}
#[repr(C)]
pub struct cuCtxCreate_v2_params_st {
    pub pctx: *mut CUcontext,
    pub flags: ::core::ffi::c_uint,
    pub dev: CUdevice,
}
#[repr(C)]
pub struct cuCtxCreate_v3_params_st {
    pub pctx: *mut CUcontext,
    pub paramsArray: *mut CUexecAffinityParam,
    pub numParams: ::core::ffi::c_int,
    pub flags: ::core::ffi::c_uint,
    pub dev: CUdevice,
}
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuCtxCreate_v4_params_st {
    pub pctx: *mut CUcontext,
    pub ctxCreateParams: *mut CUctxCreateParams,
    pub flags: ::core::ffi::c_uint,
    pub dev: CUdevice,
}
#[repr(C)]
pub struct cuCtxDestroy_params_st {
    pub ctx: CUcontext,
}
#[repr(C)]
pub struct cuCtxDestroy_v2_params_st {
    pub ctx: CUcontext,
}
#[repr(C)]
pub struct cuCtxDetach_params_st {
    pub ctx: CUcontext,
}
#[repr(C)]
pub struct cuCtxDisablePeerAccess_params_st {
    pub peerContext: CUcontext,
}
#[repr(C)]
pub struct cuCtxEnablePeerAccess_params_st {
    pub peerContext: CUcontext,
    pub Flags: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuCtxFromGreenCtx_params_st {
    pub pContext: *mut CUcontext,
    pub hCtx: CUgreenCtx,
}
#[repr(C)]
pub struct cuCtxGetApiVersion_params_st {
    pub ctx: CUcontext,
    pub version: *mut ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuCtxGetCacheConfig_params_st {
    pub pconfig: *mut CUfunc_cache,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuCtxGetCurrent_params_st {
    pub pctx: *mut CUcontext,
}
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuCtxGetDevResource_params_st {
    pub hCtx: CUcontext,
    pub resource: *mut CUdevResource,
    pub type_: CUdevResourceType,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuCtxGetDevice_params_st {
    pub device: *mut CUdevice,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cuCtxGetDevice_v2_params_st {
    pub device: *mut CUdevice,
    pub ctx: CUcontext,
}
#[repr(C)]
pub struct cuCtxGetExecAffinity_params_st {
    pub pExecAffinity: *mut CUexecAffinityParam,
    pub type_: CUexecAffinityType,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuCtxGetFlags_params_st {
    pub flags: *mut ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuCtxGetId_params_st {
    pub ctx: CUcontext,
    pub ctxId: *mut ::core::ffi::c_ulonglong,
}
#[repr(C)]
pub struct cuCtxGetLimit_params_st {
    pub pvalue: *mut usize,
    pub limit: CUlimit,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuCtxGetSharedMemConfig_params_st {
    pub pConfig: *mut CUsharedconfig,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuCtxGetStreamPriorityRange_params_st {
    pub leastPriority: *mut ::core::ffi::c_int,
    pub greatestPriority: *mut ::core::ffi::c_int,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuCtxPopCurrent_params_st {
    pub pctx: *mut CUcontext,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuCtxPopCurrent_v2_params_st {
    pub pctx: *mut CUcontext,
}
#[repr(C)]
pub struct cuCtxPushCurrent_params_st {
    pub ctx: CUcontext,
}
#[repr(C)]
pub struct cuCtxPushCurrent_v2_params_st {
    pub ctx: CUcontext,
}
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuCtxRecordEvent_params_st {
    pub hCtx: CUcontext,
    pub hEvent: CUevent,
}
#[repr(C)]
pub struct cuCtxSetCacheConfig_params_st {
    pub config: CUfunc_cache,
}
#[repr(C)]
pub struct cuCtxSetCurrent_params_st {
    pub ctx: CUcontext,
}
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuCtxSetFlags_params_st {
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuCtxSetLimit_params_st {
    pub limit: CUlimit,
    pub value: usize,
}
#[repr(C)]
pub struct cuCtxSetSharedMemConfig_params_st {
    pub config: CUsharedconfig,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cuCtxSynchronize_v2_params_st {
    pub ctx: CUcontext,
}
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuCtxWaitEvent_params_st {
    pub hCtx: CUcontext,
    pub hEvent: CUevent,
}
#[repr(C)]
pub struct cuDestroyExternalMemory_params_st {
    pub extMem: CUexternalMemory,
}
#[repr(C)]
pub struct cuDestroyExternalSemaphore_params_st {
    pub extSem: CUexternalSemaphore,
}
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuDevResourceGenerateDesc_params_st {
    pub phDesc: *mut CUdevResourceDesc,
    pub resources: *mut CUdevResource,
    pub nbResources: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuDevSmResourceSplitByCount_params_st {
    pub result: *mut CUdevResource,
    pub nbGroups: *mut ::core::ffi::c_uint,
    pub input: *const CUdevResource,
    pub remaining: *mut CUdevResource,
    pub useFlags: ::core::ffi::c_uint,
    pub minCount: ::core::ffi::c_uint,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuDevSmResourceSplitByCount_params_st {
    pub result: *mut CUdevResource,
    pub nbGroups: *mut ::core::ffi::c_uint,
    pub input: *const CUdevResource,
    pub remainder: *mut CUdevResource,
    pub flags: ::core::ffi::c_uint,
    pub minCount: ::core::ffi::c_uint,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuDevSmResourceSplit_params_st {
    pub result: *mut CUdevResource,
    pub nbGroups: ::core::ffi::c_uint,
    pub input: *const CUdevResource,
    pub remainder: *mut CUdevResource,
    pub flags: ::core::ffi::c_uint,
    pub groupParams: *mut CU_DEV_SM_RESOURCE_GROUP_PARAMS,
}
#[repr(C)]
pub struct cuDeviceCanAccessPeer_params_st {
    pub canAccessPeer: *mut ::core::ffi::c_int,
    pub dev: CUdevice,
    pub peerDev: CUdevice,
}
#[repr(C)]
pub struct cuDeviceComputeCapability_params_st {
    pub major: *mut ::core::ffi::c_int,
    pub minor: *mut ::core::ffi::c_int,
    pub dev: CUdevice,
}
#[repr(C)]
pub struct cuDeviceGetAttribute_params_st {
    pub pi: *mut ::core::ffi::c_int,
    pub attrib: CUdevice_attribute,
    pub dev: CUdevice,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuDeviceGetByPCIBusId_params_st {
    pub dev: *mut CUdevice,
    pub pciBusId: *const ::core::ffi::c_char,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuDeviceGetCount_params_st {
    pub count: *mut ::core::ffi::c_int,
}
#[repr(C)]
pub struct cuDeviceGetDefaultMemPool_params_st {
    pub pool_out: *mut CUmemoryPool,
    pub dev: CUdevice,
}
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuDeviceGetDevResource_params_st {
    pub device: CUdevice,
    pub resource: *mut CUdevResource,
    pub type_: CUdevResourceType,
}
#[repr(C)]
pub struct cuDeviceGetExecAffinitySupport_params_st {
    pub pi: *mut ::core::ffi::c_int,
    pub type_: CUexecAffinityType,
    pub dev: CUdevice,
}
#[repr(C)]
pub struct cuDeviceGetGraphMemAttribute_params_st {
    pub device: CUdevice,
    pub attr: CUgraphMem_attribute,
    pub value: *mut ::core::ffi::c_void,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cuDeviceGetHostAtomicCapabilities_params_st {
    pub capabilities: *mut ::core::ffi::c_uint,
    pub operations: *const CUatomicOperation,
    pub count: ::core::ffi::c_uint,
    pub dev: CUdevice,
}
#[repr(C)]
pub struct cuDeviceGetLuid_params_st {
    pub luid: *mut ::core::ffi::c_char,
    pub deviceNodeMask: *mut ::core::ffi::c_uint,
    pub dev: CUdevice,
}
#[repr(C)]
pub struct cuDeviceGetMemPool_params_st {
    pub pool: *mut CUmemoryPool,
    pub dev: CUdevice,
}
#[repr(C)]
pub struct cuDeviceGetName_params_st {
    pub name: *mut ::core::ffi::c_char,
    pub len: ::core::ffi::c_int,
    pub dev: CUdevice,
}
#[repr(C)]
pub struct cuDeviceGetNvSciSyncAttributes_params_st {
    pub nvSciSyncAttrList: *mut ::core::ffi::c_void,
    pub dev: CUdevice,
    pub flags: ::core::ffi::c_int,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cuDeviceGetP2PAtomicCapabilities_params_st {
    pub capabilities: *mut ::core::ffi::c_uint,
    pub operations: *const CUatomicOperation,
    pub count: ::core::ffi::c_uint,
    pub srcDevice: CUdevice,
    pub dstDevice: CUdevice,
}
#[repr(C)]
pub struct cuDeviceGetP2PAttribute_params_st {
    pub value: *mut ::core::ffi::c_int,
    pub attrib: CUdevice_P2PAttribute,
    pub srcDevice: CUdevice,
    pub dstDevice: CUdevice,
}
#[repr(C)]
pub struct cuDeviceGetPCIBusId_params_st {
    pub pciBusId: *mut ::core::ffi::c_char,
    pub len: ::core::ffi::c_int,
    pub dev: CUdevice,
}
#[repr(C)]
pub struct cuDeviceGetProperties_params_st {
    pub prop: *mut CUdevprop,
    pub dev: CUdevice,
}
#[repr(C)]
pub struct cuDeviceGetTexture1DLinearMaxWidth_params_st {
    pub maxWidthInElements: *mut usize,
    pub format: CUarray_format,
    pub numChannels: ::core::ffi::c_uint,
    pub dev: CUdevice,
}
#[repr(C)]
pub struct cuDeviceGetUuid_params_st {
    pub uuid: *mut CUuuid,
    pub dev: CUdevice,
}
#[repr(C)]
pub struct cuDeviceGetUuid_v2_params_st {
    pub uuid: *mut CUuuid,
    pub dev: CUdevice,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuDeviceGet_params_st {
    pub device: *mut CUdevice,
    pub ordinal: ::core::ffi::c_int,
}
#[repr(C)]
pub struct cuDeviceGraphMemTrim_params_st {
    pub device: CUdevice,
}
#[repr(C)]
pub struct cuDevicePrimaryCtxGetState_params_st {
    pub dev: CUdevice,
    pub flags: *mut ::core::ffi::c_uint,
    pub active: *mut ::core::ffi::c_int,
}
#[repr(C)]
pub struct cuDevicePrimaryCtxRelease_params_st {
    pub dev: CUdevice,
}
#[repr(C)]
pub struct cuDevicePrimaryCtxRelease_v2_params_st {
    pub dev: CUdevice,
}
#[repr(C)]
pub struct cuDevicePrimaryCtxReset_params_st {
    pub dev: CUdevice,
}
#[repr(C)]
pub struct cuDevicePrimaryCtxReset_v2_params_st {
    pub dev: CUdevice,
}
#[repr(C)]
pub struct cuDevicePrimaryCtxRetain_params_st {
    pub pctx: *mut CUcontext,
    pub dev: CUdevice,
}
#[repr(C)]
pub struct cuDevicePrimaryCtxSetFlags_params_st {
    pub dev: CUdevice,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuDevicePrimaryCtxSetFlags_v2_params_st {
    pub dev: CUdevice,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuDeviceRegisterAsyncNotification_params_st {
    pub device: CUdevice,
    pub callbackFunc: CUasyncCallback,
    pub userData: *mut ::core::ffi::c_void,
    pub callback: *mut CUasyncCallbackHandle,
}
#[repr(C)]
pub struct cuDeviceSetGraphMemAttribute_params_st {
    pub device: CUdevice,
    pub attr: CUgraphMem_attribute,
    pub value: *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cuDeviceSetMemPool_params_st {
    pub dev: CUdevice,
    pub pool: CUmemoryPool,
}
#[repr(C)]
pub struct cuDeviceTotalMem_params_st {
    pub bytes: *mut ::core::ffi::c_uint,
    pub dev: CUdevice,
}
#[repr(C)]
pub struct cuDeviceTotalMem_v2_params_st {
    pub bytes: *mut usize,
    pub dev: CUdevice,
}
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuDeviceUnregisterAsyncNotification_params_st {
    pub device: CUdevice,
    pub callback: CUasyncCallbackHandle,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuDriverGetVersion_params_st {
    pub driverVersion: *mut ::core::ffi::c_int,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuEventCreate_params_st {
    pub phEvent: *mut CUevent,
    pub Flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuEventDestroy_params_st {
    pub hEvent: CUevent,
}
#[repr(C)]
pub struct cuEventDestroy_v2_params_st {
    pub hEvent: CUevent,
}
#[repr(C)]
pub struct cuEventElapsedTime_params_st {
    pub pMilliseconds: *mut f32,
    pub hStart: CUevent,
    pub hEnd: CUevent,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuEventElapsedTime_v2_params_st {
    pub pMilliseconds: *mut f32,
    pub hStart: CUevent,
    pub hEnd: CUevent,
}
#[repr(C)]
pub struct cuEventQuery_params_st {
    pub hEvent: CUevent,
}
#[repr(C)]
pub struct cuEventRecordWithFlags_params_st {
    pub hEvent: CUevent,
    pub hStream: CUstream,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuEventRecordWithFlags_ptsz_params_st {
    pub hEvent: CUevent,
    pub hStream: CUstream,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuEventRecord_params_st {
    pub hEvent: CUevent,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuEventRecord_ptsz_params_st {
    pub hEvent: CUevent,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuEventSynchronize_params_st {
    pub hEvent: CUevent,
}
#[repr(C)]
pub struct cuExternalMemoryGetMappedBuffer_params_st {
    pub devPtr: *mut CUdeviceptr,
    pub extMem: CUexternalMemory,
    pub bufferDesc: *const CUDA_EXTERNAL_MEMORY_BUFFER_DESC,
}
#[repr(C)]
pub struct cuExternalMemoryGetMappedMipmappedArray_params_st {
    pub mipmap: *mut CUmipmappedArray,
    pub extMem: CUexternalMemory,
    pub mipmapDesc: *const CUDA_EXTERNAL_MEMORY_MIPMAPPED_ARRAY_DESC,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuFlushGPUDirectRDMAWrites_params_st {
    pub target: CUflushGPUDirectRDMAWritesTarget,
    pub scope: CUflushGPUDirectRDMAWritesScope,
}
#[repr(C)]
pub struct cuFuncGetAttribute_params_st {
    pub pi: *mut ::core::ffi::c_int,
    pub attrib: CUfunction_attribute,
    pub hfunc: CUfunction,
}
#[repr(C)]
pub struct cuFuncGetModule_params_st {
    pub hmod: *mut CUmodule,
    pub hfunc: CUfunction,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuFuncGetName_params_st {
    pub name: *mut *const ::core::ffi::c_char,
    pub hfunc: CUfunction,
}
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuFuncGetParamInfo_params_st {
    pub func: CUfunction,
    pub paramIndex: usize,
    pub paramOffset: *mut usize,
    pub paramSize: *mut usize,
}
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuFuncIsLoaded_params_st {
    pub state: *mut CUfunctionLoadingState,
    pub function: CUfunction,
}
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuFuncLoad_params_st {
    pub function: CUfunction,
}
#[repr(C)]
pub struct cuFuncSetAttribute_params_st {
    pub hfunc: CUfunction,
    pub attrib: CUfunction_attribute,
    pub value: ::core::ffi::c_int,
}
#[repr(C)]
pub struct cuFuncSetBlockShape_params_st {
    pub hfunc: CUfunction,
    pub x: ::core::ffi::c_int,
    pub y: ::core::ffi::c_int,
    pub z: ::core::ffi::c_int,
}
#[repr(C)]
pub struct cuFuncSetCacheConfig_params_st {
    pub hfunc: CUfunction,
    pub config: CUfunc_cache,
}
#[repr(C)]
pub struct cuFuncSetSharedMemConfig_params_st {
    pub hfunc: CUfunction,
    pub config: CUsharedconfig,
}
#[repr(C)]
pub struct cuFuncSetSharedSize_params_st {
    pub hfunc: CUfunction,
    pub bytes: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuGetErrorName_params_st {
    pub error: CUresult,
    pub pStr: *mut *const ::core::ffi::c_char,
}
#[repr(C)]
pub struct cuGetErrorString_params_st {
    pub error: CUresult,
    pub pStr: *mut *const ::core::ffi::c_char,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuGetExportTable_params_st {
    pub ppExportTable: *mut *const ::core::ffi::c_void,
    pub pExportTableId: *const CUuuid,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGetProcAddress_params_st {
    pub symbol: *const ::core::ffi::c_char,
    pub pfn: *mut *mut ::core::ffi::c_void,
    pub cudaVersion: ::core::ffi::c_int,
    pub flags: cuuint64_t,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGetProcAddress_v2_params_st {
    pub symbol: *const ::core::ffi::c_char,
    pub pfn: *mut *mut ::core::ffi::c_void,
    pub cudaVersion: ::core::ffi::c_int,
    pub flags: cuuint64_t,
    pub symbolStatus: *mut CUdriverProcAddressQueryResult,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphAddBatchMemOpNode_params_st {
    pub phGraphNode: *mut CUgraphNode,
    pub hGraph: CUgraph,
    pub dependencies: *const CUgraphNode,
    pub numDependencies: usize,
    pub nodeParams: *const CUDA_BATCH_MEM_OP_NODE_PARAMS,
}
#[repr(C)]
pub struct cuGraphAddChildGraphNode_params_st {
    pub phGraphNode: *mut CUgraphNode,
    pub hGraph: CUgraph,
    pub dependencies: *const CUgraphNode,
    pub numDependencies: usize,
    pub childGraph: CUgraph,
}
#[repr(C)]
pub struct cuGraphAddDependencies_params_st {
    pub hGraph: CUgraph,
    pub from: *const CUgraphNode,
    pub to: *const CUgraphNode,
    pub numDependencies: usize,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphAddDependencies_v2_params_st {
    pub hGraph: CUgraph,
    pub from: *const CUgraphNode,
    pub to: *const CUgraphNode,
    pub edgeData: *const CUgraphEdgeData,
    pub numDependencies: usize,
}
#[repr(C)]
pub struct cuGraphAddEmptyNode_params_st {
    pub phGraphNode: *mut CUgraphNode,
    pub hGraph: CUgraph,
    pub dependencies: *const CUgraphNode,
    pub numDependencies: usize,
}
#[repr(C)]
pub struct cuGraphAddEventRecordNode_params_st {
    pub phGraphNode: *mut CUgraphNode,
    pub hGraph: CUgraph,
    pub dependencies: *const CUgraphNode,
    pub numDependencies: usize,
    pub event: CUevent,
}
#[repr(C)]
pub struct cuGraphAddEventWaitNode_params_st {
    pub phGraphNode: *mut CUgraphNode,
    pub hGraph: CUgraph,
    pub dependencies: *const CUgraphNode,
    pub numDependencies: usize,
    pub event: CUevent,
}
#[repr(C)]
pub struct cuGraphAddExternalSemaphoresSignalNode_params_st {
    pub phGraphNode: *mut CUgraphNode,
    pub hGraph: CUgraph,
    pub dependencies: *const CUgraphNode,
    pub numDependencies: usize,
    pub nodeParams: *const CUDA_EXT_SEM_SIGNAL_NODE_PARAMS,
}
#[repr(C)]
pub struct cuGraphAddExternalSemaphoresWaitNode_params_st {
    pub phGraphNode: *mut CUgraphNode,
    pub hGraph: CUgraph,
    pub dependencies: *const CUgraphNode,
    pub numDependencies: usize,
    pub nodeParams: *const CUDA_EXT_SEM_WAIT_NODE_PARAMS,
}
#[repr(C)]
pub struct cuGraphAddHostNode_params_st {
    pub phGraphNode: *mut CUgraphNode,
    pub hGraph: CUgraph,
    pub dependencies: *const CUgraphNode,
    pub numDependencies: usize,
    pub nodeParams: *const CUDA_HOST_NODE_PARAMS,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
#[repr(C)]
pub struct cuGraphAddKernelNode_params_st {
    pub phGraphNode: *mut CUgraphNode,
    pub hGraph: CUgraph,
    pub dependencies: *const CUgraphNode,
    pub numDependencies: usize,
    pub nodeParams: *const CUDA_KERNEL_NODE_PARAMS,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphAddKernelNode_params_st {
    pub phGraphNode: *mut CUgraphNode,
    pub hGraph: CUgraph,
    pub dependencies: *const CUgraphNode,
    pub numDependencies: usize,
    pub nodeParams: *const CUDA_KERNEL_NODE_PARAMS_v1,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphAddKernelNode_v2_params_st {
    pub phGraphNode: *mut CUgraphNode,
    pub hGraph: CUgraph,
    pub dependencies: *const CUgraphNode,
    pub numDependencies: usize,
    pub nodeParams: *const CUDA_KERNEL_NODE_PARAMS,
}
#[repr(C)]
pub struct cuGraphAddMemAllocNode_params_st {
    pub phGraphNode: *mut CUgraphNode,
    pub hGraph: CUgraph,
    pub dependencies: *const CUgraphNode,
    pub numDependencies: usize,
    pub nodeParams: *mut CUDA_MEM_ALLOC_NODE_PARAMS,
}
#[repr(C)]
pub struct cuGraphAddMemFreeNode_params_st {
    pub phGraphNode: *mut CUgraphNode,
    pub hGraph: CUgraph,
    pub dependencies: *const CUgraphNode,
    pub numDependencies: usize,
    pub dptr: CUdeviceptr,
}
#[repr(C)]
pub struct cuGraphAddMemcpyNode_params_st {
    pub phGraphNode: *mut CUgraphNode,
    pub hGraph: CUgraph,
    pub dependencies: *const CUgraphNode,
    pub numDependencies: usize,
    pub copyParams: *const CUDA_MEMCPY3D,
    pub ctx: CUcontext,
}
#[repr(C)]
pub struct cuGraphAddMemsetNode_params_st {
    pub phGraphNode: *mut CUgraphNode,
    pub hGraph: CUgraph,
    pub dependencies: *const CUgraphNode,
    pub numDependencies: usize,
    pub memsetParams: *const CUDA_MEMSET_NODE_PARAMS,
    pub ctx: CUcontext,
}
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphAddNode_params_st {
    pub phGraphNode: *mut CUgraphNode,
    pub hGraph: CUgraph,
    pub dependencies: *const CUgraphNode,
    pub numDependencies: usize,
    pub nodeParams: *mut CUgraphNodeParams,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphAddNode_v2_params_st {
    pub phGraphNode: *mut CUgraphNode,
    pub hGraph: CUgraph,
    pub dependencies: *const CUgraphNode,
    pub dependencyData: *const CUgraphEdgeData,
    pub numDependencies: usize,
    pub nodeParams: *mut CUgraphNodeParams,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphBatchMemOpNodeGetParams_params_st {
    pub hNode: CUgraphNode,
    pub nodeParams_out: *mut CUDA_BATCH_MEM_OP_NODE_PARAMS,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphBatchMemOpNodeSetParams_params_st {
    pub hNode: CUgraphNode,
    pub nodeParams: *const CUDA_BATCH_MEM_OP_NODE_PARAMS,
}
#[repr(C)]
pub struct cuGraphChildGraphNodeGetGraph_params_st {
    pub hNode: CUgraphNode,
    pub phGraph: *mut CUgraph,
}
#[repr(C)]
pub struct cuGraphClone_params_st {
    pub phGraphClone: *mut CUgraph,
    pub originalGraph: CUgraph,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphConditionalHandleCreate_params_st {
    pub pHandle_out: *mut CUgraphConditionalHandle,
    pub hGraph: CUgraph,
    pub ctx: CUcontext,
    pub defaultLaunchValue: ::core::ffi::c_uint,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuGraphCreate_params_st {
    pub phGraph: *mut CUgraph,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuGraphDebugDotPrint_params_st {
    pub hGraph: CUgraph,
    pub path: *const ::core::ffi::c_char,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuGraphDestroyNode_params_st {
    pub hNode: CUgraphNode,
}
#[repr(C)]
pub struct cuGraphDestroy_params_st {
    pub hGraph: CUgraph,
}
#[repr(C)]
pub struct cuGraphEventRecordNodeGetEvent_params_st {
    pub hNode: CUgraphNode,
    pub event_out: *mut CUevent,
}
#[repr(C)]
pub struct cuGraphEventRecordNodeSetEvent_params_st {
    pub hNode: CUgraphNode,
    pub event: CUevent,
}
#[repr(C)]
pub struct cuGraphEventWaitNodeGetEvent_params_st {
    pub hNode: CUgraphNode,
    pub event_out: *mut CUevent,
}
#[repr(C)]
pub struct cuGraphEventWaitNodeSetEvent_params_st {
    pub hNode: CUgraphNode,
    pub event: CUevent,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphExecBatchMemOpNodeSetParams_params_st {
    pub hGraphExec: CUgraphExec,
    pub hNode: CUgraphNode,
    pub nodeParams: *const CUDA_BATCH_MEM_OP_NODE_PARAMS,
}
#[repr(C)]
pub struct cuGraphExecChildGraphNodeSetParams_params_st {
    pub hGraphExec: CUgraphExec,
    pub hNode: CUgraphNode,
    pub childGraph: CUgraph,
}
#[repr(C)]
pub struct cuGraphExecDestroy_params_st {
    pub hGraphExec: CUgraphExec,
}
#[repr(C)]
pub struct cuGraphExecEventRecordNodeSetEvent_params_st {
    pub hGraphExec: CUgraphExec,
    pub hNode: CUgraphNode,
    pub event: CUevent,
}
#[repr(C)]
pub struct cuGraphExecEventWaitNodeSetEvent_params_st {
    pub hGraphExec: CUgraphExec,
    pub hNode: CUgraphNode,
    pub event: CUevent,
}
#[repr(C)]
pub struct cuGraphExecExternalSemaphoresSignalNodeSetParams_params_st {
    pub hGraphExec: CUgraphExec,
    pub hNode: CUgraphNode,
    pub nodeParams: *const CUDA_EXT_SEM_SIGNAL_NODE_PARAMS,
}
#[repr(C)]
pub struct cuGraphExecExternalSemaphoresWaitNodeSetParams_params_st {
    pub hGraphExec: CUgraphExec,
    pub hNode: CUgraphNode,
    pub nodeParams: *const CUDA_EXT_SEM_WAIT_NODE_PARAMS,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphExecGetFlags_params_st {
    pub hGraphExec: CUgraphExec,
    pub flags: *mut cuuint64_t,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
pub struct cuGraphExecGetId_params_st {
    pub hGraphExec: CUgraphExec,
    pub graphId: *mut ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuGraphExecHostNodeSetParams_params_st {
    pub hGraphExec: CUgraphExec,
    pub hNode: CUgraphNode,
    pub nodeParams: *const CUDA_HOST_NODE_PARAMS,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
#[repr(C)]
pub struct cuGraphExecKernelNodeSetParams_params_st {
    pub hGraphExec: CUgraphExec,
    pub hNode: CUgraphNode,
    pub nodeParams: *const CUDA_KERNEL_NODE_PARAMS,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphExecKernelNodeSetParams_params_st {
    pub hGraphExec: CUgraphExec,
    pub hNode: CUgraphNode,
    pub nodeParams: *const CUDA_KERNEL_NODE_PARAMS_v1,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphExecKernelNodeSetParams_v2_params_st {
    pub hGraphExec: CUgraphExec,
    pub hNode: CUgraphNode,
    pub nodeParams: *const CUDA_KERNEL_NODE_PARAMS,
}
#[repr(C)]
pub struct cuGraphExecMemcpyNodeSetParams_params_st {
    pub hGraphExec: CUgraphExec,
    pub hNode: CUgraphNode,
    pub copyParams: *const CUDA_MEMCPY3D,
    pub ctx: CUcontext,
}
#[repr(C)]
pub struct cuGraphExecMemsetNodeSetParams_params_st {
    pub hGraphExec: CUgraphExec,
    pub hNode: CUgraphNode,
    pub memsetParams: *const CUDA_MEMSET_NODE_PARAMS,
    pub ctx: CUcontext,
}
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphExecNodeSetParams_params_st {
    pub hGraphExec: CUgraphExec,
    pub hNode: CUgraphNode,
    pub nodeParams: *mut CUgraphNodeParams,
}
#[repr(C)]
pub struct cuGraphExecUpdate_params_st {
    pub hGraphExec: CUgraphExec,
    pub hGraph: CUgraph,
    pub hErrorNode_out: *mut CUgraphNode,
    pub updateResult_out: *mut CUgraphExecUpdateResult,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphExecUpdate_v2_params_st {
    pub hGraphExec: CUgraphExec,
    pub hGraph: CUgraph,
    pub resultInfo: *mut CUgraphExecUpdateResultInfo,
}
#[repr(C)]
pub struct cuGraphExternalSemaphoresSignalNodeGetParams_params_st {
    pub hNode: CUgraphNode,
    pub params_out: *mut CUDA_EXT_SEM_SIGNAL_NODE_PARAMS,
}
#[repr(C)]
pub struct cuGraphExternalSemaphoresSignalNodeSetParams_params_st {
    pub hNode: CUgraphNode,
    pub nodeParams: *const CUDA_EXT_SEM_SIGNAL_NODE_PARAMS,
}
#[repr(C)]
pub struct cuGraphExternalSemaphoresWaitNodeGetParams_params_st {
    pub hNode: CUgraphNode,
    pub params_out: *mut CUDA_EXT_SEM_WAIT_NODE_PARAMS,
}
#[repr(C)]
pub struct cuGraphExternalSemaphoresWaitNodeSetParams_params_st {
    pub hNode: CUgraphNode,
    pub nodeParams: *const CUDA_EXT_SEM_WAIT_NODE_PARAMS,
}
#[repr(C)]
pub struct cuGraphGetEdges_params_st {
    pub hGraph: CUgraph,
    pub from: *mut CUgraphNode,
    pub to: *mut CUgraphNode,
    pub numEdges: *mut usize,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphGetEdges_v2_params_st {
    pub hGraph: CUgraph,
    pub from: *mut CUgraphNode,
    pub to: *mut CUgraphNode,
    pub edgeData: *mut CUgraphEdgeData,
    pub numEdges: *mut usize,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
pub struct cuGraphGetId_params_st {
    pub hGraph: CUgraph,
    pub graphId: *mut ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuGraphGetNodes_params_st {
    pub hGraph: CUgraph,
    pub nodes: *mut CUgraphNode,
    pub numNodes: *mut usize,
}
#[repr(C)]
pub struct cuGraphGetRootNodes_params_st {
    pub hGraph: CUgraph,
    pub rootNodes: *mut CUgraphNode,
    pub numRootNodes: *mut usize,
}
#[repr(C)]
pub struct cuGraphHostNodeGetParams_params_st {
    pub hNode: CUgraphNode,
    pub nodeParams: *mut CUDA_HOST_NODE_PARAMS,
}
#[repr(C)]
pub struct cuGraphHostNodeSetParams_params_st {
    pub hNode: CUgraphNode,
    pub nodeParams: *const CUDA_HOST_NODE_PARAMS,
}
#[repr(C)]
pub struct cuGraphInstantiateWithFlags_params_st {
    pub phGraphExec: *mut CUgraphExec,
    pub hGraph: CUgraph,
    pub flags: ::core::ffi::c_ulonglong,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphInstantiateWithParams_params_st {
    pub phGraphExec: *mut CUgraphExec,
    pub hGraph: CUgraph,
    pub instantiateParams: *mut CUDA_GRAPH_INSTANTIATE_PARAMS,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphInstantiateWithParams_ptsz_params_st {
    pub phGraphExec: *mut CUgraphExec,
    pub hGraph: CUgraph,
    pub instantiateParams: *mut CUDA_GRAPH_INSTANTIATE_PARAMS,
}
#[repr(C)]
pub struct cuGraphInstantiate_params_st {
    pub phGraphExec: *mut CUgraphExec,
    pub hGraph: CUgraph,
    pub phErrorNode: *mut CUgraphNode,
    pub logBuffer: *mut ::core::ffi::c_char,
    pub bufferSize: usize,
}
#[repr(C)]
pub struct cuGraphInstantiate_v2_params_st {
    pub phGraphExec: *mut CUgraphExec,
    pub hGraph: CUgraph,
    pub phErrorNode: *mut CUgraphNode,
    pub logBuffer: *mut ::core::ffi::c_char,
    pub bufferSize: usize,
}
#[repr(C)]
pub struct cuGraphKernelNodeCopyAttributes_params_st {
    pub dst: CUgraphNode,
    pub src: CUgraphNode,
}
#[repr(C)]
pub struct cuGraphKernelNodeGetAttribute_params_st {
    pub hNode: CUgraphNode,
    pub attr: CUkernelNodeAttrID,
    pub value_out: *mut CUkernelNodeAttrValue,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
#[repr(C)]
pub struct cuGraphKernelNodeGetParams_params_st {
    pub hNode: CUgraphNode,
    pub nodeParams: *mut CUDA_KERNEL_NODE_PARAMS,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphKernelNodeGetParams_params_st {
    pub hNode: CUgraphNode,
    pub nodeParams: *mut CUDA_KERNEL_NODE_PARAMS_v1,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphKernelNodeGetParams_v2_params_st {
    pub hNode: CUgraphNode,
    pub nodeParams: *mut CUDA_KERNEL_NODE_PARAMS,
}
#[repr(C)]
pub struct cuGraphKernelNodeSetAttribute_params_st {
    pub hNode: CUgraphNode,
    pub attr: CUkernelNodeAttrID,
    pub value: *const CUkernelNodeAttrValue,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
#[repr(C)]
pub struct cuGraphKernelNodeSetParams_params_st {
    pub hNode: CUgraphNode,
    pub nodeParams: *const CUDA_KERNEL_NODE_PARAMS,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphKernelNodeSetParams_params_st {
    pub hNode: CUgraphNode,
    pub nodeParams: *const CUDA_KERNEL_NODE_PARAMS_v1,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphKernelNodeSetParams_v2_params_st {
    pub hNode: CUgraphNode,
    pub nodeParams: *const CUDA_KERNEL_NODE_PARAMS,
}
#[repr(C)]
pub struct cuGraphLaunch_params_st {
    pub hGraph: CUgraphExec,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuGraphLaunch_ptsz_params_st {
    pub hGraphExec: CUgraphExec,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuGraphMemAllocNodeGetParams_params_st {
    pub hNode: CUgraphNode,
    pub params_out: *mut CUDA_MEM_ALLOC_NODE_PARAMS,
}
#[repr(C)]
pub struct cuGraphMemFreeNodeGetParams_params_st {
    pub hNode: CUgraphNode,
    pub dptr_out: *mut CUdeviceptr,
}
#[repr(C)]
pub struct cuGraphMemcpyNodeGetParams_params_st {
    pub hNode: CUgraphNode,
    pub nodeParams: *mut CUDA_MEMCPY3D,
}
#[repr(C)]
pub struct cuGraphMemcpyNodeSetParams_params_st {
    pub hNode: CUgraphNode,
    pub nodeParams: *const CUDA_MEMCPY3D,
}
#[repr(C)]
pub struct cuGraphMemsetNodeGetParams_params_st {
    pub hNode: CUgraphNode,
    pub nodeParams: *mut CUDA_MEMSET_NODE_PARAMS,
}
#[repr(C)]
pub struct cuGraphMemsetNodeSetParams_params_st {
    pub hNode: CUgraphNode,
    pub nodeParams: *const CUDA_MEMSET_NODE_PARAMS,
}
#[repr(C)]
pub struct cuGraphNodeFindInClone_params_st {
    pub phNode: *mut CUgraphNode,
    pub hOriginalNode: CUgraphNode,
    pub hClonedGraph: CUgraph,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
pub struct cuGraphNodeGetContainingGraph_params_st {
    pub hNode: CUgraphNode,
    pub phGraph: *mut CUgraph,
}
#[repr(C)]
pub struct cuGraphNodeGetDependencies_params_st {
    pub hNode: CUgraphNode,
    pub dependencies: *mut CUgraphNode,
    pub numDependencies: *mut usize,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphNodeGetDependencies_v2_params_st {
    pub hNode: CUgraphNode,
    pub dependencies: *mut CUgraphNode,
    pub edgeData: *mut CUgraphEdgeData,
    pub numDependencies: *mut usize,
}
#[repr(C)]
pub struct cuGraphNodeGetDependentNodes_params_st {
    pub hNode: CUgraphNode,
    pub dependentNodes: *mut CUgraphNode,
    pub numDependentNodes: *mut usize,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphNodeGetDependentNodes_v2_params_st {
    pub hNode: CUgraphNode,
    pub dependentNodes: *mut CUgraphNode,
    pub edgeData: *mut CUgraphEdgeData,
    pub numDependentNodes: *mut usize,
}
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphNodeGetEnabled_params_st {
    pub hGraphExec: CUgraphExec,
    pub hNode: CUgraphNode,
    pub isEnabled: *mut ::core::ffi::c_uint,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
pub struct cuGraphNodeGetLocalId_params_st {
    pub hNode: CUgraphNode,
    pub nodeId: *mut ::core::ffi::c_uint,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
pub struct cuGraphNodeGetToolsId_params_st {
    pub hNode: CUgraphNode,
    pub toolsNodeId: *mut ::core::ffi::c_ulonglong,
}
#[repr(C)]
pub struct cuGraphNodeGetType_params_st {
    pub hNode: CUgraphNode,
    pub type_: *mut CUgraphNodeType,
}
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphNodeSetEnabled_params_st {
    pub hGraphExec: CUgraphExec,
    pub hNode: CUgraphNode,
    pub isEnabled: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphNodeSetParams_params_st {
    pub hNode: CUgraphNode,
    pub nodeParams: *mut CUgraphNodeParams,
}
#[repr(C)]
pub struct cuGraphReleaseUserObject_params_st {
    pub graph: CUgraph,
    pub object: CUuserObject,
    pub count: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuGraphRemoveDependencies_params_st {
    pub hGraph: CUgraph,
    pub from: *const CUgraphNode,
    pub to: *const CUgraphNode,
    pub numDependencies: usize,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGraphRemoveDependencies_v2_params_st {
    pub hGraph: CUgraph,
    pub from: *const CUgraphNode,
    pub to: *const CUgraphNode,
    pub edgeData: *const CUgraphEdgeData,
    pub numDependencies: usize,
}
#[repr(C)]
pub struct cuGraphRetainUserObject_params_st {
    pub graph: CUgraph,
    pub object: CUuserObject,
    pub count: ::core::ffi::c_uint,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuGraphUpload_params_st {
    pub hGraph: CUgraphExec,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuGraphUpload_ptsz_params_st {
    pub hGraphExec: CUgraphExec,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuGraphicsMapResources_params_st {
    pub count: ::core::ffi::c_uint,
    pub resources: *mut CUgraphicsResource,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuGraphicsMapResources_ptsz_params_st {
    pub count: ::core::ffi::c_uint,
    pub resources: *mut CUgraphicsResource,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuGraphicsResourceGetMappedMipmappedArray_params_st {
    pub pMipmappedArray: *mut CUmipmappedArray,
    pub resource: CUgraphicsResource,
}
#[repr(C)]
pub struct cuGraphicsResourceGetMappedPointer_params_st {
    pub pDevPtr: *mut CUdeviceptr_v1,
    pub pSize: *mut ::core::ffi::c_uint,
    pub resource: CUgraphicsResource,
}
#[repr(C)]
pub struct cuGraphicsResourceGetMappedPointer_v2_params_st {
    pub pDevPtr: *mut CUdeviceptr,
    pub pSize: *mut usize,
    pub resource: CUgraphicsResource,
}
#[repr(C)]
pub struct cuGraphicsResourceSetMapFlags_params_st {
    pub resource: CUgraphicsResource,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuGraphicsResourceSetMapFlags_v2_params_st {
    pub resource: CUgraphicsResource,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuGraphicsSubResourceGetMappedArray_params_st {
    pub pArray: *mut CUarray,
    pub resource: CUgraphicsResource,
    pub arrayIndex: ::core::ffi::c_uint,
    pub mipLevel: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuGraphicsUnmapResources_params_st {
    pub count: ::core::ffi::c_uint,
    pub resources: *mut CUgraphicsResource,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuGraphicsUnmapResources_ptsz_params_st {
    pub count: ::core::ffi::c_uint,
    pub resources: *mut CUgraphicsResource,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuGraphicsUnregisterResource_params_st {
    pub resource: CUgraphicsResource,
}
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGreenCtxCreate_params_st {
    pub phCtx: *mut CUgreenCtx,
    pub desc: CUdevResourceDesc,
    pub dev: CUdevice,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGreenCtxDestroy_params_st {
    pub hCtx: CUgreenCtx,
}
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGreenCtxGetDevResource_params_st {
    pub hCtx: CUgreenCtx,
    pub resource: *mut CUdevResource,
    pub type_: CUdevResourceType,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cuGreenCtxGetId_params_st {
    pub greenCtx: CUgreenCtx,
    pub greenCtxId: *mut ::core::ffi::c_ulonglong,
}
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGreenCtxRecordEvent_params_st {
    pub hCtx: CUgreenCtx,
    pub hEvent: CUevent,
}
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGreenCtxStreamCreate_params_st {
    pub phStream: *mut CUstream,
    pub greenCtx: CUgreenCtx,
    pub flags: ::core::ffi::c_uint,
    pub priority: ::core::ffi::c_int,
}
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuGreenCtxWaitEvent_params_st {
    pub hCtx: CUgreenCtx,
    pub hEvent: CUevent,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuImportExternalMemory_params_st {
    pub extMem_out: *mut CUexternalMemory,
    pub memHandleDesc: *const CUDA_EXTERNAL_MEMORY_HANDLE_DESC,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuImportExternalSemaphore_params_st {
    pub extSem_out: *mut CUexternalSemaphore,
    pub semHandleDesc: *const CUDA_EXTERNAL_SEMAPHORE_HANDLE_DESC,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuInit_params_st {
    pub Flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuIpcCloseMemHandle_params_st {
    pub dptr: CUdeviceptr,
}
#[repr(C)]
pub struct cuIpcGetEventHandle_params_st {
    pub pHandle: *mut CUipcEventHandle,
    pub event: CUevent,
}
#[repr(C)]
pub struct cuIpcGetMemHandle_params_st {
    pub pHandle: *mut CUipcMemHandle,
    pub dptr: CUdeviceptr,
}
#[repr(C)]
pub struct cuIpcOpenEventHandle_params_st {
    pub phEvent: *mut CUevent,
    pub handle: CUipcEventHandle,
}
#[repr(C)]
pub struct cuIpcOpenMemHandle_params_st {
    pub pdptr: *mut CUdeviceptr,
    pub handle: CUipcMemHandle,
    pub Flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuIpcOpenMemHandle_v2_params_st {
    pub pdptr: *mut CUdeviceptr,
    pub handle: CUipcMemHandle,
    pub Flags: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuKernelGetAttribute_params_st {
    pub pi: *mut ::core::ffi::c_int,
    pub attrib: CUfunction_attribute,
    pub kernel: CUkernel,
    pub dev: CUdevice,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuKernelGetFunction_params_st {
    pub pFunc: *mut CUfunction,
    pub kernel: CUkernel,
}
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuKernelGetLibrary_params_st {
    pub pLib: *mut CUlibrary,
    pub kernel: CUkernel,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuKernelGetName_params_st {
    pub name: *mut *const ::core::ffi::c_char,
    pub hfunc: CUkernel,
}
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuKernelGetParamInfo_params_st {
    pub kernel: CUkernel,
    pub paramIndex: usize,
    pub paramOffset: *mut usize,
    pub paramSize: *mut usize,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuKernelSetAttribute_params_st {
    pub attrib: CUfunction_attribute,
    pub val: ::core::ffi::c_int,
    pub kernel: CUkernel,
    pub dev: CUdevice,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuKernelSetCacheConfig_params_st {
    pub kernel: CUkernel,
    pub config: CUfunc_cache,
    pub dev: CUdevice,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuLaunchCooperativeKernelMultiDevice_params_st {
    pub launchParamsList: *mut CUDA_LAUNCH_PARAMS,
    pub numDevices: ::core::ffi::c_uint,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuLaunchCooperativeKernel_params_st {
    pub f: CUfunction,
    pub gridDimX: ::core::ffi::c_uint,
    pub gridDimY: ::core::ffi::c_uint,
    pub gridDimZ: ::core::ffi::c_uint,
    pub blockDimX: ::core::ffi::c_uint,
    pub blockDimY: ::core::ffi::c_uint,
    pub blockDimZ: ::core::ffi::c_uint,
    pub sharedMemBytes: ::core::ffi::c_uint,
    pub hStream: CUstream,
    pub kernelParams: *mut *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cuLaunchCooperativeKernel_ptsz_params_st {
    pub f: CUfunction,
    pub gridDimX: ::core::ffi::c_uint,
    pub gridDimY: ::core::ffi::c_uint,
    pub gridDimZ: ::core::ffi::c_uint,
    pub blockDimX: ::core::ffi::c_uint,
    pub blockDimY: ::core::ffi::c_uint,
    pub blockDimZ: ::core::ffi::c_uint,
    pub sharedMemBytes: ::core::ffi::c_uint,
    pub hStream: CUstream,
    pub kernelParams: *mut *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cuLaunchGridAsync_params_st {
    pub f: CUfunction,
    pub grid_width: ::core::ffi::c_int,
    pub grid_height: ::core::ffi::c_int,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuLaunchGrid_params_st {
    pub f: CUfunction,
    pub grid_width: ::core::ffi::c_int,
    pub grid_height: ::core::ffi::c_int,
}
#[repr(C)]
pub struct cuLaunchHostFunc_params_st {
    pub hStream: CUstream,
    pub fn_: CUhostFn,
    pub userData: *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cuLaunchHostFunc_ptsz_params_st {
    pub hStream: CUstream,
    pub fn_: CUhostFn,
    pub userData: *mut ::core::ffi::c_void,
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuLaunchKernelEx_params_st {
    pub config: *const CUlaunchConfig,
    pub f: CUfunction,
    pub kernelParams: *mut *mut ::core::ffi::c_void,
    pub extra: *mut *mut ::core::ffi::c_void,
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuLaunchKernelEx_ptsz_params_st {
    pub config: *const CUlaunchConfig,
    pub f: CUfunction,
    pub kernelParams: *mut *mut ::core::ffi::c_void,
    pub extra: *mut *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cuLaunchKernel_params_st {
    pub f: CUfunction,
    pub gridDimX: ::core::ffi::c_uint,
    pub gridDimY: ::core::ffi::c_uint,
    pub gridDimZ: ::core::ffi::c_uint,
    pub blockDimX: ::core::ffi::c_uint,
    pub blockDimY: ::core::ffi::c_uint,
    pub blockDimZ: ::core::ffi::c_uint,
    pub sharedMemBytes: ::core::ffi::c_uint,
    pub hStream: CUstream,
    pub kernelParams: *mut *mut ::core::ffi::c_void,
    pub extra: *mut *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cuLaunchKernel_ptsz_params_st {
    pub f: CUfunction,
    pub gridDimX: ::core::ffi::c_uint,
    pub gridDimY: ::core::ffi::c_uint,
    pub gridDimZ: ::core::ffi::c_uint,
    pub blockDimX: ::core::ffi::c_uint,
    pub blockDimY: ::core::ffi::c_uint,
    pub blockDimZ: ::core::ffi::c_uint,
    pub sharedMemBytes: ::core::ffi::c_uint,
    pub hStream: CUstream,
    pub kernelParams: *mut *mut ::core::ffi::c_void,
    pub extra: *mut *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cuLaunch_params_st {
    pub f: CUfunction,
}
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuLibraryEnumerateKernels_params_st {
    pub kernels: *mut CUkernel,
    pub numKernels: ::core::ffi::c_uint,
    pub lib: CUlibrary,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuLibraryGetGlobal_params_st {
    pub dptr: *mut CUdeviceptr,
    pub bytes: *mut usize,
    pub library: CUlibrary,
    pub name: *const ::core::ffi::c_char,
}
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuLibraryGetKernelCount_params_st {
    pub count: *mut ::core::ffi::c_uint,
    pub lib: CUlibrary,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuLibraryGetKernel_params_st {
    pub pKernel: *mut CUkernel,
    pub library: CUlibrary,
    pub name: *const ::core::ffi::c_char,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuLibraryGetManaged_params_st {
    pub dptr: *mut CUdeviceptr,
    pub bytes: *mut usize,
    pub library: CUlibrary,
    pub name: *const ::core::ffi::c_char,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuLibraryGetModule_params_st {
    pub pMod: *mut CUmodule,
    pub library: CUlibrary,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuLibraryGetUnifiedFunction_params_st {
    pub fptr: *mut *mut ::core::ffi::c_void,
    pub library: CUlibrary,
    pub symbol: *const ::core::ffi::c_char,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuLibraryLoadData_params_st {
    pub library: *mut CUlibrary,
    pub code: *const ::core::ffi::c_void,
    pub jitOptions: *mut CUjit_option,
    pub jitOptionsValues: *mut *mut ::core::ffi::c_void,
    pub numJitOptions: ::core::ffi::c_uint,
    pub libraryOptions: *mut CUlibraryOption,
    pub libraryOptionValues: *mut *mut ::core::ffi::c_void,
    pub numLibraryOptions: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuLibraryLoadFromFile_params_st {
    pub library: *mut CUlibrary,
    pub fileName: *const ::core::ffi::c_char,
    pub jitOptions: *mut CUjit_option,
    pub jitOptionsValues: *mut *mut ::core::ffi::c_void,
    pub numJitOptions: ::core::ffi::c_uint,
    pub libraryOptions: *mut CUlibraryOption,
    pub libraryOptionValues: *mut *mut ::core::ffi::c_void,
    pub numLibraryOptions: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuLibraryUnload_params_st {
    pub library: CUlibrary,
}
#[repr(C)]
pub struct cuLinkAddData_params_st {
    pub state: CUlinkState,
    pub type_: CUjitInputType,
    pub data: *mut ::core::ffi::c_void,
    pub size: usize,
    pub name: *const ::core::ffi::c_char,
    pub numOptions: ::core::ffi::c_uint,
    pub options: *mut CUjit_option,
    pub optionValues: *mut *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cuLinkAddData_v2_params_st {
    pub state: CUlinkState,
    pub type_: CUjitInputType,
    pub data: *mut ::core::ffi::c_void,
    pub size: usize,
    pub name: *const ::core::ffi::c_char,
    pub numOptions: ::core::ffi::c_uint,
    pub options: *mut CUjit_option,
    pub optionValues: *mut *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cuLinkAddFile_params_st {
    pub state: CUlinkState,
    pub type_: CUjitInputType,
    pub path: *const ::core::ffi::c_char,
    pub numOptions: ::core::ffi::c_uint,
    pub options: *mut CUjit_option,
    pub optionValues: *mut *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cuLinkAddFile_v2_params_st {
    pub state: CUlinkState,
    pub type_: CUjitInputType,
    pub path: *const ::core::ffi::c_char,
    pub numOptions: ::core::ffi::c_uint,
    pub options: *mut CUjit_option,
    pub optionValues: *mut *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cuLinkComplete_params_st {
    pub state: CUlinkState,
    pub cubinOut: *mut *mut ::core::ffi::c_void,
    pub sizeOut: *mut usize,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuLinkCreate_params_st {
    pub numOptions: ::core::ffi::c_uint,
    pub options: *mut CUjit_option,
    pub optionValues: *mut *mut ::core::ffi::c_void,
    pub stateOut: *mut CUlinkState,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuLinkCreate_v2_params_st {
    pub numOptions: ::core::ffi::c_uint,
    pub options: *mut CUjit_option,
    pub optionValues: *mut *mut ::core::ffi::c_void,
    pub stateOut: *mut CUlinkState,
}
#[repr(C)]
pub struct cuLinkDestroy_params_st {
    pub state: CUlinkState,
}
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuLogsCurrent_params_st {
    pub iterator_out: *mut CUlogIterator,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuLogsDumpToFile_params_st {
    pub iterator: *mut CUlogIterator,
    pub pathToFile: *const ::core::ffi::c_char,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuLogsDumpToMemory_params_st {
    pub iterator: *mut CUlogIterator,
    pub buffer: *mut ::core::ffi::c_char,
    pub size: *mut usize,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cuLogsRegisterCallback_params_st {
    pub callbackFunc: CUlogsCallback,
    pub userData: *mut ::core::ffi::c_void,
    pub callback_out: *mut CUlogsCallbackHandle,
}
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cuLogsUnregisterCallback_params_st {
    pub callback: CUlogsCallbackHandle,
}
#[repr(C)]
pub struct cuMemAddressFree_params_st {
    pub ptr: CUdeviceptr,
    pub size: usize,
}
#[repr(C)]
pub struct cuMemAddressReserve_params_st {
    pub ptr: *mut CUdeviceptr,
    pub size: usize,
    pub alignment: usize,
    pub addr: CUdeviceptr,
    pub flags: ::core::ffi::c_ulonglong,
}
#[repr(C)]
pub struct cuMemAdvise_params_st {
    pub devPtr: CUdeviceptr,
    pub count: usize,
    pub advice: CUmem_advise,
    pub device: CUdevice,
}
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuMemAdvise_v2_params_st {
    pub devPtr: CUdeviceptr,
    pub count: usize,
    pub advice: CUmem_advise,
    pub location: CUmemLocation,
}
#[repr(C)]
pub struct cuMemAllocAsync_params_st {
    pub dptr: *mut CUdeviceptr,
    pub bytesize: usize,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemAllocAsync_ptsz_params_st {
    pub dptr: *mut CUdeviceptr,
    pub bytesize: usize,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemAllocFromPoolAsync_params_st {
    pub dptr: *mut CUdeviceptr,
    pub bytesize: usize,
    pub pool: CUmemoryPool,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemAllocFromPoolAsync_ptsz_params_st {
    pub dptr: *mut CUdeviceptr,
    pub bytesize: usize,
    pub pool: CUmemoryPool,
    pub hStream: CUstream,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemAllocHost_params_st {
    pub pp: *mut *mut ::core::ffi::c_void,
    pub bytesize: ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemAllocHost_v2_params_st {
    pub pp: *mut *mut ::core::ffi::c_void,
    pub bytesize: usize,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemAllocManaged_params_st {
    pub dptr: *mut CUdeviceptr,
    pub bytesize: usize,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemAllocPitch_params_st {
    pub dptr: *mut CUdeviceptr_v1,
    pub pPitch: *mut ::core::ffi::c_uint,
    pub WidthInBytes: ::core::ffi::c_uint,
    pub Height: ::core::ffi::c_uint,
    pub ElementSizeBytes: ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemAllocPitch_v2_params_st {
    pub dptr: *mut CUdeviceptr,
    pub pPitch: *mut usize,
    pub WidthInBytes: usize,
    pub Height: usize,
    pub ElementSizeBytes: ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemAlloc_params_st {
    pub dptr: *mut CUdeviceptr_v1,
    pub bytesize: ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemAlloc_v2_params_st {
    pub dptr: *mut CUdeviceptr,
    pub bytesize: usize,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuMemBatchDecompressAsync_params_st {
    pub paramsArray: *mut CUmemDecompressParams,
    pub count: usize,
    pub flags: ::core::ffi::c_uint,
    pub errorIndex: *mut usize,
    pub stream: CUstream,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuMemBatchDecompressAsync_ptsz_params_st {
    pub paramsArray: *mut CUmemDecompressParams,
    pub count: usize,
    pub flags: ::core::ffi::c_uint,
    pub errorIndex: *mut usize,
    pub stream: CUstream,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemCreate_params_st {
    pub handle: *mut CUmemGenericAllocationHandle,
    pub size: usize,
    pub prop: *const CUmemAllocationProp,
    pub flags: ::core::ffi::c_ulonglong,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cuMemDiscardAndPrefetchBatchAsync_params_st {
    pub dptrs: *mut CUdeviceptr,
    pub sizes: *mut usize,
    pub count: usize,
    pub prefetchLocs: *mut CUmemLocation,
    pub prefetchLocIdxs: *mut usize,
    pub numPrefetchLocs: usize,
    pub flags: ::core::ffi::c_ulonglong,
    pub hStream: CUstream,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cuMemDiscardAndPrefetchBatchAsync_ptsz_params_st {
    pub dptrs: *mut CUdeviceptr,
    pub sizes: *mut usize,
    pub count: usize,
    pub prefetchLocs: *mut CUmemLocation,
    pub prefetchLocIdxs: *mut usize,
    pub numPrefetchLocs: usize,
    pub flags: ::core::ffi::c_ulonglong,
    pub hStream: CUstream,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cuMemDiscardBatchAsync_params_st {
    pub dptrs: *mut CUdeviceptr,
    pub sizes: *mut usize,
    pub count: usize,
    pub flags: ::core::ffi::c_ulonglong,
    pub hStream: CUstream,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cuMemDiscardBatchAsync_ptsz_params_st {
    pub dptrs: *mut CUdeviceptr,
    pub sizes: *mut usize,
    pub count: usize,
    pub flags: ::core::ffi::c_ulonglong,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemExportToShareableHandle_params_st {
    pub shareableHandle: *mut ::core::ffi::c_void,
    pub handle: CUmemGenericAllocationHandle,
    pub handleType: CUmemAllocationHandleType,
    pub flags: ::core::ffi::c_ulonglong,
}
#[repr(C)]
pub struct cuMemFreeAsync_params_st {
    pub dptr: CUdeviceptr,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemFreeAsync_ptsz_params_st {
    pub dptr: CUdeviceptr,
    pub hStream: CUstream,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemFreeHost_params_st {
    pub p: *mut ::core::ffi::c_void,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemFree_params_st {
    pub dptr: CUdeviceptr_v1,
}
#[repr(C)]
pub struct cuMemFree_v2_params_st {
    pub dptr: CUdeviceptr,
}
#[repr(C)]
pub struct cuMemGetAccess_params_st {
    pub flags: *mut ::core::ffi::c_ulonglong,
    pub location: *const CUmemLocation,
    pub ptr: CUdeviceptr,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemGetAddressRange_params_st {
    pub pbase: *mut CUdeviceptr_v1,
    pub psize: *mut ::core::ffi::c_uint,
    pub dptr: CUdeviceptr_v1,
}
#[repr(C)]
pub struct cuMemGetAddressRange_v2_params_st {
    pub pbase: *mut CUdeviceptr,
    pub psize: *mut usize,
    pub dptr: CUdeviceptr,
}
#[repr(C)]
pub struct cuMemGetAllocationGranularity_params_st {
    pub granularity: *mut usize,
    pub prop: *const CUmemAllocationProp,
    pub option: CUmemAllocationGranularity_flags,
}
#[repr(C)]
pub struct cuMemGetAllocationPropertiesFromHandle_params_st {
    pub prop: *mut CUmemAllocationProp,
    pub handle: CUmemGenericAllocationHandle,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cuMemGetDefaultMemPool_params_st {
    pub pool_out: *mut CUmemoryPool,
    pub location: *mut CUmemLocation,
    pub type_: CUmemAllocationType,
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuMemGetHandleForAddressRange_params_st {
    pub handle: *mut ::core::ffi::c_void,
    pub dptr: CUdeviceptr,
    pub size: usize,
    pub handleType: CUmemRangeHandleType,
    pub flags: ::core::ffi::c_ulonglong,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemGetInfo_params_st {
    pub free: *mut ::core::ffi::c_uint,
    pub total: *mut ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemGetInfo_v2_params_st {
    pub free: *mut usize,
    pub total: *mut usize,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cuMemGetMemPool_params_st {
    pub pool: *mut CUmemoryPool,
    pub location: *mut CUmemLocation,
    pub type_: CUmemAllocationType,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemHostAlloc_params_st {
    pub pp: *mut *mut ::core::ffi::c_void,
    pub bytesize: usize,
    pub Flags: ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemHostGetDevicePointer_params_st {
    pub pdptr: *mut CUdeviceptr_v1,
    pub p: *mut ::core::ffi::c_void,
    pub Flags: ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemHostGetDevicePointer_v2_params_st {
    pub pdptr: *mut CUdeviceptr,
    pub p: *mut ::core::ffi::c_void,
    pub Flags: ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemHostGetFlags_params_st {
    pub pFlags: *mut ::core::ffi::c_uint,
    pub p: *mut ::core::ffi::c_void,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemHostRegister_params_st {
    pub p: *mut ::core::ffi::c_void,
    pub bytesize: usize,
    pub Flags: ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemHostRegister_v2_params_st {
    pub p: *mut ::core::ffi::c_void,
    pub bytesize: usize,
    pub Flags: ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemHostUnregister_params_st {
    pub p: *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cuMemImportFromShareableHandle_params_st {
    pub handle: *mut CUmemGenericAllocationHandle,
    pub osHandle: *mut ::core::ffi::c_void,
    pub shHandleType: CUmemAllocationHandleType,
}
#[repr(C)]
pub struct cuMemMapArrayAsync_params_st {
    pub mapInfoList: *mut CUarrayMapInfo,
    pub count: ::core::ffi::c_uint,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemMapArrayAsync_ptsz_params_st {
    pub mapInfoList: *mut CUarrayMapInfo,
    pub count: ::core::ffi::c_uint,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemMap_params_st {
    pub ptr: CUdeviceptr,
    pub size: usize,
    pub offset: usize,
    pub handle: CUmemGenericAllocationHandle,
    pub flags: ::core::ffi::c_ulonglong,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemPoolCreate_params_st {
    pub pool: *mut CUmemoryPool,
    pub poolProps: *const CUmemPoolProps,
}
#[repr(C)]
pub struct cuMemPoolDestroy_params_st {
    pub pool: CUmemoryPool,
}
#[repr(C)]
pub struct cuMemPoolExportPointer_params_st {
    pub shareData_out: *mut CUmemPoolPtrExportData,
    pub ptr: CUdeviceptr,
}
#[repr(C)]
pub struct cuMemPoolExportToShareableHandle_params_st {
    pub handle_out: *mut ::core::ffi::c_void,
    pub pool: CUmemoryPool,
    pub handleType: CUmemAllocationHandleType,
    pub flags: ::core::ffi::c_ulonglong,
}
#[repr(C)]
pub struct cuMemPoolGetAccess_params_st {
    pub flags: *mut CUmemAccess_flags,
    pub memPool: CUmemoryPool,
    pub location: *mut CUmemLocation,
}
#[repr(C)]
pub struct cuMemPoolGetAttribute_params_st {
    pub pool: CUmemoryPool,
    pub attr: CUmemPool_attribute,
    pub value: *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cuMemPoolImportFromShareableHandle_params_st {
    pub pool_out: *mut CUmemoryPool,
    pub handle: *mut ::core::ffi::c_void,
    pub handleType: CUmemAllocationHandleType,
    pub flags: ::core::ffi::c_ulonglong,
}
#[repr(C)]
pub struct cuMemPoolImportPointer_params_st {
    pub ptr_out: *mut CUdeviceptr,
    pub pool: CUmemoryPool,
    pub shareData: *mut CUmemPoolPtrExportData,
}
#[repr(C)]
pub struct cuMemPoolSetAccess_params_st {
    pub pool: CUmemoryPool,
    pub map: *const CUmemAccessDesc,
    pub count: usize,
}
#[repr(C)]
pub struct cuMemPoolSetAttribute_params_st {
    pub pool: CUmemoryPool,
    pub attr: CUmemPool_attribute,
    pub value: *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cuMemPoolTrimTo_params_st {
    pub pool: CUmemoryPool,
    pub minBytesToKeep: usize,
}
#[repr(C)]
pub struct cuMemPrefetchAsync_params_st {
    pub devPtr: CUdeviceptr,
    pub count: usize,
    pub dstDevice: CUdevice,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemPrefetchAsync_ptsz_params_st {
    pub devPtr: CUdeviceptr,
    pub count: usize,
    pub dstDevice: CUdevice,
    pub hStream: CUstream,
}
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuMemPrefetchAsync_v2_params_st {
    pub devPtr: CUdeviceptr,
    pub count: usize,
    pub location: CUmemLocation,
    pub flags: ::core::ffi::c_uint,
    pub hStream: CUstream,
}
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuMemPrefetchAsync_v2_ptsz_params_st {
    pub devPtr: CUdeviceptr,
    pub count: usize,
    pub location: CUmemLocation,
    pub flags: ::core::ffi::c_uint,
    pub hStream: CUstream,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cuMemPrefetchBatchAsync_params_st {
    pub dptrs: *mut CUdeviceptr,
    pub sizes: *mut usize,
    pub count: usize,
    pub prefetchLocs: *mut CUmemLocation,
    pub prefetchLocIdxs: *mut usize,
    pub numPrefetchLocs: usize,
    pub flags: ::core::ffi::c_ulonglong,
    pub hStream: CUstream,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cuMemPrefetchBatchAsync_ptsz_params_st {
    pub dptrs: *mut CUdeviceptr,
    pub sizes: *mut usize,
    pub count: usize,
    pub prefetchLocs: *mut CUmemLocation,
    pub prefetchLocIdxs: *mut usize,
    pub numPrefetchLocs: usize,
    pub flags: ::core::ffi::c_ulonglong,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemRangeGetAttribute_params_st {
    pub data: *mut ::core::ffi::c_void,
    pub dataSize: usize,
    pub attribute: CUmem_range_attribute,
    pub devPtr: CUdeviceptr,
    pub count: usize,
}
#[repr(C)]
pub struct cuMemRangeGetAttributes_params_st {
    pub data: *mut *mut ::core::ffi::c_void,
    pub dataSizes: *mut usize,
    pub attributes: *mut CUmem_range_attribute,
    pub numAttributes: usize,
    pub devPtr: CUdeviceptr,
    pub count: usize,
}
#[repr(C)]
pub struct cuMemRelease_params_st {
    pub handle: CUmemGenericAllocationHandle,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemRetainAllocationHandle_params_st {
    pub handle: *mut CUmemGenericAllocationHandle,
    pub addr: *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cuMemSetAccess_params_st {
    pub ptr: CUdeviceptr,
    pub size: usize,
    pub desc: *const CUmemAccessDesc,
    pub count: usize,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cuMemSetMemPool_params_st {
    pub location: *mut CUmemLocation,
    pub type_: CUmemAllocationType,
    pub pool: CUmemoryPool,
}
#[repr(C)]
pub struct cuMemUnmap_params_st {
    pub ptr: CUdeviceptr,
    pub size: usize,
}
#[repr(C)]
pub struct cuMemcpy2DAsync_params_st {
    pub pCopy: *const CUDA_MEMCPY2D_v1,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemcpy2DAsync_v2_params_st {
    pub pCopy: *const CUDA_MEMCPY2D,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemcpy2DAsync_v2_ptsz_params_st {
    pub pCopy: *const CUDA_MEMCPY2D,
    pub hStream: CUstream,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemcpy2DUnaligned_params_st {
    pub pCopy: *const CUDA_MEMCPY2D_v1,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemcpy2DUnaligned_v2_params_st {
    pub pCopy: *const CUDA_MEMCPY2D,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemcpy2DUnaligned_v2_ptds_params_st {
    pub pCopy: *const CUDA_MEMCPY2D,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemcpy2D_params_st {
    pub pCopy: *const CUDA_MEMCPY2D_v1,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemcpy2D_v2_params_st {
    pub pCopy: *const CUDA_MEMCPY2D,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemcpy2D_v2_ptds_params_st {
    pub pCopy: *const CUDA_MEMCPY2D,
}
#[repr(C)]
pub struct cuMemcpy3DAsync_params_st {
    pub pCopy: *const CUDA_MEMCPY3D_v1,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemcpy3DAsync_v2_params_st {
    pub pCopy: *const CUDA_MEMCPY3D,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemcpy3DAsync_v2_ptsz_params_st {
    pub pCopy: *const CUDA_MEMCPY3D,
    pub hStream: CUstream,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuMemcpy3DBatchAsync_params_st {
    pub numOps: usize,
    pub opList: *mut CUDA_MEMCPY3D_BATCH_OP,
    pub failIdx: *mut usize,
    pub flags: ::core::ffi::c_ulonglong,
    pub hStream: CUstream,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuMemcpy3DBatchAsync_ptsz_params_st {
    pub numOps: usize,
    pub opList: *mut CUDA_MEMCPY3D_BATCH_OP,
    pub failIdx: *mut usize,
    pub flags: ::core::ffi::c_ulonglong,
    pub hStream: CUstream,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cuMemcpy3DBatchAsync_v2_params_st {
    pub numOps: usize,
    pub opList: *mut CUDA_MEMCPY3D_BATCH_OP,
    pub flags: ::core::ffi::c_ulonglong,
    pub hStream: CUstream,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cuMemcpy3DBatchAsync_v2_ptsz_params_st {
    pub numOps: usize,
    pub opList: *mut CUDA_MEMCPY3D_BATCH_OP,
    pub flags: ::core::ffi::c_ulonglong,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemcpy3DPeerAsync_params_st {
    pub pCopy: *const CUDA_MEMCPY3D_PEER,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemcpy3DPeerAsync_ptsz_params_st {
    pub pCopy: *const CUDA_MEMCPY3D_PEER,
    pub hStream: CUstream,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemcpy3DPeer_params_st {
    pub pCopy: *const CUDA_MEMCPY3D_PEER,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemcpy3DPeer_ptds_params_st {
    pub pCopy: *const CUDA_MEMCPY3D_PEER,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemcpy3D_params_st {
    pub pCopy: *const CUDA_MEMCPY3D_v1,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemcpy3D_v2_params_st {
    pub pCopy: *const CUDA_MEMCPY3D,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemcpy3D_v2_ptds_params_st {
    pub pCopy: *const CUDA_MEMCPY3D,
}
#[repr(C)]
pub struct cuMemcpyAsync_params_st {
    pub dst: CUdeviceptr,
    pub src: CUdeviceptr,
    pub ByteCount: usize,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemcpyAsync_ptsz_params_st {
    pub dst: CUdeviceptr,
    pub src: CUdeviceptr,
    pub ByteCount: usize,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemcpyAtoA_params_st {
    pub dstArray: CUarray,
    pub dstOffset: ::core::ffi::c_uint,
    pub srcArray: CUarray,
    pub srcOffset: ::core::ffi::c_uint,
    pub ByteCount: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuMemcpyAtoA_v2_params_st {
    pub dstArray: CUarray,
    pub dstOffset: usize,
    pub srcArray: CUarray,
    pub srcOffset: usize,
    pub ByteCount: usize,
}
#[repr(C)]
pub struct cuMemcpyAtoA_v2_ptds_params_st {
    pub dstArray: CUarray,
    pub dstOffset: usize,
    pub srcArray: CUarray,
    pub srcOffset: usize,
    pub ByteCount: usize,
}
#[repr(C)]
pub struct cuMemcpyAtoD_params_st {
    pub dstDevice: CUdeviceptr_v1,
    pub srcArray: CUarray,
    pub srcOffset: ::core::ffi::c_uint,
    pub ByteCount: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuMemcpyAtoD_v2_params_st {
    pub dstDevice: CUdeviceptr,
    pub srcArray: CUarray,
    pub srcOffset: usize,
    pub ByteCount: usize,
}
#[repr(C)]
pub struct cuMemcpyAtoD_v2_ptds_params_st {
    pub dstDevice: CUdeviceptr,
    pub srcArray: CUarray,
    pub srcOffset: usize,
    pub ByteCount: usize,
}
#[repr(C)]
pub struct cuMemcpyAtoHAsync_params_st {
    pub dstHost: *mut ::core::ffi::c_void,
    pub srcArray: CUarray,
    pub srcOffset: ::core::ffi::c_uint,
    pub ByteCount: ::core::ffi::c_uint,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemcpyAtoHAsync_v2_params_st {
    pub dstHost: *mut ::core::ffi::c_void,
    pub srcArray: CUarray,
    pub srcOffset: usize,
    pub ByteCount: usize,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemcpyAtoHAsync_v2_ptsz_params_st {
    pub dstHost: *mut ::core::ffi::c_void,
    pub srcArray: CUarray,
    pub srcOffset: usize,
    pub ByteCount: usize,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemcpyAtoH_params_st {
    pub dstHost: *mut ::core::ffi::c_void,
    pub srcArray: CUarray,
    pub srcOffset: ::core::ffi::c_uint,
    pub ByteCount: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuMemcpyAtoH_v2_params_st {
    pub dstHost: *mut ::core::ffi::c_void,
    pub srcArray: CUarray,
    pub srcOffset: usize,
    pub ByteCount: usize,
}
#[repr(C)]
pub struct cuMemcpyAtoH_v2_ptds_params_st {
    pub dstHost: *mut ::core::ffi::c_void,
    pub srcArray: CUarray,
    pub srcOffset: usize,
    pub ByteCount: usize,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuMemcpyBatchAsync_params_st {
    pub dsts: *mut CUdeviceptr,
    pub srcs: *mut CUdeviceptr,
    pub sizes: *mut usize,
    pub count: usize,
    pub attrs: *mut CUmemcpyAttributes,
    pub attrsIdxs: *mut usize,
    pub numAttrs: usize,
    pub failIdx: *mut usize,
    pub hStream: CUstream,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuMemcpyBatchAsync_ptsz_params_st {
    pub dsts: *mut CUdeviceptr,
    pub srcs: *mut CUdeviceptr,
    pub sizes: *mut usize,
    pub count: usize,
    pub attrs: *mut CUmemcpyAttributes,
    pub attrsIdxs: *mut usize,
    pub numAttrs: usize,
    pub failIdx: *mut usize,
    pub hStream: CUstream,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cuMemcpyBatchAsync_v2_params_st {
    pub dsts: *mut CUdeviceptr,
    pub srcs: *mut CUdeviceptr,
    pub sizes: *mut usize,
    pub count: usize,
    pub attrs: *mut CUmemcpyAttributes,
    pub attrsIdxs: *mut usize,
    pub numAttrs: usize,
    pub hStream: CUstream,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cuMemcpyBatchAsync_v2_ptsz_params_st {
    pub dsts: *mut CUdeviceptr,
    pub srcs: *mut CUdeviceptr,
    pub sizes: *mut usize,
    pub count: usize,
    pub attrs: *mut CUmemcpyAttributes,
    pub attrsIdxs: *mut usize,
    pub numAttrs: usize,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemcpyDtoA_params_st {
    pub dstArray: CUarray,
    pub dstOffset: ::core::ffi::c_uint,
    pub srcDevice: CUdeviceptr_v1,
    pub ByteCount: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuMemcpyDtoA_v2_params_st {
    pub dstArray: CUarray,
    pub dstOffset: usize,
    pub srcDevice: CUdeviceptr,
    pub ByteCount: usize,
}
#[repr(C)]
pub struct cuMemcpyDtoA_v2_ptds_params_st {
    pub dstArray: CUarray,
    pub dstOffset: usize,
    pub srcDevice: CUdeviceptr,
    pub ByteCount: usize,
}
#[repr(C)]
pub struct cuMemcpyDtoDAsync_params_st {
    pub dstDevice: CUdeviceptr_v1,
    pub srcDevice: CUdeviceptr_v1,
    pub ByteCount: ::core::ffi::c_uint,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemcpyDtoDAsync_v2_params_st {
    pub dstDevice: CUdeviceptr,
    pub srcDevice: CUdeviceptr,
    pub ByteCount: usize,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemcpyDtoDAsync_v2_ptsz_params_st {
    pub dstDevice: CUdeviceptr,
    pub srcDevice: CUdeviceptr,
    pub ByteCount: usize,
    pub hStream: CUstream,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemcpyDtoD_params_st {
    pub dstDevice: CUdeviceptr_v1,
    pub srcDevice: CUdeviceptr_v1,
    pub ByteCount: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuMemcpyDtoD_v2_params_st {
    pub dstDevice: CUdeviceptr,
    pub srcDevice: CUdeviceptr,
    pub ByteCount: usize,
}
#[repr(C)]
pub struct cuMemcpyDtoD_v2_ptds_params_st {
    pub dstDevice: CUdeviceptr,
    pub srcDevice: CUdeviceptr,
    pub ByteCount: usize,
}
#[repr(C)]
pub struct cuMemcpyDtoHAsync_params_st {
    pub dstHost: *mut ::core::ffi::c_void,
    pub srcDevice: CUdeviceptr_v1,
    pub ByteCount: ::core::ffi::c_uint,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemcpyDtoHAsync_v2_params_st {
    pub dstHost: *mut ::core::ffi::c_void,
    pub srcDevice: CUdeviceptr,
    pub ByteCount: usize,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemcpyDtoHAsync_v2_ptsz_params_st {
    pub dstHost: *mut ::core::ffi::c_void,
    pub srcDevice: CUdeviceptr,
    pub ByteCount: usize,
    pub hStream: CUstream,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemcpyDtoH_params_st {
    pub dstHost: *mut ::core::ffi::c_void,
    pub srcDevice: CUdeviceptr_v1,
    pub ByteCount: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuMemcpyDtoH_v2_params_st {
    pub dstHost: *mut ::core::ffi::c_void,
    pub srcDevice: CUdeviceptr,
    pub ByteCount: usize,
}
#[repr(C)]
pub struct cuMemcpyDtoH_v2_ptds_params_st {
    pub dstHost: *mut ::core::ffi::c_void,
    pub srcDevice: CUdeviceptr,
    pub ByteCount: usize,
}
#[repr(C)]
pub struct cuMemcpyHtoAAsync_params_st {
    pub dstArray: CUarray,
    pub dstOffset: ::core::ffi::c_uint,
    pub srcHost: *const ::core::ffi::c_void,
    pub ByteCount: ::core::ffi::c_uint,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemcpyHtoAAsync_v2_params_st {
    pub dstArray: CUarray,
    pub dstOffset: usize,
    pub srcHost: *const ::core::ffi::c_void,
    pub ByteCount: usize,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemcpyHtoAAsync_v2_ptsz_params_st {
    pub dstArray: CUarray,
    pub dstOffset: usize,
    pub srcHost: *const ::core::ffi::c_void,
    pub ByteCount: usize,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemcpyHtoA_params_st {
    pub dstArray: CUarray,
    pub dstOffset: ::core::ffi::c_uint,
    pub srcHost: *const ::core::ffi::c_void,
    pub ByteCount: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuMemcpyHtoA_v2_params_st {
    pub dstArray: CUarray,
    pub dstOffset: usize,
    pub srcHost: *const ::core::ffi::c_void,
    pub ByteCount: usize,
}
#[repr(C)]
pub struct cuMemcpyHtoA_v2_ptds_params_st {
    pub dstArray: CUarray,
    pub dstOffset: usize,
    pub srcHost: *const ::core::ffi::c_void,
    pub ByteCount: usize,
}
#[repr(C)]
pub struct cuMemcpyHtoDAsync_params_st {
    pub dstDevice: CUdeviceptr_v1,
    pub srcHost: *const ::core::ffi::c_void,
    pub ByteCount: ::core::ffi::c_uint,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemcpyHtoDAsync_v2_params_st {
    pub dstDevice: CUdeviceptr,
    pub srcHost: *const ::core::ffi::c_void,
    pub ByteCount: usize,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemcpyHtoDAsync_v2_ptsz_params_st {
    pub dstDevice: CUdeviceptr,
    pub srcHost: *const ::core::ffi::c_void,
    pub ByteCount: usize,
    pub hStream: CUstream,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemcpyHtoD_params_st {
    pub dstDevice: CUdeviceptr_v1,
    pub srcHost: *const ::core::ffi::c_void,
    pub ByteCount: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuMemcpyHtoD_v2_params_st {
    pub dstDevice: CUdeviceptr,
    pub srcHost: *const ::core::ffi::c_void,
    pub ByteCount: usize,
}
#[repr(C)]
pub struct cuMemcpyHtoD_v2_ptds_params_st {
    pub dstDevice: CUdeviceptr,
    pub srcHost: *const ::core::ffi::c_void,
    pub ByteCount: usize,
}
#[repr(C)]
pub struct cuMemcpyPeerAsync_params_st {
    pub dstDevice: CUdeviceptr,
    pub dstContext: CUcontext,
    pub srcDevice: CUdeviceptr,
    pub srcContext: CUcontext,
    pub ByteCount: usize,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemcpyPeerAsync_ptsz_params_st {
    pub dstDevice: CUdeviceptr,
    pub dstContext: CUcontext,
    pub srcDevice: CUdeviceptr,
    pub srcContext: CUcontext,
    pub ByteCount: usize,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemcpyPeer_params_st {
    pub dstDevice: CUdeviceptr,
    pub dstContext: CUcontext,
    pub srcDevice: CUdeviceptr,
    pub srcContext: CUcontext,
    pub ByteCount: usize,
}
#[repr(C)]
pub struct cuMemcpyPeer_ptds_params_st {
    pub dstDevice: CUdeviceptr,
    pub dstContext: CUcontext,
    pub srcDevice: CUdeviceptr,
    pub srcContext: CUcontext,
    pub ByteCount: usize,
}
#[repr(C)]
pub struct cuMemcpy_params_st {
    pub dst: CUdeviceptr,
    pub src: CUdeviceptr,
    pub ByteCount: usize,
}
#[repr(C)]
pub struct cuMemcpy_ptds_params_st {
    pub dst: CUdeviceptr,
    pub src: CUdeviceptr,
    pub ByteCount: usize,
}
#[repr(C)]
pub struct cuMemsetD16Async_params_st {
    pub dstDevice: CUdeviceptr,
    pub us: ::core::ffi::c_ushort,
    pub N: usize,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemsetD16Async_ptsz_params_st {
    pub dstDevice: CUdeviceptr,
    pub us: ::core::ffi::c_ushort,
    pub N: usize,
    pub hStream: CUstream,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemsetD16_params_st {
    pub dstDevice: CUdeviceptr_v1,
    pub us: ::core::ffi::c_ushort,
    pub N: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuMemsetD16_v2_params_st {
    pub dstDevice: CUdeviceptr,
    pub us: ::core::ffi::c_ushort,
    pub N: usize,
}
#[repr(C)]
pub struct cuMemsetD16_v2_ptds_params_st {
    pub dstDevice: CUdeviceptr,
    pub us: ::core::ffi::c_ushort,
    pub N: usize,
}
#[repr(C)]
pub struct cuMemsetD2D16Async_params_st {
    pub dstDevice: CUdeviceptr,
    pub dstPitch: usize,
    pub us: ::core::ffi::c_ushort,
    pub Width: usize,
    pub Height: usize,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemsetD2D16Async_ptsz_params_st {
    pub dstDevice: CUdeviceptr,
    pub dstPitch: usize,
    pub us: ::core::ffi::c_ushort,
    pub Width: usize,
    pub Height: usize,
    pub hStream: CUstream,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemsetD2D16_params_st {
    pub dstDevice: CUdeviceptr_v1,
    pub dstPitch: ::core::ffi::c_uint,
    pub us: ::core::ffi::c_ushort,
    pub Width: ::core::ffi::c_uint,
    pub Height: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuMemsetD2D16_v2_params_st {
    pub dstDevice: CUdeviceptr,
    pub dstPitch: usize,
    pub us: ::core::ffi::c_ushort,
    pub Width: usize,
    pub Height: usize,
}
#[repr(C)]
pub struct cuMemsetD2D16_v2_ptds_params_st {
    pub dstDevice: CUdeviceptr,
    pub dstPitch: usize,
    pub us: ::core::ffi::c_ushort,
    pub Width: usize,
    pub Height: usize,
}
#[repr(C)]
pub struct cuMemsetD2D32Async_params_st {
    pub dstDevice: CUdeviceptr,
    pub dstPitch: usize,
    pub ui: ::core::ffi::c_uint,
    pub Width: usize,
    pub Height: usize,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemsetD2D32Async_ptsz_params_st {
    pub dstDevice: CUdeviceptr,
    pub dstPitch: usize,
    pub ui: ::core::ffi::c_uint,
    pub Width: usize,
    pub Height: usize,
    pub hStream: CUstream,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemsetD2D32_params_st {
    pub dstDevice: CUdeviceptr_v1,
    pub dstPitch: ::core::ffi::c_uint,
    pub ui: ::core::ffi::c_uint,
    pub Width: ::core::ffi::c_uint,
    pub Height: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuMemsetD2D32_v2_params_st {
    pub dstDevice: CUdeviceptr,
    pub dstPitch: usize,
    pub ui: ::core::ffi::c_uint,
    pub Width: usize,
    pub Height: usize,
}
#[repr(C)]
pub struct cuMemsetD2D32_v2_ptds_params_st {
    pub dstDevice: CUdeviceptr,
    pub dstPitch: usize,
    pub ui: ::core::ffi::c_uint,
    pub Width: usize,
    pub Height: usize,
}
#[repr(C)]
pub struct cuMemsetD2D8Async_params_st {
    pub dstDevice: CUdeviceptr,
    pub dstPitch: usize,
    pub uc: ::core::ffi::c_uchar,
    pub Width: usize,
    pub Height: usize,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemsetD2D8Async_ptsz_params_st {
    pub dstDevice: CUdeviceptr,
    pub dstPitch: usize,
    pub uc: ::core::ffi::c_uchar,
    pub Width: usize,
    pub Height: usize,
    pub hStream: CUstream,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemsetD2D8_params_st {
    pub dstDevice: CUdeviceptr_v1,
    pub dstPitch: ::core::ffi::c_uint,
    pub uc: ::core::ffi::c_uchar,
    pub Width: ::core::ffi::c_uint,
    pub Height: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuMemsetD2D8_v2_params_st {
    pub dstDevice: CUdeviceptr,
    pub dstPitch: usize,
    pub uc: ::core::ffi::c_uchar,
    pub Width: usize,
    pub Height: usize,
}
#[repr(C)]
pub struct cuMemsetD2D8_v2_ptds_params_st {
    pub dstDevice: CUdeviceptr,
    pub dstPitch: usize,
    pub uc: ::core::ffi::c_uchar,
    pub Width: usize,
    pub Height: usize,
}
#[repr(C)]
pub struct cuMemsetD32Async_params_st {
    pub dstDevice: CUdeviceptr,
    pub ui: ::core::ffi::c_uint,
    pub N: usize,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemsetD32Async_ptsz_params_st {
    pub dstDevice: CUdeviceptr,
    pub ui: ::core::ffi::c_uint,
    pub N: usize,
    pub hStream: CUstream,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemsetD32_params_st {
    pub dstDevice: CUdeviceptr_v1,
    pub ui: ::core::ffi::c_uint,
    pub N: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuMemsetD32_v2_params_st {
    pub dstDevice: CUdeviceptr,
    pub ui: ::core::ffi::c_uint,
    pub N: usize,
}
#[repr(C)]
pub struct cuMemsetD32_v2_ptds_params_st {
    pub dstDevice: CUdeviceptr,
    pub ui: ::core::ffi::c_uint,
    pub N: usize,
}
#[repr(C)]
pub struct cuMemsetD8Async_params_st {
    pub dstDevice: CUdeviceptr,
    pub uc: ::core::ffi::c_uchar,
    pub N: usize,
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuMemsetD8Async_ptsz_params_st {
    pub dstDevice: CUdeviceptr,
    pub uc: ::core::ffi::c_uchar,
    pub N: usize,
    pub hStream: CUstream,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMemsetD8_params_st {
    pub dstDevice: CUdeviceptr_v1,
    pub uc: ::core::ffi::c_uchar,
    pub N: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuMemsetD8_v2_params_st {
    pub dstDevice: CUdeviceptr,
    pub uc: ::core::ffi::c_uchar,
    pub N: usize,
}
#[repr(C)]
pub struct cuMemsetD8_v2_ptds_params_st {
    pub dstDevice: CUdeviceptr,
    pub uc: ::core::ffi::c_uchar,
    pub N: usize,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMipmappedArrayCreate_params_st {
    pub pHandle: *mut CUmipmappedArray,
    pub pMipmappedArrayDesc: *const CUDA_ARRAY3D_DESCRIPTOR,
    pub numMipmapLevels: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuMipmappedArrayDestroy_params_st {
    pub hMipmappedArray: CUmipmappedArray,
}
#[repr(C)]
pub struct cuMipmappedArrayGetLevel_params_st {
    pub pLevelArray: *mut CUarray,
    pub hMipmappedArray: CUmipmappedArray,
    pub level: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuMipmappedArrayGetMemoryRequirements_params_st {
    pub memoryRequirements: *mut CUDA_ARRAY_MEMORY_REQUIREMENTS,
    pub mipmap: CUmipmappedArray,
    pub device: CUdevice,
}
#[repr(C)]
pub struct cuMipmappedArrayGetSparseProperties_params_st {
    pub sparseProperties: *mut CUDA_ARRAY_SPARSE_PROPERTIES,
    pub mipmap: CUmipmappedArray,
}
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuModuleEnumerateFunctions_params_st {
    pub functions: *mut CUfunction,
    pub numFunctions: ::core::ffi::c_uint,
    pub mod_: CUmodule,
}
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuModuleGetFunctionCount_params_st {
    pub count: *mut ::core::ffi::c_uint,
    pub mod_: CUmodule,
}
#[repr(C)]
pub struct cuModuleGetFunction_params_st {
    pub hfunc: *mut CUfunction,
    pub hmod: CUmodule,
    pub name: *const ::core::ffi::c_char,
}
#[repr(C)]
pub struct cuModuleGetGlobal_params_st {
    pub dptr: *mut CUdeviceptr_v1,
    pub bytes: *mut ::core::ffi::c_uint,
    pub hmod: CUmodule,
    pub name: *const ::core::ffi::c_char,
}
#[repr(C)]
pub struct cuModuleGetGlobal_v2_params_st {
    pub dptr: *mut CUdeviceptr,
    pub bytes: *mut usize,
    pub hmod: CUmodule,
    pub name: *const ::core::ffi::c_char,
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuModuleGetLoadingMode_params_st {
    pub mode: *mut CUmoduleLoadingMode,
}
#[repr(C)]
pub struct cuModuleGetSurfRef_params_st {
    pub pSurfRef: *mut CUsurfref,
    pub hmod: CUmodule,
    pub name: *const ::core::ffi::c_char,
}
#[repr(C)]
pub struct cuModuleGetTexRef_params_st {
    pub pTexRef: *mut CUtexref,
    pub hmod: CUmodule,
    pub name: *const ::core::ffi::c_char,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuModuleLoadDataEx_params_st {
    pub module: *mut CUmodule,
    pub image: *const ::core::ffi::c_void,
    pub numOptions: ::core::ffi::c_uint,
    pub options: *mut CUjit_option,
    pub optionValues: *mut *mut ::core::ffi::c_void,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuModuleLoadData_params_st {
    pub module: *mut CUmodule,
    pub image: *const ::core::ffi::c_void,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuModuleLoadFatBinary_params_st {
    pub module: *mut CUmodule,
    pub fatCubin: *const ::core::ffi::c_void,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuModuleLoad_params_st {
    pub module: *mut CUmodule,
    pub fname: *const ::core::ffi::c_char,
}
#[repr(C)]
pub struct cuModuleUnload_params_st {
    pub hmod: CUmodule,
}
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuMulticastAddDevice_params_st {
    pub mcHandle: CUmemGenericAllocationHandle,
    pub dev: CUdevice,
}
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuMulticastBindAddr_params_st {
    pub mcHandle: CUmemGenericAllocationHandle,
    pub mcOffset: usize,
    pub memptr: CUdeviceptr,
    pub size: usize,
    pub flags: ::core::ffi::c_ulonglong,
}
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuMulticastBindMem_params_st {
    pub mcHandle: CUmemGenericAllocationHandle,
    pub mcOffset: usize,
    pub memHandle: CUmemGenericAllocationHandle,
    pub memOffset: usize,
    pub size: usize,
    pub flags: ::core::ffi::c_ulonglong,
}
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuMulticastCreate_params_st {
    pub mcHandle: *mut CUmemGenericAllocationHandle,
    pub prop: *const CUmulticastObjectProp,
}
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuMulticastGetGranularity_params_st {
    pub granularity: *mut usize,
    pub prop: *const CUmulticastObjectProp,
    pub option: CUmulticastGranularity_flags,
}
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuMulticastUnbind_params_st {
    pub mcHandle: CUmemGenericAllocationHandle,
    pub dev: CUdevice,
    pub mcOffset: usize,
    pub size: usize,
}
#[repr(C)]
pub struct cuOccupancyAvailableDynamicSMemPerBlock_params_st {
    pub dynamicSmemSize: *mut usize,
    pub func: CUfunction,
    pub numBlocks: ::core::ffi::c_int,
    pub blockSize: ::core::ffi::c_int,
}
#[repr(C)]
pub struct cuOccupancyMaxActiveBlocksPerMultiprocessorWithFlags_params_st {
    pub numBlocks: *mut ::core::ffi::c_int,
    pub func: CUfunction,
    pub blockSize: ::core::ffi::c_int,
    pub dynamicSMemSize: usize,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuOccupancyMaxActiveBlocksPerMultiprocessor_params_st {
    pub numBlocks: *mut ::core::ffi::c_int,
    pub func: CUfunction,
    pub blockSize: ::core::ffi::c_int,
    pub dynamicSMemSize: usize,
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuOccupancyMaxActiveClusters_params_st {
    pub numClusters: *mut ::core::ffi::c_int,
    pub func: CUfunction,
    pub config: *const CUlaunchConfig,
}
#[repr(C)]
pub struct cuOccupancyMaxPotentialBlockSizeWithFlags_params_st {
    pub minGridSize: *mut ::core::ffi::c_int,
    pub blockSize: *mut ::core::ffi::c_int,
    pub func: CUfunction,
    pub blockSizeToDynamicSMemSize: CUoccupancyB2DSize,
    pub dynamicSMemSize: usize,
    pub blockSizeLimit: ::core::ffi::c_int,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuOccupancyMaxPotentialBlockSize_params_st {
    pub minGridSize: *mut ::core::ffi::c_int,
    pub blockSize: *mut ::core::ffi::c_int,
    pub func: CUfunction,
    pub blockSizeToDynamicSMemSize: CUoccupancyB2DSize,
    pub dynamicSMemSize: usize,
    pub blockSizeLimit: ::core::ffi::c_int,
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuOccupancyMaxPotentialClusterSize_params_st {
    pub clusterSize: *mut ::core::ffi::c_int,
    pub func: CUfunction,
    pub config: *const CUlaunchConfig,
}
#[repr(C)]
pub struct cuParamSetSize_params_st {
    pub hfunc: CUfunction,
    pub numbytes: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuParamSetTexRef_params_st {
    pub hfunc: CUfunction,
    pub texunit: ::core::ffi::c_int,
    pub hTexRef: CUtexref,
}
#[repr(C)]
pub struct cuParamSetf_params_st {
    pub hfunc: CUfunction,
    pub offset: ::core::ffi::c_int,
    pub value: f32,
}
#[repr(C)]
pub struct cuParamSeti_params_st {
    pub hfunc: CUfunction,
    pub offset: ::core::ffi::c_int,
    pub value: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuParamSetv_params_st {
    pub hfunc: CUfunction,
    pub offset: ::core::ffi::c_int,
    pub ptr: *mut ::core::ffi::c_void,
    pub numbytes: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuPointerGetAttribute_params_st {
    pub data: *mut ::core::ffi::c_void,
    pub attribute: CUpointer_attribute,
    pub ptr: CUdeviceptr,
}
#[repr(C)]
pub struct cuPointerGetAttributes_params_st {
    pub numAttributes: ::core::ffi::c_uint,
    pub attributes: *mut CUpointer_attribute,
    pub data: *mut *mut ::core::ffi::c_void,
    pub ptr: CUdeviceptr,
}
#[repr(C)]
pub struct cuPointerSetAttribute_params_st {
    pub value: *const ::core::ffi::c_void,
    pub attribute: CUpointer_attribute,
    pub ptr: CUdeviceptr,
}
#[repr(C)]
pub struct cuSignalExternalSemaphoresAsync_params_st {
    pub extSemArray: *const CUexternalSemaphore,
    pub paramsArray: *const CUDA_EXTERNAL_SEMAPHORE_SIGNAL_PARAMS,
    pub numExtSems: ::core::ffi::c_uint,
    pub stream: CUstream,
}
#[repr(C)]
pub struct cuSignalExternalSemaphoresAsync_ptsz_params_st {
    pub extSemArray: *const CUexternalSemaphore,
    pub paramsArray: *const CUDA_EXTERNAL_SEMAPHORE_SIGNAL_PARAMS,
    pub numExtSems: ::core::ffi::c_uint,
    pub stream: CUstream,
}
#[repr(C)]
pub struct cuStreamAddCallback_params_st {
    pub hStream: CUstream,
    pub callback: CUstreamCallback,
    pub userData: *mut ::core::ffi::c_void,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuStreamAddCallback_ptsz_params_st {
    pub hStream: CUstream,
    pub callback: CUstreamCallback,
    pub userData: *mut ::core::ffi::c_void,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuStreamAttachMemAsync_params_st {
    pub hStream: CUstream,
    pub dptr: CUdeviceptr,
    pub length: usize,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuStreamAttachMemAsync_ptsz_params_st {
    pub hStream: CUstream,
    pub dptr: CUdeviceptr,
    pub length: usize,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuStreamBatchMemOp_params_st {
    pub stream: CUstream,
    pub count: ::core::ffi::c_uint,
    pub paramArray: *mut CUstreamBatchMemOpParams,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuStreamBatchMemOp_ptsz_params_st {
    pub stream: CUstream,
    pub count: ::core::ffi::c_uint,
    pub paramArray: *mut CUstreamBatchMemOpParams,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuStreamBatchMemOp_v2_params_st {
    pub stream: CUstream,
    pub count: ::core::ffi::c_uint,
    pub paramArray: *mut CUstreamBatchMemOpParams,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuStreamBatchMemOp_v2_ptsz_params_st {
    pub stream: CUstream,
    pub count: ::core::ffi::c_uint,
    pub paramArray: *mut CUstreamBatchMemOpParams,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuStreamBeginCaptureToGraph_params_st {
    pub hStream: CUstream,
    pub hGraph: CUgraph,
    pub dependencies: *const CUgraphNode,
    pub dependencyData: *const CUgraphEdgeData,
    pub numDependencies: usize,
    pub mode: CUstreamCaptureMode,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuStreamBeginCaptureToGraph_ptsz_params_st {
    pub hStream: CUstream,
    pub hGraph: CUgraph,
    pub dependencies: *const CUgraphNode,
    pub dependencyData: *const CUgraphEdgeData,
    pub numDependencies: usize,
    pub mode: CUstreamCaptureMode,
}
#[repr(C)]
pub struct cuStreamBeginCapture_params_st {
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuStreamBeginCapture_ptsz_params_st {
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuStreamBeginCapture_v2_params_st {
    pub hStream: CUstream,
    pub mode: CUstreamCaptureMode,
}
#[repr(C)]
pub struct cuStreamBeginCapture_v2_ptsz_params_st {
    pub hStream: CUstream,
    pub mode: CUstreamCaptureMode,
}
#[repr(C)]
pub struct cuStreamCopyAttributes_params_st {
    pub dstStream: CUstream,
    pub srcStream: CUstream,
}
#[repr(C)]
pub struct cuStreamCopyAttributes_ptsz_params_st {
    pub dst: CUstream,
    pub src: CUstream,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuStreamCreateWithPriority_params_st {
    pub phStream: *mut CUstream,
    pub flags: ::core::ffi::c_uint,
    pub priority: ::core::ffi::c_int,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuStreamCreate_params_st {
    pub phStream: *mut CUstream,
    pub Flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuStreamDestroy_params_st {
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuStreamDestroy_v2_params_st {
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuStreamEndCapture_params_st {
    pub hStream: CUstream,
    pub phGraph: *mut CUgraph,
}
#[repr(C)]
pub struct cuStreamEndCapture_ptsz_params_st {
    pub hStream: CUstream,
    pub phGraph: *mut CUgraph,
}
#[repr(C)]
pub struct cuStreamGetAttribute_params_st {
    pub hStream: CUstream,
    pub attr: CUstreamAttrID,
    pub value: *mut CUstreamAttrValue,
}
#[repr(C)]
pub struct cuStreamGetAttribute_ptsz_params_st {
    pub hStream: CUstream,
    pub attr: CUstreamAttrID,
    pub value_out: *mut CUstreamAttrValue,
}
#[repr(C)]
pub struct cuStreamGetCaptureInfo_params_st {
    pub hStream: CUstream,
    pub captureStatus_out: *mut CUstreamCaptureStatus,
    pub id_out: *mut cuuint64_t,
}
#[repr(C)]
pub struct cuStreamGetCaptureInfo_ptsz_params_st {
    pub hStream: CUstream,
    pub captureStatus_out: *mut CUstreamCaptureStatus,
    pub id_out: *mut cuuint64_t,
}
#[repr(C)]
pub struct cuStreamGetCaptureInfo_v2_params_st {
    pub hStream: CUstream,
    pub captureStatus_out: *mut CUstreamCaptureStatus,
    pub id_out: *mut cuuint64_t,
    pub graph_out: *mut CUgraph,
    pub dependencies_out: *mut *const CUgraphNode,
    pub numDependencies_out: *mut usize,
}
#[repr(C)]
pub struct cuStreamGetCaptureInfo_v2_ptsz_params_st {
    pub hStream: CUstream,
    pub captureStatus_out: *mut CUstreamCaptureStatus,
    pub id_out: *mut cuuint64_t,
    pub graph_out: *mut CUgraph,
    pub dependencies_out: *mut *const CUgraphNode,
    pub numDependencies_out: *mut usize,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuStreamGetCaptureInfo_v3_params_st {
    pub hStream: CUstream,
    pub captureStatus_out: *mut CUstreamCaptureStatus,
    pub id_out: *mut cuuint64_t,
    pub graph_out: *mut CUgraph,
    pub dependencies_out: *mut *const CUgraphNode,
    pub edgeData_out: *mut *const CUgraphEdgeData,
    pub numDependencies_out: *mut usize,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuStreamGetCaptureInfo_v3_ptsz_params_st {
    pub hStream: CUstream,
    pub captureStatus_out: *mut CUstreamCaptureStatus,
    pub id_out: *mut cuuint64_t,
    pub graph_out: *mut CUgraph,
    pub dependencies_out: *mut *const CUgraphNode,
    pub edgeData_out: *mut *const CUgraphEdgeData,
    pub numDependencies_out: *mut usize,
}
#[repr(C)]
pub struct cuStreamGetCtx_params_st {
    pub hStream: CUstream,
    pub pctx: *mut CUcontext,
}
#[repr(C)]
pub struct cuStreamGetCtx_ptsz_params_st {
    pub hStream: CUstream,
    pub pctx: *mut CUcontext,
}
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuStreamGetCtx_v2_params_st {
    pub hStream: CUstream,
    pub pCtx: *mut CUcontext,
    pub pGreenCtx: *mut CUgreenCtx,
}
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuStreamGetCtx_v2_ptsz_params_st {
    pub hStream: CUstream,
    pub pCtx: *mut CUcontext,
    pub pGreenCtx: *mut CUgreenCtx,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
pub struct cuStreamGetDevResource_params_st {
    pub hStream: CUstream,
    pub resource: *mut CUdevResource,
    pub type_: CUdevResourceType,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
pub struct cuStreamGetDevResource_ptsz_params_st {
    pub hStream: CUstream,
    pub resource: *mut CUdevResource,
    pub type_: CUdevResourceType,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuStreamGetDevice_params_st {
    pub hStream: CUstream,
    pub device: *mut CUdevice,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuStreamGetDevice_ptsz_params_st {
    pub hStream: CUstream,
    pub device: *mut CUdevice,
}
#[repr(C)]
pub struct cuStreamGetFlags_params_st {
    pub hStream: CUstream,
    pub flags: *mut ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuStreamGetFlags_ptsz_params_st {
    pub hStream: CUstream,
    pub flags: *mut ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuStreamGetGreenCtx_params_st {
    pub hStream: CUstream,
    pub phCtx: *mut CUgreenCtx,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuStreamGetId_params_st {
    pub hStream: CUstream,
    pub streamId: *mut ::core::ffi::c_ulonglong,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuStreamGetId_ptsz_params_st {
    pub hStream: CUstream,
    pub streamId: *mut ::core::ffi::c_ulonglong,
}
#[repr(C)]
pub struct cuStreamGetPriority_params_st {
    pub hStream: CUstream,
    pub priority: *mut ::core::ffi::c_int,
}
#[repr(C)]
pub struct cuStreamGetPriority_ptsz_params_st {
    pub hStream: CUstream,
    pub priority: *mut ::core::ffi::c_int,
}
#[repr(C)]
pub struct cuStreamIsCapturing_params_st {
    pub hStream: CUstream,
    pub captureStatus: *mut CUstreamCaptureStatus,
}
#[repr(C)]
pub struct cuStreamIsCapturing_ptsz_params_st {
    pub hStream: CUstream,
    pub captureStatus: *mut CUstreamCaptureStatus,
}
#[repr(C)]
pub struct cuStreamQuery_params_st {
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuStreamQuery_ptsz_params_st {
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuStreamSetAttribute_params_st {
    pub hStream: CUstream,
    pub attr: CUstreamAttrID,
    pub param: *const CUstreamAttrValue,
}
#[repr(C)]
pub struct cuStreamSetAttribute_ptsz_params_st {
    pub hStream: CUstream,
    pub attr: CUstreamAttrID,
    pub value: *const CUstreamAttrValue,
}
#[repr(C)]
pub struct cuStreamSynchronize_params_st {
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuStreamSynchronize_ptsz_params_st {
    pub hStream: CUstream,
}
#[repr(C)]
pub struct cuStreamUpdateCaptureDependencies_params_st {
    pub hStream: CUstream,
    pub dependencies: *mut CUgraphNode,
    pub numDependencies: usize,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuStreamUpdateCaptureDependencies_ptsz_params_st {
    pub hStream: CUstream,
    pub dependencies: *mut CUgraphNode,
    pub numDependencies: usize,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuStreamUpdateCaptureDependencies_v2_params_st {
    pub hStream: CUstream,
    pub dependencies: *mut CUgraphNode,
    pub dependencyData: *const CUgraphEdgeData,
    pub numDependencies: usize,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuStreamUpdateCaptureDependencies_v2_ptsz_params_st {
    pub hStream: CUstream,
    pub dependencies: *mut CUgraphNode,
    pub dependencyData: *const CUgraphEdgeData,
    pub numDependencies: usize,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuStreamWaitEvent_params_st {
    pub hStream: CUstream,
    pub hEvent: CUevent,
    pub Flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuStreamWaitEvent_ptsz_params_st {
    pub hStream: CUstream,
    pub hEvent: CUevent,
    pub Flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuStreamWaitValue32_params_st {
    pub stream: CUstream,
    pub addr: CUdeviceptr,
    pub value: cuuint32_t,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuStreamWaitValue32_ptsz_params_st {
    pub stream: CUstream,
    pub addr: CUdeviceptr,
    pub value: cuuint32_t,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuStreamWaitValue32_v2_params_st {
    pub stream: CUstream,
    pub addr: CUdeviceptr,
    pub value: cuuint32_t,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuStreamWaitValue32_v2_ptsz_params_st {
    pub stream: CUstream,
    pub addr: CUdeviceptr,
    pub value: cuuint32_t,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuStreamWaitValue64_params_st {
    pub stream: CUstream,
    pub addr: CUdeviceptr,
    pub value: cuuint64_t,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuStreamWaitValue64_ptsz_params_st {
    pub stream: CUstream,
    pub addr: CUdeviceptr,
    pub value: cuuint64_t,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuStreamWaitValue64_v2_params_st {
    pub stream: CUstream,
    pub addr: CUdeviceptr,
    pub value: cuuint64_t,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuStreamWaitValue64_v2_ptsz_params_st {
    pub stream: CUstream,
    pub addr: CUdeviceptr,
    pub value: cuuint64_t,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuStreamWriteValue32_params_st {
    pub stream: CUstream,
    pub addr: CUdeviceptr,
    pub value: cuuint32_t,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuStreamWriteValue32_ptsz_params_st {
    pub stream: CUstream,
    pub addr: CUdeviceptr,
    pub value: cuuint32_t,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuStreamWriteValue32_v2_params_st {
    pub stream: CUstream,
    pub addr: CUdeviceptr,
    pub value: cuuint32_t,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuStreamWriteValue32_v2_ptsz_params_st {
    pub stream: CUstream,
    pub addr: CUdeviceptr,
    pub value: cuuint32_t,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuStreamWriteValue64_params_st {
    pub stream: CUstream,
    pub addr: CUdeviceptr,
    pub value: cuuint64_t,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuStreamWriteValue64_ptsz_params_st {
    pub stream: CUstream,
    pub addr: CUdeviceptr,
    pub value: cuuint64_t,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuStreamWriteValue64_v2_params_st {
    pub stream: CUstream,
    pub addr: CUdeviceptr,
    pub value: cuuint64_t,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuStreamWriteValue64_v2_ptsz_params_st {
    pub stream: CUstream,
    pub addr: CUdeviceptr,
    pub value: cuuint64_t,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuSurfObjectCreate_params_st {
    pub pSurfObject: *mut CUsurfObject,
    pub pResDesc: *const CUDA_RESOURCE_DESC,
}
#[repr(C)]
pub struct cuSurfObjectDestroy_params_st {
    pub surfObject: CUsurfObject,
}
#[repr(C)]
pub struct cuSurfObjectGetResourceDesc_params_st {
    pub pResDesc: *mut CUDA_RESOURCE_DESC,
    pub surfObject: CUsurfObject,
}
#[repr(C)]
pub struct cuSurfRefGetArray_params_st {
    pub phArray: *mut CUarray,
    pub hSurfRef: CUsurfref,
}
#[repr(C)]
pub struct cuSurfRefSetArray_params_st {
    pub hSurfRef: CUsurfref,
    pub hArray: CUarray,
    pub Flags: ::core::ffi::c_uint,
}
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cuTensorMapEncodeIm2colWide_params_st {
    pub tensorMap: *mut CUtensorMap,
    pub tensorDataType: CUtensorMapDataType,
    pub tensorRank: cuuint32_t,
    pub globalAddress: *mut ::core::ffi::c_void,
    pub globalDim: *const cuuint64_t,
    pub globalStrides: *const cuuint64_t,
    pub pixelBoxLowerCornerWidth: ::core::ffi::c_int,
    pub pixelBoxUpperCornerWidth: ::core::ffi::c_int,
    pub channelsPerPixel: cuuint32_t,
    pub pixelsPerColumn: cuuint32_t,
    pub elementStrides: *const cuuint32_t,
    pub interleave: CUtensorMapInterleave,
    pub mode: CUtensorMapIm2ColWideMode,
    pub swizzle: CUtensorMapSwizzle,
    pub l2Promotion: CUtensorMapL2promotion,
    pub oobFill: CUtensorMapFloatOOBfill,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuTensorMapEncodeIm2col_params_st {
    pub tensorMap: *mut CUtensorMap,
    pub tensorDataType: CUtensorMapDataType,
    pub tensorRank: cuuint32_t,
    pub globalAddress: *mut ::core::ffi::c_void,
    pub globalDim: *const cuuint64_t,
    pub globalStrides: *const cuuint64_t,
    pub pixelBoxLowerCorner: *const ::core::ffi::c_int,
    pub pixelBoxUpperCorner: *const ::core::ffi::c_int,
    pub channelsPerPixel: cuuint32_t,
    pub pixelsPerColumn: cuuint32_t,
    pub elementStrides: *const cuuint32_t,
    pub interleave: CUtensorMapInterleave,
    pub swizzle: CUtensorMapSwizzle,
    pub l2Promotion: CUtensorMapL2promotion,
    pub oobFill: CUtensorMapFloatOOBfill,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cuTensorMapEncodeTiled_params_st {
    pub tensorMap: *mut CUtensorMap,
    pub tensorDataType: CUtensorMapDataType,
    pub tensorRank: cuuint32_t,
    pub globalAddress: *mut ::core::ffi::c_void,
    pub globalDim: *const cuuint64_t,
    pub globalStrides: *const cuuint64_t,
    pub boxDim: *const cuuint32_t,
    pub elementStrides: *const cuuint32_t,
    pub interleave: CUtensorMapInterleave,
    pub swizzle: CUtensorMapSwizzle,
    pub l2Promotion: CUtensorMapL2promotion,
    pub oobFill: CUtensorMapFloatOOBfill,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuTensorMapReplaceAddress_params_st {
    pub tensorMap: *mut CUtensorMap,
    pub globalAddress: *mut ::core::ffi::c_void,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuTexObjectCreate_params_st {
    pub pTexObject: *mut CUtexObject,
    pub pResDesc: *const CUDA_RESOURCE_DESC,
    pub pTexDesc: *const CUDA_TEXTURE_DESC,
    pub pResViewDesc: *const CUDA_RESOURCE_VIEW_DESC,
}
#[repr(C)]
pub struct cuTexObjectDestroy_params_st {
    pub texObject: CUtexObject,
}
#[repr(C)]
pub struct cuTexObjectGetResourceDesc_params_st {
    pub pResDesc: *mut CUDA_RESOURCE_DESC,
    pub texObject: CUtexObject,
}
#[repr(C)]
pub struct cuTexObjectGetResourceViewDesc_params_st {
    pub pResViewDesc: *mut CUDA_RESOURCE_VIEW_DESC,
    pub texObject: CUtexObject,
}
#[repr(C)]
pub struct cuTexObjectGetTextureDesc_params_st {
    pub pTexDesc: *mut CUDA_TEXTURE_DESC,
    pub texObject: CUtexObject,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuTexRefCreate_params_st {
    pub pTexRef: *mut CUtexref,
}
#[repr(C)]
pub struct cuTexRefDestroy_params_st {
    pub hTexRef: CUtexref,
}
#[repr(C)]
pub struct cuTexRefGetAddressMode_params_st {
    pub pam: *mut CUaddress_mode,
    pub hTexRef: CUtexref,
    pub dim: ::core::ffi::c_int,
}
#[repr(C)]
pub struct cuTexRefGetAddress_params_st {
    pub pdptr: *mut CUdeviceptr_v1,
    pub hTexRef: CUtexref,
}
#[repr(C)]
pub struct cuTexRefGetAddress_v2_params_st {
    pub pdptr: *mut CUdeviceptr,
    pub hTexRef: CUtexref,
}
#[repr(C)]
pub struct cuTexRefGetArray_params_st {
    pub phArray: *mut CUarray,
    pub hTexRef: CUtexref,
}
#[repr(C)]
pub struct cuTexRefGetBorderColor_params_st {
    pub pBorderColor: *mut f32,
    pub hTexRef: CUtexref,
}
#[repr(C)]
pub struct cuTexRefGetFilterMode_params_st {
    pub pfm: *mut CUfilter_mode,
    pub hTexRef: CUtexref,
}
#[repr(C)]
pub struct cuTexRefGetFlags_params_st {
    pub pFlags: *mut ::core::ffi::c_uint,
    pub hTexRef: CUtexref,
}
#[repr(C)]
pub struct cuTexRefGetFormat_params_st {
    pub pFormat: *mut CUarray_format,
    pub pNumChannels: *mut ::core::ffi::c_int,
    pub hTexRef: CUtexref,
}
#[repr(C)]
pub struct cuTexRefGetMaxAnisotropy_params_st {
    pub pmaxAniso: *mut ::core::ffi::c_int,
    pub hTexRef: CUtexref,
}
#[repr(C)]
pub struct cuTexRefGetMipmapFilterMode_params_st {
    pub pfm: *mut CUfilter_mode,
    pub hTexRef: CUtexref,
}
#[repr(C)]
pub struct cuTexRefGetMipmapLevelBias_params_st {
    pub pbias: *mut f32,
    pub hTexRef: CUtexref,
}
#[repr(C)]
pub struct cuTexRefGetMipmapLevelClamp_params_st {
    pub pminMipmapLevelClamp: *mut f32,
    pub pmaxMipmapLevelClamp: *mut f32,
    pub hTexRef: CUtexref,
}
#[repr(C)]
pub struct cuTexRefGetMipmappedArray_params_st {
    pub phMipmappedArray: *mut CUmipmappedArray,
    pub hTexRef: CUtexref,
}
#[repr(C)]
pub struct cuTexRefSetAddress2D_params_st {
    pub hTexRef: CUtexref,
    pub desc: *const CUDA_ARRAY_DESCRIPTOR_v1,
    pub dptr: CUdeviceptr_v1,
    pub Pitch: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuTexRefSetAddress2D_v2_params_st {
    pub hTexRef: CUtexref,
    pub desc: *const CUDA_ARRAY_DESCRIPTOR,
    pub dptr: CUdeviceptr,
    pub Pitch: usize,
}
#[repr(C)]
pub struct cuTexRefSetAddress2D_v3_params_st {
    pub hTexRef: CUtexref,
    pub desc: *const CUDA_ARRAY_DESCRIPTOR,
    pub dptr: CUdeviceptr,
    pub Pitch: usize,
}
#[repr(C)]
pub struct cuTexRefSetAddressMode_params_st {
    pub hTexRef: CUtexref,
    pub dim: ::core::ffi::c_int,
    pub am: CUaddress_mode,
}
#[repr(C)]
pub struct cuTexRefSetAddress_params_st {
    pub ByteOffset: *mut ::core::ffi::c_uint,
    pub hTexRef: CUtexref,
    pub dptr: CUdeviceptr_v1,
    pub bytes: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuTexRefSetAddress_v2_params_st {
    pub ByteOffset: *mut usize,
    pub hTexRef: CUtexref,
    pub dptr: CUdeviceptr,
    pub bytes: usize,
}
#[repr(C)]
pub struct cuTexRefSetArray_params_st {
    pub hTexRef: CUtexref,
    pub hArray: CUarray,
    pub Flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuTexRefSetBorderColor_params_st {
    pub hTexRef: CUtexref,
    pub pBorderColor: *mut f32,
}
#[repr(C)]
pub struct cuTexRefSetFilterMode_params_st {
    pub hTexRef: CUtexref,
    pub fm: CUfilter_mode,
}
#[repr(C)]
pub struct cuTexRefSetFlags_params_st {
    pub hTexRef: CUtexref,
    pub Flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuTexRefSetFormat_params_st {
    pub hTexRef: CUtexref,
    pub fmt: CUarray_format,
    pub NumPackedComponents: ::core::ffi::c_int,
}
#[repr(C)]
pub struct cuTexRefSetMaxAnisotropy_params_st {
    pub hTexRef: CUtexref,
    pub maxAniso: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuTexRefSetMipmapFilterMode_params_st {
    pub hTexRef: CUtexref,
    pub fm: CUfilter_mode,
}
#[repr(C)]
pub struct cuTexRefSetMipmapLevelBias_params_st {
    pub hTexRef: CUtexref,
    pub bias: f32,
}
#[repr(C)]
pub struct cuTexRefSetMipmapLevelClamp_params_st {
    pub hTexRef: CUtexref,
    pub minMipmapLevelClamp: f32,
    pub maxMipmapLevelClamp: f32,
}
#[repr(C)]
pub struct cuTexRefSetMipmappedArray_params_st {
    pub hTexRef: CUtexref,
    pub hMipmappedArray: CUmipmappedArray,
    pub Flags: ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cuThreadExchangeStreamCaptureMode_params_st {
    pub mode: *mut CUstreamCaptureMode,
}
#[repr(C)]
pub struct cuUserObjectCreate_params_st {
    pub object_out: *mut CUuserObject,
    pub ptr: *mut ::core::ffi::c_void,
    pub destroy: CUhostFn,
    pub initialRefcount: ::core::ffi::c_uint,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuUserObjectRelease_params_st {
    pub object: CUuserObject,
    pub count: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuUserObjectRetain_params_st {
    pub object: CUuserObject,
    pub count: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cuWaitExternalSemaphoresAsync_params_st {
    pub extSemArray: *const CUexternalSemaphore,
    pub paramsArray: *const CUDA_EXTERNAL_SEMAPHORE_WAIT_PARAMS,
    pub numExtSems: ::core::ffi::c_uint,
    pub stream: CUstream,
}
#[repr(C)]
pub struct cuWaitExternalSemaphoresAsync_ptsz_params_st {
    pub extSemArray: *const CUexternalSemaphore,
    pub paramsArray: *const CUDA_EXTERNAL_SEMAPHORE_WAIT_PARAMS,
    pub numExtSems: ::core::ffi::c_uint,
    pub stream: CUstream,
}
#[repr(C)]
pub struct cudaArrayGetInfo_v4010_params_st {
    pub desc: *mut cudaChannelFormatDesc,
    pub extent: *mut cudaExtent,
    pub flags: *mut ::core::ffi::c_uint,
    pub array: cudaArray_t,
}
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaArrayGetMemoryRequirements_v11060_params_st {
    pub memoryRequirements: *mut cudaArrayMemoryRequirements,
    pub array: cudaArray_t,
    pub device: ::core::ffi::c_int,
}
#[repr(C)]
pub struct cudaArrayGetPlane_v11020_params_st {
    pub pPlaneArray: *mut cudaArray_t,
    pub hArray: cudaArray_t,
    pub planeIdx: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaArrayGetSparseProperties_v11010_params_st {
    pub sparseProperties: *mut cudaArraySparseProperties,
    pub array: cudaArray_t,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
#[repr(C)]
pub struct cudaBindSurfaceToArray_v3020_params_st {
    pub surfref: *const surfaceReference,
    pub array: cudaArray_const_t,
    pub desc: *const cudaChannelFormatDesc,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaBindTexture2D_v3020_params_st {
    pub offset: *mut usize,
    pub texref: *const textureReference,
    pub devPtr: *const ::core::ffi::c_void,
    pub desc: *const cudaChannelFormatDesc,
    pub width: usize,
    pub height: usize,
    pub pitch: usize,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
#[repr(C)]
pub struct cudaBindTextureToArray_v3020_params_st {
    pub texref: *const textureReference,
    pub array: cudaArray_const_t,
    pub desc: *const cudaChannelFormatDesc,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
#[repr(C)]
pub struct cudaBindTextureToMipmappedArray_v5000_params_st {
    pub texref: *const textureReference,
    pub mipmappedArray: cudaMipmappedArray_const_t,
    pub desc: *const cudaChannelFormatDesc,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaBindTexture_v3020_params_st {
    pub offset: *mut usize,
    pub texref: *const textureReference,
    pub devPtr: *const ::core::ffi::c_void,
    pub desc: *const cudaChannelFormatDesc,
    pub size: usize,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaChooseDevice_v3020_params_st {
    pub device: *mut ::core::ffi::c_int,
    pub prop: *const cudaDeviceProp,
}
#[repr(C)]
pub struct cudaCreateChannelDesc_v3020_params_st {
    pub x: ::core::ffi::c_int,
    pub y: ::core::ffi::c_int,
    pub z: ::core::ffi::c_int,
    pub w: ::core::ffi::c_int,
    pub f: cudaChannelFormatKind,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaCreateSurfaceObject_v5000_params_st {
    pub pSurfObject: *mut cudaSurfaceObject_t,
    pub pResDesc: *const cudaResourceDesc,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaCreateTextureObject_v5000_params_st {
    pub pTexObject: *mut cudaTextureObject_t,
    pub pResDesc: *const cudaResourceDesc,
    pub pTexDesc: *const cudaTextureDesc,
    pub pResViewDesc: *const cudaResourceViewDesc,
}
#[repr(C)]
pub struct cudaDestroyExternalMemory_v10000_params_st {
    pub extMem: cudaExternalMemory_t,
}
#[repr(C)]
pub struct cudaDestroyExternalSemaphore_v10000_params_st {
    pub extSem: cudaExternalSemaphore_t,
}
#[repr(C)]
pub struct cudaDestroySurfaceObject_v5000_params_st {
    pub surfObject: cudaSurfaceObject_t,
}
#[repr(C)]
pub struct cudaDestroyTextureObject_v5000_params_st {
    pub texObject: cudaTextureObject_t,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaDevResourceGenerateDesc_v13010_params_st {
    pub phDesc: *mut cudaDevResourceDesc_t,
    pub resources: *mut cudaDevResource,
    pub nbResources: ::core::ffi::c_uint,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaDevSmResourceSplitByCount_v13010_params_st {
    pub result: *mut cudaDevResource,
    pub nbGroups: *mut ::core::ffi::c_uint,
    pub input: *const cudaDevResource,
    pub remaining: *mut cudaDevResource,
    pub useFlags: ::core::ffi::c_uint,
    pub minCount: ::core::ffi::c_uint,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaDevSmResourceSplit_v13010_params_st {
    pub result: *mut cudaDevResource,
    pub nbGroups: ::core::ffi::c_uint,
    pub input: *const cudaDevResource,
    pub remainder: *mut cudaDevResource,
    pub flags: ::core::ffi::c_uint,
    pub groupParams: *mut cudaDevSmResourceGroupParams,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaDeviceCanAccessPeer_v4000_params_st {
    pub canAccessPeer: *mut ::core::ffi::c_int,
    pub device: ::core::ffi::c_int,
    pub peerDevice: ::core::ffi::c_int,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaDeviceDisablePeerAccess_v4000_params_st {
    pub peerDevice: ::core::ffi::c_int,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaDeviceEnablePeerAccess_v4000_params_st {
    pub peerDevice: ::core::ffi::c_int,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaDeviceFlushGPUDirectRDMAWrites_v11030_params_st {
    pub target: cudaFlushGPUDirectRDMAWritesTarget,
    pub scope: cudaFlushGPUDirectRDMAWritesScope,
}
#[repr(C)]
pub struct cudaDeviceGetAttribute_v5000_params_st {
    pub value: *mut ::core::ffi::c_int,
    pub attr: cudaDeviceAttr,
    pub device: ::core::ffi::c_int,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaDeviceGetByPCIBusId_v4010_params_st {
    pub device: *mut ::core::ffi::c_int,
    pub pciBusId: *const ::core::ffi::c_char,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaDeviceGetCacheConfig_v3020_params_st {
    pub pCacheConfig: *mut cudaFuncCache,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaDeviceGetDefaultMemPool_v11020_params_st {
    pub memPool: *mut cudaMemPool_t,
    pub device: ::core::ffi::c_int,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaDeviceGetDevResource_v13010_params_st {
    pub device: ::core::ffi::c_int,
    pub resource: *mut cudaDevResource,
    pub type_: cudaDevResourceType,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaDeviceGetExecutionCtx_v13010_params_st {
    pub ctx: *mut cudaExecutionContext_t,
    pub device: ::core::ffi::c_int,
}
#[repr(C)]
pub struct cudaDeviceGetGraphMemAttribute_v11040_params_st {
    pub device: ::core::ffi::c_int,
    pub attr: cudaGraphMemAttributeType,
    pub value: *mut ::core::ffi::c_void,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaDeviceGetHostAtomicCapabilities_v13000_params_st {
    pub capabilities: *mut ::core::ffi::c_uint,
    pub operations: *const cudaAtomicOperation,
    pub count: ::core::ffi::c_uint,
    pub device: ::core::ffi::c_int,
}
#[repr(C)]
pub struct cudaDeviceGetLimit_v3020_params_st {
    pub pValue: *mut usize,
    pub limit: cudaLimit,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaDeviceGetMemPool_v11020_params_st {
    pub memPool: *mut cudaMemPool_t,
    pub device: ::core::ffi::c_int,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaDeviceGetNvSciSyncAttributes_v10020_params_st {
    pub nvSciSyncAttrList: *mut ::core::ffi::c_void,
    pub device: ::core::ffi::c_int,
    pub flags: ::core::ffi::c_int,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaDeviceGetP2PAtomicCapabilities_v13000_params_st {
    pub capabilities: *mut ::core::ffi::c_uint,
    pub operations: *const cudaAtomicOperation,
    pub count: ::core::ffi::c_uint,
    pub srcDevice: ::core::ffi::c_int,
    pub dstDevice: ::core::ffi::c_int,
}
#[repr(C)]
pub struct cudaDeviceGetP2PAttribute_v8000_params_st {
    pub value: *mut ::core::ffi::c_int,
    pub attr: cudaDeviceP2PAttr,
    pub srcDevice: ::core::ffi::c_int,
    pub dstDevice: ::core::ffi::c_int,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaDeviceGetPCIBusId_v4010_params_st {
    pub pciBusId: *mut ::core::ffi::c_char,
    pub len: ::core::ffi::c_int,
    pub device: ::core::ffi::c_int,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaDeviceGetSharedMemConfig_v4020_params_st {
    pub pConfig: *mut cudaSharedMemConfig,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaDeviceGetStreamPriorityRange_v5050_params_st {
    pub leastPriority: *mut ::core::ffi::c_int,
    pub greatestPriority: *mut ::core::ffi::c_int,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaDeviceGetTexture1DLinearMaxWidth_v11010_params_st {
    pub maxWidthInElements: *mut usize,
    pub fmtDesc: *const cudaChannelFormatDesc,
    pub device: ::core::ffi::c_int,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaDeviceGraphMemTrim_v11040_params_st {
    pub device: ::core::ffi::c_int,
}
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaDeviceRegisterAsyncNotification_v12040_params_st {
    pub device: ::core::ffi::c_int,
    pub callbackFunc: cudaAsyncCallback,
    pub userData: *mut ::core::ffi::c_void,
    pub callback: *mut cudaAsyncCallbackHandle_t,
}
#[repr(C)]
pub struct cudaDeviceSetCacheConfig_v3020_params_st {
    pub cacheConfig: cudaFuncCache,
}
#[repr(C)]
pub struct cudaDeviceSetGraphMemAttribute_v11040_params_st {
    pub device: ::core::ffi::c_int,
    pub attr: cudaGraphMemAttributeType,
    pub value: *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cudaDeviceSetLimit_v3020_params_st {
    pub limit: cudaLimit,
    pub value: usize,
}
#[repr(C)]
pub struct cudaDeviceSetMemPool_v11020_params_st {
    pub device: ::core::ffi::c_int,
    pub memPool: cudaMemPool_t,
}
#[repr(C)]
pub struct cudaDeviceSetSharedMemConfig_v4020_params_st {
    pub config: cudaSharedMemConfig,
}
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaDeviceUnregisterAsyncNotification_v12040_params_st {
    pub device: ::core::ffi::c_int,
    pub callback: cudaAsyncCallbackHandle_t,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaDriverGetVersion_v3020_params_st {
    pub driverVersion: *mut ::core::ffi::c_int,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaEventCreateWithFlags_v3020_params_st {
    pub event: *mut cudaEvent_t,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaEventCreate_v3020_params_st {
    pub event: *mut cudaEvent_t,
}
#[repr(C)]
pub struct cudaEventDestroy_v3020_params_st {
    pub event: cudaEvent_t,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaEventElapsedTime_v12080_params_st {
    pub ms: *mut f32,
    pub start: cudaEvent_t,
    pub end: cudaEvent_t,
}
#[cfg(any(feature = "cuda-12080", feature = "cuda-12090"))]
#[repr(C)]
pub struct cudaEventElapsedTime_v2_v12080_params_st {
    pub ms: *mut f32,
    pub start: cudaEvent_t,
    pub end: cudaEvent_t,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaEventElapsedTime_v3020_params_st {
    pub ms: *mut f32,
    pub start: cudaEvent_t,
    pub end: cudaEvent_t,
}
#[repr(C)]
pub struct cudaEventQuery_v3020_params_st {
    pub event: cudaEvent_t,
}
#[repr(C)]
pub struct cudaEventRecordWithFlags_ptsz_v11010_params_st {
    pub event: cudaEvent_t,
    pub stream: cudaStream_t,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaEventRecordWithFlags_v11010_params_st {
    pub event: cudaEvent_t,
    pub stream: cudaStream_t,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaEventRecord_ptsz_v7000_params_st {
    pub event: cudaEvent_t,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaEventRecord_v3020_params_st {
    pub event: cudaEvent_t,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaEventSynchronize_v3020_params_st {
    pub event: cudaEvent_t,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaExecutionCtxDestroy_v13010_params_st {
    pub ctx: cudaExecutionContext_t,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaExecutionCtxGetDevResource_v13010_params_st {
    pub ctx: cudaExecutionContext_t,
    pub resource: *mut cudaDevResource,
    pub type_: cudaDevResourceType,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaExecutionCtxGetDevice_v13010_params_st {
    pub device: *mut ::core::ffi::c_int,
    pub ctx: cudaExecutionContext_t,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaExecutionCtxGetId_v13010_params_st {
    pub ctx: cudaExecutionContext_t,
    pub ctxId: *mut ::core::ffi::c_ulonglong,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaExecutionCtxRecordEvent_v13010_params_st {
    pub ctx: cudaExecutionContext_t,
    pub event: cudaEvent_t,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaExecutionCtxStreamCreate_v13010_params_st {
    pub phStream: *mut cudaStream_t,
    pub ctx: cudaExecutionContext_t,
    pub flags: ::core::ffi::c_uint,
    pub priority: ::core::ffi::c_int,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaExecutionCtxSynchronize_v13010_params_st {
    pub ctx: cudaExecutionContext_t,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaExecutionCtxWaitEvent_v13010_params_st {
    pub ctx: cudaExecutionContext_t,
    pub event: cudaEvent_t,
}
#[repr(C)]
pub struct cudaExternalMemoryGetMappedBuffer_v10000_params_st {
    pub devPtr: *mut *mut ::core::ffi::c_void,
    pub extMem: cudaExternalMemory_t,
    pub bufferDesc: *const cudaExternalMemoryBufferDesc,
}
#[repr(C)]
pub struct cudaExternalMemoryGetMappedMipmappedArray_v10000_params_st {
    pub mipmap: *mut cudaMipmappedArray_t,
    pub extMem: cudaExternalMemory_t,
    pub mipmapDesc: *const cudaExternalMemoryMipmappedArrayDesc,
}
#[repr(C)]
pub struct cudaFreeArray_v3020_params_st {
    pub array: cudaArray_t,
}
#[repr(C)]
pub struct cudaFreeAsync_ptsz_v11020_params_st {
    pub devPtr: *mut ::core::ffi::c_void,
    pub hStream: cudaStream_t,
}
#[repr(C)]
pub struct cudaFreeAsync_v11020_params_st {
    pub devPtr: *mut ::core::ffi::c_void,
    pub hStream: cudaStream_t,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaFreeHost_v3020_params_st {
    pub ptr: *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cudaFreeMipmappedArray_v5000_params_st {
    pub mipmappedArray: cudaMipmappedArray_t,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaFree_v3020_params_st {
    pub devPtr: *mut ::core::ffi::c_void,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaFuncGetAttributes_v3020_params_st {
    pub attr: *mut cudaFuncAttributes,
    pub func: *const ::core::ffi::c_void,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaFuncGetName_v12030_params_st {
    pub name: *mut *const ::core::ffi::c_char,
    pub func: *const ::core::ffi::c_void,
}
#[cfg(any(
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaFuncGetParamInfo_v12040_params_st {
    pub func: *const ::core::ffi::c_void,
    pub paramIndex: usize,
    pub paramOffset: *mut usize,
    pub paramSize: *mut usize,
}
#[repr(C)]
pub struct cudaFuncSetAttribute_v9000_params_st {
    pub func: *const ::core::ffi::c_void,
    pub attr: cudaFuncAttribute,
    pub value: ::core::ffi::c_int,
}
#[repr(C)]
pub struct cudaFuncSetCacheConfig_v3020_params_st {
    pub func: *const ::core::ffi::c_void,
    pub cacheConfig: cudaFuncCache,
}
#[repr(C)]
pub struct cudaFuncSetSharedMemConfig_v4020_params_st {
    pub func: *const ::core::ffi::c_void,
    pub config: cudaSharedMemConfig,
}
#[repr(C)]
pub struct cudaGetChannelDesc_v3020_params_st {
    pub desc: *mut cudaChannelFormatDesc,
    pub array: cudaArray_const_t,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaGetDeviceCount_v3020_params_st {
    pub count: *mut ::core::ffi::c_int,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaGetDeviceFlags_v7000_params_st {
    pub flags: *mut ::core::ffi::c_uint,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaGetDeviceProperties_v12000_params_st {
    pub prop: *mut cudaDeviceProp,
    pub device: ::core::ffi::c_int,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaGetDeviceProperties_v2_v12000_params_st {
    pub prop: *mut cudaDeviceProp,
    pub device: ::core::ffi::c_int,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaGetDeviceProperties_v3020_params_st {
    pub prop: *mut cudaDeviceProp,
    pub device: ::core::ffi::c_int,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaGetDevice_v3020_params_st {
    pub device: *mut ::core::ffi::c_int,
}
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaGetDriverEntryPointByVersion_ptsz_v12050_params_st {
    pub symbol: *const ::core::ffi::c_char,
    pub funcPtr: *mut *mut ::core::ffi::c_void,
    pub cudaVersion: ::core::ffi::c_uint,
    pub flags: ::core::ffi::c_ulonglong,
    pub driverStatus: *mut cudaDriverEntryPointQueryResult,
}
#[cfg(any(
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaGetDriverEntryPointByVersion_v12050_params_st {
    pub symbol: *const ::core::ffi::c_char,
    pub funcPtr: *mut *mut ::core::ffi::c_void,
    pub cudaVersion: ::core::ffi::c_uint,
    pub flags: ::core::ffi::c_ulonglong,
    pub driverStatus: *mut cudaDriverEntryPointQueryResult,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaGetDriverEntryPoint_ptsz_v11030_params_st {
    pub symbol: *const ::core::ffi::c_char,
    pub funcPtr: *mut *mut ::core::ffi::c_void,
    pub flags: ::core::ffi::c_ulonglong,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaGetDriverEntryPoint_ptsz_v11030_params_st {
    pub symbol: *const ::core::ffi::c_char,
    pub funcPtr: *mut *mut ::core::ffi::c_void,
    pub flags: ::core::ffi::c_ulonglong,
    pub driverStatus: *mut cudaDriverEntryPointQueryResult,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaGetDriverEntryPoint_v11030_params_st {
    pub symbol: *const ::core::ffi::c_char,
    pub funcPtr: *mut *mut ::core::ffi::c_void,
    pub flags: ::core::ffi::c_ulonglong,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaGetDriverEntryPoint_v11030_params_st {
    pub symbol: *const ::core::ffi::c_char,
    pub funcPtr: *mut *mut ::core::ffi::c_void,
    pub flags: ::core::ffi::c_ulonglong,
    pub driverStatus: *mut cudaDriverEntryPointQueryResult,
}
#[repr(C)]
pub struct cudaGetErrorName_v6050_params_st {
    pub error: cudaError_t,
}
#[repr(C)]
pub struct cudaGetErrorString_v3020_params_st {
    pub error: cudaError_t,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaGetFuncBySymbol_v11000_params_st {
    pub functionPtr: *mut cudaFunction_t,
    pub symbolPtr: *const ::core::ffi::c_void,
}
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaGetKernel_v12000_params_st {
    pub kernelPtr: *mut cudaKernel_t,
    pub entryFuncAddr: *const ::core::ffi::c_void,
}
#[repr(C)]
pub struct cudaGetMipmappedArrayLevel_v5000_params_st {
    pub levelArray: *mut cudaArray_t,
    pub mipmappedArray: cudaMipmappedArray_const_t,
    pub level: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaGetSurfaceObjectResourceDesc_v5000_params_st {
    pub pResDesc: *mut cudaResourceDesc,
    pub surfObject: cudaSurfaceObject_t,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaGetSurfaceReference_v3020_params_st {
    pub surfref: *mut *const surfaceReference,
    pub symbol: *const ::core::ffi::c_void,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaGetSymbolAddress_v3020_params_st {
    pub devPtr: *mut *mut ::core::ffi::c_void,
    pub symbol: *const ::core::ffi::c_void,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaGetSymbolSize_v3020_params_st {
    pub size: *mut usize,
    pub symbol: *const ::core::ffi::c_void,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaGetTextureAlignmentOffset_v3020_params_st {
    pub offset: *mut usize,
    pub texref: *const textureReference,
}
#[repr(C)]
pub struct cudaGetTextureObjectResourceDesc_v5000_params_st {
    pub pResDesc: *mut cudaResourceDesc,
    pub texObject: cudaTextureObject_t,
}
#[repr(C)]
pub struct cudaGetTextureObjectResourceViewDesc_v5000_params_st {
    pub pResViewDesc: *mut cudaResourceViewDesc,
    pub texObject: cudaTextureObject_t,
}
#[repr(C)]
pub struct cudaGetTextureObjectTextureDesc_v5000_params_st {
    pub pTexDesc: *mut cudaTextureDesc,
    pub texObject: cudaTextureObject_t,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaGetTextureReference_v3020_params_st {
    pub texref: *mut *const textureReference,
    pub symbol: *const ::core::ffi::c_void,
}
#[repr(C)]
pub struct cudaGraphAddChildGraphNode_v10000_params_st {
    pub pGraphNode: *mut cudaGraphNode_t,
    pub graph: cudaGraph_t,
    pub pDependencies: *const cudaGraphNode_t,
    pub numDependencies: usize,
    pub childGraph: cudaGraph_t,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaGraphAddDependencies_v10000_params_st {
    pub graph: cudaGraph_t,
    pub from: *const cudaGraphNode_t,
    pub to: *const cudaGraphNode_t,
    pub numDependencies: usize,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaGraphAddDependencies_v12030_params_st {
    pub graph: cudaGraph_t,
    pub from: *const cudaGraphNode_t,
    pub to: *const cudaGraphNode_t,
    pub edgeData: *const cudaGraphEdgeData,
    pub numDependencies: usize,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaGraphAddDependencies_v2_v12030_params_st {
    pub graph: cudaGraph_t,
    pub from: *const cudaGraphNode_t,
    pub to: *const cudaGraphNode_t,
    pub edgeData: *const cudaGraphEdgeData,
    pub numDependencies: usize,
}
#[repr(C)]
pub struct cudaGraphAddEmptyNode_v10000_params_st {
    pub pGraphNode: *mut cudaGraphNode_t,
    pub graph: cudaGraph_t,
    pub pDependencies: *const cudaGraphNode_t,
    pub numDependencies: usize,
}
#[repr(C)]
pub struct cudaGraphAddEventRecordNode_v11010_params_st {
    pub pGraphNode: *mut cudaGraphNode_t,
    pub graph: cudaGraph_t,
    pub pDependencies: *const cudaGraphNode_t,
    pub numDependencies: usize,
    pub event: cudaEvent_t,
}
#[repr(C)]
pub struct cudaGraphAddEventWaitNode_v11010_params_st {
    pub pGraphNode: *mut cudaGraphNode_t,
    pub graph: cudaGraph_t,
    pub pDependencies: *const cudaGraphNode_t,
    pub numDependencies: usize,
    pub event: cudaEvent_t,
}
#[repr(C)]
pub struct cudaGraphAddExternalSemaphoresSignalNode_v11020_params_st {
    pub pGraphNode: *mut cudaGraphNode_t,
    pub graph: cudaGraph_t,
    pub pDependencies: *const cudaGraphNode_t,
    pub numDependencies: usize,
    pub nodeParams: *const cudaExternalSemaphoreSignalNodeParams,
}
#[repr(C)]
pub struct cudaGraphAddExternalSemaphoresWaitNode_v11020_params_st {
    pub pGraphNode: *mut cudaGraphNode_t,
    pub graph: cudaGraph_t,
    pub pDependencies: *const cudaGraphNode_t,
    pub numDependencies: usize,
    pub nodeParams: *const cudaExternalSemaphoreWaitNodeParams,
}
#[repr(C)]
pub struct cudaGraphAddHostNode_v10000_params_st {
    pub pGraphNode: *mut cudaGraphNode_t,
    pub graph: cudaGraph_t,
    pub pDependencies: *const cudaGraphNode_t,
    pub numDependencies: usize,
    pub pNodeParams: *const cudaHostNodeParams,
}
#[repr(C)]
pub struct cudaGraphAddKernelNode_v10000_params_st {
    pub pGraphNode: *mut cudaGraphNode_t,
    pub graph: cudaGraph_t,
    pub pDependencies: *const cudaGraphNode_t,
    pub numDependencies: usize,
    pub pNodeParams: *const cudaKernelNodeParams,
}
#[repr(C)]
pub struct cudaGraphAddMemAllocNode_v11040_params_st {
    pub pGraphNode: *mut cudaGraphNode_t,
    pub graph: cudaGraph_t,
    pub pDependencies: *const cudaGraphNode_t,
    pub numDependencies: usize,
    pub nodeParams: *mut cudaMemAllocNodeParams,
}
#[repr(C)]
pub struct cudaGraphAddMemFreeNode_v11040_params_st {
    pub pGraphNode: *mut cudaGraphNode_t,
    pub graph: cudaGraph_t,
    pub pDependencies: *const cudaGraphNode_t,
    pub numDependencies: usize,
    pub dptr: *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cudaGraphAddMemcpyNode1D_v11010_params_st {
    pub pGraphNode: *mut cudaGraphNode_t,
    pub graph: cudaGraph_t,
    pub pDependencies: *const cudaGraphNode_t,
    pub numDependencies: usize,
    pub dst: *mut ::core::ffi::c_void,
    pub src: *const ::core::ffi::c_void,
    pub count: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaGraphAddMemcpyNodeFromSymbol_v11010_params_st {
    pub pGraphNode: *mut cudaGraphNode_t,
    pub graph: cudaGraph_t,
    pub pDependencies: *const cudaGraphNode_t,
    pub numDependencies: usize,
    pub dst: *mut ::core::ffi::c_void,
    pub symbol: *const ::core::ffi::c_void,
    pub count: usize,
    pub offset: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaGraphAddMemcpyNodeToSymbol_v11010_params_st {
    pub pGraphNode: *mut cudaGraphNode_t,
    pub graph: cudaGraph_t,
    pub pDependencies: *const cudaGraphNode_t,
    pub numDependencies: usize,
    pub symbol: *const ::core::ffi::c_void,
    pub src: *const ::core::ffi::c_void,
    pub count: usize,
    pub offset: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaGraphAddMemcpyNode_v10000_params_st {
    pub pGraphNode: *mut cudaGraphNode_t,
    pub graph: cudaGraph_t,
    pub pDependencies: *const cudaGraphNode_t,
    pub numDependencies: usize,
    pub pCopyParams: *const cudaMemcpy3DParms,
}
#[repr(C)]
pub struct cudaGraphAddMemsetNode_v10000_params_st {
    pub pGraphNode: *mut cudaGraphNode_t,
    pub graph: cudaGraph_t,
    pub pDependencies: *const cudaGraphNode_t,
    pub numDependencies: usize,
    pub pMemsetParams: *const cudaMemsetParams,
}
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaGraphAddNode_v12020_params_st {
    pub pGraphNode: *mut cudaGraphNode_t,
    pub graph: cudaGraph_t,
    pub pDependencies: *const cudaGraphNode_t,
    pub numDependencies: usize,
    pub nodeParams: *mut cudaGraphNodeParams,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaGraphAddNode_v12030_params_st {
    pub pGraphNode: *mut cudaGraphNode_t,
    pub graph: cudaGraph_t,
    pub pDependencies: *const cudaGraphNode_t,
    pub dependencyData: *const cudaGraphEdgeData,
    pub numDependencies: usize,
    pub nodeParams: *mut cudaGraphNodeParams,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaGraphAddNode_v2_v12030_params_st {
    pub pGraphNode: *mut cudaGraphNode_t,
    pub graph: cudaGraph_t,
    pub pDependencies: *const cudaGraphNode_t,
    pub dependencyData: *const cudaGraphEdgeData,
    pub numDependencies: usize,
    pub nodeParams: *mut cudaGraphNodeParams,
}
#[repr(C)]
pub struct cudaGraphChildGraphNodeGetGraph_v10000_params_st {
    pub node: cudaGraphNode_t,
    pub pGraph: *mut cudaGraph_t,
}
#[repr(C)]
pub struct cudaGraphClone_v10000_params_st {
    pub pGraphClone: *mut cudaGraph_t,
    pub originalGraph: cudaGraph_t,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaGraphConditionalHandleCreate_v12030_params_st {
    pub pHandle_out: *mut cudaGraphConditionalHandle,
    pub graph: cudaGraph_t,
    pub defaultLaunchValue: ::core::ffi::c_uint,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaGraphConditionalHandleCreate_v2_v13010_params_st {
    pub pHandle_out: *mut cudaGraphConditionalHandle,
    pub graph: cudaGraph_t,
    pub ctx: cudaExecutionContext_t,
    pub defaultLaunchValue: ::core::ffi::c_uint,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaGraphCreate_v10000_params_st {
    pub pGraph: *mut cudaGraph_t,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaGraphDebugDotPrint_v11030_params_st {
    pub graph: cudaGraph_t,
    pub path: *const ::core::ffi::c_char,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaGraphDestroyNode_v10000_params_st {
    pub node: cudaGraphNode_t,
}
#[repr(C)]
pub struct cudaGraphDestroy_v10000_params_st {
    pub graph: cudaGraph_t,
}
#[repr(C)]
pub struct cudaGraphEventRecordNodeGetEvent_v11010_params_st {
    pub node: cudaGraphNode_t,
    pub event_out: *mut cudaEvent_t,
}
#[repr(C)]
pub struct cudaGraphEventRecordNodeSetEvent_v11010_params_st {
    pub node: cudaGraphNode_t,
    pub event: cudaEvent_t,
}
#[repr(C)]
pub struct cudaGraphEventWaitNodeGetEvent_v11010_params_st {
    pub node: cudaGraphNode_t,
    pub event_out: *mut cudaEvent_t,
}
#[repr(C)]
pub struct cudaGraphEventWaitNodeSetEvent_v11010_params_st {
    pub node: cudaGraphNode_t,
    pub event: cudaEvent_t,
}
#[repr(C)]
pub struct cudaGraphExecChildGraphNodeSetParams_v11010_params_st {
    pub hGraphExec: cudaGraphExec_t,
    pub node: cudaGraphNode_t,
    pub childGraph: cudaGraph_t,
}
#[repr(C)]
pub struct cudaGraphExecDestroy_v10000_params_st {
    pub graphExec: cudaGraphExec_t,
}
#[repr(C)]
pub struct cudaGraphExecEventRecordNodeSetEvent_v11010_params_st {
    pub hGraphExec: cudaGraphExec_t,
    pub hNode: cudaGraphNode_t,
    pub event: cudaEvent_t,
}
#[repr(C)]
pub struct cudaGraphExecEventWaitNodeSetEvent_v11010_params_st {
    pub hGraphExec: cudaGraphExec_t,
    pub hNode: cudaGraphNode_t,
    pub event: cudaEvent_t,
}
#[repr(C)]
pub struct cudaGraphExecExternalSemaphoresSignalNodeSetParams_v11020_params_st {
    pub hGraphExec: cudaGraphExec_t,
    pub hNode: cudaGraphNode_t,
    pub nodeParams: *const cudaExternalSemaphoreSignalNodeParams,
}
#[repr(C)]
pub struct cudaGraphExecExternalSemaphoresWaitNodeSetParams_v11020_params_st {
    pub hGraphExec: cudaGraphExec_t,
    pub hNode: cudaGraphNode_t,
    pub nodeParams: *const cudaExternalSemaphoreWaitNodeParams,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaGraphExecGetFlags_v12000_params_st {
    pub graphExec: cudaGraphExec_t,
    pub flags: *mut ::core::ffi::c_ulonglong,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaGraphExecGetId_v13010_params_st {
    pub hGraphExec: cudaGraphExec_t,
    pub graphID: *mut ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaGraphExecHostNodeSetParams_v10020_params_st {
    pub hGraphExec: cudaGraphExec_t,
    pub node: cudaGraphNode_t,
    pub pNodeParams: *const cudaHostNodeParams,
}
#[repr(C)]
pub struct cudaGraphExecKernelNodeSetParams_v10010_params_st {
    pub hGraphExec: cudaGraphExec_t,
    pub node: cudaGraphNode_t,
    pub pNodeParams: *const cudaKernelNodeParams,
}
#[repr(C)]
pub struct cudaGraphExecMemcpyNodeSetParams1D_v11010_params_st {
    pub hGraphExec: cudaGraphExec_t,
    pub node: cudaGraphNode_t,
    pub dst: *mut ::core::ffi::c_void,
    pub src: *const ::core::ffi::c_void,
    pub count: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaGraphExecMemcpyNodeSetParamsFromSymbol_v11010_params_st {
    pub hGraphExec: cudaGraphExec_t,
    pub node: cudaGraphNode_t,
    pub dst: *mut ::core::ffi::c_void,
    pub symbol: *const ::core::ffi::c_void,
    pub count: usize,
    pub offset: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaGraphExecMemcpyNodeSetParamsToSymbol_v11010_params_st {
    pub hGraphExec: cudaGraphExec_t,
    pub node: cudaGraphNode_t,
    pub symbol: *const ::core::ffi::c_void,
    pub src: *const ::core::ffi::c_void,
    pub count: usize,
    pub offset: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaGraphExecMemcpyNodeSetParams_v10020_params_st {
    pub hGraphExec: cudaGraphExec_t,
    pub node: cudaGraphNode_t,
    pub pNodeParams: *const cudaMemcpy3DParms,
}
#[repr(C)]
pub struct cudaGraphExecMemsetNodeSetParams_v10020_params_st {
    pub hGraphExec: cudaGraphExec_t,
    pub node: cudaGraphNode_t,
    pub pNodeParams: *const cudaMemsetParams,
}
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaGraphExecNodeSetParams_v12020_params_st {
    pub graphExec: cudaGraphExec_t,
    pub node: cudaGraphNode_t,
    pub nodeParams: *mut cudaGraphNodeParams,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
#[repr(C)]
pub struct cudaGraphExecUpdate_v10020_params_st {
    pub hGraphExec: cudaGraphExec_t,
    pub hGraph: cudaGraph_t,
    pub hErrorNode_out: *mut cudaGraphNode_t,
    pub updateResult_out: *mut cudaGraphExecUpdateResult,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaGraphExecUpdate_v10020_params_st {
    pub hGraphExec: cudaGraphExec_t,
    pub hGraph: cudaGraph_t,
    pub resultInfo: *mut cudaGraphExecUpdateResultInfo,
}
#[repr(C)]
pub struct cudaGraphExternalSemaphoresSignalNodeGetParams_v11020_params_st {
    pub hNode: cudaGraphNode_t,
    pub params_out: *mut cudaExternalSemaphoreSignalNodeParams,
}
#[repr(C)]
pub struct cudaGraphExternalSemaphoresSignalNodeSetParams_v11020_params_st {
    pub hNode: cudaGraphNode_t,
    pub nodeParams: *const cudaExternalSemaphoreSignalNodeParams,
}
#[repr(C)]
pub struct cudaGraphExternalSemaphoresWaitNodeGetParams_v11020_params_st {
    pub hNode: cudaGraphNode_t,
    pub params_out: *mut cudaExternalSemaphoreWaitNodeParams,
}
#[repr(C)]
pub struct cudaGraphExternalSemaphoresWaitNodeSetParams_v11020_params_st {
    pub hNode: cudaGraphNode_t,
    pub nodeParams: *const cudaExternalSemaphoreWaitNodeParams,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaGraphGetEdges_v10000_params_st {
    pub graph: cudaGraph_t,
    pub from: *mut cudaGraphNode_t,
    pub to: *mut cudaGraphNode_t,
    pub numEdges: *mut usize,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaGraphGetEdges_v12030_params_st {
    pub graph: cudaGraph_t,
    pub from: *mut cudaGraphNode_t,
    pub to: *mut cudaGraphNode_t,
    pub edgeData: *mut cudaGraphEdgeData,
    pub numEdges: *mut usize,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaGraphGetEdges_v2_v12030_params_st {
    pub graph: cudaGraph_t,
    pub from: *mut cudaGraphNode_t,
    pub to: *mut cudaGraphNode_t,
    pub edgeData: *mut cudaGraphEdgeData,
    pub numEdges: *mut usize,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaGraphGetId_v13010_params_st {
    pub hGraph: cudaGraph_t,
    pub graphID: *mut ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaGraphGetNodes_v10000_params_st {
    pub graph: cudaGraph_t,
    pub nodes: *mut cudaGraphNode_t,
    pub numNodes: *mut usize,
}
#[repr(C)]
pub struct cudaGraphGetRootNodes_v10000_params_st {
    pub graph: cudaGraph_t,
    pub pRootNodes: *mut cudaGraphNode_t,
    pub pNumRootNodes: *mut usize,
}
#[repr(C)]
pub struct cudaGraphHostNodeGetParams_v10000_params_st {
    pub node: cudaGraphNode_t,
    pub pNodeParams: *mut cudaHostNodeParams,
}
#[repr(C)]
pub struct cudaGraphHostNodeSetParams_v10000_params_st {
    pub node: cudaGraphNode_t,
    pub pNodeParams: *const cudaHostNodeParams,
}
#[repr(C)]
pub struct cudaGraphInstantiateWithFlags_v11040_params_st {
    pub pGraphExec: *mut cudaGraphExec_t,
    pub graph: cudaGraph_t,
    pub flags: ::core::ffi::c_ulonglong,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaGraphInstantiateWithParams_ptsz_v12000_params_st {
    pub pGraphExec: *mut cudaGraphExec_t,
    pub graph: cudaGraph_t,
    pub instantiateParams: *mut cudaGraphInstantiateParams,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaGraphInstantiateWithParams_v12000_params_st {
    pub pGraphExec: *mut cudaGraphExec_t,
    pub graph: cudaGraph_t,
    pub instantiateParams: *mut cudaGraphInstantiateParams,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
#[repr(C)]
pub struct cudaGraphInstantiate_v10000_params_st {
    pub pGraphExec: *mut cudaGraphExec_t,
    pub graph: cudaGraph_t,
    pub pErrorNode: *mut cudaGraphNode_t,
    pub pLogBuffer: *mut ::core::ffi::c_char,
    pub bufferSize: usize,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaGraphInstantiate_v12000_params_st {
    pub pGraphExec: *mut cudaGraphExec_t,
    pub graph: cudaGraph_t,
    pub flags: ::core::ffi::c_ulonglong,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaGraphKernelNodeCopyAttributes_v11000_params_st {
    pub hSrc: cudaGraphNode_t,
    pub hDst: cudaGraphNode_t,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaGraphKernelNodeCopyAttributes_v11000_params_st {
    pub hDst: cudaGraphNode_t,
    pub hSrc: cudaGraphNode_t,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070"
))]
#[repr(C)]
pub struct cudaGraphKernelNodeGetAttribute_v11000_params_st {
    pub hNode: cudaGraphNode_t,
    pub attr: cudaKernelNodeAttrID,
    pub value_out: *mut cudaKernelNodeAttrValue,
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaGraphKernelNodeGetAttribute_v11000_params_st {
    pub hNode: cudaGraphNode_t,
    pub attr: cudaLaunchAttributeID,
    pub value_out: *mut cudaLaunchAttributeValue,
}
#[repr(C)]
pub struct cudaGraphKernelNodeGetParams_v10000_params_st {
    pub node: cudaGraphNode_t,
    pub pNodeParams: *mut cudaKernelNodeParams,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070"
))]
#[repr(C)]
pub struct cudaGraphKernelNodeSetAttribute_v11000_params_st {
    pub hNode: cudaGraphNode_t,
    pub attr: cudaKernelNodeAttrID,
    pub value: *const cudaKernelNodeAttrValue,
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaGraphKernelNodeSetAttribute_v11000_params_st {
    pub hNode: cudaGraphNode_t,
    pub attr: cudaLaunchAttributeID,
    pub value: *const cudaLaunchAttributeValue,
}
#[repr(C)]
pub struct cudaGraphKernelNodeSetParams_v10000_params_st {
    pub node: cudaGraphNode_t,
    pub pNodeParams: *const cudaKernelNodeParams,
}
#[repr(C)]
pub struct cudaGraphLaunch_ptsz_v10000_params_st {
    pub graphExec: cudaGraphExec_t,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaGraphLaunch_v10000_params_st {
    pub graphExec: cudaGraphExec_t,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaGraphMemAllocNodeGetParams_v11040_params_st {
    pub node: cudaGraphNode_t,
    pub params_out: *mut cudaMemAllocNodeParams,
}
#[repr(C)]
pub struct cudaGraphMemFreeNodeGetParams_v11040_params_st {
    pub node: cudaGraphNode_t,
    pub dptr_out: *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cudaGraphMemcpyNodeGetParams_v10000_params_st {
    pub node: cudaGraphNode_t,
    pub pNodeParams: *mut cudaMemcpy3DParms,
}
#[repr(C)]
pub struct cudaGraphMemcpyNodeSetParams1D_v11010_params_st {
    pub node: cudaGraphNode_t,
    pub dst: *mut ::core::ffi::c_void,
    pub src: *const ::core::ffi::c_void,
    pub count: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaGraphMemcpyNodeSetParamsFromSymbol_v11010_params_st {
    pub node: cudaGraphNode_t,
    pub dst: *mut ::core::ffi::c_void,
    pub symbol: *const ::core::ffi::c_void,
    pub count: usize,
    pub offset: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaGraphMemcpyNodeSetParamsToSymbol_v11010_params_st {
    pub node: cudaGraphNode_t,
    pub symbol: *const ::core::ffi::c_void,
    pub src: *const ::core::ffi::c_void,
    pub count: usize,
    pub offset: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaGraphMemcpyNodeSetParams_v10000_params_st {
    pub node: cudaGraphNode_t,
    pub pNodeParams: *const cudaMemcpy3DParms,
}
#[repr(C)]
pub struct cudaGraphMemsetNodeGetParams_v10000_params_st {
    pub node: cudaGraphNode_t,
    pub pNodeParams: *mut cudaMemsetParams,
}
#[repr(C)]
pub struct cudaGraphMemsetNodeSetParams_v10000_params_st {
    pub node: cudaGraphNode_t,
    pub pNodeParams: *const cudaMemsetParams,
}
#[repr(C)]
pub struct cudaGraphNodeFindInClone_v10000_params_st {
    pub pNode: *mut cudaGraphNode_t,
    pub originalNode: cudaGraphNode_t,
    pub clonedGraph: cudaGraph_t,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaGraphNodeGetContainingGraph_v13010_params_st {
    pub hNode: cudaGraphNode_t,
    pub phGraph: *mut cudaGraph_t,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaGraphNodeGetDependencies_v10000_params_st {
    pub node: cudaGraphNode_t,
    pub pDependencies: *mut cudaGraphNode_t,
    pub pNumDependencies: *mut usize,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaGraphNodeGetDependencies_v12030_params_st {
    pub node: cudaGraphNode_t,
    pub pDependencies: *mut cudaGraphNode_t,
    pub edgeData: *mut cudaGraphEdgeData,
    pub pNumDependencies: *mut usize,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaGraphNodeGetDependencies_v2_v12030_params_st {
    pub node: cudaGraphNode_t,
    pub pDependencies: *mut cudaGraphNode_t,
    pub edgeData: *mut cudaGraphEdgeData,
    pub pNumDependencies: *mut usize,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaGraphNodeGetDependentNodes_v10000_params_st {
    pub node: cudaGraphNode_t,
    pub pDependentNodes: *mut cudaGraphNode_t,
    pub pNumDependentNodes: *mut usize,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaGraphNodeGetDependentNodes_v12030_params_st {
    pub node: cudaGraphNode_t,
    pub pDependentNodes: *mut cudaGraphNode_t,
    pub edgeData: *mut cudaGraphEdgeData,
    pub pNumDependentNodes: *mut usize,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaGraphNodeGetDependentNodes_v2_v12030_params_st {
    pub node: cudaGraphNode_t,
    pub pDependentNodes: *mut cudaGraphNode_t,
    pub edgeData: *mut cudaGraphEdgeData,
    pub pNumDependentNodes: *mut usize,
}
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaGraphNodeGetEnabled_v11060_params_st {
    pub hGraphExec: cudaGraphExec_t,
    pub hNode: cudaGraphNode_t,
    pub isEnabled: *mut ::core::ffi::c_uint,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaGraphNodeGetLocalId_v13010_params_st {
    pub hNode: cudaGraphNode_t,
    pub nodeId: *mut ::core::ffi::c_uint,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaGraphNodeGetToolsId_v13010_params_st {
    pub hNode: cudaGraphNode_t,
    pub toolsNodeId: *mut ::core::ffi::c_ulonglong,
}
#[repr(C)]
pub struct cudaGraphNodeGetType_v10000_params_st {
    pub node: cudaGraphNode_t,
    pub pType: *mut cudaGraphNodeType,
}
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaGraphNodeSetEnabled_v11060_params_st {
    pub hGraphExec: cudaGraphExec_t,
    pub hNode: cudaGraphNode_t,
    pub isEnabled: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaGraphNodeSetParams_v12020_params_st {
    pub node: cudaGraphNode_t,
    pub nodeParams: *mut cudaGraphNodeParams,
}
#[repr(C)]
pub struct cudaGraphReleaseUserObject_v11030_params_st {
    pub graph: cudaGraph_t,
    pub object: cudaUserObject_t,
    pub count: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaGraphRemoveDependencies_v10000_params_st {
    pub graph: cudaGraph_t,
    pub from: *const cudaGraphNode_t,
    pub to: *const cudaGraphNode_t,
    pub numDependencies: usize,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaGraphRemoveDependencies_v12030_params_st {
    pub graph: cudaGraph_t,
    pub from: *const cudaGraphNode_t,
    pub to: *const cudaGraphNode_t,
    pub edgeData: *const cudaGraphEdgeData,
    pub numDependencies: usize,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaGraphRemoveDependencies_v2_v12030_params_st {
    pub graph: cudaGraph_t,
    pub from: *const cudaGraphNode_t,
    pub to: *const cudaGraphNode_t,
    pub edgeData: *const cudaGraphEdgeData,
    pub numDependencies: usize,
}
#[repr(C)]
pub struct cudaGraphRetainUserObject_v11030_params_st {
    pub graph: cudaGraph_t,
    pub object: cudaUserObject_t,
    pub count: ::core::ffi::c_uint,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaGraphUpload_ptsz_v10000_params_st {
    pub graphExec: cudaGraphExec_t,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaGraphUpload_v10000_params_st {
    pub graphExec: cudaGraphExec_t,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaGraphicsMapResources_v3020_params_st {
    pub count: ::core::ffi::c_int,
    pub resources: *mut cudaGraphicsResource_t,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaGraphicsResourceGetMappedMipmappedArray_v5000_params_st {
    pub mipmappedArray: *mut cudaMipmappedArray_t,
    pub resource: cudaGraphicsResource_t,
}
#[repr(C)]
pub struct cudaGraphicsResourceGetMappedPointer_v3020_params_st {
    pub devPtr: *mut *mut ::core::ffi::c_void,
    pub size: *mut usize,
    pub resource: cudaGraphicsResource_t,
}
#[repr(C)]
pub struct cudaGraphicsResourceSetMapFlags_v3020_params_st {
    pub resource: cudaGraphicsResource_t,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaGraphicsSubResourceGetMappedArray_v3020_params_st {
    pub array: *mut cudaArray_t,
    pub resource: cudaGraphicsResource_t,
    pub arrayIndex: ::core::ffi::c_uint,
    pub mipLevel: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaGraphicsUnmapResources_v3020_params_st {
    pub count: ::core::ffi::c_int,
    pub resources: *mut cudaGraphicsResource_t,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaGraphicsUnregisterResource_v3020_params_st {
    pub resource: cudaGraphicsResource_t,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaGreenCtxCreate_v13010_params_st {
    pub phCtx: *mut cudaExecutionContext_t,
    pub desc: cudaDevResourceDesc_t,
    pub device: ::core::ffi::c_int,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaHostAlloc_v3020_params_st {
    pub pHost: *mut *mut ::core::ffi::c_void,
    pub size: usize,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaHostGetDevicePointer_v3020_params_st {
    pub pDevice: *mut *mut ::core::ffi::c_void,
    pub pHost: *mut ::core::ffi::c_void,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaHostGetFlags_v3020_params_st {
    pub pFlags: *mut ::core::ffi::c_uint,
    pub pHost: *mut ::core::ffi::c_void,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaHostRegister_v4000_params_st {
    pub ptr: *mut ::core::ffi::c_void,
    pub size: usize,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaHostUnregister_v4000_params_st {
    pub ptr: *mut ::core::ffi::c_void,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaImportExternalMemory_v10000_params_st {
    pub extMem_out: *mut cudaExternalMemory_t,
    pub memHandleDesc: *const cudaExternalMemoryHandleDesc,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaImportExternalSemaphore_v10000_params_st {
    pub extSem_out: *mut cudaExternalSemaphore_t,
    pub semHandleDesc: *const cudaExternalSemaphoreHandleDesc,
}
#[cfg(any(
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaInitDevice_v12000_params_st {
    pub device: ::core::ffi::c_int,
    pub deviceFlags: ::core::ffi::c_uint,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaIpcCloseMemHandle_v4010_params_st {
    pub devPtr: *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cudaIpcGetEventHandle_v4010_params_st {
    pub handle: *mut cudaIpcEventHandle_t,
    pub event: cudaEvent_t,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaIpcGetMemHandle_v4010_params_st {
    pub handle: *mut cudaIpcMemHandle_t,
    pub devPtr: *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cudaIpcOpenEventHandle_v4010_params_st {
    pub event: *mut cudaEvent_t,
    pub handle: cudaIpcEventHandle_t,
}
#[repr(C)]
pub struct cudaIpcOpenMemHandle_v4010_params_st {
    pub devPtr: *mut *mut ::core::ffi::c_void,
    pub handle: cudaIpcMemHandle_t,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaKernelSetAttributeForDevice_v12060_params_st {
    pub kernel: cudaKernel_t,
    pub attr: cudaFuncAttribute,
    pub value: ::core::ffi::c_int,
    pub device: ::core::ffi::c_int,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaLaunchCooperativeKernelMultiDevice_v9000_params_st {
    pub launchParamsList: *mut cudaLaunchParams,
    pub numDevices: ::core::ffi::c_uint,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaLaunchCooperativeKernel_ptsz_v9000_params_st {
    pub func: *const ::core::ffi::c_void,
    pub gridDim: dim3,
    pub blockDim: dim3,
    pub args: *mut *mut ::core::ffi::c_void,
    pub sharedMem: usize,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaLaunchCooperativeKernel_v9000_params_st {
    pub func: *const ::core::ffi::c_void,
    pub gridDim: dim3,
    pub blockDim: dim3,
    pub args: *mut *mut ::core::ffi::c_void,
    pub sharedMem: usize,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaLaunchHostFunc_ptsz_v10000_params_st {
    pub stream: cudaStream_t,
    pub fn_: cudaHostFn_t,
    pub userData: *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cudaLaunchHostFunc_v10000_params_st {
    pub stream: cudaStream_t,
    pub fn_: cudaHostFn_t,
    pub userData: *mut ::core::ffi::c_void,
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaLaunchKernelExC_ptsz_v11060_params_st {
    pub config: *const cudaLaunchConfig_t,
    pub func: *const ::core::ffi::c_void,
    pub args: *mut *mut ::core::ffi::c_void,
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaLaunchKernelExC_v11060_params_st {
    pub config: *const cudaLaunchConfig_t,
    pub func: *const ::core::ffi::c_void,
    pub args: *mut *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cudaLaunchKernel_ptsz_v7000_params_st {
    pub func: *const ::core::ffi::c_void,
    pub gridDim: dim3,
    pub blockDim: dim3,
    pub args: *mut *mut ::core::ffi::c_void,
    pub sharedMem: usize,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaLaunchKernel_v7000_params_st {
    pub func: *const ::core::ffi::c_void,
    pub gridDim: dim3,
    pub blockDim: dim3,
    pub args: *mut *mut ::core::ffi::c_void,
    pub sharedMem: usize,
    pub stream: cudaStream_t,
}
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaLibraryEnumerateKernels_v12060_params_st {
    pub kernels: *mut cudaKernel_t,
    pub numKernels: ::core::ffi::c_uint,
    pub lib: cudaLibrary_t,
}
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaLibraryGetGlobal_v12060_params_st {
    pub dptr: *mut *mut ::core::ffi::c_void,
    pub bytes: *mut usize,
    pub library: cudaLibrary_t,
    pub name: *const ::core::ffi::c_char,
}
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaLibraryGetKernelCount_v12060_params_st {
    pub count: *mut ::core::ffi::c_uint,
    pub lib: cudaLibrary_t,
}
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaLibraryGetKernel_v12060_params_st {
    pub pKernel: *mut cudaKernel_t,
    pub library: cudaLibrary_t,
    pub name: *const ::core::ffi::c_char,
}
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaLibraryGetManaged_v12060_params_st {
    pub dptr: *mut *mut ::core::ffi::c_void,
    pub bytes: *mut usize,
    pub library: cudaLibrary_t,
    pub name: *const ::core::ffi::c_char,
}
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaLibraryGetUnifiedFunction_v12060_params_st {
    pub fptr: *mut *mut ::core::ffi::c_void,
    pub library: cudaLibrary_t,
    pub symbol: *const ::core::ffi::c_char,
}
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaLibraryLoadData_v12060_params_st {
    pub library: *mut cudaLibrary_t,
    pub code: *const ::core::ffi::c_void,
    pub jitOptions: *mut cudaJitOption,
    pub jitOptionsValues: *mut *mut ::core::ffi::c_void,
    pub numJitOptions: ::core::ffi::c_uint,
    pub libraryOptions: *mut cudaLibraryOption,
    pub libraryOptionValues: *mut *mut ::core::ffi::c_void,
    pub numLibraryOptions: ::core::ffi::c_uint,
}
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaLibraryLoadFromFile_v12060_params_st {
    pub library: *mut cudaLibrary_t,
    pub fileName: *const ::core::ffi::c_char,
    pub jitOptions: *mut cudaJitOption,
    pub jitOptionsValues: *mut *mut ::core::ffi::c_void,
    pub numJitOptions: ::core::ffi::c_uint,
    pub libraryOptions: *mut cudaLibraryOption,
    pub libraryOptionValues: *mut *mut ::core::ffi::c_void,
    pub numLibraryOptions: ::core::ffi::c_uint,
}
#[cfg(any(feature = "cuda-12090", feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaLibraryUnload_v12060_params_st {
    pub library: cudaLibrary_t,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaLogsCurrent_v13000_params_st {
    pub iterator_out: *mut cudaLogIterator,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaLogsDumpToFile_v13000_params_st {
    pub iterator: *mut cudaLogIterator,
    pub pathToFile: *const ::core::ffi::c_char,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaLogsDumpToMemory_v13000_params_st {
    pub iterator: *mut cudaLogIterator,
    pub buffer: *mut ::core::ffi::c_char,
    pub size: *mut usize,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaLogsRegisterCallback_v13000_params_st {
    pub callbackFunc: cudaLogsCallback_t,
    pub userData: *mut ::core::ffi::c_void,
    pub callback_out: *mut cudaLogsCallbackHandle,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaLogsUnregisterCallback_v13000_params_st {
    pub callback: cudaLogsCallbackHandle,
}
#[repr(C)]
pub struct cudaMalloc3DArray_v3020_params_st {
    pub array: *mut cudaArray_t,
    pub desc: *const cudaChannelFormatDesc,
    pub extent: cudaExtent,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaMalloc3D_v3020_params_st {
    pub pitchedDevPtr: *mut cudaPitchedPtr,
    pub extent: cudaExtent,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaMallocArray_v3020_params_st {
    pub array: *mut cudaArray_t,
    pub desc: *const cudaChannelFormatDesc,
    pub width: usize,
    pub height: usize,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaMallocAsync_ptsz_v11020_params_st {
    pub devPtr: *mut *mut ::core::ffi::c_void,
    pub size: usize,
    pub hStream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMallocAsync_v11020_params_st {
    pub devPtr: *mut *mut ::core::ffi::c_void,
    pub size: usize,
    pub hStream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMallocFromPoolAsync_ptsz_v11020_params_st {
    pub ptr: *mut *mut ::core::ffi::c_void,
    pub size: usize,
    pub memPool: cudaMemPool_t,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMallocFromPoolAsync_v11020_params_st {
    pub ptr: *mut *mut ::core::ffi::c_void,
    pub size: usize,
    pub memPool: cudaMemPool_t,
    pub stream: cudaStream_t,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaMallocHost_v3020_params_st {
    pub ptr: *mut *mut ::core::ffi::c_void,
    pub size: usize,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaMallocManaged_v6000_params_st {
    pub devPtr: *mut *mut ::core::ffi::c_void,
    pub size: usize,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaMallocMipmappedArray_v5000_params_st {
    pub mipmappedArray: *mut cudaMipmappedArray_t,
    pub desc: *const cudaChannelFormatDesc,
    pub extent: cudaExtent,
    pub numLevels: ::core::ffi::c_uint,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaMallocPitch_v3020_params_st {
    pub devPtr: *mut *mut ::core::ffi::c_void,
    pub pitch: *mut usize,
    pub width: usize,
    pub height: usize,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaMalloc_v3020_params_st {
    pub devPtr: *mut *mut ::core::ffi::c_void,
    pub size: usize,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaMemAdvise_v12020_params_st {
    pub devPtr: *const ::core::ffi::c_void,
    pub count: usize,
    pub advice: cudaMemoryAdvise,
    pub location: cudaMemLocation,
}
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaMemAdvise_v2_v12020_params_st {
    pub devPtr: *const ::core::ffi::c_void,
    pub count: usize,
    pub advice: cudaMemoryAdvise,
    pub location: cudaMemLocation,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaMemAdvise_v8000_params_st {
    pub devPtr: *const ::core::ffi::c_void,
    pub count: usize,
    pub advice: cudaMemoryAdvise,
    pub device: ::core::ffi::c_int,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaMemDiscardAndPrefetchBatchAsync_ptsz_v13000_params_st {
    pub dptrs: *mut *mut ::core::ffi::c_void,
    pub sizes: *mut usize,
    pub count: usize,
    pub prefetchLocs: *mut cudaMemLocation,
    pub prefetchLocIdxs: *mut usize,
    pub numPrefetchLocs: usize,
    pub flags: ::core::ffi::c_ulonglong,
    pub stream: cudaStream_t,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaMemDiscardAndPrefetchBatchAsync_v13000_params_st {
    pub dptrs: *mut *mut ::core::ffi::c_void,
    pub sizes: *mut usize,
    pub count: usize,
    pub prefetchLocs: *mut cudaMemLocation,
    pub prefetchLocIdxs: *mut usize,
    pub numPrefetchLocs: usize,
    pub flags: ::core::ffi::c_ulonglong,
    pub stream: cudaStream_t,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaMemDiscardBatchAsync_ptsz_v13000_params_st {
    pub dptrs: *mut *mut ::core::ffi::c_void,
    pub sizes: *mut usize,
    pub count: usize,
    pub flags: ::core::ffi::c_ulonglong,
    pub stream: cudaStream_t,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaMemDiscardBatchAsync_v13000_params_st {
    pub dptrs: *mut *mut ::core::ffi::c_void,
    pub sizes: *mut usize,
    pub count: usize,
    pub flags: ::core::ffi::c_ulonglong,
    pub stream: cudaStream_t,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaMemGetDefaultMemPool_v13000_params_st {
    pub memPool: *mut cudaMemPool_t,
    pub location: *mut cudaMemLocation,
    pub type_: cudaMemAllocationType,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaMemGetInfo_v3020_params_st {
    pub free: *mut usize,
    pub total: *mut usize,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaMemGetMemPool_v13000_params_st {
    pub memPool: *mut cudaMemPool_t,
    pub location: *mut cudaMemLocation,
    pub type_: cudaMemAllocationType,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaMemPoolCreate_v11020_params_st {
    pub memPool: *mut cudaMemPool_t,
    pub poolProps: *const cudaMemPoolProps,
}
#[repr(C)]
pub struct cudaMemPoolDestroy_v11020_params_st {
    pub memPool: cudaMemPool_t,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaMemPoolExportPointer_v11020_params_st {
    pub exportData: *mut cudaMemPoolPtrExportData,
    pub ptr: *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cudaMemPoolExportToShareableHandle_v11020_params_st {
    pub shareableHandle: *mut ::core::ffi::c_void,
    pub memPool: cudaMemPool_t,
    pub handleType: cudaMemAllocationHandleType,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaMemPoolGetAccess_v11020_params_st {
    pub flags: *mut cudaMemAccessFlags,
    pub memPool: cudaMemPool_t,
    pub location: *mut cudaMemLocation,
}
#[repr(C)]
pub struct cudaMemPoolGetAttribute_v11020_params_st {
    pub memPool: cudaMemPool_t,
    pub attr: cudaMemPoolAttr,
    pub value: *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cudaMemPoolImportFromShareableHandle_v11020_params_st {
    pub memPool: *mut cudaMemPool_t,
    pub shareableHandle: *mut ::core::ffi::c_void,
    pub handleType: cudaMemAllocationHandleType,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaMemPoolImportPointer_v11020_params_st {
    pub ptr: *mut *mut ::core::ffi::c_void,
    pub memPool: cudaMemPool_t,
    pub exportData: *mut cudaMemPoolPtrExportData,
}
#[repr(C)]
pub struct cudaMemPoolSetAccess_v11020_params_st {
    pub memPool: cudaMemPool_t,
    pub descList: *const cudaMemAccessDesc,
    pub count: usize,
}
#[repr(C)]
pub struct cudaMemPoolSetAttribute_v11020_params_st {
    pub memPool: cudaMemPool_t,
    pub attr: cudaMemPoolAttr,
    pub value: *mut ::core::ffi::c_void,
}
#[repr(C)]
pub struct cudaMemPoolTrimTo_v11020_params_st {
    pub memPool: cudaMemPool_t,
    pub minBytesToKeep: usize,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaMemPrefetchAsync_ptsz_v12020_params_st {
    pub devPtr: *const ::core::ffi::c_void,
    pub count: usize,
    pub location: cudaMemLocation,
    pub flags: ::core::ffi::c_uint,
    pub stream: cudaStream_t,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaMemPrefetchAsync_ptsz_v8000_params_st {
    pub devPtr: *const ::core::ffi::c_void,
    pub count: usize,
    pub dstDevice: ::core::ffi::c_int,
    pub stream: cudaStream_t,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaMemPrefetchAsync_v12020_params_st {
    pub devPtr: *const ::core::ffi::c_void,
    pub count: usize,
    pub location: cudaMemLocation,
    pub flags: ::core::ffi::c_uint,
    pub stream: cudaStream_t,
}
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaMemPrefetchAsync_v2_ptsz_v12020_params_st {
    pub devPtr: *const ::core::ffi::c_void,
    pub count: usize,
    pub location: cudaMemLocation,
    pub flags: ::core::ffi::c_uint,
    pub stream: cudaStream_t,
}
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaMemPrefetchAsync_v2_v12020_params_st {
    pub devPtr: *const ::core::ffi::c_void,
    pub count: usize,
    pub location: cudaMemLocation,
    pub flags: ::core::ffi::c_uint,
    pub stream: cudaStream_t,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaMemPrefetchAsync_v8000_params_st {
    pub devPtr: *const ::core::ffi::c_void,
    pub count: usize,
    pub dstDevice: ::core::ffi::c_int,
    pub stream: cudaStream_t,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaMemPrefetchBatchAsync_ptsz_v13000_params_st {
    pub dptrs: *mut *mut ::core::ffi::c_void,
    pub sizes: *mut usize,
    pub count: usize,
    pub prefetchLocs: *mut cudaMemLocation,
    pub prefetchLocIdxs: *mut usize,
    pub numPrefetchLocs: usize,
    pub flags: ::core::ffi::c_ulonglong,
    pub stream: cudaStream_t,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaMemPrefetchBatchAsync_v13000_params_st {
    pub dptrs: *mut *mut ::core::ffi::c_void,
    pub sizes: *mut usize,
    pub count: usize,
    pub prefetchLocs: *mut cudaMemLocation,
    pub prefetchLocIdxs: *mut usize,
    pub numPrefetchLocs: usize,
    pub flags: ::core::ffi::c_ulonglong,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMemRangeGetAttribute_v8000_params_st {
    pub data: *mut ::core::ffi::c_void,
    pub dataSize: usize,
    pub attribute: cudaMemRangeAttribute,
    pub devPtr: *const ::core::ffi::c_void,
    pub count: usize,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaMemRangeGetAttributes_v8000_params_st {
    pub data: *mut *mut ::core::ffi::c_void,
    pub dataSizes: *mut usize,
    pub attributes: *mut cudaMemRangeAttribute,
    pub numAttributes: usize,
    pub devPtr: *const ::core::ffi::c_void,
    pub count: usize,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaMemSetMemPool_v13000_params_st {
    pub location: *mut cudaMemLocation,
    pub type_: cudaMemAllocationType,
    pub memPool: cudaMemPool_t,
}
#[repr(C)]
pub struct cudaMemcpy2DArrayToArray_ptds_v7000_params_st {
    pub dst: cudaArray_t,
    pub wOffsetDst: usize,
    pub hOffsetDst: usize,
    pub src: cudaArray_const_t,
    pub wOffsetSrc: usize,
    pub hOffsetSrc: usize,
    pub width: usize,
    pub height: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaMemcpy2DArrayToArray_v3020_params_st {
    pub dst: cudaArray_t,
    pub wOffsetDst: usize,
    pub hOffsetDst: usize,
    pub src: cudaArray_const_t,
    pub wOffsetSrc: usize,
    pub hOffsetSrc: usize,
    pub width: usize,
    pub height: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaMemcpy2DAsync_ptsz_v7000_params_st {
    pub dst: *mut ::core::ffi::c_void,
    pub dpitch: usize,
    pub src: *const ::core::ffi::c_void,
    pub spitch: usize,
    pub width: usize,
    pub height: usize,
    pub kind: cudaMemcpyKind,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMemcpy2DAsync_v3020_params_st {
    pub dst: *mut ::core::ffi::c_void,
    pub dpitch: usize,
    pub src: *const ::core::ffi::c_void,
    pub spitch: usize,
    pub width: usize,
    pub height: usize,
    pub kind: cudaMemcpyKind,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMemcpy2DFromArrayAsync_ptsz_v7000_params_st {
    pub dst: *mut ::core::ffi::c_void,
    pub dpitch: usize,
    pub src: cudaArray_const_t,
    pub wOffset: usize,
    pub hOffset: usize,
    pub width: usize,
    pub height: usize,
    pub kind: cudaMemcpyKind,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMemcpy2DFromArrayAsync_v3020_params_st {
    pub dst: *mut ::core::ffi::c_void,
    pub dpitch: usize,
    pub src: cudaArray_const_t,
    pub wOffset: usize,
    pub hOffset: usize,
    pub width: usize,
    pub height: usize,
    pub kind: cudaMemcpyKind,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMemcpy2DFromArray_ptds_v7000_params_st {
    pub dst: *mut ::core::ffi::c_void,
    pub dpitch: usize,
    pub src: cudaArray_const_t,
    pub wOffset: usize,
    pub hOffset: usize,
    pub width: usize,
    pub height: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaMemcpy2DFromArray_v3020_params_st {
    pub dst: *mut ::core::ffi::c_void,
    pub dpitch: usize,
    pub src: cudaArray_const_t,
    pub wOffset: usize,
    pub hOffset: usize,
    pub width: usize,
    pub height: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaMemcpy2DToArrayAsync_ptsz_v7000_params_st {
    pub dst: cudaArray_t,
    pub wOffset: usize,
    pub hOffset: usize,
    pub src: *const ::core::ffi::c_void,
    pub spitch: usize,
    pub width: usize,
    pub height: usize,
    pub kind: cudaMemcpyKind,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMemcpy2DToArrayAsync_v3020_params_st {
    pub dst: cudaArray_t,
    pub wOffset: usize,
    pub hOffset: usize,
    pub src: *const ::core::ffi::c_void,
    pub spitch: usize,
    pub width: usize,
    pub height: usize,
    pub kind: cudaMemcpyKind,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMemcpy2DToArray_ptds_v7000_params_st {
    pub dst: cudaArray_t,
    pub wOffset: usize,
    pub hOffset: usize,
    pub src: *const ::core::ffi::c_void,
    pub spitch: usize,
    pub width: usize,
    pub height: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaMemcpy2DToArray_v3020_params_st {
    pub dst: cudaArray_t,
    pub wOffset: usize,
    pub hOffset: usize,
    pub src: *const ::core::ffi::c_void,
    pub spitch: usize,
    pub width: usize,
    pub height: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaMemcpy2D_ptds_v7000_params_st {
    pub dst: *mut ::core::ffi::c_void,
    pub dpitch: usize,
    pub src: *const ::core::ffi::c_void,
    pub spitch: usize,
    pub width: usize,
    pub height: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaMemcpy2D_v3020_params_st {
    pub dst: *mut ::core::ffi::c_void,
    pub dpitch: usize,
    pub src: *const ::core::ffi::c_void,
    pub spitch: usize,
    pub width: usize,
    pub height: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaMemcpy3DAsync_ptsz_v7000_params_st {
    pub p: *const cudaMemcpy3DParms,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMemcpy3DAsync_v3020_params_st {
    pub p: *const cudaMemcpy3DParms,
    pub stream: cudaStream_t,
}
#[cfg(any(feature = "cuda-12080", feature = "cuda-12090"))]
#[repr(C)]
pub struct cudaMemcpy3DBatchAsync_ptsz_v12080_params_st {
    pub numOps: usize,
    pub opList: *mut cudaMemcpy3DBatchOp,
    pub failIdx: *mut usize,
    pub flags: ::core::ffi::c_ulonglong,
    pub stream: cudaStream_t,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaMemcpy3DBatchAsync_ptsz_v13000_params_st {
    pub numOps: usize,
    pub opList: *mut cudaMemcpy3DBatchOp,
    pub flags: ::core::ffi::c_ulonglong,
    pub stream: cudaStream_t,
}
#[cfg(any(feature = "cuda-12080", feature = "cuda-12090"))]
#[repr(C)]
pub struct cudaMemcpy3DBatchAsync_v12080_params_st {
    pub numOps: usize,
    pub opList: *mut cudaMemcpy3DBatchOp,
    pub failIdx: *mut usize,
    pub flags: ::core::ffi::c_ulonglong,
    pub stream: cudaStream_t,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaMemcpy3DBatchAsync_v13000_params_st {
    pub numOps: usize,
    pub opList: *mut cudaMemcpy3DBatchOp,
    pub flags: ::core::ffi::c_ulonglong,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMemcpy3DPeerAsync_ptsz_v7000_params_st {
    pub p: *const cudaMemcpy3DPeerParms,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMemcpy3DPeerAsync_v4000_params_st {
    pub p: *const cudaMemcpy3DPeerParms,
    pub stream: cudaStream_t,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaMemcpy3DPeer_ptds_v7000_params_st {
    pub p: *const cudaMemcpy3DPeerParms,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaMemcpy3DPeer_v4000_params_st {
    pub p: *const cudaMemcpy3DPeerParms,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaMemcpy3D_ptds_v7000_params_st {
    pub p: *const cudaMemcpy3DParms,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaMemcpy3D_v3020_params_st {
    pub p: *const cudaMemcpy3DParms,
}
#[repr(C)]
pub struct cudaMemcpyArrayToArray_ptds_v7000_params_st {
    pub dst: cudaArray_t,
    pub wOffsetDst: usize,
    pub hOffsetDst: usize,
    pub src: cudaArray_const_t,
    pub wOffsetSrc: usize,
    pub hOffsetSrc: usize,
    pub count: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaMemcpyArrayToArray_v3020_params_st {
    pub dst: cudaArray_t,
    pub wOffsetDst: usize,
    pub hOffsetDst: usize,
    pub src: cudaArray_const_t,
    pub wOffsetSrc: usize,
    pub hOffsetSrc: usize,
    pub count: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaMemcpyAsync_ptsz_v7000_params_st {
    pub dst: *mut ::core::ffi::c_void,
    pub src: *const ::core::ffi::c_void,
    pub count: usize,
    pub kind: cudaMemcpyKind,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMemcpyAsync_v3020_params_st {
    pub dst: *mut ::core::ffi::c_void,
    pub src: *const ::core::ffi::c_void,
    pub count: usize,
    pub kind: cudaMemcpyKind,
    pub stream: cudaStream_t,
}
#[cfg(any(feature = "cuda-12080", feature = "cuda-12090"))]
#[repr(C)]
pub struct cudaMemcpyBatchAsync_ptsz_v12080_params_st {
    pub dsts: *mut *mut ::core::ffi::c_void,
    pub srcs: *mut *mut ::core::ffi::c_void,
    pub sizes: *mut usize,
    pub count: usize,
    pub attrs: *mut cudaMemcpyAttributes,
    pub attrsIdxs: *mut usize,
    pub numAttrs: usize,
    pub failIdx: *mut usize,
    pub stream: cudaStream_t,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaMemcpyBatchAsync_ptsz_v13000_params_st {
    pub dsts: *const *mut ::core::ffi::c_void,
    pub srcs: *const *const ::core::ffi::c_void,
    pub sizes: *const usize,
    pub count: usize,
    pub attrs: *mut cudaMemcpyAttributes,
    pub attrsIdxs: *mut usize,
    pub numAttrs: usize,
    pub stream: cudaStream_t,
}
#[cfg(any(feature = "cuda-12080", feature = "cuda-12090"))]
#[repr(C)]
pub struct cudaMemcpyBatchAsync_v12080_params_st {
    pub dsts: *mut *mut ::core::ffi::c_void,
    pub srcs: *mut *mut ::core::ffi::c_void,
    pub sizes: *mut usize,
    pub count: usize,
    pub attrs: *mut cudaMemcpyAttributes,
    pub attrsIdxs: *mut usize,
    pub numAttrs: usize,
    pub failIdx: *mut usize,
    pub stream: cudaStream_t,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaMemcpyBatchAsync_v13000_params_st {
    pub dsts: *const *mut ::core::ffi::c_void,
    pub srcs: *const *const ::core::ffi::c_void,
    pub sizes: *const usize,
    pub count: usize,
    pub attrs: *mut cudaMemcpyAttributes,
    pub attrsIdxs: *mut usize,
    pub numAttrs: usize,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMemcpyFromArrayAsync_ptsz_v7000_params_st {
    pub dst: *mut ::core::ffi::c_void,
    pub src: cudaArray_const_t,
    pub wOffset: usize,
    pub hOffset: usize,
    pub count: usize,
    pub kind: cudaMemcpyKind,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMemcpyFromArrayAsync_v3020_params_st {
    pub dst: *mut ::core::ffi::c_void,
    pub src: cudaArray_const_t,
    pub wOffset: usize,
    pub hOffset: usize,
    pub count: usize,
    pub kind: cudaMemcpyKind,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMemcpyFromArray_ptds_v7000_params_st {
    pub dst: *mut ::core::ffi::c_void,
    pub src: cudaArray_const_t,
    pub wOffset: usize,
    pub hOffset: usize,
    pub count: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaMemcpyFromArray_v3020_params_st {
    pub dst: *mut ::core::ffi::c_void,
    pub src: cudaArray_const_t,
    pub wOffset: usize,
    pub hOffset: usize,
    pub count: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaMemcpyFromSymbolAsync_ptsz_v7000_params_st {
    pub dst: *mut ::core::ffi::c_void,
    pub symbol: *const ::core::ffi::c_void,
    pub count: usize,
    pub offset: usize,
    pub kind: cudaMemcpyKind,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMemcpyFromSymbolAsync_v3020_params_st {
    pub dst: *mut ::core::ffi::c_void,
    pub symbol: *const ::core::ffi::c_void,
    pub count: usize,
    pub offset: usize,
    pub kind: cudaMemcpyKind,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMemcpyFromSymbol_ptds_v7000_params_st {
    pub dst: *mut ::core::ffi::c_void,
    pub symbol: *const ::core::ffi::c_void,
    pub count: usize,
    pub offset: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaMemcpyFromSymbol_v3020_params_st {
    pub dst: *mut ::core::ffi::c_void,
    pub symbol: *const ::core::ffi::c_void,
    pub count: usize,
    pub offset: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaMemcpyPeerAsync_v4000_params_st {
    pub dst: *mut ::core::ffi::c_void,
    pub dstDevice: ::core::ffi::c_int,
    pub src: *const ::core::ffi::c_void,
    pub srcDevice: ::core::ffi::c_int,
    pub count: usize,
    pub stream: cudaStream_t,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaMemcpyPeer_v4000_params_st {
    pub dst: *mut ::core::ffi::c_void,
    pub dstDevice: ::core::ffi::c_int,
    pub src: *const ::core::ffi::c_void,
    pub srcDevice: ::core::ffi::c_int,
    pub count: usize,
}
#[repr(C)]
pub struct cudaMemcpyToArrayAsync_ptsz_v7000_params_st {
    pub dst: cudaArray_t,
    pub wOffset: usize,
    pub hOffset: usize,
    pub src: *const ::core::ffi::c_void,
    pub count: usize,
    pub kind: cudaMemcpyKind,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMemcpyToArrayAsync_v3020_params_st {
    pub dst: cudaArray_t,
    pub wOffset: usize,
    pub hOffset: usize,
    pub src: *const ::core::ffi::c_void,
    pub count: usize,
    pub kind: cudaMemcpyKind,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMemcpyToArray_ptds_v7000_params_st {
    pub dst: cudaArray_t,
    pub wOffset: usize,
    pub hOffset: usize,
    pub src: *const ::core::ffi::c_void,
    pub count: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaMemcpyToArray_v3020_params_st {
    pub dst: cudaArray_t,
    pub wOffset: usize,
    pub hOffset: usize,
    pub src: *const ::core::ffi::c_void,
    pub count: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaMemcpyToSymbolAsync_ptsz_v7000_params_st {
    pub symbol: *const ::core::ffi::c_void,
    pub src: *const ::core::ffi::c_void,
    pub count: usize,
    pub offset: usize,
    pub kind: cudaMemcpyKind,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMemcpyToSymbolAsync_v3020_params_st {
    pub symbol: *const ::core::ffi::c_void,
    pub src: *const ::core::ffi::c_void,
    pub count: usize,
    pub offset: usize,
    pub kind: cudaMemcpyKind,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMemcpyToSymbol_ptds_v7000_params_st {
    pub symbol: *const ::core::ffi::c_void,
    pub src: *const ::core::ffi::c_void,
    pub count: usize,
    pub offset: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaMemcpyToSymbol_v3020_params_st {
    pub symbol: *const ::core::ffi::c_void,
    pub src: *const ::core::ffi::c_void,
    pub count: usize,
    pub offset: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaMemcpy_ptds_v7000_params_st {
    pub dst: *mut ::core::ffi::c_void,
    pub src: *const ::core::ffi::c_void,
    pub count: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaMemcpy_v3020_params_st {
    pub dst: *mut ::core::ffi::c_void,
    pub src: *const ::core::ffi::c_void,
    pub count: usize,
    pub kind: cudaMemcpyKind,
}
#[repr(C)]
pub struct cudaMemset2DAsync_ptsz_v7000_params_st {
    pub devPtr: *mut ::core::ffi::c_void,
    pub pitch: usize,
    pub value: ::core::ffi::c_int,
    pub width: usize,
    pub height: usize,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMemset2DAsync_v3020_params_st {
    pub devPtr: *mut ::core::ffi::c_void,
    pub pitch: usize,
    pub value: ::core::ffi::c_int,
    pub width: usize,
    pub height: usize,
    pub stream: cudaStream_t,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaMemset2D_ptds_v7000_params_st {
    pub devPtr: *mut ::core::ffi::c_void,
    pub pitch: usize,
    pub value: ::core::ffi::c_int,
    pub width: usize,
    pub height: usize,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaMemset2D_v3020_params_st {
    pub devPtr: *mut ::core::ffi::c_void,
    pub pitch: usize,
    pub value: ::core::ffi::c_int,
    pub width: usize,
    pub height: usize,
}
#[repr(C)]
pub struct cudaMemset3DAsync_ptsz_v7000_params_st {
    pub pitchedDevPtr: cudaPitchedPtr,
    pub value: ::core::ffi::c_int,
    pub extent: cudaExtent,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMemset3DAsync_v3020_params_st {
    pub pitchedDevPtr: cudaPitchedPtr,
    pub value: ::core::ffi::c_int,
    pub extent: cudaExtent,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMemset3D_ptds_v7000_params_st {
    pub pitchedDevPtr: cudaPitchedPtr,
    pub value: ::core::ffi::c_int,
    pub extent: cudaExtent,
}
#[repr(C)]
pub struct cudaMemset3D_v3020_params_st {
    pub pitchedDevPtr: cudaPitchedPtr,
    pub value: ::core::ffi::c_int,
    pub extent: cudaExtent,
}
#[repr(C)]
pub struct cudaMemsetAsync_ptsz_v7000_params_st {
    pub devPtr: *mut ::core::ffi::c_void,
    pub value: ::core::ffi::c_int,
    pub count: usize,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaMemsetAsync_v3020_params_st {
    pub devPtr: *mut ::core::ffi::c_void,
    pub value: ::core::ffi::c_int,
    pub count: usize,
    pub stream: cudaStream_t,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaMemset_ptds_v7000_params_st {
    pub devPtr: *mut ::core::ffi::c_void,
    pub value: ::core::ffi::c_int,
    pub count: usize,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaMemset_v3020_params_st {
    pub devPtr: *mut ::core::ffi::c_void,
    pub value: ::core::ffi::c_int,
    pub count: usize,
}
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaMipmappedArrayGetMemoryRequirements_v11060_params_st {
    pub memoryRequirements: *mut cudaArrayMemoryRequirements,
    pub mipmap: cudaMipmappedArray_t,
    pub device: ::core::ffi::c_int,
}
#[repr(C)]
pub struct cudaMipmappedArrayGetSparseProperties_v11010_params_st {
    pub sparseProperties: *mut cudaArraySparseProperties,
    pub mipmap: cudaMipmappedArray_t,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaOccupancyAvailableDynamicSMemPerBlock_v10200_params_st {
    pub dynamicSmemSize: *mut usize,
    pub func: *const ::core::ffi::c_void,
    pub numBlocks: ::core::ffi::c_int,
    pub blockSize: ::core::ffi::c_int,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaOccupancyMaxActiveBlocksPerMultiprocessorWithFlags_v7000_params_st {
    pub numBlocks: *mut ::core::ffi::c_int,
    pub func: *const ::core::ffi::c_void,
    pub blockSize: ::core::ffi::c_int,
    pub dynamicSMemSize: usize,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaOccupancyMaxActiveBlocksPerMultiprocessor_v6050_params_st {
    pub numBlocks: *mut ::core::ffi::c_int,
    pub func: *const ::core::ffi::c_void,
    pub blockSize: ::core::ffi::c_int,
    pub dynamicSMemSize: usize,
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaOccupancyMaxActiveClusters_v11070_params_st {
    pub numClusters: *mut ::core::ffi::c_int,
    pub func: *const ::core::ffi::c_void,
    pub launchConfig: *const cudaLaunchConfig_t,
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaOccupancyMaxPotentialClusterSize_v11070_params_st {
    pub clusterSize: *mut ::core::ffi::c_int,
    pub func: *const ::core::ffi::c_void,
    pub launchConfig: *const cudaLaunchConfig_t,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaPointerGetAttributes_v4000_params_st {
    pub attributes: *mut cudaPointerAttributes,
    pub ptr: *const ::core::ffi::c_void,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaRuntimeGetVersion_v3020_params_st {
    pub runtimeVersion: *mut ::core::ffi::c_int,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaSetDeviceFlags_v3020_params_st {
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaSetDevice_v3020_params_st {
    pub device: ::core::ffi::c_int,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11070",
    feature = "cuda-12000",
    feature = "cuda-12090"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaSetDoubleForDevice_v3020_params_st {
    pub d: *mut f64,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11070",
    feature = "cuda-12000",
    feature = "cuda-12090"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaSetDoubleForHost_v3020_params_st {
    pub d: *mut f64,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaSetValidDevices_v3020_params_st {
    pub device_arr: *mut ::core::ffi::c_int,
    pub len: ::core::ffi::c_int,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaSignalExternalSemaphoresAsync_ptsz_v11020_params_st {
    pub extSemArray: *const cudaExternalSemaphore_t,
    pub paramsArray: *const cudaExternalSemaphoreSignalParams,
    pub numExtSems: ::core::ffi::c_uint,
    pub stream: cudaStream_t,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaSignalExternalSemaphoresAsync_v11020_params_st {
    pub extSemArray: *const cudaExternalSemaphore_t,
    pub paramsArray: *const cudaExternalSemaphoreSignalParams,
    pub numExtSems: ::core::ffi::c_uint,
    pub stream: cudaStream_t,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaSignalExternalSemaphoresAsync_v2_ptsz_v11020_params_st {
    pub extSemArray: *const cudaExternalSemaphore_t,
    pub paramsArray: *const cudaExternalSemaphoreSignalParams,
    pub numExtSems: ::core::ffi::c_uint,
    pub stream: cudaStream_t,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaSignalExternalSemaphoresAsync_v2_v11020_params_st {
    pub extSemArray: *const cudaExternalSemaphore_t,
    pub paramsArray: *const cudaExternalSemaphoreSignalParams,
    pub numExtSems: ::core::ffi::c_uint,
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaStreamAddCallback_ptsz_v7000_params_st {
    pub stream: cudaStream_t,
    pub callback: cudaStreamCallback_t,
    pub userData: *mut ::core::ffi::c_void,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaStreamAddCallback_v5000_params_st {
    pub stream: cudaStream_t,
    pub callback: cudaStreamCallback_t,
    pub userData: *mut ::core::ffi::c_void,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaStreamAttachMemAsync_ptsz_v7000_params_st {
    pub stream: cudaStream_t,
    pub devPtr: *mut ::core::ffi::c_void,
    pub length: usize,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaStreamAttachMemAsync_v6000_params_st {
    pub stream: cudaStream_t,
    pub devPtr: *mut ::core::ffi::c_void,
    pub length: usize,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaStreamBeginCaptureToGraph_ptsz_v12030_params_st {
    pub stream: cudaStream_t,
    pub graph: cudaGraph_t,
    pub dependencies: *const cudaGraphNode_t,
    pub dependencyData: *const cudaGraphEdgeData,
    pub numDependencies: usize,
    pub mode: cudaStreamCaptureMode,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaStreamBeginCaptureToGraph_v12030_params_st {
    pub stream: cudaStream_t,
    pub graph: cudaGraph_t,
    pub dependencies: *const cudaGraphNode_t,
    pub dependencyData: *const cudaGraphEdgeData,
    pub numDependencies: usize,
    pub mode: cudaStreamCaptureMode,
}
#[repr(C)]
pub struct cudaStreamBeginCapture_ptsz_v10000_params_st {
    pub stream: cudaStream_t,
    pub mode: cudaStreamCaptureMode,
}
#[repr(C)]
pub struct cudaStreamBeginCapture_v10000_params_st {
    pub stream: cudaStream_t,
    pub mode: cudaStreamCaptureMode,
}
#[repr(C)]
pub struct cudaStreamCopyAttributes_ptsz_v11000_params_st {
    pub dst: cudaStream_t,
    pub src: cudaStream_t,
}
#[repr(C)]
pub struct cudaStreamCopyAttributes_v11000_params_st {
    pub dstStream: cudaStream_t,
    pub srcStream: cudaStream_t,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaStreamCreateWithFlags_v5000_params_st {
    pub pStream: *mut cudaStream_t,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaStreamCreateWithPriority_v5050_params_st {
    pub pStream: *mut cudaStream_t,
    pub flags: ::core::ffi::c_uint,
    pub priority: ::core::ffi::c_int,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaStreamCreate_v3020_params_st {
    pub pStream: *mut cudaStream_t,
}
#[repr(C)]
pub struct cudaStreamDestroy_v5050_params_st {
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaStreamEndCapture_ptsz_v10000_params_st {
    pub stream: cudaStream_t,
    pub pGraph: *mut cudaGraph_t,
}
#[repr(C)]
pub struct cudaStreamEndCapture_v10000_params_st {
    pub stream: cudaStream_t,
    pub pGraph: *mut cudaGraph_t,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070"
))]
#[repr(C)]
pub struct cudaStreamGetAttribute_ptsz_v11000_params_st {
    pub hStream: cudaStream_t,
    pub attr: cudaStreamAttrID,
    pub value_out: *mut cudaStreamAttrValue,
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaStreamGetAttribute_ptsz_v11000_params_st {
    pub hStream: cudaStream_t,
    pub attr: cudaLaunchAttributeID,
    pub value_out: *mut cudaLaunchAttributeValue,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070"
))]
#[repr(C)]
pub struct cudaStreamGetAttribute_v11000_params_st {
    pub stream: cudaStream_t,
    pub attr: cudaStreamAttrID,
    pub value: *mut cudaStreamAttrValue,
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaStreamGetAttribute_v11000_params_st {
    pub stream: cudaStream_t,
    pub attr: cudaLaunchAttributeID,
    pub value: *mut cudaLaunchAttributeValue,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
#[repr(C)]
pub struct cudaStreamGetCaptureInfo_ptsz_v10010_params_st {
    pub stream: cudaStream_t,
    pub pCaptureStatus: *mut cudaStreamCaptureStatus,
    pub pId: *mut ::core::ffi::c_ulonglong,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaStreamGetCaptureInfo_ptsz_v10010_params_st {
    pub stream: cudaStream_t,
    pub captureStatus_out: *mut cudaStreamCaptureStatus,
    pub id_out: *mut ::core::ffi::c_ulonglong,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaStreamGetCaptureInfo_ptsz_v12030_params_st {
    pub stream: cudaStream_t,
    pub captureStatus_out: *mut cudaStreamCaptureStatus,
    pub id_out: *mut ::core::ffi::c_ulonglong,
    pub graph_out: *mut cudaGraph_t,
    pub dependencies_out: *mut *const cudaGraphNode_t,
    pub edgeData_out: *mut *const cudaGraphEdgeData,
    pub numDependencies_out: *mut usize,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaStreamGetCaptureInfo_v10010_params_st {
    pub stream: cudaStream_t,
    pub captureStatus_out: *mut cudaStreamCaptureStatus,
    pub id_out: *mut ::core::ffi::c_ulonglong,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaStreamGetCaptureInfo_v12030_params_st {
    pub stream: cudaStream_t,
    pub captureStatus_out: *mut cudaStreamCaptureStatus,
    pub id_out: *mut ::core::ffi::c_ulonglong,
    pub graph_out: *mut cudaGraph_t,
    pub dependencies_out: *mut *const cudaGraphNode_t,
    pub edgeData_out: *mut *const cudaGraphEdgeData,
    pub numDependencies_out: *mut usize,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaStreamGetCaptureInfo_v2_ptsz_v11030_params_st {
    pub stream: cudaStream_t,
    pub captureStatus_out: *mut cudaStreamCaptureStatus,
    pub id_out: *mut ::core::ffi::c_ulonglong,
    pub graph_out: *mut cudaGraph_t,
    pub dependencies_out: *mut *const cudaGraphNode_t,
    pub numDependencies_out: *mut usize,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaStreamGetCaptureInfo_v2_v11030_params_st {
    pub stream: cudaStream_t,
    pub captureStatus_out: *mut cudaStreamCaptureStatus,
    pub id_out: *mut ::core::ffi::c_ulonglong,
    pub graph_out: *mut cudaGraph_t,
    pub dependencies_out: *mut *const cudaGraphNode_t,
    pub numDependencies_out: *mut usize,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaStreamGetCaptureInfo_v3_ptsz_v12030_params_st {
    pub stream: cudaStream_t,
    pub captureStatus_out: *mut cudaStreamCaptureStatus,
    pub id_out: *mut ::core::ffi::c_ulonglong,
    pub graph_out: *mut cudaGraph_t,
    pub dependencies_out: *mut *const cudaGraphNode_t,
    pub edgeData_out: *mut *const cudaGraphEdgeData,
    pub numDependencies_out: *mut usize,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaStreamGetCaptureInfo_v3_v12030_params_st {
    pub stream: cudaStream_t,
    pub captureStatus_out: *mut cudaStreamCaptureStatus,
    pub id_out: *mut ::core::ffi::c_ulonglong,
    pub graph_out: *mut cudaGraph_t,
    pub dependencies_out: *mut *const cudaGraphNode_t,
    pub edgeData_out: *mut *const cudaGraphEdgeData,
    pub numDependencies_out: *mut usize,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaStreamGetDevResource_ptsz_v13010_params_st {
    pub hStream: cudaStream_t,
    pub resource: *mut cudaDevResource,
    pub type_: cudaDevResourceType,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaStreamGetDevResource_v13010_params_st {
    pub hStream: cudaStream_t,
    pub resource: *mut cudaDevResource,
    pub type_: cudaDevResourceType,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaStreamGetDevice_ptsz_v12080_params_st {
    pub hStream: cudaStream_t,
    pub device: *mut ::core::ffi::c_int,
}
#[cfg(any(
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaStreamGetDevice_v12080_params_st {
    pub hStream: cudaStream_t,
    pub device: *mut ::core::ffi::c_int,
}
#[repr(C)]
pub struct cudaStreamGetFlags_ptsz_v7000_params_st {
    pub hStream: cudaStream_t,
    pub flags: *mut ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaStreamGetFlags_v5050_params_st {
    pub hStream: cudaStream_t,
    pub flags: *mut ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaStreamGetId_ptsz_v12000_params_st {
    pub hStream: cudaStream_t,
    pub streamId: *mut ::core::ffi::c_ulonglong,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaStreamGetId_v12000_params_st {
    pub hStream: cudaStream_t,
    pub streamId: *mut ::core::ffi::c_ulonglong,
}
#[repr(C)]
pub struct cudaStreamGetPriority_ptsz_v7000_params_st {
    pub hStream: cudaStream_t,
    pub priority: *mut ::core::ffi::c_int,
}
#[repr(C)]
pub struct cudaStreamGetPriority_v5050_params_st {
    pub hStream: cudaStream_t,
    pub priority: *mut ::core::ffi::c_int,
}
#[repr(C)]
pub struct cudaStreamIsCapturing_ptsz_v10000_params_st {
    pub stream: cudaStream_t,
    pub pCaptureStatus: *mut cudaStreamCaptureStatus,
}
#[repr(C)]
pub struct cudaStreamIsCapturing_v10000_params_st {
    pub stream: cudaStream_t,
    pub pCaptureStatus: *mut cudaStreamCaptureStatus,
}
#[repr(C)]
pub struct cudaStreamQuery_ptsz_v7000_params_st {
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaStreamQuery_v3020_params_st {
    pub stream: cudaStream_t,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070"
))]
#[repr(C)]
pub struct cudaStreamSetAttribute_ptsz_v11000_params_st {
    pub hStream: cudaStream_t,
    pub attr: cudaStreamAttrID,
    pub value: *const cudaStreamAttrValue,
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaStreamSetAttribute_ptsz_v11000_params_st {
    pub hStream: cudaStream_t,
    pub attr: cudaLaunchAttributeID,
    pub value: *const cudaLaunchAttributeValue,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070"
))]
#[repr(C)]
pub struct cudaStreamSetAttribute_v11000_params_st {
    pub stream: cudaStream_t,
    pub attr: cudaStreamAttrID,
    pub param: *const cudaStreamAttrValue,
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
pub struct cudaStreamSetAttribute_v11000_params_st {
    pub stream: cudaStream_t,
    pub attr: cudaLaunchAttributeID,
    pub param: *const cudaLaunchAttributeValue,
}
#[repr(C)]
pub struct cudaStreamSynchronize_ptsz_v7000_params_st {
    pub stream: cudaStream_t,
}
#[repr(C)]
pub struct cudaStreamSynchronize_v3020_params_st {
    pub stream: cudaStream_t,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaStreamUpdateCaptureDependencies_ptsz_v11030_params_st {
    pub stream: cudaStream_t,
    pub dependencies: *mut cudaGraphNode_t,
    pub numDependencies: usize,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaStreamUpdateCaptureDependencies_ptsz_v12030_params_st {
    pub stream: cudaStream_t,
    pub dependencies: *mut cudaGraphNode_t,
    pub dependencyData: *const cudaGraphEdgeData,
    pub numDependencies: usize,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaStreamUpdateCaptureDependencies_v11030_params_st {
    pub stream: cudaStream_t,
    pub dependencies: *mut cudaGraphNode_t,
    pub numDependencies: usize,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaStreamUpdateCaptureDependencies_v12030_params_st {
    pub stream: cudaStream_t,
    pub dependencies: *mut cudaGraphNode_t,
    pub dependencyData: *const cudaGraphEdgeData,
    pub numDependencies: usize,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaStreamUpdateCaptureDependencies_v2_ptsz_v12030_params_st {
    pub stream: cudaStream_t,
    pub dependencies: *mut cudaGraphNode_t,
    pub dependencyData: *const cudaGraphEdgeData,
    pub numDependencies: usize,
    pub flags: ::core::ffi::c_uint,
}
#[cfg(any(
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaStreamUpdateCaptureDependencies_v2_v12030_params_st {
    pub stream: cudaStream_t,
    pub dependencies: *mut cudaGraphNode_t,
    pub dependencyData: *const cudaGraphEdgeData,
    pub numDependencies: usize,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaStreamWaitEvent_ptsz_v7000_params_st {
    pub stream: cudaStream_t,
    pub event: cudaEvent_t,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaStreamWaitEvent_v3020_params_st {
    pub stream: cudaStream_t,
    pub event: cudaEvent_t,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaThreadExchangeStreamCaptureMode_v10010_params_st {
    pub mode: *mut cudaStreamCaptureMode,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11070",
    feature = "cuda-12000",
    feature = "cuda-12090"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaThreadGetCacheConfig_v3020_params_st {
    pub pCacheConfig: *mut cudaFuncCache,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11070",
    feature = "cuda-12000",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaThreadGetLimit_v3020_params_st {
    pub pValue: *mut usize,
    pub limit: cudaLimit,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11070",
    feature = "cuda-12000",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaThreadSetCacheConfig_v3020_params_st {
    pub cacheConfig: cudaFuncCache,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11070",
    feature = "cuda-12000",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaThreadSetLimit_v3020_params_st {
    pub limit: cudaLimit,
    pub value: usize,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080"
))]
#[repr(C)]
#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
pub struct cudaUnbindTexture_v3020_params_st {
    pub texref: *const textureReference,
}
#[repr(C)]
pub struct cudaUserObjectCreate_v11030_params_st {
    pub object_out: *mut cudaUserObject_t,
    pub ptr: *mut ::core::ffi::c_void,
    pub destroy: cudaHostFn_t,
    pub initialRefcount: ::core::ffi::c_uint,
    pub flags: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaUserObjectRelease_v11030_params_st {
    pub object: cudaUserObject_t,
    pub count: ::core::ffi::c_uint,
}
#[repr(C)]
pub struct cudaUserObjectRetain_v11030_params_st {
    pub object: cudaUserObject_t,
    pub count: ::core::ffi::c_uint,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaWaitExternalSemaphoresAsync_ptsz_v11020_params_st {
    pub extSemArray: *const cudaExternalSemaphore_t,
    pub paramsArray: *const cudaExternalSemaphoreWaitParams,
    pub numExtSems: ::core::ffi::c_uint,
    pub stream: cudaStream_t,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
pub struct cudaWaitExternalSemaphoresAsync_v11020_params_st {
    pub extSemArray: *const cudaExternalSemaphore_t,
    pub paramsArray: *const cudaExternalSemaphoreWaitParams,
    pub numExtSems: ::core::ffi::c_uint,
    pub stream: cudaStream_t,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaWaitExternalSemaphoresAsync_v2_ptsz_v11020_params_st {
    pub extSemArray: *const cudaExternalSemaphore_t,
    pub paramsArray: *const cudaExternalSemaphoreWaitParams,
    pub numExtSems: ::core::ffi::c_uint,
    pub stream: cudaStream_t,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
pub struct cudaWaitExternalSemaphoresAsync_v2_v11020_params_st {
    pub extSemArray: *const cudaExternalSemaphore_t,
    pub paramsArray: *const cudaExternalSemaphoreWaitParams,
    pub numExtSems: ::core::ffi::c_uint,
    pub stream: cudaStream_t,
}
impl CUpti_ActivityCdpKernel__bindgen_ty_1__bindgen_ty_1 {
    #[inline]
    pub fn requested(&self) -> u8 {
        unsafe { ::core::mem::transmute(self._bitfield_1.get(0usize, 4u8) as u8) }
    }
    #[inline]
    pub fn set_requested(&mut self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            self._bitfield_1.set(0usize, 4u8, val as u64)
        }
    }
    #[inline]
    pub unsafe fn requested_raw(this: *const Self) -> u8 {
        unsafe {
            ::core::mem::transmute(<__BindgenBitfieldUnit<[u8; 1usize]>>::raw_get(
                ::core::ptr::addr_of!((*this)._bitfield_1),
                0usize,
                4u8,
            ) as u8)
        }
    }
    #[inline]
    pub unsafe fn set_requested_raw(this: *mut Self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            <__BindgenBitfieldUnit<[u8; 1usize]>>::raw_set(
                ::core::ptr::addr_of_mut!((*this)._bitfield_1),
                0usize,
                4u8,
                val as u64,
            )
        }
    }
    #[inline]
    pub fn executed(&self) -> u8 {
        unsafe { ::core::mem::transmute(self._bitfield_1.get(4usize, 4u8) as u8) }
    }
    #[inline]
    pub fn set_executed(&mut self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            self._bitfield_1.set(4usize, 4u8, val as u64)
        }
    }
    #[inline]
    pub unsafe fn executed_raw(this: *const Self) -> u8 {
        unsafe {
            ::core::mem::transmute(<__BindgenBitfieldUnit<[u8; 1usize]>>::raw_get(
                ::core::ptr::addr_of!((*this)._bitfield_1),
                4usize,
                4u8,
            ) as u8)
        }
    }
    #[inline]
    pub unsafe fn set_executed_raw(this: *mut Self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            <__BindgenBitfieldUnit<[u8; 1usize]>>::raw_set(
                ::core::ptr::addr_of_mut!((*this)._bitfield_1),
                4usize,
                4u8,
                val as u64,
            )
        }
    }
    #[inline]
    pub fn new_bitfield_1(requested: u8, executed: u8) -> __BindgenBitfieldUnit<[u8; 1usize]> {
        let mut __bindgen_bitfield_unit: __BindgenBitfieldUnit<[u8; 1usize]> = Default::default();
        __bindgen_bitfield_unit.set(0usize, 4u8, {
            let requested: u8 = unsafe { ::core::mem::transmute(requested) };
            requested as u64
        });
        __bindgen_bitfield_unit.set(4usize, 4u8, {
            let executed: u8 = unsafe { ::core::mem::transmute(executed) };
            executed as u64
        });
        __bindgen_bitfield_unit
    }
}
impl CUpti_ActivityFlag {
    pub const CUPTI_ACTIVITY_FLAG_DEVICE_ATTRIBUTE_CUDEVICE: CUpti_ActivityFlag =
        CUpti_ActivityFlag::CUPTI_ACTIVITY_FLAG_DEVICE_CONCURRENT_KERNELS;
}
impl CUpti_ActivityFlag {
    pub const CUPTI_ACTIVITY_FLAG_FLUSH_FORCED: CUpti_ActivityFlag =
        CUpti_ActivityFlag::CUPTI_ACTIVITY_FLAG_DEVICE_CONCURRENT_KERNELS;
}
impl CUpti_ActivityFlag {
    pub const CUPTI_ACTIVITY_FLAG_INSTRUCTION_VALUE_INVALID: CUpti_ActivityFlag =
        CUpti_ActivityFlag::CUPTI_ACTIVITY_FLAG_DEVICE_CONCURRENT_KERNELS;
}
impl CUpti_ActivityFlag {
    pub const CUPTI_ACTIVITY_FLAG_MARKER_COLOR_ARGB: CUpti_ActivityFlag =
        CUpti_ActivityFlag::CUPTI_ACTIVITY_FLAG_MARKER_START;
}
impl CUpti_ActivityFlag {
    pub const CUPTI_ACTIVITY_FLAG_MARKER_COLOR_NONE: CUpti_ActivityFlag =
        CUpti_ActivityFlag::CUPTI_ACTIVITY_FLAG_DEVICE_CONCURRENT_KERNELS;
}
impl CUpti_ActivityFlag {
    pub const CUPTI_ACTIVITY_FLAG_MARKER_INSTANTANEOUS: CUpti_ActivityFlag =
        CUpti_ActivityFlag::CUPTI_ACTIVITY_FLAG_DEVICE_CONCURRENT_KERNELS;
}
impl CUpti_ActivityFlag {
    pub const CUPTI_ACTIVITY_FLAG_MEMCPY_ASYNC: CUpti_ActivityFlag =
        CUpti_ActivityFlag::CUPTI_ACTIVITY_FLAG_DEVICE_CONCURRENT_KERNELS;
}
impl CUpti_ActivityFlag {
    pub const CUPTI_ACTIVITY_FLAG_MEMSET_ASYNC: CUpti_ActivityFlag =
        CUpti_ActivityFlag::CUPTI_ACTIVITY_FLAG_DEVICE_CONCURRENT_KERNELS;
}
impl CUpti_ActivityFlag {
    pub const CUPTI_ACTIVITY_FLAG_METRIC_OVERFLOWED: CUpti_ActivityFlag =
        CUpti_ActivityFlag::CUPTI_ACTIVITY_FLAG_DEVICE_CONCURRENT_KERNELS;
}
impl CUpti_ActivityFlag {
    pub const CUPTI_ACTIVITY_FLAG_METRIC_VALUE_INVALID: CUpti_ActivityFlag =
        CUpti_ActivityFlag::CUPTI_ACTIVITY_FLAG_MARKER_START;
}
impl CUpti_ActivityFlag {
    pub const CUPTI_ACTIVITY_FLAG_SHARED_ACCESS_KIND_LOAD: CUpti_ActivityFlag =
        CUpti_ActivityFlag::CUPTI_ACTIVITY_FLAG_GLOBAL_ACCESS_KIND_LOAD;
}
impl CUpti_ActivityFlag {
    pub const CUPTI_ACTIVITY_FLAG_SHARED_ACCESS_KIND_SIZE_MASK: CUpti_ActivityFlag =
        CUpti_ActivityFlag::CUPTI_ACTIVITY_FLAG_GLOBAL_ACCESS_KIND_SIZE_MASK;
}
impl CUpti_ActivityFlag {
    pub const CUPTI_ACTIVITY_FLAG_THRASHING_IN_CPU: CUpti_ActivityFlag =
        CUpti_ActivityFlag::CUPTI_ACTIVITY_FLAG_DEVICE_CONCURRENT_KERNELS;
}
impl CUpti_ActivityFlag {
    pub const CUPTI_ACTIVITY_FLAG_THROTTLING_IN_CPU: CUpti_ActivityFlag =
        CUpti_ActivityFlag::CUPTI_ACTIVITY_FLAG_DEVICE_CONCURRENT_KERNELS;
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
impl CUpti_ActivityKernel10__bindgen_ty_1__bindgen_ty_1 {
    #[inline]
    pub fn requested(&self) -> u8 {
        unsafe { ::core::mem::transmute(self._bitfield_1.get(0usize, 4u8) as u8) }
    }
    #[inline]
    pub fn set_requested(&mut self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            self._bitfield_1.set(0usize, 4u8, val as u64)
        }
    }
    #[inline]
    pub unsafe fn requested_raw(this: *const Self) -> u8 {
        unsafe {
            ::core::mem::transmute(<__BindgenBitfieldUnit<[u8; 1usize]>>::raw_get(
                ::core::ptr::addr_of!((*this)._bitfield_1),
                0usize,
                4u8,
            ) as u8)
        }
    }
    #[inline]
    pub unsafe fn set_requested_raw(this: *mut Self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            <__BindgenBitfieldUnit<[u8; 1usize]>>::raw_set(
                ::core::ptr::addr_of_mut!((*this)._bitfield_1),
                0usize,
                4u8,
                val as u64,
            )
        }
    }
    #[inline]
    pub fn executed(&self) -> u8 {
        unsafe { ::core::mem::transmute(self._bitfield_1.get(4usize, 4u8) as u8) }
    }
    #[inline]
    pub fn set_executed(&mut self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            self._bitfield_1.set(4usize, 4u8, val as u64)
        }
    }
    #[inline]
    pub unsafe fn executed_raw(this: *const Self) -> u8 {
        unsafe {
            ::core::mem::transmute(<__BindgenBitfieldUnit<[u8; 1usize]>>::raw_get(
                ::core::ptr::addr_of!((*this)._bitfield_1),
                4usize,
                4u8,
            ) as u8)
        }
    }
    #[inline]
    pub unsafe fn set_executed_raw(this: *mut Self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            <__BindgenBitfieldUnit<[u8; 1usize]>>::raw_set(
                ::core::ptr::addr_of_mut!((*this)._bitfield_1),
                4usize,
                4u8,
                val as u64,
            )
        }
    }
    #[inline]
    pub fn new_bitfield_1(requested: u8, executed: u8) -> __BindgenBitfieldUnit<[u8; 1usize]> {
        let mut __bindgen_bitfield_unit: __BindgenBitfieldUnit<[u8; 1usize]> = Default::default();
        __bindgen_bitfield_unit.set(0usize, 4u8, {
            let requested: u8 = unsafe { ::core::mem::transmute(requested) };
            requested as u64
        });
        __bindgen_bitfield_unit.set(4usize, 4u8, {
            let executed: u8 = unsafe { ::core::mem::transmute(executed) };
            executed as u64
        });
        __bindgen_bitfield_unit
    }
}
#[cfg(any(feature = "cuda-13010"))]
impl CUpti_ActivityKernel11__bindgen_ty_1__bindgen_ty_1 {
    #[inline]
    pub fn requested(&self) -> u8 {
        unsafe { ::core::mem::transmute(self._bitfield_1.get(0usize, 4u8) as u8) }
    }
    #[inline]
    pub fn set_requested(&mut self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            self._bitfield_1.set(0usize, 4u8, val as u64)
        }
    }
    #[inline]
    pub unsafe fn requested_raw(this: *const Self) -> u8 {
        unsafe {
            ::core::mem::transmute(<__BindgenBitfieldUnit<[u8; 1usize]>>::raw_get(
                ::core::ptr::addr_of!((*this)._bitfield_1),
                0usize,
                4u8,
            ) as u8)
        }
    }
    #[inline]
    pub unsafe fn set_requested_raw(this: *mut Self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            <__BindgenBitfieldUnit<[u8; 1usize]>>::raw_set(
                ::core::ptr::addr_of_mut!((*this)._bitfield_1),
                0usize,
                4u8,
                val as u64,
            )
        }
    }
    #[inline]
    pub fn executed(&self) -> u8 {
        unsafe { ::core::mem::transmute(self._bitfield_1.get(4usize, 4u8) as u8) }
    }
    #[inline]
    pub fn set_executed(&mut self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            self._bitfield_1.set(4usize, 4u8, val as u64)
        }
    }
    #[inline]
    pub unsafe fn executed_raw(this: *const Self) -> u8 {
        unsafe {
            ::core::mem::transmute(<__BindgenBitfieldUnit<[u8; 1usize]>>::raw_get(
                ::core::ptr::addr_of!((*this)._bitfield_1),
                4usize,
                4u8,
            ) as u8)
        }
    }
    #[inline]
    pub unsafe fn set_executed_raw(this: *mut Self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            <__BindgenBitfieldUnit<[u8; 1usize]>>::raw_set(
                ::core::ptr::addr_of_mut!((*this)._bitfield_1),
                4usize,
                4u8,
                val as u64,
            )
        }
    }
    #[inline]
    pub fn new_bitfield_1(requested: u8, executed: u8) -> __BindgenBitfieldUnit<[u8; 1usize]> {
        let mut __bindgen_bitfield_unit: __BindgenBitfieldUnit<[u8; 1usize]> = Default::default();
        __bindgen_bitfield_unit.set(0usize, 4u8, {
            let requested: u8 = unsafe { ::core::mem::transmute(requested) };
            requested as u64
        });
        __bindgen_bitfield_unit.set(4usize, 4u8, {
            let executed: u8 = unsafe { ::core::mem::transmute(executed) };
            executed as u64
        });
        __bindgen_bitfield_unit
    }
}
impl CUpti_ActivityKernel2__bindgen_ty_1__bindgen_ty_1 {
    #[inline]
    pub fn requested(&self) -> u8 {
        unsafe { ::core::mem::transmute(self._bitfield_1.get(0usize, 4u8) as u8) }
    }
    #[inline]
    pub fn set_requested(&mut self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            self._bitfield_1.set(0usize, 4u8, val as u64)
        }
    }
    #[inline]
    pub unsafe fn requested_raw(this: *const Self) -> u8 {
        unsafe {
            ::core::mem::transmute(<__BindgenBitfieldUnit<[u8; 1usize]>>::raw_get(
                ::core::ptr::addr_of!((*this)._bitfield_1),
                0usize,
                4u8,
            ) as u8)
        }
    }
    #[inline]
    pub unsafe fn set_requested_raw(this: *mut Self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            <__BindgenBitfieldUnit<[u8; 1usize]>>::raw_set(
                ::core::ptr::addr_of_mut!((*this)._bitfield_1),
                0usize,
                4u8,
                val as u64,
            )
        }
    }
    #[inline]
    pub fn executed(&self) -> u8 {
        unsafe { ::core::mem::transmute(self._bitfield_1.get(4usize, 4u8) as u8) }
    }
    #[inline]
    pub fn set_executed(&mut self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            self._bitfield_1.set(4usize, 4u8, val as u64)
        }
    }
    #[inline]
    pub unsafe fn executed_raw(this: *const Self) -> u8 {
        unsafe {
            ::core::mem::transmute(<__BindgenBitfieldUnit<[u8; 1usize]>>::raw_get(
                ::core::ptr::addr_of!((*this)._bitfield_1),
                4usize,
                4u8,
            ) as u8)
        }
    }
    #[inline]
    pub unsafe fn set_executed_raw(this: *mut Self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            <__BindgenBitfieldUnit<[u8; 1usize]>>::raw_set(
                ::core::ptr::addr_of_mut!((*this)._bitfield_1),
                4usize,
                4u8,
                val as u64,
            )
        }
    }
    #[inline]
    pub fn new_bitfield_1(requested: u8, executed: u8) -> __BindgenBitfieldUnit<[u8; 1usize]> {
        let mut __bindgen_bitfield_unit: __BindgenBitfieldUnit<[u8; 1usize]> = Default::default();
        __bindgen_bitfield_unit.set(0usize, 4u8, {
            let requested: u8 = unsafe { ::core::mem::transmute(requested) };
            requested as u64
        });
        __bindgen_bitfield_unit.set(4usize, 4u8, {
            let executed: u8 = unsafe { ::core::mem::transmute(executed) };
            executed as u64
        });
        __bindgen_bitfield_unit
    }
}
impl CUpti_ActivityKernel3__bindgen_ty_1__bindgen_ty_1 {
    #[inline]
    pub fn requested(&self) -> u8 {
        unsafe { ::core::mem::transmute(self._bitfield_1.get(0usize, 4u8) as u8) }
    }
    #[inline]
    pub fn set_requested(&mut self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            self._bitfield_1.set(0usize, 4u8, val as u64)
        }
    }
    #[inline]
    pub unsafe fn requested_raw(this: *const Self) -> u8 {
        unsafe {
            ::core::mem::transmute(<__BindgenBitfieldUnit<[u8; 1usize]>>::raw_get(
                ::core::ptr::addr_of!((*this)._bitfield_1),
                0usize,
                4u8,
            ) as u8)
        }
    }
    #[inline]
    pub unsafe fn set_requested_raw(this: *mut Self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            <__BindgenBitfieldUnit<[u8; 1usize]>>::raw_set(
                ::core::ptr::addr_of_mut!((*this)._bitfield_1),
                0usize,
                4u8,
                val as u64,
            )
        }
    }
    #[inline]
    pub fn executed(&self) -> u8 {
        unsafe { ::core::mem::transmute(self._bitfield_1.get(4usize, 4u8) as u8) }
    }
    #[inline]
    pub fn set_executed(&mut self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            self._bitfield_1.set(4usize, 4u8, val as u64)
        }
    }
    #[inline]
    pub unsafe fn executed_raw(this: *const Self) -> u8 {
        unsafe {
            ::core::mem::transmute(<__BindgenBitfieldUnit<[u8; 1usize]>>::raw_get(
                ::core::ptr::addr_of!((*this)._bitfield_1),
                4usize,
                4u8,
            ) as u8)
        }
    }
    #[inline]
    pub unsafe fn set_executed_raw(this: *mut Self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            <__BindgenBitfieldUnit<[u8; 1usize]>>::raw_set(
                ::core::ptr::addr_of_mut!((*this)._bitfield_1),
                4usize,
                4u8,
                val as u64,
            )
        }
    }
    #[inline]
    pub fn new_bitfield_1(requested: u8, executed: u8) -> __BindgenBitfieldUnit<[u8; 1usize]> {
        let mut __bindgen_bitfield_unit: __BindgenBitfieldUnit<[u8; 1usize]> = Default::default();
        __bindgen_bitfield_unit.set(0usize, 4u8, {
            let requested: u8 = unsafe { ::core::mem::transmute(requested) };
            requested as u64
        });
        __bindgen_bitfield_unit.set(4usize, 4u8, {
            let executed: u8 = unsafe { ::core::mem::transmute(executed) };
            executed as u64
        });
        __bindgen_bitfield_unit
    }
}
impl CUpti_ActivityKernel4__bindgen_ty_1__bindgen_ty_1 {
    #[inline]
    pub fn requested(&self) -> u8 {
        unsafe { ::core::mem::transmute(self._bitfield_1.get(0usize, 4u8) as u8) }
    }
    #[inline]
    pub fn set_requested(&mut self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            self._bitfield_1.set(0usize, 4u8, val as u64)
        }
    }
    #[inline]
    pub unsafe fn requested_raw(this: *const Self) -> u8 {
        unsafe {
            ::core::mem::transmute(<__BindgenBitfieldUnit<[u8; 1usize]>>::raw_get(
                ::core::ptr::addr_of!((*this)._bitfield_1),
                0usize,
                4u8,
            ) as u8)
        }
    }
    #[inline]
    pub unsafe fn set_requested_raw(this: *mut Self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            <__BindgenBitfieldUnit<[u8; 1usize]>>::raw_set(
                ::core::ptr::addr_of_mut!((*this)._bitfield_1),
                0usize,
                4u8,
                val as u64,
            )
        }
    }
    #[inline]
    pub fn executed(&self) -> u8 {
        unsafe { ::core::mem::transmute(self._bitfield_1.get(4usize, 4u8) as u8) }
    }
    #[inline]
    pub fn set_executed(&mut self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            self._bitfield_1.set(4usize, 4u8, val as u64)
        }
    }
    #[inline]
    pub unsafe fn executed_raw(this: *const Self) -> u8 {
        unsafe {
            ::core::mem::transmute(<__BindgenBitfieldUnit<[u8; 1usize]>>::raw_get(
                ::core::ptr::addr_of!((*this)._bitfield_1),
                4usize,
                4u8,
            ) as u8)
        }
    }
    #[inline]
    pub unsafe fn set_executed_raw(this: *mut Self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            <__BindgenBitfieldUnit<[u8; 1usize]>>::raw_set(
                ::core::ptr::addr_of_mut!((*this)._bitfield_1),
                4usize,
                4u8,
                val as u64,
            )
        }
    }
    #[inline]
    pub fn new_bitfield_1(requested: u8, executed: u8) -> __BindgenBitfieldUnit<[u8; 1usize]> {
        let mut __bindgen_bitfield_unit: __BindgenBitfieldUnit<[u8; 1usize]> = Default::default();
        __bindgen_bitfield_unit.set(0usize, 4u8, {
            let requested: u8 = unsafe { ::core::mem::transmute(requested) };
            requested as u64
        });
        __bindgen_bitfield_unit.set(4usize, 4u8, {
            let executed: u8 = unsafe { ::core::mem::transmute(executed) };
            executed as u64
        });
        __bindgen_bitfield_unit
    }
}
impl CUpti_ActivityKernel5__bindgen_ty_1__bindgen_ty_1 {
    #[inline]
    pub fn requested(&self) -> u8 {
        unsafe { ::core::mem::transmute(self._bitfield_1.get(0usize, 4u8) as u8) }
    }
    #[inline]
    pub fn set_requested(&mut self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            self._bitfield_1.set(0usize, 4u8, val as u64)
        }
    }
    #[inline]
    pub unsafe fn requested_raw(this: *const Self) -> u8 {
        unsafe {
            ::core::mem::transmute(<__BindgenBitfieldUnit<[u8; 1usize]>>::raw_get(
                ::core::ptr::addr_of!((*this)._bitfield_1),
                0usize,
                4u8,
            ) as u8)
        }
    }
    #[inline]
    pub unsafe fn set_requested_raw(this: *mut Self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            <__BindgenBitfieldUnit<[u8; 1usize]>>::raw_set(
                ::core::ptr::addr_of_mut!((*this)._bitfield_1),
                0usize,
                4u8,
                val as u64,
            )
        }
    }
    #[inline]
    pub fn executed(&self) -> u8 {
        unsafe { ::core::mem::transmute(self._bitfield_1.get(4usize, 4u8) as u8) }
    }
    #[inline]
    pub fn set_executed(&mut self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            self._bitfield_1.set(4usize, 4u8, val as u64)
        }
    }
    #[inline]
    pub unsafe fn executed_raw(this: *const Self) -> u8 {
        unsafe {
            ::core::mem::transmute(<__BindgenBitfieldUnit<[u8; 1usize]>>::raw_get(
                ::core::ptr::addr_of!((*this)._bitfield_1),
                4usize,
                4u8,
            ) as u8)
        }
    }
    #[inline]
    pub unsafe fn set_executed_raw(this: *mut Self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            <__BindgenBitfieldUnit<[u8; 1usize]>>::raw_set(
                ::core::ptr::addr_of_mut!((*this)._bitfield_1),
                4usize,
                4u8,
                val as u64,
            )
        }
    }
    #[inline]
    pub fn new_bitfield_1(requested: u8, executed: u8) -> __BindgenBitfieldUnit<[u8; 1usize]> {
        let mut __bindgen_bitfield_unit: __BindgenBitfieldUnit<[u8; 1usize]> = Default::default();
        __bindgen_bitfield_unit.set(0usize, 4u8, {
            let requested: u8 = unsafe { ::core::mem::transmute(requested) };
            requested as u64
        });
        __bindgen_bitfield_unit.set(4usize, 4u8, {
            let executed: u8 = unsafe { ::core::mem::transmute(executed) };
            executed as u64
        });
        __bindgen_bitfield_unit
    }
}
impl CUpti_ActivityKernel6__bindgen_ty_1__bindgen_ty_1 {
    #[inline]
    pub fn requested(&self) -> u8 {
        unsafe { ::core::mem::transmute(self._bitfield_1.get(0usize, 4u8) as u8) }
    }
    #[inline]
    pub fn set_requested(&mut self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            self._bitfield_1.set(0usize, 4u8, val as u64)
        }
    }
    #[inline]
    pub unsafe fn requested_raw(this: *const Self) -> u8 {
        unsafe {
            ::core::mem::transmute(<__BindgenBitfieldUnit<[u8; 1usize]>>::raw_get(
                ::core::ptr::addr_of!((*this)._bitfield_1),
                0usize,
                4u8,
            ) as u8)
        }
    }
    #[inline]
    pub unsafe fn set_requested_raw(this: *mut Self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            <__BindgenBitfieldUnit<[u8; 1usize]>>::raw_set(
                ::core::ptr::addr_of_mut!((*this)._bitfield_1),
                0usize,
                4u8,
                val as u64,
            )
        }
    }
    #[inline]
    pub fn executed(&self) -> u8 {
        unsafe { ::core::mem::transmute(self._bitfield_1.get(4usize, 4u8) as u8) }
    }
    #[inline]
    pub fn set_executed(&mut self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            self._bitfield_1.set(4usize, 4u8, val as u64)
        }
    }
    #[inline]
    pub unsafe fn executed_raw(this: *const Self) -> u8 {
        unsafe {
            ::core::mem::transmute(<__BindgenBitfieldUnit<[u8; 1usize]>>::raw_get(
                ::core::ptr::addr_of!((*this)._bitfield_1),
                4usize,
                4u8,
            ) as u8)
        }
    }
    #[inline]
    pub unsafe fn set_executed_raw(this: *mut Self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            <__BindgenBitfieldUnit<[u8; 1usize]>>::raw_set(
                ::core::ptr::addr_of_mut!((*this)._bitfield_1),
                4usize,
                4u8,
                val as u64,
            )
        }
    }
    #[inline]
    pub fn new_bitfield_1(requested: u8, executed: u8) -> __BindgenBitfieldUnit<[u8; 1usize]> {
        let mut __bindgen_bitfield_unit: __BindgenBitfieldUnit<[u8; 1usize]> = Default::default();
        __bindgen_bitfield_unit.set(0usize, 4u8, {
            let requested: u8 = unsafe { ::core::mem::transmute(requested) };
            requested as u64
        });
        __bindgen_bitfield_unit.set(4usize, 4u8, {
            let executed: u8 = unsafe { ::core::mem::transmute(executed) };
            executed as u64
        });
        __bindgen_bitfield_unit
    }
}
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
impl CUpti_ActivityKernel7__bindgen_ty_1__bindgen_ty_1 {
    #[inline]
    pub fn requested(&self) -> u8 {
        unsafe { ::core::mem::transmute(self._bitfield_1.get(0usize, 4u8) as u8) }
    }
    #[inline]
    pub fn set_requested(&mut self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            self._bitfield_1.set(0usize, 4u8, val as u64)
        }
    }
    #[inline]
    pub unsafe fn requested_raw(this: *const Self) -> u8 {
        unsafe {
            ::core::mem::transmute(<__BindgenBitfieldUnit<[u8; 1usize]>>::raw_get(
                ::core::ptr::addr_of!((*this)._bitfield_1),
                0usize,
                4u8,
            ) as u8)
        }
    }
    #[inline]
    pub unsafe fn set_requested_raw(this: *mut Self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            <__BindgenBitfieldUnit<[u8; 1usize]>>::raw_set(
                ::core::ptr::addr_of_mut!((*this)._bitfield_1),
                0usize,
                4u8,
                val as u64,
            )
        }
    }
    #[inline]
    pub fn executed(&self) -> u8 {
        unsafe { ::core::mem::transmute(self._bitfield_1.get(4usize, 4u8) as u8) }
    }
    #[inline]
    pub fn set_executed(&mut self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            self._bitfield_1.set(4usize, 4u8, val as u64)
        }
    }
    #[inline]
    pub unsafe fn executed_raw(this: *const Self) -> u8 {
        unsafe {
            ::core::mem::transmute(<__BindgenBitfieldUnit<[u8; 1usize]>>::raw_get(
                ::core::ptr::addr_of!((*this)._bitfield_1),
                4usize,
                4u8,
            ) as u8)
        }
    }
    #[inline]
    pub unsafe fn set_executed_raw(this: *mut Self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            <__BindgenBitfieldUnit<[u8; 1usize]>>::raw_set(
                ::core::ptr::addr_of_mut!((*this)._bitfield_1),
                4usize,
                4u8,
                val as u64,
            )
        }
    }
    #[inline]
    pub fn new_bitfield_1(requested: u8, executed: u8) -> __BindgenBitfieldUnit<[u8; 1usize]> {
        let mut __bindgen_bitfield_unit: __BindgenBitfieldUnit<[u8; 1usize]> = Default::default();
        __bindgen_bitfield_unit.set(0usize, 4u8, {
            let requested: u8 = unsafe { ::core::mem::transmute(requested) };
            requested as u64
        });
        __bindgen_bitfield_unit.set(4usize, 4u8, {
            let executed: u8 = unsafe { ::core::mem::transmute(executed) };
            executed as u64
        });
        __bindgen_bitfield_unit
    }
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
impl CUpti_ActivityKernel8__bindgen_ty_1__bindgen_ty_1 {
    #[inline]
    pub fn requested(&self) -> u8 {
        unsafe { ::core::mem::transmute(self._bitfield_1.get(0usize, 4u8) as u8) }
    }
    #[inline]
    pub fn set_requested(&mut self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            self._bitfield_1.set(0usize, 4u8, val as u64)
        }
    }
    #[inline]
    pub unsafe fn requested_raw(this: *const Self) -> u8 {
        unsafe {
            ::core::mem::transmute(<__BindgenBitfieldUnit<[u8; 1usize]>>::raw_get(
                ::core::ptr::addr_of!((*this)._bitfield_1),
                0usize,
                4u8,
            ) as u8)
        }
    }
    #[inline]
    pub unsafe fn set_requested_raw(this: *mut Self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            <__BindgenBitfieldUnit<[u8; 1usize]>>::raw_set(
                ::core::ptr::addr_of_mut!((*this)._bitfield_1),
                0usize,
                4u8,
                val as u64,
            )
        }
    }
    #[inline]
    pub fn executed(&self) -> u8 {
        unsafe { ::core::mem::transmute(self._bitfield_1.get(4usize, 4u8) as u8) }
    }
    #[inline]
    pub fn set_executed(&mut self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            self._bitfield_1.set(4usize, 4u8, val as u64)
        }
    }
    #[inline]
    pub unsafe fn executed_raw(this: *const Self) -> u8 {
        unsafe {
            ::core::mem::transmute(<__BindgenBitfieldUnit<[u8; 1usize]>>::raw_get(
                ::core::ptr::addr_of!((*this)._bitfield_1),
                4usize,
                4u8,
            ) as u8)
        }
    }
    #[inline]
    pub unsafe fn set_executed_raw(this: *mut Self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            <__BindgenBitfieldUnit<[u8; 1usize]>>::raw_set(
                ::core::ptr::addr_of_mut!((*this)._bitfield_1),
                4usize,
                4u8,
                val as u64,
            )
        }
    }
    #[inline]
    pub fn new_bitfield_1(requested: u8, executed: u8) -> __BindgenBitfieldUnit<[u8; 1usize]> {
        let mut __bindgen_bitfield_unit: __BindgenBitfieldUnit<[u8; 1usize]> = Default::default();
        __bindgen_bitfield_unit.set(0usize, 4u8, {
            let requested: u8 = unsafe { ::core::mem::transmute(requested) };
            requested as u64
        });
        __bindgen_bitfield_unit.set(4usize, 4u8, {
            let executed: u8 = unsafe { ::core::mem::transmute(executed) };
            executed as u64
        });
        __bindgen_bitfield_unit
    }
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
impl CUpti_ActivityKernel9__bindgen_ty_1__bindgen_ty_1 {
    #[inline]
    pub fn requested(&self) -> u8 {
        unsafe { ::core::mem::transmute(self._bitfield_1.get(0usize, 4u8) as u8) }
    }
    #[inline]
    pub fn set_requested(&mut self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            self._bitfield_1.set(0usize, 4u8, val as u64)
        }
    }
    #[inline]
    pub unsafe fn requested_raw(this: *const Self) -> u8 {
        unsafe {
            ::core::mem::transmute(<__BindgenBitfieldUnit<[u8; 1usize]>>::raw_get(
                ::core::ptr::addr_of!((*this)._bitfield_1),
                0usize,
                4u8,
            ) as u8)
        }
    }
    #[inline]
    pub unsafe fn set_requested_raw(this: *mut Self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            <__BindgenBitfieldUnit<[u8; 1usize]>>::raw_set(
                ::core::ptr::addr_of_mut!((*this)._bitfield_1),
                0usize,
                4u8,
                val as u64,
            )
        }
    }
    #[inline]
    pub fn executed(&self) -> u8 {
        unsafe { ::core::mem::transmute(self._bitfield_1.get(4usize, 4u8) as u8) }
    }
    #[inline]
    pub fn set_executed(&mut self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            self._bitfield_1.set(4usize, 4u8, val as u64)
        }
    }
    #[inline]
    pub unsafe fn executed_raw(this: *const Self) -> u8 {
        unsafe {
            ::core::mem::transmute(<__BindgenBitfieldUnit<[u8; 1usize]>>::raw_get(
                ::core::ptr::addr_of!((*this)._bitfield_1),
                4usize,
                4u8,
            ) as u8)
        }
    }
    #[inline]
    pub unsafe fn set_executed_raw(this: *mut Self, val: u8) {
        unsafe {
            let val: u8 = ::core::mem::transmute(val);
            <__BindgenBitfieldUnit<[u8; 1usize]>>::raw_set(
                ::core::ptr::addr_of_mut!((*this)._bitfield_1),
                4usize,
                4u8,
                val as u64,
            )
        }
    }
    #[inline]
    pub fn new_bitfield_1(requested: u8, executed: u8) -> __BindgenBitfieldUnit<[u8; 1usize]> {
        let mut __bindgen_bitfield_unit: __BindgenBitfieldUnit<[u8; 1usize]> = Default::default();
        __bindgen_bitfield_unit.set(0usize, 4u8, {
            let requested: u8 = unsafe { ::core::mem::transmute(requested) };
            requested as u64
        });
        __bindgen_bitfield_unit.set(4usize, 4u8, {
            let executed: u8 = unsafe { ::core::mem::transmute(executed) };
            executed as u64
        });
        __bindgen_bitfield_unit
    }
}
impl<Storage> __BindgenBitfieldUnit<Storage> {
    #[inline]
    pub const fn new(storage: Storage) -> Self {
        Self { storage }
    }
}
impl<Storage> __BindgenBitfieldUnit<Storage>
where
    Storage: AsRef<[u8]> + AsMut<[u8]>,
{
    #[inline]
    fn extract_bit(byte: u8, index: usize) -> bool {
        let bit_index = if cfg!(target_endian = "big") {
            7 - (index % 8)
        } else {
            index % 8
        };
        let mask = 1 << bit_index;
        byte & mask == mask
    }
    #[inline]
    pub fn get_bit(&self, index: usize) -> bool {
        debug_assert!(index / 8 < self.storage.as_ref().len());
        let byte_index = index / 8;
        let byte = self.storage.as_ref()[byte_index];
        Self::extract_bit(byte, index)
    }
    #[inline]
    pub unsafe fn raw_get_bit(this: *const Self, index: usize) -> bool {
        debug_assert!(index / 8 < core::mem::size_of::<Storage>());
        let byte_index = index / 8;
        let byte = *(core::ptr::addr_of!((*this).storage) as *const u8).offset(byte_index as isize);
        Self::extract_bit(byte, index)
    }
    #[inline]
    fn change_bit(byte: u8, index: usize, val: bool) -> u8 {
        let bit_index = if cfg!(target_endian = "big") {
            7 - (index % 8)
        } else {
            index % 8
        };
        let mask = 1 << bit_index;
        if val {
            byte | mask
        } else {
            byte & !mask
        }
    }
    #[inline]
    pub fn set_bit(&mut self, index: usize, val: bool) {
        debug_assert!(index / 8 < self.storage.as_ref().len());
        let byte_index = index / 8;
        let byte = &mut self.storage.as_mut()[byte_index];
        *byte = Self::change_bit(*byte, index, val);
    }
    #[inline]
    pub unsafe fn raw_set_bit(this: *mut Self, index: usize, val: bool) {
        debug_assert!(index / 8 < core::mem::size_of::<Storage>());
        let byte_index = index / 8;
        let byte =
            (core::ptr::addr_of_mut!((*this).storage) as *mut u8).offset(byte_index as isize);
        *byte = Self::change_bit(*byte, index, val);
    }
    #[inline]
    pub fn get(&self, bit_offset: usize, bit_width: u8) -> u64 {
        debug_assert!(bit_width <= 64);
        debug_assert!(bit_offset / 8 < self.storage.as_ref().len());
        debug_assert!((bit_offset + (bit_width as usize)) / 8 <= self.storage.as_ref().len(),);
        let mut val = 0;
        for i in 0..(bit_width as usize) {
            if self.get_bit(i + bit_offset) {
                let index = if cfg!(target_endian = "big") {
                    bit_width as usize - 1 - i
                } else {
                    i
                };
                val |= 1 << index;
            }
        }
        val
    }
    #[inline]
    pub unsafe fn raw_get(this: *const Self, bit_offset: usize, bit_width: u8) -> u64 {
        debug_assert!(bit_width <= 64);
        debug_assert!(bit_offset / 8 < core::mem::size_of::<Storage>());
        debug_assert!((bit_offset + (bit_width as usize)) / 8 <= core::mem::size_of::<Storage>(),);
        let mut val = 0;
        for i in 0..(bit_width as usize) {
            if Self::raw_get_bit(this, i + bit_offset) {
                let index = if cfg!(target_endian = "big") {
                    bit_width as usize - 1 - i
                } else {
                    i
                };
                val |= 1 << index;
            }
        }
        val
    }
    #[inline]
    pub fn set(&mut self, bit_offset: usize, bit_width: u8, val: u64) {
        debug_assert!(bit_width <= 64);
        debug_assert!(bit_offset / 8 < self.storage.as_ref().len());
        debug_assert!((bit_offset + (bit_width as usize)) / 8 <= self.storage.as_ref().len(),);
        for i in 0..(bit_width as usize) {
            let mask = 1 << i;
            let val_bit_is_set = val & mask == mask;
            let index = if cfg!(target_endian = "big") {
                bit_width as usize - 1 - i
            } else {
                i
            };
            self.set_bit(index + bit_offset, val_bit_is_set);
        }
    }
    #[inline]
    pub unsafe fn raw_set(this: *mut Self, bit_offset: usize, bit_width: u8, val: u64) {
        debug_assert!(bit_width <= 64);
        debug_assert!(bit_offset / 8 < core::mem::size_of::<Storage>());
        debug_assert!((bit_offset + (bit_width as usize)) / 8 <= core::mem::size_of::<Storage>(),);
        for i in 0..(bit_width as usize) {
            let mask = 1 << i;
            let val_bit_is_set = val & mask == mask;
            let index = if cfg!(target_endian = "big") {
                bit_width as usize - 1 - i
            } else {
                i
            };
            Self::raw_set_bit(this, index + bit_offset, val_bit_is_set);
        }
    }
}
impl<T> __BindgenUnionField<T> {
    #[inline]
    pub const fn new() -> Self {
        __BindgenUnionField(::core::marker::PhantomData)
    }
    #[inline]
    pub unsafe fn as_ref(&self) -> &T {
        ::core::mem::transmute(self)
    }
    #[inline]
    pub unsafe fn as_mut(&mut self) -> &mut T {
        ::core::mem::transmute(self)
    }
}
impl<T> ::core::clone::Clone for __BindgenUnionField<T> {
    #[inline]
    fn clone(&self) -> Self {
        *self
    }
}
impl<T> ::core::cmp::Eq for __BindgenUnionField<T> {}
impl<T> ::core::cmp::PartialEq for __BindgenUnionField<T> {
    fn eq(&self, _other: &__BindgenUnionField<T>) -> bool {
        true
    }
}
impl<T> ::core::default::Default for __BindgenUnionField<T> {
    #[inline]
    fn default() -> Self {
        Self::new()
    }
}
impl<T> ::core::fmt::Debug for __BindgenUnionField<T> {
    fn fmt(&self, fmt: &mut ::core::fmt::Formatter<'_>) -> ::core::fmt::Result {
        fmt.write_str("__BindgenUnionField")
    }
}
impl<T> ::core::hash::Hash for __BindgenUnionField<T> {
    fn hash<H: ::core::hash::Hasher>(&self, _state: &mut H) {}
}
impl<T> ::core::marker::Copy for __BindgenUnionField<T> {}
#[repr(C)]
#[derive(Copy, Clone)]
pub union CUpti_ActivityCdpKernel__bindgen_ty_1 {
    pub both: u8,
    pub config: CUpti_ActivityCdpKernel__bindgen_ty_1__bindgen_ty_1,
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union CUpti_ActivityDeviceAttribute__bindgen_ty_2 {
    pub vDouble: f64,
    pub vUint32: u32,
    pub vUint64: u64,
    pub vInt32: i32,
    pub vInt64: i64,
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union CUpti_ActivityEnvironment__bindgen_ty_1 {
    pub speed: CUpti_ActivityEnvironment__bindgen_ty_1__bindgen_ty_1,
    pub temperature: CUpti_ActivityEnvironment__bindgen_ty_1__bindgen_ty_2,
    pub power: CUpti_ActivityEnvironment__bindgen_ty_1__bindgen_ty_3,
    pub cooling: CUpti_ActivityEnvironment__bindgen_ty_1__bindgen_ty_4,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
#[derive(Copy, Clone)]
pub union CUpti_ActivityKernel10__bindgen_ty_1 {
    pub both: u8,
    pub config: CUpti_ActivityKernel10__bindgen_ty_1__bindgen_ty_1,
}
#[cfg(any(feature = "cuda-13010"))]
#[repr(C)]
#[derive(Copy, Clone)]
pub union CUpti_ActivityKernel11__bindgen_ty_1 {
    pub both: u8,
    pub config: CUpti_ActivityKernel11__bindgen_ty_1__bindgen_ty_1,
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union CUpti_ActivityKernel2__bindgen_ty_1 {
    pub both: u8,
    pub config: CUpti_ActivityKernel2__bindgen_ty_1__bindgen_ty_1,
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union CUpti_ActivityKernel3__bindgen_ty_1 {
    pub both: u8,
    pub config: CUpti_ActivityKernel3__bindgen_ty_1__bindgen_ty_1,
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union CUpti_ActivityKernel4__bindgen_ty_1 {
    pub both: u8,
    pub config: CUpti_ActivityKernel4__bindgen_ty_1__bindgen_ty_1,
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union CUpti_ActivityKernel5__bindgen_ty_1 {
    pub both: u8,
    pub config: CUpti_ActivityKernel5__bindgen_ty_1__bindgen_ty_1,
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union CUpti_ActivityKernel6__bindgen_ty_1 {
    pub both: u8,
    pub config: CUpti_ActivityKernel6__bindgen_ty_1__bindgen_ty_1,
}
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Copy, Clone)]
pub union CUpti_ActivityKernel7__bindgen_ty_1 {
    pub both: u8,
    pub config: CUpti_ActivityKernel7__bindgen_ty_1__bindgen_ty_1,
}
#[cfg(any(
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Copy, Clone)]
pub union CUpti_ActivityKernel8__bindgen_ty_1 {
    pub both: u8,
    pub config: CUpti_ActivityKernel8__bindgen_ty_1__bindgen_ty_1,
}
#[cfg(any(
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Copy, Clone)]
pub union CUpti_ActivityKernel9__bindgen_ty_1 {
    pub both: u8,
    pub config: CUpti_ActivityKernel9__bindgen_ty_1__bindgen_ty_1,
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union CUpti_ActivityMemory2__bindgen_ty_1__bindgen_ty_1 {
    pub size: u64,
    pub processId: u64,
}
#[cfg(any(
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Copy, Clone)]
pub union CUpti_ActivityMemory3__bindgen_ty_1__bindgen_ty_1 {
    pub size: u64,
    pub processId: u64,
}
#[cfg(any(
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Copy, Clone)]
pub union CUpti_ActivityMemory4__bindgen_ty_1__bindgen_ty_1 {
    pub size: u64,
    pub processId: u64,
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union CUpti_ActivityObjectKindId {
    pub pt: CUpti_ActivityObjectKindId__bindgen_ty_1,
    pub dcs: CUpti_ActivityObjectKindId__bindgen_ty_2,
}
#[cfg(any(
    feature = "cuda-11040",
    feature = "cuda-11050",
    feature = "cuda-11060",
    feature = "cuda-11070",
    feature = "cuda-11080",
    feature = "cuda-12000",
    feature = "cuda-12010",
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090"
))]
#[repr(C)]
#[derive(Copy, Clone)]
pub union CUpti_MetricValue {
    pub metricValueDouble: f64,
    pub metricValueUint64: u64,
    pub metricValueInt64: i64,
    pub metricValuePercent: f64,
    pub metricValueThroughput: u64,
    pub metricValueUtilizationLevel: CUpti_MetricValueUtilizationLevel,
}
#[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
#[repr(C)]
#[derive(Copy, Clone)]
pub union CUpti_MetricValue {
    pub metricValueDouble: f64,
    pub metricValueUint64: u64,
    pub metricValueInt64: i64,
    pub metricValuePercent: f64,
    pub metricValueThroughput: u64,
    pub metricValueUtilizationLevel: CUpti_MetricValueUtilizationLevel,
    pub metricValueNvtxExtendedPayload: u64,
}
#[cfg(any(
    feature = "cuda-12020",
    feature = "cuda-12030",
    feature = "cuda-12040",
    feature = "cuda-12050",
    feature = "cuda-12060",
    feature = "cuda-12080",
    feature = "cuda-12090",
    feature = "cuda-13000",
    feature = "cuda-13010"
))]
#[repr(C)]
#[derive(Copy, Clone)]
pub union CUpti_StateData__bindgen_ty_1 {
    pub notification: CUpti_StateData__bindgen_ty_1__bindgen_ty_1,
}
#[cfg(not(feature = "dynamic-loading"))]
extern "C" {
    pub fn cuptiActivityConfigurePCSampling(
        ctx: CUcontext,
        config: *mut CUpti_ActivityPCSamplingConfig,
    ) -> CUptiResult;
    pub fn cuptiActivityConfigureUnifiedMemoryCounter(
        config: *mut CUpti_ActivityUnifiedMemoryCounterConfig,
        count: u32,
    ) -> CUptiResult;
    pub fn cuptiActivityDisable(kind: CUpti_ActivityKind) -> CUptiResult;
    pub fn cuptiActivityDisableContext(context: CUcontext, kind: CUpti_ActivityKind)
        -> CUptiResult;
    pub fn cuptiActivityEnable(kind: CUpti_ActivityKind) -> CUptiResult;
    #[cfg(any(
        feature = "cuda-12080",
        feature = "cuda-12090",
        feature = "cuda-13000",
        feature = "cuda-13010"
    ))]
    pub fn cuptiActivityEnableAllSyncRecords(enable: u8) -> CUptiResult;
    #[cfg(any(
        feature = "cuda-12060",
        feature = "cuda-12080",
        feature = "cuda-12090",
        feature = "cuda-13000",
        feature = "cuda-13010"
    ))]
    pub fn cuptiActivityEnableAllocationSource(enable: u8) -> CUptiResult;
    #[cfg(any(
        feature = "cuda-11070",
        feature = "cuda-11080",
        feature = "cuda-12000",
        feature = "cuda-12010",
        feature = "cuda-12020",
        feature = "cuda-12030",
        feature = "cuda-12040",
        feature = "cuda-12050",
        feature = "cuda-12060",
        feature = "cuda-12080",
        feature = "cuda-12090",
        feature = "cuda-13000",
        feature = "cuda-13010"
    ))]
    pub fn cuptiActivityEnableAndDump(kind: CUpti_ActivityKind) -> CUptiResult;
    pub fn cuptiActivityEnableContext(context: CUcontext, kind: CUpti_ActivityKind) -> CUptiResult;
    #[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
    pub fn cuptiActivityEnableCudaEventDeviceTimestamps(enable: u8) -> CUptiResult;
    #[cfg(any(
        feature = "cuda-12030",
        feature = "cuda-12040",
        feature = "cuda-12050",
        feature = "cuda-12060",
        feature = "cuda-12080",
        feature = "cuda-12090",
        feature = "cuda-13000",
        feature = "cuda-13010"
    ))]
    pub fn cuptiActivityEnableDeviceGraph(enable: u8) -> CUptiResult;
    #[cfg(any(
        feature = "cuda-12050",
        feature = "cuda-12060",
        feature = "cuda-12080",
        feature = "cuda-12090",
        feature = "cuda-13000",
        feature = "cuda-13010"
    ))]
    pub fn cuptiActivityEnableDriverApi(cbid: CUpti_CallbackId, enable: u8) -> CUptiResult;
    #[cfg(any(
        feature = "cuda-12080",
        feature = "cuda-12090",
        feature = "cuda-13000",
        feature = "cuda-13010"
    ))]
    pub fn cuptiActivityEnableHWTrace(enable: u8) -> CUptiResult;
    pub fn cuptiActivityEnableLatencyTimestamps(enable: u8) -> CUptiResult;
    pub fn cuptiActivityEnableLaunchAttributes(enable: u8) -> CUptiResult;
    #[cfg(any(
        feature = "cuda-12050",
        feature = "cuda-12060",
        feature = "cuda-12080",
        feature = "cuda-12090",
        feature = "cuda-13000",
        feature = "cuda-13010"
    ))]
    pub fn cuptiActivityEnableRuntimeApi(cbid: CUpti_CallbackId, enable: u8) -> CUptiResult;
    pub fn cuptiActivityFlush(context: CUcontext, streamId: u32, flag: u32) -> CUptiResult;
    pub fn cuptiActivityFlushAll(flag: u32) -> CUptiResult;
    pub fn cuptiActivityFlushPeriod(time: u32) -> CUptiResult;
    pub fn cuptiActivityGetAttribute(
        attr: CUpti_ActivityAttribute,
        valueSize: *mut usize,
        value: *mut ::core::ffi::c_void,
    ) -> CUptiResult;
    pub fn cuptiActivityGetNextRecord(
        buffer: *mut u8,
        validBufferSizeBytes: usize,
        record: *mut *mut CUpti_Activity,
    ) -> CUptiResult;
    pub fn cuptiActivityGetNumDroppedRecords(
        context: CUcontext,
        streamId: u32,
        dropped: *mut usize,
    ) -> CUptiResult;
    pub fn cuptiActivityPopExternalCorrelationId(
        kind: CUpti_ExternalCorrelationKind,
        lastId: *mut u64,
    ) -> CUptiResult;
    pub fn cuptiActivityPushExternalCorrelationId(
        kind: CUpti_ExternalCorrelationKind,
        id: u64,
    ) -> CUptiResult;
    pub fn cuptiActivityRegisterCallbacks(
        funcBufferRequested: CUpti_BuffersCallbackRequestFunc,
        funcBufferCompleted: CUpti_BuffersCallbackCompleteFunc,
    ) -> CUptiResult;
    #[cfg(any(
        feature = "cuda-11060",
        feature = "cuda-11070",
        feature = "cuda-11080",
        feature = "cuda-12000",
        feature = "cuda-12010",
        feature = "cuda-12020",
        feature = "cuda-12030",
        feature = "cuda-12040",
        feature = "cuda-12050",
        feature = "cuda-12060",
        feature = "cuda-12080",
        feature = "cuda-12090",
        feature = "cuda-13000",
        feature = "cuda-13010"
    ))]
    pub fn cuptiActivityRegisterTimestampCallback(
        funcTimestamp: CUpti_TimestampCallbackFunc,
    ) -> CUptiResult;
    pub fn cuptiActivitySetAttribute(
        attr: CUpti_ActivityAttribute,
        valueSize: *mut usize,
        value: *mut ::core::ffi::c_void,
    ) -> CUptiResult;
    pub fn cuptiComputeCapabilitySupported(
        major: ::core::ffi::c_int,
        minor: ::core::ffi::c_int,
        support: *mut ::core::ffi::c_int,
    ) -> CUptiResult;
    pub fn cuptiDeviceEnumEventDomains(
        device: CUdevice,
        arraySizeBytes: *mut usize,
        domainArray: *mut CUpti_EventDomainID,
    ) -> CUptiResult;
    pub fn cuptiDeviceEnumMetrics(
        device: CUdevice,
        arraySizeBytes: *mut usize,
        metricArray: *mut CUpti_MetricID,
    ) -> CUptiResult;
    pub fn cuptiDeviceGetAttribute(
        device: CUdevice,
        attrib: CUpti_DeviceAttribute,
        valueSize: *mut usize,
        value: *mut ::core::ffi::c_void,
    ) -> CUptiResult;
    pub fn cuptiDeviceGetEventDomainAttribute(
        device: CUdevice,
        eventDomain: CUpti_EventDomainID,
        attrib: CUpti_EventDomainAttribute,
        valueSize: *mut usize,
        value: *mut ::core::ffi::c_void,
    ) -> CUptiResult;
    pub fn cuptiDeviceGetNumEventDomains(device: CUdevice, numDomains: *mut u32) -> CUptiResult;
    pub fn cuptiDeviceGetNumMetrics(device: CUdevice, numMetrics: *mut u32) -> CUptiResult;
    #[cfg(any(
        feature = "cuda-11040",
        feature = "cuda-11050",
        feature = "cuda-11060",
        feature = "cuda-11070",
        feature = "cuda-11080",
        feature = "cuda-12000",
        feature = "cuda-12010"
    ))]
    pub fn cuptiDeviceGetTimestamp(context: CUcontext, timestamp: *mut u64) -> CUptiResult;
    pub fn cuptiDeviceSupported(dev: CUdevice, support: *mut ::core::ffi::c_int) -> CUptiResult;
    pub fn cuptiDeviceVirtualizationMode(
        dev: CUdevice,
        mode: *mut CUpti_DeviceVirtualizationMode,
    ) -> CUptiResult;
    pub fn cuptiDisableKernelReplayMode(context: CUcontext) -> CUptiResult;
    pub fn cuptiEnableAllDomains(enable: u32, subscriber: CUpti_SubscriberHandle) -> CUptiResult;
    pub fn cuptiEnableCallback(
        enable: u32,
        subscriber: CUpti_SubscriberHandle,
        domain: CUpti_CallbackDomain,
        cbid: CUpti_CallbackId,
    ) -> CUptiResult;
    pub fn cuptiEnableDomain(
        enable: u32,
        subscriber: CUpti_SubscriberHandle,
        domain: CUpti_CallbackDomain,
    ) -> CUptiResult;
    pub fn cuptiEnableKernelReplayMode(context: CUcontext) -> CUptiResult;
    pub fn cuptiEnumEventDomains(
        arraySizeBytes: *mut usize,
        domainArray: *mut CUpti_EventDomainID,
    ) -> CUptiResult;
    pub fn cuptiEnumMetrics(
        arraySizeBytes: *mut usize,
        metricArray: *mut CUpti_MetricID,
    ) -> CUptiResult;
    pub fn cuptiEventDomainEnumEvents(
        eventDomain: CUpti_EventDomainID,
        arraySizeBytes: *mut usize,
        eventArray: *mut CUpti_EventID,
    ) -> CUptiResult;
    pub fn cuptiEventDomainGetAttribute(
        eventDomain: CUpti_EventDomainID,
        attrib: CUpti_EventDomainAttribute,
        valueSize: *mut usize,
        value: *mut ::core::ffi::c_void,
    ) -> CUptiResult;
    pub fn cuptiEventDomainGetNumEvents(
        eventDomain: CUpti_EventDomainID,
        numEvents: *mut u32,
    ) -> CUptiResult;
    pub fn cuptiEventGetAttribute(
        event: CUpti_EventID,
        attrib: CUpti_EventAttribute,
        valueSize: *mut usize,
        value: *mut ::core::ffi::c_void,
    ) -> CUptiResult;
    pub fn cuptiEventGetIdFromName(
        device: CUdevice,
        eventName: *const ::core::ffi::c_char,
        event: *mut CUpti_EventID,
    ) -> CUptiResult;
    pub fn cuptiEventGroupAddEvent(
        eventGroup: CUpti_EventGroup,
        event: CUpti_EventID,
    ) -> CUptiResult;
    pub fn cuptiEventGroupCreate(
        context: CUcontext,
        eventGroup: *mut CUpti_EventGroup,
        flags: u32,
    ) -> CUptiResult;
    pub fn cuptiEventGroupDestroy(eventGroup: CUpti_EventGroup) -> CUptiResult;
    pub fn cuptiEventGroupDisable(eventGroup: CUpti_EventGroup) -> CUptiResult;
    pub fn cuptiEventGroupEnable(eventGroup: CUpti_EventGroup) -> CUptiResult;
    pub fn cuptiEventGroupGetAttribute(
        eventGroup: CUpti_EventGroup,
        attrib: CUpti_EventGroupAttribute,
        valueSize: *mut usize,
        value: *mut ::core::ffi::c_void,
    ) -> CUptiResult;
    pub fn cuptiEventGroupReadAllEvents(
        eventGroup: CUpti_EventGroup,
        flags: CUpti_ReadEventFlags,
        eventValueBufferSizeBytes: *mut usize,
        eventValueBuffer: *mut u64,
        eventIdArraySizeBytes: *mut usize,
        eventIdArray: *mut CUpti_EventID,
        numEventIdsRead: *mut usize,
    ) -> CUptiResult;
    pub fn cuptiEventGroupReadEvent(
        eventGroup: CUpti_EventGroup,
        flags: CUpti_ReadEventFlags,
        event: CUpti_EventID,
        eventValueBufferSizeBytes: *mut usize,
        eventValueBuffer: *mut u64,
    ) -> CUptiResult;
    pub fn cuptiEventGroupRemoveAllEvents(eventGroup: CUpti_EventGroup) -> CUptiResult;
    pub fn cuptiEventGroupRemoveEvent(
        eventGroup: CUpti_EventGroup,
        event: CUpti_EventID,
    ) -> CUptiResult;
    pub fn cuptiEventGroupResetAllEvents(eventGroup: CUpti_EventGroup) -> CUptiResult;
    pub fn cuptiEventGroupSetAttribute(
        eventGroup: CUpti_EventGroup,
        attrib: CUpti_EventGroupAttribute,
        valueSize: usize,
        value: *mut ::core::ffi::c_void,
    ) -> CUptiResult;
    pub fn cuptiEventGroupSetDisable(eventGroupSet: *mut CUpti_EventGroupSet) -> CUptiResult;
    pub fn cuptiEventGroupSetEnable(eventGroupSet: *mut CUpti_EventGroupSet) -> CUptiResult;
    pub fn cuptiEventGroupSetsCreate(
        context: CUcontext,
        eventIdArraySizeBytes: usize,
        eventIdArray: *mut CUpti_EventID,
        eventGroupPasses: *mut *mut CUpti_EventGroupSets,
    ) -> CUptiResult;
    pub fn cuptiEventGroupSetsDestroy(eventGroupSets: *mut CUpti_EventGroupSets) -> CUptiResult;
    pub fn cuptiFinalize() -> CUptiResult;
    pub fn cuptiGetAutoBoostState(
        context: CUcontext,
        state: *mut CUpti_ActivityAutoBoostState,
    ) -> CUptiResult;
    pub fn cuptiGetCallbackName(
        domain: CUpti_CallbackDomain,
        cbid: u32,
        name: *mut *const ::core::ffi::c_char,
    ) -> CUptiResult;
    pub fn cuptiGetCallbackState(
        enable: *mut u32,
        subscriber: CUpti_SubscriberHandle,
        domain: CUpti_CallbackDomain,
        cbid: CUpti_CallbackId,
    ) -> CUptiResult;
    pub fn cuptiGetContextId(context: CUcontext, contextId: *mut u32) -> CUptiResult;
    pub fn cuptiGetDeviceId(context: CUcontext, deviceId: *mut u32) -> CUptiResult;
    #[cfg(any(
        feature = "cuda-12020",
        feature = "cuda-12030",
        feature = "cuda-12040",
        feature = "cuda-12050",
        feature = "cuda-12060",
        feature = "cuda-12080",
        feature = "cuda-12090",
        feature = "cuda-13000",
        feature = "cuda-13010"
    ))]
    pub fn cuptiGetErrorMessage(
        result: CUptiResult,
        str_: *mut *const ::core::ffi::c_char,
    ) -> CUptiResult;
    #[cfg(any(
        feature = "cuda-12030",
        feature = "cuda-12040",
        feature = "cuda-12050",
        feature = "cuda-12060",
        feature = "cuda-12080",
        feature = "cuda-12090",
        feature = "cuda-13000",
        feature = "cuda-13010"
    ))]
    pub fn cuptiGetGraphExecId(graphExec: CUgraphExec, pId: *mut u32) -> CUptiResult;
    pub fn cuptiGetGraphId(graph: CUgraph, pId: *mut u32) -> CUptiResult;
    pub fn cuptiGetGraphNodeId(node: CUgraphNode, nodeId: *mut u64) -> CUptiResult;
    pub fn cuptiGetLastError() -> CUptiResult;
    pub fn cuptiGetNumEventDomains(numDomains: *mut u32) -> CUptiResult;
    pub fn cuptiGetNumMetrics(numMetrics: *mut u32) -> CUptiResult;
    pub fn cuptiGetResultString(
        result: CUptiResult,
        str_: *mut *const ::core::ffi::c_char,
    ) -> CUptiResult;
    pub fn cuptiGetStreamId(
        context: CUcontext,
        stream: CUstream,
        streamId: *mut u32,
    ) -> CUptiResult;
    pub fn cuptiGetStreamIdEx(
        context: CUcontext,
        stream: CUstream,
        perThreadStream: u8,
        streamId: *mut u32,
    ) -> CUptiResult;
    pub fn cuptiGetThreadIdType(type_: *mut CUpti_ActivityThreadIdType) -> CUptiResult;
    pub fn cuptiGetTimestamp(timestamp: *mut u64) -> CUptiResult;
    pub fn cuptiGetVersion(version: *mut u32) -> CUptiResult;
    pub fn cuptiKernelReplaySubscribeUpdate(
        updateFunc: CUpti_KernelReplayUpdateFunc,
        customData: *mut ::core::ffi::c_void,
    ) -> CUptiResult;
    pub fn cuptiMetricCreateEventGroupSets(
        context: CUcontext,
        metricIdArraySizeBytes: usize,
        metricIdArray: *mut CUpti_MetricID,
        eventGroupPasses: *mut *mut CUpti_EventGroupSets,
    ) -> CUptiResult;
    pub fn cuptiMetricEnumEvents(
        metric: CUpti_MetricID,
        eventIdArraySizeBytes: *mut usize,
        eventIdArray: *mut CUpti_EventID,
    ) -> CUptiResult;
    pub fn cuptiMetricEnumProperties(
        metric: CUpti_MetricID,
        propIdArraySizeBytes: *mut usize,
        propIdArray: *mut CUpti_MetricPropertyID,
    ) -> CUptiResult;
    pub fn cuptiMetricGetAttribute(
        metric: CUpti_MetricID,
        attrib: CUpti_MetricAttribute,
        valueSize: *mut usize,
        value: *mut ::core::ffi::c_void,
    ) -> CUptiResult;
    pub fn cuptiMetricGetIdFromName(
        device: CUdevice,
        metricName: *const ::core::ffi::c_char,
        metric: *mut CUpti_MetricID,
    ) -> CUptiResult;
    pub fn cuptiMetricGetNumEvents(metric: CUpti_MetricID, numEvents: *mut u32) -> CUptiResult;
    pub fn cuptiMetricGetNumProperties(metric: CUpti_MetricID, numProp: *mut u32) -> CUptiResult;
    pub fn cuptiMetricGetRequiredEventGroupSets(
        context: CUcontext,
        metric: CUpti_MetricID,
        eventGroupSets: *mut *mut CUpti_EventGroupSets,
    ) -> CUptiResult;
    pub fn cuptiMetricGetValue(
        device: CUdevice,
        metric: CUpti_MetricID,
        eventIdArraySizeBytes: usize,
        eventIdArray: *mut CUpti_EventID,
        eventValueArraySizeBytes: usize,
        eventValueArray: *mut u64,
        timeDuration: u64,
        metricValue: *mut CUpti_MetricValue,
    ) -> CUptiResult;
    pub fn cuptiMetricGetValue2(
        metric: CUpti_MetricID,
        eventIdArraySizeBytes: usize,
        eventIdArray: *mut CUpti_EventID,
        eventValueArraySizeBytes: usize,
        eventValueArray: *mut u64,
        propIdArraySizeBytes: usize,
        propIdArray: *mut CUpti_MetricPropertyID,
        propValueArraySizeBytes: usize,
        propValueArray: *mut u64,
        metricValue: *mut CUpti_MetricValue,
    ) -> CUptiResult;
    pub fn cuptiSetEventCollectionMode(
        context: CUcontext,
        mode: CUpti_EventCollectionMode,
    ) -> CUptiResult;
    pub fn cuptiSetThreadIdType(type_: CUpti_ActivityThreadIdType) -> CUptiResult;
    pub fn cuptiSubscribe(
        subscriber: *mut CUpti_SubscriberHandle,
        callback: CUpti_CallbackFunc,
        userdata: *mut ::core::ffi::c_void,
    ) -> CUptiResult;
    #[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
    pub fn cuptiSubscribe_v2(
        subscriber: *mut CUpti_SubscriberHandle,
        callback: CUpti_CallbackFunc,
        userdata: *mut ::core::ffi::c_void,
        pParams: *mut CUpti_SubscriberParams,
    ) -> CUptiResult;
    pub fn cuptiSupportedDomains(
        domainCount: *mut usize,
        domainTable: *mut CUpti_DomainTable,
    ) -> CUptiResult;
    pub fn cuptiUnsubscribe(subscriber: CUpti_SubscriberHandle) -> CUptiResult;
}
#[cfg(feature = "dynamic-loading")]
mod loaded {
    use super::*;
    pub unsafe fn cuptiActivityConfigurePCSampling(
        ctx: CUcontext,
        config: *mut CUpti_ActivityPCSamplingConfig,
    ) -> CUptiResult {
        (culib().cuptiActivityConfigurePCSampling)(ctx, config)
    }
    pub unsafe fn cuptiActivityConfigureUnifiedMemoryCounter(
        config: *mut CUpti_ActivityUnifiedMemoryCounterConfig,
        count: u32,
    ) -> CUptiResult {
        (culib().cuptiActivityConfigureUnifiedMemoryCounter)(config, count)
    }
    pub unsafe fn cuptiActivityDisable(kind: CUpti_ActivityKind) -> CUptiResult {
        (culib().cuptiActivityDisable)(kind)
    }
    pub unsafe fn cuptiActivityDisableContext(
        context: CUcontext,
        kind: CUpti_ActivityKind,
    ) -> CUptiResult {
        (culib().cuptiActivityDisableContext)(context, kind)
    }
    pub unsafe fn cuptiActivityEnable(kind: CUpti_ActivityKind) -> CUptiResult {
        (culib().cuptiActivityEnable)(kind)
    }
    #[cfg(any(
        feature = "cuda-12080",
        feature = "cuda-12090",
        feature = "cuda-13000",
        feature = "cuda-13010"
    ))]
    pub unsafe fn cuptiActivityEnableAllSyncRecords(enable: u8) -> CUptiResult {
        (culib().cuptiActivityEnableAllSyncRecords)(enable)
    }
    #[cfg(any(
        feature = "cuda-12060",
        feature = "cuda-12080",
        feature = "cuda-12090",
        feature = "cuda-13000",
        feature = "cuda-13010"
    ))]
    pub unsafe fn cuptiActivityEnableAllocationSource(enable: u8) -> CUptiResult {
        (culib().cuptiActivityEnableAllocationSource)(enable)
    }
    #[cfg(any(
        feature = "cuda-11070",
        feature = "cuda-11080",
        feature = "cuda-12000",
        feature = "cuda-12010",
        feature = "cuda-12020",
        feature = "cuda-12030",
        feature = "cuda-12040",
        feature = "cuda-12050",
        feature = "cuda-12060",
        feature = "cuda-12080",
        feature = "cuda-12090",
        feature = "cuda-13000",
        feature = "cuda-13010"
    ))]
    pub unsafe fn cuptiActivityEnableAndDump(kind: CUpti_ActivityKind) -> CUptiResult {
        (culib().cuptiActivityEnableAndDump)(kind)
    }
    pub unsafe fn cuptiActivityEnableContext(
        context: CUcontext,
        kind: CUpti_ActivityKind,
    ) -> CUptiResult {
        (culib().cuptiActivityEnableContext)(context, kind)
    }
    #[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
    pub unsafe fn cuptiActivityEnableCudaEventDeviceTimestamps(enable: u8) -> CUptiResult {
        (culib().cuptiActivityEnableCudaEventDeviceTimestamps)(enable)
    }
    #[cfg(any(
        feature = "cuda-12030",
        feature = "cuda-12040",
        feature = "cuda-12050",
        feature = "cuda-12060",
        feature = "cuda-12080",
        feature = "cuda-12090",
        feature = "cuda-13000",
        feature = "cuda-13010"
    ))]
    pub unsafe fn cuptiActivityEnableDeviceGraph(enable: u8) -> CUptiResult {
        (culib().cuptiActivityEnableDeviceGraph)(enable)
    }
    #[cfg(any(
        feature = "cuda-12050",
        feature = "cuda-12060",
        feature = "cuda-12080",
        feature = "cuda-12090",
        feature = "cuda-13000",
        feature = "cuda-13010"
    ))]
    pub unsafe fn cuptiActivityEnableDriverApi(cbid: CUpti_CallbackId, enable: u8) -> CUptiResult {
        (culib().cuptiActivityEnableDriverApi)(cbid, enable)
    }
    #[cfg(any(
        feature = "cuda-12080",
        feature = "cuda-12090",
        feature = "cuda-13000",
        feature = "cuda-13010"
    ))]
    pub unsafe fn cuptiActivityEnableHWTrace(enable: u8) -> CUptiResult {
        (culib().cuptiActivityEnableHWTrace)(enable)
    }
    pub unsafe fn cuptiActivityEnableLatencyTimestamps(enable: u8) -> CUptiResult {
        (culib().cuptiActivityEnableLatencyTimestamps)(enable)
    }
    pub unsafe fn cuptiActivityEnableLaunchAttributes(enable: u8) -> CUptiResult {
        (culib().cuptiActivityEnableLaunchAttributes)(enable)
    }
    #[cfg(any(
        feature = "cuda-12050",
        feature = "cuda-12060",
        feature = "cuda-12080",
        feature = "cuda-12090",
        feature = "cuda-13000",
        feature = "cuda-13010"
    ))]
    pub unsafe fn cuptiActivityEnableRuntimeApi(cbid: CUpti_CallbackId, enable: u8) -> CUptiResult {
        (culib().cuptiActivityEnableRuntimeApi)(cbid, enable)
    }
    pub unsafe fn cuptiActivityFlush(context: CUcontext, streamId: u32, flag: u32) -> CUptiResult {
        (culib().cuptiActivityFlush)(context, streamId, flag)
    }
    pub unsafe fn cuptiActivityFlushAll(flag: u32) -> CUptiResult {
        (culib().cuptiActivityFlushAll)(flag)
    }
    pub unsafe fn cuptiActivityFlushPeriod(time: u32) -> CUptiResult {
        (culib().cuptiActivityFlushPeriod)(time)
    }
    pub unsafe fn cuptiActivityGetAttribute(
        attr: CUpti_ActivityAttribute,
        valueSize: *mut usize,
        value: *mut ::core::ffi::c_void,
    ) -> CUptiResult {
        (culib().cuptiActivityGetAttribute)(attr, valueSize, value)
    }
    pub unsafe fn cuptiActivityGetNextRecord(
        buffer: *mut u8,
        validBufferSizeBytes: usize,
        record: *mut *mut CUpti_Activity,
    ) -> CUptiResult {
        (culib().cuptiActivityGetNextRecord)(buffer, validBufferSizeBytes, record)
    }
    pub unsafe fn cuptiActivityGetNumDroppedRecords(
        context: CUcontext,
        streamId: u32,
        dropped: *mut usize,
    ) -> CUptiResult {
        (culib().cuptiActivityGetNumDroppedRecords)(context, streamId, dropped)
    }
    pub unsafe fn cuptiActivityPopExternalCorrelationId(
        kind: CUpti_ExternalCorrelationKind,
        lastId: *mut u64,
    ) -> CUptiResult {
        (culib().cuptiActivityPopExternalCorrelationId)(kind, lastId)
    }
    pub unsafe fn cuptiActivityPushExternalCorrelationId(
        kind: CUpti_ExternalCorrelationKind,
        id: u64,
    ) -> CUptiResult {
        (culib().cuptiActivityPushExternalCorrelationId)(kind, id)
    }
    pub unsafe fn cuptiActivityRegisterCallbacks(
        funcBufferRequested: CUpti_BuffersCallbackRequestFunc,
        funcBufferCompleted: CUpti_BuffersCallbackCompleteFunc,
    ) -> CUptiResult {
        (culib().cuptiActivityRegisterCallbacks)(funcBufferRequested, funcBufferCompleted)
    }
    #[cfg(any(
        feature = "cuda-11060",
        feature = "cuda-11070",
        feature = "cuda-11080",
        feature = "cuda-12000",
        feature = "cuda-12010",
        feature = "cuda-12020",
        feature = "cuda-12030",
        feature = "cuda-12040",
        feature = "cuda-12050",
        feature = "cuda-12060",
        feature = "cuda-12080",
        feature = "cuda-12090",
        feature = "cuda-13000",
        feature = "cuda-13010"
    ))]
    pub unsafe fn cuptiActivityRegisterTimestampCallback(
        funcTimestamp: CUpti_TimestampCallbackFunc,
    ) -> CUptiResult {
        (culib().cuptiActivityRegisterTimestampCallback)(funcTimestamp)
    }
    pub unsafe fn cuptiActivitySetAttribute(
        attr: CUpti_ActivityAttribute,
        valueSize: *mut usize,
        value: *mut ::core::ffi::c_void,
    ) -> CUptiResult {
        (culib().cuptiActivitySetAttribute)(attr, valueSize, value)
    }
    pub unsafe fn cuptiComputeCapabilitySupported(
        major: ::core::ffi::c_int,
        minor: ::core::ffi::c_int,
        support: *mut ::core::ffi::c_int,
    ) -> CUptiResult {
        (culib().cuptiComputeCapabilitySupported)(major, minor, support)
    }
    pub unsafe fn cuptiDeviceEnumEventDomains(
        device: CUdevice,
        arraySizeBytes: *mut usize,
        domainArray: *mut CUpti_EventDomainID,
    ) -> CUptiResult {
        (culib().cuptiDeviceEnumEventDomains)(device, arraySizeBytes, domainArray)
    }
    pub unsafe fn cuptiDeviceEnumMetrics(
        device: CUdevice,
        arraySizeBytes: *mut usize,
        metricArray: *mut CUpti_MetricID,
    ) -> CUptiResult {
        (culib().cuptiDeviceEnumMetrics)(device, arraySizeBytes, metricArray)
    }
    pub unsafe fn cuptiDeviceGetAttribute(
        device: CUdevice,
        attrib: CUpti_DeviceAttribute,
        valueSize: *mut usize,
        value: *mut ::core::ffi::c_void,
    ) -> CUptiResult {
        (culib().cuptiDeviceGetAttribute)(device, attrib, valueSize, value)
    }
    pub unsafe fn cuptiDeviceGetEventDomainAttribute(
        device: CUdevice,
        eventDomain: CUpti_EventDomainID,
        attrib: CUpti_EventDomainAttribute,
        valueSize: *mut usize,
        value: *mut ::core::ffi::c_void,
    ) -> CUptiResult {
        (culib().cuptiDeviceGetEventDomainAttribute)(device, eventDomain, attrib, valueSize, value)
    }
    pub unsafe fn cuptiDeviceGetNumEventDomains(
        device: CUdevice,
        numDomains: *mut u32,
    ) -> CUptiResult {
        (culib().cuptiDeviceGetNumEventDomains)(device, numDomains)
    }
    pub unsafe fn cuptiDeviceGetNumMetrics(device: CUdevice, numMetrics: *mut u32) -> CUptiResult {
        (culib().cuptiDeviceGetNumMetrics)(device, numMetrics)
    }
    #[cfg(any(
        feature = "cuda-11040",
        feature = "cuda-11050",
        feature = "cuda-11060",
        feature = "cuda-11070",
        feature = "cuda-11080",
        feature = "cuda-12000",
        feature = "cuda-12010"
    ))]
    pub unsafe fn cuptiDeviceGetTimestamp(context: CUcontext, timestamp: *mut u64) -> CUptiResult {
        (culib().cuptiDeviceGetTimestamp)(context, timestamp)
    }
    pub unsafe fn cuptiDeviceSupported(
        dev: CUdevice,
        support: *mut ::core::ffi::c_int,
    ) -> CUptiResult {
        (culib().cuptiDeviceSupported)(dev, support)
    }
    pub unsafe fn cuptiDeviceVirtualizationMode(
        dev: CUdevice,
        mode: *mut CUpti_DeviceVirtualizationMode,
    ) -> CUptiResult {
        (culib().cuptiDeviceVirtualizationMode)(dev, mode)
    }
    pub unsafe fn cuptiDisableKernelReplayMode(context: CUcontext) -> CUptiResult {
        (culib().cuptiDisableKernelReplayMode)(context)
    }
    pub unsafe fn cuptiEnableAllDomains(
        enable: u32,
        subscriber: CUpti_SubscriberHandle,
    ) -> CUptiResult {
        (culib().cuptiEnableAllDomains)(enable, subscriber)
    }
    pub unsafe fn cuptiEnableCallback(
        enable: u32,
        subscriber: CUpti_SubscriberHandle,
        domain: CUpti_CallbackDomain,
        cbid: CUpti_CallbackId,
    ) -> CUptiResult {
        (culib().cuptiEnableCallback)(enable, subscriber, domain, cbid)
    }
    pub unsafe fn cuptiEnableDomain(
        enable: u32,
        subscriber: CUpti_SubscriberHandle,
        domain: CUpti_CallbackDomain,
    ) -> CUptiResult {
        (culib().cuptiEnableDomain)(enable, subscriber, domain)
    }
    pub unsafe fn cuptiEnableKernelReplayMode(context: CUcontext) -> CUptiResult {
        (culib().cuptiEnableKernelReplayMode)(context)
    }
    pub unsafe fn cuptiEnumEventDomains(
        arraySizeBytes: *mut usize,
        domainArray: *mut CUpti_EventDomainID,
    ) -> CUptiResult {
        (culib().cuptiEnumEventDomains)(arraySizeBytes, domainArray)
    }
    pub unsafe fn cuptiEnumMetrics(
        arraySizeBytes: *mut usize,
        metricArray: *mut CUpti_MetricID,
    ) -> CUptiResult {
        (culib().cuptiEnumMetrics)(arraySizeBytes, metricArray)
    }
    pub unsafe fn cuptiEventDomainEnumEvents(
        eventDomain: CUpti_EventDomainID,
        arraySizeBytes: *mut usize,
        eventArray: *mut CUpti_EventID,
    ) -> CUptiResult {
        (culib().cuptiEventDomainEnumEvents)(eventDomain, arraySizeBytes, eventArray)
    }
    pub unsafe fn cuptiEventDomainGetAttribute(
        eventDomain: CUpti_EventDomainID,
        attrib: CUpti_EventDomainAttribute,
        valueSize: *mut usize,
        value: *mut ::core::ffi::c_void,
    ) -> CUptiResult {
        (culib().cuptiEventDomainGetAttribute)(eventDomain, attrib, valueSize, value)
    }
    pub unsafe fn cuptiEventDomainGetNumEvents(
        eventDomain: CUpti_EventDomainID,
        numEvents: *mut u32,
    ) -> CUptiResult {
        (culib().cuptiEventDomainGetNumEvents)(eventDomain, numEvents)
    }
    pub unsafe fn cuptiEventGetAttribute(
        event: CUpti_EventID,
        attrib: CUpti_EventAttribute,
        valueSize: *mut usize,
        value: *mut ::core::ffi::c_void,
    ) -> CUptiResult {
        (culib().cuptiEventGetAttribute)(event, attrib, valueSize, value)
    }
    pub unsafe fn cuptiEventGetIdFromName(
        device: CUdevice,
        eventName: *const ::core::ffi::c_char,
        event: *mut CUpti_EventID,
    ) -> CUptiResult {
        (culib().cuptiEventGetIdFromName)(device, eventName, event)
    }
    pub unsafe fn cuptiEventGroupAddEvent(
        eventGroup: CUpti_EventGroup,
        event: CUpti_EventID,
    ) -> CUptiResult {
        (culib().cuptiEventGroupAddEvent)(eventGroup, event)
    }
    pub unsafe fn cuptiEventGroupCreate(
        context: CUcontext,
        eventGroup: *mut CUpti_EventGroup,
        flags: u32,
    ) -> CUptiResult {
        (culib().cuptiEventGroupCreate)(context, eventGroup, flags)
    }
    pub unsafe fn cuptiEventGroupDestroy(eventGroup: CUpti_EventGroup) -> CUptiResult {
        (culib().cuptiEventGroupDestroy)(eventGroup)
    }
    pub unsafe fn cuptiEventGroupDisable(eventGroup: CUpti_EventGroup) -> CUptiResult {
        (culib().cuptiEventGroupDisable)(eventGroup)
    }
    pub unsafe fn cuptiEventGroupEnable(eventGroup: CUpti_EventGroup) -> CUptiResult {
        (culib().cuptiEventGroupEnable)(eventGroup)
    }
    pub unsafe fn cuptiEventGroupGetAttribute(
        eventGroup: CUpti_EventGroup,
        attrib: CUpti_EventGroupAttribute,
        valueSize: *mut usize,
        value: *mut ::core::ffi::c_void,
    ) -> CUptiResult {
        (culib().cuptiEventGroupGetAttribute)(eventGroup, attrib, valueSize, value)
    }
    pub unsafe fn cuptiEventGroupReadAllEvents(
        eventGroup: CUpti_EventGroup,
        flags: CUpti_ReadEventFlags,
        eventValueBufferSizeBytes: *mut usize,
        eventValueBuffer: *mut u64,
        eventIdArraySizeBytes: *mut usize,
        eventIdArray: *mut CUpti_EventID,
        numEventIdsRead: *mut usize,
    ) -> CUptiResult {
        (culib().cuptiEventGroupReadAllEvents)(
            eventGroup,
            flags,
            eventValueBufferSizeBytes,
            eventValueBuffer,
            eventIdArraySizeBytes,
            eventIdArray,
            numEventIdsRead,
        )
    }
    pub unsafe fn cuptiEventGroupReadEvent(
        eventGroup: CUpti_EventGroup,
        flags: CUpti_ReadEventFlags,
        event: CUpti_EventID,
        eventValueBufferSizeBytes: *mut usize,
        eventValueBuffer: *mut u64,
    ) -> CUptiResult {
        (culib().cuptiEventGroupReadEvent)(
            eventGroup,
            flags,
            event,
            eventValueBufferSizeBytes,
            eventValueBuffer,
        )
    }
    pub unsafe fn cuptiEventGroupRemoveAllEvents(eventGroup: CUpti_EventGroup) -> CUptiResult {
        (culib().cuptiEventGroupRemoveAllEvents)(eventGroup)
    }
    pub unsafe fn cuptiEventGroupRemoveEvent(
        eventGroup: CUpti_EventGroup,
        event: CUpti_EventID,
    ) -> CUptiResult {
        (culib().cuptiEventGroupRemoveEvent)(eventGroup, event)
    }
    pub unsafe fn cuptiEventGroupResetAllEvents(eventGroup: CUpti_EventGroup) -> CUptiResult {
        (culib().cuptiEventGroupResetAllEvents)(eventGroup)
    }
    pub unsafe fn cuptiEventGroupSetAttribute(
        eventGroup: CUpti_EventGroup,
        attrib: CUpti_EventGroupAttribute,
        valueSize: usize,
        value: *mut ::core::ffi::c_void,
    ) -> CUptiResult {
        (culib().cuptiEventGroupSetAttribute)(eventGroup, attrib, valueSize, value)
    }
    pub unsafe fn cuptiEventGroupSetDisable(
        eventGroupSet: *mut CUpti_EventGroupSet,
    ) -> CUptiResult {
        (culib().cuptiEventGroupSetDisable)(eventGroupSet)
    }
    pub unsafe fn cuptiEventGroupSetEnable(eventGroupSet: *mut CUpti_EventGroupSet) -> CUptiResult {
        (culib().cuptiEventGroupSetEnable)(eventGroupSet)
    }
    pub unsafe fn cuptiEventGroupSetsCreate(
        context: CUcontext,
        eventIdArraySizeBytes: usize,
        eventIdArray: *mut CUpti_EventID,
        eventGroupPasses: *mut *mut CUpti_EventGroupSets,
    ) -> CUptiResult {
        (culib().cuptiEventGroupSetsCreate)(
            context,
            eventIdArraySizeBytes,
            eventIdArray,
            eventGroupPasses,
        )
    }
    pub unsafe fn cuptiEventGroupSetsDestroy(
        eventGroupSets: *mut CUpti_EventGroupSets,
    ) -> CUptiResult {
        (culib().cuptiEventGroupSetsDestroy)(eventGroupSets)
    }
    pub unsafe fn cuptiFinalize() -> CUptiResult {
        (culib().cuptiFinalize)()
    }
    pub unsafe fn cuptiGetAutoBoostState(
        context: CUcontext,
        state: *mut CUpti_ActivityAutoBoostState,
    ) -> CUptiResult {
        (culib().cuptiGetAutoBoostState)(context, state)
    }
    pub unsafe fn cuptiGetCallbackName(
        domain: CUpti_CallbackDomain,
        cbid: u32,
        name: *mut *const ::core::ffi::c_char,
    ) -> CUptiResult {
        (culib().cuptiGetCallbackName)(domain, cbid, name)
    }
    pub unsafe fn cuptiGetCallbackState(
        enable: *mut u32,
        subscriber: CUpti_SubscriberHandle,
        domain: CUpti_CallbackDomain,
        cbid: CUpti_CallbackId,
    ) -> CUptiResult {
        (culib().cuptiGetCallbackState)(enable, subscriber, domain, cbid)
    }
    pub unsafe fn cuptiGetContextId(context: CUcontext, contextId: *mut u32) -> CUptiResult {
        (culib().cuptiGetContextId)(context, contextId)
    }
    pub unsafe fn cuptiGetDeviceId(context: CUcontext, deviceId: *mut u32) -> CUptiResult {
        (culib().cuptiGetDeviceId)(context, deviceId)
    }
    #[cfg(any(
        feature = "cuda-12020",
        feature = "cuda-12030",
        feature = "cuda-12040",
        feature = "cuda-12050",
        feature = "cuda-12060",
        feature = "cuda-12080",
        feature = "cuda-12090",
        feature = "cuda-13000",
        feature = "cuda-13010"
    ))]
    pub unsafe fn cuptiGetErrorMessage(
        result: CUptiResult,
        str_: *mut *const ::core::ffi::c_char,
    ) -> CUptiResult {
        (culib().cuptiGetErrorMessage)(result, str_)
    }
    #[cfg(any(
        feature = "cuda-12030",
        feature = "cuda-12040",
        feature = "cuda-12050",
        feature = "cuda-12060",
        feature = "cuda-12080",
        feature = "cuda-12090",
        feature = "cuda-13000",
        feature = "cuda-13010"
    ))]
    pub unsafe fn cuptiGetGraphExecId(graphExec: CUgraphExec, pId: *mut u32) -> CUptiResult {
        (culib().cuptiGetGraphExecId)(graphExec, pId)
    }
    pub unsafe fn cuptiGetGraphId(graph: CUgraph, pId: *mut u32) -> CUptiResult {
        (culib().cuptiGetGraphId)(graph, pId)
    }
    pub unsafe fn cuptiGetGraphNodeId(node: CUgraphNode, nodeId: *mut u64) -> CUptiResult {
        (culib().cuptiGetGraphNodeId)(node, nodeId)
    }
    pub unsafe fn cuptiGetLastError() -> CUptiResult {
        (culib().cuptiGetLastError)()
    }
    pub unsafe fn cuptiGetNumEventDomains(numDomains: *mut u32) -> CUptiResult {
        (culib().cuptiGetNumEventDomains)(numDomains)
    }
    pub unsafe fn cuptiGetNumMetrics(numMetrics: *mut u32) -> CUptiResult {
        (culib().cuptiGetNumMetrics)(numMetrics)
    }
    pub unsafe fn cuptiGetResultString(
        result: CUptiResult,
        str_: *mut *const ::core::ffi::c_char,
    ) -> CUptiResult {
        (culib().cuptiGetResultString)(result, str_)
    }
    pub unsafe fn cuptiGetStreamId(
        context: CUcontext,
        stream: CUstream,
        streamId: *mut u32,
    ) -> CUptiResult {
        (culib().cuptiGetStreamId)(context, stream, streamId)
    }
    pub unsafe fn cuptiGetStreamIdEx(
        context: CUcontext,
        stream: CUstream,
        perThreadStream: u8,
        streamId: *mut u32,
    ) -> CUptiResult {
        (culib().cuptiGetStreamIdEx)(context, stream, perThreadStream, streamId)
    }
    pub unsafe fn cuptiGetThreadIdType(type_: *mut CUpti_ActivityThreadIdType) -> CUptiResult {
        (culib().cuptiGetThreadIdType)(type_)
    }
    pub unsafe fn cuptiGetTimestamp(timestamp: *mut u64) -> CUptiResult {
        (culib().cuptiGetTimestamp)(timestamp)
    }
    pub unsafe fn cuptiGetVersion(version: *mut u32) -> CUptiResult {
        (culib().cuptiGetVersion)(version)
    }
    pub unsafe fn cuptiKernelReplaySubscribeUpdate(
        updateFunc: CUpti_KernelReplayUpdateFunc,
        customData: *mut ::core::ffi::c_void,
    ) -> CUptiResult {
        (culib().cuptiKernelReplaySubscribeUpdate)(updateFunc, customData)
    }
    pub unsafe fn cuptiMetricCreateEventGroupSets(
        context: CUcontext,
        metricIdArraySizeBytes: usize,
        metricIdArray: *mut CUpti_MetricID,
        eventGroupPasses: *mut *mut CUpti_EventGroupSets,
    ) -> CUptiResult {
        (culib().cuptiMetricCreateEventGroupSets)(
            context,
            metricIdArraySizeBytes,
            metricIdArray,
            eventGroupPasses,
        )
    }
    pub unsafe fn cuptiMetricEnumEvents(
        metric: CUpti_MetricID,
        eventIdArraySizeBytes: *mut usize,
        eventIdArray: *mut CUpti_EventID,
    ) -> CUptiResult {
        (culib().cuptiMetricEnumEvents)(metric, eventIdArraySizeBytes, eventIdArray)
    }
    pub unsafe fn cuptiMetricEnumProperties(
        metric: CUpti_MetricID,
        propIdArraySizeBytes: *mut usize,
        propIdArray: *mut CUpti_MetricPropertyID,
    ) -> CUptiResult {
        (culib().cuptiMetricEnumProperties)(metric, propIdArraySizeBytes, propIdArray)
    }
    pub unsafe fn cuptiMetricGetAttribute(
        metric: CUpti_MetricID,
        attrib: CUpti_MetricAttribute,
        valueSize: *mut usize,
        value: *mut ::core::ffi::c_void,
    ) -> CUptiResult {
        (culib().cuptiMetricGetAttribute)(metric, attrib, valueSize, value)
    }
    pub unsafe fn cuptiMetricGetIdFromName(
        device: CUdevice,
        metricName: *const ::core::ffi::c_char,
        metric: *mut CUpti_MetricID,
    ) -> CUptiResult {
        (culib().cuptiMetricGetIdFromName)(device, metricName, metric)
    }
    pub unsafe fn cuptiMetricGetNumEvents(
        metric: CUpti_MetricID,
        numEvents: *mut u32,
    ) -> CUptiResult {
        (culib().cuptiMetricGetNumEvents)(metric, numEvents)
    }
    pub unsafe fn cuptiMetricGetNumProperties(
        metric: CUpti_MetricID,
        numProp: *mut u32,
    ) -> CUptiResult {
        (culib().cuptiMetricGetNumProperties)(metric, numProp)
    }
    pub unsafe fn cuptiMetricGetRequiredEventGroupSets(
        context: CUcontext,
        metric: CUpti_MetricID,
        eventGroupSets: *mut *mut CUpti_EventGroupSets,
    ) -> CUptiResult {
        (culib().cuptiMetricGetRequiredEventGroupSets)(context, metric, eventGroupSets)
    }
    pub unsafe fn cuptiMetricGetValue(
        device: CUdevice,
        metric: CUpti_MetricID,
        eventIdArraySizeBytes: usize,
        eventIdArray: *mut CUpti_EventID,
        eventValueArraySizeBytes: usize,
        eventValueArray: *mut u64,
        timeDuration: u64,
        metricValue: *mut CUpti_MetricValue,
    ) -> CUptiResult {
        (culib().cuptiMetricGetValue)(
            device,
            metric,
            eventIdArraySizeBytes,
            eventIdArray,
            eventValueArraySizeBytes,
            eventValueArray,
            timeDuration,
            metricValue,
        )
    }
    pub unsafe fn cuptiMetricGetValue2(
        metric: CUpti_MetricID,
        eventIdArraySizeBytes: usize,
        eventIdArray: *mut CUpti_EventID,
        eventValueArraySizeBytes: usize,
        eventValueArray: *mut u64,
        propIdArraySizeBytes: usize,
        propIdArray: *mut CUpti_MetricPropertyID,
        propValueArraySizeBytes: usize,
        propValueArray: *mut u64,
        metricValue: *mut CUpti_MetricValue,
    ) -> CUptiResult {
        (culib().cuptiMetricGetValue2)(
            metric,
            eventIdArraySizeBytes,
            eventIdArray,
            eventValueArraySizeBytes,
            eventValueArray,
            propIdArraySizeBytes,
            propIdArray,
            propValueArraySizeBytes,
            propValueArray,
            metricValue,
        )
    }
    pub unsafe fn cuptiSetEventCollectionMode(
        context: CUcontext,
        mode: CUpti_EventCollectionMode,
    ) -> CUptiResult {
        (culib().cuptiSetEventCollectionMode)(context, mode)
    }
    pub unsafe fn cuptiSetThreadIdType(type_: CUpti_ActivityThreadIdType) -> CUptiResult {
        (culib().cuptiSetThreadIdType)(type_)
    }
    pub unsafe fn cuptiSubscribe(
        subscriber: *mut CUpti_SubscriberHandle,
        callback: CUpti_CallbackFunc,
        userdata: *mut ::core::ffi::c_void,
    ) -> CUptiResult {
        (culib().cuptiSubscribe)(subscriber, callback, userdata)
    }
    #[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
    pub unsafe fn cuptiSubscribe_v2(
        subscriber: *mut CUpti_SubscriberHandle,
        callback: CUpti_CallbackFunc,
        userdata: *mut ::core::ffi::c_void,
        pParams: *mut CUpti_SubscriberParams,
    ) -> CUptiResult {
        (culib().cuptiSubscribe_v2)(subscriber, callback, userdata, pParams)
    }
    pub unsafe fn cuptiSupportedDomains(
        domainCount: *mut usize,
        domainTable: *mut CUpti_DomainTable,
    ) -> CUptiResult {
        (culib().cuptiSupportedDomains)(domainCount, domainTable)
    }
    pub unsafe fn cuptiUnsubscribe(subscriber: CUpti_SubscriberHandle) -> CUptiResult {
        (culib().cuptiUnsubscribe)(subscriber)
    }
    pub struct Lib {
        __library: ::libloading::Library,
        pub cuptiActivityConfigurePCSampling: unsafe extern "C" fn(
            ctx: CUcontext,
            config: *mut CUpti_ActivityPCSamplingConfig,
        ) -> CUptiResult,
        pub cuptiActivityConfigureUnifiedMemoryCounter: unsafe extern "C" fn(
            config: *mut CUpti_ActivityUnifiedMemoryCounterConfig,
            count: u32,
        )
            -> CUptiResult,
        pub cuptiActivityDisable: unsafe extern "C" fn(kind: CUpti_ActivityKind) -> CUptiResult,
        pub cuptiActivityDisableContext:
            unsafe extern "C" fn(context: CUcontext, kind: CUpti_ActivityKind) -> CUptiResult,
        pub cuptiActivityEnable: unsafe extern "C" fn(kind: CUpti_ActivityKind) -> CUptiResult,
        #[cfg(any(
            feature = "cuda-12080",
            feature = "cuda-12090",
            feature = "cuda-13000",
            feature = "cuda-13010"
        ))]
        pub cuptiActivityEnableAllSyncRecords: unsafe extern "C" fn(enable: u8) -> CUptiResult,
        #[cfg(any(
            feature = "cuda-12060",
            feature = "cuda-12080",
            feature = "cuda-12090",
            feature = "cuda-13000",
            feature = "cuda-13010"
        ))]
        pub cuptiActivityEnableAllocationSource: unsafe extern "C" fn(enable: u8) -> CUptiResult,
        #[cfg(any(
            feature = "cuda-11070",
            feature = "cuda-11080",
            feature = "cuda-12000",
            feature = "cuda-12010",
            feature = "cuda-12020",
            feature = "cuda-12030",
            feature = "cuda-12040",
            feature = "cuda-12050",
            feature = "cuda-12060",
            feature = "cuda-12080",
            feature = "cuda-12090",
            feature = "cuda-13000",
            feature = "cuda-13010"
        ))]
        pub cuptiActivityEnableAndDump:
            unsafe extern "C" fn(kind: CUpti_ActivityKind) -> CUptiResult,
        pub cuptiActivityEnableContext:
            unsafe extern "C" fn(context: CUcontext, kind: CUpti_ActivityKind) -> CUptiResult,
        #[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
        pub cuptiActivityEnableCudaEventDeviceTimestamps:
            unsafe extern "C" fn(enable: u8) -> CUptiResult,
        #[cfg(any(
            feature = "cuda-12030",
            feature = "cuda-12040",
            feature = "cuda-12050",
            feature = "cuda-12060",
            feature = "cuda-12080",
            feature = "cuda-12090",
            feature = "cuda-13000",
            feature = "cuda-13010"
        ))]
        pub cuptiActivityEnableDeviceGraph: unsafe extern "C" fn(enable: u8) -> CUptiResult,
        #[cfg(any(
            feature = "cuda-12050",
            feature = "cuda-12060",
            feature = "cuda-12080",
            feature = "cuda-12090",
            feature = "cuda-13000",
            feature = "cuda-13010"
        ))]
        pub cuptiActivityEnableDriverApi:
            unsafe extern "C" fn(cbid: CUpti_CallbackId, enable: u8) -> CUptiResult,
        #[cfg(any(
            feature = "cuda-12080",
            feature = "cuda-12090",
            feature = "cuda-13000",
            feature = "cuda-13010"
        ))]
        pub cuptiActivityEnableHWTrace: unsafe extern "C" fn(enable: u8) -> CUptiResult,
        pub cuptiActivityEnableLatencyTimestamps: unsafe extern "C" fn(enable: u8) -> CUptiResult,
        pub cuptiActivityEnableLaunchAttributes: unsafe extern "C" fn(enable: u8) -> CUptiResult,
        #[cfg(any(
            feature = "cuda-12050",
            feature = "cuda-12060",
            feature = "cuda-12080",
            feature = "cuda-12090",
            feature = "cuda-13000",
            feature = "cuda-13010"
        ))]
        pub cuptiActivityEnableRuntimeApi:
            unsafe extern "C" fn(cbid: CUpti_CallbackId, enable: u8) -> CUptiResult,
        pub cuptiActivityFlush:
            unsafe extern "C" fn(context: CUcontext, streamId: u32, flag: u32) -> CUptiResult,
        pub cuptiActivityFlushAll: unsafe extern "C" fn(flag: u32) -> CUptiResult,
        pub cuptiActivityFlushPeriod: unsafe extern "C" fn(time: u32) -> CUptiResult,
        pub cuptiActivityGetAttribute: unsafe extern "C" fn(
            attr: CUpti_ActivityAttribute,
            valueSize: *mut usize,
            value: *mut ::core::ffi::c_void,
        ) -> CUptiResult,
        pub cuptiActivityGetNextRecord: unsafe extern "C" fn(
            buffer: *mut u8,
            validBufferSizeBytes: usize,
            record: *mut *mut CUpti_Activity,
        ) -> CUptiResult,
        pub cuptiActivityGetNumDroppedRecords: unsafe extern "C" fn(
            context: CUcontext,
            streamId: u32,
            dropped: *mut usize,
        ) -> CUptiResult,
        pub cuptiActivityPopExternalCorrelationId: unsafe extern "C" fn(
            kind: CUpti_ExternalCorrelationKind,
            lastId: *mut u64,
        ) -> CUptiResult,
        pub cuptiActivityPushExternalCorrelationId:
            unsafe extern "C" fn(kind: CUpti_ExternalCorrelationKind, id: u64) -> CUptiResult,
        pub cuptiActivityRegisterCallbacks: unsafe extern "C" fn(
            funcBufferRequested: CUpti_BuffersCallbackRequestFunc,
            funcBufferCompleted: CUpti_BuffersCallbackCompleteFunc,
        ) -> CUptiResult,
        #[cfg(any(
            feature = "cuda-11060",
            feature = "cuda-11070",
            feature = "cuda-11080",
            feature = "cuda-12000",
            feature = "cuda-12010",
            feature = "cuda-12020",
            feature = "cuda-12030",
            feature = "cuda-12040",
            feature = "cuda-12050",
            feature = "cuda-12060",
            feature = "cuda-12080",
            feature = "cuda-12090",
            feature = "cuda-13000",
            feature = "cuda-13010"
        ))]
        pub cuptiActivityRegisterTimestampCallback:
            unsafe extern "C" fn(funcTimestamp: CUpti_TimestampCallbackFunc) -> CUptiResult,
        pub cuptiActivitySetAttribute: unsafe extern "C" fn(
            attr: CUpti_ActivityAttribute,
            valueSize: *mut usize,
            value: *mut ::core::ffi::c_void,
        ) -> CUptiResult,
        pub cuptiComputeCapabilitySupported: unsafe extern "C" fn(
            major: ::core::ffi::c_int,
            minor: ::core::ffi::c_int,
            support: *mut ::core::ffi::c_int,
        ) -> CUptiResult,
        pub cuptiDeviceEnumEventDomains: unsafe extern "C" fn(
            device: CUdevice,
            arraySizeBytes: *mut usize,
            domainArray: *mut CUpti_EventDomainID,
        ) -> CUptiResult,
        pub cuptiDeviceEnumMetrics: unsafe extern "C" fn(
            device: CUdevice,
            arraySizeBytes: *mut usize,
            metricArray: *mut CUpti_MetricID,
        ) -> CUptiResult,
        pub cuptiDeviceGetAttribute: unsafe extern "C" fn(
            device: CUdevice,
            attrib: CUpti_DeviceAttribute,
            valueSize: *mut usize,
            value: *mut ::core::ffi::c_void,
        ) -> CUptiResult,
        pub cuptiDeviceGetEventDomainAttribute: unsafe extern "C" fn(
            device: CUdevice,
            eventDomain: CUpti_EventDomainID,
            attrib: CUpti_EventDomainAttribute,
            valueSize: *mut usize,
            value: *mut ::core::ffi::c_void,
        ) -> CUptiResult,
        pub cuptiDeviceGetNumEventDomains:
            unsafe extern "C" fn(device: CUdevice, numDomains: *mut u32) -> CUptiResult,
        pub cuptiDeviceGetNumMetrics:
            unsafe extern "C" fn(device: CUdevice, numMetrics: *mut u32) -> CUptiResult,
        #[cfg(any(
            feature = "cuda-11040",
            feature = "cuda-11050",
            feature = "cuda-11060",
            feature = "cuda-11070",
            feature = "cuda-11080",
            feature = "cuda-12000",
            feature = "cuda-12010"
        ))]
        pub cuptiDeviceGetTimestamp:
            unsafe extern "C" fn(context: CUcontext, timestamp: *mut u64) -> CUptiResult,
        pub cuptiDeviceSupported:
            unsafe extern "C" fn(dev: CUdevice, support: *mut ::core::ffi::c_int) -> CUptiResult,
        pub cuptiDeviceVirtualizationMode: unsafe extern "C" fn(
            dev: CUdevice,
            mode: *mut CUpti_DeviceVirtualizationMode,
        ) -> CUptiResult,
        pub cuptiDisableKernelReplayMode: unsafe extern "C" fn(context: CUcontext) -> CUptiResult,
        pub cuptiEnableAllDomains:
            unsafe extern "C" fn(enable: u32, subscriber: CUpti_SubscriberHandle) -> CUptiResult,
        pub cuptiEnableCallback: unsafe extern "C" fn(
            enable: u32,
            subscriber: CUpti_SubscriberHandle,
            domain: CUpti_CallbackDomain,
            cbid: CUpti_CallbackId,
        ) -> CUptiResult,
        pub cuptiEnableDomain: unsafe extern "C" fn(
            enable: u32,
            subscriber: CUpti_SubscriberHandle,
            domain: CUpti_CallbackDomain,
        ) -> CUptiResult,
        pub cuptiEnableKernelReplayMode: unsafe extern "C" fn(context: CUcontext) -> CUptiResult,
        pub cuptiEnumEventDomains: unsafe extern "C" fn(
            arraySizeBytes: *mut usize,
            domainArray: *mut CUpti_EventDomainID,
        ) -> CUptiResult,
        pub cuptiEnumMetrics: unsafe extern "C" fn(
            arraySizeBytes: *mut usize,
            metricArray: *mut CUpti_MetricID,
        ) -> CUptiResult,
        pub cuptiEventDomainEnumEvents: unsafe extern "C" fn(
            eventDomain: CUpti_EventDomainID,
            arraySizeBytes: *mut usize,
            eventArray: *mut CUpti_EventID,
        ) -> CUptiResult,
        pub cuptiEventDomainGetAttribute: unsafe extern "C" fn(
            eventDomain: CUpti_EventDomainID,
            attrib: CUpti_EventDomainAttribute,
            valueSize: *mut usize,
            value: *mut ::core::ffi::c_void,
        ) -> CUptiResult,
        pub cuptiEventDomainGetNumEvents: unsafe extern "C" fn(
            eventDomain: CUpti_EventDomainID,
            numEvents: *mut u32,
        ) -> CUptiResult,
        pub cuptiEventGetAttribute: unsafe extern "C" fn(
            event: CUpti_EventID,
            attrib: CUpti_EventAttribute,
            valueSize: *mut usize,
            value: *mut ::core::ffi::c_void,
        ) -> CUptiResult,
        pub cuptiEventGetIdFromName: unsafe extern "C" fn(
            device: CUdevice,
            eventName: *const ::core::ffi::c_char,
            event: *mut CUpti_EventID,
        ) -> CUptiResult,
        pub cuptiEventGroupAddEvent:
            unsafe extern "C" fn(eventGroup: CUpti_EventGroup, event: CUpti_EventID) -> CUptiResult,
        pub cuptiEventGroupCreate: unsafe extern "C" fn(
            context: CUcontext,
            eventGroup: *mut CUpti_EventGroup,
            flags: u32,
        ) -> CUptiResult,
        pub cuptiEventGroupDestroy:
            unsafe extern "C" fn(eventGroup: CUpti_EventGroup) -> CUptiResult,
        pub cuptiEventGroupDisable:
            unsafe extern "C" fn(eventGroup: CUpti_EventGroup) -> CUptiResult,
        pub cuptiEventGroupEnable:
            unsafe extern "C" fn(eventGroup: CUpti_EventGroup) -> CUptiResult,
        pub cuptiEventGroupGetAttribute: unsafe extern "C" fn(
            eventGroup: CUpti_EventGroup,
            attrib: CUpti_EventGroupAttribute,
            valueSize: *mut usize,
            value: *mut ::core::ffi::c_void,
        ) -> CUptiResult,
        pub cuptiEventGroupReadAllEvents: unsafe extern "C" fn(
            eventGroup: CUpti_EventGroup,
            flags: CUpti_ReadEventFlags,
            eventValueBufferSizeBytes: *mut usize,
            eventValueBuffer: *mut u64,
            eventIdArraySizeBytes: *mut usize,
            eventIdArray: *mut CUpti_EventID,
            numEventIdsRead: *mut usize,
        ) -> CUptiResult,
        pub cuptiEventGroupReadEvent: unsafe extern "C" fn(
            eventGroup: CUpti_EventGroup,
            flags: CUpti_ReadEventFlags,
            event: CUpti_EventID,
            eventValueBufferSizeBytes: *mut usize,
            eventValueBuffer: *mut u64,
        ) -> CUptiResult,
        pub cuptiEventGroupRemoveAllEvents:
            unsafe extern "C" fn(eventGroup: CUpti_EventGroup) -> CUptiResult,
        pub cuptiEventGroupRemoveEvent:
            unsafe extern "C" fn(eventGroup: CUpti_EventGroup, event: CUpti_EventID) -> CUptiResult,
        pub cuptiEventGroupResetAllEvents:
            unsafe extern "C" fn(eventGroup: CUpti_EventGroup) -> CUptiResult,
        pub cuptiEventGroupSetAttribute: unsafe extern "C" fn(
            eventGroup: CUpti_EventGroup,
            attrib: CUpti_EventGroupAttribute,
            valueSize: usize,
            value: *mut ::core::ffi::c_void,
        ) -> CUptiResult,
        pub cuptiEventGroupSetDisable:
            unsafe extern "C" fn(eventGroupSet: *mut CUpti_EventGroupSet) -> CUptiResult,
        pub cuptiEventGroupSetEnable:
            unsafe extern "C" fn(eventGroupSet: *mut CUpti_EventGroupSet) -> CUptiResult,
        pub cuptiEventGroupSetsCreate: unsafe extern "C" fn(
            context: CUcontext,
            eventIdArraySizeBytes: usize,
            eventIdArray: *mut CUpti_EventID,
            eventGroupPasses: *mut *mut CUpti_EventGroupSets,
        ) -> CUptiResult,
        pub cuptiEventGroupSetsDestroy:
            unsafe extern "C" fn(eventGroupSets: *mut CUpti_EventGroupSets) -> CUptiResult,
        pub cuptiFinalize: unsafe extern "C" fn() -> CUptiResult,
        pub cuptiGetAutoBoostState: unsafe extern "C" fn(
            context: CUcontext,
            state: *mut CUpti_ActivityAutoBoostState,
        ) -> CUptiResult,
        pub cuptiGetCallbackName: unsafe extern "C" fn(
            domain: CUpti_CallbackDomain,
            cbid: u32,
            name: *mut *const ::core::ffi::c_char,
        ) -> CUptiResult,
        pub cuptiGetCallbackState: unsafe extern "C" fn(
            enable: *mut u32,
            subscriber: CUpti_SubscriberHandle,
            domain: CUpti_CallbackDomain,
            cbid: CUpti_CallbackId,
        ) -> CUptiResult,
        pub cuptiGetContextId:
            unsafe extern "C" fn(context: CUcontext, contextId: *mut u32) -> CUptiResult,
        pub cuptiGetDeviceId:
            unsafe extern "C" fn(context: CUcontext, deviceId: *mut u32) -> CUptiResult,
        #[cfg(any(
            feature = "cuda-12020",
            feature = "cuda-12030",
            feature = "cuda-12040",
            feature = "cuda-12050",
            feature = "cuda-12060",
            feature = "cuda-12080",
            feature = "cuda-12090",
            feature = "cuda-13000",
            feature = "cuda-13010"
        ))]
        pub cuptiGetErrorMessage: unsafe extern "C" fn(
            result: CUptiResult,
            str_: *mut *const ::core::ffi::c_char,
        ) -> CUptiResult,
        #[cfg(any(
            feature = "cuda-12030",
            feature = "cuda-12040",
            feature = "cuda-12050",
            feature = "cuda-12060",
            feature = "cuda-12080",
            feature = "cuda-12090",
            feature = "cuda-13000",
            feature = "cuda-13010"
        ))]
        pub cuptiGetGraphExecId:
            unsafe extern "C" fn(graphExec: CUgraphExec, pId: *mut u32) -> CUptiResult,
        pub cuptiGetGraphId: unsafe extern "C" fn(graph: CUgraph, pId: *mut u32) -> CUptiResult,
        pub cuptiGetGraphNodeId:
            unsafe extern "C" fn(node: CUgraphNode, nodeId: *mut u64) -> CUptiResult,
        pub cuptiGetLastError: unsafe extern "C" fn() -> CUptiResult,
        pub cuptiGetNumEventDomains: unsafe extern "C" fn(numDomains: *mut u32) -> CUptiResult,
        pub cuptiGetNumMetrics: unsafe extern "C" fn(numMetrics: *mut u32) -> CUptiResult,
        pub cuptiGetResultString: unsafe extern "C" fn(
            result: CUptiResult,
            str_: *mut *const ::core::ffi::c_char,
        ) -> CUptiResult,
        pub cuptiGetStreamId: unsafe extern "C" fn(
            context: CUcontext,
            stream: CUstream,
            streamId: *mut u32,
        ) -> CUptiResult,
        pub cuptiGetStreamIdEx: unsafe extern "C" fn(
            context: CUcontext,
            stream: CUstream,
            perThreadStream: u8,
            streamId: *mut u32,
        ) -> CUptiResult,
        pub cuptiGetThreadIdType:
            unsafe extern "C" fn(type_: *mut CUpti_ActivityThreadIdType) -> CUptiResult,
        pub cuptiGetTimestamp: unsafe extern "C" fn(timestamp: *mut u64) -> CUptiResult,
        pub cuptiGetVersion: unsafe extern "C" fn(version: *mut u32) -> CUptiResult,
        pub cuptiKernelReplaySubscribeUpdate: unsafe extern "C" fn(
            updateFunc: CUpti_KernelReplayUpdateFunc,
            customData: *mut ::core::ffi::c_void,
        ) -> CUptiResult,
        pub cuptiMetricCreateEventGroupSets: unsafe extern "C" fn(
            context: CUcontext,
            metricIdArraySizeBytes: usize,
            metricIdArray: *mut CUpti_MetricID,
            eventGroupPasses: *mut *mut CUpti_EventGroupSets,
        ) -> CUptiResult,
        pub cuptiMetricEnumEvents: unsafe extern "C" fn(
            metric: CUpti_MetricID,
            eventIdArraySizeBytes: *mut usize,
            eventIdArray: *mut CUpti_EventID,
        ) -> CUptiResult,
        pub cuptiMetricEnumProperties: unsafe extern "C" fn(
            metric: CUpti_MetricID,
            propIdArraySizeBytes: *mut usize,
            propIdArray: *mut CUpti_MetricPropertyID,
        ) -> CUptiResult,
        pub cuptiMetricGetAttribute: unsafe extern "C" fn(
            metric: CUpti_MetricID,
            attrib: CUpti_MetricAttribute,
            valueSize: *mut usize,
            value: *mut ::core::ffi::c_void,
        ) -> CUptiResult,
        pub cuptiMetricGetIdFromName: unsafe extern "C" fn(
            device: CUdevice,
            metricName: *const ::core::ffi::c_char,
            metric: *mut CUpti_MetricID,
        ) -> CUptiResult,
        pub cuptiMetricGetNumEvents:
            unsafe extern "C" fn(metric: CUpti_MetricID, numEvents: *mut u32) -> CUptiResult,
        pub cuptiMetricGetNumProperties:
            unsafe extern "C" fn(metric: CUpti_MetricID, numProp: *mut u32) -> CUptiResult,
        pub cuptiMetricGetRequiredEventGroupSets: unsafe extern "C" fn(
            context: CUcontext,
            metric: CUpti_MetricID,
            eventGroupSets: *mut *mut CUpti_EventGroupSets,
        ) -> CUptiResult,
        pub cuptiMetricGetValue: unsafe extern "C" fn(
            device: CUdevice,
            metric: CUpti_MetricID,
            eventIdArraySizeBytes: usize,
            eventIdArray: *mut CUpti_EventID,
            eventValueArraySizeBytes: usize,
            eventValueArray: *mut u64,
            timeDuration: u64,
            metricValue: *mut CUpti_MetricValue,
        ) -> CUptiResult,
        pub cuptiMetricGetValue2: unsafe extern "C" fn(
            metric: CUpti_MetricID,
            eventIdArraySizeBytes: usize,
            eventIdArray: *mut CUpti_EventID,
            eventValueArraySizeBytes: usize,
            eventValueArray: *mut u64,
            propIdArraySizeBytes: usize,
            propIdArray: *mut CUpti_MetricPropertyID,
            propValueArraySizeBytes: usize,
            propValueArray: *mut u64,
            metricValue: *mut CUpti_MetricValue,
        ) -> CUptiResult,
        pub cuptiSetEventCollectionMode: unsafe extern "C" fn(
            context: CUcontext,
            mode: CUpti_EventCollectionMode,
        ) -> CUptiResult,
        pub cuptiSetThreadIdType:
            unsafe extern "C" fn(type_: CUpti_ActivityThreadIdType) -> CUptiResult,
        pub cuptiSubscribe: unsafe extern "C" fn(
            subscriber: *mut CUpti_SubscriberHandle,
            callback: CUpti_CallbackFunc,
            userdata: *mut ::core::ffi::c_void,
        ) -> CUptiResult,
        #[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
        pub cuptiSubscribe_v2: unsafe extern "C" fn(
            subscriber: *mut CUpti_SubscriberHandle,
            callback: CUpti_CallbackFunc,
            userdata: *mut ::core::ffi::c_void,
            pParams: *mut CUpti_SubscriberParams,
        ) -> CUptiResult,
        pub cuptiSupportedDomains: unsafe extern "C" fn(
            domainCount: *mut usize,
            domainTable: *mut CUpti_DomainTable,
        ) -> CUptiResult,
        pub cuptiUnsubscribe:
            unsafe extern "C" fn(subscriber: CUpti_SubscriberHandle) -> CUptiResult,
    }
    impl Lib {
        pub unsafe fn new<P>(path: P) -> Result<Self, ::libloading::Error>
        where
            P: AsRef<::std::ffi::OsStr>,
        {
            let library = ::libloading::Library::new(path)?;
            Self::from_library(library)
        }
        pub unsafe fn from_library<L>(library: L) -> Result<Self, ::libloading::Error>
        where
            L: Into<::libloading::Library>,
        {
            let __library = library.into();
            let cuptiActivityConfigurePCSampling = __library
                .get(b"cuptiActivityConfigurePCSampling\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiActivityConfigureUnifiedMemoryCounter = __library
                .get(b"cuptiActivityConfigureUnifiedMemoryCounter\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiActivityDisable = __library
                .get(b"cuptiActivityDisable\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiActivityDisableContext = __library
                .get(b"cuptiActivityDisableContext\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiActivityEnable = __library
                .get(b"cuptiActivityEnable\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            #[cfg(any(
                feature = "cuda-12080",
                feature = "cuda-12090",
                feature = "cuda-13000",
                feature = "cuda-13010"
            ))]
            let cuptiActivityEnableAllSyncRecords = __library
                .get(b"cuptiActivityEnableAllSyncRecords\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            #[cfg(any(
                feature = "cuda-12060",
                feature = "cuda-12080",
                feature = "cuda-12090",
                feature = "cuda-13000",
                feature = "cuda-13010"
            ))]
            let cuptiActivityEnableAllocationSource = __library
                .get(b"cuptiActivityEnableAllocationSource\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            #[cfg(any(
                feature = "cuda-11070",
                feature = "cuda-11080",
                feature = "cuda-12000",
                feature = "cuda-12010",
                feature = "cuda-12020",
                feature = "cuda-12030",
                feature = "cuda-12040",
                feature = "cuda-12050",
                feature = "cuda-12060",
                feature = "cuda-12080",
                feature = "cuda-12090",
                feature = "cuda-13000",
                feature = "cuda-13010"
            ))]
            let cuptiActivityEnableAndDump = __library
                .get(b"cuptiActivityEnableAndDump\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiActivityEnableContext = __library
                .get(b"cuptiActivityEnableContext\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            #[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
            let cuptiActivityEnableCudaEventDeviceTimestamps = __library
                .get(b"cuptiActivityEnableCudaEventDeviceTimestamps\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            #[cfg(any(
                feature = "cuda-12030",
                feature = "cuda-12040",
                feature = "cuda-12050",
                feature = "cuda-12060",
                feature = "cuda-12080",
                feature = "cuda-12090",
                feature = "cuda-13000",
                feature = "cuda-13010"
            ))]
            let cuptiActivityEnableDeviceGraph = __library
                .get(b"cuptiActivityEnableDeviceGraph\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            #[cfg(any(
                feature = "cuda-12050",
                feature = "cuda-12060",
                feature = "cuda-12080",
                feature = "cuda-12090",
                feature = "cuda-13000",
                feature = "cuda-13010"
            ))]
            let cuptiActivityEnableDriverApi = __library
                .get(b"cuptiActivityEnableDriverApi\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            #[cfg(any(
                feature = "cuda-12080",
                feature = "cuda-12090",
                feature = "cuda-13000",
                feature = "cuda-13010"
            ))]
            let cuptiActivityEnableHWTrace = __library
                .get(b"cuptiActivityEnableHWTrace\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiActivityEnableLatencyTimestamps = __library
                .get(b"cuptiActivityEnableLatencyTimestamps\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiActivityEnableLaunchAttributes = __library
                .get(b"cuptiActivityEnableLaunchAttributes\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            #[cfg(any(
                feature = "cuda-12050",
                feature = "cuda-12060",
                feature = "cuda-12080",
                feature = "cuda-12090",
                feature = "cuda-13000",
                feature = "cuda-13010"
            ))]
            let cuptiActivityEnableRuntimeApi = __library
                .get(b"cuptiActivityEnableRuntimeApi\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiActivityFlush = __library
                .get(b"cuptiActivityFlush\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiActivityFlushAll = __library
                .get(b"cuptiActivityFlushAll\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiActivityFlushPeriod = __library
                .get(b"cuptiActivityFlushPeriod\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiActivityGetAttribute = __library
                .get(b"cuptiActivityGetAttribute\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiActivityGetNextRecord = __library
                .get(b"cuptiActivityGetNextRecord\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiActivityGetNumDroppedRecords = __library
                .get(b"cuptiActivityGetNumDroppedRecords\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiActivityPopExternalCorrelationId = __library
                .get(b"cuptiActivityPopExternalCorrelationId\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiActivityPushExternalCorrelationId = __library
                .get(b"cuptiActivityPushExternalCorrelationId\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiActivityRegisterCallbacks = __library
                .get(b"cuptiActivityRegisterCallbacks\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            #[cfg(any(
                feature = "cuda-11060",
                feature = "cuda-11070",
                feature = "cuda-11080",
                feature = "cuda-12000",
                feature = "cuda-12010",
                feature = "cuda-12020",
                feature = "cuda-12030",
                feature = "cuda-12040",
                feature = "cuda-12050",
                feature = "cuda-12060",
                feature = "cuda-12080",
                feature = "cuda-12090",
                feature = "cuda-13000",
                feature = "cuda-13010"
            ))]
            let cuptiActivityRegisterTimestampCallback = __library
                .get(b"cuptiActivityRegisterTimestampCallback\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiActivitySetAttribute = __library
                .get(b"cuptiActivitySetAttribute\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiComputeCapabilitySupported = __library
                .get(b"cuptiComputeCapabilitySupported\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiDeviceEnumEventDomains = __library
                .get(b"cuptiDeviceEnumEventDomains\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiDeviceEnumMetrics = __library
                .get(b"cuptiDeviceEnumMetrics\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiDeviceGetAttribute = __library
                .get(b"cuptiDeviceGetAttribute\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiDeviceGetEventDomainAttribute = __library
                .get(b"cuptiDeviceGetEventDomainAttribute\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiDeviceGetNumEventDomains = __library
                .get(b"cuptiDeviceGetNumEventDomains\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiDeviceGetNumMetrics = __library
                .get(b"cuptiDeviceGetNumMetrics\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            #[cfg(any(
                feature = "cuda-11040",
                feature = "cuda-11050",
                feature = "cuda-11060",
                feature = "cuda-11070",
                feature = "cuda-11080",
                feature = "cuda-12000",
                feature = "cuda-12010"
            ))]
            let cuptiDeviceGetTimestamp = __library
                .get(b"cuptiDeviceGetTimestamp\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiDeviceSupported = __library
                .get(b"cuptiDeviceSupported\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiDeviceVirtualizationMode = __library
                .get(b"cuptiDeviceVirtualizationMode\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiDisableKernelReplayMode = __library
                .get(b"cuptiDisableKernelReplayMode\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEnableAllDomains = __library
                .get(b"cuptiEnableAllDomains\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEnableCallback = __library
                .get(b"cuptiEnableCallback\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEnableDomain = __library
                .get(b"cuptiEnableDomain\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEnableKernelReplayMode = __library
                .get(b"cuptiEnableKernelReplayMode\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEnumEventDomains = __library
                .get(b"cuptiEnumEventDomains\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEnumMetrics = __library
                .get(b"cuptiEnumMetrics\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEventDomainEnumEvents = __library
                .get(b"cuptiEventDomainEnumEvents\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEventDomainGetAttribute = __library
                .get(b"cuptiEventDomainGetAttribute\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEventDomainGetNumEvents = __library
                .get(b"cuptiEventDomainGetNumEvents\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEventGetAttribute = __library
                .get(b"cuptiEventGetAttribute\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEventGetIdFromName = __library
                .get(b"cuptiEventGetIdFromName\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEventGroupAddEvent = __library
                .get(b"cuptiEventGroupAddEvent\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEventGroupCreate = __library
                .get(b"cuptiEventGroupCreate\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEventGroupDestroy = __library
                .get(b"cuptiEventGroupDestroy\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEventGroupDisable = __library
                .get(b"cuptiEventGroupDisable\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEventGroupEnable = __library
                .get(b"cuptiEventGroupEnable\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEventGroupGetAttribute = __library
                .get(b"cuptiEventGroupGetAttribute\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEventGroupReadAllEvents = __library
                .get(b"cuptiEventGroupReadAllEvents\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEventGroupReadEvent = __library
                .get(b"cuptiEventGroupReadEvent\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEventGroupRemoveAllEvents = __library
                .get(b"cuptiEventGroupRemoveAllEvents\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEventGroupRemoveEvent = __library
                .get(b"cuptiEventGroupRemoveEvent\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEventGroupResetAllEvents = __library
                .get(b"cuptiEventGroupResetAllEvents\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEventGroupSetAttribute = __library
                .get(b"cuptiEventGroupSetAttribute\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEventGroupSetDisable = __library
                .get(b"cuptiEventGroupSetDisable\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEventGroupSetEnable = __library
                .get(b"cuptiEventGroupSetEnable\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEventGroupSetsCreate = __library
                .get(b"cuptiEventGroupSetsCreate\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiEventGroupSetsDestroy = __library
                .get(b"cuptiEventGroupSetsDestroy\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiFinalize = __library
                .get(b"cuptiFinalize\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiGetAutoBoostState = __library
                .get(b"cuptiGetAutoBoostState\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiGetCallbackName = __library
                .get(b"cuptiGetCallbackName\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiGetCallbackState = __library
                .get(b"cuptiGetCallbackState\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiGetContextId = __library
                .get(b"cuptiGetContextId\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiGetDeviceId = __library
                .get(b"cuptiGetDeviceId\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            #[cfg(any(
                feature = "cuda-12020",
                feature = "cuda-12030",
                feature = "cuda-12040",
                feature = "cuda-12050",
                feature = "cuda-12060",
                feature = "cuda-12080",
                feature = "cuda-12090",
                feature = "cuda-13000",
                feature = "cuda-13010"
            ))]
            let cuptiGetErrorMessage = __library
                .get(b"cuptiGetErrorMessage\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            #[cfg(any(
                feature = "cuda-12030",
                feature = "cuda-12040",
                feature = "cuda-12050",
                feature = "cuda-12060",
                feature = "cuda-12080",
                feature = "cuda-12090",
                feature = "cuda-13000",
                feature = "cuda-13010"
            ))]
            let cuptiGetGraphExecId = __library
                .get(b"cuptiGetGraphExecId\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiGetGraphId = __library
                .get(b"cuptiGetGraphId\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiGetGraphNodeId = __library
                .get(b"cuptiGetGraphNodeId\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiGetLastError = __library
                .get(b"cuptiGetLastError\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiGetNumEventDomains = __library
                .get(b"cuptiGetNumEventDomains\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiGetNumMetrics = __library
                .get(b"cuptiGetNumMetrics\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiGetResultString = __library
                .get(b"cuptiGetResultString\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiGetStreamId = __library
                .get(b"cuptiGetStreamId\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiGetStreamIdEx = __library
                .get(b"cuptiGetStreamIdEx\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiGetThreadIdType = __library
                .get(b"cuptiGetThreadIdType\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiGetTimestamp = __library
                .get(b"cuptiGetTimestamp\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiGetVersion = __library
                .get(b"cuptiGetVersion\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiKernelReplaySubscribeUpdate = __library
                .get(b"cuptiKernelReplaySubscribeUpdate\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiMetricCreateEventGroupSets = __library
                .get(b"cuptiMetricCreateEventGroupSets\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiMetricEnumEvents = __library
                .get(b"cuptiMetricEnumEvents\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiMetricEnumProperties = __library
                .get(b"cuptiMetricEnumProperties\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiMetricGetAttribute = __library
                .get(b"cuptiMetricGetAttribute\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiMetricGetIdFromName = __library
                .get(b"cuptiMetricGetIdFromName\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiMetricGetNumEvents = __library
                .get(b"cuptiMetricGetNumEvents\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiMetricGetNumProperties = __library
                .get(b"cuptiMetricGetNumProperties\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiMetricGetRequiredEventGroupSets = __library
                .get(b"cuptiMetricGetRequiredEventGroupSets\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiMetricGetValue = __library
                .get(b"cuptiMetricGetValue\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiMetricGetValue2 = __library
                .get(b"cuptiMetricGetValue2\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiSetEventCollectionMode = __library
                .get(b"cuptiSetEventCollectionMode\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiSetThreadIdType = __library
                .get(b"cuptiSetThreadIdType\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiSubscribe = __library
                .get(b"cuptiSubscribe\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            #[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
            let cuptiSubscribe_v2 = __library
                .get(b"cuptiSubscribe_v2\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiSupportedDomains = __library
                .get(b"cuptiSupportedDomains\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            let cuptiUnsubscribe = __library
                .get(b"cuptiUnsubscribe\0")
                .map(|sym| *sym)
                .expect("Expected symbol in library");
            Ok(Self {
                __library,
                cuptiActivityConfigurePCSampling,
                cuptiActivityConfigureUnifiedMemoryCounter,
                cuptiActivityDisable,
                cuptiActivityDisableContext,
                cuptiActivityEnable,
                #[cfg(any(
                    feature = "cuda-12080",
                    feature = "cuda-12090",
                    feature = "cuda-13000",
                    feature = "cuda-13010"
                ))]
                cuptiActivityEnableAllSyncRecords,
                #[cfg(any(
                    feature = "cuda-12060",
                    feature = "cuda-12080",
                    feature = "cuda-12090",
                    feature = "cuda-13000",
                    feature = "cuda-13010"
                ))]
                cuptiActivityEnableAllocationSource,
                #[cfg(any(
                    feature = "cuda-11070",
                    feature = "cuda-11080",
                    feature = "cuda-12000",
                    feature = "cuda-12010",
                    feature = "cuda-12020",
                    feature = "cuda-12030",
                    feature = "cuda-12040",
                    feature = "cuda-12050",
                    feature = "cuda-12060",
                    feature = "cuda-12080",
                    feature = "cuda-12090",
                    feature = "cuda-13000",
                    feature = "cuda-13010"
                ))]
                cuptiActivityEnableAndDump,
                cuptiActivityEnableContext,
                #[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
                cuptiActivityEnableCudaEventDeviceTimestamps,
                #[cfg(any(
                    feature = "cuda-12030",
                    feature = "cuda-12040",
                    feature = "cuda-12050",
                    feature = "cuda-12060",
                    feature = "cuda-12080",
                    feature = "cuda-12090",
                    feature = "cuda-13000",
                    feature = "cuda-13010"
                ))]
                cuptiActivityEnableDeviceGraph,
                #[cfg(any(
                    feature = "cuda-12050",
                    feature = "cuda-12060",
                    feature = "cuda-12080",
                    feature = "cuda-12090",
                    feature = "cuda-13000",
                    feature = "cuda-13010"
                ))]
                cuptiActivityEnableDriverApi,
                #[cfg(any(
                    feature = "cuda-12080",
                    feature = "cuda-12090",
                    feature = "cuda-13000",
                    feature = "cuda-13010"
                ))]
                cuptiActivityEnableHWTrace,
                cuptiActivityEnableLatencyTimestamps,
                cuptiActivityEnableLaunchAttributes,
                #[cfg(any(
                    feature = "cuda-12050",
                    feature = "cuda-12060",
                    feature = "cuda-12080",
                    feature = "cuda-12090",
                    feature = "cuda-13000",
                    feature = "cuda-13010"
                ))]
                cuptiActivityEnableRuntimeApi,
                cuptiActivityFlush,
                cuptiActivityFlushAll,
                cuptiActivityFlushPeriod,
                cuptiActivityGetAttribute,
                cuptiActivityGetNextRecord,
                cuptiActivityGetNumDroppedRecords,
                cuptiActivityPopExternalCorrelationId,
                cuptiActivityPushExternalCorrelationId,
                cuptiActivityRegisterCallbacks,
                #[cfg(any(
                    feature = "cuda-11060",
                    feature = "cuda-11070",
                    feature = "cuda-11080",
                    feature = "cuda-12000",
                    feature = "cuda-12010",
                    feature = "cuda-12020",
                    feature = "cuda-12030",
                    feature = "cuda-12040",
                    feature = "cuda-12050",
                    feature = "cuda-12060",
                    feature = "cuda-12080",
                    feature = "cuda-12090",
                    feature = "cuda-13000",
                    feature = "cuda-13010"
                ))]
                cuptiActivityRegisterTimestampCallback,
                cuptiActivitySetAttribute,
                cuptiComputeCapabilitySupported,
                cuptiDeviceEnumEventDomains,
                cuptiDeviceEnumMetrics,
                cuptiDeviceGetAttribute,
                cuptiDeviceGetEventDomainAttribute,
                cuptiDeviceGetNumEventDomains,
                cuptiDeviceGetNumMetrics,
                #[cfg(any(
                    feature = "cuda-11040",
                    feature = "cuda-11050",
                    feature = "cuda-11060",
                    feature = "cuda-11070",
                    feature = "cuda-11080",
                    feature = "cuda-12000",
                    feature = "cuda-12010"
                ))]
                cuptiDeviceGetTimestamp,
                cuptiDeviceSupported,
                cuptiDeviceVirtualizationMode,
                cuptiDisableKernelReplayMode,
                cuptiEnableAllDomains,
                cuptiEnableCallback,
                cuptiEnableDomain,
                cuptiEnableKernelReplayMode,
                cuptiEnumEventDomains,
                cuptiEnumMetrics,
                cuptiEventDomainEnumEvents,
                cuptiEventDomainGetAttribute,
                cuptiEventDomainGetNumEvents,
                cuptiEventGetAttribute,
                cuptiEventGetIdFromName,
                cuptiEventGroupAddEvent,
                cuptiEventGroupCreate,
                cuptiEventGroupDestroy,
                cuptiEventGroupDisable,
                cuptiEventGroupEnable,
                cuptiEventGroupGetAttribute,
                cuptiEventGroupReadAllEvents,
                cuptiEventGroupReadEvent,
                cuptiEventGroupRemoveAllEvents,
                cuptiEventGroupRemoveEvent,
                cuptiEventGroupResetAllEvents,
                cuptiEventGroupSetAttribute,
                cuptiEventGroupSetDisable,
                cuptiEventGroupSetEnable,
                cuptiEventGroupSetsCreate,
                cuptiEventGroupSetsDestroy,
                cuptiFinalize,
                cuptiGetAutoBoostState,
                cuptiGetCallbackName,
                cuptiGetCallbackState,
                cuptiGetContextId,
                cuptiGetDeviceId,
                #[cfg(any(
                    feature = "cuda-12020",
                    feature = "cuda-12030",
                    feature = "cuda-12040",
                    feature = "cuda-12050",
                    feature = "cuda-12060",
                    feature = "cuda-12080",
                    feature = "cuda-12090",
                    feature = "cuda-13000",
                    feature = "cuda-13010"
                ))]
                cuptiGetErrorMessage,
                #[cfg(any(
                    feature = "cuda-12030",
                    feature = "cuda-12040",
                    feature = "cuda-12050",
                    feature = "cuda-12060",
                    feature = "cuda-12080",
                    feature = "cuda-12090",
                    feature = "cuda-13000",
                    feature = "cuda-13010"
                ))]
                cuptiGetGraphExecId,
                cuptiGetGraphId,
                cuptiGetGraphNodeId,
                cuptiGetLastError,
                cuptiGetNumEventDomains,
                cuptiGetNumMetrics,
                cuptiGetResultString,
                cuptiGetStreamId,
                cuptiGetStreamIdEx,
                cuptiGetThreadIdType,
                cuptiGetTimestamp,
                cuptiGetVersion,
                cuptiKernelReplaySubscribeUpdate,
                cuptiMetricCreateEventGroupSets,
                cuptiMetricEnumEvents,
                cuptiMetricEnumProperties,
                cuptiMetricGetAttribute,
                cuptiMetricGetIdFromName,
                cuptiMetricGetNumEvents,
                cuptiMetricGetNumProperties,
                cuptiMetricGetRequiredEventGroupSets,
                cuptiMetricGetValue,
                cuptiMetricGetValue2,
                cuptiSetEventCollectionMode,
                cuptiSetThreadIdType,
                cuptiSubscribe,
                #[cfg(any(feature = "cuda-13000", feature = "cuda-13010"))]
                cuptiSubscribe_v2,
                cuptiSupportedDomains,
                cuptiUnsubscribe,
            })
        }
    }
    pub unsafe fn is_culib_present() -> bool {
        let lib_names = ["cupti"];
        let choices = lib_names
            .iter()
            .map(|l| crate::get_lib_name_candidates(l))
            .flatten();
        for choice in choices {
            if Lib::new(choice).is_ok() {
                return true;
            }
        }
        false
    }
    pub unsafe fn culib() -> &'static Lib {
        static LIB: std::sync::OnceLock<Lib> = std::sync::OnceLock::new();
        LIB.get_or_init(|| {
            let lib_names = std::vec!["cupti"];
            let choices: std::vec::Vec<_> = lib_names
                .iter()
                .map(|l| crate::get_lib_name_candidates(l))
                .flatten()
                .collect();
            for choice in choices.iter() {
                if let Ok(lib) = Lib::new(choice) {
                    return lib;
                }
            }
            crate::panic_no_lib_found(lib_names[0], &choices);
        })
    }
}
#[cfg(feature = "dynamic-loading")]
pub use loaded::*;
